const exec = require('child_process').exec;
const models = require('../models');
var moment = require('moment');
const {
  actionType,
  publishBlockedServices
} = require('../routes/common/event-publisher');

module.exports = {
  job: async function(manualTrigger = false) {
    const transaction = await models.sequelize.transaction();
    try {
      //Drop View Of Exit
      await models.sequelize.query(
        `
        DROP VIEW IF EXISTS clientsView;
      `,
        { transaction }
      );

      //Make View Of Client Table
      await models.sequelize.query(
        `
        CREATE VIEW clientsView AS
        SELECT
	      c.*, csi.id as csiid, csi.client_id , csi.state , csi.ocs_agent_enabled 
        FROM Clients c
        LEFT JOIN ClientSoftwareInventory csi ON c.id = csi.client_id;
      `,
        { transaction }
      );

      // Equate UUID from tags
      await models.sequelize.query(
        `
        UPDATE ocs.hardware h RIGHT JOIN ocs.accountinfo a ON h.ID = a.HARDWARE_ID
        SET h.UUID = SUBSTRING(a.tag, LOCATE(':', a.tag) + 1, 36)
        WHERE h.UUID IS NULL
        AND a.tag != SUBSTRING(a.tag, LOCATE(':', a.tag) + 1, 36);
      `,
        { transaction }
      );

      // Delete Software Master List Data
      await models.sequelize.query(
        `
        DELETE FROM SoftwareMasterLists WHERE manually_added = 0;
      `,
        { transaction }
      );

      // Populate Software Master List Data
      await models.sequelize.query(
        `
        INSERT INTO SoftwareMasterLists (
          software_name, software_version, software_publisher, 
          os, os_version, os_comments, manually_added
        )
        SELECT DISTINCT sn.NAME, sv.VERSION, sp.PUBLISHER, hw.OSNAME, hw.OSVERSION, hw.OSCOMMENTS, 0
        FROM ocs.software s
        INNER JOIN ocs.software_name sn ON s.NAME_ID = sn.ID
        INNER JOIN ocs.software_version sv ON s.VERSION_ID = sv.ID
        INNER JOIN ocs.software_publisher sp ON s.PUBLISHER_ID = sp.ID
        INNER JOIN ocs.hardware hw ON s.HARDWARE_ID = hw.ID
        INNER JOIN DeviceInventories di ON hw.UUID = di.device_id AND di.is_reference_machine = 1;
      `,
        { transaction }
      );

      // Delete Forbidden Software List Data
      await models.sequelize.query(
        `
        DELETE FROM ForbiddenSoftwareLists;
      `,
        { transaction }
      );

      const [forbiddenList] = await models.sequelize.query(
        `
        SELECT DISTINCT c.id, c.Name, hw.UUID, sn.NAME, sv.VERSION, sp.PUBLISHER
        FROM ocs.software s
        INNER JOIN ocs.software_name sn ON s.NAME_ID = sn.ID
        INNER JOIN ocs.software_version sv ON s.VERSION_ID = sv.ID
        INNER JOIN ocs.software_publisher sp ON s.PUBLISHER_ID = sp.ID
        INNER JOIN ocs.hardware hw ON s.HARDWARE_ID = hw.ID
        INNER JOIN DeviceInventories d ON d.device_id = hw.UUID
        INNER JOIN clientsView c ON c.device_id = d.id
        INNER JOIN DeviceInventories di ON hw.UUID = di.device_id AND di.is_reference_machine = 0
        LEFT JOIN SoftwareMasterLists sml 
        ON (sn.NAME, sv.VERSION, sp.PUBLISHER) = (sml.software_name, sml.software_version, sml.software_publisher)
        WHERE sml.id IS NULL;
      `,
        { transaction }
      );

      const data = forbiddenList.map(data => {
        return {
          client_id: data.id,
          client_name: data.Name,
          device_id: data.UUID,
          software_name: data.NAME,
          software_version: data.VERSION,
          software_publisher: data.PUBLISHER
        }
      });

      if (data && data.length) {
        // Here Instead of Saving all items at time we will save them in batches
        const chunkSize = 10000; // Define the desired chunk size
  
        // Calculate the number of chunks needed
        const totalChunks = Math.ceil(data.length / chunkSize);
  
        // Process the data in chunks
        for (let i = 0; i < totalChunks; i++) {
          const start = i * chunkSize;
          const end = start + chunkSize;
          const chunk = data.slice(start, end);

          // Perform the desired operation with the chunk
          await models.ForbiddenSoftwareLists.bulkCreate(chunk, {
            ignoreDuplicates: true,
            transaction
          });
        }
      }
      
      // Undetected Devices refer to devices which are not present on OCS inventory
      await models.sequelize.query(
        `
        UPDATE DeviceInventories di
        LEFT JOIN ocs.hardware hw ON di.device_id = hw.UUID
        SET di.inventory_sync_state = 'Undetected', di.last_sync_time = NOW()
        WHERE hw.ID IS NULL;
      `,
        { transaction }
      );

      // Passed Devices refer to devices which are present on OCS inventory
      await models.sequelize.query(
        `
        UPDATE DeviceInventories di
        INNER JOIN ocs.hardware hw ON di.device_id = hw.UUID
        LEFT JOIN ForbiddenSoftwareLists fsl ON di.device_id = fsl.device_id
        SET di.inventory_sync_state = 'Passed', di.last_sync_time = NOW()
        WHERE fsl.id IS NULL;
      `,
        { transaction }
      );

      // Failed Devices which are not present on OCS inventory and Device Inventory
      await models.sequelize.query(
        `
        UPDATE DeviceInventories di
        INNER JOIN ocs.hardware hw ON di.device_id = hw.UUID
        INNER JOIN ForbiddenSoftwareLists fsl ON di.device_id = fsl.device_id
        SET di.inventory_sync_state = 'Failed', di.last_sync_time = NOW()
        WHERE di.inventory_sync_state = 'Passed' OR di.inventory_sync_state = 'Undetected';
      `,
        { transaction }
      );

      // Expire last inventory report
      /*await models.sequelize.query(`
        UPDATE DeviceInventories d INNER JOIN ocs.hardware h ON d.device_id = h.UUID
        SET d.inventory_sync_state = 'Undetected', d.last_sync_time = NOW()
        WHERE IFNULL(DATEDIFF(NOW(), h.LASTDATE), 0) > 1;
      `, { transaction });*/

      // Update Clients states
      await models.sequelize.query(
        `
        UPDATE ClientSoftwareInventory csi
        INNER JOIN Clients c ON csi.client_id = c.id
        INNER JOIN DeviceInventories d ON c.device_id = d.id
        SET csi.state = d.inventory_sync_state
        `
      );

      // this code will run only when device validation perform manually from button
      if (manualTrigger == true) {
        // Generate Alert for Stale devices
        await models.sequelize.query(
          `
          INSERT INTO AlertLogs (type, message, severity, createdAt, updatedAt)
          SELECT 'Device Health', 
          CONCAT('Device ', c.Name, ' (', c.id, ') has an outdated software inventory'), 
          'Medium',
          NOW(), NOW()
          FROM DeviceInventories d 
          INNER JOIN ocs.hardware h ON d.device_id = h.UUID
          INNER JOIN clientsView c ON c.device_id = d.id
          WHERE IFNULL(DATEDIFF(NOW(), h.LASTDATE), 0) > 1;
        `,
          { transaction }
        );

        // Generate Alert for Failed devices
        await models.sequelize.query(
          `
          INSERT INTO AlertLogs (type, message, severity, createdAt, updatedAt)
          SELECT 'Unauthorized Software', 
          CONCAT('\', fsl.software_name, '\',  ' installed on ', c.Name, ' (', c.id, ')'), 
          'High',
          NOW(), NOW()
          FROM clientsView c 
          INNER JOIN ForbiddenSoftwareLists fsl ON c.id = fsl.client_id
          WHERE c.state = 'Failed';
        `,
          { transaction }
        );
      }

      // Delete Blocked Services
      await models.sequelize.query(
        `
        DELETE FROM BlockedServices;
      `,
        { transaction }
      );

      // Get Failed devices
      const clients = await models.sequelize.query(
        `
        SELECT c.id AS id, c.user_id, d.last_sync_time AS last_sync_time
        FROM clientsView c INNER JOIN DeviceInventories d
        ON c.device_id = d.id
        WHERE c.state  IN ('Failed', 'Undetected');
      `,
        { transaction }
      );

      for (let c = 0; c < clients[0].length; c++) {
        // Get Software policies
        const policies = await models.sequelize.query(
          `
          SELECT id, grace_period, all_services, updatedAt, createdAt, block_services FROM SoftwarePolicies WHERE (
            all_clients = 1 
            OR (
              id IN (
                SELECT DISTINCT policy_id FROM ClientPolicies WHERE policy_type_id = 4 AND client_id = ${clients[0][c].id}
              ) 
              OR id IN (
                SELECT DISTINCT policy_id FROM ClientGroupPolicies WHERE policy_type_id = 4 AND client_group_id IN (
                  SELECT DISTINCT groupId FROM GroupClients WHERE clientId = ${clients[0][c].id}
                )
              )
              )
        
            OR (
              id IN (
                SELECT DISTINCT policy_id FROM UserPolicies WHERE policy_type_id = 4 AND user_id = ${clients[0][c].user_id}
              ) 
              OR id IN (
                SELECT DISTINCT policy_id FROM UserGroupPolicies WHERE policy_type_id = 4 AND user_group_id  IN (
                  SELECT DISTINCT user_group_id FROM UserGroupPolicies ug
                  Inner join UserGroupsPivot ugp on ugp.group_id  = ug.user_group_id 
                  WHERE ugp.user_id = ${clients[0][c].user_id}
                  )
              )
              
            )
          )
          AND block_services = 1;
        `,
          { transaction }
        );

        // Loop through all policies
        for (let p = 0; p < policies[0].length; p++) {
          if (
            moment() >=
            moment(policies[0][p].updatedAt).add(
              policies[0][p].grace_period,
              'minutes'
            )
          ) {
            await models.sequelize.query(
              `
              INSERT INTO BlockedServices (service_id, client_id, policy_id)
              SELECT DISTINCT id, ${clients[0][c].id}, ${policies[0][p].id} FROM Services WHERE ${policies[0][p].all_services} = 1 OR (
                id IN (
                  Select DISTINCT sp2.service_id 
                  FROM SoftwarePolicies sp 
                  INNER JOIN ServicePolicies sp2 ON sp2.policy_id = sp.id 
                  INNER JOIN Services s ON s.id = sp2.service_id 
                  WHERE sp2.policy_type_id  = 4 and sp.is_enabled = 1 AND sp2.policy_id = ${policies[0][p].id}
                ) 
                OR id IN (
                  Select DISTINCT s.id 
                  FROM SoftwarePolicies sp 
                  INNER JOIN ServiceGroupPolicies sgp ON sgp.policy_id  = sp.id 
                  INNER JOIN ServiceGroups sg ON sg.id = sgp.service_group_id
                  INNER JOIN GroupServices gs ON gs.groupId = sg.id  
                  INNER JOIN Services s ON s.id = gs.serviceId
                  WHERE sgp.policy_type_id = 4 AND sp.id = ${policies[0][p].id} AND sp.is_enabled = 1
                )
              );
            `,
              { transaction }
            );
          }
        }
      }
      //Drop View Of Exit
      await models.sequelize.query(
        `
        DROP VIEW IF EXISTS clientsView;
      `,
        { transaction }
      );
      await transaction.commit();
      await publishBlockedServices(actionType.UPDATED, {});
    } catch (e) {
      await transaction.rollback();
      return Promise.reject(e);
    }
  },
  schedule: function(syncTime) {
    return new Promise((resolve, reject) => {
      global.cron.send({
        type: 'SCHEDULE',
        expression: `0 0 */${syncTime} * * *`,
        taskName: 'DEVICE_VALIDATION',
        options: {
          scheduled: true
        }
      });

      resolve();
    });
  },
  destroy: function() {
    return new Promise((resolve, reject) => {
      global.cron.send({
        type: 'DESTROY',
        taskName: 'DEVICE_VALIDATION'
      });
      resolve();
    });
  }
};
