const config = require('../config/config');

const mailer = require('../utils/mailer');
const models = require('../models');
const moment = require('moment');
const url = require('url');
const cache = require('../config/cache');

const EMAIL_TEMPLATE_PATH = 'templates/license_reminder.html';
const EMAIL_VIOLATION_TEMPLATE_PATH =
  'templates/license_violation_reminder.html';

const LICENSE_EXPIRY_CRON_TOKEN = 'LICENSE_EXPIRY_CRON_TOKEN'

module.exports = async function() {
  
  const expiryLicenseToken = new Date().getTime().toString();
  await cache.setAsync(LICENSE_EXPIRY_CRON_TOKEN, expiryLicenseToken);

  models.License.findOne({
    order: [['id', 'DESC']],
    limit: 1
  })
    .then(async license => {
      if (license) {
        const {
          license_submission_date,
          number_of_days,
          expiry_reminder_sent_at,
          licensee,
          licensee_email: licenseeEmail,
          violated
        } = license;

        if (violated) {
          return new Promise((resolve, reject) =>
            mailer.sendMail(
              licenseeEmail,
              'Invalid License Uploaded',
              EMAIL_VIOLATION_TEMPLATE_PATH,
              {
                message:
                  'License violation has been detected, please contact Invisily Support for resumption of your services.'
              },
              (err, mailResponse) => {
                if (err) {
                  return reject(err);
                }

                resolve({
                  license,
                  mailResponse
                });
              }
            )
          );
        }

        const licenseSubmittedAt = moment(license_submission_date);
        const licenseValidUntil = licenseSubmittedAt.add(
          number_of_days,
          'days'
        );
        const daysLeft = licenseValidUntil.diff(moment(), 'days', true);
        const isExpiringToday = licenseValidUntil.isSame(moment(), 'day');
        const isExpiringTomorrow = licenseValidUntil.isSame(
          moment()
            .startOf('day')
            .add(1, 'd'),
          'day'
        );
        const daysLeftApprox = Math.floor(daysLeft);

        const lastReminderSentAt = moment(expiry_reminder_sent_at);
        const isSentToday = lastReminderSentAt.isSame(moment(), 'day');
        const isSentThisHour =
          moment().diff(lastReminderSentAt, 'hours', true) < 1;

        if (daysLeft <= 30) {
          if (!isSentToday) {
            /* (daysLeft === 1 && !isSentThisHour) || */
            const subject =
              daysLeft <= 0
                ? 'Your License has Expired'
                : isExpiringToday
                ? 'Your License will Expire Today'
                : isExpiringTomorrow
                ? 'Your License will Expire Tomorrow'
                : `Your License is About to Expire in ${daysLeftApprox} Days`;

            const message = `${
              daysLeft <= 0
                ? 'Your license has expired'
                : isExpiringToday
                ? 'Your license will expire today'
                : isExpiringTomorrow
                ? 'Your license will expire tomorrow'
                : `Your license is about to expire in ${daysLeftApprox} days`
            }. Please purchase a new license to prevent termination of all services under your installation.`;

            const projectConfig = config.project[process.env.NODE_ENV];

            const { protocol, host } = url.parse(projectConfig.offPremApiUrl);
            const baseURL = `${protocol}//${host}`;

            const templateParameters = {
              licensee,
              message,
              inv_logo_url: `${baseURL}/assets/images/invisily-logo.png`,
              ebryx_logo_url: `${baseURL}/assets/images/invisily-logo.png`
            };

            const redisLicenseToken = await cache.getAsync(LICENSE_EXPIRY_CRON_TOKEN)
            if(redisLicenseToken !== expiryLicenseToken) {
              return Promise.reject("License expiry email job already running");
            }
            return new Promise((resolve, reject) =>
              mailer.sendMail(
                licenseeEmail,
                subject,
                EMAIL_TEMPLATE_PATH,
                templateParameters,
                (err, mailResponse) => {
                  if (err) {
                    return reject(err);
                  }

                  resolve({
                    license,
                    mailResponse
                  });
                }
              )
            );
          } else {
            return Promise.reject(new Error('REMINDER_ALREADY_SENT'));
          }
        } else {
          return Promise.reject(new Error('REMINDER_NOT_NEEDED'));
        }
      }

      return Promise.reject(new Error('NO_LICENSE_TO_REMIND_FOR'));
    })
    .then(({ license, mailResponse }) => {
      console.log(mailResponse);

      if (mailResponse.accepted.indexOf(license.licensee_email) > -1) {
        return license.update({
          expiry_reminder_sent_at: moment()
        });
      }

      return Promise.reject(new Error('REMINDER_FAILED_FOR_LICENSEE_EMAIL'));
    })
    .then(() => {
      console.log('Updated Expiry Reminder Date');
    })
    .catch(err => {
      console.log(err && (err.message || err));
    });
};
