const models = require('../models');
const moment = require('moment');
const Ldap = require("ldapjs");
const { Op } = require('sequelize');
const { Severity } = require('../shared/enums/severity');

/**
 * 
 * @param {*} givenTime  Date Time in any Format
 * @returns updated with Active Directory Time formate 
 */
const getDateTimeInADFormat = function(givenTime){
  const year = givenTime.getUTCFullYear();
  const month = (givenTime.getUTCMonth() + 1).toString().padStart(2, '0');
  const day = givenTime.getUTCDate().toString().padStart(2, '0');
  const hours = givenTime.getUTCHours().toString().padStart(2, '0');
  const minutes = givenTime.getUTCMinutes().toString().padStart(2, '0');
  const seconds = givenTime.getUTCSeconds().toString().padStart(2, '0');
  const newTime = `${year}${month}${day}${hours}${minutes}${seconds}.0Z`;
  return newTime;
}

/**
 *  Send Notification for AD sync
 */
const pushNotification = async function(message, severity){
  try{
    const type = 'AD sync';
    await models.AlertLogs.create({type, message, severity});
   }catch(err){
     console.log("Notification Error =>",err);
   }
   return;
}

function combineDcValues(dnString) {
  const parts = dnString.split(",");
  const dcParts = parts.filter(part => !part.startsWith("OU="));
  const dcValues = dcParts.map(part => part.split("=")[1]);

  return dcValues.join(".");
}

/**
 * Create LocalUsers from AD users.
 * @param {*} activeDirUsers AD users list
 */
const createADToLocalUsers = async function(activeDirUsers, ldapSettings, ldapConfig, newLastSyncTime) {
  try{
    const allowedDomains = [];
    const invalidDomainUsers = [];

    const ldaps = await models.Ldap.findAll({
      attributes: ['LDAP_dc']
    });

    ldaps.forEach(ldap => {
      allowedDomains.push(combineDcValues(ldap.LDAP_dc));
    });

    activeDirUsers = activeDirUsers.filter(user => {
      const emailDomain = user.email.split('@')[1];
      if (!allowedDomains.includes(emailDomain)) {
        invalidDomainUsers.push(user);
        return false;
      }
      return allowedDomains.includes(emailDomain);
    });

    if (invalidDomainUsers.length > 0) {
      invalidDomainUsers.map(invalidUser => pushNotification(`Invalid domains found for user ${invalidUser.userName}`, Severity.LOW))
    }

    // create the List of Emails from the Active Directory Users
    const usersEmails = activeDirUsers.map(user=>{
      return user.email; 
    });

    // Get Those users who alredy exist in LocalUsers Table
    let alreadyExistUserEmails = await models.LocalUsers.findAll({
      attributes:['email'],
      where: {
        email: {
          [Op.in]: usersEmails
        }
      }
    });

    // Create Email List that Exist on both side in AD and LocalUsers
    if(alreadyExistUserEmails && alreadyExistUserEmails.length){
      alreadyExistUserEmails=alreadyExistUserEmails.map(item=>{
        return item.dataValues.email;
      })
    }
    // Filter and Map those users who does not exist in LocalUsers Table 
    const usersToSave = activeDirUsers
    .filter(user=>!alreadyExistUserEmails.includes(user.email))
    .map(user=>{
      return {
        name: user.name,
        userName: user.userName,
        email: user.email,
        reachable_email: user.email,
        identity_provider: 'Active Directory'
      }
    });

    console.log('Users to save in db:>> ', usersToSave.length);
    if(usersToSave && usersToSave.length){
      // Here Instead of Saving all items at time we will save them in batches
      const chunkSize = 1000; // Define the desired chunk size

      // Calculate the number of chunks needed
      const totalChunks = Math.ceil(usersToSave.length / chunkSize);

      // Process the users in chunks
      for (let i = 0; i < totalChunks; i++) {
        const start = i * chunkSize;
        const end = start + chunkSize;
        const chunk = usersToSave.slice(start, end);

        // Perform the desired operation with the chunk
        const res=await models.LocalUsers.bulkCreate(chunk,{
          ignoreDuplicates: true
        });
      }
      pushNotification('AD users synced successfully', Severity.INFORMATIONAL);
      pushNotification('AD sync finished', Severity.INFORMATIONAL)
    }else {
      pushNotification('No new user found in AD to sync', Severity.INFORMATIONAL);
    }

    if (ldapConfig.directory_type == 'Active Directory') {
      ldapSettings.update({
        last_ldap_sync_end_time: newLastSyncTime
      });
    }
  }catch(err){
     console.log("Error =>",err);
     pushNotification('Error in AD sync', Severity.MEDIUM);
  }
}

/**
 * Connect Active Directory and Get All AD users those are created 
 * after last sync time. 
 * @param {*} ldap object contains ldap configrations
 * @param {*} syncTime last sync time 
 */
const connectActiveDirectory = async function (ldapConfig, syncTime, ldapSettings) {
  return new Promise((resolve, reject) => {
    const client = Ldap.createClient({ url: ldapConfig.LDAP_url });

    client.bind(ldapConfig.LDAP_user, ldapConfig.LDAP_password, async (err) => {
      if (err) {
        console.log('LDAP Bind Error:', err);
        pushNotification('Error: AD sync failed to connect with LDAP', Severity.LOW);
        return reject(err);
      }

      console.log("LDAP Bind Successful!");
      const lastSyncTime = getDateTimeInADFormat(syncTime);

      let opts;
      if (ldapConfig.directory_type == 'Active Directory') {
        opts = {
          filter: `(&(objectCategory=person)(objectClass=user)(whenCreated>=${lastSyncTime}))`, // AD search filter
          scope: "sub",
          paged: true // if false then fails after fetching 1000 users
        };
      }
      else {
        // Ldap
        opts = {
          filter: "(objectClass=person)", // ldap search filter
          scope: "sub",
        };
      }

      const newLastSyncTime = moment();
      let users = [];

      client.search(ldapConfig.LDAP_dc, opts, (err, res) => {
        if (err) {
          console.log('LDAP Search Error:', err);
          pushNotification('Error: AD sync search failed', Severity.LOW);
          client.unbind();
          return reject(err);
        }

        res.on('searchEntry', (entry) => {
          const user = {
            name: entry.object.displayName || entry.object.cn,
            userName: entry.object.uid || entry.object.sAMAccountName,
            email: entry.object.mail || entry.object.userPrincipalName,
          };
          users.push(user);
        });

        res.on('searchReference', (referral) => {
          console.log('LDAP Referral: ', referral.uris.join());
        });

        res.on('error', (err) => {
          console.log('LDAP Search Error:', err);
          pushNotification('Error: AD sync search error', Severity.LOW);
          reject(err);
        });

        res.on('end', async () => {
          console.log(`Search Completed. Total users retrieved: ${users.length}`);

          if (users.length === 0) {
            pushNotification('AD sync finished, no user found in Active Directory to Sync', Severity.INFORMATIONAL);
            return resolve();
          }

          await createADToLocalUsers(users, ldapSettings, ldapConfig, newLastSyncTime);
          client.unbind();
          resolve();
        });
      });
    });
  });
};

module.exports = async function(){
  try{
    const ldapConfigs = await models.Ldap.findAll({
      order: [['createdAt', 'DESC']]
    });

    if (!ldapConfigs || ldapConfigs.length === 0) {
      pushNotification('AD sync: LDAP Configurations not found', Severity.LOW);
      return;
    }
    
    for (const ldapConfig of ldapConfigs) {
      try {
        // Get LDAP Settings for the current LDAP configuration
        const ldapSettings = await models.Settings.findOne({
          order: [['id', 'DESC']],
          attributes: ['id', 'dirsync_enabled', 'last_ldap_sync_end_time', 'is_ldap_syncing']
        });
    
        if (!ldapSettings) {
          pushNotification('AD sync: LDAP Settings not found', Severity.LOW);
          continue; // Continue to the next LDAP configuration
        }
    
        const setting = ldapSettings.dataValues;

        if (!setting.dirsync_enabled) {
          console.log('AD sync is disabled for LDAP configuration: ', ldapConfig.id);
        } else {
          pushNotification(`AD sync started for LDAP configuration: ${ldapConfig.id}`, Severity.INFORMATIONAL);
          await connectActiveDirectory(ldapConfig.dataValues, setting.last_ldap_sync_end_time, ldapSettings);
        }
      } catch (err) {
        console.log('Error syncing LDAP configuration: ', err);
        pushNotification('Error: AD sync failed for LDAP configuration', Severity.LOW);
      }
    }
    
   

  }catch(err){
    console.log("Error : ",err);
    pushNotification('Error : AD sync failed', Severity.MEDIUM);
    return err;
  } 
}
