var models = require('../models');
const { parseAsync } = require('json2csv')
const path = require('path');
var fs = require('fs');
const moment = require('moment');
const archiver = require('archiver');

/**
 * Get the Admin access logs
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
const getLogs = async (req, res, next) => {
  try {
    const {
      limit,
      offset,
      sort = 'createdAt',
      dir = 'desc',
      searchKey = '',
      startDate,
      endDate,
      priority
    } = req.query;

    const options = {
      attributes: ['createdAt', 'action', 'userId', 'priority'],
      include: [
        {
          model: models.User,
          attributes: ['firstName', 'lastName'],
          include: [
            {
              model: models.Role,
              attributes: ['title']
            }
          ],
          where: {}
        }
      ],
      where: {
        createdAt: {
          [Op.gt]: startDate,
          [Op.lt]: endDate
        }
      },
      distinct: true,
      order: [],
      limit: limit,
      offset: offset
    };

    options.include[0].where = {
      ...options.include[0].where,
      [Op.and]: [
        models.sequelize.where(
          models.sequelize.fn(
            'concat',
            models.sequelize.col('firstName'),
            ' ',
            models.sequelize.col('lastName')
          ),
          {
            [Op.like]: `%${searchKey.trim()}%`
          }
        )
      ]
    };

    const allowedSortValues = ['createdAt', 'firstName'];

    if (sort == 'firstName') {
      options.subQuery = false;

      if (allowedSortValues.includes(sort)) {
        options.order.push([models.User, models.sequelize.literal(sort), dir]);
      }
    } else {
      if (allowedSortValues.includes(sort)) {
        options.order.push([models.sequelize.literal(sort), dir]);
      }
    }

    if(priority) {
      options.where = {
        ...options.where,
        priority: priority.toLowerCase()
      }
    }

    const result = await models.AdminAccessLogs.paginate(options);

    for (let i = 0; i < result.data.length; i++) {
      let query = `
        SELECT DISTINCT t.id as temp_id, t.name, t.description
        FROM Templates t
        INNER JOIN TemplatePermissions tp ON tp.temp_id = t.id
        INNER JOIN Permissions p ON p.id = tp.permission_id
        INNER JOIN Privilege pv ON pv.id = p.privilege_id
        INNER JOIN UserTemplates ut ON ut.temp_permission_id = tp.id
        INNER JOIN Users u ON u.id = ut.user_id
        WHERE u.id = ${result.data[i].userId}
      `;
      
      let template = (await models.sequelize.query(query))[0][0];
      
      let templateName = template.name; 
      
      // storing template data in user object
      result.data[i].dataValues.template = templateName;
    }

    res.json(result);
  } catch (error) {
    next(error);
  }
};

const exportCSV = async (req, res, next) => {
  try {

    const {
      searchKey = '',
      startDate,
      endDate,
      priority,
      version
    } = req.query;
    const user = req.user;
    const timestamp = moment().format("YYYY-MM-DD_HH-mm-ss");
    const fileName = `invisily-${version}-admin-access-logs-${user.id}-${timestamp}`
    const zipFileName = `${fileName}.zip`;
    const csvFileName = `${fileName}.csv`;
    const Path = path.join(__dirname, '../storage/' + zipFileName);

    const options = {
      attributes: ['createdAt', 'action', 'userId', 'priority'],
      include: [
        {
          model: models.User,
          attributes: ['firstName', 'lastName'],
          include: [
            {
              model: models.Role,
              attributes: ['title']
            }
          ],
          where: {}
        }
      ],
      where: {
        createdAt: {
          [Op.gt]: startDate,
          [Op.lt]: endDate
        }
      },
      distinct: true,
      order: [],
      raw:true
    };

    options.include[0].where = {
      ...options.include[0].where,
      [Op.and]: [
        models.sequelize.where(
          models.sequelize.fn(
            'concat',
            models.sequelize.col('firstName'),
            ' ',
            models.sequelize.col('lastName')
          ),
          {
            [Op.like]: `%${searchKey.trim()}%`
          }
        )
      ]
    };

    if(priority) {
      options.where = {
        ...options.where,
        priority: priority.toLowerCase()
      }
    }

    const result = await models.AdminAccessLogs.findAll(options);
      const fields = [
        {
          label: 'ID',
          value: 'userId'
        },
        {
          label: 'NAME',
          value: 'User.firstName'
        },
        {
          label: 'ROLE',
          value: 'User.Roles.title'
        },
        {
          label: 'LOG DESCRIPTION',
          value: 'action'
        },
        {
          label: 'PRIORITY',
          value: 'priority'
        },
        {
          label: 'TIMESTAMP',
          value: 'createdAt'
        }
      ];

      const opts = { fields, quote: '' };

      const csv = await parseAsync(result, opts)

      const zipStream = fs.createWriteStream(Path);

      const archive = archiver('zip', {
        zlib: { level: 9 }
      });

      archive.pipe(zipStream);

      archive.append(csv, { name: csvFileName });

      archive.on('error', err => {
        res.status(500).send('Error occurred while creating the ZIP file.');
      });

      archive.finalize();

      zipStream.on('close', () => {
        res.download(Path, zipFileName, err => {
          if (err) {
            throw err;
          } else {
            fs.unlinkSync(Path);
          }
        });
      });
  } catch (error) {
      console.log('error:', error.message)
      res.status(500).send(error.message)
  }
}  

module.exports = {
  getLogs,
  exportCSV
};
