const models = require('../models');
const httpErrors = require('http-errors');
const _ = require('lodash');
const { parseAsync } = require('json2csv')
const path = require('path');
var fs = require('fs');
const moment = require('moment');
const archiver = require('archiver');

/**
 * Get the list of polices
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
module.exports.getAlerts = async (req, res, next) => {
  try {
    const { 
      limit, 
      offset, 
      sort = 'createdAt', 
      dir = 'desc', 
      type,
      message,
      startDate,
      endDate,
      alertType, 
      alertSeverity
    } = req.query;

    const options = {
      where: {
        createdAt: {
          [Op.gt]: startDate,
          [Op.lt]: endDate
        }
      },
      limit,
      offset,
      order: []
    };

    const allowedSortValues = [
      'severity',
      'type',
      'createdAt'
    ];

    if (allowedSortValues.includes(sort)) {
      options.order.push([models.sequelize.literal(sort), dir]);
    }

    if (message && message.trim().length) {
      options.where = {
        ...options.where,
        [Op.or]: [
          {
            message: {
              [Op.like]: '%' + message.trim() + '%'
            }
          }
        ]
      };
    }

    if (alertType) {
      options.where = {
        ...options.where,
        type: alertType.toLowerCase()
      }
    }

    if (alertSeverity) {
      options.where = {
        ...options.where,
        severity: alertSeverity.toLowerCase()
      }
    }

    const result = await models.AlertLogs.paginate(options);
    res.json(result);
  } catch (error) {
    next(error);
  }
};

/**
 * Get alert notifications
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
module.exports.getAlertsNotification = async (req, res, next) => {
  try {
    const { limit = 10, offset = 0, id } = req.query;

    const options = {
      where:{},
      limit: +limit,
      offset,
      order: [['createdAt', 'DESC']]
    };


    if (id) {
      options.where = {
        id: {
          [Op.gt]: id
        }
      };
    }

    const result = await models.AlertLogs.findAll(options);
    res.json(result);
  } catch (error) {
    next(error);
  }
};

/**
 * Purge all data
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */

module.exports.purgeData = async (req, res, next) => {
  try {
    await models.sequelize.query('TRUNCATE TABLE AlertLogs;');
    return res.json({message: `Data deleted successfully.`});
  } catch (error) {
    next(error);
  }
}

/**
 * Get distinct Type
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */

module.exports.getDistinctType = async (req, res, next) => {
  try {
    const typeList = await models.sequelize.query(`
      SELECT DISTINCT type FROM AlertLogs;
    `);

    const finalList = typeList[0].map(type=>{
      return type.type;
    });
    res.json(finalList);

  } catch (error) {
      next(error);
  }
}

/**
 * Get distinct Severity
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */

module.exports.getDistinctSeverity = async (req, res, next) => {
  try {
    const severityList = await models.sequelize.query(`
      SELECT DISTINCT severity FROM AlertLogs WHERE severity IS NOT NULL;
    `);

    // Define the desired order of severities
    const severityOrder = ['Informational', 'Low', 'Medium', 'High', 'Critical'];

    // Extract the severities from the query result
    const severities = severityList[0].map(severity => severity.severity);

    // Sort the severities based on the severity order
    const orderedSeverities = severities.sort((a, b) => {
      return severityOrder.indexOf(a) - severityOrder.indexOf(b);
    });

    res.json(orderedSeverities);
  } catch (error) {
    next(error);
  }
};

module.exports.exportCSV = async (req, res, next) => {
    try {
       const { 
          searchKeyword,
          startDate,
          endDate,
          alertType,
          version
        } = req.query;
        const user = req.user;
        const currentTimestamp = moment().toISOString();
        const alphanumericTimestamp = currentTimestamp.replace(/[-:.]/g, '');
        const fileName = `invisily-${version}-alerts-${user.id}-${alphanumericTimestamp}`
        const zipFileName = `${fileName}.zip`;
        const csvFileName = `${fileName}.csv`;
        const Path = path.join(__dirname, '../storage/' + zipFileName);

        const options = {
          where: {
            createdAt: {
              [Op.gt]: startDate,
              [Op.lt]: endDate
            }
          },
          order: [],
          raw: true
        };
    
        if (searchKeyword && searchKeyword.trim().length) {
          options.where = {
            ...options.where,
            [Op.or]: [
              {
                message: {
                  [Op.like]: '%' + searchKeyword.trim() + '%'
                }
              }
            ]
          };
        }
    
        if(alertType) {
          options.where = {
            ...options.where,
            type: alertType.toLowerCase()
          }
        }
        
        const result = await models.AlertLogs.findAll(options);
        const fields = [
          {
            label: 'ID',
            value: 'id'
          },
          {
            label: 'TYPE',
            value: 'type'
          },
          {
            label: 'DESCRIPTION',
            value: 'message'
          },
          {
            label: 'SEVERITY',
            value: 'severity'
          },
          {
            label: 'TIMESTAMP',
            value: 'createdAt'
          },
          {
            label: 'UPDATED DATE',
            value: 'updatedAt'
          }
        ];

        const opts = { fields, quote: '' };
        const csv = await parseAsync(result, opts)

        const zipStream = fs.createWriteStream(Path);
  
        const archive = archiver('zip', {
          zlib: { level: 9 }
        });
  
        archive.pipe(zipStream);
        const jsonBuffer = Buffer.from(csv, 'utf-8');
        var ReadableStream = require('stream');
        var zipInputStream = new ReadableStream.Readable();
        zipInputStream.push(jsonBuffer);
        zipInputStream.push(null);
        archive.append(zipInputStream, { name: csvFileName });
  
        archive.on('error', err => {
          res.status(500).send('Error occurred while creating the ZIP file.');
        });
  
        archive.finalize();
        
        zipStream.on('close', () => {
          res.download(Path, zipFileName, err => {
            if (err) {
              throw err;
            } else {
              fs.unlinkSync(Path);
            }
          });
        });
    } catch (error) {
        res.status(500).send(error.message)
    }
}