var moment = require('moment');
var fs = require('fs-extra');
const childProcess = require('child_process');
var _prepend = require('prepend');
const { promisify } = require('util');
const prepend = promisify(_prepend);
const exec = promisify(childProcess.exec);
var httpErrors = require('http-errors');
var models = require('../models');

var projectConfig = require('../config/config')[process.env.NODE_ENV];
const { insertIntoAdminLogs } = require('../routes/common/adminLogs');
const { getBackupCreateMessage, getBackupDeleteMessage } = require('../config/adminLogs.config');
var storageFolderPath = '/var/www/html/admin-gui/storage/backups/';


/**
 * Create backup and download
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
const createBackup = async (req, res, next) => {

    try {
        const time = moment().format('');
        const fileName = 'dump-' + time + '.sql';
        var filePath = '/var/www/html/admin-gui/storage/backups';
    
        await exec(`mysqldump \
                    --opt -h ${projectConfig.host} -u ${projectConfig.username} \
                    --password="${projectConfig.password}" ${projectConfig.database} \
                    > ${filePath}/${fileName}`
                    // --ignore-table=${projectConfig.database}.Users
                );
    
        // const migrations = await models.sequelize.query('SELECT * FROM SequelizeMeta');
    
        // const migrationsArray = [];
        // for (migration in migrations[0]) {
        //   migrationsArray.push(migrations[0][migration].name);
        // }
    
        // await prepend(filePath + '/' + fileName,`/* Current Database migrations are: \n ${migrationsArray.toString()} \n */`);
    
        const backup = await models.Backups.create({
            userId: req.user.id,
            filePath: storageFolderPath + fileName,
            versionInfo: req.body.portalVersion
        });
    
        if(!backup){
            throw new httpErrors.BadRequest('Error in creating backup');
        }

        insertIntoAdminLogs(
            getBackupCreateMessage(time),
            req
        )
    
        res.json({
            id: backup.id
        });

    } catch (error) {
        next(error)
    }

}

const restoreBackup = async (req, res, next) => {
    try {
        let backupFileName = '';

        const backup = await models.Backups.findOne({
            where: { 
                id: req.params.id 
            }
        });
        
        if(!backup)
            throw new httpErrors.BadRequest('Error in restoring backup');
        else
            backupFileName =  backup.dataValues.filePath;
            
        await exec(`mysql -h ${projectConfig.host} -u ${projectConfig.username} --password=${projectConfig.password} ${projectConfig.database} < ${backupFileName}`);
        res.json("Successfully Restored Backup!");

    } catch (error) {
        next(error)
    }
}


/**
 * Get all backups from database
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
const getBackups = async (req, res, next) => {

    try {
        const { 
          limit,
          offset,
          sort='createdAt',
          dir='desc',
        } = req.query;
    
        const options = {
            limit: limit,
            offset: offset,
            order : [],
            attributes: ['id', 'userId', 'createdAt', 'updatedAt', 'versionInfo']
        }
    
        const allowedSortValues = [
          'createdAt',
          'id',
          'versionInfo'
        ];
    
        if (allowedSortValues.includes(sort)) {
          options.order.push([models.sequelize.literal(sort), dir]);
        }
    
        const result = await models.Backups.paginate(options);
        res.json(result);
    } catch (error) {
        next(error);
    }

}


/**
 * Download backup againt specific id
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
const downloadUrl = async (req, res, next) => {

    try {
        const backup = await models.Backups.find({
            where: { 
                id: req.params.id 
            }
        });

        if(!backup){
            throw new httpErrors.NotFound('Backup does not exist against this Id.');
        }

        res.download(backup.filePath);

    } catch (error) {
        next(error);
    }

}

/**
 * Delete backup againt specific id
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
const deleteBackup = async (req, res, next) => {
    
    try {
        const backup = await models.Backups.find({
            where: { 
                id: req.params.id 
            }
        });
    
        if(!backup){
            throw new httpErrors.NotFound('Backup not found.');
        }
    
        await Promise.all([fs.remove(backup.filePath), backup.destroy()]);
        let name = backup.filePath.split('/')
        name = name[name.length - 1].replace("dump-", "").replace(".sql", "")
        
        insertIntoAdminLogs(
            getBackupDeleteMessage(name),
            req
        )
        res.json('Backup deleted successfully.');
    } catch (error) {
        next(error);
    }

};


module.exports = {
    createBackup,
    getBackups,
    downloadUrl,
    deleteBackup,
    restoreBackup
};
