var express = require('express');
var router = express.Router();
const httpErrors = require('http-errors');
var geoip = require('geoip-lite2');
var moment = require('moment');
const axios = require('axios');
const redisClient = require('../config/cache')
const util = require('util');

var models = require('../models');
var utils = require('../utils/common');
var projectConfig = require('../config/config')['project'][
  process.env.NODE_ENV
];
let settings = require('../routes/common/settings');
const helper = require('../routes/common/helper');
const { Op } = require('sequelize');

const graphFilters = [
  'month',
  'week',
  'year'
]

const getCount = async (req, res, next) => {
  try {

    let mySettigs = await models.Settings.findOne({
      order: [['id', 'DESC']]
    });
    let controllerHealthInterval = 10;
    if(mySettigs) {
      controllerHealthInterval = mySettigs.health_interval || 10;
    }

    const [
      onlineServers,
      clients,
      clientsActive,
      secondaryControllersCount,
      controllersActive,
      gateways,
      gatewaysActive,
      ldaps,
      setting,
      adminsCount,
      servicesCount,
      totalPolicies,
      alertLogsCount,
      primaryControllers,
      secondaryControllers,
    ] = await Promise.all([
      utils.getOnlineServers(utils.serverStatusKeys.PORTALS),
      models.Client.count({
        where: { }
      }),
      utils.getOnlineCount(utils.serverStatusKeys.CLIENTS),
      models.Controller.count({
        where: {
          roleId: 2
        }
      }),
      utils.getOnlineCount(utils.serverStatusKeys.CONTROLLERS),
      models.Gateway.count({
        where: { }
      }),
      utils.getOnlineCount(utils.serverStatusKeys.GATEWAYS),
      models.Ldap.count({
        where: { }
      }),
      models.Settings.findOne({
        attributes: ['id'],
        order: [['id', 'DESC']]
      }),
      models.User.count({ 
        where: { }
      }),
      models.Service.count({
        where: { }
      }),
      models.sequelize.query(`
      SELECT 
      (select count(*) from RoutingPolicies)
      +(select count(*) from OsPolicies)
      +(select count(*) from SoftwarePolicies)
      +(select count(*) from DevicePolicies)
      +(select count(*) from Entitlements)
      +(select count(*) from DataVolumePolicies)
      +(select count(*) from SoftwareProcessPolicies)
      `),
      models.AlertLogs.count(),
      models.sequelize.query(`
        SELECT c.id, c.connected_clients, CASE WHEN TIMESTAMPDIFF(SECOND, c.heart_beat_sent_at, CURRENT_TIMESTAMP) < ${controllerHealthInterval + 130} THEN 1 ELSE 0 
        END AS Online FROM Controllers c INNER JOIN ComponentRoles cr ON c.roleId = cr.id AND cr.role = 'Primary' AND c.heart_beat = 1 ;
      `),
      models.sequelize.query(`
        SELECT c.id, c.connected_clients, CASE WHEN TIMESTAMPDIFF(SECOND, c.heart_beat_sent_at, CURRENT_TIMESTAMP) < ${controllerHealthInterval + 130} THEN 1 ELSE 0 
        END AS Online FROM Controllers c INNER JOIN ComponentRoles cr ON c.roleId = cr.id AND cr.role = 'Secondary' AND c.heart_beat = 1 ;
      `)
    ]);

    let onlineSecondaryControllers = 0

    secondaryControllers[0] && secondaryControllers[0].forEach((controller) => {
      if(controller.Online) {
        onlineSecondaryControllers += 1;
      }
    })

    let policiesCount = 0;
    try {
      policiesCount = Object.values(totalPolicies[0][0])[0] || 0;
    } catch(e) {
      policiesCount = 0;
    }
    return res.json({
      clients: clients,
      clientsActive: clientsActive,
      secondaryControllersCount: secondaryControllersCount,
      onlineSecondaryControllers,
      controllersActive: controllersActive,
      gateways: gateways,
      gatewaysActive: gatewaysActive,
      isSettingsSet: setting[0] && setting[0].isSettingsSet,
      ldaps: ldaps,
      status: true,
      mfaServerStatus: onlineServers.includes(utils.serverStatusKeys.MFA_SERVER),
      onboardingPotalStatus: onlineServers.includes(utils.serverStatusKeys.ONBOARDING_PORTAL),
      contentPortalStatus: onlineServers.includes(utils.serverStatusKeys.CONTENT_PORTAL),
      adminsCount,
      servicesCount,
      policiesCount: policiesCount, 
      alertLogsCount,
      primaryControllerStatus: primaryControllers[0] && primaryControllers[0].some(primaryController => primaryController.Online),
    });
  } catch (error) {
    next(error);
  }
};

const markersClients = async (req, res, next) => {
  try {
    const clients = await models.Client.findAll({
      where: { }
    });
    if (!clients) {
      throw new httpErrors.NotFound('No client found.');
    }
    res.json(clients);
  } catch (error) {
    next(error);
  }
};

const markersControllers = async (req, res, next) => {
  try {
    const startTime = Date.now();
  
    const cachedData = await redisClient.getAsync(`markersControllers`);
    
    if (cachedData) {
      const parsedData = JSON.parse(cachedData);
      const endTime = Date.now();
      const executionTime = endTime - startTime;
      console.log(`Execution Time (markersControllers) with cache: ${executionTime}ms`);
      return res.status(200).json(parsedData);
    }

    const responseData = await markersControllersCron();

    res.status(200).json(responseData);
  } catch (error) {
    next(error);
  }
};

const markersControllersCron = async () => {
  try {
    const startTime = Date.now();

    settings.purgeClientLog();
    var markers = {};
    var is_private = false;
    var publicIP = projectConfig.publicIP;
    const controllers = await models.Controller.findAll({
      where: {
        heart_beat: 1
      }
    });

    if (!controllers) {
      throw new httpErrors.NotFound('No controller found.');
    }

    controllers.forEach(controller => {
      if (controller) {
        if (controller.public_ipv4_address) {
          let marker = {
            name: controller.Name,
            id: controller.id,
            ip: controller.public_ipv4_address,
            controller: 1,
            gateway: 0,
            client: 0
          };
          let parts = marker.ip.split('.');
          is_private =
            parts[0] === '10' ||
            parts[0] === '172' ||
            (parts[0] === '192' && parts[1] === '168');
          if (is_private) {
            marker.ip = publicIP;
          }
          var geo = geoip.lookup(marker.ip);
          if (geo) {
            if (geo.ll) {
              if (!markers[marker.ip]) {
                markers[marker.ip] = [];
              }
              marker.ln = geo.ll[1];
              marker.lt = geo.ll[0];
              markers['' + marker.ip + ''].push(marker);
            }
          }
        }
      }
    });

    const gateways = await models.Gateway.findAll({
      where: {
        heart_beat: 1
      }
    });

    if (!gateways) {
      throw new httpErrors.NotFound('No gateway found.');
    }

    gateways.forEach(gateway => {
      if (gateway) {
        if (gateway.public_ipv4_address) {
          let marker = {
            name: gateway.Name,
            id: gateway.id,
            ip: gateway.public_ipv4_address,
            controller: 0,
            gateway: 1,
            client: 0
          };
          let parts = marker.ip.split('.');
          is_private =
            parts[0] === '10' ||
            parts[0] === '172' ||
            (parts[0] === '192' && parts[1] === '168');
          if (is_private) {
            marker.ip = publicIP;
          }
          var geo = geoip.lookup(marker.ip);
          if (geo) {
            if (geo.ll) {
              if (!markers[marker.ip]) {
                markers[marker.ip] = [];
              }
              marker.ln = geo.ll[1];
              marker.lt = geo.ll[0];
              markers['' + marker.ip + ''].push(marker);
            }
          }
        }
      }
    });

    const clients = await models.Client.findAll({
      where: {
        geoip_track: 1,
        heart_beat: 1
      }
    });

    if (!clients) {
      throw new httpErrors.NotFound('No clients found.');
    }
    clients.forEach(function(client) {
      if (client) {
        if (!client.ipv4_address) {
          client.ipv4_address = publicIP;
        }
        if (client.ipv4_address) {
          let marker = {
            name: client.Name,
            id: client.id,
            ip: client.ipv4_address,
            controller: 0,
            client: 1,
            gateway: 0
          };
          let parts = marker.ip.split('.');
          is_private =
            parts[0] === '10' ||
            parts[0] === '172' ||
            (parts[0] === '192' && parts[1] === '168');
          if (is_private) {
            marker.ip = publicIP;
          }
          var geo = geoip.lookup(marker.ip);
          if (geo) {
            if (geo.ll) {
              if (!markers[marker.ip]) {
                markers[marker.ip] = [];
              }
              marker.ln = geo.ll[1];
              marker.lt = geo.ll[0];
              markers['' + marker.ip + ''].push(marker);
            }
          }
        }
      }
    });

    const responseData = markers;

    const setexAsync = util.promisify(redisClient.setex).bind(redisClient);

    await setexAsync(`markersControllers`, 3600 * 2, JSON.stringify(responseData));

    const endTime = Date.now();
    const executionTime = endTime - startTime;
    console.log(`Execution Time (markersControllers) without cache: ${executionTime}ms`);

    return responseData;
  } catch (error) {
    throw new httpErrors.BadRequest(error)
  }
};

const getOs = async (req, res, next) => {
  try {
    const startTime = Date.now();
  
    const cachedData = await redisClient.getAsync(`getOs`);
    
    if (cachedData) {
      const parsedData = JSON.parse(cachedData);
      const endTime = Date.now();
      const executionTime = endTime - startTime;
      console.log(`Execution Time (getOs) with cache: ${executionTime}ms`);
      return res.status(200).json(parsedData);
    }

    const responseData = await getOsCron();

    res.status(200).json(responseData);
  } catch (error) {
    next(error);
  }
};

const getOsCron = async () => {
  try {    
    const startTime = Date.now();

    const options = {
      where: {
        device_os: {
          [Op.ne]: 'unknown',
          [Op.notIn]: [
            'ubuntu',
            'ubuntuserver'
          ]
        }
      },
      group: 'device_os',
      attributes: ['device_os']
    };

    const os = await models.Client.count(options);
  
    for (var i = 0; i < os.length; i++) {
      os[i].name = os[i].device_os;
      os[i].y = os[i].count;
      delete os[i].device_os;
      delete os[i].count;
    }

    const responseData = { 
      data: os,
      status: true
    };

    const setexAsync = util.promisify(redisClient.setex).bind(redisClient);

    await setexAsync(`getOs`, 3600 * 2, JSON.stringify(responseData));

    const endTime = Date.now();
    const executionTime = endTime - startTime;
    console.log(`Execution Time (getOs) without cache: ${executionTime}ms`);

    return responseData;
  } catch (error) {
    throw new httpErrors.BadRequest(error)
  }
};

const test = async (req, res, next) => {
  var utc = moment()
    .utc()
    .subtract(1, 'minutes')
    .format('YYYY-MM-DD HH:mm:ss');
  var currentTime = moment()
    .utc()
    .subtract(1, 'minutes')
    .local()
    .format('YYYY-MM-DD HH:mm:ss');
  res.json({
    oldtime: currentTime,
    utc: utc
  });
};

const userServices = async (req, res, next) => {
  try {
    const startTime = Date.now();

    let { filter } = req.query
    if(!filter || filter == undefined || !graphFilters.some(x => x === filter))
    {
      filter = 'month'
    }

    const cachedData = await redisClient.getAsync(`userServices:${filter}`);
    
    if (cachedData) {
      const parsedData = JSON.parse(cachedData);
      const endTime = Date.now();
      const executionTime = endTime - startTime;
      console.log(`Execution Time (userServices:${filter}) with cache: ${executionTime}ms`);
      return res.status(200).json(parsedData);
    }

    const responseData = await userServicesCron(filter);

    res.status(200).json(responseData);
  } catch (error) {
    next(error);
  }

};

const userServicesCron = async (filter) => {
  try {
    const startTime = Date.now();

    if(!filter || filter == undefined || !graphFilters.some(x => x === filter))
    {
      filter = 'month'
    }
    
    const query = {
      where: {},
      attributes: [
        'agent_id',
        [
          models.sequelize.fn(
            'count',
            models.sequelize.fn('distinct', models.sequelize.col('service_id'))
          ),
          'totalServices'
        ]
      ],
      include: [
        {
          model: models.Client,
          attributes: ['Name'],
          required: true
        }
      ],
      group: ['agent_id'],
      order: [[models.sequelize.literal('totalServices DESC')]],
      limit: 10
    }

    let dateRange = null
    if(filter === 'week')
    {
      dateRange = utils.getDateRange({
        epoch: true,
        rangeType: 'L7D'
      });
    }
    if(filter === 'month')
    {
      dateRange = utils.getDateRange({
        epoch: true,
        rangeType: 'M'
      });
    }
    if(filter === 'year')
    {
      dateRange = utils.getDateRange({
        epoch: true,
        rangeType: 'Y'
      });
    }

    let data = await models.ServiceAccessLogs.findAll({
      ...query,
      where: {
        log_start_time: {
          [Op.between]: [dateRange.startDate, dateRange.endDate]
        }
      }
    })

    data = data.map(({ Client = {}, ...rest }) => ({
      name: (Client || {}).Name || 'Unknown',
      value: +rest.dataValues.totalServices
    }))

    const responseData = { [filter]: data };

    const setexAsync = util.promisify(redisClient.setex).bind(redisClient);

    await setexAsync(`userServices:${filter}`, 3600 * 2, JSON.stringify(responseData));

    const endTime = Date.now();
    const executionTime = endTime - startTime;
    console.log(`Execution Time (userServices:${filter}) without cache: ${executionTime}ms`);

    return responseData
  } catch (error) {
    throw new httpErrors.BadRequest(error)
  }

};

const userVolumes = async (req, res, next) => {
  try {
    const startTime = Date.now();

    let { filter } = req.query
    if(!filter || filter == undefined || !graphFilters.some(x => x === filter))
    {
      filter = 'month'
    }
  
    const cachedData = await redisClient.getAsync(`userVolumes:${filter}`);
    
    if (cachedData) {
      const parsedData = JSON.parse(cachedData);
      const endTime = Date.now();
      const executionTime = endTime - startTime;
      console.log(`Execution Time (userVolumes:${filter}) with cache: ${executionTime}ms`);
      return res.status(200).json(parsedData);
    }

    const responseData = await userVolumesCron(filter);

    res.status(200).json(responseData);
  } catch (error) {
    next(error);
  }
};

const userVolumesCron = async (filter) => {
  try {
    const startTime = Date.now();

    if(!filter || filter == undefined || !graphFilters.some(x => x === filter))
    {
      filter = 'month'
    }

    const query = {
      where: {},
      attributes: [
        'agent_id',
        [
          models.sequelize.fn(
            'sum',
            models.sequelize.literal(
              '(client_tx_bytes + service_tx_bytes) / 1048576'
            )
          ),
          'totalVolume'
        ]
      ],
      include: [
        {
          model: models.Client,
          attributes: ['Name'],
          required: true
        }
      ],
      group: ['agent_id'],
      order: [[models.sequelize.literal('totalVolume DESC')]],
      limit: 10
    };

    let dateRange = null
    if(filter === 'week')
    {
      dateRange = utils.getDateRange({
        epoch: true,
        rangeType: 'L7D'
      });
    }
    if(filter === 'month')
    {
      dateRange = utils.getDateRange({
        epoch: true,
        rangeType: 'M'
      });
    }
    if(filter === 'year')
    {
      dateRange = utils.getDateRange({
        epoch: true,
        rangeType: 'Y'
      });
    }

    let data = await models.ServiceAccessLogs.findAll({
      ...query,
      where: {
        log_start_time: {
          [Op.between]: [dateRange.startDate, dateRange.endDate]
        }
      }
    })

    data = data.map(({ Client = {}, ...rest }) => ({
      name: (Client || {}).Name || 'Unknown',
      value: Number(rest.dataValues.totalVolume)
    }))

    const responseData = { [filter]: data };

    const setexAsync = util.promisify(redisClient.setex).bind(redisClient);

    await setexAsync(`userVolumes:${filter}`, 3600 * 2, JSON.stringify(responseData));

    const endTime = Date.now();
    const executionTime = endTime - startTime;
    console.log(`Execution Time (userVolumes:${filter}) without cache: ${executionTime}ms`);

    return responseData;
  } catch (error) {
    throw new httpErrors.BadRequest(error)
  }
};

const serviceUsers = async (req, res, next) => {
  try {
    const startTime = Date.now();

    let { filter } = req.query;
    if(!filter || filter == undefined || !graphFilters.some(x => x === filter))
    {
      filter = 'month'
    }

    const cachedData = await redisClient.getAsync(`serviceUsers:${filter}`);
    
    if (cachedData) {
      const parsedData = JSON.parse(cachedData);
      const endTime = Date.now();
      const executionTime = endTime - startTime;
      console.log(`Execution Time (serviceUsers:${filter}) with cache: ${executionTime}ms`);
      return res.status(200).json(parsedData);
    }

    const responseData = await serviceUsersCron(filter);

    res.status(200).json(responseData);
  } catch (error) {
    next(error);
  }
};

const serviceUsersCron = async (filter) => {
  try {
    const startTime = Date.now();

    if(!filter || filter == undefined || !graphFilters.some(x => x === filter))
    {
      filter = 'month'
    }

    const query = {
      where: {},
      attributes: [
        'service_id',
        [
          models.sequelize.fn(
            'count',
            models.sequelize.fn(
              'distinct',
              models.sequelize.col('ServiceAccessLogs.agent_id')
            )
          ),
          'totalUsers'
        ]
      ],
      include: [
        {
          model: models.Service,
          attributes: ['Name'],
          required: true
        },
        {
          model: models.Client,
          attributes: [],
          required: true
        }
      ],
      group: ['service_id'],
      order: [[models.sequelize.literal('totalUsers DESC')]],
      limit: 10
    };
    let dateRange = null
    if(filter === 'week')
    {
      dateRange = utils.getDateRange({
        epoch: true,
        rangeType: 'L7D'
      });
    }
    if(filter === 'month')
    {
      dateRange = utils.getDateRange({
        epoch: true,
        rangeType: 'M'
      });
    }
    if(filter === 'year')
    {
      dateRange = utils.getDateRange({
        epoch: true,
        rangeType: 'Y'
      });
    }

    let data = await models.ServiceAccessLogs.findAll({
      ...query,
      where: {
        log_start_time: {
          [Op.between]: [dateRange.startDate, dateRange.endDate]
        }
      }
    })

    data = data.map(({ Service = {}, ...rest }) => ({
      name: (Service || {}).Name || 'Unknown',
      value: Number(rest.dataValues.totalUsers)
    }))
    
    const responseData = { [filter]: data };

    const setexAsync = util.promisify(redisClient.setex).bind(redisClient);

    await setexAsync(`serviceUsers:${filter}`, 3600 * 2, JSON.stringify(responseData));

    const endTime = Date.now();
    const executionTime = endTime - startTime;
    console.log(`Execution Time (serviceUsers:${filter}) without cache: ${executionTime}ms`);

    return responseData;
  } catch (error) {
    throw new httpErrors.BadRequest(error)
  }
};

const serviceVolumes = async (req, res, next) => {
  try {
    const startTime = Date.now();

    let { filter } = req.query
    if(!filter || filter == undefined || !graphFilters.some(x => x === filter))
    {
      filter = 'month'
    }

    const cachedData = await redisClient.getAsync(`serviceVolumes:${filter}`);
    
    if (cachedData) {
      const parsedData = JSON.parse(cachedData);
      const endTime = Date.now();
      const executionTime = endTime - startTime;
      console.log(`Execution Time (serviceVolumes:${filter}) with cache: ${executionTime}ms`);
      return res.status(200).json(parsedData);
    }

    const responseData =  await serviceVolumesCron(filter);

    res.status(200).json(responseData);
  } catch (error) {
    next(error);
  }
};

const serviceVolumesCron = async (filter) => {
  try {
    const startTime = Date.now();

    if(!filter || filter == undefined || !graphFilters.some(x => x === filter))
    {
      filter = 'month'
    }

    const query = {
      where: {},
      attributes: [
        'service_id',
        [
          models.sequelize.fn(
            'count',
            models.sequelize.fn('distinct', models.sequelize.col('agent_id'))
          ),
          'totalUsers'
        ],
        [
          models.sequelize.fn(
            'sum',
            models.sequelize.literal(
              '(client_tx_bytes + service_tx_bytes) / 1048576'
            )
          ),
          'totalVolume'
        ]
      ],
      include: [
        {
          model: models.Service,
          attributes: ['Name'],
          required: true
        }
      ],
      group: ['service_id'],
      order: [[models.sequelize.literal('totalVolume DESC')]],
      limit: 10
    };
    let dateRange = null
    if(filter === 'week')
    {
      dateRange = utils.getDateRange({
        epoch: true,
        rangeType: 'L7D'
      });
    }
    if(filter === 'month')
    {
      dateRange = utils.getDateRange({
        epoch: true,
        rangeType: 'M'
      });
    }
    if(filter === 'year')
    {
      dateRange = utils.getDateRange({
        epoch: true,
        rangeType: 'Y'
      });
    }
    let data = await models.ServiceAccessLogs.findAll({
        ...query,
        where: {
          log_start_time: {
            [Op.between]: [dateRange.startDate, dateRange.endDate]
          }
        }
      })

    data = data.map(({ Service = {}, ...rest }) => ({
      name: (Service || {}).Name || 'Unknown',
      value: Number(rest.dataValues.totalVolume)
    }));

    const responseData = { [filter]: data };

    const setexAsync = util.promisify(redisClient.setex).bind(redisClient);

    await setexAsync(`serviceVolumes:${filter}`, 3600 * 2, JSON.stringify(responseData));

    const endTime = Date.now();
    const executionTime = endTime - startTime;
    console.log(`Execution Time (serviceVolumes:${filter}) without cache: ${executionTime}ms`);

    return responseData;
  } catch (error) {
    throw new httpErrors.BadRequest(error)
  }
};

const graph = async (req, res, next) => {
  try {
    const startTime = Date.now();
  
    const cachedData = await redisClient.getAsync(`graph`);
    
    if (cachedData) {
      const parsedData = JSON.parse(cachedData);
      const endTime = Date.now();
      const executionTime = endTime - startTime;
      console.log(`Execution Time (graph) with cache: ${executionTime}ms`);
      return res.status(200).json(parsedData);
    }

    const responseData = await graphCron();
  
    res.status(200).json(responseData);
  } catch (error) {
    next(error);
  }
};

const graphCron = async () => {
  try {
    const startTime = Date.now();

    const servicesSql = models.Service.count({
      where: { },
      distinct: true,
      include: [
        {
          model: models.ServiceAccessLogs,
          required: true
        }
      ]
    });
  
    const clientsSql = models.Client.count({
      where: {
        client_status: 4
      },
      distinct: true,
      include: [
        {
          model: models.ServiceAccessLogs,
          required: true
        }
      ]
    });
  
    const totalServicesSql = models.Service.count({
      where: { }
    });
  
    const totalClientsSql = models.Client.count({
      where: {
        client_status: 4
      }
    });

    try {
      const [
        services = [],
        users = [],
        totalServices,
        totalClients
      ] = await Promise.all([
        servicesSql,
        clientsSql,
        totalServicesSql,
        totalClientsSql
      ]);

      const responseData = {
        utilizationByUsers: totalClients ? +((users / totalClients) * 100).toFixed(0) : 0,
        utilizationByServices: totalServices?  +((services / totalServices) * 100).toFixed(0) : 0
      };

      const setexAsync = util.promisify(redisClient.setex).bind(redisClient);

      await setexAsync(`graph`, 3600 * 2, JSON.stringify(responseData));

      const endTime = Date.now();
      const executionTime = endTime - startTime;
      console.log(`Execution Time (graph) without cache: ${executionTime}ms`);
  
      return responseData;
    } catch (error) {
      next(error);
    }
  } catch (error) {
    throw new httpErrors.BadRequest(error)
  }
};


const getDeviceStatistics = async (req, res, next) => {
  try {
    const startTime = Date.now();

    const { interval = 'days' } = req.query;
    const allowedIntervals = ['days', 'month', 'year']
    if(!allowedIntervals.includes(interval)) {
      throw new httpErrors.BadRequest('Invalid interval value')
    }

    const cachedData = await redisClient.getAsync(`getDeviceStatistics:${interval}`);
    
    if (cachedData) {
      const parsedData = JSON.parse(cachedData);
      const endTime = Date.now();
      const executionTime = endTime - startTime;
      console.log(`Execution Time (getDeviceStatistics:${interval}) with cache: ${executionTime}ms`);
      return res.status(200).json(parsedData);
    }

    const responseData =  await getDeviceStatisticsCron(interval);

    res.status(200).json(responseData);
  } catch(e) {
    next(e);
  }
}

const getDeviceStatisticsCron = async (interval) => {
  try {
    const startTime = Date.now();

    const allowedIntervals = ['days', 'month', 'year']
    if(!allowedIntervals.includes(interval)) {
      throw new httpErrors.BadRequest('Invalid interval value')
    }

    function mapResults(data) {
      return data.map(row => {
        return {
          date: row.createdAt,
          clients: row.clients,
          devices: row.devices,
          users: row.users
        }
      })
    }

    let result = []

    if(interval === allowedIntervals[0]) {
      result = await models.CountHistory.findAll({
        where: {
          createdAt: {
            [Op.and]: [
              {
                [Op.gte]: moment().subtract(5, 'days').toDate()
              },
              {
                [Op.lte]: moment().toDate()
              }
            ]
          }
        },
      });
      result = mapResults(result)
    }

    if(interval === allowedIntervals[1]) {
      result = await models.CountHistory.findAll({
        where: {
          createdAt: {
            [Op.and]: [
              {
                [Op.gte]: moment().subtract(30, 'days').toDate()
              },
              {
                [Op.lte]: moment().toDate()
              }
            ]
          }
        },
      });
      result = mapResults(result)
    }

    if(interval === allowedIntervals[2]) {
       const history = await models.sequelize.query(`
          SELECT LEFT(createdAt , 7) AS date,
            max(clients) AS clients,
            max(users) AS users,
            max(devices) AS devices      
          FROM CountHistory ch
          GROUP BY LEFT(createdAt, 7) limit 0, 12;
      `)
      result = history[0]
    }

    const responseData = result;

    const setexAsync = util.promisify(redisClient.setex).bind(redisClient);

    await setexAsync(`getDeviceStatistics:${interval}`, 3600 * 2, JSON.stringify(responseData));

    const endTime = Date.now();
    const executionTime = endTime - startTime;
    console.log(`Execution Time (getDeviceStatistics:${interval}) without cache: ${executionTime}ms`);

    return responseData;
  } catch(error) {
    throw new httpErrors.BadRequest(error)
  }
}

const getDevicesCount = async (req, res, next) => {
  try {
    // const startTime = Date.now();

    const { oldVersion } = req.query;
    const mobileOS = ['ios', 'android'];
    const serverOs = ["", 'ubuntuserver18', 'windowsserver', 'centosserver', 'ubuntuserver20armv8']
    const desktopOs = ['macintosh', 'ubuntu18', 'windows']

  
    // const cachedData = await redisClient.getAsync(`getDevicesCount`);
    
    // if (cachedData) {
    //   const parsedData = JSON.parse(cachedData);
    //   // console.log("Cached Data: ", parsedData);
    //   const endTime = Date.now();
    //   const executionTime = endTime - startTime;
    //   // console.log(`Execution Time with cache: ${executionTime}ms`);
    //   return res.status(200).json(parsedData);
    // }

    const [desktopVersion, mobileVersion, serverVersion] = await Promise.all([
      models.Client.findOne({
        where: {
          device_os: desktopOs
        },
        attributes: [
          [
            models.sequelize.fn('max', models.sequelize.col('client_version')), "version"
          ]
        ]
      }),
      models.Client.findOne({
        where: {
          device_os: mobileOS
        },
        attributes: [
          [
            models.sequelize.fn('max', models.sequelize.col('client_version')), "version"
          ]
        ]
      }),
      models.Client.findOne({
        where: {
          device_os: {
            [models.sequelize.Op.notIn]: [
              ...mobileOS,
              ...desktopOs
            ]
          }
        },
        attributes: [
          [
            models.sequelize.fn('max', models.sequelize.col('client_version')), "version"
          ]
        ]
      })
    ])

    const [desktopClients, mobileClients, serverClients] = await Promise.all([
      models.Client.findAll({
        where: {
          client_version: oldVersion === 'false'? (desktopVersion.dataValues.version || " "): {
            [Op.not]: (desktopVersion.dataValues.version || " ")
          }
        },
        attributes: ['device_id'],
        raw: true
      }),
      models.Client.findAll({
        where: {
          client_version: oldVersion === 'false'? (mobileVersion.dataValues.version || " "): {
            [Op.not]: (mobileVersion.dataValues.version || " ")
          }
        },
        attributes: ['device_id'],
        raw: true
      }),
      models.Client.findAll({
        where: {
          client_version: oldVersion === 'false'? (serverVersion.dataValues.version || " "): {
            [Op.not]: (serverVersion.dataValues.version || " ")
          }
        },
        attributes: ['device_id'],
        raw: true
      })
    ])

    const desktopIds = desktopClients && desktopClients.map(client => client.device_id) || [];
    const mobileIds = mobileClients && mobileClients.map(client => client.device_id) || [];
    const serverIds = serverClients && serverClients.map(client => client.device_id) || [];

    const today = new Date()
    today.setHours(0, 0, 0, 0);

    const [
      // bridgeDeviceCount, 
      // clientDeviceCount, 
      totalMobileCount, 
      mobileCountWithVersion, 
      totalServerCount, 
      serverCountWithVersion, 
      totalDesktopCount, 
      desktopCountWithVersion , 
      allVersions, 
      todayConnectedClients, 
      yesterdayConnectedClients
    ] = await Promise.all([
      models.DeviceInventories.count({ where: {
        os: mobileOS
      }}),
      models.DeviceInventories.count({ where: {
        os: mobileOS,
        id: mobileIds
      }}),
      models.DeviceInventories.count({ where: {
        [Op.or]: [
          {
            [Op.and]: [
              {
                os: {
                  [Op.notIn]: mobileOS
                }
              },
              {
                os: {
                  [Op.notIn]: desktopOs
                }
              }
            ],
          }, {
            device_type: 'Thin Client (Bridge Client)'
          }
        ]
      }}),
      models.DeviceInventories.count({ where: {
        [Op.or]: [
          {
            [Op.and]: [
              {
                os: {
                  [Op.notIn]: mobileOS
                }
              },
              {
                os: {
                  [Op.notIn]: desktopOs
                }
              }
            ],
          }, {
            device_type: 'Thin Client (Bridge Client)'
          }
        ],
        id: serverIds
      }}),
      models.DeviceInventories.count({ where: {
        os: desktopOs
      }}),
      models.DeviceInventories.count({ where: {
        os: desktopOs,
        id: desktopIds
      }}),
      models.Update.findAll({
        attributes: [
          [models.sequelize.fn('DISTINCT', models.sequelize.col('buildVersion')) ,'version'],
      ],
      raw: true
      }),
      models.Connection.findAll({
        where: {
          [Op.and]: [
            {
              CONNECTION_ATTEMPT_TIME: {
                [Op.lte]: moment().toDate()
              }
            },
            {
              CONNECTION_ATTEMPT_TIME: {
                [Op.gte]: moment(today).toDate()
              }
            }
          ]
        },
        attributes: [
          [models.sequelize.fn('DISTINCT', models.sequelize.col('agent_id')) ,'agent_id'],
        ],
      }),
      models.Connection.findAll({
        where: {
          [Op.and]: [
            {
              CONNECTION_ATTEMPT_TIME: {
                [Op.lte]: moment(today).toDate()
              }
            },
            {
              CONNECTION_ATTEMPT_TIME: {
                [Op.gte]: moment(today).subtract(1, 'days').toDate()
              }
            }
          ]
        },
        attributes: [
          [models.sequelize.fn('DISTINCT', models.sequelize.col('agent_id')) ,'agent_id'],
        ],
      })
    ])

    const todayClientIds = todayConnectedClients.map(connection => connection.agent_id)
    const yesterdayClientIds = yesterdayConnectedClients.map(connection => connection.agent_id)

    const [todayClients, yesterdayClients, todayServers, yesterdayServers] = await Promise.all([
      models.Client.count({ 
        where: {
          id: todayClientIds,
          client_type: 'Client'
        }
      }),
      models.Client.count({ 
        where: {
          id: yesterdayClientIds,
          client_type: 'Client'
        }
      }),
      models.Client.count({ 
        where: {
          id: todayClientIds,
          client_type: 'Server'
        }
      }),
      models.Client.count({ 
        where: {
          id: yesterdayClientIds,
          client_type: 'Server'
        }
      }),
    ])

    const data = [{
      name: "Client Based",
      today: todayClients,
      yesterday: yesterdayClients
    }, {
      name:  "Bridge Based",
      today: todayServers,
      yesterday: yesterdayServers
    }];

    const responseData = {
      devices: data,
      percentageByTypes: {
        mobile: Math.round((mobileCountWithVersion/totalMobileCount) * 100) || 0,
        totalMobile: totalMobileCount,
        mobileCountWithVersion: mobileCountWithVersion,
        server: Math.round((serverCountWithVersion/totalServerCount) * 100) || 0,
        totalServer: totalServerCount,
        serverCountWithVersion,
        desktop: Math.round((desktopCountWithVersion/totalDesktopCount) * 100) || 0,
        totalDesktop: totalDesktopCount,
        desktopCountWithVersion
      },
      versions: allVersions.map(update => {
        return update.version
      })
    };

    // const setexAsync = util.promisify(redisClient.setex).bind(redisClient);

    // // await setexAsync(`getDevicesCount`, 1800, JSON.stringify(responseData));
    // await setexAsync(`getDevicesCount`, 60, JSON.stringify(responseData));

    // // console.log("DB Data: ", responseData);

    // const endTime = Date.now();
    // const executionTime = endTime - startTime;
    // console.log(`Execution Time without cache: ${executionTime}ms`);

    res.status(200).json(responseData);
  } catch(e) {
    next(e);
  }
}


const getClientsStatistics = async (req, res, next) => {
  try {
    const { type = 'client' } = req.params;

    const startTime = Date.now();
  
    const cachedData = await redisClient.getAsync(`getClientsStatistics:${type}`);
    
    if (cachedData) {
      const parsedData = JSON.parse(cachedData);
      const endTime = Date.now();
      const executionTime = endTime - startTime;
      console.log(`Execution Time (getClientsStatistics:${type}) with cache: ${executionTime}ms`);
      return res.status(200).json(parsedData);
    }

    const responseData = await getClientsStatisticsCron(type);

    res.status(200).json(responseData);
  } catch(e){
    next(e)
  }
}

const getClientsStatisticsCron = async (type) => {
  try {
    const startTime = Date.now();
  
    const options = {
      where: {},
      order: []
    };

    let startDate = utils.dateToEpoch(moment().subtract(2, 'days'));
    let endDate = utils.dateToEpoch(moment());

    options.where = {
      ...options.where,
      log_start_time: {
        $between: [startDate, endDate]
      },
      log_end_time: {
        $between: [startDate, endDate]
      }
    };

    const attributes = [ 
        ['createdAt', 'hours']
      ]
    if(type === 'client' ) {
      attributes.push([
        models.sequelize.fn(
          'sum',
          models.sequelize.literal(
            'client_tx_bytes / 1024'
          )
        ),
        'consumed'
      ])
    }

    if(type === 'service') {
      attributes.push([
        models.sequelize.fn(
          'sum',
          models.sequelize.literal(
            'service_tx_bytes / 1024'
          )
        ),
        'consumed'
      ])
    }

    const result = await models.ServiceAccessLogs.findAll({
      ...options,
      attributes: attributes,
      include: [
        {
          model: models.Service,
          attributes: [],
          required: true
        },
        {
          model: models.Client,
          attributes: [],
          required: true
        }
      ],
      group: ['hours'],
      raw: true
    });

    const uniqueClients = await models.sequelize.query(`select DISTINCT c1.AGENT_ID  from Connections c1 where c1.CONNECTION_ATTEMPT_TIME  > FROM_UNIXTIME(${startDate}) AND c1.AGENT_ID NOT in (select DISTINCT c1.AGENT_ID  from Connections c1 where c1.CONNECTION_ATTEMPT_TIME  < FROM_UNIXTIME(${startDate})) `)

    const responseData = {
      data: result,
      uniqueClientsCount: uniqueClients && uniqueClients[0] && uniqueClients[0].length || 0
    };

    const setexAsync = util.promisify(redisClient.setex).bind(redisClient);

    await setexAsync(`getClientsStatistics:${type}`, 3600 * 2, JSON.stringify(responseData));

    const endTime = Date.now();
    const executionTime = endTime - startTime;
    console.log(`Execution Time (getClientsStatistics:${type}) without cache: ${executionTime}ms`);

    return responseData;
  } catch(e){
    throw new httpErrors.BadRequest(error)
  }
}

const getTopSummary = async (req, res, next) => {
  try {
    // Top 5 Latest Services updated or Discovered

    // List of newly created services
    
    // Top 5 Latest Users updated
    
    // List of newly created/synced users
    
    // Top 5 Latest Devices onboarded
    
    // List of newly added devices
    
    // Top 5 Latest policies defined or updated
    
    // List of newly defined policies

    const startTime = Date.now();
  
    const cachedData = await redisClient.getAsync(`getTopSummary`);
    
    if (cachedData) {
      const parsedData = JSON.parse(cachedData);
      const endTime = Date.now();
      const executionTime = endTime - startTime;
      console.log(`Execution Time with cache: ${executionTime}ms`);
      return res.status(200).json(parsedData);
    }

    const [topServices] = await Promise.all([
      model.Services.findAll({
        limit: 5,
        order: [['created_at', 'desc']]
      })
    ])

    const responseData = topServices;

    const setexAsync = util.promisify(redisClient.setex).bind(redisClient);

    // await setexAsync(`getTopSummary`, 1800, JSON.stringify(responseData));
    await setexAsync(`getTopSummary`, 3600 * 2, JSON.stringify(responseData));


    const endTime = Date.now();
    const executionTime = endTime - startTime;
    console.log(`Execution Time without cache: ${executionTime}ms`);

    res.status(200).json(responseData);
  } catch(error) {
    throw new httpErrors.BadRequest(error)
  }
}

module.exports = {
  getCount,
  markersClients,
  markersControllers,
  getOs,
  test,
  userServices,
  userVolumes,
  serviceUsers,
  serviceVolumes,
  graph,
  getDevicesCount,
  getDeviceStatistics,
  getClientsStatistics,
  getTopSummary,
  //DASHBOARD CRONS
  markersControllersCron,
  getOsCron,
  userServicesCron,
  userVolumesCron,
  serviceUsersCron,
  serviceVolumesCron,
  graphCron,
  getDeviceStatisticsCron,
  getClientsStatisticsCron
};
