var httpErrors = require('http-errors');
var models = require('../models');
var fs = require('fs');
var sanitize = require('sanitize-filename');
const csv = require('fast-csv');
const Openssl = require('../openssl/openssl.js');
const OpensslObj = new Openssl();
const crypto = require('crypto');
const {
  promisify
} = require('util');
const writeFile = promisify(fs.writeFile);
const { actionType, publishMsgFile } = require('../routes/common/event-publisher');
const { Severity } = require('../shared/enums/severity');

process.on('message', async (fileData) => {
  let deviceCounter = 1;
  let file = fileData;
  let encodedFileData = file.data.split(';base64,').pop();
  let fileName = sanitize(file.name);
  let licenseExpired = false;
  const filePath = '/var/www/html/admin-gui/storage/deviceInventories/' + fileName;
  const license = await models.License.findOne({
    order: [
        ['id', 'DESC']
    ],
    limit: 1
  });

  if(!license) {
    throw new httpErrors.Conflict('License not found.');
  }

  const clientCount = await models.Client.count({});

  await writeFile(filePath, encodedFileData, 'base64');
  await new Promise((resolve, reject) => {
    const csvStream = fs.createReadStream(filePath);
    csvStream.pipe(csv.parse({
        headers: false
    }))
    .on("data", function (data) {
        if(licenseExpired) {
          return;
        }
        deviceUploadHandler(data, deviceCounter++, null, license, clientCount)
        .then(resData => {
            if(resData.errorMessage != '') {
              if(resData.licenseExpired) {
                licenseExpired = true;
              }
              return models.AlertLogs.create({
                type: "Device Upload",
                message: resData.errorMessage,
                severity: Severity.LOW
              }, { transaction: null });
            }
            else {
              return resolve();
            }
        })
        .then(res => {
            models.sequelize.query(
              'UPDATE Clients SET agent_id = id where agent_id = 0;', { transaction: null }
            )
            .then(res => {
              return resolve();
            });
        });
    })
    .on("end", function() {
      setTimeout(() => {
        models.AlertLogs.create({
          type: "Device Upload",
          message: "Device CSV Upload Completed.",
          severity: Severity.INFORMATIONAL
        }, { transaction: null })
        .then(res => {
          return resolve();
        });
      }, deviceCounter * 60);
    })
    .on("error", function(error) {
        reject(error);
    })
  });
});

const getOSTag = osName => {
    switch(osName) {
      case 'Ubuntu Desktop':
        return 'ubuntu18';
      case 'Ubuntu Server':
        return 'ubuntuserver18';
      case 'Ubuntu Server (ARMv8)':
        return 'ubuntuserver20armv8';
      case 'Windows':
        return 'windows';
      case 'Windows Server':
        return 'windowsserver';
      case 'Macintosh':
        return 'macintosh';
      case 'iOS':
        return 'ios';
      case 'Android':
        return 'android';
      case 'CentOS Server':
        return 'centosserver';
      default:
        return osName;
    }
}

const deviceUploadHandler = async (data, deviceCounter, transaction, license, clientCount) => {
    if(deviceCounter == 1) {
      return {
        errorMessage: ''
      }
    }
    const regExVal = {
      uuid: RegExp(/^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$/),
      mac: RegExp(/^([0-9A-Fa-f]{2}[:]){5}([0-9A-Fa-f]{2})|([0-9a-fA-F]{4}\\.[0-9a-fA-F]{4}\\.[0-9a-fA-F]{4})$/),
      mobile: RegExp(/^[a-zA-Z0-9]{7,}$/)
    }
    const supportedDeviceTypes = ['Desktop', 'IoT Bridge Client', 'Laptop', 'Virtual Desktop', 'Physical Server', 'Smartphone', 'Tablet',
    'Thin Client (Bridge Client)', 'Server Bridge Client', 'Virtual Server', 'ARM Platform'];

    let errorMessage = '';
    do {
      let deviceType = data[0].trim();
      let deviceName = data[1].trim();
      let userEmailAddress = data[2].trim();
      let macAddress = data[3].trim();
      let deviceId = data[4].trim();
      let organization = data[5].trim();
      let department = data[6].trim();
      let os = data[7].trim();
      let iotDeviceType = data[8].trim();
      let hypervisorVersion = data[9].trim();
      let IP = data[10].trim();

      if(!deviceType || deviceType == '') {
        errorMessage = `Row ${deviceCounter}: Device Name not found.`;
        break;
      }

      if(supportedDeviceTypes.indexOf(deviceType) == -1) {
        errorMessage = `Row ${deviceCounter}: Unsupported device type.`;
        break;
      }

      if(deviceType === 'ARM Platform') {
        os = "ubuntuserver20armv8";
      }

      if(!deviceName || deviceName == '') {
        errorMessage = `Row ${deviceCounter}: Device Name not found.`;
        break;
      }

      if((!macAddress || macAddress == '') &&
      ['IoT Bridge Client', 'Server Bridge Client', 'Thin Client (Bridge Client)'].includes(deviceType)) {
        errorMessage = `Row ${deviceCounter}: MAC Address not found.`;
        break;
      }

      if((!!macAddress && macAddress != '') &&
      ['IoT Bridge Client', 'Server Bridge Client', 'Thin Client (Bridge Client)'].includes(deviceType)) {
        if(!regExVal.mac.test(macAddress)) {
          errorMessage = `Row ${deviceCounter}: Please provide a valid MAC Address.`;
          break;
        }
      }

      if((!deviceId || deviceId == '') &&
      !['IoT Bridge Client', 'Server Bridge Client', 'Thin Client (Bridge Client)'].includes(deviceType)) {
        errorMessage = `Row ${deviceCounter}: Device Identification not found.`;
        break;
      }

      if(['Desktop', 'Laptop', 'Virtual Desktop'].includes(deviceType)) {
        if(!regExVal.uuid.test(deviceId)) {
          errorMessage = `Row ${deviceCounter}: Please provide a valid UUID.`;
          break;
        }
        if(!os || os == '') {
          errorMessage = `Row ${deviceCounter}: OS information not found.`;
          break;
        }
        if(!['Macintosh', 'Ubuntu Desktop', 'Windows'].includes(os)) {
          errorMessage = `Row ${deviceCounter}: ${os} is an unsupported OS.`;
          break;
        }
      }

      if(['Smartphone', 'Tablet'].includes(deviceType)) {
        if(!regExVal.mobile.test(deviceId)) {
          errorMessage = `Row ${deviceCounter}: Device Id must be at least 7 characters long.`;
          break;
        }
        if(!os || os == '') {
          errorMessage = `Row ${deviceCounter}: OS information not found.`;
          break;
        }
        if(!['Android', 'iOS'].includes(os)) {
          errorMessage = `Row ${deviceCounter}: ${os} is an unsupported OS.`;
          break;
        }
      }

      if(['Physical Server'].includes(deviceType)) {
        if(!deviceId || deviceId == '') {
          errorMessage = `Row ${deviceCounter}: Device Id not found.`;
          break;
        }
        if(!os || os == '') {
          errorMessage = `Row ${deviceCounter}: OS information not found.`;
          break;
        }
        if(!['Ubuntu Server', 'Ubuntu Server (ARMv8)', 'CentOS Server'].includes(os)) {
          errorMessage = `Row ${deviceCounter}: ${os} is an unsupported OS.`;
          break;
        }
        if(!IP){
          errorMessage = `Row ${deviceCounter}: IP information not found.`;
          break;
        }
      }

      if(['Virtual Server'].includes(deviceType)) {
        if(!deviceId || deviceId == '') {
          errorMessage = `Row ${deviceCounter}: Device Id not found.`;
          break;
        }
        if(!os || os == '') {
          errorMessage = `Row ${deviceCounter}: OS information not found.`;
          break;
        }
        if(!['Ubuntu Server', 'CentOS Server', 'Windows Server'].includes(os)) {
          errorMessage = `Row ${deviceCounter}: ${os} is an unsupported OS.`;
          break;
        }
        if(!hypervisorVersion || hypervisorVersion == '') {
          errorMessage = `Row ${deviceCounter}: Hypervisor version not found.`;
          break;
        }
        if(!IP){
          errorMessage = `Row ${deviceCounter}: IP information not found.`;
          break;
        }
      }

      if(['ARM Platform'].includes(deviceType)) {
        if(!deviceId || deviceId == '') {
          errorMessage = `Row ${deviceCounter}: Device Id not found.`;
          break;
        }
      }

      if(['IoT Bridge Client'].includes(deviceType)) {
        if(!os || os == '') {
          errorMessage = `Row ${deviceCounter}: OS information not found.`;
          break;
        }
        if(!['Ubuntu Server (ARMv8)'].includes(os)) {
          errorMessage = `Row ${deviceCounter}: ${os} is an unsupported OS.`;
          break;
        }
        if(!iotDeviceType || iotDeviceType == '') {
          errorMessage = `Row ${deviceCounter}: IoT Device Type not found.`;
          break;
        }
      }

      if(!organization || organization == '') {
        errorMessage = `Row ${deviceCounter}: Organization not found.`;
        break;
      }

      if(!department || department == '') {
        errorMessage = `Row ${deviceCounter}: Department not found.`;
        break;
      }

      if(['IoT Bridge Client', 'ARM Platform',
      'Server Bridge Client', 'Thin Client (Bridge Client)'].includes(deviceType)) {
        if(userEmailAddress || userEmailAddress != '') {
          errorMessage = `Row ${deviceCounter}: Device Type is meant to be agentless.`;
          break;
        }
      }

      let isServerType = [
        'IoT Bridge Client', 'Physical Server', 'Thin Client (Bridge Client)', 'Server Bridge Client', 'Virtual Server', 'ARM Platform'
      ].indexOf(deviceType) != -1;

      if(['IoT Bridge Client', 'Thin Client (Bridge Client)', 'Server Bridge Client'].indexOf(deviceType) != -1) {
        deviceId = macAddress;
      }

      if(!!os && os != '') {
        os = getOSTag(os);
      }

      let [encKey, hmacKey] = await Promise.all([
        OpensslObj.generateEncKey(),
        OpensslObj.generateHMacKey()
      ]);
      if(encKey.error || hmacKey.error) {
        break;
      }
      let enc_key = encKey.key;
      let HMacKey = hmacKey.key;

      let duplicateDeviceName = await models.DeviceInventories.findOne({
        where: { device_name: deviceName }
      });
      if(duplicateDeviceName) {
        errorMessage = `Row ${deviceCounter}: Device with name '${deviceName}' already exists in the database.`;
        break;
      }

      let duplicateUUID = await models.DeviceInventories.findOne({
        where: { device_id: deviceId }
      });
      if(duplicateUUID) {
        errorMessage = `Row ${deviceCounter}: Device with ID/UUID '${deviceId}' is already present in the database.`;
        break;
      }

      let organizationRecord = await models.Organizations.findOne({
        where: { name: organization }
      });
      if(!organizationRecord) {
        errorMessage = `Row ${deviceCounter}: Organization '${organization}' not found.`;
        break;
      }

      let departmentRecords = await models.Departments.findAll({
        where: { name: department }
      });
      if(departmentRecords.length == 0) {
        errorMessage = `Row ${deviceCounter}: Department '${department}' not found.`;
        break;
      }
      let deptArray = [];
      departmentRecords.map(dept => deptArray.push(dept.organization_id));

      if(organizationRecord && deptArray.length != 0) {
        if(deptArray.indexOf(organizationRecord.id) == -1) {
          errorMessage = `Row ${deviceCounter}: Department '${department}' is not associated with Organization ${organization}.`;
          break;
        }
      }

      let userRecord = await models.LocalUsers.findOne({
        where: { email: userEmailAddress }
      });
      if(!userRecord && userEmailAddress != '') {
        errorMessage = `Row ${deviceCounter}: User with email address '${userEmailAddress}' not found.`;
      }
      if(!!userRecord && userEmailAddress != '') {
        if(departmentRecords[deptArray.indexOf(organizationRecord.id)].id != userRecord.department) {
          errorMessage = `Row ${deviceCounter}: User with email address '${userEmailAddress}' doesn't belong to department '${department}' of organization ${organization}.`;
          break;
        }
      }

      let insertedDevice = await models.DeviceInventories.create({
          deviceSerialNumber: deviceId,
          device_name: deviceName,
          device_type: deviceType,
          device_id: deviceId,
          deviceSerialNumber: deviceId,
          department: departmentRecords[deptArray.indexOf(organizationRecord.id)].id,
          mac_address: macAddress,
          os,
          os_version: '',
          userName: '',
          has_agent: true,
          ip_address: IP ? IP : ''
        }, {
          returning: true,
          transaction
      });

      if(userRecord) {
        if(clientCount + deviceCounter -1 > license.max_number_of_clients) {
          return {
            errorMessage: 'Client limit allowed by License has reached.',
            licenseExpired: true
          }
        }
        await models.DeviceInventories.update({ user_id: userRecord.id }, {
          where: { id: insertedDevice.id },
          transaction
        });

        const MAX_RETRIES = 10;
        let retries = 0;
        var createdClient;
        while (retries < MAX_RETRIES) {
          try {
              createdClient = await models.Client.create({
              user: userRecord.userName,
              device_hardware_id: crypto
                .createHash('md5')
                .update(insertedDevice.device_id + '')
                .digest('hex'),
              device_os: os ? os : null,
              enc_key,
              HMacKey,
              agent_id: 0,
              is_local: userRecord.identity_provider == 'Local User' ? 1 : 0,
              client_type: isServerType ? 'Server' : 'Client',
              Name: '{' + userRecord.name + '}@{' + deviceName + '}',
              Email: userRecord.email,
              client_status: 1,
              access_restrictions: 0,
              device_os_full_name: os,
              user_id: userRecord.id,
              device_id: insertedDevice.id,
              serial_number: deviceId,
              client_mfa: userRecord.client_mfa,
              is_strong_mfa: userRecord.is_strong_mfa,
              biometric_state: userRecord.biometric_state,
              client_darkening: false,
              ENC_KEY_LEN: 32,
              HMAC_KEY_LEN: 24
            }, {
              returning: true,
              transaction
            });
            await models.ClientSoftwareInventory.create({
              client_id: createdClient.id
            }, { returning: true, transaction, raw: true, plain: true });
            break;
          }
          catch (error) {
            retries++;
            console.error(`Client creation failed. Retrying... (${retries}/${MAX_RETRIES})`, error);
            await new Promise(resolve => setTimeout(resolve, 50));
          }
        }

        await publishMsgFile(actionType.ADDED, {
          id: createdClient.id,
          data: {
              enc_key: createdClient.enc_key,
              ENC_KEY_LEN: createdClient.ENC_KEY_LEN,
              HMacKey: createdClient.HMacKey,
              HMAC_KEY_LEN: createdClient.HMAC_KEY_LEN,
              device_hardware_id: createdClient.device_hardware_id
          },
          type: createdClient.client_type
        })
      }
    } while (false);

    return {
      errorMessage
    }
  }