const { fork } = require('child_process');
var httpErrors = require('http-errors');
var models = require('../models');
var fs = require('fs');
var moment = require('moment');
var fsExtra = require('fs-extra');
var csvWriter = require('csv-writer').createObjectCsvWriter;
var sanitize = require('sanitize-filename');
const { job } = require('../cronjobs/device_validation');
const csv = require('fast-csv');
const Openssl = require('../openssl/openssl.js');
const crypto = require('crypto');
const OpensslObj = new Openssl();
const { promisify } = require('util');
const readFile = promisify(fs.readFile);
const writeFile = promisify(fs.writeFile);
const path = require('path');
const {
  actionType,
  publishDevices,
  publishClients,
  publishMsgFile,
  clientColumnsToPublish,
  publishSoftwarePolicy
} = require('../routes/common/event-publisher');
const { createNewClient } = require('./localUsers');
const { Op } = require('sequelize');
const { notEmptyQueryParam } = require('../routes/common/helper');

const {
  getDeviceUpdateMessage,
  getDeviceCreateMessage,
  getDeviceDeleteMessage,
  getDeviceValidationMessage
} = require('../config/adminLogs.config');

const { insertIntoAdminLogs } = require('../routes/common/adminLogs');

const getOSTag = osName => {
  switch (osName) {
    case 'Ubuntu Desktop':
      return 'ubuntu18';
    case 'Ubuntu Server':
      return 'ubuntuserver18';
    case 'Ubuntu Server (ARMv8)':
      return 'ubuntuserver20armv8';
    case 'Windows':
      return 'windows';
    case 'Windows Server':
      return 'windowsserver';
    case 'Macintosh':
      return 'macintosh';
    case 'iOS':
      return 'ios';
    case 'Android':
      return 'android';
    case 'CentOS Server':
      return 'centosserver';
    default:
      return osName;
  }
};

const getDevices = async (req, res, next) => {
  try {
    const {
      limit,
      offset,
      sort = 'id',
      dir = 'desc',
      searchTerm = '',
      deviceType = '',
      organization = '',
      department = '',
      location = '',
      osVersion = '',
      userId = null,
      os = null,
      referenceMachineCheck
    } = req.query;

    let options = {
      limit: +limit,
      offset: +offset,
      order: [],
      include: [
        {
          model: models.Departments,
          attributes: ['id', 'name'],
          include: [
            {
              model: models.Organizations
            }
          ]
        }
      ],
      raw: true
    };
    if (notEmptyQueryParam(searchTerm)) {
      options.where = {
        [Op.or]: {
          device_name: {
            [Op.like]: `%${searchTerm.trim()}%`
          },
          device_id: {
            [Op.like]: `%${searchTerm.trim()}%`
          }
        }
      };
    }

    if (notEmptyQueryParam(deviceType)) {
      options.where = {
        ...options.where,
        device_type: {
          [Op.like]: `%${deviceType}%`
        }
      };
    }

    if (notEmptyQueryParam(organization)) {
      const organizationDoc = await models.Organizations.findOne({
        where: {
          name: organization
        }
      });
      if (!organizationDoc) {
        throw new httpErrors.NotFound('Organization not found.');
      }
      const departments = await models.Departments.findAll({
        where: {
          organization_id: organizationDoc.id
        }
      });
      if (departments.length == 0) {
        throw new httpErrors.NotFound(
          'No departments associated with the Organization.'
        );
      }
      let deptFiltering = [];
      departments.map(department => {
        deptFiltering.push(department.id);
      });
      options.where = {
        ...options.where,
        department: {
          [Op.in]: deptFiltering
        }
      };
    }

    if (notEmptyQueryParam(department)) {
      options.where = {
        ...options.where,
        department: {
          [Op.eq]: department
        }
      };
    }

    if (notEmptyQueryParam(location)) {
      options.where = {
        ...options.where,
        location: {
          [Op.eq]: location
        }
      };
    }

    if (notEmptyQueryParam(osVersion)) {
      options.where = {
        ...options.where,
        os_version: {
          [Op.like]: `%${osVersion}%`
        }
      };
    }

    if (userId) {
      options.where = {
        ...options.where,
        user_id: {
          [Op.eq]: userId
        }
      };
    }

    if (notEmptyQueryParam(os)) {
      options.where = {
        ...options.where,
        os
      };
    }

    const allowedSortValues = ['device_name', 'id'];

    if (allowedSortValues.includes(sort)) {
      options.order.push([models.sequelize.literal(sort), dir]);
    }

    if (referenceMachineCheck == 'true') {
      options.where = {
        ...options.where,
        is_reference_machine: true
      };
    }

    const result = await models.DeviceInventories.paginate(options);

    res.json(result);
  } catch (error) {
    next(error);
  }
};
const getWorkloadDevices = async (req, res, next) => {
  try {
    const {
      limit,
      offset,
      sort = 'id',
      dir = 'desc',
      searchTerm = '',
      deviceType = '',
      organization = '',
      department = '',
      location = '',
      osVersion = '',
      userId = null,
      os = null,
      referenceMachineCheck
    } = req.query;

    let options = {
      limit: +limit,
      offset: +offset,
      order: [],
      include: [
        {
          model: models.Departments,
          attributes: ['id', 'name'],
          include: [
            {
              model: models.Organizations
            }
          ]
        }
      ],
      raw: true
    };
    if (notEmptyQueryParam(searchTerm)) {
      options.where = {
        [Op.or]: {
          device_name: {
            [Op.like]: `%${searchTerm.trim()}%`
          },
          device_id: {
            [Op.like]: `%${searchTerm.trim()}%`
          }
        }
      };
    }

    if (notEmptyQueryParam(deviceType)) {
      options.where = {
        ...options.where,
        device_type: {
          [Op.like]: `%${deviceType}%`
        }
      };
    }

    if (notEmptyQueryParam(organization)) {
      const organizationDoc = await models.Organizations.findOne({
        where: {
          name: organization
        }
      });
      if (!organizationDoc) {
        throw new httpErrors.NotFound('Organization not found.');
      }
      const departments = await models.Departments.findAll({
        where: {
          organization_id: organizationDoc.id
        }
      });
      if (departments.length == 0) {
        throw new httpErrors.NotFound(
          'No departments associated with the Organization.'
        );
      }
      let deptFiltering = [];
      departments.map(department => {
        deptFiltering.push(department.id);
      });
      options.where = {
        ...options.where,
        department: {
          [Op.in]: deptFiltering
        }
      };
    }

    if (notEmptyQueryParam(department)) {
      options.where = {
        ...options.where,
        department: {
          [Op.eq]: department
        }
      };
    }

    if (notEmptyQueryParam(location)) {
      options.where = {
        ...options.where,
        location: {
          [Op.eq]: location
        }
      };
    }

    if (notEmptyQueryParam(osVersion)) {
      options.where = {
        ...options.where,
        os_version: {
          [Op.like]: `%${osVersion}%`
        }
      };
    }

    if (userId) {
      options.where = {
        ...options.where,
        user_id: {
          [Op.eq]: userId
        }
      };
    }

    if (notEmptyQueryParam(os)) {
      options.where = {
        ...options.where,
        os
      };
    }

    const allowedSortValues = ['device_name', 'id'];

    if (allowedSortValues.includes(sort)) {
      options.order.push([models.sequelize.literal(sort), dir]);
    }

    if (referenceMachineCheck == 'true') {
      options.where = {
        ...options.where,
        is_reference_machine: true
      };
    }

    options.where = {
      ...options.where,
      [Op.or]: [
        { device_type: 'Physical Server' },
        { device_type: 'Virtual Server' }
      ]
    };
    const result = await models.DeviceInventories.paginate(options);

    res.json(result);
  } catch (error) {
    next(error);
  }
};

const getAvailableDevices = async (req, res, next) => {
  try {
    const {
      serverSideDeviceFound,
      mobileDeviceFound,
      departmentId
    } = req.params;

    let excludedDeviceTypes = [
      'IoT Bridge Client',
      'Server Bridge Client',
      'Thin Client (Bridge Client)'
    ];

    if (serverSideDeviceFound != 'false') {
      excludedDeviceTypes = [
        ...excludedDeviceTypes,
        'Physical Server',
        'Virtual Server'
      ];
    }

    const mobileFilter =
      serverSideDeviceFound != 'false' && mobileDeviceFound == 'false'
        ? {
            device_type: { [Op.in]: ['Smartphone', 'Tablet'] }
          }
        : {};

    const options = {
      where: {
        user_id: {
          [Op.eq]: null
        },
        ...mobileFilter,
        [Op.not]: {
          device_type: excludedDeviceTypes
        }
      },
      attributes: ['device_name', 'id']
    };

    // const myOrganization = await models.sequelize.query(`
    //   SELECT d.id, d.name FROM LocalUsers as a
    //   left join Departments as b on a.department = b.id
    //   left join Organizations as c on c.id = b.organization_id
    //   left join Departments as d on d.organization_id = c.id
    //   where a.id = ${userId}
    // `)

    // if(myOrganization && myOrganization[0].length > 0) {

    const department = await models.Departments.findOne({
      where: {
        id: departmentId
      }
    });

    if (department) {
      options.where['department'] = department.id;
    }

    const result = await models.DeviceInventories.findAll(options);

    res.json(result);
  } catch (error) {
    next(error);
  }
};

const clientsforAvailableDevices = async (req, res, next) => {
  try {
    const { devices } = req.body;

    if(!devices || devices.length === 0) {
      return res.json([]);
    }

    const incomingDeviceIds = devices.map(device => device.id);

    const clientsData = await models.Client.findAll({
      attributes: ['device_id'],
      distinct: true,
    });

    const clientDeviceIdsSet = new Set(clientsData.map(client => client.device_id));

    const deviceWithNoClient = incomingDeviceIds.filter(deviceId => !clientDeviceIdsSet.has(deviceId));

    const filteredClientData = await models.Client.findAll({
      attributes: ['device_id'],
      where: {
        device_id: {
          [Op.in]: incomingDeviceIds,
        },
        user: {
          [Op.is]: null,
        },
      },
    });

    const deviceWithUserNull = filteredClientData.map(client => client.device_id);

    const allDevices = [...deviceWithNoClient, ...deviceWithUserNull];
    const uniqueDeviceIds = new Set(allDevices);

    const filteredDevices = devices.filter(device => uniqueDeviceIds.has(device.id));
  
    res.json(filteredDevices);
  } catch (error) {
    next(error);
  }
};

const getAllLocations = async (req, res, next) => {
  try {
    const locationResults = await models.sequelize.query(
      `SELECT DISTINCT location FROM DeviceInventories`
    );

    let locations = locationResults[0].map(row => {
      return row.location;
    });

    res.json(locations);
  } catch (error) {
    next(error);
  }
};

const devicesValidationTask = async (req, res, next) => {
  try {
    let manualTrigger = true;
    await job(manualTrigger);

    device = 'device';
    insertIntoAdminLogs(getDeviceValidationMessage(device), req);
    res.json('Task successful.');
  } catch (error) {
    next(error);
  }
};

const getDevicesForUser = async (req, res, next) => {
  try {
    const {
      limit,
      offset,
      sort = 'id',
      dir = 'desc',
      searchTerm = ''
    } = req.query;

    const user_id = req.params.id;

    const options = {
      limit: +limit,
      offset: +offset,
      order: [],
      include: [
        {
          model: models.Departments,
          attributes: ['id', 'name']
        }
      ]
    };

    if (searchTerm.trim()) {
      options.where = {
        [Op.or]: {
          device_name: {
            [Op.like]: `%${searchTerm.trim()}%`
          },
          device_id: {
            [Op.like]: `%${searchTerm.trim()}%`
          }
        }
      };
    }

    const allowedSortValues = ['name', 'id'];

    if (allowedSortValues.includes(sort)) {
      options.order.push([models.sequelize.literal(sort), dir]);
    }

    options.where = {
      ...options.where,
      user_id: {
        [Op.eq]: user_id
      }
    };

    const result = await models.DeviceInventories.paginate(options);

    res.json(result);
  } catch (error) {
    next(error);
  }
};

const deleteDevice = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
        const device = await models.DeviceInventories.findOne({
          where: {
            id: req.params.id,
          }
        });

        if (!device) {
          throw new httpErrors.NotFound('Device not found');
        }

        if(device.user_id) {
          throw new httpErrors.NotFound(`Device is bound to user ${device.userName}.`);
        }

        const manualMicrosegDevice = await models.S2sManualRule.findOne({
          where: {
            [Op.or]: [
              { source_ip: device.ip_address },
              { destination_ip: device.ip_address }
            ]
          }
        })

        const autoMicrosegDevice = await models.S2sDiscoveryData.findOne({
          where: {
            [Op.or]: [
              { source_ip: device.ip_address },
              { destination_ip: device.ip_address }
            ]
          }
        })

        if(manualMicrosegDevice || autoMicrosegDevice) {
          throw new httpErrors.BadRequest("Can't delete this device because it is a part of a microsegment.")
        }

        await models.Client.destroy({
          where: {
            device_id: device.id,
            user: null
          }
        })

        const microsegmentSources = await models.Microsegments.findAll({
          where: {
            deviceId: device.id
          }
        })
        if(microsegmentSources && microsegmentSources.length > 0) {
          throw new httpErrors.BadRequest("Can't delete this device because it is a part of a microsegment.")
        }

        const bridgeDevice = await models.ServerBridgeDeviceId.findOne({
          where: {
            device_id: req.params.id,
          }
        });

        if (bridgeDevice) {
          const associatedBridge = await models.ServerBridge.findOne({
            where: {
              id: bridgeDevice.serverBridgeId,
            }
          });
          throw new httpErrors.NotFound(`Device is bound to bridge ${associatedBridge.name}.`);
        }

        await device.destroy({transaction});

        await transaction.commit();

        insertIntoAdminLogs(
          getDeviceDeleteMessage(device.device_name),
          req
        )

        res.json('Device deleted successfully');

    } catch (error) {
        await transaction.rollback();
        next(error);
    }
};

const getDevice = async (req, res, next) => {
  try {

    const { id } = req.params;

    // Validate or sanitize the `id` to ensure it's of the expected type
    if (!/^\d+$/.test(id)) {
      return res.status(400).json({ error: "Invalid device ID" });
    }

    const options = {
      where: {
        id: id
      }
    };

    const deviceData = await models.DeviceInventories.findOne(options);

    const clientSetting = await models.ClientSetting.findOne({});

    const query = `SELECT id, workloadId, deviceId 
                   FROM sdp.WorkloadDevicePivots 
                   WHERE deviceId = :deviceId;`;

    const deviceExist = await models.sequelize.query(query, {
      replacements: { deviceId: id }, // Replaces `:deviceId` safely
      type: models.sequelize.QueryTypes.SELECT
    });

    const response = {
      deviceData,
      global_darkening: clientSetting.global_darkening,
      in_workload: deviceExist.length > 0 ? true : false
    };

    res.json(response);
  } catch (error) {
    next(error);
  }
};

const getGlobalDarkening = async (req, res, next) => {
  try {
    const clientSetting = await models.ClientSetting.findOne({});

    const response = {
      global_darkening: clientSetting.global_darkening
    };

    res.json(response);
  } catch (error) {
    next(error);
  }
};

const createDevice = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    let {
      userName = '',
      deviceTag = '',
      device_name = '',
      device_type = '',
      device_id = 0,
      department = '',
      location = '',
      ip_address = '',
      mac_address = '',
      host_name = '',
      model = '',
      manufacturer = '',
      device_groups,
      iot_device_type = '',
      hypervisor_version = '',
      user_id = null,
      darkening_state = false,
      os,
      os_version = '',
      risk_score = 0,
      is_reference_machine = false,
      geoip_track = true
    } = req.body.device;

    if (department != '') {
      department = department.deptId;
    }

    if (
      [
        'IoT Bridge Client',
        'Thin Client (Bridge Client)',
        'Server Bridge Client'
      ].indexOf(device_type) != -1
    ) {
      mac_address = device_id;
    }

    const deviceSearch1 = await models.DeviceInventories.findOne({
      where: {
        device_id
      }
    });

    if (deviceSearch1) {
      throw new httpErrors.NotFound(
        'Device already exists with ID: ' + device_id
      );
    }

    const deviceSearch2 = await models.DeviceInventories.findOne({
      where: {
        device_name
      }
    });

    if (deviceSearch2) {
      throw new httpErrors.NotFound(
        'Device already exists with name: ' + device_name
      );
    }

    if (device_type === 'ARM Platform') {
      os = 'ubuntuserver20armv8';
    }

    if (
      (device_type === 'Physical Server' || device_type === 'Virtual Server') &&
      !ip_address
    ) {
      throw new httpErrors.BadRequest('Ip address is required');
    } else if (device_type === 'Physical Server' && ip_address) {
      console.log('ip_address', ip_address);

      const ipExistsQuery = `
      SELECT DISTINCT 
        workloads.id, 
        workloads.name, 
        deviceInventories.device_name, 
        deviceInventories.ip_address  
      FROM Workloads AS workloads
      INNER JOIN WorkloadDevicePivots AS workloadDevicePivots 
        ON workloadDevicePivots.workloadId = workloads.id 
      INNER JOIN DeviceInventories AS deviceInventories 
        ON deviceInventories.id = workloadDevicePivots.deviceId 
      WHERE deviceInventories.ip_address = :ip_address`;
      console.log(ipExistsQuery);

      const ipExistsResult = (
      await models.sequelize.query(ipExistsQuery, {
        replacements: { ip_address }, // Bind parameter
        type: models.sequelize.QueryTypes.SELECT, // Ensure proper type
      })
    );
      console.log('hello', ipExistsResult);
      // if(ipExistsResult.length > 0) throw new httpErrors.BadRequest("Ip address must be unique")
    }

    let deviceGroupString = '';
    if (device_groups) {
      for (var index = 0; index < device_groups.length; index++) {
        deviceGroupString += device_groups[index] + ', ';
      }
    }

    if (ip_address) {
      const isValid = ValidateIPaddress(ip_address);
      if (!isValid)
        throw new httpErrors.BadRequest(
          'You have entered an invalid IP address!'
        );
    }

    const createdDevice = await models.DeviceInventories.create(
      {
        userName,
        deviceSerialNumber: device_id,
        deviceTag,
        device_name,
        device_type,
        device_id,
        department,
        location,
        ip_address,
        mac_address,
        host_name,
        model,
        manufacturer,
        device_groups: deviceGroupString,
        iot_device_type,
        hypervisor_version,
        user_id,
        has_agent: false,
        darkening_state,
        os,
        os_version,
        risk_score,
        is_reference_machine,
        geoip_track
      },
      {
        transaction
      }
    );

    if (user_id) {
      const userRecord = await models.LocalUsers.findOne({
        where: {
          id: user_id
        }
      });

      if (userRecord) {
        await createNewClient(userRecord, createdDevice, transaction);
      } else {
        throw new httpErrors.NotFound('user not found');
      }
    }

    if (!user_id) {
      if (
        device_type === 'Physical Server' ||
        device_type === 'Virtual Server'
      ) {
        //Create S2S Client
        // const existingClients = await models.Client.findAll({
        //   where: {
        //     device_id: dbDevices
        //   }
        // });
        // if (existingClients && existingClients.length > 0) {
        //   throw new httpErrors.BadRequest('Invalid devices');
        // }
        let dbDevices = [createdDevice.id];
        const [encKeys, hmacKeys] = await Promise.all([
          Promise.all(dbDevices.map(x => OpensslObj.generateEncKey())),
          Promise.all(dbDevices.map(x => OpensslObj.generateHMacKey()))
        ]);

        const workloadName = 'Default';

        const client = {
          Name: '{' + 'S2S Client' + '}@{' + createdDevice.device_name + '}',
          ipv4_address: createdDevice.ip_address,
          device_os: createdDevice.os,
          client_type: 'Server',
          client_status: 1,
          risk_score: createdDevice.risk_score,
          client_darkening: createdDevice.darkening_state,
          client_os_version: '',
          serial_number: createdDevice.device_id + '',
          device_hardware_id: crypto
            .createHash('md5')
            .update(createdDevice.device_id + '')
            .digest('hex'),
          agent_id: 0,
          enc_key: encKeys[0].key,
          HMacKey: hmacKeys[0].key,
          is_local: 1,
          device_id: createdDevice.id,
          auto_updates: false,
          user_auth_required: false
        };

        const dbClient = await models.Client.create(client, { transaction });

        const softwareInventoryData = {
          client_id: dbClient.id
        };
        await models.ClientSoftwareInventory.create(softwareInventoryData, {
          transaction
        });

        await models.sequelize.query('UPDATE Clients SET agent_id = id;', {
          transaction
        });

        const workload = await models.Workloads.findOne({
          where: {
            name: workloadName
          }
        });

        if (!workload) {
          throw new httpErrors.NotFound('Workload not found');
        }

        const pivot = {
          workloadId: workload.id,
          deviceId: dbClient.device_id,
          clientId: dbClient.id
        };

        await models.WorkloadDevicePivots.create(pivot, { transaction });
      }
    }
    await transaction.commit();

    insertIntoAdminLogs(getDeviceCreateMessage(device_name), req);

    res.json('Device created successfully');
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const updateDevice = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    let {
      deviceTag = '',
      device_name = '',
      device_type = '',
      device_id = 0,
      department = '',
      location = '',
      ip_address = '',
      mac_address = '',
      host_name = '',
      model = '',
      manufacturer = '',
      device_groups,
      iot_device_type = '',
      hypervisor_version = '',
      darkening_state = false,
      os = '',
      os_version = '',
      risk_score = 0,
      is_reference_machine = false,
      geoip_track = true
    } = req.body.device;

    if (department != '') {
      department = department.deptId;
    }

    if (
      [
        'IoT Bridge Client',
        'Thin Client (Bridge Client)',
        'Server Bridge Client'
      ].indexOf(device_type) != -1
    ) {
      mac_address = device_id;
    }

    if (device_type === 'ARM Platform') {
      os = 'ubuntuserver20armv8';
    }

    let deviceGroupString = '';
    if (device_groups) {
      for (var index = 0; index < device_groups.length; index++) {
        deviceGroupString += device_groups[index] + ', ';
      }
    }

    const updateClause = {
      deviceSerialNumber: device_id,
      deviceTag,
      device_name,
      device_type,
      device_id,
      department,
      location,
      ip_address,
      mac_address,
      host_name,
      model,
      manufacturer,
      device_groups: deviceGroupString,
      iot_device_type,
      hypervisor_version,
      darkening_state,
      os,
      os_version,
      risk_score,
      is_reference_machine,
      geoip_track
    };

    const deviceToUpdate = await models.DeviceInventories.findOne({
      where: {
        id: req.params.id
      }
    });

    if (!deviceToUpdate) {
      throw new httpErrors.NotFound('No device found to update');
    }

    const updatedDevice = await deviceToUpdate.update(updateClause, {
      transaction
    });

    // await models.DeviceInventories.update(updateClause, {
    //     where: {
    //       id: req.params.id
    //     },
    //     transaction
    // });

    const relevantClients = await models.Client.findAll({
      where: {
        device_id: req.params.id
      }
    });

    const isBridgeDevice =
      deviceToUpdate.device_type === 'Server Bridge Client';
    if (isBridgeDevice) {
      os = 'ubuntuserver18';
    }

    for (let j = 0; j < relevantClients.length; j++) {
      const isWorkloadDevice = relevantClients[j].user === null;
      let arr = relevantClients[j].Name.split('}@{');
      if (!(isBridgeDevice || isWorkloadDevice)) {
        // It's a normal user@device client, so update device part
        // which is after @
        relevantClients[j].Name = arr[0] + '}@{' + device_name + '}';
      } else {
        // It's either a server bridge slave client or s2s workload client
        // Both have client name as device@<bridge name or workload name>
        // So, update device part which is before @
        relevantClients[j].Name = '{' + device_name + '}@{' + arr[1];
      }
      relevantClients[j].device_os = os;
      relevantClients[j].device_os_full_name = os;
      relevantClients[j].geoip_track = geoip_track;
      relevantClients[j].client_darkening = darkening_state;
      await relevantClients[j].save({ transaction });
    }

    let changedColumns = Object.keys(updatedDevice._changed);

    if (changedColumns.includes('darkening_state')) {
      const otherClients = relevantClients.filter(
        _c => _c.client_type == 'Client'
      );
      const serverClients = relevantClients.filter(
        _c => _c.client_type == 'Server'
      );

      await Promise.all([
        otherClients.length
          ? publishClients(actionType.UPDATED, {
              ids: otherClients.map(x => x.id),
              data: {
                client_darkening: darkening_state
              },
              type: 'Client'
            })
          : [],

        serverClients.length
          ? publishClients(actionType.UPDATED, {
              ids: serverClients.map(x => x.id),
              data: {
                client_darkening: darkening_state
              },
              type: 'Server'
            })
          : []
      ]);
    }

    await transaction.commit();

    await publishDevices(actionType.UPDATED, {
      id: parseInt(req.params.id)
    });

    insertIntoAdminLogs(getDeviceUpdateMessage(device_name), req);

    res.json('Device updated successfully');
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const getClients = async (req, res, next) => {
  try {
    const response = await models.Client.findAll({
      attributes: ['Name', 'agent_id'],
      where: {
        device_id: req.params.id
      }
    });
    res.json(response);
  } catch (error) {
    next(error);
  }
};

const toggleDarkeningState = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    const deviceRecord = await models.DeviceInventories.findOne({
      where: {
        id: req.params.id
      }
    });

    if (!deviceRecord) {
      throw new httpErrors.NotFound('Device Not Found.');
    }

    const currentState = deviceRecord.darkening_state;

    deviceRecord.darkening_state = !currentState;

    await deviceRecord.save({ transaction });

    const relevantClient = await models.Client.findOne({
      where: {
        device_id: req.params.id,
        user: {
          [Op.not]: null
        }
      }
    });

    if (relevantClient) {
      relevantClient.client_darkening = !currentState;
      await relevantClient.save({ transaction });
      await publishClients(actionType.UPDATED, {
        ids: [relevantClient.id],
        data: {
          client_darkening: !currentState
        },
        type: relevantClient.client_type
      });
    }

    await transaction.commit();

    res.json('Device updated successfully');
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const uploadCsv = (req, res, next) => {
  try {
    const child = fork(__dirname + '/deviceUploadTask.js');
    child.send(req.body.fileData);
    res.json([]);
  } catch (error) {
    next(error);
  }
};

const downloadCsv = async (req, res, next) => {
  try {
    const samplePath = path.join(
      __dirname,
      '../storage/device-inventory-sample.csv'
    );
    return res.download(samplePath);

    // const filePath = '/var/www/html/admin-gui/storage/deviceInventories/device-inventory.csv';

    // const csvFile = csvWriter({
    //   path: filePath,
    //   header: [{
    //       id: 'userID',
    //       title: 'UserID'
    //     },
    //     {
    //       id: 'userEmailAddress',
    //       title: 'User Email Address'
    //     },
    //     {
    //       id: 'deviceName',
    //       title: 'Device Name*'
    //     },
    //     {
    //       id: 'deviceType',
    //       title: 'Device Type*'
    //     },
    //     {
    //       id: 'macAddress',
    //       title: 'MAC Address*'
    //     },
    //     {
    //       id: 'uuid',
    //       title: 'UUID*'
    //     },
    //     {
    //       id: 'organization',
    //       title: 'Organization'
    //     },
    //     {
    //       id: 'department',
    //       title: 'Department'
    //     },
    //     {
    //       id: 'manufacturer',
    //       title: 'Manufacturer'
    //     },
    //     {
    //       id: 'model',
    //       title: 'Model'
    //     },
    //     {
    //       id: 'os',
    //       title: 'OS'
    //     },
    //     {
    //       id: 'osVersion',
    //       title: 'OS Version'
    //     }
    //   ]
    // });

    // await csvFile.writeRecords([{
    //   userID: '',
    //   userEmailAddress: '',
    //   deviceName: '',
    //   deviceType: '',
    //   macAddress: '',
    //   uuid: '',
    //   organization: '',
    //   department: '',
    //   manufacturer: '',
    //   model: '',
    //   os: '',
    //   osVersion: ''
    // }]);

    // const fileData = await readFile(filePath, 'utf8');

    // var linesExceptFirst = fileData.split('\n').slice(1).join('\n');

    // await writeFile(filePath, linesExceptFirst);

    // res.download(filePath);
    // fsExtra.remove(filePath)
  } catch (error) {
    next(error);
  }
};

const bulkActions = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    const {
      selectAll = false,
      selectedIds = [],
      organization = '',
      device_type = '',
      department = '',
      location = '',
      os = '',
      searchKey = '',
      type = 'darkening_state',
      status = false
    } = req.body;

    const deviceActions = {
      client_mfa: status => ({ client_mfa: status }),
      biometric_state: status => ({ biometric_state: status }),
      is_strong_mfa: status => ({ is_strong_mfa: status }),
      darkening_state: status => ({ darkening_state: status }),
      reference_machine: status => ({ is_reference_machine: status })
    };

    const clientActions = {
      client_mfa: status => ({ client_mfa: status }),
      biometric_state: status => ({ biometric_state: status }),
      is_strong_mfa: status => ({ is_strong_mfa: status }),
      darkening_state: status => ({ client_darkening: status })
    };

    const options = {
      attributes: ['id', 'inventory_sync_state', 'device_name'],
      where: {
        id: {
          ...(selectAll
            ? {
                [Op.notIn]: selectedIds
              }
            : {
                [Op.in]: selectedIds
              })
        },
        ...(searchKey
          ? {
              [Op.or]: {
                device_name: {
                  [Op.like]: `%${searchKey.trim()}%`
                },
                device_id: {
                  [Op.like]: `%${searchKey.trim()}%`
                }
              }
            }
          : {}),
        ...(location ? { location } : {}),
        ...(device_type
          ? { device_type: { [Op.like]: `%${device_type}%` } }
          : {})
      }
    };

    const Devices = await models.DeviceInventories.findAll(options, {
      transaction
    });

    if (os) {
      options.where = {
        ...options.where,
        os
      };
    }

    if (organization) {
      const myOrganization = await models.Organizations.findOne({
        where: {
          name: organization
        },
        attributes: ['id']
      });
      if (myOrganization) {
        const departmentOptions = {
          organization_id: myOrganization.id
        };
        if (department) {
          departmentOptions.id = department;
        }
        const myDepartments = await models.Departments.findAll({
          where: departmentOptions
        });
        if (myDepartments && myDepartments.length > 0) {
          options.where.department = {
            [Op.in]: myDepartments.map(department => department.id)
          };
        }
      }
    }

    const [updatedDevices] = await Promise.all([
      models.DeviceInventories.findAll(options, { transaction }),
      models.DeviceInventories.update(
        { ...deviceActions[type](status) },
        { ...options, transaction }
      )
    ]);
    if (type != 'reference_machine') {
      const [updatedClients] = await Promise.all([
        models.Client.findAll(
          {
            attributes: ['id', 'client_type'],
            where: {
              device_id: {
                [Op.in]: updatedDevices.map(x => x.id)
              }
            }
          },
          { transaction }
        ),
        models.Client.update(
          { ...clientActions[type](status) },
          {
            where: {
              device_id: {
                [Op.in]: updatedDevices.map(x => x.id)
              }
            }
          },
          { transaction }
        )
      ]);

      const otherClients = updatedClients.filter(
        _c => _c.client_type == 'Client'
      );
      const serverClients = updatedClients.filter(
        _c => _c.client_type == 'Server'
      );

      await Promise.all([
        serverClients.length
          ? publishClients(actionType.UPDATED, {
              ids: serverClients.map(_ac => _ac.id),
              data: {
                ...clientActions[type](status)
              },
              type: 'Server'
            })
          : [],
        otherClients.length
          ? publishClients(actionType.UPDATED, {
              ids: otherClients.map(_ac => _ac.id),
              data: {
                ...clientActions[type](status)
              },
              type: 'Client'
            })
          : []
      ]);
    }

    await transaction.commit();

    for (const device of Devices) {
      insertIntoAdminLogs(getDeviceUpdateMessage(device.device_name), req);
    }

    res.send('Devices updated');
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const ValidateIPaddress = ipaddress => {
  if (
    /^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/.test(
      ipaddress
    )
  ) {
    return true;
  }
  return false;
};

module.exports = {
  getDevices,
  deleteDevice,
  getDevice,
  getWorkloadDevices,
  getAvailableDevices,
  clientsforAvailableDevices,
  createDevice,
  updateDevice,
  getDevicesForUser,
  toggleDarkeningState,
  uploadCsv,
  downloadCsv,
  bulkActions,
  getAllLocations,
  devicesValidationTask,
  getGlobalDarkening,
  getClients
};
