const models = require('../models');

const httpErrors = require('http-errors');
const _ = require('lodash');
const { convertToArray } = require('../utils/common');
const {
  actionType,
  publishGatewayGroups
} = require('../routes/common/event-publisher');
const { Sequelize } = require('sequelize');
const servicegroups = require('./servicegroups');
const {
  getGatewayGroupCreateMessage,
  getGatewayGroupEditMessage,
  getGatewayGroupDeleteMessage
} = require('../config/adminLogs.config');

const { insertIntoAdminLogs } = require('../routes/common/adminLogs');

Sequelize.options.deprecations = { operatorsAliases: false };

/**
 * Get the list of gateway groups
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
module.exports.getGroups = async (req, res, next) => {
  try {
    const { limit, offset, sort = 'name', dir = 'asc', name } = req.query;

    const options = {
      where: {},
      limit,
      offset,
      order: [],
      distinct: true
    };

    const allowedSortValues = ['name', 'createdAt', 'totalGateways'];

    if (allowedSortValues.includes(sort)) {
      options.order.push([models.sequelize.literal(sort), dir]);
    }

    if (name) {
      options.where = {
        ...options.where,
        name: {
          [Op.like]: '%' + name + '%'
        }
      };
    }

    const result = await models.GatewayGroups.paginate({
      ...options,
      subQuery: false,
      attributes: [
        'updatedAt',
        'createdAt',
        'name',
        'description',
        'id',
        [
          Sequelize.fn('COUNT', Sequelize.col('GatewayGroupsGateways.id')),
          'totalGateways'
        ]
      ],
      include: [
        {
          model: models.GatewayGroupPivot,
          as: 'GatewayGroupsGateways',
          required: false,
          attributes: []
        }
      ],
      group: ['name', 'GatewayGroups`.`id', 'GatewayGroups.createdAt']
    });

    res.json(result);
  } catch (error) {
    next(error);
  }
};

/**
 * Get the gateway group by id
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
module.exports.getGroupById = async (req, res, next) => {
  const { id } = req.params;
  try {
    const group = await models.GatewayGroups.findOne({
      where: {
        id
      }
    });

    if (!group) throw new httpErrors.NotFound('Group not found');

    const gateways = await models.Gateway.findAll({
      include: [
        {
          model: models.GatewayGroupPivot,
          attributes: [],
          as: 'GatewayGatewayGroups',
          required: true,
          where: {
            gateway_group_id: id
          }
        }
      ]
    });

    return res.json({
      ...group.toJSON(),
      gateways: gateways
    });
  } catch (error) {
    next(error);
  }
};

/**
 * Create a gateway group
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
module.exports.createGroup = async (req, res, next) => {
  const { name, gateways, description, addAllGateways = false } = req.body;
  let transaction = null;

  const obj = {
    name
  };

  let gatewaysIdsArray = convertToArray(gateways) || [];
  let clonedGWArray = [];

  for (let j = 0; j < gatewaysIdsArray.length; j++) {
    let currGW = await models.Gateway.findOne({
      where: {
        id: gatewaysIdsArray[j]
      },
      attributes: ['Name']
    });

    let clonedGW = await models.Gateway.findAll({
      where: {
        Name: {
          [Op.like]: `%${currGW.Name.trim()}%`
        }
      },
      attributes: ['id']
    });

    clonedGW = clonedGW.map(gateway => gateway.id);

    for (let k = 0; k < clonedGW.length; k++) {
      if (!gatewaysIdsArray.includes(clonedGW[k]))
        clonedGWArray.push(clonedGW[k]);
    }
  }
  gatewaysIdsArray = [...gatewaysIdsArray, ...clonedGWArray];

  if (
    addAllGateways === true &&
    (!gatewaysIdsArray || gatewaysIdsArray.length < 1)
  ) {
    let allGateways = await models.Gateway.findAll({
      where: {},
      attributes: ['id']
    });

    gatewaysIdsArray = allGateways.map(gateway => gateway.id);
  }

  const gatewayGroupObject = {
    name,
    GatewayGroupsGateways: gatewaysIdsArray.map(g_id => ({ gateway_id: g_id })),
    description
  };

  try {
    transaction = await models.sequelize.transaction();
    const group = await models.GatewayGroups.create(gatewayGroupObject, {
      include: [
        {
          model: models.GatewayGroupPivot,
          as: 'GatewayGroupsGateways'
        }
      ],
      transaction
    });

    await publishGatewayGroups(actionType.ADDED, {
      id: group.id,
      all: 0,
      new_ids: gatewaysIdsArray,
      del_ids: []
    });

    await transaction.commit();

    insertIntoAdminLogs(getGatewayGroupCreateMessage(name), req);

    return res.json(group);
  } catch (error) {
    if (transaction) {
      await transaction.rollback();
    }
    if (error.name === 'SequelizeUniqueConstraintError') {
      error.message = 'Group with this name already exist';
    }
    next(error);
  }
};

/**
 * Update group
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
module.exports.updateGroup = async (req, res, next) => {
  const { id } = req.params;

  let {
    name,
    gateways,
    description,
    appendGateways = false,
    addAllGateways = false
  } = req.body;

  gateways = gateways.map(gateway => parseInt(gateway));
  if (addAllGateways === true) {
    const allGateways = await models.Gateway.findAll({});
    gateways = allGateways.map(gateway => gateway.id);
  }

  const transaction = await models.sequelize.transaction();
  try {
    const gatewayGroup = await models.GatewayGroups.findOne({
      attributes: ['id', 'name'],
      where: { id },
      include: [
        {
          attributes: ['gateway_id', 'gateway_group_id'],
          model: models.GatewayGroupPivot,
          as: 'GatewayGroupsGateways'
        }
      ],
      transaction
    });

    if (!gatewayGroup) throw new httpErrors.NotFound('Group not found');

    const gatewaysIdList = gatewayGroup.GatewayGroupsGateways.map(
      ggp => ggp.gateway_id
    );

    let gatewaysToAdd = _.difference(gateways, gatewaysIdList);
    let gatewaysToDelete = _.difference(gatewaysIdList, gateways);

    let clonedGWArray = [];
    for (let j = 0; j < gatewaysToAdd.length; j++) {
      let currGW = await models.Gateway.findOne({
        where: {
          id: gatewaysToAdd[j]
        },
        attributes: ['Name']
      });

      let clonedGW = await models.Gateway.findAll({
        where: {
          Name: {
            [Op.like]: `%${currGW.Name.trim()}%`
          }
        },
        attributes: ['id']
      });

      clonedGW = clonedGW.map(gateway => gateway.id);

      for (let k = 0; k < clonedGW.length; k++) {
        if (!gatewaysToAdd.includes(clonedGW[k]))
          clonedGWArray.push(clonedGW[k]);
      }
    }
    gatewaysToAdd = [...gatewaysToAdd, ...clonedGWArray];

    clonedGWArray = [];
    for (let j = 0; j < gatewaysToDelete.length; j++) {
      let currGW = await models.Gateway.findOne({
        where: {
          id: gatewaysToDelete[j]
        },
        attributes: ['Name']
      });

      let clonedGW = await models.Gateway.findAll({
        where: {
          Name: {
            [Op.like]: `%${currGW.Name.trim()}%`
          }
        },
        attributes: ['id']
      });

      clonedGW = clonedGW.map(gateway => gateway.id);

      for (let k = 0; k < clonedGW.length; k++) {
        if (!gatewaysToDelete.includes(clonedGW[k]))
          clonedGWArray.push(clonedGW[k]);
      }
    }
    gatewaysToDelete = [...gatewaysToDelete, ...clonedGWArray];

    await Promise.all([
      models.GatewayGroupPivot.bulkCreate(
        gatewaysToAdd.map(g => ({
          gateway_id: g,
          gateway_group_id: id
        })),
        {
          transaction
        }
      )
    ]);

    if (appendGateways !== true) {
      models.GatewayGroupPivot.destroy(
        {
          where: {
            gateway_id: {
              [Op.in]: gatewaysToDelete
            },
            gateway_group_id: id
          }
        },
        {
          transaction
        }
      );
    }

    let updateOptions = {};

    if (name !== gatewayGroup.name) {
      updateOptions.name = name;
    }

    if (description !== gatewayGroup.description) {
      updateOptions.description = description;
    }

    if (Object.keys(updateOptions).length > 0) {
      await gatewayGroup.update(updateOptions, { transaction });
    }

    await publishGatewayGroups(actionType.UPDATED, {
      id: gatewayGroup.id,
      new_ids: gatewaysToAdd,
      del_ids: gatewaysToDelete,
      all: 0
    });

    await transaction.commit();

    insertIntoAdminLogs(getGatewayGroupEditMessage(name), req);

    return res.json(gatewayGroup);
  } catch (error) {
    await transaction.rollback();
    if (error.name === 'SequelizeUniqueConstraintError') {
      error.message = 'Group with this name already exist';
    }
    next(error);
  }
};

/**
 * Delete a group by id
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
module.exports.deleteGroup = async (req, res, next) => {
  const { id, name } = req.params;
  const transaction = await models.sequelize.transaction();
  try {
    const [group, relatedGateways] = await Promise.all([
      models.GatewayGroups.findOne({
        where: { id },
        transaction
      }),
      models.GatewayGroupPivot.findAll({
        where: {
          gateway_group_id: id
        },
        transaction
      })
    ]);

    if (!group) throw new httpErrors.NotFound('Group not found');

    if (group.id == 1) {
      throw new httpErrors.NotFound("Can't delete default group");
    }

    await publishGatewayGroups(actionType.DELETED, {
      id: group.id,
      new_ids: [],
      del_ids: relatedGateways.map(_g => _g.gateway_id),
      all: 0
    });

    await Promise.all([
      group.destroy({ transaction }),
      models.GatewayGroupPivot.destroy({
        where: {
          gateway_group_id: id
        },
        transaction
      })
    ]);

    await transaction.commit();

    insertIntoAdminLogs(
      getGatewayGroupDeleteMessage(group.dataValues.name),
      req
    );

    return res.json(group);
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

/**
 * Delete a group by id
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
module.exports.dropdown = async (req, res, next) => {
  try {
    const group = await models.GatewayGroups.findAll();

    if (group.length) {
      return res.json(group);
    }

    const groupList = await models.GatewayGroups.findAll({
      attributes: ['id', 'name']
    });

    return res.json(groupList);
  } catch (error) {
    next(error);
  }
};

module.exports.getGatewaysOfGroup = async (req, res, next) => {
  try {
    const { groupId } = req.params;
    const {
      offset = 0,
      limit = 10,
      sort = 'id',
      dir = 'asc',
      searchKey = ''
    } = req.query;

    const gatewayOptions = {
      attributes: [
        'id',
        'agent_id',
        'name',
        'buildOs',
        'connected_clients',
        'port',
        'protocol',
        'ipv4_address',
        'is_unique'
      ],
      where: {}
    };

    if (searchKey.trim()) {
      gatewayOptions.where.name = {
        [Op.like]: `%${searchKey.trim()}%`
      };
    }
    const group = await models.GatewayGroups.findOne({
      where: {
        id: groupId
      },
      order: [[models.Gateway, sort, dir]],
      include: [
        {
          model: models.Gateway,
          required: false,
          ...gatewayOptions
        }
      ]
    });

    if (!group) throw new httpErrors.NotFound('Group not found');

    const gateways = group.Gateways.slice(
      parseInt(offset) * parseInt(limit),
      parseInt(offset) * parseInt(limit) + parseInt(limit)
    );

    return res.json({
      total: group.Gateways.length,
      data: gateways,
      name: group.name
    });
  } catch (error) {
    next(error);
  }
};

module.exports.getServicesOfGroup = async (req, res, next) => {
  try {
    const { groupId } = req.params;
    const {
      offset = 0,
      limit = 10,
      sort = 'id',
      dir = 'asc',
      searchKey = ''
    } = req.query;

    // Case # 1 =
    // check if any routing policy have added all service or service groups
    // case # 1 = current gateway group selected + all services
    const response = {
      all: false
    };

    const routingPolicies = await models.RoutingPolicies.findAll({
      where: {
        all_services: true
      },
      include: [
        {
          model: models.GatewayGroups,
          where: {
            id: groupId
          },
          required: true
        }
      ]
    });

    if (routingPolicies.length > 0) {
      response.all = true;
    }

    if (!response.all) {
      // case # 4 = all gateways and all services selected for routing policy
      const routingPolicies3 = await models.RoutingPolicies.findAll({
        where: {
          all_services: true,
          all_gateways: true
        }
      });
      //console.log('In first (If Condition)');
      //console.log('Length of routingPolicies3 : ', routingPolicies3.length);
      //console.log('Length of routingPolicies : ', routingPolicies.length);

      response.all = routingPolicies3 && routingPolicies3.length > 0;
      //console.log('End Value of response.all : ', response.all);
    }

    if (!response.all) {
      //console.log('In first (If Condition)');

      let ids = [];

      // case #2 all gateway groups + some services
      const routingPolicies2 = await models.RoutingPolicies.findAll({
        where: {
          all_gateways: true
        },
        include: [
          {
            model: models.Service,
            required: true
          }
        ]
      });

      //console.log('Length of routingPolicies2 : ', routingPolicies2.length);

      if (routingPolicies2 && routingPolicies2.length > 0) {
        routingPolicies2.forEach(policy => {
          const services = policy.Services;
          services && services.forEach(service => ids.push(service.id));
        });

        //console.log('Length of services : ', services.length);
      }

      const serviceGroupIdQuery = `SELECT distinct sgrp.service_group_id,
      rp.name as routingPolicyName,
      rp.id as routingPolicyId
      FROM GatewayGroupPolicies pv
      right join RoutingPolicies rp on pv.policy_id = rp.id
      right join ServiceGroupPolicies sgrp on sgrp.policy_id = rp.id
      where pv.policy_type_id = 3 and sgrp.policy_type_id = 3 and pv.gateway_group_id = :group_id`;

      let serviceGroupIds = (
            await models.sequelize.query(serviceGroupIdQuery, {
              replacements: { group_id : groupId }, // Bind parameter
              type: models.sequelize.QueryTypes.SELECT, // Ensure proper type
            })
      )

      let policies = [];
      let newArray = [];

      if (serviceGroupIds) {
        const ifPolicyAlreadyExist = serviceGroupObj => {
          const index = policies[serviceGroupObj.service_group_id].findIndex(
            policy => policy.id === serviceGroupObj.routingPolicyId
          );
          return index > -1;
        };

        serviceGroupIds.forEach(obj => {
          ids.push(obj.service_group_id);
          if (!policies[obj.service_group_id]) {
            policies[obj.service_group_id] = [];
          }
          if (!ifPolicyAlreadyExist(obj)) {
            policies[obj.service_group_id].push({
              id: obj.routingPolicyId,
              name: obj.routingPolicyName
            });
          }
        });

        let allgroupedservices;
        if (ids) {
          for (let i = 0; i < ids.length; i++) {
            selectedGroupId = ids[i];
            allgroupedservices = await getServicesOfGroupfun(selectedGroupId);
            allgroupedservices.data.forEach(element => {
              newArray.push(element);
            });
          }
        }
      }

      const serviceIdsQuery = `SELECT distinct srp.service_id,
       rp.name as routingPolicyName, rp.id as routingPolicyId
       FROM GatewayGroupPolicies pv
       right join RoutingPolicies rp on pv.policy_id = rp.id
       right join ServicePolicies srp on srp.policy_id = rp.id
       where pv.policy_type_id = 3 and srp.policy_type_id = 3 and pv.gateway_group_id = :group_id`;
       
       let servicesIds = (
        await models.sequelize.query(serviceIdsQuery, {
          replacements: { group_id : groupId }, // Bind parameter
          type: models.sequelize.QueryTypes.SELECT, // Ensure proper type
        })
      )

      let policies1 = {}; // {serviceId: [array Of Routing Policies]}
      const ifPolicyAlreadyExist = serviceObj => {
        const index = policies1[serviceObj.service_id].findIndex(
          policy => policy.id === serviceObj.routingPolicyId
        );
        return index > -1;
      };

      servicesIds.forEach(obj => {
        ids.push(obj.service_id);
        if (!policies1[obj.service_id]) {
          policies1[obj.service_id] = [];
        }
        if (!ifPolicyAlreadyExist(obj)) {
          policies1[obj.service_id].push({
            id: obj.routingPolicyId,
            name: obj.routingPolicyName
          });
        }
      });

      const servicesOptions = {
        where: {
          id: ids
        },
        attributes: [
          'id',
          'Name',
          'port',
          'protocol',
          'ipv4_address',
          'type',
          'dummy_port',
          'mappedurl'
        ],
        order: [],
        offset: parseInt(offset),
        limit: parseInt(limit)
      };

      const allowedSortValues = [
        'id',
        'Name',
        'protocol',
        'type',
        'ipv4_address'
      ];

      if (allowedSortValues.includes(sort)) {
        servicesOptions.order.push([models.sequelize.literal(sort), dir]);
      }

      if (searchKey.trim()) {
        servicesOptions.where.Name = {
          [Op.like]: `%${searchKey.trim()}%`
        };
      }

      let services = await models.Service.paginate(servicesOptions);

      await Promise.all(services.data);
      newArray.map(ele => {
        services.data.push(ele);
      });

      //console.log(" In the end of 2nd if Condition ")
      //console.log(" Length of services.data.length : ", services.data.length)

      services.recordsTotal = services.data.length;
      response.data = services;
      //console.log(services);
    }

    if (response.all) {
      //console.log('In Third (If Condition)');

      // if all services were selected
      const servicesOptions = {
        attributes: [
          'id',
          'Name',
          'port',
          'protocol',
          'ipv4_address',
          'type',
          'dummy_port',
          'mappedurl'
        ],
        order: [],
        offset: parseInt(offset),
        limit: parseInt(limit)
      };

      const allowedSortValues = [
        'id',
        'Name',
        'protocol',
        'type',
        'ipv4_address'
      ];

      if (allowedSortValues.includes(sort)) {
        servicesOptions.order.push([models.sequelize.literal(sort), dir]);
      }

      if (searchKey.trim()) {
        servicesOptions.where.Name = {
          [Op.like]: `%${searchKey.trim()}%`
        };
      }

      let services = await models.Service.paginate(servicesOptions);

      // mapping routing policies to servies again
      // services.data = services.data.map(service => {
      //   service.dataValues.routingPolicy = policies[service.id]
      //   return service
      // })

      await Promise.all(services.data);
      //console.log('📄 Check if Pagination Works ', services);
      response.data = services;
    }

    res.status(200).json(response);
  } catch (error) {
    next(error);
  }
};

// This api is not being used and contain issues

// module.exports.getServicesGroupsOfGatewayGroups = async (req, res, next) => {
//   try {
//     const { groupId } = req.params;
//     // check if any routing policy have added all service or service groups
//     // case # 1 = current gateway group selected + all services
//     const response = {
//       all: false
//     };

//     const routingPolicies = await models.RoutingPolicies.findAll({
//         where: {
//           all_services: true
//         },
//         include: [
//           {
//             model: models.GatewayGroups,
//             where: {
//               id: groupId
//             },
//             required: true
//           }
//         ]
//       });

//       if(routingPolicies.length > 0) {
//         response.all = true;
//       }

//       if(!response.all) {
//         // case # 4 = all gateways and all services selected for routing policy
//         const routingPolicies3 = await models.RoutingPolicies.findAll({
//           where: {
//             all_services: true,
//             all_gateways: true
//           }
//         });
//         response.all = routingPolicies3 && routingPolicies3.length > 0
//       }

//       if(!response.all){
//         let ids = []
//       // all gateways selected + some services selectede
//       const routingPolicies2 = await models.RoutingPolicies.findAll({
//         where: {
//           all_gateways: true
//         },
//         include: [
//           {
//             model: models.ServiceGroup,
//             required: true
//           }
//         ]
//       });

//       if(routingPolicies2 && routingPolicies2.length > 0) {
//         routingPolicies2.forEach(policy => {
//           const serviceGroups = policy.ServiceGroups;
//           serviceGroups && serviceGroups.forEach(serviceGroup => ids.push(serviceGroup.id));
//         });
//       }

//       // case # 3 some gateways selected + some services selected

//       const serviceGroupIdQuery = `SELECT distinct sgrp.service_group_id, rp.name as routingPolicyName, rp.id as routingPolicyId FROM GatewayGroupPolicies pv right join RoutingPolicies rp on pv.policy_id = rp.id right join ServiceGroupPolicies sgrp on sgrp.policy_id = rp.id where pv.policy_type_id = 3 and sgrp.policy_type_id = 3 and pv.gateway_group_id = ${groupId}`
//       let serviceGroupIds  = await models.sequelize.query(serviceGroupIdQuery)

//       let policies = []

//       const ifPolicyAlreadyExist = (serviceGroupObj) => {
//         const index = policies[serviceGroupObj.service_group_id].findIndex(policy => policy.id === serviceGroupObj.routingPolicyId)
//           return index > -1
//       }

//       serviceGroupIds[0].forEach(obj => {
//         ids.push(obj.service_group_id)
//         if(!policies[obj.service_group_id])
//         {
//           policies[obj.service_group_id] = []
//         }
//         if( !ifPolicyAlreadyExist(obj)) {
//           policies[obj.service_group_id].push({ id: obj.routingPolicyId, name: obj.routingPolicyName  })
//         }
//       })

//       const serviceGroups = await models.ServiceGroup.paginate({
//         where: {
//           id: ids
//         }
//       });
//       response.data = serviceGroups
//     }
//     let dataArray = response.data;
//     let allgroupedservices;
//     let newArray = [];
//     if(dataArray.recordsTotal > 0){
//       for(let i=0; i < dataArray.recordsTotal; i++){
//         selectedGroupId = dataArray.data[i].id;
//         allgroupedservices = await getServicesOfGroupfun(selectedGroupId);
//          allgroupedservices.data.forEach(element => {
//            newArray.push(element);
//          });
//       }
//     }
//     response.data = newArray;
//     res.status(200).json(response)

//   } catch (error) {
//     next(error);
//   }
// };

async function getServicesOfGroupfun(groupId) {
  try {
    const offset = 0;
    const limit = 10;
    const sort = 'Name';
    const dir = 'asc';
    const searchKey = '';

    const servicesOptions = {
      attributes: [
        'id',
        'Name',
        'port',
        'protocol',
        'ipv4_address',
        'type',
        'dummy_port',
        'mappedurl'
      ],
      raw: true,
      where: {}
    };

    if (searchKey.trim()) {
      servicesOptions.where.Name = {
        [Op.like]: `%${searchKey.trim()}%`
      };
    }

    const group = await models.ServiceGroup.findOne({
      where: {
        id: groupId
      },
      order: [[models.Service, sort, dir]],
      include: [
        {
          model: models.Service,
          ...servicesOptions,
          required: false
        }
      ]
    });

    if (!group) throw new httpErrors.NotFound('Group not found');

    const services = group.Services.slice(
      parseInt(offset) * parseInt(limit),
      parseInt(offset) * parseInt(limit) + parseInt(limit)
    );
    return {
      total: group.Services.length,
      data: services,
      name: group.name
    };
  } catch (error) {
    //console.log(error);
  }
}

module.exports.bulkDeleteGroups = async (req, res, next) => {
  try {
    const { groupIds, excludeGroups } = req.body;
    const { allSelected, search } = req.query;

    let options = {
      where: {
        id: {
          [Op.not]: [1]
        }
      }
    };

    if (allSelected == 'true') {
      options.where.id = {
        [Op.not]: [1, ...excludeGroups]
      };

      if (search.trim()) {
        options.where.name = {
          [Op.like]: `%${search.trim()}%`
        };
      }
    } else {
      options.where.id = groupIds;
    }

    let gatewayGroups = await models.GatewayGroups.findAll({
      ...options,
      attributes: ['id', 'name']
    });

    groupId = gatewayGroups.map(gw => gw.id);
    groupNames = gatewayGroups.map(gw => gw.name);

    console.log('Group IDs: ', groupId);
    console.log('Group Names: ', groupNames);

    const clients = await models.GatewayGroups.destroy(options);

    const pivotDestroy = await models.GatewayGroupPivot.destroy({
      where: {
        gateway_group_id: {
          [Op.in]: groupId
        }
      }
    });

    for (const group of groupNames) {
      console.log('group :>> ', group);
      insertIntoAdminLogs(
        getGatewayGroupDeleteMessage(group),
        req
      );
    }

    res.status(200).json({ message: 'Gateway groups deleted' });
  } catch (e) {
    next(e);
  }
};
