var httpErrors = require('http-errors');
const _ = require('lodash');
var models = require('../models');
var Openssl = require('../openssl/openssl.js');
var OpensslObj = new Openssl();
let projectConfig = require('../config/config')['project'][
  process.env.NODE_ENV
];
let path = require('path');
let appDir = path.dirname(require.main.filename);
appDir = appDir.substring(0, appDir.lastIndexOf('/'));
let helper = require('../routes/common/helper');
let settings = require('../routes/common/settings');
let { exec } = require('child_process');
let fs = require('fs');
let fsExtra = require('fs-extra');
let archiver = require('archiver');
const moment = require('moment');
const { promisify } = require('util');
const utils = require('../utils/common');
const emailQueue = require('../queues/email');
// const { addtoQueue } = require('../queues/uploadToContent');
const { generateLink } = require('../routes/common/content-upload');
const {
  actionType,
  publishGateways,
  publishGatewaySubnet,
  publishMsgFileGateways,
  publishGatewaySoftware
} = require('../routes/common/event-publisher');
const { Op } = require('sequelize');
const pretty = require('prettysize');
const copyFilePromise = promisify(fs.copyFile);

const scriptsPath = path.join(__dirname, '..', 'scripts');

let mapping = {
  ubuntu: 'deb',
  centosserver: 'rpm',
  windows: 'exe',
  windowsserver: 'exe',
  macintosh: 'dmg',
  ubuntuserver: 'deb',
  ubuntuserver18: 'deb',
  ubuntuserver20armv8: 'deb',
  ubuntu18: 'deb',
  android: 'apk',
  ios: 'ipa'
};

let installerType = {
  ubuntu18: 'linux64',
  ubuntuserver: 'linux64',
  ubuntuserver18: 'linux64',
  ubuntuserver20armv8: 'linux64',
  ubuntu: 'linux64',
  windows: 'win64',
  windowsserver: 'win64',
  macintosh: 'mac',
  android: 'android',
  ios: 'ios',
  centosserver: 'linux64'
};
var emailLinkExpiryInMinutes = 60;

async function setExpiryTime() {
  emailLinkExpiryInMinutes = await settings.getExpirySettings();
}

setExpiryTime();

const { insertIntoAdminLogs } = require('../routes/common/adminLogs');
const {
  getGatewayEditMessage,
  getGatewayCreateMessage,
  getGatewayDeleteMessage,
  getGatewayEmailMessage,
  getGatewayDownloadlMessage,
  getGatewayEnableMessage,
  getGatewayDisableMessage,
  getCloneMessage
} = require('../config/adminLogs.config');

/* GET gateways listing. */
const getGateways = async (req, res, next) => {
  try {
    const {
      limit,
      offset,
      sort = 'id',
      dir = 'desc',
      action,
      searchKey = '',
      protocol = '',
      buildOs = '',
      all = 'false',
      unique = 'true',
      except = []
    } = req.query;

    let options = {
      where: {},
      limit: parseInt(limit),
      offset: parseInt(offset) * parseInt(limit),
      order: [],
      attributes: {
        exclude: [
          'HMacKey',
          'cacert',
          'clientcrt',
          'clientpem',
          'counter',
          'createdAt',
          'deployment_location',
          'device_binding',
          'enc_key',
          ,
          'hardware_id',
          'isDownloadPackage',
          'link_key',
          'link_key_expiry',
          'onPrenDownloadLink',
          'packageDownloaded',
          'public_ipv4_address',
          'public_ipv4_port',
          'spa_port',
          'updatedAt'
        ]
      },
      include: [
        {
          model: models.GatewayHealth
        }
      ]
    };

    if (unique == 'true') {
      options = {
        ...options,
        where: {
          ...options.where,
          is_unique: 1
        }
      };
    }

    if (all === 'true') {
      delete options['limit'];
      delete options['offset'];
    }

    if (searchKey.trim()) {
      options.where.Name = {
        [Op.like]: `%${searchKey.trim()}%`
      };
    }

    if (buildOs && buildOs !== 'undefined' && buildOs !== 'null') {
      options.where.buildOs = buildOs.trim();
    }

    if (protocol && protocol !== 'undefined' && protocol !== 'null') {
      options.where.protocol = protocol.trim();
    }

    // const myGatewayGroup = JSON.parse(gatewayGroups)
    // if(myGatewayGroup && myGatewayGroup.length > 0)
    // {
    //   options.where.gateway_group_id = myGatewayGroup
    // }

    if (action === 'active') {
      options.where.is_active = 1;
    }

    let skip = [];
    try {
      skip = JSON.parse(except);
    } catch (e) {
      console.log('error');
    }

    if (Array.isArray(skip) && skip.length > 0) {
      options.where['id'] = {
        [Op.not]: skip
      };
    }

    const allowedSortValues = ['Name', 'id'];

    if (allowedSortValues.includes(sort)) {
      options.order.push([models.sequelize.literal(sort), dir]);
    }

    const result = await models.Gateway.findAndCountAll(options);

    res.json({
      recordsTotal: result.count,
      data: result.rows
    });
  } catch (error) {
    next(error);
  }
};

const getGatewayServices = async (req, res, next) => {
  try {
    const gateway = await models.Gateway.findOne({
      where: {
        id: req.params.id
      }
    });

    if (!gateway) {
      throw httpErrors.NotFound('Gateway not found.');
    }

    let policyQuery = `CALL GetPoliciesForGateway(${gateway.dataValues.id})`;

    let policyResults = await models.sequelize.query(policyQuery);

    let policyOrganizedData = {};
    policyResults.map(value => {
      if (!policyOrganizedData[value?.policy_id]) {
        policyOrganizedData[value?.policy_id] = {
          policy_id: value?.policy_id,
          name: value?.policy_name,
          services: []
        };

        let services = policyResults
          .filter(obj => obj.policy_id === value.policy_id)
          .map(obj => obj.name);

        policyOrganizedData[value?.policy_id].services = services.join(
          ', '
        );
      }
    });

    let policyData = Object.values(policyOrganizedData);

    let all_services = false;

    for (let data of policyResults) {
      if (data.all_services) {
        all_services = true;
      }
    }

    let serviceQuery, serviceResults;
    if (all_services) {
      serviceQuery = `
        SELECT services.id, services.Name as name, services.ipv4_address, services.type, services.mappedurl
        FROM Services services
      `;
      serviceResults = (await models.sequelize.query(serviceQuery))[0];
    }

    else {
      serviceQuery = `CALL GetServicesForGateway(${gateway.dataValues.id})`;
      serviceResults = await models.sequelize.query(serviceQuery);
    }

    let serviceGroupQuery = `CALL GetServiceGroupsForGateway(${gateway.dataValues.id})`;
    
    let serviceGroupResults = await models.sequelize.query(serviceGroupQuery);

    let groupOrganizedData = {};
    serviceGroupResults.map((value, index) => {
      if (!groupOrganizedData[value?.id]) {
        groupOrganizedData[value?.id] = {
          id: value.id,
          name: value.name,
          createdAt: value.created_at,
          description: value.description,
          services: []
        };

        let services = serviceGroupResults
          .filter(obj => obj.id === value.id)
          .map(obj => obj.service_name);

        groupOrganizedData[value?.id].services = services.join(', ');
      }
    });

    let userServicesGroup = Object.values(groupOrganizedData);

    res.json({
      policies: policyData,
      gatewayServices: serviceResults,
      gatewayServiceGroup: userServicesGroup,
    });
  } catch (error) {
    next(error);
  }
}

const getQuickGateways = async (req, res, next) => {
  try {
    const { searchKey, except } = req.query;

    const options = {
      where: {},
      attributes: ['id', 'Name']
    };

    if (searchKey && searchKey.trim()) {
      options.where.Name = {
        [Op.like]: `%${searchKey.trim()}%`
      };
    }

    let skip = [];
    try {
      skip = JSON.parse(except);
    } catch (e) {
      console.log('error');
    }

    if (Array.isArray(skip) && skip.length > 0) {
      options.where['id'] = {
        [Op.not]: skip
      };
    }

    const gateways = await models.Gateway.findAll(options);

    res.status(200).json({
      data: gateways
    });
  } catch (error) {
    next(error);
  }
};

const status = async (req, res, next) => {
  try {
    const { gateway: gatewayObj } = req.body;

    const gateway = await models.Gateway.findOne({
      where: {
        id: gatewayObj.id
      }
    });

    if (!gateway) {
      throw new httpErrors.NotFound('No gateway found.');
    }

    gatewayObj.is_active = !gatewayObj.is_active;
    await gateway.update(gatewayObj);

    let changedColumns = Object.keys(gateway._changed);
    changedColumns = changedColumns.filter(c => ['is_active'].includes(c));

    if (changedColumns.length) {
      const message = {
        ids: [gateway.id],
        data: {
          is_active: gateway.is_active
        }
      };

      let logMessage;

      if (gatewayObj.is_active) {
        logMessage = getGatewayEnableMessage(gateway.Name);
      } else {
        logMessage = getGatewayDisableMessage(gateway.Name);
      }

      insertIntoAdminLogs(logMessage, req);

      await publishGateways(actionType.UPDATED, message);
    }

    res.json({
      success: true,
      message: 'Update Successfully'
    });
  } catch (error) {
    next(error);
  }
};

updateGateway = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    const { gateway } = req.body;

    const { services, controller } = gateway;

    const { id } = req.params;

    const gatewayData = await models.Gateway.findOne({
      where: {
        id: id
      }
    });

    if (!gatewayData) {
      throw new httpErrors.NotFound('Gateway not found.');
    }

    const isUnique = await checkIsGatewayUniqueOrNot(gateway, true);
    if (!isUnique) {
      throw new httpErrors.Conflict(
        'Name, Private port, private ip should be unique'
      );
    }

    if(gateway.proxy_enabled == false){

      let nonProxiedServicesAttached = `
                    SELECT services.id as serviceId, services.Name as serviceName, services.isGateWayAssigned AS isGateway, services.is_active AS isActive,services.proxy_enabled,
                   routingPolicies.id AS routingPolicyId, routingPolicies.name AS routingPolicyName,
                   routingPolicies.id as RoutingPoliciesId, gateways.id as gatewayId, gateways.Name as gatewayName
                FROM Services services
                INNER JOIN GroupServices AS groupServices ON groupServices.serviceId = services.id
                INNER JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
                INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
                INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = serviceGroupPolicies.policy_id
                INNER JOIN GatewayGroupPolicies AS gatewayGroupPolicies on gatewayGroupPolicies.policy_id  =  routingPolicies.id
                INNER JOIN GatewayGroups AS gatewayGroups on gatewayGroups.id = gatewayGroupPolicies.gateway_group_id
                INNER JOIN GatewayGroupPivots gatewayGroupPivots on gatewayGroupPivots.gateway_group_id = gatewayGroups.id
                INNER JOIN Gateways AS gateways on gateways.id = gatewayGroupPivots.gateway_id
				        WHERE gateways.id = ${id}
                UNION

                -- Services with Gateways Groups -> Routing Policy
                SELECT services.id as serviceId, services.Name as serviceName, services.isGateWayAssigned AS isGateway, services.is_active AS isActive,services.proxy_enabled, 
                   routingPolicies.id AS routingPolicyId, routingPolicies.name AS routingPolicyName,
                   routingPolicies.id as RoutingPoliciesId, gateways.id as gatewayId, gateways.Name as gatewayName
                FROM Services services
                INNER JOIN ServicePolicies servicePolicies on servicePolicies.service_id = services.id
                INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = servicePolicies.policy_id
                INNER JOIN GatewayGroupPolicies AS gatewayGroupPolicies on gatewayGroupPolicies.policy_id  =  routingPolicies.id
                INNER JOIN GatewayGroups AS gatewayGroups on gatewayGroups.id = gatewayGroupPolicies.gateway_group_id
                INNER JOIN GatewayGroupPivots gatewayGroupPivots on gatewayGroupPivots.gateway_group_id = gatewayGroups.id
                INNER JOIN Gateways AS gateways on gateways.id = gatewayGroupPivots.gateway_id
				        WHERE gateways.id = ${id}
                UNION

                 -- Services with Gateways -> Routing Policy
                SELECT services.id as serviceId, services.Name as serviceName, services.isGateWayAssigned AS isGateway, services.is_active AS isActive,services.proxy_enabled, 
                   routingPolicies.id AS routingPolicyId, routingPolicies.name AS routingPolicyName,
                   routingPolicies.id as RoutingPoliciesId, gateways.id as gatewayId, gateways.Name as gatewayName
                FROM Services services
                INNER JOIN ServicePolicies servicePolicies on servicePolicies.service_id = services.id
                INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = servicePolicies.policy_id
                INNER JOIN GatewayPolicies AS gatewayPolicies on gatewayPolicies.policy_id =  routingPolicies.id
                INNER JOIN Gateways AS gateways on gateways.id = gatewayPolicies.gateway_id
				        WHERE gateways.id = ${id}
                UNION

                -- Services with Gateways Groups -> Routing Policy
                SELECT services.id as serviceId, services.Name as serviceName, services.isGateWayAssigned AS isGateway, services.is_active AS isActive,services.proxy_enabled, 
                   routingPolicies.id AS routingPolicyId, routingPolicies.name AS routingPolicyName,
                   routingPolicies.id as RoutingPoliciesId, gateways.id as gatewayId, gateways.Name as gatewayName
                FROM Services services
                INNER JOIN GroupServices AS groupServices ON groupServices.serviceId = services.id
                INNER JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
                INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
                INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = serviceGroupPolicies.policy_id
                INNER JOIN GatewayGroupPolicies AS gatewayGroupPolicies on gatewayGroupPolicies.policy_id  =  routingPolicies.id
                INNER JOIN GatewayPolicies AS gatewayPolicies on gatewayPolicies.policy_id =  routingPolicies.id
                INNER JOIN Gateways AS gateways on gateways.id = gatewayPolicies.gateway_id
	              WHERE gateways.id = ${id}`;

         const nonProxiedServicesAttachedResult = (
            await models.sequelize.query(nonProxiedServicesAttached)
            )[0];

         console.log('nonProxiedServicesAttachedResult :>> ', nonProxiedServicesAttachedResult);
        
         const hasProxiedService = nonProxiedServicesAttachedResult.some(
          (row) => row.proxy_enabled === 1
        );
        
        if (hasProxiedService) {
          throw new httpErrors(`Non-proxied gateways cannot be assigned to proxied services.`);
        }
  }

      
      // const serviceIds = assignedServices.map(service=> service.serviceId);
      // const  servicesToDelete = _.difference(serviceIds, Object.values(services));
      // const  servicesToAdd = _.difference(Object.values(services), serviceIds);

      // if (servicesToAdd.length || servicesToDelete.length) {
      //    await Promise.all([
      //     models.GatewayService.bulkCreate(servicesToAdd.map(service => ({
      //       gatewayId: id,
      //       serviceId: service
      //     })), { transaction }),
      //     models.GatewayService.destroy({
      //       where:{
      //         serviceId:{
      //           [Op.in]:servicesToDelete
      //         }
      //       },
      //       transaction
      //     })
      //    ]);
      // }

      // console.log(servicesToAdd);
      // console.log(servicesToDelete);

      // await Promise.all([
      //   models.Service.update({
      //     isGateWayAssigned: false
      //   },{
      //     where: {
      //       id: {
      //         [Op.in]:servicesToDelete
      //       }
      //     },
      //     transaction
      //   }),
      //   models.Service.update({
      //     isGateWayAssigned: true
      //   },{
      //     where: {
      //       id: {
      //         [Op.in]:servicesToAdd
      //       }
      //     },
      //     transaction
      //   })

      // ])

      //TODO: Need improvement

      // const servicesData = await models.Service.findAll({
      //   where: { }
      // });

      // if (servicesData.length) {
      //   // const ids = servicesData.map(service => service.service_id);
      //   // const count = await models.GatewayService.count({
      //   //   where: {
      //   //     serviceId: {
      //   //       [Op.in]: ids
      //   //     }
      //   //   },
      //   //   attributes: ['serviceId'],
      //   //   group: ['serviceId']
      //   // })

      //   // const idHash = count.reduce((acc, v) => {

      //   //     if(!acc[v.serviceId] && !servicesToDelete.includes(v.serviceId)) acc[v.serviceId] = v;
      //   //     return acc;
      //   // }, {});

      //   // await Promise.all(
      //   //   servicesData.map((service) => {
      //   //     return service.update({
      //   //       isGateWayAssigned: !!(idHash[service.service_id] || {}).count
      //   //     }, {
      //   //         transaction
      //   //     });
      //   //   })
      //   // )
      // }

      if(gateway.communication_protocol == 0){
        gateway.security_protocol = 'TLS'
      }else if(gateway.communication_protocol == 1){
        gateway.security_protocol = 'DTLS'
      }

      await gatewayData.update(gateway, {
        transaction
      });

      insertIntoAdminLogs(getGatewayEditMessage(gatewayData.Name), req);

      let changedColumns = Object.keys(gatewayData._changed);
      changedColumns = changedColumns.filter(c => ['is_active'].includes(c));

      if (changedColumns.length) {
        const message = {
          ids: [gatewayData.id],
          data: {
            is_active: gatewayData.is_active
          }
        };

        await publishGateways(actionType.UPDATED, message);
      }

      let _publishMessageChangedColumns = Object.keys(gatewayData._changed);
      _publishMessageChangedColumns = _publishMessageChangedColumns.filter(_c =>
        ['enc_key', 'HMacKey', 'hardware_id'].includes(_c)
      );

      if (_publishMessageChangedColumns.length) {
        await publishMsgFileGateways(actionType.UPDATED, {
          id: gatewayData.id,
          data: {
            enc_key: gatewayData.enc_key,
            HMacKey: gatewayData.HMacKey,
            hardware_id: gatewayData.hardware_id
          }
        });
      }

      await transaction.commit();

      generateInstallerZip(gatewayData, '0', false);

      res.json({
        success: true,
        message: 'Edited Successfully'
      });
    } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

/* Get a single gateway detail */
const getGateway = async (req, res, next) => {
  try {
    const gateway = await models.Gateway.findAll({
      include: [
        { model: models.Service, where: {}, required: false }
      ],
      where: {
        id: req.params.id
      }
    });

    if (!gateway) {
      throw new httpErrors.NotFound('No gateway found.');
    }

    res.json(gateway);
  } catch (error) {
    next(error);
  }
};

const getGatewaysOSAndProtocols = async (req, res, next) => {
  try {
    const protocols = [],
      os = [];

    const gateways = await models.Gateway.findAll({
      attributes: ['protocol', 'buildOs']
    });

    gateways.forEach(gateway => {
      protocols.push(gateway.protocol);
      os.push(gateway.buildOs);
    });

    res.json({
      protocols: [...new Set(protocols)],
      os: [...new Set(os)]
    });
  } catch (error) {
    next(error);
  }
};

//check whether gateway already exist or not
const checkIsGatewayUniqueOrNot = (gateway, update) => {
  const { Name, ipv4_address, port, agent_id, is_unique } = gateway;
  let whereClause = {
    is_unique: 1,
    $or: [
      {
        ipv4_address: {
          $eq: ipv4_address
        }
      },
      {
        Name: {
          $eq: Name
        }
      }
    ]
  };
  if (update) {
    whereClause.agent_id = {
      $ne: agent_id
    };
  }
  return models.Gateway.count({
    where: whereClause
  }).then(count => {
    console.log(count);
    if (count >= 1) {
      return false;
    }
    return true;
  });
};

// add new gateway
const addGateway = async (req, res, next) => {
  let { gateway } = req.body;
  let { services, controller } = gateway;
  const transaction = await models.sequelize.transaction();
  try {
    const [encKey, hmacKey] = await Promise.all([
      OpensslObj.generateEncKey(),
      OpensslObj.generateHMacKey()
    ]);

    const availableSubnets = await models.sequelize.query(
      `
      SELECT subnet_ip AS subnet_ip
      FROM GatewaySubnetIPLists 
      WHERE excluded = 0 AND subnet_ip NOT IN (
        SELECT subnet_ip FROM Gateways WHERE is_unique = 1
      ) LIMIT 1;
    `,
      { transaction }
    );

    gateway.enc_key = encKey.key;

    gateway.HMacKey = hmacKey.key;

    if (availableSubnets[0].length == 1) {
      gateway.subnet_ip = availableSubnets[0][0].subnet_ip;
    }

    gateway.agent_id = 0;

    gateway.is_unique = 1;

    if(gateway.communication_protocol == 0){
      gateway.security_protocol = 'TLS'
    }else if(gateway.communication_protocol == 1){
      gateway.security_protocol = 'DTLS'
    }

    const isUnique = await checkIsGatewayUniqueOrNot(gateway, false);
    if (!isUnique) {
      throw new httpErrors.Conflict('Name, Port, private ip should be unique');
    }

    const gatewayData = await models.Gateway.create(gateway, { transaction });

    gatewayData.agent_id = gatewayData.id;
    await gatewayData.save({ transaction });

    // Save controller gateway
    // Save services
    // const gatewayServices = [];
    // for (let service in services) {
    //   let gatewayService = {
    //     gatewayId: gatewayData.id,
    //     serviceId: service
    //   };
    //   gatewayServices.push(gatewayService);
    // }

    // if (gatewayServices.length === 0) {

    generateInstallerZip(gatewayData, '0', true);

    await publishMsgFileGateways(actionType.ADDED, {
      id: gatewayData.id,
      data: {
        enc_key: gatewayData.enc_key,
        HMacKey: gatewayData.HMacKey,
        hardware_id: gatewayData.hardware_id
      }
    });

    await transaction.commit();
    insertIntoAdminLogs(getGatewayCreateMessage(gatewayData.Name), req);

    return res.json({
      success: true,
      message:
        availableSubnets[0].length == 1
          ? 'Created successfully'
          : 'Subnets not available'
    });
    // }

    // const serviceToAdd = Object.values(services);

    // await Promise.all([
    //     models.GatewayService.bulkCreate(gatewayServices,{ transaction }),
    //     generateInstallerZip(gatewayData, '0'),
    //     models.Service.update({
    //       isGateWayAssigned: true
    //     },{
    //       where: {
    //         id: {
    //           [Op.in]: serviceToAdd
    //         }
    //       },
    //       transaction
    //     })
    // ]);

    // await transaction.commit();

    // res.json({
    //   success: true,
    //   message: 'Created Successfully'
    // });

    // var counts = servicesData.length;
    // var saved = 0;
    // servicesData.forEach(async function(serviceSingle) {
    //   const GS = await models.GatewayService.count({
    //     where: {
    //       serviceId: serviceSingle.service_id
    //     }
    //   });

    //   if (GS > 0) {
    //     serviceSingle.isGateWayAssigned = true;
    //   } else {
    //     serviceSingle.isGateWayAssigned = false;
    //   }
    //   await serviceSingle.save({ transaction });
    //   saved++;
    //   if (saved === counts) {
    //     await transaction.commit();
    //     res.json({
    //       success: true,
    //       message: 'Created Successfully'
    //     });
    //   }

    // });
  } catch (error) {
    await transaction.rollback();
    return next(error);
  }
};

const cloneGateway = async (req, res, next) => {
  let { gateway } = req.body;
  let { services, controller } = gateway;
  const originaleGatewayName = gateway.Name;
  const transaction = await models.sequelize.transaction();
  try {
    const [encKey, hmacKey] = await Promise.all([
      OpensslObj.generateEncKey(),
      OpensslObj.generateHMacKey()
    ]);

    const gatewayCloneCount = await models.Gateway.count({
      where: {
        subnet_ip: gateway.subnet_ip,
        is_unique: 0
      }
    });

    const gatewayGroupPivots = await models.GatewayGroupPivot.findAll({
      where: {
        gateway_id: gateway.id
      },
      attributes: ['gateway_group_id']
    });

    const gatewayRoutingPolicies = await models.GatewayPolicies.findAll({
      where: {
        gateway_id: gateway.id,
        policy_type_id: 3
      },
      attributes: ['policy_id']
    });

    gateway.Name += `_${gatewayCloneCount}`;
    gateway.enc_key = encKey.key;
    gateway.HMacKey = hmacKey.key;
    gateway.agent_id = 0;
    gateway.is_unique = 0;
    gateway.id = null;
    gateway.buildOs = gateway.buildOs || 'ubuntuserver18';
    gateway.port = 0;
    gateway.ipv4_address = '';
    gateway.hardware_id = '00000000000000000000000000000000';

    const gatewayData = await models.Gateway.create(gateway, { transaction });
    gatewayData.agent_id = gatewayData.id;
    await gatewayData.save({ transaction });

    for (let j = 0; j < gatewayGroupPivots.length; j++) {
      await models.GatewayGroupPivot.create(
        {
          gateway_group_id: gatewayGroupPivots[j].gateway_group_id,
          gateway_id: gatewayData.id
        },
        { transaction }
      );
    }

    for (let j = 0; j < gatewayRoutingPolicies.length; j++) {
      await models.GatewayPolicies.create(
        {
          policy_id: gatewayRoutingPolicies[j].policy_id,
          policy_type_id: 3,
          gateway_id: gatewayData.id
        },
        { transaction }
      );
    }

    //let controllerGateway = {
    //  gatewayId: gatewayData.id,
    //  controllerId: controller
    //};

    //await models.ControllerGateway.create(controllerGateway, { transaction });

    generateInstallerZip(gatewayData, '0');

    await transaction.commit();

    insertIntoAdminLogs(
      getCloneMessage(originaleGatewayName, gateway.Name),
      req
    );

    return res.json({
      success: true,
      message: 'Created successfully'
    });
  } catch (error) {
    await transaction.rollback();
    return next(error);
  }
};

const deleteGateway = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    const gateway = await models.Gateway.findOne({
      where: {
        id: req.params.id
      }
    });

    if (!gateway) {
      throw new httpErrors.NotFound('No gateway found to delete.');
    }

    await Promise.all([
      models.GatewayService.destroy({
        where: {
          gatewayId: req.params.id
        },
        transaction
      })
    ]);

    // const assignedServicesIds = assignedServices.map(service => service.serviceId);

    await Promise.all([
      // models.Service.update({
      //   isGateWayAssigned: false
      // }, {
      //   where: {
      //     id: {
      //       [Op.in]: assignedServicesIds
      //     }
      //   },
      //   transaction
      // }),
      gateway.destroy({
        transaction
      }),
      models.Gateway.destroy({
        where: {
          id: req.params.id
        },
        transaction
      }),
      models.GatewayGroupPivot.destroy({
        where: {
          gateway_id: req.params.id
        },
        transaction
      }),
      models.GatewayPolicies.destroy({
        where: {
          gateway_id: req.params.id,
          policy_type_id: 3
        },
        transaction
      })
    ]);

    let changedColumns = Object.keys(gateway._changed);
    changedColumns = changedColumns.filter(c => ['is_active'].includes(c));

    if (changedColumns.length) {
      const message = {
        ids: [gateway.id],
        data: {
          is_active: gateway.is_active
        }
      };

      await publishGateways(actionType.DELETED, message);
    }

    await transaction.commit();
    insertIntoAdminLogs(getGatewayDeleteMessage(gateway.Name), req);

    res.json({
      success: true,
      message: 'Delete Successfully'
    });
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

//for download
const downloadPackage = async (req, res, next) => {
  try {
    let key = req.params.key;
    let message = '';
    const options = {
      where: {
        id: key
      }
    };
    const gateway = await models.Gateway.findOne(options);
    if (!gateway) {
      message =
        'Unable to find installation package for gateway component, please request for installer';
      helper.sendEmailLinkErrorHtmlResponse(message, 404, res);
    }

    let os = gateway.buildOs;
    let folderPath =
      `${projectConfig.serviceInstallerPath}/gateway/output/` + os;
    let filePath = `${folderPath}/invisily-gateway-` + installerType[os];

    // console.log('gateway file path is ', filePath);
    // let downloadFile = `invisily-gateway-` + installerType[os];

    // downloadFile += '-' + gateway.id + '.zip';
    filePath += '-' + gateway.id + '.zip';
    if (!fs.existsSync(filePath)) {
      message =
        'Unable to find installation package for gateway component, please request for installer';
      helper.sendEmailLinkErrorHtmlResponse(message, 404, res);
    }

    // const linkExpiry = moment.tz(gateway.link_key_expiry, moment.tz.guess());

    // console.log('Gateway [' + gateway.Name + '] installation package expiry is: ', linkExpiry.format());

    // if (!(moment().isBefore(linkExpiry))) {
    //   message = 'Gateway [' + gateway.Name + '] installation package is Expired, please request for a new link';
    //   helper.sendEmailLinkErrorHtmlResponse(message, 400, res);
    // }

    // console.log('Gateway [' + gateway.Name + '] installation package is NOT expired');

    // let gatewayOs = os === 'ubuntuserver' ? 'ubuntuserver18'
    //               : os === 'ubuntuserver20armv8' ? 'ubuntuserver20armv8'
    //               : os === 'centosserver' ? 'centosserver'
    //               : 'windowsserver';

    let gatewayOs = os;
    console.log('gateway is ', gatewayOs, ' actual is ', os);

    let packageName = await utils.getPackageName(
      gatewayOs,
      'gateway',
      gateway.id
    );

    res.download(filePath, packageName, async function(err) {
      // file download is successful, expire link now
      if (err) {
        throw httpErrors.BadRequest('Error Occurred');
      }

      const updatedGateway = await gateway.update({
        link_key_expiry: moment(),
        packageDownloaded: true
      });

      if (updatedGateway) {
        console.log('Updated expiry of Gateway [' + gateway.Name + '] to now');
      }
    });

    insertIntoAdminLogs(getGatewayDownloadlMessage(gateway.Name), req);
  } catch (error) {
    next(error);
  }
};

function sendEmailData(options, extraOptions = {}) {
  if (options.recipient) {
    let mailOptions = {
      recipients: options.recipient,
      subject: options.emailSubject,
      file: `${appDir}/templates/` + options.templateFileName,
      context: {
        user_name: options.recipientName,
        link_expiry_in_minutes: options.linkExpiryInMinutes,
        confirm_link: options.downloadLink,
        on_premise_confirm_link: options.onPremiseDownloadLink,
        off_premise_confirm_link: options.offPremiseDownloadLink,
        on_premise_Device_link: options.onPremDeviceIdLink,
        off_premise_Device_link: options.offPremDeviceIdLink,
        component: options.component,
        onPremApiUrl: options.onPremApiUrl,
        device_os: extraOptions.device_os,
        package_expiry_time: extraOptions.package_expiry_time,
        onPremWebMobileUrl: options.onPremWebMobileUrl,
        offPremWebMobileUrl: options.offPremWebMobileUrl
      }
    };
    emailQueue.addtoQueue(mailOptions);
  }
}

//for download
const downloadGateway = async (req, res, next) => {
  try {
    const { id: gatewayId, state: packageState } = req.params;

    const gateway = await models.Gateway.findOne({
      where: {
        id: gatewayId
      }
    });

    if (!gateway) {
      throw new httpErrors.NotFound('No gateway found.');
    }

    const link = await generateLink({
      os: gateway.buildOs,
      componentId: gateway.id,
      component: 'gateway',
      expiry_time: emailLinkExpiryInMinutes,
      filename:
        `invisily-gateway-` + installerType[gateway.buildOs] + '-' + gateway.id
    });

    sendEmailData(
      prepareEmailOptions(
        gateway.email,
        'Invisily Gateway package download link',
        'download_link.html',
        `${process.env.CONTENT_URL}/api/v1/links/download/${link.key}`,
        'User',
        emailLinkExpiryInMinutes,
        'Gateway'
      )
    );

    insertIntoAdminLogs(getGatewayEmailMessage(gateway.Name), req);

    res.json({
      status: true,
      message: `Check your email for Gateway installer download link`
    });
  } catch (error) {
    next(error);
  }
};

//for gateway health
const getGatewaysHealth = async (req, res, next) => {
  try {
    const { limit, offset } = req.query;
    const options = {
      limit: limit ? limit : 10,
      offset: offset ? offset : 0,
      include: [
        {
          model: models.Gateway,
          attributes: ['Name'],
          required: false
        }
      ]
    };

    let gatewaysHealth = await models.GatewayHealth.paginate(options);
    gatewaysHealth.data = gatewaysHealth.data.map(health => {
      return {
        id: health.id,
        Gateway: health.Gateway,
        storageTotal: health.storageTotal,
        storageFree: health.storageFree,
        cpuUtilization: health.cpuUtilization,
        mainProcessCPUUtilization: health.mainProcessCPUUtilization,
        memTotal: pretty(health.memTotal * 1024),
        memFree: pretty(health.memFree * 1024),
        storageTotal: pretty(health.storageTotal * 1024),
        storageFree: pretty(health.storageFree * 1024)
      };
    });
    res.status(200).send(gatewaysHealth);
  } catch (error) {
    next(error);
  }
};

/**
 * Available gateways
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
const gatewaysAvailable = async (req, res, next) => {
  const { id } = req.params;
  const { force } = req.query;
  try {
    // if(force !== 'true')
    // {
    //   const count = await models.GatewayGroups.count()

    //   if(count) {
    //     return res.json([]);
    //   }
    // }

    const services = await models.Gateway.findAll({});

    return res.json(services);
  } catch (error) {
    next(error);
  }
};

let prepareEmailOptions = (
  recipient,
  emailSubject,
  templateFileName,
  downloadLink,
  recipientName,
  linkExpiryInMinutes,
  component
) => {
  return {
    recipient: recipient,
    emailSubject: emailSubject,
    templateFileName: templateFileName,
    downloadLink: downloadLink,
    recipientName: recipientName,
    linkExpiryInMinutes: linkExpiryInMinutes,
    component: component
  };
};

let generateSnifferCfgFile = (gateway, controller) => {
  let config = {
    SPA_pkt_dest_port: gateway.spa_port,
    SPA_pkt_length: 128,
    server_port1: '"' + gateway.port + '"',
    server_port1_conn_time_expiry: 5,
    server_protocol: '"' + 'tcp' + '"',
    serviceMessagesFilePath: '"' + './msg/' + '"',
    networkInterface: '"' + 'networkid_value_placeholder' + '"',
    component_IPAddress: '"' + gateway.ipv4_address + '"',
    del_auth_file: 0
  };
  return Object.keys(config).reduce(
    (acc, key) => `${acc}${key}=${config[key]}\n`,
    ''
  );
};

let generateGatewayCfgFile = (gateway, controller) => {
  let config = {
    gatePort: gateway.port,
    gateIPAddress: '"' + gateway.ipv4_address + '"',
    gateServerCertFilePath: '"' + 'certs/server.crt' + '"',
    gateServerKeyPemFilePath: '"' + 'certs/server.pem' + '"',
    gatePemPassword: '"' + 'SZNEe62D' + '"',
    gateServerRootCertFilePath: '"' + 'certs/cacert.pem' + '"',
    gateServerCAlocation: '"' + 'certs' + '"',
    gateVTI_devName: '"' + 'gat' + '"',
    gateserviceMessagesFilePath: '"' + './msg/' + '"',
    get_user: 0,
    ctrlspaPort: controller.spa_port,
    ctrlPort: controller.port,
    ctrlPubPort: controller.public_ipv4_port,
    ctrlIPAddress: '"' + controller.ipv4_address + '"',
    ctrlPubIPAddress: '"' + controller.public_ipv4_address + '"',
    ctrlClientCertFilePath: '"' + 'certs/client.crt' + '"',
    ctrlClientKeyPemFilePath: '"' + 'certs/client.pem' + '"',
    ctrlPemPassword: '"' + 'SZNEe62D' + '"',
    ctrlClientRootCertFilePath: '"' + 'certs/cacert.pem' + '"',
    ctrlClientCAlocation: '"' + 'certs' + '"',
    agent_ID: gateway.agent_id,
    tls_encryption: gateway.tls_encryption === true ? 1 : 0,
    device_binding: gateway.device_binding === true ? 1 : 0,
    proxy_enabled: gateway.proxy_enabled ? 1 : 0,
    NTPIP: process.env.CONTENT_URL,
    is_device_profiler_enabled: gateway.is_device_profiler_enabled ? 1 : 0,
  };
  if (gateway.counter === null || gateway.counter === undefined) {
    config.counter = 1;
  } else {
    config.counter = gateway.counter;
  }
  config.hmac_key_len = 24;
  config.henc_key_len = 32;
  config.henc_key = '"' + gateway.enc_key.split('=')[0] + '"';
  config.henc_key_iv = '"' + gateway.enc_key.split('=')[1] + '"';
  config.hmac_key = '"' + gateway.HMacKey.replace('\n', '').trim() + '"';
  // config.auth_userName = '"' + "sdpdummy" + '"';
  // config.auth_password = '"' + "Sdp@1234" + '"';
  config.networkInterface = '"' + 'networkid_value_placeholder' + '"';
  config.on_premises = gateway.deployment_location;

  // const webUrl = gateway.deployment_location === 1 ? projectConfig.onPremApiUrl : projectConfig.offPremApiUrl;
  // config.NTPIP = `"${utils.getHostOrIpFromUrl(webUrl)}"`;
  config.NTPIP = `"${utils.getHostOrIpFromUrl(config.NTPIP)}"`;

  return Object.keys(config).reduce(
    (acc, key) => `${acc}${key}=${config[key]};\n`,
    ''
  );
};

let writeCfgFile = (gateway, installerDir, resolve, reject) => {
  models.Controller.findOne({
    where: {
      roleId: 1
    }
  }).then(function(controller) {
    if (controller) {
      console.log('Found controller record in DB');
      fs.writeFileSync(
        `${installerDir}/gateway_config.cfg`,
        generateGatewayCfgFile(gateway, controller)
      );
      fs.writeFileSync(
        `${installerDir}/sniffer_config.cfg`,
        generateSnifferCfgFile(gateway, controller)
      );
      resolve();
    } else {
      console.log('No controller record found in DB');
      reject();
    }
  });
};

let writeControllerDataFile = (installerDir, resolve, reject) => {
  models.Controller.findAll()
  .then(function(controller) {
    if (controller) {
      console.log('Found controllers records in DB');
      const result = { controllers: [] };
        controller.forEach(entry => {
            const controllerData = {
                bio_port: entry.mfa_server_port,
                id: entry.id,
                obp_port: entry.onboarding_port,
                obp_url: entry.onboarding_url,
                private_ip: entry.ipv4_address,
                private_port: entry.port,
                public_ips: entry.public_ipv4_address,
                public_port: entry.public_ipv4_port,
                spa_port: entry.spa_port,
            };
          result.controllers.push(controllerData);
        });

      stringifyResult = JSON.stringify(result);

      fs.writeFileSync(
        `${installerDir}/controllers.data`,
        stringifyResult
      );
      resolve();
    } else {
      console.log('No controllers record found in DB');
      reject();
    }
  });
};

let zipInstaller = (
  gateway,
  installerDir,
  packageDir,
  type,
  installerType,
  resolve,
  reject
) => {
  let output = fs.createWriteStream(
    `${packageDir}/invisily-gateway-` +
      installerType +
      '-' +
      gateway.id +
      `.zip`
  );
  let archive = archiver('zip', {
    zlib: { level: 9 } // Sets the compression level.
  });
  output.on('close', function() {
    console.log('zip generated');
    resolve();
  });
  archive.on('error', function(err) {
    reject(err);
  });
  // pipe archive data to the file
  archive.pipe(output);
  archive.append(fs.createReadStream(`${installerDir}/gateway_config.cfg`), {
    name: 'gateway_config.cfg'
  });
  archive.append(fs.createReadStream(`${installerDir}/sniffer_config.cfg`), {
    name: 'sniffer_config.cfg'
  });
  archive.append(fs.createReadStream(`${installerDir}/controllers.data`), {
    name: 'controllers.data'
  });
  archive.append(fs.createReadStream(`${installerDir}/rlup.enc`), {
    name: `rlup.enc`
  });
  archive.append(fs.createReadStream(`${installerDir}/installer.${type}`), {
    name: `installer.${type}`
  });
  archive.append(
    fs.createReadStream(`${installerDir}/install_invisily_gateway.sh`),
    { name: `install_invisily_gateway.sh` }
  );
  archive.append(
    fs.createReadStream(`${installerDir}/Installation_Guide_Gateway.txt`),
    { name: `Installation_Guide_Gateway.txt` }
  );
  archive.directory(`${installerDir}/certs`, 'certs');
  archive.finalize();
};

function createDir(dirPath) {
  return fsExtra
    .ensureDir(dirPath)
    .then(() => {
      console.log('created directory strcuture successfully ' + dirPath);
    })
    .catch(err => {
      console.log('Error saving attached file for Updates' + err);
      throw err;
    });
}

function generateInstallerZip(gateway, packageState, isNew = true) {
  return setExpiryTime().then(function() {
    console.log(packageState);
    gateway.packageDownloaded = false;
    gateway.installer_status = 'processing';
    if (packageState === '1') {
      gateway.isDownloadPackage = true;
      gateway.save();
    } else {
      gateway.isDownloadPackage = false;
      gateway.save();
    }
    var time = moment()
      .utc()
      .local()
      .format('YYYY-MM-DD HH:mm:ss');
    gateway.updatedAt = time;
    gateway.installer_status = 'processing';
    gateway.save();
    let os = gateway.buildOs;
    let dirName =
      '/temp' +
      gateway.id +
      '-' +
      moment()
        .utc()
        .local()
        .format('YYYY-MM-DD-HH-mm-ss') +
      '-' +
      Math.random()
        .toString(36)
        .substring(2);
    let packageDir =
      `${projectConfig.serviceInstallerPath}/gateway/output/` + os;
    let key = `${gateway.updatedAt.getTime()}:` + os;
    let installerDir =
      `${projectConfig.serviceInstallerPath}/gateway/input/` + os + dirName;
    let certDirectory = `${installerDir}/certs`;
    let installerFile =
      `${projectConfig.serviceInstallerPath}/gateway/input/` +
      os +
      `/installer.${mapping[os]}`;
    let installInvSH =
      `${projectConfig.serviceInstallerPath}/gateway/input/` +
      os +
      `/install_invisily_gateway.sh`;
    let installationGuide =
      `${projectConfig.serviceInstallerPath}/gateway/input/` +
      os +
      `/Installation_Guide_Gateway.txt`;

    console.log('installerDir ...' + installerDir);
    console.log('packageDir ...' + packageDir);
    console.log('key ...' + key);
    console.log('certDirectory ...' + certDirectory);
    console.log('installerFile ...' + installerFile);

    createDir(certDirectory);
    createDir(packageDir);
    createDir(installerDir);
    // helper.mkDir(certDirectory);
    // helper.mkDir(packageDir);
    // helper.mkDir(installerDir);

    // asynchronous chaining
    return new Promise((resolve, reject) => {
      exec(
        `${projectConfig.serviceInstallerPath}/give-write-permission.sh ${installerDir} ${installerFile}`,
        (error, stdout, stderr) => {
          console.log('stdout ... ' + stdout);
          if (error) {
            reject(error);
          } else {
            resolve();
          }
        }
      );
    })
      .then(
        () =>
          new Promise((resolve, reject) => {
            exec(
              `${projectConfig.serviceInstallerPath}/give-write-permission.sh ${installerDir} ${installInvSH}`,
              (error, stdout, stderr) => {
                console.log('stdout ... ' + stdout);
                if (error) {
                  reject(error);
                } else {
                  resolve();
                }
              }
            );
          })
      )
      .then(
        () =>
          new Promise((resolve, reject) => {
            exec(
              `${projectConfig.serviceInstallerPath}/give-write-permission.sh ${installerDir} ${installationGuide}`,
              (error, stdout, stderr) => {
                console.log('stdout ... ' + stdout);
                if (error) {
                  reject(error);
                } else {
                  resolve();
                }
              }
            );
          })
      )
      .then(
        () =>
          new Promise((resolve, reject) => {
            // generate certificates
            // this will also generate the certificates in the given folder
            exec(
              `sudo ${projectConfig.serviceInstallerPath}/generate-ssl-certs.sh ${key} ${certDirectory}`,
              (error, stdout, stderr) => {
                if (error) {
                  reject(error);
                } else {
                  // generate copy of client certificates and name them as server certificates
                  fsExtra.copy(
                    certDirectory + '/client.crt',
                    certDirectory + '/server.crt',
                    err => {
                      if (err) {
                        reject(err);
                        console.log(err && (err.message || err));
                        //throw err;
                      }

                      fsExtra.copy(
                        certDirectory + '/client.pem',
                        certDirectory + '/server.pem',
                        err => {
                          if (err) throw err;
                          console.log('client.pem was copied to server.pem');
                          resolve();
                        }
                      );
                    }
                  );
                }
              }
            );
          })
      )
      .then(
        () =>
          new Promise((resolve, reject) => {
            console.log('Generating encrypted files');

            let encryption_command = `${scriptsPath}/gw_rlup.sh '${process.env.LOCAL_USER}' '${process.env.LOCAL_PASSWORD}' '${gateway.HMacKey}'`;
            console.log('Encryption command', encryption_command);
            exec(encryption_command, (error, stdout, stderr) => {
              if (error) {
                return reject(error);
              }

              copyFilePromise(
                '/tmp/gw_rlup/rlup.enc',
                installerDir + '/rlup.enc'
              )
                .then(resolve)
                .catch(reject);
            });
          })
      )
      .then(
        () =>
          new Promise((resolve, reject) => {
            console.log('Generating configuration file');
            writeCfgFile(gateway, installerDir, resolve, reject);
          })
      )
      .then(
        () =>
          new Promise((resolve, reject) => {
            console.log('Generating controllers data file');
            writeControllerDataFile(installerDir, resolve, reject);
          })
      )
      .then(
        () =>
          new Promise((resolve, reject) => {
            console.log('Generating Zip file');
            zipInstaller(
              gateway,
              installerDir,
              packageDir,
              mapping[os],
              installerType[os],
              resolve,
              reject
            );
          })
      )
      .then(() => {
        // save file in database
        console.log('Saving Gateway certificate files');
        gateway.cacert = fs.readFileSync(`${certDirectory}/cacert.pem`);
        gateway.clientcrt = fs.readFileSync(`${certDirectory}/client.crt`);
        gateway.clientpem = fs.readFileSync(`${certDirectory}/client.pem`);
        gateway.link_key = key;
        gateway.link_key_expiry = moment().add(
          emailLinkExpiryInMinutes,
          'minutes'
        );
        return gateway.save();
      })
      .then(
        () =>
          new Promise((resolve, reject) => {
            exec(
              `${projectConfig.serviceInstallerPath}/delete-temp-folder.sh ${installerDir}`,
              (error, stdout, stderr) => {
                console.log('Executing delete-temp-folder.sh');
                console.log('stdout ... ' + stdout);
                if (error) {
                  reject(error);
                } else {
                  resolve();
                }
              }
            );
            gateway.installer_status = 'success';
            gateway.onPrenDownloadLink =
              `${projectConfig.onPremApiUrl}/gateways/download/service/` +
              new Buffer(key).toString('base64');
            gateway.save();
          })
      )
      .then(async () => {
        const fileName = `invisily-gateway-${installerType[gateway.buildOs]}-${
          gateway.id
        }`;

        require('../queues/uploadToContent').addtoQueue({
          os: os,
          component: 'gateway',
          folderPath: packageDir + '/',
          filename:
            `invisily-gateway-` +
            installerType[gateway.buildOs] +
            '-' +
            gateway.id
        });

        const link = await generateLink({
          os: gateway.buildOs,
          componentId: gateway.id,
          component: 'gateway',
          expiry_time: 60,
          filename: fileName
        });

        const downloadLink = `${process.env.CONTENT_URL}/api/v1/links/download/${link.key}`;

        gateway.pkg_url = downloadLink;
        await gateway.save();

        const message = {
          updates: [
            {
              URL: downloadLink,
              Agent_ID: gateway.agent_id
            }
          ],
          is_configuration_changed: !isNew
        };

        const softwareUpdate = await models.Update.findOne({
          where: {
            buildComponent: 'gateway',
            buildOs: gateway.buildOs,
            isDeleted: false
          }
        });

        if (softwareUpdate) {
          message['version'] = softwareUpdate.buildVersion;
        }

        await publishGatewaySoftware(actionType.UPDATED, message);

        return;
      })
      .catch(err => {
        console.log(err && (err.message || err));
        gateway.installer_status = 'error';
        gateway.save();
        exec(
          `${projectConfig.serviceInstallerPath}/delete-temp-folder.sh ${installerDir}`,
          (error, stdout, stderr) => {
            console.log('Executing delete-temp-folder.sh');
            console.log('stdout ... ' + stdout);
            if (error) {
              console.log(error && (error.message || error));
            } else {
              console.log('Temp Gateway folder deleted.');
            }
          }
        );
      });
  });
}

const getGatewayHealthStats = async (req, res, next) => {
  try {
    let { interval = 1 } = req.query;
    const { gatewayId } = req.params;

    const allowedIntervals = [1, 5, 30];
    if (!allowedIntervals.includes(parseInt(interval))) {
      throw new httpErrors.BadRequest('Invalid interval value');
    }

    const gateway = await models.Gateway.findOne({
      id: gatewayId
    });

    if (!gateway) {
      throw new httpErrors.BadRequest('Invalid gateway id');
    }

    function mapData(data, keys = []) {
      return data.map(row => {
        let response = {
          time: row.createdAt
        };
        keys &&
          keys.forEach(key => {
            if (row[key] || row[key] === 0) {
              response[key] = row[key];
            }
          });
        return response;
      });
    }

    const minDate = moment()
      .subtract(interval, 'days')
      .format();
    const maxDate = moment().format();

    const groupBy =
      interval === 1 ? `group by createdAt` : `group by day(createdAt)`;

    let query = `
        select createdAt,
        AVG(memTotal) / 1024 as memTotal, 
        AVG(memFree) / 1024 as memFree , 
        AVG(storageFree) / 1024 as storageFree , 
        AVG(storageTotal) / 1024 as storageTotal ,
        AVG(cpuUtilization) as cpuUtilization ,
        AVG(mainProcessCPUUtilization) as mainProcessCPUUtilization , 
        AVG(snifferProcessCPUUtilization) as snifferProcessCPUUtilization 
        from GatewayRuntimeHealths crh 
        where crh.gatewayId = ${gatewayId} AND 
        crh.createdAt >= '${minDate}' AND 
        crh.createdAt <= '${maxDate}'
        ${groupBy}, createdAt
      `;
    const [result, gatewayHealth] = await Promise.all([
      models.sequelize.query(query),
      models.GatewayHealth.findOne({
        where: {
          gatewayId
        }
      })
    ]);

    res.status(200).send({
      lastUpdated: gatewayHealth && gatewayHealth.updatedAt,
      os: gatewayHealth && gatewayHealth.OS,
      version: gatewayHealth && gatewayHealth.componentVersion,
      cpuDetail: gatewayHealth && gatewayHealth.CPU,
      lastErrorLog: gatewayHealth && gatewayHealth.lastErrorLog,
      lastSnifferErrorLog: gatewayHealth && gatewayHealth.snifferLastErrorLog,
      memory: mapData(result[0], ['memFree', 'memTotal']),
      storage: mapData(result[0], ['storageTotal', 'storageFree']),
      cpu: mapData(result[0], [
        'cpuUtilization',
        'snifferProcessCPUUtilization',
        'mainProcessCPUUtilization'
      ])
    });
  } catch (e) {
    next(e);
  }
};

module.exports = {
  downloadGateway,
  deleteGateway,
  getGateway,
  getGatewayServices,
  status,
  getGateways,
  updateGateway,
  addGateway,
  downloadPackage,
  gatewaysAvailable,
  gatewayZipGenerator: generateInstallerZip,
  getGatewaysOSAndProtocols,
  cloneGateway,
  getQuickGateways,
  getGatewaysHealth,
  getGatewayHealthStats
};
