var httpErrors = require('http-errors');
const moment = require('moment');
const models = require('../models');
const _ = require('lodash')
const { actionType, publishClientGroups, publishTrustedHosts } = require('../routes/common/event-publisher');
const { insertIntoAdminLogs } = require('../routes/common/adminLogs');
const { getGroupCreateMessage, getGroupEditMessage, getGroupDeleteMessage, getClientAddToGroupMessage, getClientDeleteFromGroupMessage } = require('../config/adminLogs.config');

const getGroups = async (req, res, next) => {
    try {
        const {
          limit,
          offset,
          sort = 'id',
          dir = 'desc',
          searchTerm = ''
        } = req.query;
    
        const options = {
          order: [
            [models.sequelize.col('GroupDetail.client_count'), 'DESC'],
            [models.sequelize.col('createdAt'), 'DESC']
          ],
          attributes: ['id', 'name', 'description', 'createdAt', 'updatedAt', 'is_ldap_deleted'],
          include: [{
            model: models.GroupDetail,
            attributes: ['groupId', 'service_count', 'client_count']
          }],
          where: {},
          limit: limit,
          offset: offset,
          order: [
            [sort, dir]
          ]
        }
    
        if (searchTerm.trim()) {
          options.where = {
            name: {
              [Op.like]: `%${searchTerm.trim()}%`
            }
          }
        } else {
          options.where = { }
        }
    
        const result = await models.Group.paginate(options);
        res.json(result);
    
    } catch (error) {
        next(error);
    }
}

const getAvailableGroups = async (req, res, next) => {
  try {
      const groups = await models.Group.findAll({
        where: { }
      })

      res.status(200).json({
        data: groups
      })
  
  } catch (error) {
      next(error);
  }
}


const addClient = async (req, res, next) => {
    try {
        const groupId = +req.params.groupId;
        const clientId = +req.params.clientId;
    
        const group = await models.Group.find({
          where: {
            id: groupId
          }
        });
    
        if (!group) {
          throw new httpErrors.NotFound('Group not found.');
        }
    
        group.addClient(clientId);
    
        await models.Client.update({
          updatedAt: moment()
        }, {
          where: {
            id: clientId
          }
        });

        await publishClientGroups(actionType.UPDATED, {
          id: groupId,
          new_ids: [clientId],
          del_ids: []
        });
    
        res.send('');
    } catch (error) {
        next(error);
    }
}


const deleteClient = async (req, res, next) => {
    try {
        const groupId = +req.params.groupId;
        const clientId = +req.params.clientId;
    
        const group = await models.Group.find({
          where: {
            id: groupId
          }
        });
    
        if (!group) {
          throw new httpErrors.NotFound('Group not found.');
        }
    
        group.removeClient(clientId);
    
        await models.Client.update({
          updatedAt: moment()
        }, {
          where: {
            id: clientId
          }
        });

        await publishClientGroups(actionType.UPDATED, {
          id: groupId,
          new_ids: [],
          del_ids: [clientId]
        });
    
        res.send('');
    } catch (error) {
        next(error);
    }
}

const getGroupClients = async (req, res, next) => {
  try {
      const groupId = +req.params.groupId;
  
      const group = await models.Group.find({
        where: {
          id: groupId
        },
        include: [
          {
            model: models.Client,
            required: false,
            where: { }
          }
        ]
      });
  
      if (!group) {
        throw new httpErrors.NotFound('Group not found.');
      }

      res.status(200).json({
        data: group.Clients || []
      });
  } catch (error) {
      next(error);
  }
}

const getGroup = async (req, res, next) => {
    try {
        const groupId = +req.params.groupId;
    
        const group = await models.Group.find({
          where: {
            id: groupId
          }
        });
    
        if (!group) {
          throw new httpErrors.NotFound('Group not found.');
        }
    
        res.json(group);
    } catch (error) {
        next(error);
    }
}


const getGroupClientsWithPagination = async (req, res, next) => {
  try {
      const groupId = +req.params.groupId;

      let {
        limit,
        offset,
        sort = 'id',
        dir = 'desc',
        searchTerm = '',
        type = '',
        os = ''
      } = req.query;


      const clientOptions = {
        model: models.Client,
        required: false,
        where: { }
      }

      if(type && type != 'null' && type != 'undefined') {
        const types = ['Server', 'Client']
        type = types.includes(type) ? type: types[0]
        clientOptions.where.client_type = type
      }

      if(os && os != 'null' && os != 'undefined') {
        clientOptions.where.device_os = os
      }

      if(searchTerm.trim()){
        clientOptions.where.Name = {
          [Op.like]: `%${searchTerm.trim()}%`
        }
      }
  
      const options = {
        where: {
          id: groupId
        },
        include: [clientOptions],
        order: [
          [models.Client, sort, dir]
        ],
      }

      const group = await models.Group.find(options);
  
      if (!group) {
        throw new httpErrors.NotFound('Group not found.');
      }

      const clients = group.Clients.slice(parseInt(offset) * parseInt(limit), (parseInt(offset) * parseInt(limit)) + parseInt(limit))
      
      res.json({
        total: group.Clients.length,
        data: clients,
        name: group.name
      });
  } catch (error) {
      next(error);
  }
}


const getClients = async (req, res, next) => {
    try {
        const groupId = +req.params.groupId;
    
        const term = req.params.term === '__MEMBERS__' ?
          req.params.term : new Buffer(req.params.term, 'base64')
          .toString('ascii');
    
    
        const baseQuery = {
          order: [
            [
              models.sequelize.fn(
                'count',
                models.sequelize.col('GroupClients.clientId')
              ),
              'DESC'
            ],
            'Name'
          ],
          attributes: [
            'id',
            'Name',
            'device_os',
            [
              models.sequelize.fn(
                'count',
                models.sequelize.col('GroupClients.clientId')
              ),
              'is_member'
            ]
          ],
          include: [{
            model: models.GroupClient,
            attributes: [],
            where: {
              groupId
            },
            required: false
          }],
          group: ['Client.id']
        };
    
        const getGroupAllClientsMatchingTerm = {
          ...baseQuery,
          where: {
            $and: {
              $or: {
                Name: {
                  $like: `%${term}%`
                },
                user: {
                  $like: `%${term}%`
                },
                device_os: {
                  $like: `%${term}%`
                },
                id: {
                  $like: `%${term}%`
                }
              }
            }
          }
        };
    
        const getGroupMemberClients = {
          ...baseQuery,
          having: models.Sequelize.literal(`is_member = 1`),
          where: { }
        };
    
        const clients = await models.Client.findAll(
          term === '__MEMBERS__' ?
          getGroupMemberClients :
          getGroupAllClientsMatchingTerm
        );
    
        res.json({
          clients
        });
    } catch (error) {
        next(error);
    }
}


const checkDuplicateName = (name, id) => {
  const whereClause = {
    name
  };

  if (id) {
    whereClause.id = {
      $ne: id
    };
  }

  return models.Group.find({
    where: whereClause
  }).then(group => {
    if (!group) {
      return Promise.resolve();
    } else {
      throw new httpErrors.NotFound('Group with same name already exist');
    }
  });
};


const addGroup = async (req, res, next) => {
    const transaction = await models.sequelize.transaction();
    try {
      let {
        services,
        hosts,
        clientIds = [],
        addAllClients,
        clientType = 'Server',
        ...groupData
      } = req.body;

      await checkDuplicateName(groupData.name);
      
      if(addAllClients === true && (!clientIds || clientIds.length < 1)) {

        const clientTypes = ['Server', 'Client']

        clientType = clientTypes.includes(clientType)? clientType: clientTypes[0]

        const allClients = await models.Client.findAll({
          where: {
            client_type: clientType
          },
          attributes: ['id'],
        })
        clientIds = allClients.map(client => client.id)
      }

      const clientTypes = await models.Client.findAll({
        where: {
          id: clientIds
        },
        attributes: ['client_type'],
        group: ['client_type']
      })

      if(clientTypes.length > 1) {
        throw new Error('Mix of server and end user clients not allowed')
      }

      const newGroup = await models.Group.create(groupData,{
        transaction
      });
      
      const hostsWithGroup = hosts.map(host => ({
        ...host,
        parentId: newGroup.id,
        parentType: 'Group'
      }));

      const clientsOfGroup = clientIds.map(clientId => {
        return {
          clientId,
          groupId: newGroup.id
        }
      })
      
      const servicesWithGroup = Object.keys(services).map(serviceId => ({
        serviceId: serviceId,
        groupId: newGroup.id
      }));
    
      await Promise.all([
        models.GroupService.bulkCreate(servicesWithGroup, {
          transaction
        }),
        models.GlobalIps.bulkCreate(hostsWithGroup, {
          transaction
        }),
        models.GroupClient.bulkCreate(clientsOfGroup, { transaction })
      ]);
  
      await transaction.commit();

      const affectedClients = await models.GroupClient.findAll({
        attributes: [
          'clientId'
        ],
        where: {
          groupId: newGroup.id
        }
      });

      const affectedClientIds = affectedClients.map(x => x.clientId);

      await publishClientGroups(actionType.ADDED, {
        id: newGroup.id,
        name: newGroup.name,
        new_ids: affectedClientIds,
        del_ids: []
      });

      insertIntoAdminLogs(
        getGroupCreateMessage(newGroup.name),
        req
      )

      res.json(`Group '${groupData.name}' created successfully.`);
    
    } catch (error) {
      await transaction.rollback();
      next(error);
    }
}


const updateGroup = async (req, res, next) => {
    try {
        const groupId = +req.params.id;
        let { 
          name, 
          clientIds = [], 
          appendClients = false,
          addAllClients = false,
          clientType = 'Server',
          description
         } = req.body;

         if(addAllClients === true) {

          const clientTypes = ['Server', 'Client']

          clientType = clientTypes.includes(clientType)? clientType: clientTypes[0]

          const allClients = await models.Client.findAll({
            where: {
              client_type: clientType
            },
            attributes: ['id']
          })
          clientIds = allClients.map(client => client.id)
        }
        
        const transaction = await models.sequelize.transaction();
      
        await checkDuplicateName(name, groupId);
        const group = await models.Group.find({
          where: {
            id: groupId
          }
        });
        
        if (!group){
          throw new httpErrors.NotFound('Group with this id does not exist');
        }
      
        await group.update({
          name,
          description,
          updatedAt: moment()
        }, { transaction });

        const oldClients = await group.getClients({ attributes: ['id'] })
        const oldClientIds = oldClients.map(client => client.id)
        
        const clientsToDelete = _.difference(oldClientIds, clientIds).map(clientId => ({ clientId, groupId: group.id}))
        const clientsToAdd = _.difference(clientIds, oldClientIds).map(clientId => ({ clientId, groupId: group.id}))

        let clinetsToCheck = oldClientIds.filter(clientId => clientsToDelete.includes(clientId))
        const clientTypes = await models.Client.findAll({
          where: {
            id: [...clientIds, ...clinetsToCheck]
          },
          attributes: ['client_type'],
          group: ['client_type']
        })

        if(clientTypes.length > 1) {
          throw new httpErrors('Mix of server and end user clients not allowed')
        }



        await models.GroupClient.bulkCreate(clientsToAdd, { transaction })

        if(clientsToDelete.length > 0 && appendClients !== true) {
          let query = 'DELETE FROM `GroupClients` WHERE '
          clientsToDelete.forEach((row, index) => {
            let or = ''
            if(index < clientsToDelete.length - 1)
            or = 'OR'
            query = `${query} (clientId = ${parseInt(row.clientId)} AND groupId = ${parseInt(row.groupId)}) ${or}`
          })
          await models.sequelize.query(query, { transaction })
        }

        await transaction.commit()

        insertGroupClientLogs(
          req, 
          groupId, 
          getClientAddToGroupMessage, 
          clientsToAdd.map(client => client.clientId)
        );

        insertGroupClientLogs(
          req, 
          groupId, 
          getClientDeleteFromGroupMessage, 
          clientsToDelete.map(client => client.clientId)
        );

        const affectedClients = await models.GroupClient.findAll({
          attributes: [
            'clientId'
          ],
          where: {
            groupId 
          }
        });

        const affectedClientIds = affectedClients.map(x => x.clientId);

        const idsToDelete = clientsToDelete.map(x => x.clientId);

        const idsToAdd = clientsToAdd.map(x => x.clientId);

        if(idsToDelete.length > 0 || idsToAdd.length > 0 || group._changed.hasOwnProperty('name')) {
          await publishClientGroups(actionType.UPDATED, {
            id: groupId,
            name: name,
            new_ids: idsToAdd,
            del_ids: idsToDelete
          });
        }
        
        //if(affectedClientIds.length > 0) {
        //  await publishTrustedHosts(actionType.UPDATED, {
        //    ids: affectedClientIds,
        //    parentType: 'Group'
        //  });
        //}
        insertIntoAdminLogs(
          getGroupEditMessage(name),
          req
        )
        return res.json(`Group '${name}' updated successfully.`);
    } catch (error) {
        next(error);
    }
}


const deleteGroup = async (req, res, next) => {
    const transaction = await models.sequelize.transaction();
    try {
      const id = +req.params.id;
  
      await models.Group.destroy({
        where: {
          id
        },
        transaction
      });
    
      const [,groupClients] = await Promise.all([
        models.GroupService.destroy({
          where: {
            groupId: id
          },
          transaction
        }),
        models.GroupClient.findAll({
          attributes: ['clientId'],
          where: {
            groupId: id
          },
          transaction
        })
      ]);

      await models.GroupClient.destroy({
        where: {
          groupId: id
        },
        transaction
      });

      const affectectedClientIds = groupClients.map(g => g.clientId);

      await Promise.all([
        publishClientGroups(actionType.DELETED, {
          ids: [id],
          new_ids: [],
          del_ids: affectectedClientIds
        }),
        publishTrustedHosts(actionType.DELETED, {
          ids: affectectedClientIds,
          parentType: 'Group'
        })
      ]);

      const group = await models.Group.findOne({
        where: {
          id
        }
      });

      await transaction.commit();
      
      insertIntoAdminLogs(
        getGroupDeleteMessage(group.name),
        req
      )

      res.json(`Group deleted successfully.`);
    } catch (error) {
      await transaction.rollback();
      next(error);
    }
}

const bulkDeleteGroups = async(req, res, next) => {
  try {
    const { groupIds, excludeGroups } = req.body
    const { allSelected, search } = req.query

    const options =  {
      where: { }
    }

    if(allSelected == 'true' ) {
      options.where.id = {
        [Op.not]: excludeGroups
      }

      if(search.trim()) { 
        options.where.name  = {
          [Op.like]: `%${search.trim()}%`
        }
      }

    } else {
      options.where.id = groupIds
    }

    const groupsToDelete = await models.Group.findAll(options);
    const groupsToDeleteIds = groupsToDelete.map(group => group.id)
    await models.Group.destroy(options) 
    const groupClients = await models.GroupClient.findAll({
        attributes: ['clientId'],
        where: {
          groupId: groupsToDeleteIds
        },
      })

    const affectectedClientIds = groupClients.map(g => g.clientId);


    if(groupsToDeleteIds && groupsToDeleteIds.length > 0) {
      await Promise.all([
        publishClientGroups(actionType.DELETED, {
          ids: groupsToDeleteIds,
          new_ids: [],
          del_ids: affectectedClientIds
        }),
        publishTrustedHosts(actionType.DELETED, {
          ids: affectectedClientIds,
          parentType: 'Group'
        })
      ]);
    } 

    res.status(200).json({message: 'Client groups deleted'})


  } catch(e) {
    next(e)
  }
}

const insertGroupClientLogs = async (
  req, 
  groupId, 
  getMessageFunc,    /// method to get relevent logs message for example you can pass getClientAddToGroupMessage or getClientDeleteFromGroupMessage method according to condition. other params will be same 
  clientIds
) => {
  models.Group.findById(groupId).then(group => {
    clientIds.map(clientId => {
      models.Client.findById(clientId).then(client => {
        insertIntoAdminLogs(
          getMessageFunc(client.Name, group.name),
          req
        )
      })
    })
  });

}


module.exports = {
    deleteGroup,
    updateGroup,
    getClients,
    getGroup,
    addClient,
    deleteClient,
    addGroup,
    deleteClient,
    getGroups,
    getGroupClientsWithPagination,
    getAvailableGroups,
    bulkDeleteGroups,
    getGroupClients
};