const moment = require('moment');
var httpErrors = require('http-errors');
var models = require('../models');
var dirSyncManager = require('../routes/common/dirsync-manager');
const fs = require('fs');
const execSync = require('child_process').execSync;
const projectConfig = require('../config/config')['project'][
  process.env.NODE_ENV
];
const { insertIntoAdminLogs } = require('../routes/common/adminLogs')
const { getLdapCreateMessage, getLdapEditMessage, getLdapDeleteMessage } = require('../config/adminLogs.config');
const { publishLDAP, actionType } = require('../routes/common/event-publisher');

/* GET users listing. */
const getClients = async (req, res, next) => {
  try {
    const {
      limit,
      offset,
      sort='createdAt',
      dir='desc',
      searchKey=''
    } = req.query;

    let options = {
      where: { },
      limit: limit,
      offset: offset,
      order: []
    };

    const allowedSortValues = [
      'createdAt',
      'id'
    ];

    if(searchKey.trim()){
      options.where.LDAP_user = {
        [Op.like]: `%${searchKey.trim()}%`
      }
    }

    if (allowedSortValues.includes(sort)) {
      options.order.push([models.sequelize.literal(sort), dir]);
    }

    const result = await models.Ldap.paginate(options);
    res.json(result);

  } catch (error) {
    next(error);
  }
}


function importCertInKeyStore(ldap) {
  if (ldap.LDAP_url.indexOf('ldaps') > -1) {
    return new Promise((resolve, reject) => {
      const { componentInstallationPath } = projectConfig;
      const CAFileName = `${componentInstallationPath}/storage/temp/ca-${moment().get(
        'millisecond'
      )}.pem`;
      const CAPemAlias = `ca-ldaps-webportal-inv`;
      fs.writeFileSync(CAFileName, ldap.LDAP_ca);
      try {
        execSync(
          `keytool -delete -alias ${CAPemAlias} -keystore ${process.env.JAVA_HOME}/lib/security/cacerts -storepass ${process.env.JAVA_KEYSTORE_PASS}`
        );
      } catch (err) {
        console.log(err && (err.message || err));
      }

      try {
        const keyToolOutput = execSync(
          `keytool -import -trustcacerts -noprompt -file ${CAFileName} -alias ${CAPemAlias} -keystore ${process.env.JAVA_HOME}/lib/security/cacerts -storepass ${process.env.JAVA_KEYSTORE_PASS}`
        );
        resolve(keyToolOutput.toString());
      } catch (err) {
        reject(err.output.toString());
      }
    });
  }

  return Promise.resolve();
}


//update ldap
const updateLdap = async (req, res, next) => {
  try {
    const { ldap:reqLdap } = req.body;
  
    const ldap = await models.Ldap.find({
        where: {
          id: req.params.id
        }
    });

    if(!ldap){
      throw new httpErrors.NotFound("No ldap to update.")
    }

    await importCertInKeyStore(reqLdap);
    await ldap.update(reqLdap, {
      event: 'update'
    });

    let fullUrl = reqLdap.LDAP_url.split('://');
    if (fullUrl.length > 1) {
      fullUrl = fullUrl[1];
    }
    if (fullUrl.split(':').length > 1) {
      const [url, port] = fullUrl.split(':');
      await models.GlobalIps.findOrCreate({
        where: {
          port: port,
          address: url,
          direction: 0,
          protocol: 'tcp'
        },
        defaults: {
          createdAt: moment(),
          updatedAt: moment(),
          label: 'LDAP'
        }
      });
    }
    insertIntoAdminLogs(
      getLdapEditMessage(),
      req
    )

    res.json({
      success: true,
      message: 'Update Successfully'
    });
  
  } catch (error) {
    next(error);
  }
}


/* Get a single user detail */
const getLdapUser = async (req, res, next) => {
  try {

    const ldapUser = await models.Ldap.findOne({
      where: {
        id: req.params.id
      }
    });
    
    if(!ldapUser){
      throw new httpErrors.NotFound("ldap user not found.");
    }

    ldapUser.LDAP_ca = ldapUser.LDAP_ca instanceof Buffer ? ldapUser.LDAP_ca.toString() : ldapUser.LDAP_ca;
    ldapUser.LDAP_cert = ldapUser.LDAP_cert instanceof Buffer ? ldapUser.LDAP_cert.toString() : ldapUser.LDAP_cert;
    res.json(ldapUser);

  } catch (error) {
    next(error);
  }
}


const addUser = async (req, res, next) => {
  try {
    const { ldap } = req.body;
    await models.Ldap.create(ldap);
    
    insertIntoAdminLogs(
      getLdapCreateMessage(),
      req
    )

    res.json({
      success: true,
      message: 'Add Successfully'
    });

  } catch (error) {
    next(error);
  }
}


const duplicationCheck = async (req, res, next) => {
  try {

    const { value } = req.body;

    //TODO : 
    const ldap  = await models.Ldap.find({
      where: {
        LDAP_user: value
      }
    })

    if (!ldap) {
      return res.send(200);
      
    }

    res.send(400);
    
  } catch (error) {
    next(error);
  }
}

// delete a single user
const deleteLdap = async (req, res, next) => {
  
  const transaction = await models.sequelize.transaction();

  try {

    const settings = await models.Settings.findOne({
      limit: 1,
      order: [['id', 'DESC']]
    });

    if(!settings){
      throw new httpErrors.BadGateway('Failed to delete LDAP user.');
    }

    if (settings.dirsync_enabled) {
      await settings.update({
        dirsync_enabled: 0
      }, { transaction });
    }

    const ldap = await models.Ldap.find({
      where: {
        id: req.params.id
      }
    })

    if(ldap){
      await ldap.destroy({
        transaction,
        event: 'delete'
      });
    }

    const cron = req.app.get('cron');
    await dirSyncManager.destroy(cron);
    await transaction.commit();

    const dataValues = ldap;
    const message = {
      data: {
        LDAP_url: dataValues.LDAP_url,
        LDAP_user: dataValues.LDAP_user,
        LDAP_ca: dataValues.LDAP_ca,
        LDAP_ca_name: dataValues.LDAP_ca_name,
        LDAP_timeout: dataValues.LDAP_timeout,
        LDAP_connectTimeout: dataValues.LDAP_connectTimeout,
        LDAP_id: dataValues.id
      }
    };
  
    await publishLDAP(actionType.DELETED, message);

    insertIntoAdminLogs(
      getLdapDeleteMessage(),
      req
    )

    res.json({
      success: true,
      message: 'Delete Successfully'
    });


  } catch (error) {
    await transaction.rollback();
    next(error);
  }
}


module.exports = {
  getClients,
  updateLdap,
  duplicationCheck,
  addUser,
  getLdapUser,
  deleteLdap
};
