var path = require('path');
var fs = require('fs');
var fsExtra = require('fs-extra');
var httpErrors = require('http-errors');
// const NodeRSA = require('node-rsa');
var models = require('../models');
let helper = require('../routes/common/helper');
let language = require('../routes/common/language');
let appDir = path.dirname(require.main.filename);
appDir = appDir.substring(0, appDir.lastIndexOf('/'));
var moment = require('moment');
const argon2 = require('argon2');
var signFilePath = '/var/www/html/admin-gui/storage/temp/';
var signPubKey = process.env.OPENSSL_PUBLIC_KEY;
const { actionType, publishLicense } = require('../routes/common/event-publisher');
const { insertIntoAdminLogs } = require('../routes/common/adminLogs');
const { getUserCreateMessage } = require('../config/adminLogs.config');
const { prepareEmailOptions, prepareOnPremiseVerificationLink, prepareOffPremiseVerificationLink, prepareInvLogoLink, prepareEbrLogoLink } = require('./users');

var emailTemplate = 'email_verification_link.html';

/**
 * Get User List
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
const getUserList = async (req, res, next) => {
    try {

      const license = await models.License.findOne({
        attributes: [
          'id',
          'max_number_of_clients',
          'number_of_days',
          'license_submission_date',
          'mfa_feature',
          'darkening_feature',
          'licensee',
          'licensee_email',
          'max_number_of_services',
          'violated',
          's2s_microseg_feature'
        ],
        order: [
          [
            'id',
            'DESC'
          ]
        ],
        limit: 1
      });
  
      if (license && license.violated) {
        res.setHeader('X-License-Days', 'Invalid');
        return res.json({
          license,
          valid: false,
          processing: true
        });
      } else if (license && (license.number_of_days || license.number_of_days === 0)) {
        const validUntil = moment(license.license_submission_date).add(license.number_of_days, 'days');
        const difference = moment(validUntil).diff(moment(), 'seconds');
        const xLixense = moment(validUntil).diff(moment(), 'days', true);
        res.setHeader('X-License-Days', xLixense);
        res.json({
          license,
          valid: difference >= 0,
          xLicenseDays: Math.floor(xLixense)
        });
      } else {
        if (license && !license.number_of_days) {
          res.setHeader('X-License-Days', 'Processing');
          return res.json({
            license,
            valid: false,
            processing: true
          });
        } else {
          res.setHeader('X-License-Days', '-1');
        }
        throw new httpErrors.NotFound('License not found');
      }

    } catch (error) {
      next(error);
    }
}


/*  TODO : Need to refactor this after 3.10 release */

/**
 * CREATE new settings.
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
const createNewSettings = async (req, res, next) => {
    const attachment = req.body.attachment;
    const data = attachment.data.split(';base64,').pop();
    const sigFileData = req.body.sigFileData.data.split(';base64,').pop();
    const buff = new Buffer(data, 'base64');
    const licenseData = buff.toString('base64');
    const licenseSigData = new Buffer(sigFileData, 'base64').toString('ascii');
  
    models.License.findOne({
        attributes: ['sign_file'],
        where: {
          sign_file: sigFileData
        }
      })
      .then(license => {
        if (!license) {
          //write license txt file in temp directory
          return new Promise((resolve, reject) => {
            fs.writeFile(
              `${signFilePath}license.txt`,
              data, {
                encoding: 'base64'
              },
              error => {
                if (error) {
                  reject(error);
                } else {
                  resolve();
                }
              }
            );
          });
        } else {
          console.log('License is already being used.');
          const error = new Error('License is already being used.');
          error.name = 'AlreadyUsed';
          throw error;
        }
      })
      .then(() => {
        //write sign file in temp directory
        return new Promise((resolve, reject) => {
          fs.writeFile(
            `${signFilePath}license.sign`,
            sigFileData, {
              encoding: 'base64'
            },
            error => {
              if (error) {
                reject(error);
              } else {
                resolve();
              }
            }
          );
        });
      })
      .then(() => {
        //verify sig file with encrypyted license
        return helper.verifySignatureFile(
          `${signFilePath}license.txt`,
          `${signFilePath}license.sign`,
          signPubKey
        );
      })
      .then(() => {
        return models.License.findOne({
          order: [
            ['id', 'DESC']
          ],
          limit: 1
        });
      })
      .then(async license => {
        if (license) {
          const license = await models.License.findAll();
          
          // only create user on first license (no license data in table)
          if (!(license && license.length > 0)) {
            let err = await createAdmin(license.licensee_email);

            if (err !== null) {
              throw new httpErrors.BadRequest(err);
            }
          }

          return models.License.create({
            max_number_of_clients: license.max_number_of_clients,
            max_number_of_services: license.max_number_of_services,
            license_signature: license.license_signature,
            mfa_feature: license.mfa_feature,
            darkening_feature: license.darkening_feature,
            licensee: license.licensee,
            licensee_email: license.licensee_email,
            encrypted_license: licenseData,
            sign_file: sigFileData,
            license_submission_date: moment().endOf('day'),
            s2s_microseg_feature: license.s2s_microseg_feature
          });
        } else {
          return models.License.create({
            encrypted_license: licenseData,
            license_submission_date: moment().endOf('day'),
            sign_file: sigFileData
          });
        }
      })
      .then(function(data){
        return Promise.all([
          data,
          publishLicense(actionType.ADDED, {
            data: {
              id: data.id,
              encrypted_license: data.encrypted_license,
              sign_file: data.sign_file,
              license_submission_date: moment().endOf('day').format("YYYY-MM-DD HH:mm:ss")
            }
          })
        ]);
      })
      .then(function([data, ]) {
        fsExtra.removeSync(`${signFilePath}license.txt`);
        fsExtra.removeSync(`${signFilePath}license.sign`);
        res.json({
          data: data,
          success: true,
          message: language.get['ADD_SUCCESS']
        });
      })
      .catch(error => {
        if (error.name === 'AlreadyUsed') {
          res.status(201).send({
            success: false,
            message: 'License is already being used.'
          });
        }
        else {
          fsExtra.removeSync(`${signFilePath}license.txt`);
          res.status(201).send({
            success: false,
            message: 'License Verification failed.'
          });
        }
      });
}

const createAdmin = async (email) => {
  let error = null;
  try {
    let emailLinkExpiryInMinutes = 60;

    const duplicateEmail = await models.User.findOne({
      where: {
        email: email
      }
    });

    if (duplicateEmail) {
      return error = 'User with given email already exists';
    }

    // create admin user
    let user = {
      firstName: "Admin",
      lastName: "User",
      email: email,
      password: "Invisily123!"
    }

    const key =
    Math.random()
      .toString(36)
      .substring(2) +
    Math.random()
      .toString(36)
      .substring(2);
    user.email_verification_link_expiry = moment().add(
      emailLinkExpiryInMinutes,
      'minutes'
    );
    user.email_verification_link = key;
    const hash = await argon2.hash(user.password);
    user.password = hash;

    const data = await models.User.create(user);
    data.setRoles(2);

    // assign admin template to above created user 
    let query1 = `
      SELECT DISTINCT t.id as temp_id, t.name, t.description, tp.id as temp_permission_id
      FROM Templates t
      INNER JOIN TemplatePermissions tp ON tp.temp_id = t.id
      INNER JOIN Permissions p ON p.id = tp.permission_id
      INNER JOIN Privilege pv ON pv.id = p.privilege_id
      WHERE t.id = 2
    `;

    let templatePermissions = (await models.sequelize.query(query1))[0];

    if (templatePermissions.length !== 0) {
      for (let j = 0; j < templatePermissions.length; j++) {
        const l = await models.UserTemplates.create(
          {
            temp_permission_id: templatePermissions[j].temp_permission_id,
            user_id: data.id
          }
        );
      }
    }

    let query2 = `
      SELECT id, name
      FROM Templates
      WHERE id BETWEEN 2 AND 6
      ORDER BY id
    `;

    // update created_by of all templates except infra admin
    let templates = (await models.sequelize.query(query2))[0];

    if (templates.length !== 0) {
      for (let i = 0; i < templates.length; i++) {
        await models.UserTemplatePivot.create(
          {
            temp_id: templates[i].id,
            user_id: data.id
          }
        );
      }
    }
    
    helper.sendVerificationEmail(
      prepareEmailOptions(
        user.email,
        language.get['USER_EMAIL_VERIFICATION'],
        emailTemplate,
        prepareOnPremiseVerificationLink(key),
        prepareOffPremiseVerificationLink(key),
        user.firstName,
        emailLinkExpiryInMinutes,
        prepareInvLogoLink(),
        prepareEbrLogoLink()
      )
    );

    return error;
  } catch (err) {
    error = err.message
    return error;
  }
}

module.exports = {
    getUserList,
    createNewSettings,
    createAdmin
};
