var models = require('../models');
var {intToIp, getDateRange, dateToEpoch, convertToArray} = require('../utils/common');
var utils = require('../utils/common');
var moment = require('moment');
const {parseAsync} = require('json2csv')
const path = require('path');
var fs = require('fs');

/**
 * Get the summary of services
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
module.exports.servicesSummary = async (req, res, next) => {
    try {
        const {
            limit,
            offset,
            sort = 'service_name',
            dir = 'asc',
            serviceId,
            clientId,
        } = req.query;

        let startDate = req.query.startDate;
        let endDate = req.query.endDate || moment();

        const options = {
            where: {},
            limit,
            offset,
            order: [],
            mapper: {
                'bytes_sent': 'bytes_sent',
                'bytes_received': 'bytes_received',
                'total_users': 'total_users',
                'Service.Name': 'service_name',
                'Service.Gateways.Name': 'gateway_name',
                'service_physical_ip': {
                    mapTo: 'service_physical_ip',
                    fn: intToIp,
                },
                'Service.ServiceGroupCount': 'ServiceGroupCount',
                'Service.AccessPoliciesCount': 'AccessPoliciesCount',
                'Service.RoutingPoliciesCount': 'RoutingPoliciesCount'
            }
        };
        const allowedSortValues = ['bytes_sent', 'bytes_received', 'total_users', 'service_name', 'gateway_name']

        if (allowedSortValues.includes(sort)) {
            if (sort === options.mapper["Service.Gateways.Name"])
                options.order.push([{model: models.Service}, {model: models.Gateway}, 'Name', dir]);
            else if (sort === options.mapper["Service.Name"])
                options.order.push([models.sequelize.literal("`Service.Name`"), dir]);
            else
                options.order.push([models.sequelize.literal(sort), dir]);
        }

        if (startDate && endDate) {

            startDate = dateToEpoch(startDate);
            endDate = dateToEpoch(endDate);

            options.where = {
                ...options.where,
                log_start_time: {
                    [Op.lte]: endDate,
                },
                log_end_time: {
                    [Op.gte]: startDate
                }
            }
        }

        if (serviceId) {
            options.where = {
                ...options.where,
                service_id: {
                    [Op.in]: convertToArray(serviceId)
                }
            }
        }

        if (clientId) {
            options.where = {
                ...options.where,
                agent_id: {
                    [Op.in]: convertToArray(clientId)
                }
            }
        }

        const result = await models.ServiceAccessLogs.paginate({
            ...options,
            attributes: [
                'service_id',
                [
                    models.sequelize.fn('max', models.sequelize.col('ServiceAccessLogs.agent_id')), 'id'
                ],
                [
                    models.sequelize.fn('max', models.sequelize.col('service_physical_ip')), 'service_physical_ip'
                ],
                [
                    models.sequelize.fn(
                        'count', models.sequelize.fn('distinct', models.sequelize.col('ServiceAccessLogs.agent_id'))
                    ), 'total_users'
                ],
                [
                    models.sequelize.fn(
                        'sum', models.sequelize.literal('client_tx_bytes / 1048576')
                    ), 'bytes_sent'
                ],
                [
                    models.sequelize.fn(
                        'sum', models.sequelize.literal('service_tx_bytes / 1048576')
                    ), 'bytes_received'
                ]
            ],
            include: [{
                model: models.Service,
                attributes: [
                    'Name',
                    'id',
                    [models.sequelize.literal(
                        '(SELECT COUNT(*) FROM GroupServices WHERE GroupServices.serviceId = Service.id)'), "ServiceGroupCount"
                    ],
                    [models.sequelize.literal(
                        `(
                            SELECT (
                              SELECT COUNT(DISTINCT serviceId)
                              FROM (
                                SELECT id AS serviceId
                                FROM ServicePolicies
                                WHERE ServicePolicies.policy_type_id = 1
                                AND ServicePolicies.service_id = Service.id
                                UNION
                                SELECT serviceId
                                FROM GroupServices
                                WHERE GroupServices.serviceId = Service.id
                                AND groupId IN (
                                  SELECT service_group_id
                                  FROM ServiceGroupPolicies
                                  WHERE ServiceGroupPolicies.policy_type_id = 1
                                )
                              ) AS T1
                            ) + (
                              SELECT COUNT(*)
                              FROM Entitlements
                              WHERE all_services = 1
                            )
                        )`), 'AccessPoliciesCount'
                    ],
                    [models.sequelize.literal(
                        `(
                            SELECT (
                              SELECT COUNT(DISTINCT serviceId)
                              FROM (
                                SELECT id AS serviceId
                                FROM ServicePolicies
                                WHERE ServicePolicies.policy_type_id = 3
                                AND ServicePolicies.service_id = Service.id
                                UNION
                                SELECT serviceId
                                FROM GroupServices
                                WHERE GroupServices.serviceId = Service.id
                                AND groupId IN (
                                  SELECT service_group_id
                                  FROM ServiceGroupPolicies
                                  WHERE ServiceGroupPolicies.policy_type_id = 3
                                )
                              ) AS T1
                            ) + (
                              SELECT COUNT(*)
                              FROM RoutingPolicies
                              WHERE all_services = 1
                            )
                        )`), "RoutingPoliciesCount"
                    ]
                ],
                required: true
            }, {
                model: models.Client,
                attributes: [],
                required: true,
            }],
            group: ['ServiceAccessLogs.service_id'],
            raw: true
        });
        if (result.data) {
            result.data = result.data.map(d => ({
                ...d,
                service_name: d.service_name || 'Unknown',
                gateway_name: d.gateway_name || 'Unknown',
                bytes_sent: +(Number(d.bytes_sent).toFixed(3)),
                bytes_received: +(Number(d.bytes_received).toFixed(3))
            }))
        }
        res.json(result);
    } catch (error) {
        next(error);
    }
}

/**
 * Get the summary of services
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
module.exports.devicesReports = async (req, res, next) => {
    try {
        const {
            limit,
            offset,
            sort = 'device_name',
            dir = 'asc',
        } = req.query;

        const options = {
            where: {user_id: null},
            limit,
            offset,
            order: []
        };
        const allowedSortValues = ['device_name', 'os_version', 'userName', 'createdAt']

        if (allowedSortValues.includes(sort)) {
            options.order.push([models.sequelize.literal(sort), dir]);
        }
        const result = await models.DeviceInventories.paginate(options);
        res.json(result);
    } catch (error) {
        next(error);
    }
}


/**
 * Get the summary of services
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
module.exports.servicesReports = async (req, res, next) => {
    try {
        const {
            limit,
            offset,
            sort = 'Name',
            dir = 'asc',
        } = req.query;

        let startDate = req.query.startDate;
        let endDate = req.query.endDate || moment();

        const options = {
            attributes: [
                "id",
                "Name",
                [models.sequelize.literal(
                    '(SELECT COUNT(*) FROM GroupServices WHERE GroupServices.serviceId = Service.id)'), "ServiceGroupCount"
                ],
                [models.sequelize.literal(
                    `(
                        SELECT (
                          SELECT COUNT(DISTINCT serviceId)
                          FROM (
                            SELECT id AS serviceId
                            FROM ServicePolicies
                            WHERE ServicePolicies.policy_type_id = 1
                            AND ServicePolicies.service_id = Service.id
                            UNION
                            SELECT serviceId
                            FROM GroupServices
                            WHERE GroupServices.serviceId = Service.id
                            AND groupId IN (
                              SELECT service_group_id
                              FROM ServiceGroupPolicies
                              WHERE ServiceGroupPolicies.policy_type_id = 1
                            )
                          ) AS T1
                        ) + (
                          SELECT COUNT(*)
                          FROM Entitlements
                          WHERE all_services = 1
                        )
                    )`), 'AccessPoliciesCount'
                ],
                [models.sequelize.literal(
                    `(
                        SELECT (
                          SELECT COUNT(DISTINCT serviceId)
                          FROM (
                            SELECT id AS serviceId
                            FROM ServicePolicies
                            WHERE ServicePolicies.policy_type_id = 3
                            AND ServicePolicies.service_id = Service.id
                            UNION
                            SELECT serviceId
                            FROM GroupServices
                            WHERE GroupServices.serviceId = Service.id
                            AND groupId IN (
                              SELECT service_group_id
                              FROM ServiceGroupPolicies
                              WHERE ServiceGroupPolicies.policy_type_id = 3
                            )
                          ) AS T1
                        ) + (
                          SELECT COUNT(*)
                          FROM RoutingPolicies
                          WHERE all_services = 1
                        )
                    )`), "RoutingPoliciesCount"
                ]
            ],
            limit,
            offset,
            order: []
        };
        const allowedSortValues = ['Name', 'ServiceGroupCount', 'AccessPoliciesCount', 'RoutingPoliciesCount']

        if (allowedSortValues.includes(sort)) {
            options.order.push([models.sequelize.literal(sort), dir]);
        }


        // if(startDate && endDate) {

        //   startDate = dateToEpoch(startDate);
        //   endDate = dateToEpoch(endDate);

        //   options.where = {
        //     ...options.where,
        //     log_start_time: {
        //       [Op.lte]: endDate,
        //     },
        //     log_end_time: {
        //       [Op.gte]: startDate
        //     }
        //   }
        // }

        const result = await models.Service.paginate(options);
        // attributes:[
        //   "id",
        //   "Name",
        //   [models.sequelize.fn("COUNT","Group.id"), "ServiceGroupCount"],
        //   [models.sequelize.fn("COUNT","Entitlements.id"), "EntitlementsCount"],
        //   [models.sequelize.fn("COUNT","RoutingPolicies.id"), "RoutingPoliciesCount"]
        // ],
        // include:[{
        //   model: models.Group,
        //   attributes: [],
        // },
        // {
        //   model: models.Entitlements,
        //   attributes: [],
        // },
        // {
        //   model: models.RoutingPolicies,
        //   attributes: [],
        // }],
        // group: ['Service.id'],
        // raw: true
        // });
        res.json(result);
    } catch (error) {
        next(error);
    }
}


/**
 * Get the summary of services
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
module.exports.UnUsedpolicyReports = async (req, res, next) => {
    try {
        const {
            limit,
            offset,
            sort = 'Name',
            dir = 'asc',
        } = req.query;

        const options = {
            where: {
                id: {
                    [Op.notIn]: models.sequelize.literal(
                        `(
                            SELECT DISTINCT s.id
                            FROM Services s
                            LEFT JOIN ServicePolicies sp ON s.id = sp.service_id
                            LEFT JOIN GroupServices gs ON gs.serviceId = s.id
                            LEFT JOIN ServiceGroupPolicies sgp ON sgp.service_group_id = gs.groupId
                            RIGHT JOIN Entitlements e ON (e.id = sp.policy_id OR e.id = sgp.policy_id)
                            WHERE (sp.policy_type_id = 1 OR sgp.policy_type_id = 1)
                            AND (e.is_enabled = 1)

                            UNION

                            SELECT s.id
                            FROM Services s
                            WHERE (
                                SELECT DISTINCT e.all_services
                                FROM Entitlements e
                                WHERE (e.all_services = 1 AND e.is_enabled = 1)
                            )
                        )`
                    )
                }
            },
            limit,
            offset,
            order: []
        };
        const allowedSortValues = ['Name', 'os_version', 'userName', 'createdAt']

        if (allowedSortValues.includes(sort)) {
            options.order.push([models.sequelize.literal(sort), dir]);
        }

        const result = await models.Service.paginate(options);
        res.json(result);
    } catch (error) {
        next(error);
    }
}


/**
 * Get the summary of services
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
module.exports.WithoutRoutingpolicyReports = async (req, res, next) => {
    try {

        const policies = await models.sequelize.query("select id from RoutingPolicies where all_services = 1");
        if (policies[0].length != 0) {
            return res.json({});
        }


        const {
            limit,
            offset,
            sort = 'Name',
            dir = 'asc',
        } = req.query;

        const options = {
            where: {
                id: {
                    [Op.notIn]: models.sequelize.literal(`(SELECT service_id FROM ServicePolicies WHERE policy_type_id = 3 UNION
                        SELECT serviceId from GroupServices where groupId in (
                        SELECT service_group_id from ServiceGroupPolicies
                        where policy_type_id = 3
                    ) )`)
                }
            },
            limit,
            offset,
            order: []
        };
        const allowedSortValues = ['Name', 'createdAt', 'createdById', 'notes']

        if (allowedSortValues.includes(sort)) {
            options.order.push([models.sequelize.literal(sort), dir]);
        }

        const result = await models.Service.paginate(options);
        for (const element of result.data) {
            const user = await models.User.findOne({
                where: {
                    id: element.dataValues.createdById
                },
                attributes: ['firstName']
            });

            const firstName = user ? user.dataValues.firstName : null;
            element.dataValues['firstName'] = firstName;
        }

        res.json(result);
    } catch (error) {
        next(error);
    }
}


module.exports.UnassignedServicespolicyReports = async (req, res, next) => {
    try {
        const {
            limit,
            offset,
            sort = 'Name',
            dir = 'asc',
        } = req.query;
        let result = [];
        const check = await models.RoutingPolicies.findOne({
            where: {
                all_services: '1'
            }
        });

        if (check) {
            const options = {
                where: {
                    id: 0
                },
                limit,
                offset,
                order: []
            };
            const allowedSortValues = ['Name', 'createdAt', 'createdById', 'notes']
            if (allowedSortValues.includes(sort)) {
                options.order.push([models.sequelize.literal(sort), dir]);
            }
            result = await models.Service.paginate(options);
        } else {
            let SGroupData = [];
            let serviceIds = [];
            let SGroupPolicies = await models.ServiceGroupPolicies.findAll({
                attributes: ['service_group_id'],
                raw: true
            });
            await SGroupPolicies.map(element => {
                return SGroupData.push(element.service_group_id);
            });
            let servicedata = await models.ServicePolicies.findAll({
                attributes: ['service_id'],
                raw: true
            });
            servicedata.map((e) => {
                serviceIds.push(e.service_id);
            });
            let serviceGroupdata = await models.GroupService.findAll({
                where: {
                    groupId: {
                        [Op.in]: SGroupData
                    }
                },
                attributes: ['serviceId'],
                raw: true
            });
            serviceGroupdata.map((e) => {
                serviceIds.push(e.serviceId);
            });
            let distIds = [...new Set(serviceIds)];
            const options = {
                where: {
                    [Op.and]: [
                        {id: {[Op.notIn]: models.sequelize.literal('(SELECT service_id FROM ServicePolicies)')}},
                        {id: {[Op.notIn]: distIds}}
                    ]
                },
                limit,
                offset,
                order: []
            };
            const allowedSortValues = ['Name', 'createdAt', 'createdById', 'notes']

            if (allowedSortValues.includes(sort)) {
                options.order.push([models.sequelize.literal(sort), dir]);
            }
            result = await models.Service.paginate(options);
        }

        for (const element of result.data) {
            const user = await models.User.findOne({
                where: {
                    id: element.dataValues.createdById
                },
                attributes: ['firstName']
            });

            const firstName = user ? user.dataValues.firstName : null;
            element.dataValues['firstName'] = firstName;
        }

        res.json(result);
    } catch (error) {
        next(error);
    }
}

/**
 * Get the summary of services
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
module.exports.InfrastructureControllersReports = async (req, res, next) => {
    try {
        const {
            limit,
            offset,
            sort = 'Name',
            dir = 'asc',
        } = req.query;

        const options = {
            attributes: [
                "Name",
                [models.sequelize.literal(
                    '(SELECT memTotal FROM ControllerHealths WHERE ControllerHealths.controllerId = Controller.id)'), "ControllerHealth"
                ]
            ],
            limit,
            offset,
            order: []
        };
        const allowedSortValues = ['Name', 'ControllerHealth', 'createdAt']

        if (allowedSortValues.includes(sort)) {
            options.order.push([models.sequelize.literal(sort), dir]);
        }

        const result = await models.Controller.paginate(options);
        const controllersActive = await utils.getOnlineCount(utils.serverStatusKeys.CONTROLLERS);
        res.json({
            result,
            controllersActive
        });
    } catch (error) {
        next(error);
    }
}


/**
 * Get the summary of services
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
module.exports.InfrastructureGatewaysReports = async (req, res, next) => {
    try {
        const {
            limit,
            offset,
            sort = 'Name',
            dir = 'asc',
        } = req.query;

        const options = {
            attributes: [
                "Name",
                [models.sequelize.literal(
                    '(SELECT memTotal FROM GatewayHealths WHERE GatewayHealths.gatewayId = Gateway.id)'), "GatewayHealth"
                ]
            ],
            limit,
            offset,
            order: []
        };
        const allowedSortValues = ['Name', 'GatewayHealth', 'createdAt']

        if (allowedSortValues.includes(sort)) {
            options.order.push([models.sequelize.literal(sort), dir]);
        }
        const directories_count = await models.LocalUsers.count({
            where: {
                identity_provider: 'Active Directory'
            }
        });
        const last_sync_time = await models.Settings.findOne({
            attributes: ['last_ldap_sync_end_time'],
            order: [['id', 'DESC']],
            limit: 1
        });
        const result = await models.Gateway.paginate(options);
        const gatewaysActive = await utils.getOnlineCount(utils.serverStatusKeys.GATEWAYS);
        res.json({
            last_sync_time,
            directories_count,
            result,
            gatewaysActive
        });
    } catch (error) {
        next(error);
    }
}


/**
 * Get the summary of users who accessed services
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
module.exports.userServicesSummary = async (req, res, next) => {
    try {
        const {
            limit,
            offset,
            sort = 'client_name',
            dir = 'asc',
            clientId,
            serviceId,
        } = req.query;


        let startDate = req.query.startDate;
        let endDate = req.query.endDate || moment();

        const options = {
            where: {},
            limit,
            offset,
            order: [],
            mapper: {
                'bytes_sent': 'bytes_sent',
                'bytes_received': 'bytes_received',
                'total_services': 'total_services',
                'Client.Name': 'client_name',
                'Client.device_os': 'device_os',
                'Client.device_hardware_id': 'device_hardware_id',
                'client_physical_ip': {
                    mapTo: 'client_physical_ip',
                    fn: intToIp,
                }
            }
        };

        const allowedSortValues = ['bytes_sent', 'bytes_received', 'total_services', 'client_name'];

        if (allowedSortValues.includes(sort)) {
            if (sort === options.mapper["Client.Name"])
                options.order.push([models.Client, 'Name', dir]);
            else
                options.order.push([models.sequelize.literal(sort), dir]);
        }


        if (startDate && endDate) {

            startDate = dateToEpoch(startDate);
            endDate = dateToEpoch(endDate);

            options.where = {
                ...options.where,
                log_start_time: {
                    [Op.lte]: endDate,
                },
                log_end_time: {
                    [Op.gte]: startDate
                }
            }
        }

        if (clientId) {
            options.where = {
                ...options.where,
                agent_id: {
                    [Op.in]: convertToArray(clientId)
                }
            }
        }

        if (serviceId) {
            options.where = {
                ...options.where,
                service_id: {
                    [Op.in]: convertToArray(serviceId)
                }
            }
        }

        const result = await models.ServiceAccessLogs.paginate({
            ...options,
            attributes: [
                'agent_id',
                [
                    models.sequelize.fn('max', models.sequelize.col('ServiceAccessLogs.service_id')), 'id'
                ],
                [
                    models.sequelize.fn('max', models.sequelize.col('client_physical_ip')), 'client_physical_ip'
                ],
                [
                    models.sequelize.fn(
                        'count', models.sequelize.fn('distinct', models.sequelize.col('ServiceAccessLogs.service_id'))
                    ), 'total_services'
                ],
                [
                    models.sequelize.fn(
                        'sum', models.sequelize.literal('client_tx_bytes / 1048576')
                    ), 'bytes_sent'
                ],
                [
                    models.sequelize.fn(
                        'sum', models.sequelize.literal('service_tx_bytes / 1048576')
                    ), 'bytes_received'
                ]
            ],
            include: [{
                model: models.Client,
                attributes: ['Name', 'ipv4_address', 'device_os', 'device_hardware_id'],
                required: true
            }],
            group: ['ServiceAccessLogs.agent_id'],
            raw: true
        });
        if (result.data) {
            result.data = result.data.map(d => ({
                ...d,
                client_name: d.client_name || 'Unknown',
                device_os: d.device_os || 'Unknown',
                bytes_sent: +(Number(d.bytes_sent).toFixed(3)),
                bytes_received: +(Number(d.bytes_received).toFixed(3))
            }))
        }
        res.json(result);
    } catch (error) {
        next(error);
    }
}


/**
 * Get the  graph summary of users who accessed services
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
module.exports.servicesSummaryGraph = async (req, res, next) => {
    try {
        const {
            serviceId,
            clientId,
        } = req.query;

        let startDate = req.query.startDate;
        let endDate = req.query.endDate || moment();

        const options = {
            where: {},
            order: []
        };

        if (startDate && endDate) {

            startDate = dateToEpoch(startDate);
            endDate = dateToEpoch(endDate);

            options.where = {
                ...options.where,
                log_start_time: {
                    [Op.lte]: endDate,
                },
                log_end_time: {
                    [Op.gte]: startDate
                }
            }
        }

        if (serviceId) {
            options.where = {
                ...options.where,
                service_id: {
                    [Op.in]: convertToArray(serviceId)
                }
            }
        }

        if (clientId) {
            options.where = {
                ...options.where,
                agent_id: {
                    [Op.in]: convertToArray(clientId)
                }
            }
        }

        const result = await models.ServiceAccessLogs.findAll({
            ...options,
            attributes: [
                'service_id',
                [
                    models.sequelize.fn('max', models.sequelize.col('ServiceAccessLogs.agent_id')), 'id'
                ],
                [
                    models.sequelize.fn('max', models.sequelize.col('service_physical_ip')), 'service_physical_ip'
                ],
                [
                    models.sequelize.fn(
                        'count', models.sequelize.fn('distinct', models.sequelize.col('ServiceAccessLogs.agent_id'))
                    ), 'total_users'
                ],
                [
                    models.sequelize.fn(
                        'sum', models.sequelize.literal('(client_tx_bytes + service_tx_bytes) / 1048576')
                    ), 'total_volume'
                ]
            ],
            include: [{
                model: models.Service,
                required: true,
                attributes: ['Name', 'service_id'],
            }],
            group: ['service_id'],
            raw: true
        });
        res.json(result);
    } catch (error) {
        next(error);
    }
}


/**
 * Get the summary of users who accessed services
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
module.exports.userServicesSummaryGraph = async (req, res, next) => {
    try {
        const {
            clientId,
            serviceId,
        } = req.query;


        const options = {
            where: {},
            order: [],
        };


        let date = req.query.startDate || moment();

        let {startDate, endDate} = getDateRange({
            rangeType: 'D',
            date: date,
            epoch: true
        });

        options.where = {
            ...options.where,
            log_start_time: {
                $between: [startDate, endDate]
            },
            log_end_time: {
                $between: [startDate, endDate]
            },
        }

        if (clientId) {
            options.where = {
                ...options.where,
                agent_id: {
                    [Op.in]: convertToArray(clientId)
                }
            }
        }

        if (serviceId) {
            options.where = {
                ...options.where,
                service_id: {
                    [Op.in]: convertToArray(serviceId)
                }
            }
        }

        const result = await models.ServiceAccessLogs.findAll({
            ...options,
            attributes: [
                [
                    models.sequelize.fn('max', models.sequelize.col('ServiceAccessLogs.service_id')), 'id'
                ],
                [
                    models.sequelize.fn(
                        'sum', models.sequelize.literal('(client_tx_bytes + service_tx_bytes) / 1048576')
                    ), 'total_volume'
                ],
                [
                    models.sequelize.fn(
                        'HOUR', models.sequelize.fn(
                            'FROM_UNIXTIME', models.sequelize.col('log_start_time')
                        )
                    ), 'hours'
                ]
            ],
            include: [{
                model: models.Service,
                attributes: [],
                required: true
            }, {
                model: models.Client,
                attributes: [],
                required: true,
            }],
            group: ['hours'],
            raw: true
        });
        res.json(result);
    } catch (error) {
        next(error);
    }
}


/**
 * Get the summary of services
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
module.exports.getServices = async (req, res, next) => {
    try {
        const result = await models.Service.findAll({
            include: [{
                model: models.ServiceAccessLogs,
                attributes: [],
                required: true
            }],
            attributes: [
                [models.sequelize.fn('DISTINCT', models.sequelize.literal('Service.service_id')), 'service_id'],
                'Name',
                [models.sequelize.literal(
                    '(SELECT COUNT(*) FROM GroupServices WHERE GroupServices.serviceId = Service.id)'), "ServiceGroupCount"
                ],
                [models.sequelize.literal(
                    `(
                        SELECT (
                          SELECT COUNT(DISTINCT serviceId)
                          FROM (
                            SELECT id AS serviceId
                            FROM ServicePolicies
                            WHERE ServicePolicies.policy_type_id = 1
                            AND ServicePolicies.service_id = Service.id
                            UNION
                            SELECT serviceId
                            FROM GroupServices
                            WHERE GroupServices.serviceId = Service.id
                            AND groupId IN (
                              SELECT service_group_id
                              FROM ServiceGroupPolicies
                              WHERE ServiceGroupPolicies.policy_type_id = 1
                            )
                          ) AS T1
                        ) + (
                          SELECT COUNT(*)
                          FROM Entitlements
                          WHERE all_services = 1
                        )
                    )`), 'AccessPoliciesCount'
                ],
                [models.sequelize.literal(
                    `(
                        SELECT (
                          SELECT COUNT(DISTINCT serviceId)
                          FROM (
                            SELECT id AS serviceId
                            FROM ServicePolicies
                            WHERE ServicePolicies.policy_type_id = 3
                            AND ServicePolicies.service_id = Service.id
                            UNION
                            SELECT serviceId
                            FROM GroupServices
                            WHERE GroupServices.serviceId = Service.id
                            AND groupId IN (
                              SELECT service_group_id
                              FROM ServiceGroupPolicies
                              WHERE ServiceGroupPolicies.policy_type_id = 3
                            )
                          ) AS T1
                        ) + (
                          SELECT COUNT(*)
                          FROM RoutingPolicies
                          WHERE all_services = 1
                        )
                    )`), "RoutingPoliciesCount"
                ]
            ],
            raw: true
        });
        res.json(result);
    } catch (error) {
        next(error);
    }
}

/**
 * Get the summary of services
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
module.exports.getClients = async (req, res, next) => {
    try {
        const result = await models.Client.findAll({
            include: [{
                model: models.ServiceAccessLogs,
                attributes: [],
                required: true
            }],
            attributes: [
                [models.sequelize.fn('DISTINCT', models.sequelize.literal('Client.agent_id')), 'agent_id'],
                'Name',
                'device_os'
            ],
            raw: true
        });
        res.json(result);
    } catch (error) {
        next(error);
    }
}

module.exports.exportCSVDevices = async (req, res, next) => {
    try {
        const Path = path.join(__dirname, '../storage/devices-reports.csv');
        const options = {
            where: {user_id: null},
            order: [],
            raw: true
        };

        const result = await models.DeviceInventories.findAll(options);
        const fields = [
            {
                label: 'ID',
                value: 'id'
            },
            {
                label: 'DEVICE NAME',
                value: 'device_name'
            },
            {
                label: 'CREATION DATE',
                value: 'createdAt'
            }
        ];
        const opts = {fields, quote: ''};
        await parseAsync(result, opts)
            .then(async csv => {
                await fs.writeFile(Path, csv, function (err, data) {
                    if (err) {
                        throw err;
                    } else {
                        return res.download(Path);
                    }
                });
            })
            .catch(err => console.error(err));
    } catch (error) {
        res.status(500).send(error.message)
    }
}


module.exports.exportCSVServices = async (req, res, next) => {
    try {
        const Path = path.join(__dirname, '../storage/services-reports.csv');
        const options = {
            attributes: [
                "id",
                "Name",
                [models.sequelize.literal(
                    '(SELECT COUNT(*) FROM GroupServices WHERE GroupServices.serviceId = Service.id)'), "ServiceGroupCount"
                ],
                [models.sequelize.literal(
                    `(
                        SELECT (
                          SELECT COUNT(DISTINCT serviceId)
                          FROM (
                            SELECT id AS serviceId
                            FROM ServicePolicies
                            WHERE ServicePolicies.policy_type_id = 1
                            AND ServicePolicies.service_id = Service.id
                            UNION
                            SELECT serviceId
                            FROM GroupServices
                            WHERE GroupServices.serviceId = Service.id
                            AND groupId IN (
                              SELECT service_group_id
                              FROM ServiceGroupPolicies
                              WHERE ServiceGroupPolicies.policy_type_id = 1
                            )
                          ) AS T1
                        ) + (
                          SELECT COUNT(*)
                          FROM Entitlements
                          WHERE all_services = 1
                        )
                    )`), 'AccessPoliciesCount'
                ],
                [models.sequelize.literal(
                    `(
                        SELECT (
                          SELECT COUNT(DISTINCT serviceId)
                          FROM (
                            SELECT id AS serviceId
                            FROM ServicePolicies
                            WHERE ServicePolicies.policy_type_id = 3
                            AND ServicePolicies.service_id = Service.id
                            UNION
                            SELECT serviceId
                            FROM GroupServices
                            WHERE GroupServices.serviceId = Service.id
                            AND groupId IN (
                              SELECT service_group_id
                              FROM ServiceGroupPolicies
                              WHERE ServiceGroupPolicies.policy_type_id = 3
                            )
                          ) AS T1
                        ) + (
                          SELECT COUNT(*)
                          FROM RoutingPolicies
                          WHERE all_services = 1
                        )
                    )`), "RoutingPoliciesCount"
                ]
            ],
            order: [],
            raw: true
        };

        const result = await models.Service.findAll(options);
        const fields = [
            {
                label: 'ID',
                value: 'id'
            },
            {
                label: 'SERVICE NAME',
                value: 'Name'
            },
            {
                label: 'SERVICE GROUP',
                value: 'ServiceGroupCount'
            },
            {
                label: 'ACCESS POLICIES',
                value: 'AccessPoliciesCount'
            },
            {
                label: 'ROUTING POLICIES',
                value: 'RoutingPoliciesCount'
            }
        ];
        const opts = {fields, quote: ''};
        await parseAsync(result, opts)
            .then(async csv => {
                await fs.writeFile(Path, csv, function (err, data) {
                    if (err) {
                        throw err;
                    } else {
                        return res.download(Path);
                    }
                });
            })
            .catch(err => console.error(err));
    } catch (error) {
        res.status(500).send(error.message)
    }
}


module.exports.InfrastructureExportCSV = async (req, res, next) => {
    try {
        let Path;
        let fields = [];
        let result;
        const {
            value
        } = req.query;

        if (value == '1') {
            Path = path.join(__dirname, '../storage/infrastructure-controller-reports.csv');
            const options = {
                attributes: [
                    "Name",
                    [models.sequelize.literal(
                        '(SELECT memTotal FROM ControllerHealths WHERE ControllerHealths.controllerId = Controller.id)'), "ControllerHealth"
                    ]
                ],
                order: [],
                raw: true
            };

            const controllersActive = await utils.getOnlineCount(utils.serverStatusKeys.CONTROLLERS);
            result = await models.Controller.findAll(options).map((res) => {
                res.controllersActive = controllersActive;
                return res;
            });

            fields = [
                {
                    label: 'ID',
                    value: 'id'
                },
                {
                    label: 'CONTROLLER NAME',
                    value: 'Name'
                },
                {
                    label: 'CONTROLLER HEALTH TOTAL MEMORY',
                    value: 'ControllerHealth'
                },
                {
                    label: 'ACTIVE CONTROLLERS',
                    value: 'controllersActive'
                }
            ];

        } else {
            Path = path.join(__dirname, '../storage/infrastructure-gateways-reports.csv');
            const options = {
                attributes: [
                    "Name",
                    [models.sequelize.literal(
                        '(SELECT memTotal FROM GatewayHealths WHERE GatewayHealths.gatewayId = Gateway.id)'), "GatewayHealth"
                    ]
                ],
                order: [],
                raw: true
            };

            const gatewaysActive = await utils.getOnlineCount(utils.serverStatusKeys.GATEWAYS);
            result = await models.Gateway.findAll(options).map((res) => {
                res.gatewaysActive = gatewaysActive;
                return res;
            });

            fields = [
                {
                    label: 'ID',
                    value: 'id'
                },
                {
                    label: 'GATEWAY NAME',
                    value: 'Name'
                },
                {
                    label: 'GATEWAY HEALTH TOTAL MEMORY',
                    value: 'GatewayHealth'
                },
                {
                    label: 'GATEWAY ACTIVE',
                    value: 'gatewaysActive'
                }
            ];

        }

        const opts = {fields, quote: ''};
        await parseAsync(result, opts)
            .then(async csv => {
                await fs.writeFile(Path, csv, function (err, data) {
                    if (err) {
                        throw err;
                    } else {
                        return res.download(Path);
                    }
                });
            })
            .catch(err => console.error(err));
    } catch (error) {
        res.status(500).send(error.message)
    }
}

module.exports.PolicyExportCSV = async (req, res, next) => {
    try {
        let Path;
        let fields = [];
        let result;
        const {
            value
        } = req.query;

        if (value == '1') {
            Path = path.join(__dirname, '../storage/policy-unused-entitlements-reports.csv');
            const options = {
                where: {
                    id: {
                        [Op.notIn]: models.sequelize.literal('(SELECT service_id FROM ServicePolicies WHERE policy_type_id = 1)')
                    }
                },
                order: [],
                raw: true
            };
            result = await models.Service.findAll(options);
            fields = [
                {
                    label: 'ID',
                    value: 'id'
                },
                {
                    label: 'SERVICE NAME',
                    value: 'Name'
                },
                {
                    label: 'ASSIGNED TO',
                    value: 'ServiceGroupCount'
                },
                {
                    label: 'DATE ASSIGNED',
                    value: 'createdAt'
                }
            ];

        } else {
            Path = path.join(__dirname, '../storage/services-without-routing-policy-reports.csv');
            const options = {
                where: {
                    id: {
                        [Op.notIn]: models.sequelize.literal('(SELECT service_id FROM ServicePolicies WHERE policy_type_id = 3)')
                    }
                },
                order: [],
                raw: true
            };

            result = await models.Service.findAll(options);

            fields = [
                {
                    label: 'ID',
                    value: 'id'
                },
                {
                    label: 'SERVICE NAME',
                    value: 'Name'
                },
                {
                    label: 'DATE CREATION',
                    value: 'createdAt'
                },
                {
                    label: 'ACTION BY',
                    value: 'createdById'
                }
            ];

        }

        const opts = {fields, quote: ''};
        await parseAsync(result, opts)
            .then(async csv => {
                await fs.writeFile(Path, csv, function (err, data) {
                    if (err) {
                        throw err;
                    } else {
                        return res.download(Path);
                    }
                });
            })
            .catch(err => console.error(err));
    } catch (error) {
        res.status(500).send(error.message)
    }
}


/**
 * Get the summary of users
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */

module.exports.usersReports = async (req, res, next) => {
    try {
        const {
            limit,
            offset,
            sort = 'id',
            dir = 'asc',
            searchTerm = ''
        } = req.query;

        const options = {
            where: {
                is_login_successfull: '1',
                [Op.or]: [
                    { client_name: { [Op.like]: `%${searchTerm}%` } },
                    { location: { [Op.like]: `%${searchTerm}%` } },
                    { client_userName: { [Op.like]: `%${searchTerm}%` } },
                    { client_device_os: { [Op.like]: `%${searchTerm}%` } }
                ]
            },
            limit,
            offset,
            order: []
        };
        const allowedSortValues = ['client_name', 'SESSION_ID', 'client_userName', 'CONNECTION_ATTEMPT_TIME']

        if (allowedSortValues.includes(sort)) {
            options.order.push([models.sequelize.literal(sort), dir]);
        }

        const result = await models.Connection.paginate(options);
        res.json(result);
    } catch (error) {
        next(error);
    }
}

module.exports.usersSessionReports = async (req, res, next) => {
    try{
        const {
            limit,
            offset,
            sort = '',
            dir = '',
            userId,
            searchTerm = ''
        } = req.query;

        const user = await models.LocalUsers.findOne({
            attributes: ['userName'],
            where: {
                id: userId,
            },
        });

        if (!user) {
            return res.status(404).json({ error: 'User not found' });
        }
        
        const options = {
            where: {
                is_login_successfull: '1',
                client_userName: user.userName,
                [Op.or]: [
                    { client_name: { [Op.like]: `%${searchTerm}%` } },
                    { location: { [Op.like]: `%${searchTerm}%` } },
                    { client_userName: { [Op.like]: `%${searchTerm}%` } },
                    { client_device_os: { [Op.like]: `%${searchTerm}%` } },
                    { failure_cause: { [Op.like]: `%${searchTerm}%` } }
                ]
            },
            limit,
            offset,
            order: [],
        };

        const allowedSortValues = ['client_name', 'SESSION_ID', 'client_userName', 'CONNECTION_ATTEMPT_TIME', 'sort', 'id'];

        if (allowedSortValues.includes(sort)) {            
            options.order.push([models.sequelize.literal(sort), dir]);
        }

        const result = await models.Connection.paginate(options);
        res.json(result);

    } catch (error) {
        next(error);
    }
}

module.exports.usersAuthenticationFailedReports = async (req, res, next) => {
    try {
        const {
            limit,
            offset,
            sort = 'id',
            dir = 'asc',
            searchTerm = ''
        } = req.query;

        const options = {
            where: {
                is_login_successfull: '0',
                [Op.or]: [
                    { client_name: { [Op.like]: `%${searchTerm}%` } },
                    { location: { [Op.like]: `%${searchTerm}%` } },
                    { client_userName: { [Op.like]: `%${searchTerm}%` } },
                    { client_device_os: { [Op.like]: `%${searchTerm}%` } },
                    { failure_cause: { [Op.like]: `%${searchTerm}%` } } 
                ]
            },
            limit,
            offset,
            order: []
        };
        const allowedSortValues = ['client_name', 'SESSION_ID', 'client_userName', 'CONNECTION_ATTEMPT_TIME']

        if (allowedSortValues.includes(sort)) {
            options.order.push([models.sequelize.literal(sort), dir]);
        }
        const result = await models.Connection.paginate(options);
        res.json(result);
    } catch (error) {
        next(error);
    }
}
module.exports.usersSessionFailedReports = async (req, res, next) => {
    try {
        const {
            limit,
            offset,
            sort = '',
            dir = '',
            userId,
            searchTerm = ''
        } = req.query;

        const user = await models.LocalUsers.findOne({
            attributes: ['userName'],
            where: {
                id: userId,
            },
        });

        if (!user) {
            return res.status(404).json({ error: 'User not found' });
        }

        const options = {
            where: {
                is_login_successfull: '0',
                client_userName: user.userName,
                [Op.or]: [
                    { client_name: { [Op.like]: `%${searchTerm}%` } },
                    { location: { [Op.like]: `%${searchTerm}%` } },
                    { client_userName: { [Op.like]: `%${searchTerm}%` } },
                    { client_device_os: { [Op.like]: `%${searchTerm}%` } },
                    { failure_cause: { [Op.like]: `%${searchTerm}%` } } 
                ]
            },
            limit,
            offset,
            order: [],
        };
        const allowedSortValues = ['client_name', 'SESSION_ID', 'client_userName', 'CONNECTION_ATTEMPT_TIME', 'sort', 'id']

        if (allowedSortValues.includes(sort)) {
            options.order.push([models.sequelize.literal(sort), dir]);
        }
        
        const result = await models.Connection.paginate(options);
        res.json(result);
    } catch (error) {
        next(error);
    }

}

module.exports.onboardingClientsReport = async (req, res, next) => {
    try {
        const {
            limit,
            offset,
            sort = 'Name',
            dir = 'asc',
            type, // check report type (1 for onboarded, 0 for non onboarded)
            search
        } = req.query;

        const options = {
            attributes: ['id', 'Name', 'device_hardware_id', 'device_os', 'user'],
            where: {
                client_status: type === '1' ? 4 : { [Op.not]: 4 },
                [Op.or]: [
                    { Name: { [Op.like]: `%${search}%` } },
                    { device_hardware_id: { [Op.like]: `%${search}%` } },
                    { device_os: { [Op.like]: `%${search}%` } }
                ]
            },
            limit,
            offset,
            order: []
        };

        const allowedSortValues = ['Name', 'device_hardware_id', 'device_os', 'user']
        
        if (type === '1') {
            options.attributes.push('onboarded_at');
            allowedSortValues.push('onboarded_at');
        }
        
        if (allowedSortValues.includes(sort)) {
            options.order.push([models.sequelize.literal(sort), dir]);
        }
        const result = await models.Client.paginate(options);
        res.json(result);
    } catch (error) {
        next(error);
    }
}

module.exports.servicesAccessedByUsers = async (req, res, next) =>{
    try{
        const {
            limit,
            offset,
            sort = 'id',
            dir = 'asc',
            searchTerm = '',
            clientId,
            serviceType,
            from,
            to
        } = req.query;
        let whereCond={ 
                [Op.or]: [
                    { Name: { [Op.like]: `%${searchTerm}%` } },
                ]
            };
            if(clientId){
                whereCond={
                    agent_id: {
                        [Op.in]:convertToArray(clientId)
                    },
                    ...whereCond
                };
            }
        let  whereCondService={}
        if(serviceType){
            whereCondService={
                type:serviceType,
                ...whereCondService
            }
        }
        let whereCondLog={}
        if(from&&to){
            whereCondLog={
                log_end_time:{
                    [Op.between]:[from,to]
                },
                ...whereCondLog
            }
        }

        let data = await models.ServiceAccessLogs.paginate({
            where:{...whereCondLog},
            include: [
              {
                model: models.Service,
                attributes: ['Name','type'],
                where:{...whereCondService},
                required: true
              },
              {
                model: models.Client,
                attributes: ['Name'],
                where: {
                  ...whereCond
                },
                required: true,
                include:[
                    {
                        model:models.DeviceInventories,
                        attributes: ['device_name']
                    }
                ]
              },
              {
                model:models.Connection,
                where:{SESSION_ID:models.sequelize.literal('ServiceAccessLogs.session_id')},
                attributes:['CONNECTION_ATTEMPT_TIME','DISCONNECTION_TIME'],
                required: false
              }
            ],
            limit,
            offset,
        })
        let topFive = await models.ServiceAccessLogs.findAll({
            where: {},
            attributes: [
              'agent_id',
              [
                models.sequelize.fn(
                  'count',
                  models.sequelize.fn('distinct', models.sequelize.col('service_id'))
                ),
                'totalServices'
              ],
              [
                models.sequelize.fn(
                  'max',
                  models.sequelize.col('log_end_time')
                ),
                'LastSession'
              ],
            ],
            include: [
              {
                model: models.Client,
                attributes: ['Name'],
                required: true,
                include:[
                    {
                        model:models.DeviceInventories,
                        attributes: ['device_name']
                    }
                ]
              }
            ],
            group: ['agent_id'],
            order: [[models.sequelize.literal('totalServices DESC')]],
            limit: 5
          }
            )
     
      res.json({data,topFive:{data:topFive,recordsTotal:5}});
    } catch (error) {
        next(error);
    }
} 

module.exports.serviceToUserReport = async (req, res, next) =>{
    try{
        const {
            limit,
            offset,
            sort = 'id',
            dir = 'asc',
            searchTerm = '',
            serviceId,
            serviceType,
            from,
            to
        } = req.query;

        let whereCond={ 
            [Op.or]: [
                { Name: { [Op.like]: `%${searchTerm}%` } },
            ]
        };
        if(serviceId){
            whereCond={
                id: {
                    [Op.in]:convertToArray(serviceId)
                },
                ...whereCond
            };
        }
        if(serviceType){
            whereCond={
                type:serviceType,
                ...whereCond
            }
        }
        let whereCondLog={}
        if(from&&to){
            whereCondLog={
                log_end_time:{
                    [Op.between]:[from,to]
                },
                ...whereCondLog
            }
        }

        let data = await models.ServiceAccessLogs.paginate({
            where:{...whereCondLog},
            include: [
              {
                model: models.Service,
                attributes: ['Name','type','id'],
                where:{...whereCond},
                required: true
              },
              {
                model: models.Client,
                attributes: ['Name','user'],
                required: true,
                include:[
                    {
                        model:models.DeviceInventories,
                        attributes: ['device_name','os']
                    }
                ]
              },
              {
                model:models.Connection,
                where:{SESSION_ID:models.sequelize.literal('ServiceAccessLogs.session_id')},
                attributes:['CONNECTION_ATTEMPT_TIME','DISCONNECTION_TIME'],
                required: false
              }
            ],
            limit,
            offset,
        })
        let topFive = await models.ServiceAccessLogs.findAll({
            where: {},
            attributes: [
              'service_id',
              [
                models.sequelize.fn(
                  'count',
                  models.sequelize.fn('distinct', models.sequelize.col('agent_id'))
                ),
                'totalUsers'
              ],
              [
                models.sequelize.fn(
                  'max',
                  models.sequelize.col('log_end_time')
                ),
                'LastSession'
              ],
            ],
            include: [
              {
                model: models.Service,
                attributes: ['Name','type'],
                required: true,
               
              }
            ],
            group: ['service_id'],
            order: [[models.sequelize.literal('totalUsers DESC')]],
            limit: 5,
            // subQuery: true
          })
     
      res.json({data,topFive:{data:topFive,recordsTotal:5}});
    } catch (error) {
        next(error);
    }
}