const httpErrors = require('http-errors');
const models = require('../models');
const Sequelize = require('sequelize');
const fsExtra = require('fs-extra');
const fs = require('fs');
const moment = require('moment');
const archiver = require('archiver');
const path = require('path');
const psl = require('psl');
const installerPath = '/var/local/installers/saasBridges/';

Sequelize.options.deprecations = { operatorsAliases: false };

const { 
    actionType, 
    publishClients
} = require('../routes/common/event-publisher');

const { 
  getServerCreateMessage,
  getServerEditMessage,
  getServerDeleteMessage,
  serverTypes
} = require('../config/adminLogs.config');

const { insertIntoAdminLogs } = require('../routes/common/adminLogs');

const getBridges = async (req, res, next) => {
    try {
        const {
            limit,
            offset,
            sort = 'id',
            dir = 'desc',
            search = ''
        } = req.query;

        const options = {
            limit,
            offset,
            order: [[sort, dir]]
        }

        if(search.trim()){
            options.where = {
                name: {
                    [Op.like]: `%${search}%`
                }
            }
        }

        const bridges = await models.SaasBridge.paginate(options);

        res.json(bridges);

    } catch (error) {
        next(error);
    }
}

const getBridge = async (req, res, next) => {
    try {
        const { id } = req.params;

        const bridge = await models.SaasBridge.findOne({
            where: {
                id
            }
        });

        if(!bridge){
            throw new httpErrors.NotFound('No SaaS bridge found.');
        }

        res.json(bridge);
    } catch (error) {
        next(error);
    }
}

const createBridge = async (req, res, next) => {
    try {
        const { name, clientId } = req.body;

        const bridge = await models.SaasBridge.findOne({
            where: {
                name
            }
        });

        const isClientId = await models.SaasBridge.findOne({
          where: {
            clientId
          }
        })

        req.body.authAgentDomain = `login.${getDomainName(req.body.authAgentDomain)}`;

        if(bridge){
            throw new httpErrors.Conflict('SaaS bridge already exist with same name.');
        }

        if(isClientId){
          throw new httpErrors.Conflict('Saas bridge with this client is already available. Please select a new client')
        }

        const createdBridge = await models.SaasBridge.create(req.body);

        insertIntoAdminLogs(
          getServerCreateMessage(name, serverTypes.SAAS_BRIDGE),
          req
        )

        res.json(createdBridge);
        
    } catch (error) {
        next(error);
    }
}

const updateBridge = async (req, res, next) => {
    try {
        const { id } = req.params;
        const { name } = req.body;

        const bridgeToUpdate = await models.SaasBridge.findOne({
            where: {
                id
            }
        });

        if(!bridgeToUpdate){
            throw new httpErrors.NotFound('No SaaS bridge found.');
        }

        const isNameExist = await models.SaasBridge.findOne({
            where: {
                name,
                id: {
                    [Op.ne]: +id
                }
            }
        });

        if(isNameExist){
            throw new httpErrors.Conflict('SaaS bridge already exist with same name.');
        }

        req.body.authAgentDomain = `login.${getDomainName(req.body.authAgentDomain)}`;

        const zipPath = `${installerPath}${'saas-bridge'}-${bridgeToUpdate.name.replace(
            /\s/g,
            ''
          )}-${id}.zip`;

        await Promise.all([
            fsExtra.remove(zipPath),
            bridgeToUpdate.update(req.body)
        ]);

        insertIntoAdminLogs(
          getServerEditMessage(name, serverTypes.SAAS_BRIDGE),
          req
        )

        res.json(bridgeToUpdate);

    } catch (error) {
        next(error);
    }
}

const deleteBridge = async (req, res, next) => {
    try {
        const { id } = req.params;

        const bridge = await models.SaasBridge.findOne({
            where: {
                id
            }
        });

        if(!bridge){
            throw new httpErrors.NotFound('No SaaS bridge found.');
        }

        const zipPath = `${installerPath}${'saas-bridge'}-${bridge.name.replace(
            /\s/g,
            ''
          )}-${id}.zip`;

        await Promise.all([
            fsExtra.remove(zipPath),
            bridge.destroy(),
            publishClients(actionType.DELETED, {
              ids: [
                bridge.clientId
              ],
              type: 'Server',
              data: {
                is_active: 0
              }
            })
        ]);

        insertIntoAdminLogs(
          getServerDeleteMessage(bridge.name, serverTypes.SAAS_BRIDGE),
          req
        )

        res.json(bridge);
    } catch (error) {
        next(error);
    }
}

const getClients = async (req, res, next) => {
    try {
        const {
            search,
            offset = 0,
            limit = 10
        } = req.query;

        const QUERY = `
        SELECT *
        FROM (
            SELECT Clients.id, Clients.Name, Clients.user, Clients.device_os, ClientPolicies.policy_id
            FROM Clients
            INNER JOIN ClientPolicies on ClientPolicies.client_id = Clients.id
            WHERE (
                Clients.Name like '%${search.trim()}%' 
                OR Clients.user like '%${search.trim()}%' 
                OR Clients.id like '%${search.trim()}%'
            ) 
            AND Clients.device_os = 'ubuntuserver18'
            AND Clients.client_type = 'Server'
            AND ClientPolicies.policy_type_id = 1
            UNION
            SELECT c.id, c.Name, c.user, c.device_os, cgp.policy_id
            FROM ClientGroupPolicies cgp
            JOIN \`Groups\` g on g.id = cgp.client_group_id
            JOIN GroupClients gc on g.id = gc.groupId
            join Clients c on c.id = gc.clientId
            WHERE (
                c.Name like '%${search.trim()}%' 
                OR c.user like '%${search.trim()}%' 
                OR c.id like '%${search.trim()}%'
            )
            AND c.device_os = 'ubuntuserver18'
            AND c.client_type = 'Server'
            AND cgp.policy_type_id = 1
        ) As tb
        LIMIT ${offset}, ${limit}`;

        const clients = await models.sequelize.query(QUERY, {
            type: Sequelize.QueryTypes.SELECT
        });

        res.json(clients);
    } catch (error) {
        next(error);
    }
}

const getClientServices = async (req, res, next) => { 
    try {
        const { id } = req.params;

        const QUERY = `
            SELECT *
            FROM (
                SELECT s.id, s.Name, s.port, s.mappedurl
                FROM Services s
                INNER JOIN ServicePolicies sp on sp.service_id = s.id
                INNER JOIN Entitlements ap on ap.id = sp.policy_id
                JOIN ClientPolicies cp on cp.policy_id = ap.id
                WHERE cp.client_id=${id} 
                AND sp.policy_type_id = 1
                AND cp.policy_type_id = 1
                UNION
                SELECT s2.id, s2.Name, s2.port, s2.mappedurl
                FROM ServiceGroupPolicies sgp
                JOIN ServiceGroups sg on sg.id = sgp.service_group_id
                JOIN GroupServices gs on sg.id = gs.groupId
                join Services s2 on s2.id = gs.serviceId
                JOIN Entitlements ap1 on ap1.id = sgp.policy_id
                JOIN ClientPolicies cp1 on cp1.policy_id = ap1.id
                WHERE cp1.client_id=${id}
                and sgp.policy_type_id = 1
                and cp1.policy_type_id = 1
                UNION
                SELECT s1.id, s1.Name, s1.port, s1.mappedurl
                FROM Services s1 WHERE ( 
                Select DISTINCT e.all_services from Entitlements e 
                JOIN ClientPolicies cp2 on cp2.policy_id = e.id
                WHERE (e.all_services = 1 AND e.is_enabled = 1
                AND cp2.client_id=${id}
                AND cp2.policy_type_id = 1))
            ) As tb`;
            
        const services = await models.sequelize.query(QUERY, {
            type: Sequelize.QueryTypes.SELECT
        });

        const _services = services.filter(s => s.port.split(',').length == 1 && s.mappedurl != 'None' );

        res.json(_services);
    } catch (error) {
        next(error);
    }
}

const downloadBridgeFromLink = async (req, res, next) => {
    const { token } = req.params;
  
    try {
      const bridge = await models.ServerBridge.findOne({
        where: {
          email_token: token
        }
      });
  
      if (!bridge) {
        throw new httpErrors.NotFound(`SaaS Bridge not found`);
      }
  
      const { id, name, email_token_expiry, email_token_used } = bridge;
  
      if (email_token_used) {
        throw new httpErrors.UnprocessableEntity(
          'Link is already used.please request for a new link'
        );
      }
  
      if (!moment().isBefore(email_token_expiry)) {
        throw new httpErrors.UnprocessableEntity(
          'Link is expired.please request for a new link'
        );
      }
  
      const zipPath = `${installerPath}${`saas-bridge`}-${name.replace(
        /\s/g,
        ''
      )}-${id}.zip`;
  
      await bridge.update({ email_token_used: 1 });
  
      return res.download(zipPath);
    } catch (error) {
      next(error);
    }
};
  
const downloadBridge = async (req, res, next) => {
    const { id } = req.params;
    let zipPath;
  
    try {
      zipPath = await makePackage(id);
      return res.download(zipPath);
    } catch (error) {
      if (zipPath) {
        await fsExtra.remove(zipPath);
      }
  
      next(error);
    }
};

async function makePackage(id, isEmail = false) {
    let name;
    let email;
  
    let bridge = await models.SaasBridge.findOne({
      where: { id },
      include: [
        {
          model: models.Client,
          attributes: [
              'Email',
              'Name',
              'device_os'
            ]
        },
        {
            model: models.Service,
            attributes: [
                'port',
                'Name',
                'mappedurl',
                'dummy_port'
              ]
          }
      ]
    });
  
    if (!bridge){
      throw new httpErrors.NotFound('Saas Bridge not found');
    }

  
    let zipPath = `${installerPath}${`saas-bridge`}-${bridge.name.replace(
      /\s/g,
      ''
    )}-${id}.zip`;
  
    await fsExtra.ensureDir(installerPath);
  
    const clientInstallerPath = `/var/local/installers/client/input/ubuntuserver18/installer.deb`;
  
    const [zipExists, installerExists] = await Promise.all([
      fsExtra.pathExists(zipPath),
      fsExtra.pathExists(clientInstallerPath)
    ]);
  
    if (isEmail) {
      name = bridge.Client.dataValues.Name;
      email = bridge.Client.dataValues.Email;
    }
  
    if (zipExists) {
      return !isEmail ? zipPath : [zipPath, { email, name, bridge }];
    }
  
    if (!installerExists) {
      throw new httpErrors.NotFound(
        `Ubuntu Server Installer does not exist. Please upload first Ubuntu Server package`
      );
    }
  
    await createZip(zipPath, bridge);
  
    return !isEmail ? zipPath : [zipPath, { email, name, bridge }];
}
  
async function createZip(zipPath, bridge) {
    return new Promise((resolve, reject) => {
      let output = fs.createWriteStream(zipPath);

      const configFile = {
        Name: `"${bridge.name}"`,
        Service: `"${bridge.Service ? bridge.Service.dataValues.mappedurl : ''}"`,         
        ServicePort: `"${bridge.Service ? bridge.Service.dataValues.dummy_port.split(',')[0] : 0 }"`,                     
        MappedDomain: `"${bridge.mappedDomain}"`,
        AuthAgentDomain: `"${bridge.authAgentDomain}"`,
        AuthenticationService: `"${bridge.authService}"`,             
        AuthenticationServiceClientID: `"${bridge.authServiceClientId}"`,
        AuthenticationServiceClientSecret: `"${bridge.authServiceClientSecret}"`,
        AuthenticationServiceURL: `"${bridge.authServiceUrl}"`,      
        AuthenticationServiceTokenURL: `"${bridge.authServiceTokenUrl}"`,  
        AuthenticationServiceUserInfoURL: `"${bridge.authServiceUserInfoUrl}"`
    }

      let configText = Object.keys(configFile).reduce((acc, key) => `${acc}${key}=${configFile[key]}\n`, '');
  
      const script ='install_invisily_saas_bridge.sh';
    
      const guide = 'Installation_Guide_SaaS_Bridge.txt';
  
      const shFilePath = path.join(
        __dirname,
        `../scripts/${script}`
      );
      const guideFilePath = path.join(
        __dirname,
        `../scripts/${guide}`
      );
      const serviceListFilePath = path.join(
        __dirname,
        '../scripts/service_list.html'
      );
  
      let archive = archiver('zip', {
        zlib: { level: 9 }
      });
  
      archive.pipe(output);
  
      output.on('close', () => {
        resolve(zipPath);
      });
  
      archive.on('error', error => {
        reject(error);
      });
  
      const clientInstallerPath = `/var/local/installers/client/input/ubuntuserver18/installer.deb`
  
      archive.file(`${clientInstallerPath}`, { name: 'installer.deb' });
  
      archive.file(shFilePath, { name: script });
  
      const jsonBuffer = Buffer.from(configText, 'utf-8');
        var ReadableStream = require('stream');
        var zipInputStream = new ReadableStream.Readable();
        zipInputStream.push(jsonBuffer);
        zipInputStream.push(null);
  
      archive.append(zipInputStream, { name: 'saas_bridge.conf' });
  
      archive.file(guideFilePath, {
        name: guide
      });
  
      archive.file(serviceListFilePath, { name: 'service_list.html' });
  
      archive.finalize();
    });
}
  
const getDomainName = (url) => {
    const _url = url.replace(/(^\w+:|^)\/\//g, '');
    const parsed = psl.parse(_url);
    return parsed.domain;
}

module.exports = {
    getBridges,
    getBridge,
    createBridge,
    updateBridge,
    deleteBridge,
    getClients,
    getClientServices,
    downloadBridge,
    downloadBridgeFromLink
}