const models = require('../models');
const httpErrors = require('http-errors');
const _ = require('lodash');
const fsExtra = require('fs-extra');
const fs = require('fs');
const Sequelize = require('sequelize');
const installerPath = '/var/local/installers/serverBridges/';
const archiver = require('archiver');
const path = require('path');
const crypto = require('crypto');
const helper = require('../routes/common/helper.js');
const moment = require('moment');
const uuid = require('uuid');
var Openssl = require('../openssl/openssl.js');
var OpensslObj = new Openssl();
const projectConfig = require('../config/config')['project'][
  process.env.NODE_ENV
];
Sequelize.options.deprecations = { operatorsAliases: false };

const {
  actionType,
  publishServerBridgeDeviceIDs,
  publishClients,
  publishMsgFile,
  publishAccessPolicy,
  publishTrustedHosts
} = require('../routes/common/event-publisher');
const { Op } = require('sequelize');

const {
  getServerCreateMessage,
  getServerEditMessage,
  getServerDeleteMessage,
  getServerDisableMessage,
  getServerEnableMessage,
  serverTypes
} = require('../config/adminLogs.config');

const { insertIntoAdminLogs } = require('../routes/common/adminLogs');

/**
 * get Server Bridge
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */

module.exports.getServerClients = async (req, res, next) => {
  try {
    let offset = +req.query.offset || 0;
    const limit = +req.query.limit || 10;

    const { name, type } = req.query;

    offset = offset * limit;

    let where = {
      device_os:
        type == 'iot-bridge' ? 'ubuntuserver20armv8' : 'ubuntuserver18',
      '$ServerBridge.clientId$': null,
      client_status: 1,
      client_type: 'server'
    };

    if (name && name !== 'null') {
      where = {
        ...where,
        name: {
          [Op.like]: `%${name}%`
        }
      };
    }

    if (type == 'b2b-bridge' || type == 'rbi-bridge') {
      where = {
        ...where,
        device_os: {
          [Op.in]: ['ubuntuserver20armv8', 'ubuntuserver18']
        }
      };
    }

    const clients = await models.Client.findAll({
      limit,
      offset,
      where,
      attributes: ['id', 'name', 'user', 'device_os'],
      include: [
        {
          model: models.ServerBridge,
          required: false,
          attributes: []
        }
      ]
    });

    return res.json(clients);
  } catch (error) {
    next(error);
  }
};

/**
 * get Server Bridge
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */

module.exports.getBridges = async (req, res, next) => {
  try {
    const { limit, offset, sort = 'id', dir = 'desc', name } = req.query;
    const type = req.query.type || 'server-bridge';

    const options = {
      limit,
      offset,
      order: [],
      where: {
        type
      }
    };
    const allowedSortValues = ['id', 'name'];

    if (allowedSortValues.includes(sort)) {
      options.order.push([models.sequelize.literal(sort), dir]);
    }

    if (name) {
      options.where = {
        ...options.where,
        name: {
          [Op.like]: `%${name}%`
        }
      };
    }

    let result = await models.ServerBridge.paginate({
      ...options
    });

    const clientOnlineStatusArray = result.data.map(bridge =>
      bridge.getClientOnlineStatus()
    );

    const clientStatus = await Promise.all(clientOnlineStatusArray);

    result = {
      ...result,
      data: result.data.map((bridge, index) => {
        return {
          ...bridge.dataValues,
          is_online: clientStatus[index]
        };
      })
    };

    return res.json(result);
  } catch (error) {
    next(error);
  }
};

/**
 * create new Server Bridge
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */

module.exports.createBridge = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    const { name, clientId, deviceIds, physicalDevice, devices } = req.body;

    const type = req.query.type || 'server-bridge';

    const brdigeObject = {
      type,
      name,
      clientId
    };

    const _devices = await models.DeviceInventories.findAll({
      where: {
        id: {
          [Op.in]: devices
        }
      }
    });

    const [encKeys, hmacKeys] = await Promise.all([
      Promise.all(_devices.map(x => OpensslObj.generateEncKey())),
      Promise.all(_devices.map(x => OpensslObj.generateHMacKey()))
    ]);

    const clients = _devices.map((device, i) => ({
      Name: mergeUserAndBridgeNames(device.device_name, name),
      ipv4_address: device.ip_address,
      user: 'bridge-' + new Date().getTime(),
      device_os:
        type == 'iot-bridge' ? 'ubuntuserver20armv8' : 'ubuntuserver18',
      client_type: 'Server',
      client_status: 4,
      risk_score: device.risk_score,
      client_darkening: device.darkening_state,
      client_os_version: '18.04',
      device_hardware_id: crypto
        .createHash('md5')
        .update(device.device_id + '')
        .digest('hex'),
      agent_id: 0,
      enc_key: encKeys[i].key,
      HMacKey: hmacKeys[i].key,
      is_local: 1,
      device_id: device.id,
      auto_updates: false,
      user_auth_required: false
    }));

    const [bridge, created_clients] = await Promise.all([
      models.ServerBridge.create(brdigeObject, { transaction }),
      models.Client.bulkCreate(clients, { transaction })
    ]);

    const clientIds = await created_clients.map(client => client.id);

    const softwareInventoryData = clientIds.map(clientId => ({
      client_id: clientId
    }));

    await Promise.all(
      softwareInventoryData.map(data =>
        models.ClientSoftwareInventory.create(data, { transaction })
      )
    );

    device_client_map = created_clients.map((c, i) => ({
      client_id: c.id,
      device_id: _devices[i].id,
      serverBridgeId: bridge.id,
      enc_key: c.enc_key,
      HMacKey: c.HMacKey,
      device_hardware_id: c.device_hardware_id
    }));

    const [, , masterClient] = await Promise.all([
      models.ServerBridgeDeviceId.bulkCreate(device_client_map, {
        transaction
      }),
      models.Client.update(
        {
          client_status: 1
        },
        {
          where: {
            id: clientId
          },
          transaction
        }
      ),
      models.Client.findOne({
        attributes: ['id', 'enc_key', 'HMacKey', 'device_hardware_id'],
        where: {
          id: clientId
        },
        raw: true
      })
    ]);

    await Promise.all([
      [...device_client_map, masterClient].map(c =>
        publishMsgFile(actionType.ADDED, {
          id: c.client_id || c.id,
          data: {
            enc_key: c.enc_key,
            HMacKey: c.HMacKey,
            device_hardware_id: c.device_hardware_id
          }
        })
      ),
      models.sequelize.query('UPDATE Clients SET agent_id = id;', {
        transaction
      })
    ]);

    // const brdigeObject = {
    //   type,
    //   name,
    //   clientId,
    //   serverBridgeIds: deviceIds.map(device => ({ deviceId: device })),
    // };

    // const bridge = await models.ServerBridge.create(brdigeObject, {
    //   include: [
    //     {
    //       model: models.ServerBridgeDeviceId,
    //       as: 'serverBridgeIds'
    //     }
    //   ]
    // });
    await transaction.commit();

    insertIntoAdminLogs(getServerCreateMessage(name, type), req);

    return res.json(bridge);
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

module.exports.createRBIBridge = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    let { name, clientId, userIds } = req.body;

    name = name.trim();

    const type = 'rbi-bridge';

    const brdigeObject = {
      type,
      name,
      clientId
    };

    const _users = await models.LocalUsers.findAll({
      where: {
        id: {
          [Op.in]: userIds
        }
      }
    });

    const serverClient = await models.Client.findOne({
      attributes: ['device_os'],
      where: {
        id: clientId
      }
    });

    const [encKeys, hmacKeys, serialNumbers] = await Promise.all([
      Promise.all(_users.map(x => OpensslObj.generateEncKey())),
      Promise.all(_users.map(x => OpensslObj.generateHMacKey())),
      Promise.all(_users.map(x => uuid.v4()))
    ]);

    const clients = _users.map((user, i) => ({
      Name: mergeUserAndBridgeNames(user.name,name),
      user: user.userName,
      client_type: 'Server',
      client_status: 4,
      client_darkening: true,
      device_os: serverClient.device_os,
      serial_number: serialNumbers[i] + '',
      device_hardware_id: crypto
        .createHash('md5')
        .update(serialNumbers[i] + '')
        .digest('hex'),
      agent_id: 0,
      enc_key: encKeys[i].key,
      HMacKey: hmacKeys[i].key,
      user_id: user.id,
      auto_updates: false,
      is_local: user.identity_provider === 'Local User' ? 1 : 0,
      user_auth_required: false
    }));

    console.log("brdigeObject ", brdigeObject)
    const bridge = await models.ServerBridge.create(brdigeObject, {
      transaction
    });


    const myService = await models.Service.findOne({
        where: {
            mappedurl: {
                [Op.like]: `%api.invisily%`
            }
        }
    });

    if(!myService) {
      throw new httpErrors.BadRequest('api.invisily service not found')
    }

    const myService2 = await models.Service.findOne({
        where: {
            mappedurl: {
                [Op.like]: `%software-inventory.server%`
            }
        }
    });

    if (!myService2) {
        throw new httpErrors.BadRequest('Software Inventory Server service not found')
    }

    const accessPolicy = await models.Entitlements.create({
      name: bridge.name + new Date().getTime(),
      policy_type: 'User Agent'
    }, { transaction })
    await models.ServicePolicies.create({
      service_id: myService.id,
      policy_type_id: 1,
      policy_id: accessPolicy.id
    }, {transaction});
    await models.ClientPolicies.create({
      client_id: clientId,
      policy_type_id: 1,
      policy_id: accessPolicy.id
    }, {transaction});

    const created_clients = await models.Client.bulkCreate(clients, {
      transaction
    });

    const clientIds = await created_clients.map(client => client.id);

    const softwareInventoryData = clientIds.map(clientId => ({
      client_id: clientId
    }));

    await Promise.all(
      softwareInventoryData.map(data =>
        models.ClientSoftwareInventory.create(data, { transaction })
      )
    );

    created_clients.forEach(client => {
      client.generateOtp();
    });

    const bridgeUsers = created_clients.map((c, i) => ({
      user_id: c.user_id,
      client_id: c.id,
      serverBridgeId: bridge.id
    }));

    const userPivots = await models.ServerBridgeUserId.bulkCreate(bridgeUsers, {
      transaction
    });

    created_clients.map((c) => {
      // creating trusted hosts async
      const host = {
        address: '0.0.0.0',
        port: 6901,
        protocol: 'tcp',
        label: 'vnc',
        direction: 1,
        parentId: c.id,
        parentType: 'Client',
        clientId: c.id
      };
      models.GlobalIps.create(host).then(trustedHost => {
        publishTrustedHosts(actionType.ADDED, {
          ids: [c.id],
          parentType: 'Client'
        });
      });
    });
    await Promise.all([
      created_clients.map(c =>
        publishMsgFile(actionType.ADDED, {
          id: c.id,
          data: {
            enc_key: c.enc_key,
            HMacKey: c.HMacKey,
            device_hardware_id: c.device_hardware_id
          }
        })
      ),
      models.sequelize.query('UPDATE Clients SET agent_id = id;', {
        transaction
      })
    ]);

    await transaction.commit();

    publishAccessPolicy(actionType.ADDED, {
      id: accessPolicy.id,
      mode: 2,
      data: {
          name: accessPolicy.name,
          access_restriction: +accessPolicy.access_restriction,
          access_start_time: accessPolicy.access_start_time,
          access_end_time: accessPolicy.access_end_time,
          months: accessPolicy.months,
          days: accessPolicy.days,
          all_clients: +accessPolicy.all_clients,
          all_services: +accessPolicy.all_services,
          policy_type: accessPolicy.policy_type
      }
    });

    insertIntoAdminLogs(
      getServerCreateMessage(name, serverTypes.RBI_BRIDGE),
      req
    )

    return res.json(bridge);
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

module.exports.createB2BBridge = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    let { name, clientId, userIds } = req.body;

    name = name.trim();

    const type = 'b2b-bridge';

    const brdigeObject = {
      type,
      name,
      clientId
    };

    const _users = await models.LocalUsers.findAll({
      where: {
        id: {
          [Op.in]: userIds
        }
      }
    });

    const serverClient = await models.Client.findOne({
      attributes: ['device_os'],
      where: {
        id: clientId
      }
    });

    const [encKeys, hmacKeys, serialNumbers] = await Promise.all([
      Promise.all(_users.map(x => OpensslObj.generateEncKey())),
      Promise.all(_users.map(x => OpensslObj.generateHMacKey())),
      Promise.all(_users.map(x => uuid.v4()))
    ]);

    const clients = _users.map((user, i) => ({
      Name: mergeUserAndBridgeNames(user.name, name),
      user: user.userName,
      client_type: 'Server',
      client_status: 4,
      client_darkening: true,
      device_os: serverClient.device_os,
      serial_number: serialNumbers[i] + '',
      device_hardware_id: crypto
        .createHash('md5')
        .update(serialNumbers[i] + '')
        .digest('hex'),
      agent_id: 0,
      enc_key: encKeys[i].key,
      HMacKey: hmacKeys[i].key,
      user_id: user.id,
      auto_updates: false,
      is_local: user.identity_provider === 'Local User' ? 1 : 0,
      user_auth_required: false
    }));

    console.log('brdigeObject ', brdigeObject);
    const bridge = await models.ServerBridge.create(brdigeObject, {
      transaction
    });

    const myService = await models.Service.findOne({
      where: {
        mappedurl: {
          [Op.like]: `%api.invisily%`
        }
      }
    });

    if (!myService) {
      throw new httpErrors.BadRequest('api.invisily service not found');
    }

    const myService2 = await models.Service.findOne({
      where: {
        mappedurl: {
          [Op.like]: `%software-inventory.server%`
        }
      }
    });

    if (!myService2) {
      throw new httpErrors.BadRequest(
        'Software Inventory Server service not found'
      );
    }

    const accessPolicy = await models.Entitlements.create(
      {
        name: bridge.name + new Date().getTime(),
        policy_type: 'User Agent'
      },
      { transaction }
    );
    await models.ServicePolicies.create(
      {
        service_id: myService.id,
        policy_type_id: 1,
        policy_id: accessPolicy.id
      },
      { transaction }
    );
    await models.ClientPolicies.create(
      {
        client_id: clientId,
        policy_type_id: 1,
        policy_id: accessPolicy.id
      },
      { transaction }
    );

    const created_clients = await models.Client.bulkCreate(clients, {
      transaction
    });

    const clientIds = await created_clients.map(client => client.id);

    const softwareInventoryData = clientIds.map(clientId => ({
      client_id: clientId
    }));

    await Promise.all(
      softwareInventoryData.map(data =>
        models.ClientSoftwareInventory.create(data, { transaction })
      )
    );

    created_clients.forEach(client => {
      client.generateOtp();
    });

    const bridgeUsers = created_clients.map((c, i) => ({
      user_id: c.user_id,
      client_id: c.id,
      serverBridgeId: bridge.id
    }));

    const userPivots = await models.ServerBridgeUserId.bulkCreate(bridgeUsers, {
      transaction
    });

    created_clients.map(c => {
      // creating trusted hosts async
      const host = {
        address: '0.0.0.0',
        port: 6901,
        protocol: 'tcp',
        label: 'vnc',
        direction: 1,
        parentId: c.id,
        parentType: 'Client',
        clientId: c.id
      };
      models.GlobalIps.create(host).then(trustedHost => {
        publishTrustedHosts(actionType.ADDED, {
          ids: [c.id],
          parentType: 'Client'
        });
      });
    });
    await Promise.all([
      created_clients.map(c =>
        publishMsgFile(actionType.ADDED, {
          id: c.id,
          data: {
            enc_key: c.enc_key,
            HMacKey: c.HMacKey,
            device_hardware_id: c.device_hardware_id
          }
        })
      ),
      models.sequelize.query('UPDATE Clients SET agent_id = id;', {
        transaction
      })
    ]);

    await transaction.commit();

    publishAccessPolicy(actionType.ADDED, {
      id: accessPolicy.id,
      mode: 2,
      data: {
        name: accessPolicy.name,
        access_restriction: +accessPolicy.access_restriction,
        access_start_time: accessPolicy.access_start_time,
        access_end_time: accessPolicy.access_end_time,
        months: accessPolicy.months,
        days: accessPolicy.days,
        all_clients: +accessPolicy.all_clients,
        all_services: +accessPolicy.all_services,
        policy_type: accessPolicy.policy_type
      }
    });

    insertIntoAdminLogs(
      getServerCreateMessage(name, serverTypes.B2B_BRIDGE),
      req
    );

    return res.json(bridge);
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

/**
 * Update Server Bridge
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */

module.exports.updatebridge = async (req, res, next) => {
  const { id } = req.params;
  const { name, deviceIds, devices } = req.body;
  const transaction = await models.sequelize.transaction();

  try {
    const bridge = await models.ServerBridge.findOne({
      where: {
        id
      },
      include: [
        {
          model: models.ServerBridgeDeviceId,
          as: 'serverBridgeIds'
        }
      ]
    });

    if (!bridge) {
      throw new httpErrors.NotFound('Bridge with this id does not exist');
    }

    const zipPath = `${installerPath}${bridge.type}-${bridge.name.replace(
      /\s/g,
      ''
    )}-${id}.zip`;

    await fsExtra.remove(zipPath);

    const deviceIdsExists = bridge.serverBridgeIds.map(
      bridge => bridge.device_id
    );

    const deletedDeviceIds = _.difference(deviceIdsExists, devices);
    const addDeviceIds = _.difference(devices, deviceIdsExists);

    // Clients to delete
    const clientsToDelete = await models.ServerBridgeDeviceId.findAll({
      attributes: ['client_id', 'device_id'],
      where: {
        device_id: deletedDeviceIds,
        serverBridgeId: bridge.id
      },
      raw: true
    });

    // Clients to add
    const _devices = await models.DeviceInventories.findAll({
      where: {
        id: {
          [Op.in]: addDeviceIds
        }
      }
    });

    const [encKeys, hmacKeys] = await Promise.all([
      Promise.all(_devices.map(x => OpensslObj.generateEncKey())),
      Promise.all(_devices.map(x => OpensslObj.generateHMacKey()))
    ]);

    const clients = _devices.map((device, i) => ({
      Name: mergeUserAndBridgeNames(device.device_name, name),
      ipv4_address: device.ip_address,
      user: 'bridge-' + new Date().getTime(),
      device_os:
        bridge.type == 'iot-bridge' ? 'ubuntuserver20armv8' : 'ubuntuserver18',
      client_type: 'Server',
      client_status: 4,
      risk_score: device.risk_score,
      client_darkening: device.darkening_state,
      client_os_version: '18.04',
      device_hardware_id: crypto
        .createHash('md5')
        .update(device.device_id + '')
        .digest('hex'),
      agent_id: 0,
      enc_key: encKeys[i].key,
      HMacKey: hmacKeys[i].key,
      is_local: 1,
      device_id: device.id,
      auto_updates: false,
      user_auth_required: false
    }));

    const created_clients = await models.Client.bulkCreate(clients, {
      transaction
    });

    const clientIds = await created_clients.map(client => client.id);

    const softwareInventoryData = clientIds.map(clientId => ({
      client_id: clientId
    }));

    await Promise.all(
      softwareInventoryData.map(data =>
        models.ClientSoftwareInventory.create(data, { transaction })
      )
    );

    device_client_map = created_clients.map((c, i) => ({
      client_id: c.id,
      device_id: _devices[i].id,
      serverBridgeId: bridge.id,
      enc_key: c.enc_key,
      HMacKey: c.HMacKey,
      device_hardware_id: c.device_hardware_id
    }));

    // Clients to add

    await Promise.all([
      models.ServerBridgeDeviceId.destroy({
        where: {
          device_id: {
            [Op.in]: deletedDeviceIds
          },
          serverBridgeId: bridge.id
        },
        transaction
      }),
      models.ServerBridgeDeviceId.bulkCreate(device_client_map, {
        transaction
      }),
      bridge.update(
        {
          name
        },
        { transaction }
      ),
      models.Client.destroy({
        where: {
          id: {
            [Op.in]: clientsToDelete.map(x => x.client_id)
          }
        },
        transaction
      })
    ]);

    const dataToPublish = await models.ServerBridgeDeviceId.findAll({
      attributes: ['client_id', 'device_id', 'serverBridgeId'],
      where: {
        serverBridgeId: bridge.id,
        device_id: {
          [Op.in]: [
            ...new Set([
              ...addDeviceIds,
              ..._.difference(deviceIdsExists, deletedDeviceIds)
            ])
          ]
        }
      },
      include: [
        {
          model: models.DeviceInventories,
          attributes: ['mac_address', 'id']
        }
      ],
      raw: true,
      transaction
    });

    await Promise.all([
      publishServerBridgeDeviceIDs(actionType.UPDATED, {
        id: bridge.clientId,
        mac_ids: dataToPublish.map(x => ({
          agent_id: x.client_id,
          mac: x['DeviceInventory.mac_address']
        }))
      }),
      clientsToDelete.length
        ? publishClients(actionType.DELETED, {
            ids: clientsToDelete.map(x => x.client_id),
            type: 'Server',
            data: {}
          })
        : [],
      device_client_map.map(c =>
        publishMsgFile(actionType.ADDED, {
          id: c.client_id,
          data: {
            enc_key: c.enc_key,
            HMacKey: c.HMacKey,
            device_hardware_id: c.device_hardware_id
          }
        })
      ),
      models.sequelize.query('UPDATE Clients SET agent_id = id;', {
        transaction
      })
    ]);

    await transaction.commit();

    insertIntoAdminLogs(getServerEditMessage(name, bridge.type), req);

    return res.json(bridge);
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

module.exports.updateRBIbridge = async (req, res, next) => {
  const { id } = req.params;
  const { name, userIds } = req.body;
  const transaction = await models.sequelize.transaction();

  try {
    const bridge = await models.ServerBridge.findOne({
      where: {
        id
      },
      include: [
        {
          model: models.ServerBridgeUserId,
          as: 'serverBridgeUserIds'
        }
      ]
    });

    console.log('bridge :>> ', bridge);

    const serverClient = await models.Client.findOne({
      attributes: ['device_os'],
      where: {
        id: bridge.clientId
      }
    });

    if (!bridge) {
      throw new httpErrors.NotFound('Bridge with this id does not exist');
    }

    const zipPath = `${installerPath}${bridge.type}-${bridge.name.replace(
      /\s/g,
      ''
    )}-${id}.zip`;

    await fsExtra.remove(zipPath);

    const userIdsExists = bridge.serverBridgeUserIds.map(user => {
      return user.user_id;
    });

    const addedClients = bridge.serverBridgeUserIds.map(user => {
      return user.client_id;
    });

    const deletedUserIds = _.difference(userIdsExists, userIds);
    const addUserIds = _.difference(userIds, userIdsExists);

    // Clients to delete
    const clientsToDelete = await models.ServerBridgeUserId.findAll({
      attributes: ['client_id', 'user_id'],
      where: {
        user_id: deletedUserIds,
        serverBridgeId: bridge.id
      },
      raw: true
    });

    // Clients to add
    const _users = await models.LocalUsers.findAll({
      where: {
        id: {
          [Op.in]: addUserIds
        }
      }
    });

    await models.Client.update({
      device_os: serverClient.device_os
    }, {
      where: {
        id: {
          [Op.in]: addedClients
        }
      }
    });

    const [encKeys, hmacKeys, serialNumbers] = await Promise.all([
      Promise.all(_users.map(x => OpensslObj.generateEncKey())),
      Promise.all(_users.map(x => OpensslObj.generateHMacKey())),
      Promise.all(_users.map(x => uuid.v4()))
    ]);

    const clients = _users.map((user, i) => ({
      Name: mergeUserAndBridgeNames(user.name,name),
      user: user.userName,
      client_type: 'Server',
      client_status: 4,
      client_darkening: true,
      device_os: serverClient.device_os,
      serial_number: serialNumbers[i] + '',
      device_hardware_id: crypto
        .createHash('md5')
        .update(serialNumbers[i] + '')
        .digest('hex'),
      agent_id: 0,
      enc_key: encKeys[i].key,
      HMacKey: hmacKeys[i].key,
      user_id: user.id,
      auto_updates: false,
      is_local: user.identity_provider === 'Local User' ? 1 : 0,
      user_auth_required: false
    }));

    const created_clients = await models.Client.bulkCreate(clients, {
      transaction
    });

    const clientIds = await created_clients.map(client => client.id);

    const softwareInventoryData = clientIds.map(clientId => ({
      client_id: clientId
    }));

    await Promise.all(
      softwareInventoryData.map(data =>
        models.ClientSoftwareInventory.create(data, { transaction })
      )
    );

    created_clients.forEach(client => {
      client.generateOtp();
    });

    const serverBridgeUsers = created_clients.map((c, i) => ({
      user_id: c.user_id,
      client_id: c.id,
      serverBridgeId: bridge.id
    }));

    created_clients.map((c) => {
      // creating trusted hosts async
      const host = {
        address: '0.0.0.0',
        port: 6901,
        protocol: 'tcp',
        label: 'vnc',
        direction: 1,
        parentId: c.id,
        parentType: 'Client',
        clientId: c.id
      };
      models.GlobalIps.create(host).then(trustedHost => {
        publishTrustedHosts(actionType.ADDED, {
          ids: [c.id],
          parentType: 'Client'
        });
      })
    })


    // Clients to add

    await Promise.all([
      models.ServerBridgeUserId.destroy({
        where: {
          user_id: {
            [Op.in]: deletedUserIds
          },
          serverBridgeId: bridge.id
        },
        transaction
      }),
      models.ServerBridgeUserId.bulkCreate(serverBridgeUsers, {
        transaction
      }),
      bridge.update(
        {
          name
        },
        { transaction }
      ),
      models.Client.destroy({
        where: {
          id: {
            [Op.in]: clientsToDelete.map(x => x.client_id)
          }
        },
        transaction
      })
    ]);

    await Promise.all([
      clientsToDelete.length
        ? publishClients(actionType.DELETED, {
            ids: clientsToDelete.map(x => x.client_id),
            type: 'Server',
            data: {}
          })
        : [],
      created_clients.map(c =>
        publishMsgFile(actionType.ADDED, {
          id: c.id,
          data: {
            enc_key: c.enc_key,
            HMacKey: c.HMacKey,
            device_hardware_id: c.device_hardware_id
          }
        })
      ),
      models.sequelize.query('UPDATE Clients SET agent_id = id;', {
        transaction
      })
    ]);

    await transaction.commit();

    insertIntoAdminLogs(
      getServerEditMessage(name, serverTypes.RBI_BRIDGE),
      req
    )

    return res.json(bridge);
  } catch (error) {
    console.log('error: ', error);
    await transaction.rollback();
    next(error);
  }
};

module.exports.updateB2Bbridge = async (req, res, next) => {
  const { id } = req.params;
  const { name, userIds } = req.body;
  const transaction = await models.sequelize.transaction();

  try {
    const bridge = await models.ServerBridge.findOne({
      where: {
        id
      },
      include: [
        {
          model: models.ServerBridgeUserId,
          as: 'serverBridgeUserIds'
        }
      ]
    });

    const serverClient = await models.Client.findOne({
      attributes: ['device_os'],
      where: {
        id: bridge.clientId
      }
    });

    if (!bridge) {
      throw new httpErrors.NotFound('Bridge with this id does not exist');
    }

    const zipPath = `${installerPath}${bridge.type}-${bridge.name.replace(
      /\s/g,
      ''
    )}-${id}.zip`;

    await fsExtra.remove(zipPath);

    const userIdsExists = bridge.serverBridgeUserIds.map(user => {
      return user.user_id;
    });

    const addedClients = bridge.serverBridgeUserIds.map(user => {
      return user.client_id;
    });

    const deletedUserIds = _.difference(userIdsExists, userIds);
    const addUserIds = _.difference(userIds, userIdsExists);

    // Clients to delete
    const clientsToDelete = await models.ServerBridgeUserId.findAll({
      attributes: ['client_id', 'user_id'],
      where: {
        user_id: deletedUserIds,
        serverBridgeId: bridge.id
      },
      raw: true
    });

    // Clients to add
    const _users = await models.LocalUsers.findAll({
      where: {
        id: {
          [Op.in]: addUserIds
        }
      }
    });

    await models.Client.update(
      {
        device_os: serverClient.device_os
      },
      {
        where: {
          id: {
            [Op.in]: addedClients
          }
        }
      }
    );

    const [encKeys, hmacKeys, serialNumbers] = await Promise.all([
      Promise.all(_users.map(x => OpensslObj.generateEncKey())),
      Promise.all(_users.map(x => OpensslObj.generateHMacKey())),
      Promise.all(_users.map(x => uuid.v4()))
    ]);

    const clients = _users.map((user, i) => ({
      Name: mergeUserAndBridgeNames(user.name, name),
      user: user.userName,
      client_type: 'Server',
      client_status: 4,
      client_darkening: true,
      device_os: serverClient.device_os,
      serial_number: serialNumbers[i] + '',
      device_hardware_id: crypto
        .createHash('md5')
        .update(serialNumbers[i] + '')
        .digest('hex'),
      agent_id: 0,
      enc_key: encKeys[i].key,
      HMacKey: hmacKeys[i].key,
      user_id: user.id,
      auto_updates: false,
      is_local: user.identity_provider === 'Local User' ? 1 : 0,
      user_auth_required: false
    }));

    const created_clients = await models.Client.bulkCreate(clients, {
      transaction
    });

    const clientIds = await created_clients.map(client => client.id);

    const softwareInventoryData = clientIds.map(clientId => ({
      client_id: clientId
    }));

    await Promise.all(
      softwareInventoryData.map(data =>
        models.ClientSoftwareInventory.create(data, { transaction })
      )
    );

    created_clients.forEach(client => {
      client.generateOtp();
    });

    const serverBridgeUsers = created_clients.map((c, i) => ({
      user_id: c.user_id,
      client_id: c.id,
      serverBridgeId: bridge.id
    }));

    created_clients.map(c => {
      // creating trusted hosts async
      const host = {
        address: '0.0.0.0',
        port: 6901,
        protocol: 'tcp',
        label: 'vnc',
        direction: 1,
        parentId: c.id,
        parentType: 'Client',
        clientId: c.id
      };
      models.GlobalIps.create(host).then(trustedHost => {
        publishTrustedHosts(actionType.ADDED, {
          ids: [c.id],
          parentType: 'Client'
        });
      });
    });

    // Clients to add

    await Promise.all([
      models.ServerBridgeUserId.destroy({
        where: {
          user_id: {
            [Op.in]: deletedUserIds
          },
          serverBridgeId: bridge.id
        },
        transaction
      }),
      models.ServerBridgeUserId.bulkCreate(serverBridgeUsers, {
        transaction
      }),
      bridge.update(
        {
          name
        },
        { transaction }
      ),
      models.Client.destroy({
        where: {
          id: {
            [Op.in]: clientsToDelete.map(x => x.client_id)
          }
        },
        transaction
      })
    ]);

    await Promise.all([
      clientsToDelete.length
        ? publishClients(actionType.DELETED, {
            ids: clientsToDelete.map(x => x.client_id),
            type: 'Server',
            data: {}
          })
        : [],
      created_clients.map(c =>
        publishMsgFile(actionType.ADDED, {
          id: c.id,
          data: {
            enc_key: c.enc_key,
            HMacKey: c.HMacKey,
            device_hardware_id: c.device_hardware_id
          }
        })
      ),
      models.sequelize.query('UPDATE Clients SET agent_id = id;', {
        transaction
      })
    ]);

    await transaction.commit();

    insertIntoAdminLogs(
      getServerEditMessage(name, serverTypes.B2B_BRIDGE),
      req
    );

    return res.json(bridge);
  } catch (error) {
    console.log('error: ', error);
    await transaction.rollback();
    next(error);
  }
};

/**
 * Delete specific Server Bridge
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */

module.exports.deleteBridge = async (req, res, next) => {
  const { id } = req.params;
  const transaction = await models.sequelize.transaction();
  try {
    const bridge = await models.ServerBridge.findOne({
      where: { id }
    });

    if (!bridge) {
      const type =
        bridge.type == 'iot-bridge'
          ? 'IoT Bridge'
          : bridge.type == 'client-bridge'
          ? 'Client Bridge'
          : bridge.type == 'b2b-bridge'
          ? 'B2B Bridge'
          : bridge.type == 'rbi-bridge'
          ? 'RBI Bridge'
          : 'Server Bridge';
      throw new httpErrors.NotFound(`${type} not found`);
    }

    // delete slave clients
    const serverBridgeIds = await models.ServerBridgeDeviceId.findAll({
      attributes: ['client_id', 'device_id'],
      where: {
        serverBridgeId: bridge.id
      },
      raw: true
    });

    const serverBridgeUserIds = await models.ServerBridgeUserId.findAll({
      attributes: ['client_id', 'user_id'],
      where: {
        serverBridgeId: bridge.id
      },
      raw: true
    });

    const slaveClientIds = [];

    serverBridgeIds.forEach(cp => {
      slaveClientIds.push(cp.client_id);
    });
    serverBridgeUserIds.forEach(cp => {
      slaveClientIds.push(cp.client_id);
    });

    if (slaveClientIds.length > 0) {
      const servicePoliciesData = [];

      for (let i = 0; i < slaveClientIds.length; i++) {
        const clientPolicies = await models.ClientPolicies.findAll({
          where: {
            client_id: slaveClientIds[i]
          },
          transaction
        });

        for (let k = 0; k < clientPolicies.length; k++) {
          const servicePolicies = await models.ServicePolicies.findAll({
            attributes: ['id', 'policy_id', 'policy_type_id'],
            where: {
              policy_id: clientPolicies[k].policy_id,
              policy_type_id: clientPolicies[k].policy_type_id
            }
          });

          const serviceGroupPolicies = await models.ServiceGroupPolicies.findAll(
            {
              attributes: ['id', 'policy_id', 'policy_type_id'],
              where: {
                policy_id: clientPolicies[k].policy_id,
                policy_type_id: clientPolicies[k].policy_type_id
              }
            }
          );

          servicePolicies.forEach(cp => {
            const policyObject = new Object();
            policyObject.id = cp.id;
            policyObject.policy_id = cp.policy_id;
            policyObject.policy_type_id = cp.policy_type_id;
            policyObject.type = 1; // policy

            servicePoliciesData.push(policyObject);
          });
          serviceGroupPolicies.forEach(cp => {
            const policyObject = new Object();
            policyObject.id = cp.id;
            policyObject.policy_id = cp.policy_id;
            policyObject.policy_type_id = cp.policy_type_id;
            policyObject.type = 2; // group policies

            servicePoliciesData.push(policyObject);
          });
        }

        const clientPolicyIds = clientPolicies.map(value => value.id);
        await models.ClientPolicies.destroy({
          where: {
            id: clientPolicyIds
          },
          transaction
        });
      }

      for (let j = 0; j < servicePoliciesData.length; j++) {
        let deleteFlag = false;
        const { id, policy_id, policy_type_id, type } = servicePoliciesData[j];

        if (policy_type_id === 1) {
          let query = `
            SELECT e.id as policy_id, cp.id as client_policy_id, cp.policy_type_id as policy_type
            FROM Entitlements e 
            INNER JOIN ClientPolicies cp ON cp.policy_id = e.id
            WHERE e.id = ${policy_id} AND cp.policy_type_id = ${policy_type_id} AND cp.client_id NOT IN (${slaveClientIds})

            UNION

            SELECT e.id as policy_id, cp.id as client_policy_id, cp.policy_type_id as policy_type
            FROM Entitlements e 
            INNER JOIN ClientGroupPolicies cp ON cp.policy_id = e.id
            WHERE e.id = ${policy_id} AND cp.policy_type_id = ${policy_type_id}
          `;

          let results = (await models.sequelize.query(query))[0];

          if (results.length === 0) {
            await models.Entitlements.destroy({
              where: {
                id: policy_id
              },
              transaction
            });
            deleteFlag = true;
          }
        }
        if (policy_type_id === 2) {
          let query = `
            SELECT e.id as policy_id, cp.id as client_policy_id, cp.policy_type_id as policy_type
            FROM DevicePolicies e 
            INNER JOIN ClientPolicies cp ON cp.policy_id = e.id
            WHERE e.id = ${policy_id} AND cp.policy_type_id = ${policy_type_id} AND cp.client_id NOT IN (${slaveClientIds})

            UNION

            SELECT e.id as policy_id, cp.id as client_policy_id, cp.policy_type_id as policy_type
            FROM DevicePolicies e 
            INNER JOIN ClientGroupPolicies cp ON cp.policy_id = e.id
            WHERE e.id = ${policy_id} AND cp.policy_type_id = ${policy_type_id}
          `;

          let results = (await models.sequelize.query(query))[0];

          if (results.length === 0) {
            await models.DevicePolicies.destroy({
              where: {
                id: policy_id
              },
              transaction
            });
            deleteFlag = true;
          }
        }
        if (policy_type_id === 4) {
          let query = `
            SELECT e.id as policy_id, cp.id as client_policy_id, cp.policy_type_id as policy_type
            FROM SoftwarePolicies e 
            INNER JOIN ClientPolicies cp ON cp.policy_id = e.id
            WHERE e.id = ${policy_id} AND cp.policy_type_id = ${policy_type_id} AND cp.client_id NOT IN (${slaveClientIds})

            UNION

            SELECT e.id as policy_id, cp.id as client_policy_id, cp.policy_type_id as policy_type
            FROM SoftwarePolicies e 
            INNER JOIN ClientGroupPolicies cp ON cp.policy_id = e.id
            WHERE e.id = ${policy_id} AND cp.policy_type_id = ${policy_type_id}
          `;

          let results = (await models.sequelize.query(query))[0];

          if (results.length === 0) {
            await models.SoftwarePolicies.destroy({
              where: {
                id: policy_id
              },
              transaction
            });
            deleteFlag = true;
          }
        }
        if (policy_type_id === 5) {
          let query = `
            SELECT e.id as policy_id, cp.id as client_policy_id, cp.policy_type_id as policy_type
            FROM SoftwareProcessPolicies e 
            INNER JOIN ClientPolicies cp ON cp.policy_id = e.id
            WHERE e.id = ${policy_id} AND cp.policy_type_id = ${policy_type_id} AND cp.client_id NOT IN (${slaveClientIds})

            UNION

            SELECT e.id as policy_id, cp.id as client_policy_id, cp.policy_type_id as policy_type
            FROM SoftwareProcessPolicies e 
            INNER JOIN ClientGroupPolicies cp ON cp.policy_id = e.id
            WHERE e.id = ${policy_id} AND cp.policy_type_id = ${policy_type_id}
          `;

          let results = (await models.sequelize.query(query))[0];

          if (results.length === 0) {
            await models.SoftwareProcessPolicies.destroy({
              where: {
                id: policy_id
              },
              transaction
            });
            deleteFlag = true;
          }
        }
        if (policy_type_id === 6) {
          let query = `
            SELECT e.id as policy_id, cp.id as client_policy_id, cp.policy_type_id as policy_type
            FROM DataVolumePolicies e 
            INNER JOIN ClientPolicies cp ON cp.policy_id = e.id
            WHERE e.id = ${policy_id} AND cp.policy_type_id = ${policy_type_id} AND cp.client_id NOT IN (${slaveClientIds})

            UNION

            SELECT e.id as policy_id, cp.id as client_policy_id, cp.policy_type_id as policy_type
            FROM DataVolumePolicies e
            INNER JOIN ClientGroupPolicies cp ON cp.policy_id = e.id
            WHERE e.id = ${policy_id} AND cp.policy_type_id = ${policy_type_id}
          `;

          let results = (await models.sequelize.query(query))[0];

          if (results.length === 0) {
            await models.DataVolumePolicies.destroy({
              where: {
                id: policy_id
              },
              transaction
            });
            deleteFlag = true;
          }
        }

        // delete only if no other client has the policy assigned
        if (deleteFlag) {
          if (type === 1) {
            await models.ServicePolicies.destroy({
              where: {
                id: id
              },
              transaction
            });
          } else {
            await models.ServiceGroupPolicies.destroy({
              where: {
                id: id
              },
              transaction
            });
          }
        }
      }
    }

    // to delete default bridge entitlement
    const clientPolicy = await models.ClientPolicies.findAll(
      {
        where: {
          client_id: bridge.clientId,
          policy_type_id: 1
        }
      },
      { transaction }
    );

    if (clientPolicy) {
      for (const policy of clientPolicy) {
        const entitlement = await models.Entitlements.findOne({
          where: {
            id: policy.policy_id,
            access_start_time: null,
            access_end_time: null
          },
          transaction
        });

        const policyId = entitlement?.id;

        if (policyId) {
          await models.ClientPolicies.destroy({
            where: {
              policy_id: policyId,
              policy_type_id: 1
            },
            transaction
          });
          await models.ServicePolicies.destroy({
            where: {
              policy_id: policyId,
              policy_type_id: 1
            },
            transaction
          });
          await models.ClientGroupPolicies.destroy({
            where: {
              policy_id: policyId,
              policy_type_id: 1
            },
            transaction
          });
          await models.ServiceGroupPolicies.destroy({
            where: {
              policy_id: policyId,
              policy_type_id: 1
            },
            transaction
          });
          await models.Entitlements.destroy({
            where: { id: policyId },
            transaction
          });
        }
      }
    }

    await Promise.all([
      bridge.destroy({
        transaction
      }),
      models.ServerBridgeDeviceId.destroy({
        where: {
          serverBridgeId: bridge.id
        },
        transaction
      }),
      models.ServerBridgeUserId.destroy({
        where: {
          serverBridgeId: bridge.id
        },
        transaction
      }),
      models.Client.destroy({
        where: {
          id: {
            [Op.in]: [
              ...serverBridgeIds.map(x => x.client_id),
              ...serverBridgeUserIds.map(x => x.client_id)
            ]
          }
        },
        transaction
      }),
      publishClients(actionType.DELETED, {
        ids: [
          ...serverBridgeIds.map(x => x.client_id),
          ...serverBridgeUserIds.map(x => x.client_id)
        ],
        type: 'Server',
        data: {}
      }),
      publishClients(actionType.DELETED, {
        ids: [bridge.clientId],
        type: 'Server',
        data: {
          is_active: 0
        }
      })
    ]);

    const zipPath = `${installerPath}${bridge.type}-${bridge.name.replace(
      /\s/g,
      ''
    )}-${id}.zip`;
    ``;

    await fsExtra.remove(zipPath);

    await transaction.commit();

    insertIntoAdminLogs(
      getServerDeleteMessage(bridge.dataValues.name, bridge.type),
      req
    );

    return res.json(bridge);
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

/**
 * Toggle enabled value of specific Server Bridge
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */

module.exports.setEnable = async (req, res, next) => {
  const { id, value } = req.params;
  const transaction = await models.sequelize.transaction();

  try {
    const bridge = await models.ServerBridge.findOne({
      where: { id },
      include: [
        {
          model: models.Client,
          attributes: ['id', 'is_active']
        }
      ]
    });

    if (!bridge) {
      const type =
        bridge.type == 'iot-bridge'
          ? 'IoT Bridge'
          : bridge.type == 'client-bridge'
          ? 'Client Bridge'
          : bridge.type == 'b2b-bridge'
          ? 'B2B Bridge'
          : 'Server Bridge';
      throw new httpErrors.NotFound(`${type} not found`);
    }

    let serverBridgeIds;
    if (bridge.type == 'b2b-bridge') {
      serverBridgeIds = await models.ServerBridgeUserId.findAll({
        attributes: ['client_id', 'user_id'],
        where: {
          serverBridgeId: bridge.id
        },
        raw: true
      });
    } else {
      serverBridgeIds = await models.ServerBridgeDeviceId.findAll({
        attributes: ['client_id', 'device_id'],
        where: {
          serverBridgeId: bridge.id
        },
        raw: true
      });
    }

    await Promise.all([
      bridge.update({ isEnable: value }, { transaction }),
      bridge.Client.update(
        {
          is_active: value
        },
        { transaction }
      ),
      models.Client.update(
        {
          is_active: value
        },
        {
          where: {
            id: {
              [Op.in]: serverBridgeIds.map(x => x.client_id)
            }
          },
          transaction
        }
      ),
      publishClients(actionType.UPDATED, {
        ids: [bridge.clientId, ...serverBridgeIds.map(x => x.client_id)],
        type: 'Server',
        data: {
          is_active: value.toString() == 'true' ? 1 : 0
        }
      })
    ]);

    await transaction.commit();

    let logMessage;
    if (value === 'true') {
      logMessage = getServerEnableMessage(bridge.name, bridge.type);
    } else {
      logMessage = getServerDisableMessage(bridge.name, bridge.type);
    }

    insertIntoAdminLogs(logMessage, req);

    return res.json(bridge);
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

/**
 * Toggle enabled value of specific Server Bridge
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */

module.exports.getSingleBridge = async (req, res, next) => {
  const { id } = req.params;

  try {
    const bridge = await models.ServerBridge.findOne({
      where: {
        id
      },
      include: [
        {
          model: models.Client,
          attributes: ['id', 'device_os', 'name', 'user']
        },
        {
          model: models.DeviceInventories
        },
        {
          model: models.ServerBridgeUserId,
          attributes: ['id', 'user_id'],
          as: 'serverBridgeUserIds',
          include: [
            {
              model: models.LocalUsers
            }
          ]
        },
        {
          model: models.ServerBridgeDeviceId,
          as: 'serverBridgeIds',
          attributes: ['id', 'deviceId'],
          include: [
            {
              model: models.DeviceInventories
            }
          ]
        }
      ],
      attributes: {
        exclude: ['createdAt', 'updatedAt']
      }
    });

    console.log(bridge.serverBridgeIds.map(x => x.DeviceInventory));

    if (!bridge) {
      const type =
        bridge.type == 'iot-bridge'
          ? 'IoT Bridge'
          : bridge.type == 'client-bridge'
          ? 'Client Bridge'
          : bridge.type === 'b2b-bridge'
          ? 'B2B Bridge'
          : 'Server Bridge';
      throw new httpErrors.NotFound(`${type} not found`);
    }

    return res.json(bridge);
  } catch (error) {
    next(error);
  }
};

/**
 * Download the package from link
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */

module.exports.downloadBridgeFromLink = async (req, res, next) => {
  const { token } = req.params;

  try {
    const bridge = await models.ServerBridge.findOne({
      where: {
        email_token: token
      }
    });

    if (!bridge) {
      const type =
        bridge.type == 'iot-bridge'
          ? 'IoT Bridge'
          : bridge.type == 'client-bridge'
          ? 'Client Bridge'
          : 'Server Bridge';
      throw new httpErrors.NotFound(`${type} not found`);
    }

    const { id, name, email_token_expiry, email_token_used } = bridge;

    if (email_token_used) {
      throw new httpErrors.UnprocessableEntity(
        'Link is already used.please request for a new link'
      );
    }

    if (!moment().isBefore(email_token_expiry)) {
      throw new httpErrors.UnprocessableEntity(
        'Link is expired.please request for a new link'
      );
    }

    const zipPath = `${installerPath}${bridge.type}-${name.replace(
      /\s/g,
      ''
    )}-${id}.zip`;

    await bridge.update({ email_token_used: 1 });

    return res.download(zipPath);
  } catch (error) {
    next(error);
  }
};

/**
 * Download the package
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */

module.exports.downloadBridge = async (req, res, next) => {
  const { id } = req.params;
  let zipPath;

  try {
    zipPath = await makePackage(id);
    return res.download(zipPath);
  } catch (error) {
    if (zipPath) {
      await fsExtra.remove(zipPath);
    }
    console.log('---------------picking error------------------', error);
    return 'adadadad';
    // next(error);
  }
};

/**
 * Send the package
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */

module.exports.sendBridge = async (req, res, next) => {
  const { id } = req.params;
  let zipPath;
  let user;

  try {
    [zipPath, user] = await makePackage(id, true);

    const email_token =
      Math.random()
        .toString(36)
        .substring(2) +
      user.bridge.id +
      Math.random()
        .toString(36)
        .substring(2);

    await user.bridge.update({
      email_token_expiry: moment().add(1, 'hours'),
      email_token,
      email_token_used: 0
    });

    helper.sendInstallerEmail({
      recipient: user.email,
      emailSubject: 'Invisily Software Installation',
      recipientName: user.name,
      onPremiseDownloadLink: `${projectConfig.onPremApiUrl}/serverBridge/bridge/downloadPackage/${email_token}`,
      offPremiseDownloadLink: `${projectConfig.offPremApiUrl}/serverBridge/bridge/downloadPackage/${email_token}`,
      templateFileName: 'client-server_bridge_package.html',
      package_expiry_time: '1 hour'
    });

    return res.json({ status: true });
  } catch (error) {
    if (zipPath) {
      await fsExtra.remove(zipPath);
    }

    next(error);
  }
};

/**
 * Send the package
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */

module.exports.getPhysicalDevices = async (req, res, next) => {
  try {
    const { search = '', limit = 10, offset = 0 } = req.query;

    const devices = await models.DeviceInventories.paginate({
      attributes: ['id', 'userName', 'deviceTag', 'device_name', 'mac_address'],
      where: {
        device_type: {
          [Op.in]: ['Physical Server', 'ARM Platform']
        },
        [Op.or]: {
          userName: {
            [Op.like]: `%${search.trim()}%`
          },
          deviceSerialNumber: {
            [Op.like]: `%${search.trim()}%`
          },
          deviceTag: {
            [Op.like]: `%${search.trim()}%`
          },
          mac_address: {
            [Op.like]: `%${search.trim()}%`
          },
          device_name: {
            [Op.like]: `%${search.trim()}%`
          }
        }
      },
      limit: +limit,
      offset: +offset
    });

    res.json(devices.data);
  } catch (error) {
    next(error);
  }
};

/**
 * Send the package
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */

module.exports.getDevices = async (req, res, next) => {
  try {
    const { type = '', search = '', offset = 0, limit = 10 } = req.query;

    const device_type =
      type == 'server-bridge'
        ? 'Server Bridge Client'
        : type == 'iot-bridge'
        ? 'IoT Bridge Client'
        : 'Thin Client (Bridge Client)';

    const devices = await models.DeviceInventories.paginate({
      attributes: ['id', 'userName', 'deviceTag', 'device_name', 'mac_address'],
      include: [
        {
          model: models.Client,
          attributes: ['device_id'],
          required: false
        }
      ],
      where: {
        '$Client.device_id$': null,
        device_type: device_type,
        [Op.or]: {
          userName: {
            [Op.like]: `%${search.trim()}%`
          },
          deviceSerialNumber: {
            [Op.like]: `%${search.trim()}%`
          },
          deviceTag: {
            [Op.like]: `%${search.trim()}%`
          },
          mac_address: {
            [Op.like]: `%${search.trim()}%`
          },
          device_name: {
            [Op.like]: `%${search.trim()}%`
          }
        }
      },
      limit: +limit,
      offset: +offset
    });

    res.json(devices.data);
  } catch (error) {
    next(error);
  }
};

/**
 * make package if does not exist
 * @param {Number} id
 * @param {Boolean} isEmail
 */

async function makePackage(id, isEmail = false) {
  let name;
  let email;

  let bridge = await models.ServerBridge.findAll({
    where: { id },
    attributes: ['id', 'name', 'type'],
    include: [
      {
        model: models.Client,
        attributes: ['Email', 'Name', 'device_os', 'id']
      }
    ]
  });

  if (!bridge || !bridge.length)
    throw new httpErrors.NotFound('Bridge not found');

  bridge = bridge[0];

  let zipPath;

  if (bridge.type == 'b2b-bridge') {
    const rdiType = 'rdi-bridge'
    zipPath = `${installerPath}${rdiType}-${bridge.name.replace(
      /\s/g,
      ''
    )}-${id}.zip`;
  } else {
    zipPath = `${installerPath}${bridge.type}-${bridge.name.replace(
      /\s/g,
      ''
    )}-${id}.zip`;
  }

  await fsExtra.ensureDir(installerPath);

  let device_os =
    bridge.type == 'iot-bridge' ? 'ubuntuserver20armv8' : 'ubuntuserver18';

  if (bridge.type == 'b2b-bridge') {
    device_os = bridge.Client.dataValues.device_os;
  }

  if(bridge.type == 'rbi-bridge') {
    device_os = bridge.Client.dataValues.device_os;
  }

  const clientInstallerPath = `/var/local/installers/client/input/${device_os}/installer.deb`;

  const [zipExists, installerExists] = await Promise.all([
    fsExtra.pathExists(zipPath),
    fsExtra.pathExists(clientInstallerPath)
  ]);

  if (isEmail) {
    const clientId = bridge.Client.dataValues.id;
    const myClient = await models.Client.findByPk(clientId);
    let myLocalUser = { name: bridge.Client.dataValues.Name };
    if (myClient) {
      myLocalUser = await myClient.getLocalUser();
    }
    name = myLocalUser.name;
    email = bridge.Client.dataValues.Email;
  }

  if (zipExists) {
    return !isEmail ? zipPath : [zipPath, { email, name, bridge }];
  }

  let os =
    bridge.type == 'iot-bridge' ? 'Ubuntu Server (ARMv8)' : 'Ubuntu Server';

  if (bridge.type == 'b2b-bridge') {
    os = bridge.Client.dataValues.device_os;
  }

  if(bridge.type == 'rbi-bridge') {
    os = bridge.Client.dataValues.device_os;
  }

  if (!installerExists) {
    throw new httpErrors.NotFound(
      `${os} Installer does not exist.Please upload first ${os} package`
    );
  }

  await createZip(zipPath, bridge.type, bridge.name, os);

  return !isEmail ? zipPath : [zipPath, { email, name, bridge }];
}

/**
 * create zip of given files
 * @param {String} zipPath
 * @param {String} macAddresses
 * @return {Promise}
 */

async function createZip(zipPath, type = 'server-bridge', bridgeName, os) {
  return new Promise((resolve, reject) => {
    let output = fs.createWriteStream(zipPath);

    // const script = type == 'iot-bridge' ? 'install_invisily_iot_bridge.sh' : 'install_invisily_thin_client_bridge.sh';

    // const guide = type == 'iot-bridge' ? 'Installation_Guide_IoT_Bridge.txt' : 'Installation_Guide_Thin_Client_Bridge.txt';

    const script =
      type == 'iot-bridge'
        ? 'install_invisily_iot_bridge.sh'
        : type == 'client-bridge'
        ? 'install_invisily_thin_client_bridge.sh'
        : type == 'b2b-bridge'
        ? 'install_invisily_rdi_bridge.sh'
        : type == 'rbi-bridge'
        ? 'install_invisily_rbi_bridge.sh'
        : 'install_invisily_server_bridge.sh';

    const guide =
      type == 'iot-bridge'
        ? 'Installation_Guide_IoT_Bridge.txt'
        : type == 'client-bridge'
        ? 'Installation_Guide_Thin_Client_Bridge.txt'
        : type === 'b2b-bridge'
        ? 'Installation_Guide_RDI_Bridge.txt'
        : type === 'rbi-bridge'
        ? 'Installation_Guide_RBI_Bridge.txt'
        : 'Installation_Guide_Server_Bridge.txt';

    const shFilePath = path.join(__dirname, `../scripts/${script}`);
    const guideFilePath = path.join(__dirname, `../scripts/${guide}`);
    const serviceListFilePath = path.join(
      __dirname,
      '../scripts/service_list.html'
    );

    const deviceProfilerPath = path.join(
      __dirname,
      `../scripts/device-profiler.zip`
    );

    let archive = archiver('zip', {
      zlib: { level: 9 }
    });

    archive.pipe(output);

    output.on('close', () => {
      resolve(zipPath);
    });

    archive.on('error', error => {
      reject(error);
    });

    let device_os =
      type == 'iot-bridge' ? 'ubuntuserver20armv8' : 'ubuntuserver18';

    if (type == 'b2b-bridge' || type == 'rbi-bridge') {
      device_os = os;
    }

    const clientInstallerPath = `/var/local/installers/client/input/${device_os}/installer.deb`;

    if (type === 'b2b-bridge' || type === 'rbi-bridge') {
      const configBuffer = Buffer.from(`name=${bridgeName}`, 'utf-8');
      var ReadableStream = require('stream');
        var zipInputStream = new ReadableStream.Readable();
        zipInputStream.push(configBuffer);
        zipInputStream.push(null);
  
      archive.append(zipInputStream, { name: 'settings.cfg' });
    }

    archive.file(`${clientInstallerPath}`, { name: 'installer.deb' });

    archive.file(shFilePath, { name: script });
    if (type === 'b2b-bridge') {
      const b2bBridgeZip = path.join(__dirname, `../scripts/RdiBridge.zip`);
      archive.file(`${b2bBridgeZip}`, { name: 'RdiBridge.zip' });
    }

    if (type === 'rbi-bridge') {
      const rbiBridgeZip = path.join(__dirname, `../scripts/RbiBridge.zip`);
      archive.file(`${rbiBridgeZip}`, { name: 'RbiBridge.zip' });
    }

    // archive.append(macAddresses, { name: 'device_binding.conf' });
    if (type !== 'b2b-bridge' && type !== 'rbi-bridge') {
      archive.file(deviceProfilerPath, { name: 'device-profiler.zip' });
    }

    archive.file(guideFilePath, {
      name: guide
    });

    archive.file(serviceListFilePath, { name: 'service_list.html' });

    archive.finalize();
  });
}

const mergeUserAndBridgeNames = (clientName, bridgeName) => {
  return '{' + clientName + '}@{' + bridgeName + '}';
};
