const moment = require('moment');
const models = require('../models');
const httpErrors = require('http-errors');
const { actionType, publishServiceGroups } = require('../routes/common/event-publisher');
const { getServiceGroupCreateMessage, getServiceGroupEditMessage, getServiceGroupDeleteMessage } = require('../config/adminLogs.config');
const { insertIntoAdminLogs } = require('../routes/common/adminLogs');

const checkDuplicateName = async (name, id) => {
  const whereClause = {
    name
  };
  if (id) {
    whereClause.id = {
      $ne: id
    };
  }
  const group = await models.ServiceGroup.find({
    where: whereClause,
    attributes: ['id']
  });
  if (!group) {
    return Promise.resolve();
  } else {
    let error = new Error('Service group with this name already exist');
    error.status = 400;
    return Promise.reject(error);
  }
};

module.exports = {
  getAllGroups: async (req, res, next) => {
    try {

      let {
        limit = 10,
        offset = 0,
        search = '',
        sort = 'serviceCount',
        sortDir = 'desc'
      } = req.query;

      const allowedSortValues = ['serviceCount', 'name', 'id']

      sort = allowedSortValues.includes(sort) ? sort: allowedSortValues[0]

      fetchQuery = `
        SELECT ServiceGroup.id, ServiceGroup.name, count(GroupServices.groupId) AS serviceCount,
        ServiceGroup.description, ServiceGroup.updatedAt
        FROM ServiceGroups AS ServiceGroup
        LEFT OUTER JOIN GroupServices AS GroupServices ON ServiceGroup.id = GroupServices.groupId
        WHERE ServiceGroup.name LIKE :search
        GROUP BY ServiceGroup.id
        ORDER BY :sort :sortDir
        LIMIT :offset, :limit
      `;

      const [data, groups] = await Promise.all([
        models.sequelize.query(fetchQuery, {
          replacements: {
            search: `%${search.trim()}%`,
            offset: +offset,
            limit: +limit,
            sort : sort,
            sortDir : sortDir,
          },
          type: models.sequelize.QueryTypes.SELECT
        }),
        models.ServiceGroup.findAll({
          attributes: [
            'id',
            [
              models.sequelize.fn(
                'count',
                models.sequelize.col('GroupServices.groupId')
              ),
              'serviceCount'
            ],
          ],
          include: [
            {
              model: models.GroupService,
              attributes: []
            }
          ],
          group: [
            'ServiceGroup.id'
          ],
          where: {
            name: {
              [Op.like]: `%${search.trim()}%`
            }
          },
          order: [
            [
              models.sequelize.literal('serviceCount'), 'DESC'
            ]
          ]
        })
      ]);

      // const groups = await models.ServiceGroup.scope('getAll').findAll();
      return res.json({
        data,
        recordsTotal: groups.length
      });
    } catch (error) {
      next(error);
    }
  },

  getGroupServicesWithPagination: async (req, res, next) => {
    try{
      const { groupId } = req.params
      const {
        offset = 0,
        limit = 10,
        sort='Name',
        dir='asc',
        searchKey=''
      } = req.query

      const servicesOptions = {
        attributes: ['id', 'Name', 'port', 'protocol', 'ipv4_address', 'type', 'dummy_port', 'mappedurl'],
        where: {}
      }

      if(searchKey.trim()){
        servicesOptions.where.Name = {
          [Op.like]: `%${searchKey.trim()}%`
        }
      }

      const group = await models.ServiceGroup.findOne({
        where: {
          id: groupId
        },
        order: [
          [models.Service, sort, dir]
        ],
        include: [
          { 
            model: models.Service,
            ...servicesOptions,
            required: false,
          }
        ]
      })

      if (!group) throw new httpErrors.NotFound('Group not found');

      const services = group.Services.slice(parseInt(offset) * parseInt(limit), (parseInt(offset) * parseInt(limit)) + parseInt(limit))
      return res.json({
        total: group.Services.length,
        data: services,
        name: group.name
      });




    }
    catch(e){
      next(e)
    }
  },

  getAllMacthingServices: async (req, res, next) => {
    try {
      const groupId = +req.params.groupId;
      const { searchServiceTerm } = req.params;
      const term =
        searchServiceTerm === '__SERVICES__'
          ? searchServiceTerm
          : new Buffer(searchServiceTerm, 'base64').toString('ascii');
      const services = await models.Service.scope({
        method: ['GroupServices', groupId, term]
      }).findAll();
      return res.json(services);
    } catch (error) {
      next(error);
    }
  },

  addService: async (req, res, next) => {
    const groupId = parseInt(req.params.groupId, 10);
    const serviceId = parseInt(req.params.serviceId, 10);
    try {
      const group = await models.ServiceGroup.find({
        where: {
          id: groupId
        }
      });
      await group.addService(serviceId);
      
      await publishServiceGroups(actionType.UPDATED, {
        id: groupId,
        new_ids: [serviceId],
        del_ids: []
      });
      
      return res.send('');
    } catch (error) {
      next(error);
    }
  },
  deleteService: async (req, res, next) => {
    const groupId = parseInt(req.params.groupId, 10);
    const serviceId = parseInt(req.params.serviceId, 10);

    try {
      const group = await models.ServiceGroup.find({
        where: {
          id: groupId
        }
      });
      await group.removeService(serviceId);

      await publishServiceGroups(actionType.UPDATED, {
        id: groupId,
        new_ids: [],
        del_ids: [serviceId]
      });

      return res.send('');
    } catch (error) {
      next(error);
    }
  },
  editGroup: async (req, res, next) => {
    const groupId = parseInt(req.params.id, 10);
    let { 
      name, 
      description, 
      serviceIds, 
      appendServices = false ,
      addAllServices = false
    } = req.body;
    try {

      serviceIds = serviceIds.map(service => parseInt(service))
      if(addAllServices === true) {
        const allServices = await models.Service.findAll({
          attributes: ['id']
        })
        serviceIds = allServices.map(service => service.id)
      }

      await checkDuplicateName(name, groupId);
      const group = await models.ServiceGroup.find({
        where: {
          id: groupId
        }
      });
      if (!group)
        return res.status(400).send('Group with this id does not exist');

      const transaction = await models.sequelize.transaction();
      await group.update({ name, updatedAt: moment(), description }, { transaction });  
      const allServices = await models.Service.findAll({
        where: {
          id: serviceIds
        },
        attirbutes: ['id', 'is_persistent']
      });
      const oldServices = await group.getServices()

      let serviceIdsToCheck = allServices.map(service => service.id);
      if(appendServices) {
        serviceIdsToCheck = [
          ...serviceIdsToCheck,
          ...oldServices.map(service => service.id)
        ]
      }
      const persistentServiceCheck = await models.Service.findAll({
        where: {
          id: serviceIdsToCheck
        },
        attributes: ['is_persistent'],
        group: ['is_persistent']
      })

      if(persistentServiceCheck.length > 1) {
        throw new httpErrors.BadRequest('Mix of persistent and non persistent services not allowed')
      }
      
      let is_persistent = (persistentServiceCheck[0] && persistentServiceCheck[0].is_persistent) || false;
      
      await group.update({ name, updatedAt: moment(), description, is_persistent }, { transaction });  
      
      
      if(!appendServices) {
        // if append Services is true then old services will not be replaced by new and vice versa
        await Promise.all(oldServices.map(async service => {
          await group.removeService(service.dataValues.id, { transaction });
        }))
      }
      
      await Promise.all(allServices.map(async service => {
        await group.addService(service.id, { transaction });
      }))

      await publishServiceGroups(actionType.UPDATED, {
        id: groupId,
        name: name,
        new_ids: [],
        del_ids: []
      });
      
      await transaction.commit()

      insertIntoAdminLogs(
        getServiceGroupEditMessage(name),
        req
      )
      
      return res.json(`Group '${name}' updated successfully.`);
    } catch (error) {
      next(error);
    }
  },
  getGroupDetail: async (req, res, next) => {
    const groupId = parseInt(req.params.id, 10);
    try {
      const group = await models.ServiceGroup.findOne({
        where: {
          id: groupId
        },
        include: [
          { model: models.Service, required: false }
        ]
      })
      if (!group)
        return res.status(400).send('Group with this id does not exist');
      
      return res.json(group);
    } catch (error) {
      next(error);
    }
  },
  addGroup: async (req, res, next) => {
    let { name, description, serviceIds, addAllServices } = req.body;
    try {
      await checkDuplicateName(name);
      if(addAllServices && (!serviceIds || serviceIds.length < 1)) {
        // adding all current services to this group
        const services = await models.Service.findAll({
          where: { },
          attributes: ['id']
        })
        serviceIds = services.map(service => service.id)
      }
      const transaction = await models.sequelize.transaction();
      const persistentServiceCheck = await models.Service.findAll({
        where: {
          id: serviceIds
        },
        attributes: ['is_persistent'],
        group: ['is_persistent']
      })

      if(persistentServiceCheck.length > 1) {
        throw new httpErrors.BadRequest('Mix of persistent and non persistent services not allowed')
      }
      
      let is_persistent = persistentServiceCheck[0] ? persistentServiceCheck[0].is_persistent: false;

      const group = await models.ServiceGroup.create({ name, description, is_persistent }, { transaction });

      const allServices = await models.Service.findAll({
        where: {
          id: serviceIds
        }
      });
      await Promise.all(allServices.map(async service => {
        await group.addService(service.id, { transaction });
      }))

      await publishServiceGroups(actionType.ADDED, {
        id: group.id,
        name: group.name,
        new_ids: [],
        del_ids: []
      });

      await transaction.commit();

      insertIntoAdminLogs(
        getServiceGroupCreateMessage(name),
        req
      )
      
      return res.json({ group, message: 'Service Group added successfully' });
    } catch (error) {
      next(error);
    }
  },
  deleteGroup: async (req, res, next) => {
    try {
      const id = parseInt(req.params.id);

      console.log('=============id===========', id);
      const groups = await models.ServiceGroup.findAll({
        where: {id: id},
        attributes: ['name']
      })

      console.log('grouip=================',groups[0].dataValues.name);
      await models.ServiceGroup.destroy({
        where: { 
          id 
        }
      });
      await models.GroupService.destroy({
        where: {
          groupId: id
        }
      });

      await publishServiceGroups(actionType.DELETED, {
        ids: [id],
        new_ids: [],
        del_ids: []
      });

      insertIntoAdminLogs(
        getServiceGroupDeleteMessage(groups[0].dataValues.name),
        req
      )

      res.json(`Group deleted successfully.`);
    } catch (error) {
      next(error);
    }
  },
  getAvailableGroups: async (req, res, next) => {
    try{ 
      const groups = await models.ServiceGroup.findAll({
        where: { },
        attributes: ['id', 'name', 'description']
      })
      res.status(200).json({
        data: groups
      })
    }
    catch(e) {
      next(e)
    }
  },
  bulkDeleteGroups: async(req, res, next) => {
  try {
    const { groupIds, excludeGroups } = req.body
    const { allSelected, search } = req.query

    const options =  {
      where: { }
    }

    if(allSelected == 'true' ) {
      options.where.id = {
        [Op.not]: excludeGroups
      }

      if(search.trim()) { 
        options.where.name  = {
          [Op.like]: `%${search.trim()}%`
        }
      }

    } else {
      options.where.id = groupIds
    }

    const groupsToDelete = await models.ServiceGroup.findAll(options) 
    const groupsToDeleteIds = groupsToDelete.map(group => group.id)
    await models.ServiceGroup.destroy(options) 

    await publishServiceGroups(actionType.DELETED, {
      ids: groupsToDeleteIds,
      new_ids: [],
      del_ids: []
    });

    for (const group of groupsToDelete) {
      insertIntoAdminLogs(
        getServiceGroupDeleteMessage(group.dataValues.name),
        req
      );
    }
    

    res.status(200).json({message: 'Service groups deleted'})


  } catch(e) {
    next(e)
  }
}
};
