var path = require('path');
var httpErrors = require('http-errors');
let language = require('../routes/common/language');
var models = require('../models');
let appDir = path.dirname(require.main.filename);
appDir = appDir.substring(0, appDir.lastIndexOf('/'));
var dirSyncManager = require('../routes/common/dirsync-manager');
var moment = require('moment');

const {
  actionType,
  publishSIEM,
  publishGatewaySettings,
  publishClientSettings,
  publishMaxClients,
  publishLogOut,
  publishGatewaySubnet,
  publishCoreAutoUpdate,
  publishControllerConfiguration,
  publishGatewaySoftware,
  publishControllerSoftware,
  publishProfilingConfig
} = require('../routes/common/event-publisher');
const { sendOTPQr, sendOtpQrBulkUsers } = require('./clients');
const device_validation = require('../cronjobs/device_validation');
const ldap_sync = require('../cronjobs/sync-ldap');

const {
  getSettingsUpdateMessage,
  getClientSettingUpdateMessage,
  getCoreSettingUpdateMessage
} = require('../config/adminLogs.config');

const { insertIntoAdminLogs } = require('../routes/common/adminLogs');

const getSettings = async (req, res, next) => {
  try {
    const user = req.user;

    const options = {
      order: [['id', 'DESC']],
      limit: 1
    };

    const settingOptions = {
      attributes: { exclude: ['smtp_password'] },
      ...options
    };

    const licenseOptions = {
      attributes: ['mfa_feature', 'darkening_feature'],
      ...options
    };

    const ldapOptions = {
      attributes: ['id'],
      ...options,
      where: {}
    };

    const [settings, license, ldap] = await Promise.all([
      models.Settings.findAll(settingOptions),
      models.License.findOne(licenseOptions),
      models.Ldap.findOne(ldapOptions)
    ]);

    let getUserSetting = await models.UserTimeSetting.findOne({
      where: {
        user_id: user.id
      }
    });

    if (!getUserSetting) {
      getUserSetting = await models.UserTimeSetting.create({
        user_id: user.id,
        time_type: 1,
        timezone: ''
      });
    }

    res.json({
      data: settings,
      license: license,
      ldap,
      status: true,
      message: 'Data Fetched',
      timesetting: getUserSetting,
    });
  } catch (error) {
    next(error);
  }
};

const getTimeSettings = async (req, res, next) => {
  try {
    const user = req.user;

    let getUserSetting = await models.UserTimeSetting.findOne({
      where: {
        user_id: user.id
      }
    });

    if (!getUserSetting) {
      getUserSetting = await models.UserTimeSetting.create({
        user_id: user.id,
        time_type: 1,
        timezone: ''
      });
    }
  
    const isDarkMode = await models.User.findOne({
      where: {
        id: user.id
      },
    });

    res.json({
      status: true,
      message: 'Data Fetched',
      timesetting: getUserSetting,
      isDark : isDarkMode.is_dark
    });
  } catch (error) {
    next(error);
  }
};
const getTimeZones = async (req, res, next) => {
  try {
    const timeFormats = await models.TimeFormats.findAll({});

    res.json({
      timeFormats
    });
  } catch (error) {
    next(error);
  }
};

const getAddressSpaceSettings = async (req, res, next) => {
  try {
    const addressClasses = await models.AddressClasses.findAll({});
    const subnetIPList = await models.GatewaySubnetIPList.findAll({});
    const activeConfig = await models.ActiveAddressSpaceConfig.findOne();
    const gatewayCount = await models.Gateway.count({
      where: {
        is_unique: 1
      }
    });

    res.json({
      addressClasses,
      activeConfig,
      subnetIPList,
      gatewayCount
    });
  } catch (error) {
    next(error);
  }
};

const updateAddressSpaceSettings = async (req, res, next) => {
  try {
    const {
      addressClass,
      supportedGateways,
      uniqueGateways,
      exceptionList = []
    } = req.body.addressSpacePayload;

    let changed = false;
    const selectedClass = await models.AddressClasses.findOne({
      where: {
        class_name: addressClass.value
      }
    });

    const activeConfigPresent = await models.ActiveAddressSpaceConfig.count({});

    if (activeConfigPresent == 1) {
      const activeAddressSpaceConfig = await models.ActiveAddressSpaceConfig.findOne(
        { where: {} }
      );
      // console.log('values to upate', selectedClass)
      const dedicatedGateways = Math.round(Math.log(supportedGateways) + 1);
      const totalSubnetBits = Math.round(
        Math.log(supportedGateways) + selectedClass.net_mask_bits + 1
      );

      await activeAddressSpaceConfig.update({
        active_class: selectedClass.id,
        required_gateways: parseInt(uniqueGateways),
        supported_gateways: supportedGateways,
        dedicated_gateway_bits: dedicatedGateways,
        total_subnet_bits: totalSubnetBits
      });

      if (
        activeAddressSpaceConfig._changed &&
        Object.keys(activeAddressSpaceConfig._changed).length > 0
      ) {
        changed = true;
      }
    } else {
      await models.ActiveAddressSpaceConfig.create({
        active_class: selectedClass.id,
        required_gateways: uniqueGateways,
        supported_gateways: supportedGateways,
        dedicated_gateway_bits: Math.log(supportedGateways) + 1,
        total_subnet_bits:
          Math.log(supportedGateways) + selectedClass.net_mask_bits + 1,
        updatedAt: new Date()
      });
      changed = true;
    }

    let oldGatewaysSubnets = await models.GatewaySubnetIPList.findAll({
      attributes: ['subnet_ip', 'excluded']
    });

    let normalizedOldGateways = {};

    oldGatewaysSubnets.forEach(gatewaySubnet => {
      normalizedOldGateways[gatewaySubnet.subnet_ip] = gatewaySubnet.excluded;
    });

    await models.GatewaySubnetIPList.destroy({
      where: {},
      truncate: true
    });

    const newNormalizedGateways = {};

    const subnetIPList = [];
    exceptionList.forEach(ip => {
      newNormalizedGateways[ip.ip] = ip.excluded;
      subnetIPList.push({
        subnet_ip: ip.ip,
        excluded: ip.excluded ? 1 : 0,
        createdAt: new Date(),
        updatedAt: new Date()
      });
    });

    await models.GatewaySubnetIPList.bulkCreate(subnetIPList);

    Object.keys(normalizedOldGateways).forEach(subnet => {
      if (newNormalizedGateways[subnet] !== normalizedOldGateways[subnet]) {
        changed = true;
      }
    });

    Object.keys(newNormalizedGateways).forEach(subnet => {
      if (normalizedOldGateways[subnet] !== newNormalizedGateways[subnet]) {
        changed = true;
      }
    });

    const consideredSubnets = exceptionList.filter(ip => ip.excluded == false);

    const gateways = await models.Gateway.findAll({
      where: {
        is_unique: 1
      }
    });

    if (gateways.length <= consideredSubnets.length) {
      for (let g = 0; g < gateways.length; g++) {
        const gatewayClones = await models.Gateway.findAll({
          where: {
            is_unique: 0,
            subnet_ip: gateways[g].subnet_ip
          }
        });

        gateways[g].subnet_ip = consideredSubnets[g].ip;

        await gateways[g].save();
        if (
          gateways[g]._changed &&
          Object.keys(gateways[g]._changed).length > 0
        ) {
          changed = true;
        }

        for (let c = 0; c < gatewayClones.length; c++) {
          gatewayClones[c].subnet_ip = gateways[g].subnet_ip;
          await gatewayClones[c].save();
          if (
            gatewayClones[c]._changed &&
            Object.keys(gatewayClones[c]._changed).length > 0
          ) {
            changed = true;
          }
        }
      }
    } else {
      throw new httpErrors.Conflict(
        'Subnet count is less than Unique Gateway count.'
      );
    }

    if (changed) {
      await publishGatewaySubnet(actionType.UPDATED, {
        ids: [],
        data: {}
      });
    }

    res.json({
      success: true,
      message: 'Settings added Successfully.'
    });
  } catch (error) {
    next(next);
  }
};

const mfaSettings = async (req, res, next) => {
  try {
    let mfa = 0;
    let otp = 0;
    let biometrics = 0;
    let response = {};

    const { clientID } = req.body;

    const options = {
      order: [['id', 'DESC']],
      limit: 1
    };

    const licenseOptions = {
      attributes: ['mfa_feature'],
      ...options
    };

    const settingOptions = {
      ...options
    };

    const license = await models.License.findOne(licenseOptions);
    if (!license || !license.mfa_feature) {
      const clientOptions = {
        where: {
          agent_id: clientID
        }
      };
      const client = models.Client.findOne(clientOptions);
      const userName = client.user;
      response.username = userName;
      response.success = true;
      response.mfa = 0;
      response.mfa_mode_otp = 0;
      response.mfa_mode_biometrics = 0;
      return res.json(response);
    }

    const settings = await models.Settings.findAll(settingOptions);

    if (settings[0].mfa_server_state === true) {
      mfa = 1;
    } else {
      mfa = 0;
    }
    if (settings[0].otp_state === true) {
      otp = 1;
    } else {
      otp = 0;
    }
    if (settings[0].biometrics_state === true) {
      biometrics = 1;
    } else {
      biometrics = 0;
    }

    response.mfa = mfa;
    response.mfa_mode_otp = otp;
    response.mfa_mode_biometrics = biometrics;

    const clientOptions = {
      where: {
        agent_id: clientID
      }
    };

    const client = await models.Client.findOne(clientOptions);

    const userName = client.user;
    response.username = userName;
    response.success = true;
    res.json(response);
  } catch (error) {
    next(error);
  }
};

const createNewSettings = async (req, res, next) => {
  try {
    const { setting } = req.body;

    const isSettingExist = await models.Settings.findOne();

    if (isSettingExist) {
      throw new httpErrors.Conflict('Settings already exist.');
    }

    const settings = await models.Settings.create(setting);

    if (!settings) {
      throw new httpErrors.BadGateway('Failed to create settings.');
    }

    const cron = req.app.get('cron');

    let result;
    if (setting.dirsync_enabled) {
      result = await dirSyncManager.schedule(
        cron,
        setting.dirsync_time,
        setting.dirclean_time
      );
    } else {
      result = await dirSyncManager.destroy(cron);
    }

    if (!result) {
      throw new httpErrors.BadGateway('Failed to create settings.');
    }

    res.json({
      success: true,
      message: 'Settings added Successfully.'
    });
  } catch (error) {
    next(next);
  }
};

const updateSettings = async (req, res, next) => {
  try {
    const {
      dirsync_enabled,
      dirsync_time,
      session_expiry,
      password_expiry_days,
      gapiKey,
      date_format,
      admin_mfa,
      smtp_user,
      smtp_password,
      smtp_service,
      smtp_from,
      package_link_expiry,
      purge_allowed,
      auto_purge_days,
      smtp_host,
      smtp_port,
      isSettingsSet,
      isFirstLogin,
      recaptcha_site_key,
      user_time_type,
      timezone,
      oauth_config,
      oauth_client_id,
      oauth_client_secret,
      oauth_tenant_id,
      oauth_refresh_token,
      oauth_from
    } = req.body.setting;

    const settings = {
      dirsync_enabled,
      dirsync_time,
      session_expiry,
      password_expiry_days,
      gapiKey,
      date_format,
      admin_mfa,
      smtp_user,
      smtp_password,
      smtp_service,
      smtp_from,
      package_link_expiry,
      purge_allowed,
      auto_purge_days,
      smtp_host,
      smtp_port,
      isSettingsSet,
      isFirstLogin,
      recaptcha_site_key,
      oauth_config,
      oauth_client_id,
      oauth_client_secret,
      oauth_tenant_id,
      oauth_refresh_token,
      oauth_from
    };

    const user = req.user;
    if (!settings.smtp_password) {
      delete settings.smtp_password;
    }

    settings.is_ldap_syncing = false;

    const options = {
      where: {
        id: req.params.id
      }
    };
    const setting = await models.Settings.findOne(options);

    if (!setting) {
      throw new httpErrors.NotFound('Settings not found');
    }
    // Code to Run on AD sync when dirsync_enabled setItemis update from false to true
    let is_AD_sync = !setting.dirsync_enabled && settings.dirsync_enabled;

    const [data] = await Promise.all([
      setting.update(settings),
      new Promise((resolve, reject) => {
        const cron = req.app.get('cron');

        if (setting.dirsync_enabled) {
          dirSyncManager
            .schedule(cron, setting.dirsync_time, setting.dirclean_time)
            .then(resolve)
            .catch(reject);
        } else {
          dirSyncManager
            .destroy(cron)
            .then(resolve)
            .catch(reject);
        }
      })
    ]);

    if (!data) {
      throw new httpErrors.BadGateway('Failed updating the settings.');
    }

    const getUserSetting = await models.UserTimeSetting.findOne({
      where: {
        user_id: user.id
      }
    });

    if (getUserSetting) {
      await models.UserTimeSetting.update(
        {
          time_type: user_time_type,
          timezone: user_time_type == 3 ? timezone : ''
        },
        {
          where: {
            user_id: user.id
          }
        }
      );
    } else {
      await models.UserTimeSetting.create({
        user_id: user.id,
        time_type: user_time_type,
        timezone: user_time_type == 3 ? timezone : ''
      });
    }

    if (is_AD_sync) {
      // Execute the code for ldap_sync AD sync
      await ldap_sync();
    }

    insertIntoAdminLogs(getSettingsUpdateMessage(), req);

    res.json({
      success: true,
      message: 'Settings Updated Successfully'
    });
  } catch (error) {
    next(error);
  }
};

const clientSettings = async (req, res, next) => {
  try {
    const options = {
      order: [['id', 'DESC']],
      limit: 1
    };

    const [license, clientSetting] = await Promise.all([
      await models.License.findOne({
        attributes: ['mfa_feature', 'darkening_feature'],
        ...options
      }),
      await models.ClientSetting.findOne({
        attributes: { exclude: ['createdAt', 'updatedAt'] },
        ...options
      })
    ]);

    return res.json({
      data: clientSetting,
      license,
      status: true
    });
  } catch (error) {
    next(error);
  }
};

const updateClientSettings = async (req, res, next) => {
  try {
    const {
      auto_update_state,
      mfa_server_state,
      otp_state,
      global_darkening,
      user_pass_validity_period,
      process_reporting_interval,
      android_link,
      ios_link,
      idle_session_timeout,
      is_device_profiler_enabled
    } = req.body.setting;

    const clientSettingObject = {
      auto_update_state,
      mfa_server_state,
      otp_state,
      global_darkening,
      user_pass_validity_period,
      process_reporting_interval,
      android_link,
      ios_link,
      idle_session_timeout,
      is_device_profiler_enabled
    };

    const setting = await models.ClientSetting.findOne({
      limit: 1,
      order: [['id', 'DESC']],
      attributes: { exclude: ['createdAt', 'updatedAt'] }
    });

    const updatedSettings = await setting.update(clientSettingObject);

    let changedColumns = Object.keys(updatedSettings._changed);
    changedColumns = changedColumns.filter(c =>
      [
        'auto_update_state',
        'mfa_server_state',
        'otp_state',
        'global_darkening',
        'user_pass_validity_period',
        'process_reporting_interval',
        'idle_session_timeout',
        'is_device_profiler_enabled'
      ].includes(c)
    );

    if (!updatedSettings) {
      throw httpErrors.BadGateway('Failed to update settings.');
    }

    if (changedColumns.length) {
      const columns = {};
      changedColumns.forEach(column => {
        columns[column] = updatedSettings[column];
      });

      await publishClientSettings(actionType.UPDATED, {
        data: columns
      });
    }

    if (setting._changed.hasOwnProperty('mfa_server_state')) {
      sendOtpToUsers();
    }

    insertIntoAdminLogs(getClientSettingUpdateMessage(), req);

    res.json({
      success: true,
      message: 'Settings Updated Successfully'
    });
  } catch (error) {
    next(error);
  }
};

const sendOtpToUsers = async () => {
  const setting = await models.ClientSetting.findOne({
    limit: 1,
    order: [['id', 'DESC']],
    attributes: { exclude: ['createdAt', 'updatedAt'] }
  });

  if (setting.mfa_server_state) {
    const localUsers = await models.LocalUsers.findAll({
      where: {
        client_mfa: true
      },
      attributes: ['id']
    });

    const userIds = localUsers.map(user => user.id);
    const clients = await models.Client.findAll({
      where: {
        user_id: userIds
      }
    });

    sendOtpQrBulkUsers(clients);
  }
};

const checkGatewaysSoftwareUpdate = async () => {
  const update = await models.Update.findAll({
    where: {
      buildComponent: 'gateway'
    }
  });
  const gateways = await models.Gateway.findAll({
    include: [
      {
        model: models.GatewayHealth,
        attributes: ['componentVersion']
      }
    ]
  });

  gateways.map(async gateway => {
    const myUpdate = update.find(obj => obj.buildOs === gateway.buildOs);
    if (myUpdate) {
      if (myUpdate.buildVersion && gateway.GatewayHealth.componentVersion) {
        if (myUpdate.buildVersion !== gateway.GatewayHealth.componentVersion) {
          const message = {
            version: myUpdate.buildVersion,
            updates: [
              {
                URL: gateway.pkg_url,
                Agent_ID: gateway.agent_id
              }
            ],
            is_configuration_changed: true
          };
          await publishGatewaySoftware(actionType.UPDATED, message);
        }
      }
    }
  });
};

const checkControllerSoftwareUpdate = async () => {
  const update = await models.Update.findAll({
    where: {
      buildComponent: 'controller'
    }
  });
  const controllers = await models.Controller.findAll({
    include: [
      {
        model: models.ControllerHealth,
        attributes: ['componentVersion']
      }
    ]
  });

  controllers.map(async controller => {
    const myUpdate = update.find(obj => obj.buildOs === controller.device_os);
    if (myUpdate) {
      if (
        myUpdate.buildVersion &&
        controller.ControllerHealth.componentVersion
      ) {
        if (
          myUpdate.buildVersion !== controller.ControllerHealth.componentVersion
        ) {
          const message = {
            version: myUpdate.buildVersion,
            updates: [
              {
                URL: controller.pkg_url,
                Agent_ID: controller.controller_id
              }
            ],
            is_configuration_changed: true
          };
          await publishControllerSoftware(actionType.UPDATED, message);
        }
      }
    }
  });
};

const updateCoreSettings = async (req, res, next) => {
  try {
    const {
      max_clients_per_controller,
      siem_host,
      siem_port,
      siem_protocol,
      health_status,
      data_reporting_interval,
      load_stats,
      data_monitoring_interval,
      core_auto_update,
      in_activity_timeout,
      mtls_blocking_timeout,
      mtls_frequency,
      load_stats_interval,
      health_interval,
      device_validation_freq,
      profiling_server_host,
      profiling_server_port,
      profiling_server_protocol,
      profiling_enabled
    } = req.body.setting;

    const coreSettingObject = {
      max_clients_per_controller,
      siem_host,
      siem_port,
      siem_protocol,
      health_status,
      data_reporting_interval,
      core_auto_update,
      load_stats,
      data_monitoring_interval,
      in_activity_timeout: +in_activity_timeout,
      mtls_frequency: +mtls_frequency,
      mtls_blocking_timeout: +mtls_blocking_timeout,
      health_interval: +health_interval,
      load_stats_interval: +load_stats_interval,
      device_validation_freq,
      profiling_server_host,
      profiling_server_port,
      profiling_server_protocol,
      profiling_enabled
    };

    const setting = await models.Settings.findOne({
      limit: 1,
      order: [['id', 'DESC']],
      attributes: {
        exclude: ['createdAt', 'updatedAt']
      }
    });

    await setting.update(coreSettingObject);
    if (!setting) {
      throw httpErrors.BadGateway('Failed to Update Core Settings.');
    }

    if (
      setting._changed.hasOwnProperty('siem_host') ||
      setting._changed.hasOwnProperty('siem_port') ||
      setting._changed.hasOwnProperty('siem_protocol')
    ) {
      await publishSIEM(actionType.UPDATED, {
        data: {
          siem_host: setting.siem_host,
          siem_port: +setting.siem_port,
          siem_protocol: setting.siem_protocol
        }
      });
    }
    // Publish or send Notification on change of DPSC
    if (
      setting._changed.hasOwnProperty('profiling_server_host') ||
      setting._changed.hasOwnProperty('profiling_server_port') ||
      setting._changed.hasOwnProperty('profiling_server_protocol') ||
      setting._changed.hasOwnProperty('profiling_enabled')
    ) {
      await publishProfilingConfig(actionType.UPDATED, {
        data: {
          profiling_enabled: setting.profiling_enabled,
          profiling_server_host: setting.profiling_server_host,
          profiling_server_port: +setting.profiling_server_port,
          profiling_server_protocol: setting.profiling_server_protocol
        }
      });
    }

    if (setting._changed.hasOwnProperty('core_auto_update')) {
      await publishCoreAutoUpdate(actionType.UPDATED, {
        data: {
          auto_update: core_auto_update
        }
      });
      if (core_auto_update) {
        checkGatewaysSoftwareUpdate();
        checkControllerSoftwareUpdate();
      }
    }

    if (
      setting._changed.hasOwnProperty('load_stats') ||
      setting._changed.hasOwnProperty('data_monitoring_interval') ||
      setting._changed.hasOwnProperty('health_status') ||
      setting._changed.hasOwnProperty('data_reporting_interval')
    ) {
      await publishGatewaySettings(actionType.UPDATED, {
        data: {
          load_stats: setting.load_stats,
          data_monitoring_interval: setting.data_monitoring_interval,
          health_status: setting.health_status,
          data_reporting_interval: setting.data_reporting_interval
        }
      });
    }

    if (
      setting._changed.hasOwnProperty('in_activity_timeout') ||
      setting._changed.hasOwnProperty('mtls_frequency') ||
      setting._changed.hasOwnProperty('load_stats_interval') ||
      setting._changed.hasOwnProperty('health_interval') ||
      setting._changed.hasOwnProperty('mtls_blocking_timeout')
    ) {
      await publishControllerConfiguration(actionType.UPDATED, {
        data: {
          in_activity_timeout: setting.in_activity_timeout,
          mtls_frequency: setting.mtls_frequency,
          load_stats_interval: setting.load_stats_interval,
          health_interval: setting.health_interval,
          mtls_blocking_timeout: setting.mtls_blocking_timeout
        }
      });
    }

    if (setting._changed.hasOwnProperty('max_clients_per_controller')) {
      await publishMaxClients(actionType.UPDATED, {
        data: {
          max_clients_per_controller: +setting.max_clients_per_controller
        }
      });
    }

    if (setting._changed.hasOwnProperty('device_validation_freq')) {
      device_validation.destroy();
      device_validation.schedule(device_validation_freq);
    }

    insertIntoAdminLogs(getCoreSettingUpdateMessage(), req);

    res.json({
      success: true,
      message: 'Core Settings Updated Successfully'
    });
  } catch (error) {
    next(error);
  }
};

const forceLogOut = async (req, res, next) => {
  try {
    await publishLogOut(actionType.UPDATED, {
      ids: [],
      code: 62
    });

    res.json({
      message: 'Logout process initiated.'
    });
  } catch (error) {
    next(error);
  }
};

module.exports = {
  updateClientSettings,
  clientSettings,
  updateSettings,
  createNewSettings,
  getSettings,
  getTimeZones,
  mfaSettings,
  updateCoreSettings,
  forceLogOut,
  getAddressSpaceSettings,
  updateAddressSpaceSettings,
  getTimeSettings
};
