const { fork } = require('child_process');
var httpErrors = require('http-errors');
var models = require('../models');
var fs = require('fs');
var fsExtra = require('fs-extra');
var csvWriter = require('csv-writer').createObjectCsvWriter;
var sanitize = require('sanitize-filename');
const csv = require('fast-csv');
const Openssl = require('../openssl/openssl.js');
const OpensslObj = new Openssl();
const { promisify } = require('util');
const readFile = promisify(fs.readFile);
const writeFile = promisify(fs.writeFile);
const path = require('path');
const {
  actionType,
  publishClients,
  publishMsgFile,
  clientColumnsToPublish,
  publishSoftwareAdd,
  publishSoftwareUpdate,
  publishSoftwareRemove
} = require('../routes/common/event-publisher');
const { createNewClient } = require('./localUsers');
const { Op } = require('sequelize');
const { notEmptyQueryParam } = require('../routes/common/helper');
const { Promise } = require('bluebird');
const { spawn } = require('child_process');

const {
  getSoftwareUpdateMessage,
  getSoftwareCreateMessage,
  getSoftwareDeleteMessage,
  getProcessBindingUpdateMessage,
  getProcessBindingCreateMessage,
  getProcessBindingDeleteMessage
} = require('../config/adminLogs.config');

const { insertIntoAdminLogs } = require('../routes/common/adminLogs');

const getSoftwares = async (req, res, next) => {
  try {
    const {
      limit,
      offset,
      sort = 'id',
      dir = 'desc',
      searchTerm = '',
      os = ''
    } = req.query;

    const options = {
      limit: +limit,
      offset: +offset,
      order: []
    };
    if (notEmptyQueryParam(searchTerm)) {
      options.where = {
        [Op.or]: {
          software_name: {
            [Op.like]: `%${searchTerm.trim()}%`
          },
          software_version: {
            [Op.like]: `%${searchTerm.trim()}%`
          },
          os_version: {
            [Op.like]: `%${searchTerm.trim()}%`
          },
          software_publisher: {
            [Op.like]: `%${searchTerm.trim()}%`
          },
          os_comments: {
            [Op.like]: `%${searchTerm.trim()}%`
          }
        }
      };
    }

    if (notEmptyQueryParam(os)) {
      options.where = {
        ...options.where,
        os
      };
    }

    const allowedSortValues = ['software_name', 'id'];

    if (allowedSortValues.includes(sort)) {
      options.order.push([models.sequelize.literal(sort), dir]);
    }

    const result = await models.SoftwareMasterLists.paginate(options);

    res.json(result);
  } catch (error) {
    next(error);
  }
};

const getSoftwaresProcessBinding = async (req, res, next) => {
  try {
    const {
      limit,
      offset,
      sort = 'id',
      dir = 'desc',
      searchTerm = ''
    } = req.query;

    const options = {
      limit: +limit,
      offset: +offset,
      order: []
    };
    if (notEmptyQueryParam(searchTerm)) {
      options.where = {
        [Op.or]: {
          software: {
            [Op.like]: `%${searchTerm.trim()}%`
          },
          binding_name: {
            [Op.like]: `%${searchTerm.trim()}%`
          },
          os: {
            [Op.like]: `%${searchTerm.trim()}%`
          }
        }
      };
    }
    const allowedSortValues = ['software', 'id'];

    if (allowedSortValues.includes(sort)) {
      options.order.push([models.sequelize.literal(sort), dir]);
    }

    const result = await models.SoftwareInstances.paginate(options);

    res.json(result);
  } catch (error) {
    next(error);
  }
};

const deleteSoftware = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    const software = await models.SoftwareMasterLists.findOne({
      where: {
        id: req.params.id
      }
    });

    if (!software) {
      throw new httpErrors.NotFound('Software not found');
    }

    await software.destroy({ transaction });

    await transaction.commit();

    insertIntoAdminLogs(getSoftwareDeleteMessage(software.software_name), req);

    res.json('Software deleted successfully');
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const deleteSoftwareProcessBinding = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    const software = await models.SoftwareInstances.findOne({
      where: {
        id: req.params.id
      }
    });

    if (!software) {
      throw new httpErrors.NotFound('Software not found');
    }

    await models.SoftwareInstances.destroy({
      where: {
        id: req.params.id
      },
      transaction
    });

    await models.SoftwareProcessBindings.destroy({
      where: {
        software_id: req.params.id
      },
      transaction
    });

    await publishSoftwareRemove(actionType.DELETED, {
      ids: [parseInt(req.params.id)]
    });

    // await software.destroy({transaction});

    await transaction.commit();

    insertIntoAdminLogs(getProcessBindingDeleteMessage(software.software), req);

    res.json('Software deleted successfully');
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const createSoftware = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    let {
      software_name,
      software_version,
      software_publisher,
      os,
      os_version,
      os_comments
    } = req.body.software;

    const manually_added = 'true';

    const createdSoftware = await models.SoftwareMasterLists.create(
      {
        software_name,
        software_version,
        software_publisher,
        os,
        os_version,
        os_comments,
        manually_added
      },
      {
        transaction
      }
    );

    await createdSoftware.save({ transaction });
    await transaction.commit();

    insertIntoAdminLogs(getSoftwareCreateMessage(software_name), req);

    res.json('Software created successfully');
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const createSoftwareProcessBinding = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    let {
      software,
      os,
      compliance,
      process_list,
      software_id = '',
      is_edr,
      ProcessesList = []
    } = req.body.software;
    letterOnlyOs = `${os}`;
    letterOnlyOs = letterOnlyOs.replace(/[0-9.]/g, '');
    const binding_name = `${software}_${letterOnlyOs}`;

    // Check if row with the same software name and OS already exists
    const existingSoftware = await models.SoftwareInstances.findOne({
      where: {
        software,
        os
      }
    });

    if (existingSoftware) {
      return res
        .status(400)
        .send('Software with the same name and OS already exists');
    }

    const createdSoftware = await models.SoftwareInstances.create(
      {
        software,
        os,
        compliance,
        is_edr,
        binding_name
      },
      {
        transaction
      }
    );
    await createdSoftware.save({ transaction });
    software_id = createdSoftware.id;
    ProcessesList = process_list.split(',');
    ProcessesList = ProcessesList.map(item => {
      return {
        software_id,
        process: item.trim()
      };
    });

    await models.SoftwareProcessBindings.bulkCreate(ProcessesList, {
      transaction
    });

    await publishSoftwareAdd(actionType.ADDED, {
      ids: [parseInt(software_id)]
    });

    await transaction.commit();

    insertIntoAdminLogs(getProcessBindingCreateMessage(software), req);
    res.json('Software Processes Bind successfully');
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const getSoftware = async (req, res, next) => {
  try {
    const options = {
      where: {
        id: req.params.id
      }
    };

    const softwareData = await models.SoftwareMasterLists.findOne(options);

    res.json(softwareData);
  } catch (error) {
    next(error);
  }
};

const getSoftwareProcessBinding = async (req, res, next) => {
  try {
    const options = {
      where: {
        id: req.params.id
      },
      raw: true
    };
    const option2 = {
      attributes: ['process'],
      where: {
        software_id: req.params.id
      },
      raw: true
    };
    const softwareData = await models.SoftwareInstances.findOne(options);
    const softwareProcesses = await models.SoftwareProcessBindings.findAll(
      option2
    );
    let processList = Array.prototype.map
      .call(softwareProcesses, function(item) {
        return item.process;
      })
      .join(',');
    softwareData.ProcessesList = processList;
    res.json(softwareData);
  } catch (error) {
    next(error);
  }
};

const getDistinctOS = async (req, res, next) => {
  try {
    const osList = await models.sequelize.query(`
      SELECT DISTINCT os FROM SoftwareMasterLists;
    `);
    const finalList = osList[0].map(os => {
      return os.os;
    });
    res.json(finalList);
  } catch (error) {
    next(error);
  }
};

const updateSoftware = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    let {
      software_name,
      software_version,
      software_publisher,
      os,
      os_version,
      os_comments
    } = req.body.software;

    const manually_added = 'true';

    const updateClause = {
      software_name,
      software_version,
      software_publisher,
      os,
      os_version,
      os_comments,
      manually_added
    };

    const softwareToUpdate = await models.SoftwareMasterLists.findOne({
      where: {
        id: req.params.id
      }
    });

    if (!softwareToUpdate) {
      throw new httpErrors.NotFound('No software found to update');
    }

    await softwareToUpdate.update(updateClause, { transaction });

    await transaction.commit();

    insertIntoAdminLogs(getSoftwareUpdateMessage(software_name), req);

    res.json('Software updated successfully');
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const updateSoftwareProcessBinding = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    let {
      software,
      os,
      compliance,
      process_list,
      software_id = '',
      ProcessesList = []
    } = req.body.software;
    letterOnlyOs = `${os}`;
    letterOnlyOs = letterOnlyOs.replace(/[0-9.]/g, '');
    const binding_name = `${software}_${letterOnlyOs}`;

    // Check if row with the same software name and OS already exists
    const existingSoftware = await models.SoftwareInstances.findOne({
      where: {
        software,
        os,
        id: { [models.Sequelize.Op.not]: req.params.id }
      }
    });

    if (existingSoftware) {
      return res
        .status(400)
        .send('Software with the same name and OS already exists');
    }

    const updateClause = {
      binding_name,
      software,
      os,
      compliance
    };

    const softwareToUpdate = await models.SoftwareInstances.findOne({
      where: {
        id: req.params.id
      }
    });

    if (!softwareToUpdate) {
      throw new httpErrors.NotFound('No software found to update');
    }

    await softwareToUpdate.update(updateClause, { transaction });

    software_id = softwareToUpdate.id;

    //delete record
    await models.SoftwareProcessBindings.destroy({
      where: {
        software_id
      },
      transaction
    });

    //create
    ProcessesList = process_list.split(',');
    ProcessesList = ProcessesList.map(item => {
      return {
        software_id,
        process: item.trim()
      };
    });
    await models.SoftwareProcessBindings.bulkCreate(ProcessesList, {
      transaction
    });

    await publishSoftwareUpdate(actionType.UPDATED, {
      ids: [parseInt(software_id)]
    });

    await transaction.commit();

    insertIntoAdminLogs(getProcessBindingUpdateMessage(software), req);

    res.json('Software updated successfully');
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

// const uploadCsv = (req, res, next) => {
//   try {
//     const child = fork(__dirname + '/deviceUploadTask.js');
//     child.send(req.body.fileData);
//     res.json([]);
//   } catch (error) {
//     next(error);
//   }
// }

// const downloadCsv = async (req, res, next) => {
//   try {

//     const samplePath = path.join(__dirname, '../storage/device-inventory-sample.csv');
//     return res.download(samplePath);

//   } catch (error) {
//     next(error);
//   }
// }

const downloadCsv = async (req, res, next) => {
  try {
    const samplePath = path.join(
      __dirname,
      '../storage/device-inventory-sample.csv'
    );
    return res.download(samplePath);
  } catch (error) {
    next(error);
  }
};

const bulkActions = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    let { selectAll, selectedIds = [], searchWord, os } = req.body;

    const options = {
      where: {
        id: {
          ...(selectAll
            ? {
                [Op.notIn]: selectedIds
              }
            : {
                [Op.in]: selectedIds
              })
        },
        ...(searchWord
          ? {
              [Op.or]: {
                software_name: {
                  [Op.like]: `%${searchWord.trim()}%`
                },
                software_version: {
                  [Op.like]: `%${searchWord.trim()}%`
                },
                os_version: {
                  [Op.like]: `%${searchWord.trim()}%`
                },
                software_publisher: {
                  [Op.like]: `%${searchWord.trim()}%`
                },
                os_comments: {
                  [Op.like]: `%${searchWord.trim()}%`
                }
              }
            }
          : {})
      }
    };

    if (os) {
      options.where = {
        ...options.where,
        os
      };
    }

    await models.SoftwareMasterLists.destroy(options, { transaction }),
      await transaction.commit();
    res.json('Software Master List Updated successfully');
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const deleteCurrentSoftwareMasterList = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    let { selectedIds = [] } = req.body;

    console.log('>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> :>> ', selectedIds);

    if (!Array.isArray(selectedIds)) {
      console.error('selectedIds is not an array.');
      return res.status(400).json({ error: 'Invalid selectedIds format' });
    }

    await models.SoftwareMasterLists.destroy({
      where: {
        id: selectedIds
      },
      transaction
    });

    await transaction.commit();

    return res.status(200).json({ success: true });
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const bulkActionsSoftwareProcessBinding = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    let { selectAll, selectedIds = [], searchWord } = req.body;

    const options = {
      where: {
        id: {
          ...(selectAll
            ? {
                [Op.notIn]: selectedIds
              }
            : {
                [Op.in]: selectedIds
              })
        },
        ...(searchWord
          ? {
              [Op.or]: {
                software: {
                  [Op.like]: `%${searchWord.trim()}%`
                },
                binding_name: {
                  [Op.like]: `%${searchWord.trim()}%`
                },
                os: {
                  [Op.like]: `%${searchWord.trim()}%`
                }
              }
            }
          : {})
      }
    };
    selectedIds = await models.SoftwareInstances.findAll(options);
    var str = selectedIds
      .map(item => {
        return item.id;
      })
      .join(',');
    selectedIds = str.split(',').map(element => {
      return Number(element);
    });

    await models.sequelize.query(
      `
      DELETE FROM SoftwareInstances
      WHERE id IN (${selectedIds});
      `,
      { transaction }
    );
    await models.sequelize.query(
      `
      DELETE FROM SoftwareProcessBindings
      WHERE software_id IN (${selectedIds});
      `,
      { transaction }
    );
    let idsInInt = selectedIds.map(id => {
      return parseInt(id);
    });
    await transaction.commit();
    await publishSoftwareRemove(actionType.DELETED, {
      id: idsInInt
    });
    res.json('Software Deleted successfully');
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const deleteCurrentSoftwareProcessBinding = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    let { selectedIds = [] } = req.body;

    if (!Array.isArray(selectedIds)) {
      console.error('selectedIds is not an array.');
      return res.status(400).json({ error: 'Invalid selectedIds format' });
    }

    let softwareInstanceDeleteQuery = `DELETE FROM SoftwareInstances
      WHERE id IN :selectedIds;`;

     
    await models.sequelize.query(softwareInstanceDeleteQuery, {
      replacements: { selectedIds: selectedIds },
      type: models.sequelize.QueryTypes.DELETE,
    })
       
    let softwareProcessBindingsQuery = `DELETE FROM SoftwareProcessBindings
      WHERE software_id IN :selectedIds;`;

    await models.sequelize.query(softwareProcessBindingsQuery, {
      replacements: { selectedIds: selectedIds },
      type: models.sequelize.QueryTypes.DELETE,
    })

    let idsInInt = selectedIds.map(id => {
      return parseInt(id);
    });

    await transaction.commit();

    await publishSoftwareRemove(actionType.DELETED, {
      id: idsInInt
    });

    res.json('Software Deleted successfully');
    return res.status(200).json({ success: true });
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const bulkActionsForbidden = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    let { selectAll, selectedIds = [], searchKeyword } = req.body;

    const options = {
      attributes: ['id'],
      where: {
        id: {
          ...(selectAll
            ? {
                [Op.notIn]: selectedIds
              }
            : {
                [Op.in]: selectedIds
              })
        },
        ...(searchKeyword
          ? {
              [Op.or]: {
                software_name: {
                  [Op.like]: `%${searchKeyword.trim()}%`
                },
                software_version: {
                  [Op.like]: `%${searchKeyword.trim()}%`
                },
                device_id: {
                  [Op.like]: `%${searchKeyword.trim()}%`
                },
                client_id: {
                  [Op.like]: `%${searchKeyword.trim()}%`
                },
                client_name: {
                  [Op.like]: `%${searchKeyword.trim()}%`
                },
                software_publisher: {
                  [Op.like]: `%${searchKeyword.trim()}%`
                }
              }
            }
          : {})
      },
      raw: true
    };

    selectedIds = await models.ForbiddenSoftwareLists.findAll(options);
    var str = selectedIds
      .map(item => {
        return item.id;
      })
      .join(',');
    selectedIds = str.split(',').map(element => {
      return Number(element);
    });

    await models.sequelize.query(
      `
      INSERT INTO SoftwareMasterLists (
        software_name, software_version, software_publisher,
        os, os_version, os_comments, manually_added
      )
      SELECT DISTINCT fs.software_name, fs.software_version,
      fs.software_publisher, hw.OSNAME, hw.OSVERSION, hw.OSCOMMENTS, 1
      FROM ForbiddenSoftwareLists fs
      INNER JOIN ocs.hardware hw ON fs.device_id = hw.UUID
      WHERE fs.id IN (${selectedIds});
    `,
      { transaction }
    );

    await models.sequelize.query(
      `
      DELETE t1 FROM ForbiddenSoftwareLists t1
      INNER JOIN ForbiddenSoftwareLists t2
      ON t1.software_name = t2.software_name
      AND t1.software_version = t2.software_version
      AND t1.software_publisher = t2.software_publisher
      WHERE t2.id IN (${selectedIds});
    `,
      { transaction }
    );

    await transaction.commit();
    res.json('Software List Updated successfully');
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const importSoftwareList = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    await models.sequelize.query(
      `
      UPDATE ocs.hardware h RIGHT JOIN ocs.accountinfo a ON h.ID = a.HARDWARE_ID
      SET h.UUID = SUBSTRING(a.tag, LOCATE(':', a.tag) + 1, 36)
      WHERE h.UUID IS NULL
      AND a.tag != SUBSTRING(a.tag, LOCATE(':', a.tag) + 1, 36);
  `,
      { transaction }
    );

    await models.sequelize.query(
      `
      DELETE FROM SoftwareMasterLists WHERE manually_added = 0;
    `,
      { transaction }
    );

    await models.sequelize.query(
      `
      INSERT INTO SoftwareMasterLists (
        software_name, software_version, software_publisher,
        os, os_version, os_comments, manually_added
      )
      SELECT DISTINCT sn.NAME, sv.VERSION, sp.PUBLISHER, hw.OSNAME, hw.OSVERSION, hw.OSCOMMENTS, 0
      FROM ocs.software s
      INNER JOIN ocs.software_name sn ON s.NAME_ID = sn.ID
      INNER JOIN ocs.software_version sv ON s.VERSION_ID = sv.ID
      INNER JOIN ocs.software_publisher sp ON s.PUBLISHER_ID = sp.ID
      INNER JOIN ocs.hardware hw ON s.HARDWARE_ID = hw.ID
      WHERE hw.UUID IN (
        SELECT device_id FROM DeviceInventories WHERE is_reference_machine = 1
      );
    `,
      { transaction }
    );

    await transaction.commit();

    res.json('Software List imported successfully');
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

/**
 * Get the list of polices
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
const getForbiddenSoftwares = async (req, res, next) => {
  try {
    const { limit, offset, sort = 'id', dir = 'desc', searchTerm } = req.query;

    const options = {
      limit,
      offset,
      order: []
    };

    const allowedSortValues = ['client_id', 'software_name'];

    if (allowedSortValues.includes(sort)) {
      options.order.push([models.sequelize.literal(sort), dir]);
    }

    if (notEmptyQueryParam(searchTerm)) {
      options.where = {
        [Op.or]: {
          software_name: {
            [Op.like]: `%${searchTerm.trim()}%`
          },
          software_version: {
            [Op.like]: `%${searchTerm.trim()}%`
          },
          device_id: {
            [Op.like]: `%${searchTerm.trim()}%`
          },
          client_id: {
            [Op.like]: `%${searchTerm.trim()}%`
          },
          client_name: {
            [Op.like]: `%${searchTerm.trim()}%`
          },
          software_publisher: {
            [Op.like]: `%${searchTerm.trim()}%`
          }
        }
      };
    }

    const result = await models.ForbiddenSoftwareLists.paginate(options);
    res.json(result);
  } catch (error) {
    next(error);
  }
};

const removeObseleteVersions = async (req, res, next) => {
  try {
    const currentDirectory = process.cwd();
    const scriptFolderPath = path.join(currentDirectory, 'scripts');

    process.chdir(scriptFolderPath);
    const scriptCommand = './delete-obsolete-versions.sh';
    const scriptArgs = [];

    const runScript = () => {
      return new Promise((resolve, reject) => {
        const childProcess = spawn('bash', [scriptCommand]);

        let scriptOutput = '';

        childProcess.stdout.on('data', data => {
          scriptOutput += data;
        });

        childProcess.stderr.on('data', data => {
          const parentDirectory = path.resolve(scriptFolderPath, '..');
          process.chdir(parentDirectory);
          reject(new Error(data));
        });

        childProcess.on('exit', code => {
          if (code === 0) {
            resolve(scriptOutput.trim());
          } else {
            reject(new Error(`Bash script exited with code ${code}`));
          }
        });
      });
    };

    const scriptOutput = await runScript();
    const parentDirectory = path.resolve(scriptFolderPath, '..');
    process.chdir(parentDirectory);
    res.json(scriptOutput);
  } catch (error) {
    next(error);
  }
};

module.exports = {
  getSoftwares,
  getSoftwaresProcessBinding,
  deleteSoftware,
  deleteSoftwareProcessBinding,
  createSoftware,
  createSoftwareProcessBinding,
  getSoftware,
  getSoftwareProcessBinding,
  updateSoftware,
  updateSoftwareProcessBinding,
  // uploadCsv,
  // downloadCsv,
  bulkActions,
  deleteCurrentSoftwareMasterList,
  deleteCurrentSoftwareProcessBinding,
  bulkActionsForbidden,
  bulkActionsSoftwareProcessBinding,
  importSoftwareList,
  getForbiddenSoftwares,
  getDistinctOS,
  removeObseleteVersions
};
