var httpErrors = require('http-errors');
const models = require('../models');
const redisClient = require('../config/cache');
const util = require('util');
const { Severity } = require('../shared/enums/severity');

const getTemplates = async (req, res, next) => {
  try {
    let query = `
      SELECT DISTINCT t.id as temp_id, t.name, t.description, p.id as permission_id, p.create_permission, p.read_permission, p.update_permission, p.delete_permission, p.privilege_id  
      FROM Templates t
      INNER JOIN UserTemplatePivot utp ON utp.temp_id = t.id
      INNER JOIN TemplatePermissions tp ON tp.temp_id = t.id
      INNER JOIN Permissions p ON p.id = tp.permission_id
      WHERE utp.user_id = ${req.user.id}
    `;

    let results = (await models.sequelize.query(query))[0];

    let templateData = {};

    results.forEach(value => {
      if (!templateData[value?.temp_id]) {
        templateData[value?.temp_id] = {
          temp_id: value?.temp_id,
          name: value?.name,
          description: value?.description,
          permissions: []
        };
      }

      let permissions = results
        .filter(obj => obj.temp_id === value.temp_id)
        .map(obj => ({
          perm_id: obj.permission_id,
          create_permission: obj.create_permission,
          read_permission: obj.read_permission,
          update_permission: obj.update_permission,
          delete_permission: obj.delete_permission,
          privilege_id: obj.privilege_id
        }));

      templateData[value?.temp_id].permissions = permissions;
    });

    let filteredData = Object.values(templateData);

    res.json(filteredData);
  } catch (e) {
    next(e);
  }
};

const getCreatedTemplates = async (req, res, next) => {
  try {
    const searchTerm = req.query.searchKey || '';
    const page = parseInt(req.query.offset) || 0;
    const pageSize = parseInt(req.query.limit) || 10;
    const dir = req.query.dir === 'asc' ? 'ASC' : 'DESC';
    const sort = 'name';

    const userId = req.user.id;

    let countQuery = `
      SELECT COUNT(DISTINCT t.id) AS total
      FROM Templates t
      INNER JOIN TemplatePermissions tp ON tp.temp_id = t.id
      INNER JOIN Permissions p ON p.id = tp.permission_id
      WHERE t.created_by = :userId
    `;

    let query = `
      SELECT DISTINCT t.id as temp_id, t.name, t.description, p.id as permission_id, p.create_permission, p.read_permission, p.update_permission, p.delete_permission, p.privilege_id  
      FROM Templates t
      INNER JOIN TemplatePermissions tp ON tp.temp_id = t.id
      INNER JOIN Permissions p ON p.id = tp.permission_id
      WHERE t.created_by = :userId
      ORDER BY name ${dir}
    `;

    const [countResult] = await models.sequelize.query(countQuery, {
      replacements: { userId },
      type: models.sequelize.QueryTypes.SELECT,
    });

    let results = await models.sequelize.query(query, {
      replacements: { userId },
      type: models.sequelize.QueryTypes.SELECT,
    })

    let templateData = {};

    results.forEach(value => {
      if (!templateData[value?.temp_id]) {
        templateData[value?.temp_id] = {
          temp_id: value?.temp_id,
          name: value?.name,
          description: value?.description,
          permissions: []
        };
      }

      let permissions = results
        .filter(obj => obj.temp_id === value.temp_id)
        .map(obj => ({
          perm_id: obj.permission_id,
          create_permission: obj.create_permission,
          read_permission: obj.read_permission,
          update_permission: obj.update_permission,
          delete_permission: obj.delete_permission,
          privilege_id: obj.privilege_id
        }));

      templateData[value?.temp_id].permissions = permissions;
    });

    let filteredData = Object.values(templateData);

    if (searchTerm) {
      filteredData = filteredData.filter(
        template =>
          template.name.includes(searchTerm) ||
          template.description.includes(searchTerm)
      );
    }

    const startIndex = page * pageSize;
    const endIndex = startIndex + pageSize;
    const paginatedData = filteredData.slice(startIndex, endIndex);

    const totalRecords = countResult.total;

    paginatedData.sort((a, b) => {
      const aValue = a[sort];
      const bValue = b[sort];

      if (aValue < bValue) {
        return dir === 'asc' ? -1 : 1;
      }
      if (aValue > bValue) {
        return dir === 'asc' ? 1 : -1;
      }
      return 0;
    });

    res.json({ paginatedData: paginatedData, recordsTotal: totalRecords });
  } catch (e) {
    next(e);
  }
};

const getTemplateById = async (req, res, next) => {
  try {
    const id = req.params.id;

    let query = `
      SELECT DISTINCT t.id as temp_id, t.name, t.description, p.id as permission_id, p.create_permission, p.read_permission, p.update_permission, p.delete_permission, p.privilege_id  
      FROM Templates t 
      INNER JOIN TemplatePermissions tp ON tp.temp_id = t.id 
      INNER JOIN Permissions p ON p.id = tp.permission_id 
      WHERE t.id  = ${id}
    `;

    let results = (await models.sequelize.query(query))[0];

    let templateData = {};

    results.forEach(value => {
      if (!templateData[value?.temp_id]) {
        templateData[value?.temp_id] = {
          temp_id: value?.temp_id,
          name: value?.name,
          description: value?.description,
          permissions: []
        };
      }

      let permissions = results
        .filter(obj => obj.temp_id === value.temp_id)
        .map(obj => ({
          perm_id: obj.permission_id,
          create_permission: obj.create_permission,
          read_permission: obj.read_permission,
          update_permission: obj.update_permission,
          delete_permission: obj.delete_permission,
          privilege_id: obj.privilege_id
        }));

      templateData[value?.temp_id].permissions = permissions;
    });

    let filteredData = Object.values(templateData);

    res.json(filteredData);
  } catch (error) {
    next(error);
  }
};

const getTemplateUsersCount = async (req, res, next) => {
  try {
    let tempId = req.params.id;
    
    let query = `
      SELECT t.id, t.name, COUNT(DISTINCT u.id) as count
      FROM Users u 
      INNER JOIN UserTemplates ut ON ut.user_id = u.id
      INNER JOIN TemplatePermissions tp ON tp.id = ut.temp_permission_id 
      INNER JOIN Templates t ON t.id = tp.temp_id 
      INNER JOIN Permissions p ON p.id = tp.permission_id 
      WHERE t.id = ${tempId}
    `;
 
    let [results] = (await models.sequelize.query(query))[0];

    res.json(results);
  } catch (error) {
    next(e);
  }
}

const getUserTemplate = async (req, res, next) => {
  try {
    const userData = await getUserPermissions(req.user.id, next);

    if (userData) {
      const expirationTime = req.user.exp; // token expiry time
      const currentTime = Math.floor(Date.now() / 1000); // convert current time to seconds

      // Calculate the time difference (in seconds) to set redis expiry time
      const timeDifference = expirationTime - currentTime;

      if (timeDifference < 0) {
        throw new httpErrors.Forbidden(
          'Session expiry time exceeded, please login again'
        );
      }

      const setexAsync = util.promisify(redisClient.setex).bind(redisClient);

      await setexAsync(
        `userTemplate:${req.user.id}`,
        timeDifference,
        JSON.stringify(userData)
      );
    }

    res.json(userData);
  } catch (error) {
    next(error);
  }
};

const getPossibleAdditionalPermissions = async (req, res, next) => {
  try {
    // fetch admin permissions
    const adminData = await getUserPermissions(req.user.id, next);

    // fetch infra permissions
    let query2 = 'SELECT id FROM Users LIMIT 1';
    let [firstInfraUser] = (await models.sequelize.query(query2))[0];

    const infraId = firstInfraUser.id;

    const infraData = await getUserPermissions(infraId, next);

    // Function to check if an object with a specific perm_id exists in adminData
    const isPrivilegeInAdminData = privilegeName =>
      adminData.some(item => item.privilege_name === privilegeName);

    // Filter infraData to include only objects not present in adminData (excluding license management and infra dashboard)
    const additionalPrivilege = infraData.filter(
      item =>
        !isPrivilegeInAdminData(item.privilege_name) &&
        item.privilege_name !== 'License Management' &&
        item.privilege_name !== 'Infrastructure Dashboard'
    );

    res.json(additionalPrivilege);
  } catch (error) {
    next(error);
  }
};

const getAdditionalPermissions = async (req, res, next) => {
  try {
    const searchTerm = req.query.searchKey || '';
    const page = parseInt(req.query.offset) || 0;
    const pageSize = parseInt(req.query.limit) || 10;
    const dir = req.query.dir && req.query.dir.toLowerCase() === 'asc' ? 'ASC' : 'DESC';
    const sort = 'email';

    let countQueryWhere = '';
    let queryWhere = '';
    const replacements = { 
      userId: req.user.id, 
      excludedIds: [1, 2, 3, 4, 5, 6], 
      queryDir: dir 
    };

    if (req.user.tempId === 1) {
      countQueryWhere = `WHERE ta.id NOT IN (:excludedIds) AND ta.user_id IS NOT NULL`;

      queryWhere = `WHERE ta.id NOT IN (:excludedIds) AND ta.is_approved = 0 AND ta.approver_id IS NULL`;
    }

    else if (req.user.tempId === 2) {
      countQueryWhere = `WHERE ta.user_id = :userId AND ta.id NOT IN (:excludedIds)`;

      queryWhere = `WHERE u.id = :userId AND ta.id NOT IN (:excludedIds)`;
    }

    else {
      throw new httpErrors.BadRequest(
        'User is not allowed to access additional permissions'
      );
    }

    let countQuery = `
      SELECT COUNT(DISTINCT ta.id) AS total
      FROM TemplateApproval ta
      ${countQueryWhere}
    `;

    let query = `
        SELECT DISTINCT ta.id, ta.is_approved, ta.requestee_message, ta.approver_message, ta.createdAt, t.id as temp_id, t.description, ta.user_id, u.email, p.privilege_id, pv.name, p.id as permission_id, p.create_permission, p.read_permission, p.update_permission, p.delete_permission
        FROM Templates t
        INNER JOIN TemplateApproval ta ON ta.temp_id = t.id 
        INNER JOIN Users u ON u.id = ta.user_id 
        INNER JOIN TemplatePermissions tp ON tp.temp_id = t.id
        INNER JOIN Permissions p ON p.id = tp.permission_id
        INNER JOIN Privilege pv ON pv.id = p.privilege_id
        ${queryWhere}
        ORDER BY name ${replacements.queryDir}
      `;

    const [results] = await models.sequelize.query(query, { replacements });
    const [countResult] = await models.sequelize.query(countQuery, { replacements });

    let templateData = {};

    results.forEach(value => {
      if (!templateData[value?.temp_id]) {
        templateData[value?.temp_id] = {
          id: value?.id,
          temp_id: value?.temp_id,
          description: value?.description,
          is_approved: value?.is_approved,
          user_id: value?.user_id,
          email: value?.email,
          requestee_message: value?.requestee_message,
          approver_message: value?.approver_message,
          createdAt: value?.createdAt,
          permissions: []
        };
      }

      let permissions = results
        .filter(obj => obj.temp_id === value.temp_id)
        .map(obj => ({
          perm_id: obj.permission_id,
          create_permission: obj.create_permission,
          read_permission: obj.read_permission,
          update_permission: obj.update_permission,
          delete_permission: obj.delete_permission,
          privilege_id: obj.privilege_id,
          privilege_name: obj.name
        }));

      templateData[value?.temp_id].permissions = permissions;

      // Concatenate privilege_name values into a single string
      templateData[value?.temp_id].stringifyedPermissions = permissions
      .map(permission => permission.privilege_name)
      .join(', ');
    });

    let filteredData = Object.values(templateData);

    // paginate the filtered data
    if (searchTerm) {
      filteredData = filteredData.filter(
        template =>
          template.email.includes(searchTerm) 
      );
    }

    const startIndex = page * pageSize;
    const endIndex = startIndex + pageSize;
    const paginatedData = filteredData.slice(startIndex, endIndex);

    const totalRecords = countResult.total;

    paginatedData.sort((a, b) => {
      const aValue = a[sort];
      const bValue = b[sort];

      if (aValue < bValue) {
        return dir === 'asc' ? -1 : 1;
      }
      if (aValue > bValue) {
        return dir === 'asc' ? 1 : -1;
      }
      return 0;
    });

    res.json({ paginatedData: paginatedData, recordsTotal: totalRecords });
  } catch (error) {
    next(error);
  }
};

const getCreatedAdditionalPermissions = async (req, res, next) => {
  try {
    const query = `
      SELECT DISTINCT ta.id, ta.requestee_message, ta.approver_message, p.id as perm_id, p.create_permission, p.read_permission, p.update_permission, p.delete_permission, p.privilege_id, pv.name as privilege_name, pv.privilege_id as parent_id, pv.is_crud_allowed
      FROM Templates t
      INNER JOIN TemplatePermissions tp ON tp.temp_id = t.id
      INNER JOIN Permissions p ON p.id = tp.permission_id
      INNER JOIN Privilege pv ON pv.id = p.privilege_id
      INNER JOIN TemplateApproval ta ON ta.temp_id = t.id
      INNER JOIN UserTemplates ut ON ut.temp_permission_id = tp.id
      INNER JOIN Users u ON u.id = ut.user_id
      WHERE ta.id NOT IN (1, 2, 3, 4, 5, 6) AND ta.id = :id
      ORDER BY p.privilege_id
    `;

    const results = await models.sequelize.query(query, {
      replacements: { id: req.params.id },
      type: models.Sequelize.QueryTypes.SELECT
    });

    res.json(results);
  } catch (error) {
    next(error);
  }
};

const createTemplate = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();

  try {
    const { name, description, permissions } = req.body;

    const isDefaultName = defaultTemplates.includes(name)

    if (isDefaultName) {
      throw new httpErrors.BadRequest('Cannot set name same as the default template names');
    }

    const duplicateName = await models.Templates.findOne({
      where: {
        name,
        created_by: req.user.id
      },
      transaction: transaction
    });

    if (duplicateName) {
      throw new httpErrors.BadRequest('Template with this name already exists');
    }

    const template = await models.Templates.create(
      {
        name,
        description,
        created_by: req.user.id
      },
      { transaction: transaction }
    );

    await models.TemplateApproval.create(
      {
        is_approved: 1,
        temp_id: template.id
      },
      { transaction: transaction }
    );

    await models.UserTemplatePivot.create(
      {
        temp_id: template.id,
        user_id: req.user.id
      },
      { transaction: transaction }
    );

    const error = await createPermissions(
      req,
      template.id,
      permissions,
      null,
      false,
      transaction,
      next
    );

    // if incoming error value is not null, throw error
    if (error) {
      throw new httpErrors.BadRequest(error);
    }

    await transaction.commit();
    res.json({ success: true, message: 'Template Created Successfully' });
  } catch (error) {
    if (transaction && transaction.finished !== 'commit') {
      await transaction.rollback();
    }
    next(error);
  }
};

const createAdditionalPermissions = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();

  try {
    let userId = req.user.id;
    let template;
    let tempId;

    // creating unique template name
    let name = `Admin:${req.user.id} Additional ${Date.now()}`
    let description = 'Additional permissions'
    const { message, permissions } = req.body;

    if (req.user.tempId !== 2) {
      throw new httpErrors.BadRequest(
        'Only admin is allowed to request for additional permissions'
      );
    }

    // if user has a pending request, don't allow any addional request to be sent
    let query2 = `
      SELECT * 
      FROM TemplateApproval ta 
      WHERE ta.user_id = ${req.user.id} AND ta.is_approved = 0 AND ta.approver_id IS NULL 
    `;

    let [existingRequest] = (await models.sequelize.query(query2))[0];

    if (existingRequest) {
      throw new httpErrors.BadRequest('User already has a pending request, try again once it is resolved');
    }

    template = await models.Templates.create(
      {
        name,
        description
      },
      { transaction: transaction }
    );

    await models.TemplateApproval.create(
      {
        is_approved: 0,
        requestee_message: message,
        user_id: req.user.id,
        temp_id: template.id
      },
      { transaction: transaction }
    );

    tempId = template.id;

    const error = await createPermissions(
      req,
      tempId,
      permissions,
      userId,
      true,
      transaction,
      next
    );

    // if incoming error value is not null, throw error
    if (error) {
      throw new httpErrors.BadRequest(error);
    }

    const user = await models.User.findOne({
      where: {
        id: req.user.id
      }
    })

    let userName = `${user.firstName} ${user.lastName}`
    let type = 'Admin Request';
    let eventMsg = `An additional permissions request is created by ${userName}`;

    await models.AlertLogs.create({
        type,
        message: eventMsg,
        severity: Severity.INFORMATIONAL
      }
    );

    await transaction.commit();

    res.json({ success: true, message: 'Requested Generated Successfully' });
  } catch (error) {
    if (transaction && transaction.finished !== 'commit') {
      await transaction.rollback();
    }
    next(error);
  }
};

const updateTemplate = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();

  try {
    const { name, description, permissions } = req.body;

    const templateId = await models.Templates.findOne({
      where: {
        id: req.params.id
      },
      transaction: transaction
    });

    if (!templateId) {
      throw new httpErrors.NotFound('Template with given Id not found');
    }

    // checking default and duplictate name and updating template only if incoming name is different from them
    if(!(templateId.name === name)) {
      if (!name || !name.trim()) {
        throw new httpErrors.BadRequest('Template name cannot be empty');
      }

      const isDefaultName = defaultTemplates.includes(name)

      if (isDefaultName) {
        throw new httpErrors.BadRequest('Cannot set name same as the default template names');
      }

      const duplicateName = await models.Templates.findOne({
        where: {
          name,
          created_by: req.user.id
        },
        transaction: transaction
      });

      if (duplicateName) {
        throw new httpErrors.BadRequest(
          'Template with this name already exists'
        );
      }
    }

    let userId = await removeTemplateData(req.params.id, transaction, next);

    // insert into template permissions, permissions and user templates again
    const error = await createPermissions(
      req,
      req.params.id,
      permissions,
      userId ? userId[0] : null,
      false,
      transaction,
      next
    );

    // if incoming error value is not null, throw error
    if (error) {
      throw new httpErrors.BadRequest(error);
    }

    await transaction.commit();
    res.json({ success: true, message: 'Template Updated Successfully' });
  } catch (error) {
    if (transaction && transaction.finished !== 'commit') {
      await transaction.rollback();
    }
    next(error);
  }
};

const updateAdditionalTemplate = async (req, res, next) => {
  try {
    const transaction = await models.sequelize.transaction();

    let { isApproved, message } = req.body;

    const templateId = await models.TemplateApproval.findOne({
      where: {
        id: req.params.id
      }
    });

    if (!templateId) {
      throw new httpErrors.NotFound('Template not found');
    }

    if (req.user.tempId !== 1) {
      throw new httpErrors.BadRequest(
        'Only infrastructure admin is allowed to update additional permissions request'
      );
    }

    if (isApproved == 0 && !message) {
      throw new httpErrors.BadRequest(
        'Kindly input a reason as to why the request is declined'
      );
    }
    
    if (isApproved == 1 && !message) {
      message = 'Approved';
    }

    await models.TemplateApproval.update(
      {
        is_approved: isApproved,
        approver_message: message,
        approver_id: req.user.id
      },
      {
        where: {
          id: req.params.id
        },
        transaction: transaction
      }
    );

    const infra = await models.User.findOne({
      where: {
        id: req.user.id
      }
    })

    const admin = await models.User.findOne({
      where: {
        id: templateId.user_id
      }
    })

    let infraName = `${infra.firstName} ${infra.lastName}`
    let adminName = `${admin.firstName} ${admin.lastName}`
    let type = 'Admin Request';
    let response = isApproved ? 'approved' : 'rejected'
    let eventMsg = `An additional permissions request is ${response} by ${infraName} (initiated by ${adminName})`;

    await models.AlertLogs.create({
        type,
        message: eventMsg,
        severity: Severity.INFORMATIONAL
      }
    );
    
    await transaction.commit();
    res.json({ success: true, message: 'Additional Template Updated Successfully' });
  } catch (error) {
    next(error);
  }
};

const deleteTemplate = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();

  try {
    // not deleting, just updating as the template could be assigned to user(s)
    await models.Templates.update(
      {
        created_by: null
      },
      {
        where: {
          id: req.params.id
        },
        transaction: transaction
      }
    );

    await models.UserTemplatePivot.destroy({
      where: { temp_id: req.params.id },
      transaction: transaction
    });

    await transaction.commit();
    res.json({ success: true, message: 'Template Deleted Successfully' });
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const deleteAdditionalPermissions = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    const templateApproval = await models.TemplateApproval.findOne({
      where: {
        id: req.params.id
      }
    });

    if (!templateApproval) {
      throw new httpErrors.NotFound('Additional Template with given Id not found');
    }

    await removeTemplateData(templateApproval.temp_id, transaction, next)

    await models.Templates.destroy({
      where: {
        id: templateApproval.temp_id
      },
      transaction: transaction
    });

    await transaction.commit();
    res.json({ success: true, message: 'Additional Template Deleted Successfully' });
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
}

// Utility methods
async function getUserPermissions(userId, next) {
  try {
    const user = await models.User.findByPk(userId);

    if (!user) {
      throw new httpErrors.NotFound('User not found');
    }

    const query = `
      SELECT DISTINCT t.id as temp_id, t.name as temp_name, p.id as perm_id, 
      p.create_permission, p.read_permission, p.update_permission, p.delete_permission, 
      p.privilege_id, pv.name as privilege_name, pv.privilege_id as parent_id, 
      pv.is_crud_allowed, 
      CASE WHEN pv.privilege_id IS NULL THEN 0 ELSE 1 END AS is_parent
      FROM Templates t
      INNER JOIN TemplatePermissions tp ON tp.temp_id = t.id
      INNER JOIN Permissions p ON p.id = tp.permission_id
      INNER JOIN Privilege pv ON pv.id = p.privilege_id
      INNER JOIN TemplateApproval ta ON ta.temp_id = t.id
      INNER JOIN UserTemplates ut ON ut.temp_permission_id = tp.id
      INNER JOIN Users u ON u.id = ut.user_id
      WHERE u.id = :userId AND ta.is_approved = 1
      ORDER BY COALESCE(pv.privilege_id, p.privilege_id), is_parent, pv.privilege_id, p.privilege_id;
    `;

    const results = await models.sequelize.query(query, {
      replacements: { userId: user.id },
      type: models.Sequelize.QueryTypes.SELECT
    });

    const privilegeMap = new Map();

    // Iterate through the list and keep track of the object with the highest permission_id for each privilege_id (to tackle duplicate privilege_id objects and keep the one that was asssigned later)
    results.forEach(obj => {
      const privilege_id = obj.privilege_id;
      if (!privilegeMap.has(privilege_id) || obj.perm_id > privilegeMap.get(privilege_id).perm_id) {
        privilegeMap.set(privilege_id, obj);
      }
    });

    // Extract the values from the Map to get the final updated list
    const resultList = Array.from(privilegeMap.values());

    return resultList;
  } catch (error) {
    next(error);
  }
}

async function createPermissions(
  req,
  tempId,
  permissions,
  userId,
  isAdditional,
  transaction,
  next
) {
  try {
    let error = null;

    // checking if permissions (CRUD data) were sent in the request
    if (!permissions || permissions.length === 0) {
      error = 'CRUD values required to make template';
      return error;
    }

    const userData = await getUserPermissions(req.user.id, next);

    // if user has permissions assigned
    if (userData) {
      // only validate permissions if the request is not of additional permissions type 
      if (!isAdditional) {
        // check to see the user can only create a template which has permissions same as his own or lower, can't create a template higher
        const error = validatePermissions(userData, permissions);
        // if error is not null, return error
        if (error) {
          return error;
        }
      }

      // looping over the permissions array in request to insert data in permissions and template permissions tables
      for (let i = 0; i < permissions.length; i++) {
        const permission = permissions[i];

        const createdPermission = await models.Permissions.create(
          {
            create_permission: permission.create_permission,
            read_permission: permission.read_permission,
            update_permission: permission.update_permission,
            delete_permission: permission.delete_permission,
            privilege_id: permission.privilege_id
          },
          { transaction: transaction }
        );

        const tempPermissions = await models.TemplatePermissions.create(
          {
            permission_id: createdPermission.id,
            temp_id: tempId
          },
          { transaction: transaction }
        );

        if (userId) {
          await models.UserTemplates.create(
            {
              temp_permission_id: tempPermissions.id,
              user_id: userId
            },
            { transaction: transaction }
          );
        }
      }
    }

    return error; // returning null as error
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
}

function validatePermissions(userPermissions, incomingPermissions) {
  let error = null;
  // Extract privilege_ids from the user's permissions
  const userPrivilegeIds = new Set(
    userPermissions.map(permission => permission.privilege_id)
  );

  // Iterate through incoming permissions
  for (const incomingPermission of incomingPermissions) {
    // Check if the privilege_id is in the user's assigned permissions
    if (!userPrivilegeIds.has(incomingPermission.privilege_id)) {
      error = `Privilege validation error. User doesn't have access to assign the given previleges.`;
      return error;
    }

    // Find the user's permission for the corresponding privilege_id
    const userPermission = userPermissions.find(
      permission => permission.privilege_id === incomingPermission.privilege_id
    );

    // Check each permission type
    const permissionTypes = [
      'create_permission',
      'read_permission',
      'update_permission',
      'delete_permission'
    ];
    for (const permissionType of permissionTypes) {
      // Check if the incoming permission is allowed based on user's permissions
      if (incomingPermission[permissionType] > userPermission[permissionType]) {
        error = `Permissions validation error. User does not have sufficient ${permissionType.replace(
          '_permission',
          ''
        )} permission for privilege_id ${incomingPermission.privilege_id}.`;
        return error;
      }
    }
  }

  return error; // returning null as error
}

async function removeTemplateData(tempId, transaction, next) {
  try {
    let userId; // vairbale to check if template is assigned to any user / which user

    // finding template permissions so we can extract permissions and user templates
    const templatePermissions = await models.TemplatePermissions.findAll({
      where: {
        temp_id: tempId
      },
      transaction: transaction
    });

    if (templatePermissions && templatePermissions.length > 0) {
      const permissionIds = templatePermissions.map(
        instance => instance.dataValues.permission_id
      );
      const tempPermIds = templatePermissions.map(
        instance => instance.dataValues.id
      );

      const userTemplates = await models.UserTemplates.findAll({
        where: {
          temp_permission_id: tempPermIds
        },
        transaction: transaction
      });

      if (userTemplates && userTemplates.length > 0) {
        const userIds = userTemplates.map(
          instance => instance.dataValues.user_id
        );

        userId = [...new Set(userIds)];
      }

      // deleting the permissions data so we can repopulate it later, template permissions and user templates data will be deleted automatically (through CASCADE)
      await models.Permissions.destroy({
        where: {
          id: permissionIds
        },
        transaction: transaction
      });
    }

    return userId;
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const defaultTemplates = ["Infrastructure Admin", "Admin", "Team Manager", "Policy Admin", "Report Viewer", "API User"];

module.exports = {
  getTemplates,
  getTemplateUsersCount,
  getCreatedTemplates,
  getTemplateById,
  getUserTemplate,
  getPossibleAdditionalPermissions,
  getAdditionalPermissions,
  getCreatedAdditionalPermissions,
  createTemplate,
  createAdditionalPermissions,
  updateTemplate,
  updateAdditionalTemplate,
  deleteTemplate,
  deleteAdditionalPermissions
};
