const moment = require('moment');
var httpErrors = require('http-errors');
const models = require('../models');

const { actionType, publishTrustedHosts } = require('../routes/common/event-publisher');
const { insertIntoAdminLogs } = require('../routes/common/adminLogs');
const { getHostCreateMessage, getHostEditMessage, getHostDeleteMessage } = require('../config/adminLogs.config');

const HOST_LIMIT = 500;

const getHosts = async (req, res, next) => {
  try {
    const {
      limit,
      offset,
      sort = 'createdAt',
      dir = 'desc',
      parentType,
      parentId,
      searchTerm = ''
    } = req.query;

    let searchKey = searchTerm.trim();

    const options = {
      where: { parentType, parentId },
      limit: +limit,
      offset: +offset,
      order: []
    };

    const allowedSortValues = ['createdAt', 'label', 'port', 'id', 'address'];

    if (searchKey) {
      options.where = {
        [Op.and]: {
          parentId: {
            [Op.eq]: `${parentId}`
          },
          parentType: {
            [Op.eq]: `${parentType}`
          },
          [Op.or]: {
            label: {
              [Op.like]: `%${searchKey}%`
            },
            address: {
              [Op.like]: `%${searchKey}%`
            }
          }
        }
      };
    }

    if (allowedSortValues.includes(sort)) {
      options.order.push([models.sequelize.literal(sort), dir]);
    }

    const result = await models.GlobalIps.paginate(options);
    result.status = true;
    result.message = 'Data Fetched';
    res.json(result);
  } catch (error) {
    next(error);
  }
};

const createHost = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    const reqHost = req.body;
    const parentType = req.params.parentType;
    const parentId = parseInt(req.params.parentId);

    const host = {
      ...reqHost,
      parentId,
      parentType,
      // Backwards-compatibility with Controller 1.6.2.0
      clientId: parentType === 'Client' ? parentId : 0
    };

    const { address, port, protocol, direction, label } = host;

    let count = await models.GlobalIps.count({
      where: {
        parentId, parentType
      }
    });

    if (!(count < HOST_LIMIT)) {
      throw new httpErrors.BadRequest('Host limit reached.');
    }

    count = await models.GlobalIps.count({
      where: {
        label,
        address,
        port,
        protocol,
        direction,
        parentId,
        parentType
      }
    });


    if (count !== 0) {
      //throw new httpErrors.Conflict('Host already exist with same details.');
    }

    await models.GlobalIps.create(host, { transaction });

    // await models[parentType].update(
    //   {
    //     updatedAt: moment()
    //   },
    //   {
    //     where: {
    //       id: parentId
    //     },
    //     transaction
    //   }
    // );

    const affectedClients = await models.GroupClient.findAll({
      attributes: [
        'clientId'
      ],
      where: {
        groupId: parentId
      }
    });

    const affectedClientIds = affectedClients.map(x => x.clientId);

    await publishTrustedHosts(actionType.ADDED, {
      ids: parentType == 'Group' ? affectedClientIds
        : parentType !== 'Settings' ? [parentId] : [],
      parentType
    });

    await transaction.commit();
    insertIntoAdminLogs(
      getHostCreateMessage(),
      req
    )
    res.json({
      success: true,
      message: 'Trusted Host record added successfully.'
    });

  } catch (error) {
    await transaction.rollback();
    next(error);
  }
}

const updateHost = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    const reqHost = req.body;
    const parentType = req.params.parentType;
    const parentId = parseInt(req.params.parentId);
    const id = parseInt(req.params.id);

    // Only rest of the fields need to be updated
    const { id: _id, createdAt, updatedAt, ...host } = reqHost;

    const { address, port, protocol, direction, label } = host;

    let count = await models.GlobalIps.count({
      // Host ID should not be equal to `id` to prevent it matching with itself
      where: {
        address,
        port,
        protocol,
        direction,
        parentId,
        parentType,
        id: {
          $not: id
        }
      }
    });

    if (count !== 0) {
      throw new httpErrors.Conflict('Host already exist with same details.');
    }

    count = await models.GlobalIps.count({
      where: {
        label,
        parentId,
        parentType,
        id: {
          $not: id
        }
      }
    });

    if (count !== 0) {
      throw new httpErrors.Conflict('Host already exist with same label.');
    }

    await Promise.all([
      models.GlobalIps.update(host, {
        where: {
          parentId,
          parentType,
          id
        },
        transaction
      }),
      models[parentType].update(
        {
          updatedAt: moment()
        },
        {
          where: {
            id: parentId
          },
          transaction
        }
      )
    ]);

    const affectedClients = await models.GroupClient.findAll({
      attributes: [
        'clientId'
      ],
      where: {
        groupId: parentId
      }
    });

    const affectedClientIds = affectedClients.map(x => x.clientId);

    await publishTrustedHosts(actionType.UPDATED, {
      ids: parentType == 'Group' ? affectedClientIds
        : parentType !== 'Settings' ? [parentId] : [],
      parentType
    });

    await transaction.commit();

    insertIntoAdminLogs(
      getHostEditMessage(),
      req
    )
    res.json({
      success: true,
      message: 'Trusted Host record updated successfully.'
    });
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const deleteHost = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    const parentType = req.params.parentType;
    const parentId = parseInt(req.params.parentId);
    const id = parseInt(req.params.id);
    const host = req.params.host;
    const port = req.params.port;
    const protocol = req.params.protocol;

    const autoDiscovery = await models.S2sDiscoveryData.findOne({
      where: {
        [Op.and]: [
          { source_ip: host },
          { destination_port: port },
          { protocol: protocol },
        ]
      }
    });

    if (autoDiscovery) {
      // autoDiscovery.allowed_comm = false;
      // await autoDiscovery.save();
    }

    const manualDiscovery = await models.S2sManualRule.findOne({
      where: {
        [Op.and]: [
          { source_ip: host },
          { destination_port: port },
          { protocol: protocol },
        ]
      }
    });

    if (manualDiscovery) {
      // manualDiscovery.allowed_comm = false;
      // await manualDiscovery.save();
    }

    await models.GlobalIps.destroy({
      where: {
        parentType,
        parentId,
        id
      },
      transaction
    });

    if (parentType == 'Client' || parentType == 'Server') {
      await models.Client.update(
        {
          updatedAt: moment()
        },
        {
          where: {
            id: parentId
          },
          transaction
        }
      );
    }
    else {
      await models[parentType].update(
        {
          updatedAt: moment()
        },
        {
          where: {
            id: parentId
          },
          transaction
        }
      );
    }

    const affectedClients = await models.GroupClient.findAll({
      attributes: [
        'clientId'
      ],
      where: {
        groupId: parentId
      }
    });

    const affectedClientIds = affectedClients.map(x => x.clientId);

    await publishTrustedHosts(actionType.DELETED, {
      ids: parentType == 'Group' ? affectedClientIds
        : parentType !== 'Settings' ? [parentId] : [],
      parentType
    });

    await transaction.commit();
    insertIntoAdminLogs(
      getHostDeleteMessage(),
      req
    )
    res.json({
      success: true,
      message: 'Trusted Host record deleted successfully.'
    });
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const deleteTrustedHost = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    const parentId = parseInt(req.params.parentId);
    const parentType = 'Client';
    const address = req.params.address;
    const port = req.params.port;
    const protocol = req.params.protocol;

    await models.GlobalIps.destroy({
      where: {
        address: address,
        port: port,
        protocol: protocol,
        parentId: parentId,
        parentType: parentType
      },
      transaction
    });

    await models.Client.update(
      {
        updatedAt: moment()
      },
      {
        where: {
          id: parentId
        },
        transaction
      }
    );

    const affectedClients = await models.GroupClient.findAll({
      attributes: [
        'clientId'
      ],
      where: {
        groupId: parentId
      }
    });

    const affectedClientIds = affectedClients.map(x => x.clientId);

    await publishTrustedHosts(actionType.DELETED, {
      ids: parentType == 'Group' ? affectedClientIds
        : parentType !== 'Settings' ? [parentId] : [],
      parentType
    });

    await transaction.commit();
    insertIntoAdminLogs(
      getHostDeleteMessage(),
      req
    )
    res.json({
      success: true,
      message: 'Trusted Host record deleted successfully.'
    });
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

module.exports = {
  getHosts,
  createHost,
  updateHost,
  deleteHost,
  deleteTrustedHost
};
