var httpErrors = require('http-errors');
var path = require('path');
var fs = require('fs');
var fsExtra = require('fs-extra');
var archiver = require('archiver');
var appDir = path.dirname(require.main.filename);
const md5File = require('md5-file');
var { exec } = require('child_process');
var models = require('../models');
var projectConfig = require('../config/config')['project'][
  process.env.NODE_ENV
];
appDir = appDir.substring(0, appDir.lastIndexOf('/'));
let helper = require('../routes/common/helper');
var signFilePath = '/var/www/html/admin-gui/storage/temp/';
var signPubKey = process.env.OPENSSL_PUBLIC_KEY;
const utils = require('../utils/common');

const contentQueue = require('../queues/uploadToContent');

let fileTypeMeta = {
  ubuntu: 'deb',
  centosserver: 'rpm',
  windows: 'exe',
  windowsserver: 'exe',
  macintosh: 'pkg',
  ubuntuserver: 'deb',
  ubuntuserver18: 'deb',
  ubuntuserver20armv8: 'deb',
  ubuntu18: 'deb',
  android: 'apk',
  ios: 'ipa'
};

/* GET updates listing. */
const getUpdates = async (req, res, next) => {
  try {
    const {
      limit,
      offset,
      sort = 'createdAt',
      dir = 'desc',
      searchTerm = ''
    } = req.query;

    const options = {
      where: {
        isDeleted: false
      },
      limit: limit,
      offset: offset,
      order: []
    };

    if (searchTerm.trim()) {
      options.where.buildOs = {
        [Op.like]: `%${searchTerm.trim().replace(/\s/g, '')}%`
      };
    }

    const allowedSortValues = ['createdAt', 'id'];

    if (allowedSortValues.includes(sort)) {
      options.order.push([models.sequelize.literal(sort), dir]);
    }

    const result = await models.Update.paginate(options);
    res.json(result);
  } catch (error) {
    next(error);
  }
};

/* Get a single update detail */
const getUpdate = async (req, res, next) => {
  try {
    const data = await models.Update.findOne({
      where: {
        id: req.params.id,
        isDeleted: false
      }
    });
    res.json(data);
  } catch (error) {
    next(error);
  }
};

/* CREATE new update. */
const addUpdate = async (req, res, next) => {
  try {
    req.setTimeout(500000);
    res.setTimeout(500000);

    const { update, attachment, sigFileData } = req.body;

    if (!validateSaveUpdateRequest(update, attachment, res)) {
      throw new httpErrors.UnprocessableEntity('Package Verification failed');
    }

    const primaryController = await models.Controller.findOne({
      where: {
        roleId: 1
      },
      include: [
        {
          model: models.ComponentRole,
          as: 'Role',
          attributes: [],
          where: {
            role: 'Primary'
          }
        }
      ]
    });

    if (primaryController) {
      update.primaryControllerSpaPort = primaryController.spa_port;
    }

    await saveUpdate(update, attachment, sigFileData);

    res.json({
      success: true,
      message: 'Update added Successfully'
    });
  } catch (error) {
    next(error);
  }
};

//download package
const downloadPackage = async (req, res, next) => {
  try {
    let { buildComponent, buildOs } = req.params;

    let filePath =
      `${projectConfig.serviceInstallerPath}/` +
      buildComponent.toLowerCase() +
      '/input/' +
      buildOs.toLowerCase() +
      `/installer.${
        buildOs.toLowerCase() == 'android'
          ? 'apk'
          : buildOs.toLowerCase() == 'ios'
          ? 'ipa'
          : 'zip'
      }`;

    if (buildOs === 'ios') {
      filePath =
        `${projectConfig.componentInstallationPath}/` +
        'public/assets/plist/installer.ipa';
    }

    let packageName = await utils.getPackageName(buildOs, buildComponent);

    res.download(filePath, packageName);
  } catch (error) {
    next(error);
  }
};

/* Update 'Update' record */
const updatePackge = async (req, res, next) => {
  try {
    req.setTimeout(500000);
    res.setTimeout(500000);

    const { update, attachment, sigFileData } = req.body;

    if (!validateSaveUpdateRequest(update, attachment, res)) {
      throw new httpErrors.UnprocessableEntity('Package Verification failed');
    }

    const primaryController = await models.Controller.findOne({
      where: {
        roleId: 1
      },
      include: [
        {
          model: models.ComponentRole,
          as: 'Role',
          attributes: [],
          where: {
            role: 'Primary'
          }
        }
      ]
    });

    if (primaryController) {
      update.primaryControllerSpaPort = primaryController.spa_port;
    }

    await saveUpdate(update, attachment, sigFileData);

    res.json({
      success: true,
      message: 'Update saved Successfully'
    });
  } catch (error) {
    next(error);
  }
};

// Delete an update by Id
const deletePackge = async (req, res, next) => {
  try {
    const update = await models.Update.find({
      where: {
        id: req.params.id
      }
    });

    if (!update) {
      throw new httpErrors.NotFound('No package to update.');
    }

    await update.update({
      isDeleted: true
    });

    res.json({
      success: true,
      message: 'Delete Successfully'
    });
  } catch (error) {
    next(error);
  }
};

function saveUpdate(update, attachment, sigFileData) {
  let component = update.buildComponent;
  let version = update.buildVersion;
  let os = update.buildOs;
  let description = update.description;
  let is_settings_changed = update.is_settings_changed;

  return Promise.all([
    saveFileOnDisk(update, attachment, sigFileData),
    models.Update.find({
      where: { buildComponent: component, buildOs: os, isDeleted: false }
    })
  ]).then(([fileDiskPath, softwareUpdate]) => {
    contentQueue.addtoQueue({
      os,
      component: component,
      folderPath: fileDiskPath,
      originalInstaller: true
    });

    var filePath = `${fileDiskPath}/installer.` + fileTypeMeta[os];
    if (os === 'macintosh') {
      filePath = `${fileDiskPath}/installer.zip`;
    }

    const hash = md5File.sync(filePath);
    utils
      .getSignatureFile(
        path.join(__dirname, '..', 'packageSignatureKeys/update_priv.pem'),
        filePath,
        './storage/temp/signatures',
        `${os}-package`
      )
      .then(signaturePath => {
        const data = fs.readFileSync(signaturePath, 'base64');
        return Promise.all([
          data,
          fsExtra.remove(`./storage/temp/signatures/${os}-package.sign`)
        ]);
      })
      .then(([data]) => {
        if (softwareUpdate) {
          //update
          console.log('software update exist');
          return softwareUpdate.update({
            buildPath: fileDiskPath,
            buildVersion: version,
            description: description,
            is_settings_changed: is_settings_changed,
            buildFileHash: hash,
            buildFileSignature: data
          });
        } else {
          //create
          console.log('software update does not exist');
          return models.Update.create({
            ...update,
            buildPath: fileDiskPath,
            buildFileHash: hash,
            buildFileSignature: data
          });
        }
      })
      .catch(err => {
        next(err);
      });
  });
}

function createManifestFile(fileName) {
  let uri = projectConfig.onPremApiUrl.replace('/web', '');
  uri = uri.replace('/api', '');
  let text = `<?xml version="1.0" encoding="UTF-8"?>
        <!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
        <plist version="1.0">
        <dict>
            <key>items</key>
            <array>
                <dict>
                    <key>assets</key>
                    <array>
                        <dict>
                            <key>kind</key>
                            <string>software-package</string>
                            <key>url</key>
                            <string>${uri}/assets/plist/${fileName}</string>
                        </dict>
                        <dict>
                            <key>kind</key>
                            <string>display-image</string>
                            <key>url</key>
                            <string>https://ebryxsdp.firebaseapp.com/MallSage.png</string>
                        </dict>
                        <dict>
                            <key>kind</key>
                            <string>full-size-image</string>
                            <key>url</key>
                            <string>https://ebryxsdp.firebaseapp.com/MallSage.png</string>
                        </dict>
                    </array>
                    <key>metadata</key>
                    <dict>
                        <key>bundle-identifier</key>
                        <string>com.ebryx.SDPClient</string>
                        <key>bundle-version</key>
                        <string>1.0.3</string>
                        <key>kind</key>
                        <string>software</string>
                        <key>title</key>
                        <string>SDPClient</string>
                    </dict>
                </dict>
            </array>
        </dict>
        </plist>
        `;
  try {
    fs.writeFile(
      `${projectConfig.componentInstallationPath}/` +
        'public/assets/plist/manifest.plist',
      text,
      function(err) {
        if (err) {
          console.log(err && (err.message || err));
        } else {
          console.log('Manifest File written');
        }
      }
    );
  } catch (err) {
    console.log(
      '=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+='
    );
    console.log(err && (err.message || err));
    console.log(
      '=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+='
    );
  }
}

function saveFileOnDisk(update, attachment, sigFileData) {
  let names = attachment.name.split('.');
  let fileTypeExt = names[names.length - 1];
  let fileName = 'installer.' + fileTypeExt;
  let fileDiskPath;

  let component = update.buildComponent.toLowerCase();
  let os = update.buildOs.toLowerCase();

  console.log('File extension:' + fileTypeExt + 'os ' + os);

  let encoded = attachment.data.split(';base64,').pop();
  let sigData = sigFileData.data.split(';base64,').pop();

  let filePath = '';

  if (os === 'ios') {
    //componentInstallationPath is the path where web gui is deployed
    filePath =
      `${projectConfig.componentInstallationPath}/` + 'public/assets/plist/';
    createManifestFile(fileName);
  } else {
    filePath =
      `${projectConfig.serviceInstallerPath}/` +
      component +
      '/input/' +
      os +
      '/';
  }
  console.log('File path: ' + filePath);

  return fsExtra
    .ensureDir(filePath)
    .then(() => {
      console.log('created directory strcuture successfully ' + filePath);

      return new Promise((resolve, reject) => {
        fs.writeFile(
          filePath + fileName,
          encoded,
          { encoding: 'base64' },
          function(error) {
            if (error) {
              reject(error);
            } else {
              resolve();
            }
          }
        );
      });
    })
    .then(() => {
      //write sign file in temp directory
      return new Promise((resolve, reject) => {
        fs.writeFile(
          `${signFilePath}inv_client.sign`,
          sigData,
          { encoding: 'base64' },
          error => {
            if (error) {
              reject(error);
            } else {
              resolve();
            }
          }
        );
      });
    })
    .then(() => {
      //validate package with sign file that is written
      return helper.verifySignatureFile(
        filePath + fileName,
        `${signFilePath}inv_client.sign`,
        signPubKey
      );
    })
    .then(() => {
      return fsExtra.remove(`${signFilePath}inv_client.sign`);
    })
    .then(() => {
      console.log('File successfully written on disk and verified');
      fileDiskPath = filePath + fileName;
      if (component === 'client' || component === 's2s-client') {
        console.log(filePath);
        return makeClientPackage(
          filePath,
          os,
          component,
          update.primaryControllerSpaPort
        );
      } else {
        return Promise.resolve(filePath);
      }
    });
}

async function makeClientPackage(filePath, os, component, spa_port = null) {
  const controller = await models.Controller.findOne({
    where: {
      roleId: 1
    }
  });
  let settingsFileContents = `invisily_url = "${controller.onboarding_url}:${controller.onboarding_port}/api"\nspa_port=${controller.spa_port}`;
  settingsFileContents += os === 'windowsserver' ? '\nauto_login = 1' : '';
  // settingsFileContents += `\notp_port = ${process.env.OTP_PORT}`;
  // settingsFileContents += `\nmfa_port = ${process.env.MFA_PORT}`;
  //settingsFileContents.replace(process.env.port,443);
  return new Promise((resolve, reject) => {
    fs.writeFile(`${filePath}settings.cfg`, settingsFileContents, function(
      error
    ) {
      if (error) {
        reject(error);
      } else {
        resolve();
      }
    });
  })
    .then(() => {
      if (os.includes('ubuntu')) {
        let scriptFileName = 'invisily_settings_client.sh';
        if (os.includes('server')) {
          scriptFileName = 'invisily_settings_client_server.sh';
        }
        return new Promise((resolve, reject) => {
          exec(
            `${projectConfig.serviceInstallerPath}/${component}/input/${os}/${scriptFileName} /var/local/installers/${component}/input/${os}`,
            (error, stdout, stderr) => {
              if (error) {
                console.log(error && (error.message || error));
                reject(error);
              } else {
                console.log('SH file run');
                resolve(filePath);
              }
            }
          );
        });
      } else if (os === 'macintosh') {
        return Promise.resolve(filePath);
        // return new Promise((resolve, reject) => {
        //   exec(
        //     `${projectConfig.serviceInstallerPath}/${component}/input/${os}/invisily_settings_mac_client.sh /var/local/installers/${component}/input/${os}`,
        //     (error, stdout, stderr) => {
        //       if (error) {
        //         console.log(error && (error.message || error));
        //         reject(error);
        //       } else {
        //         console.log('SH file run');
        //         resolve(filePath);
        //       }
        //     }
        //   );
        // });
      } else if (os === 'windows' || os === 'windowsserver') {
        return generateZipFile(
          filePath,
          '.exe',
          os,
          component === 's2s-client'
            ? 'Installation_Guide_S2S_Client_Win.pdf'
            : 'Installation_Guide_Client_Win.pdf'
        );
      } else {
        return Promise.resolve(filePath);
      }
    })
    .then(filePath => {
      console.log(os + ' ' + filePath);
      if (os === 'windows' || os === 'windowsserver') {
        return Promise.resolve(filePath);
      } else {
        if (os === 'macintosh') {
          const oldPath =
            projectConfig.componentInstallationPath +
            '/ocs_pkgs/macOS_agent.pkg';
          const newPath = filePath + 'installer-temp/macOS_agent.pkg';
          fsExtra.copySync(oldPath, newPath);
          return generateZipFile(
            filePath,
            '.pkg',
            'macintosh',
            'Installation_Guide_Client_Mac.pdf',
            component
          );
        } else if (os.includes('ubuntu')) {
          if (os.includes('server')) {
            let localInstallationName =
              component === 's2s-client'
                ? 'Installation_Guide_S2S_Client_Deamon.txt'
                : 'Installation_Guide_Client_Deamon.txt';

            if (os.includes('ubuntuserver20')) {
              localInstallationName =
                component === 's2s-client'
                  ? 'Installation_Guide_S2S_Client_Deamon_20.txt'
                  : 'Installation_Guide_Client_Deamon_20.txt';
            }
            return generateZipFile(
              filePath,
              '.deb',
              os,
              localInstallationName,
              component
            );
          } else {
            var localInstallationName = 'Installation_Guide_Client.pdf';
            if (os === 'ubuntu18') {
              localInstallationName = 'Installation_Guide_Client_Ubuntu.pdf';
            }
            return generateZipFile(
              filePath,
              '.deb',
              os,
              localInstallationName,
              component
            );
          }
        } else if (os.includes('centosserver')) {
          let localInstallationName =
            component === 's2s-client'
              ? 'Installation_Guide_S2S_Client_Deamon_CentOS.txt'
              : 'Installation_Guide_Client_Deamon.txt';
          return generateZipFile(
            filePath,
            '.rpm',
            os,
            localInstallationName,
            component
          );
        } else {
          return Promise.resolve(filePath);
        }
      }
    });
}

const generateZipFile = (
  parentPath,
  extension,
  os,
  guideFileName,
  component
) => {
  console.log('zip path ', parentPath, ' ', guideFileName);

  return fsExtra
    .ensureDir(`${parentPath}installer-temp`)
    .then(() => {
      return new Promise((resolve, reject) => {
        fs.copyFile(
          `${parentPath}installer${extension}`,
          `${parentPath}installer-temp/invisily_client${extension}`,
          error => {
            if (error) {
              reject(error);
            } else {
              resolve();
            }
          }
        );
      });
    })
    .then(() => {
      if (
        os === 'windows' ||
        os === 'windowsserver' ||
        os === 'centosserver' ||
        os === 'macintosh'
      ) {
        return new Promise((resolve, reject) => {
          fs.copyFile(
            `${parentPath}settings.cfg`,
            `${parentPath}installer-temp/settings.cfg`,
            error => {
              if (error) {
                reject(error);
              } else {
                resolve();
              }
            }
          );
        });
      } else {
        return Promise.resolve();
      }
    })
    .then(() => {
      const isS2sClient = component === 's2s-client';

      if (os === 'centosserver' || isS2sClient) {
        let scriptName = 'install_centos_server.sh';
        if (isS2sClient) {
          if (os === 'centosserver') {
            scriptName = 'install_centos_server_s2s.sh';
          }
          if (os === 'ubuntuserver20armv8') {
            scriptName = 'install_ubuntu20_server_s2s.sh';
          }
          if (os === 'ubuntuserver18') {
            scriptName = 'install_ubuntu_server_s2s.sh';
          }
        }

        return new Promise((resolve, reject) => {
          fs.copyFile(
            `${parentPath}${scriptName}`,
            `${parentPath}installer-temp/${scriptName}`,
            error => {
              if (error) {
                reject(error);
              } else {
                resolve();
              }
            }
          );
        });
      } else {
        return Promise.resolve();
      }
    })
    .then(() => {
      return new Promise((resolve, reject) => {
        fs.copyFile(
          `${parentPath}${guideFileName}`,
          `${parentPath}installer-temp/${guideFileName}`,
          error => {
            if (error) {
              reject(error);
            } else {
              resolve();
            }
          }
        );
      });
    })
    .then(() => {
      return new Promise((resolve, reject) => {
        const archive = archiver('zip', {
          zlib: { level: 9 }
        });
        const zipPath = `${parentPath}/installer.zip`;

        const zipStream = fs.createWriteStream(zipPath);

        zipStream.on('close', function() {
          console.log('zip generated');
          resolve(parentPath);
        });
        zipStream.on('error', function(err) {
          console.log('error in generating zip');
          reject(err);
        });

        archive.on('error', function(error) {
          reject(error);
        });

        archive.pipe(zipStream);
        archive.directory(`${parentPath}installer-temp`, false);
        archive.finalize();
      });
    })
    .then(() => {
      return fsExtra.remove(`${parentPath}installer-temp`);
    })
    .then(() => {
      return Promise.resolve(parentPath);
    });
};

function validateSaveUpdateRequest(update, attachment, response) {
  let result = true;
  let errorMessages = [];

  if (helper.isEmpty(update.buildOs)) {
    result = false;
    errorMessages.push('os is mandatory');
  }
  if (helper.isEmpty(update.buildComponent)) {
    result = false;
    errorMessages.push('version is mandatory');
  }
  if (helper.isEmpty(update.buildComponent)) {
    result = false;
    errorMessages.push('component is mandatory');
  }
  if (helper.isEmpty(attachment)) {
    result = false;
    errorMessages.push('attachment is mandatory');
  }
  if (!result) {
    response.status(400).send({
      statusText: errorMessages
    });
  }
  return result;
}
module.exports = {
  getUpdates,
  getUpdate,
  updatePackge,
  deletePackge,
  downloadPackage,
  addUpdate
};
