const createHttpError = require('http-errors');
const { Op } = require('sequelize');
const models = require('../models/index');
const { publishWhiteListedHosts, actionType } = require('../routes/common/event-publisher');

const createWhiteListedHosts = async (req, res, next) => {
    try{
        const { port, host, protocol } = req.body;
        const { controllerId } = req.params;

        const controller = await getControllerById(controllerId);
        if(!controller) throw new createHttpError(400, "Invalid controller id.");
        
        const oldHost = await models.WhiteListedHost.findOne({ where: {
            port,
            host,
            protocol,
            controllerId
        }});
        if(oldHost) throw new createHttpError(409, "Record already exist.");

        const newHost = await models.WhiteListedHost.create({
            port,
            host,
            protocol,
            controllerId
        });

        publishWhiteListedHosts(actionType.ADDED, {
            id: parseInt(controllerId),
            hosts: [
                {
                    id: newHost.id,
                    Host: newHost.host,
                    Port: newHost.port,
                    Protocol: newHost.protocol
                }
            ] 
        });

        res.status(200).send(newHost);
    } catch(e) {
        next(e);
    }
}

const getWhiteListedHosts = async (req, res, next) => {
    try {
        const { controllerId } = req.params;

        const controller = await getControllerById(controllerId);
        if(!controller) throw new createHttpError(400, "Invalid controller id.");
        
        const hosts = await models.WhiteListedHost.findAll({ where: {
            controllerId
        }});

        res.status(200).json({ data: hosts });
    }catch(e) {
        next(e);
    }
}

const updateWhiteListedHost = async (req, res, next) => {
    try {
        const { hostId } = req.params;
        const {
            port,
            host,
            protocol,
            controllerId
        } = req.body;

        const whiteListedHost = await models.WhiteListedHost.findByPk(hostId);
        if(!whiteListedHost) throw new createHttpError(400, "Invalid host id.");

        const whiteListedHostByPort = await models.WhiteListedHost.findOne({
            where: {
                port,
                id: {
                    [Op.ne]: hostId
                },
                controllerId
            }
        });
        if(whiteListedHostByPort) {
            throw new createHttpError(400, "Port already occupied");
        }

        const controller = await getControllerById(controllerId);
        if(!controller) throw new createHttpError(400, "Invalid controller id.");
        
        const updated = await whiteListedHost.update({ port, host, protocol, controllerId });

        publishWhiteListedHosts(actionType.UPDATED, {
            id: parseInt(controllerId),
            hosts: [
                {
                    id: whiteListedHost.id,
                    Host: host,
                    Port: port,
                    Protocol: protocol
                }
            ] 
        });

        res.status(200).json({ message: "Whitelisted host updated."});

    }catch(e) {
        next(e);
    }
}


const deleteWhiteListedHost = async (req, res, next) => {
    try {
        const { hostId } = req.params;

        const whiteListedHost = await models.WhiteListedHost.findByPk(hostId);
        if(!whiteListedHost) throw new createHttpError(400, "Invalid host id.");

        await whiteListedHost.destroy();

        publishWhiteListedHosts(actionType.DELETED, {
            id: parseInt(whiteListedHost.controllerId),
            hosts: [
                {
                    id: whiteListedHost.id,
                    Host: whiteListedHost.host,
                    Port: whiteListedHost.port,
                    Protocol: whiteListedHost.protocol
                }
            ] 
        });

        res.status(200).json({ message: "Whitelisted host deleted."});

    }catch(e) {
        next(e);
    }
}


const getControllerById = (controllerId) => {
    return models.Controller.findByPk(controllerId);
}

module.exports = {
    createWhiteListedHosts,
    getWhiteListedHosts,
    updateWhiteListedHost,
    deleteWhiteListedHost
}