const models = require('../models');
const httpErrors = require('http-errors');
const { Op } = require('sequelize');
const { rmSync } = require('fs-extra');
var Openssl = require('../openssl/openssl.js');
var OpensslObj = new Openssl();
const crypto = require('crypto');
const _ = require('lodash');
const { sequelize, Sequelize } = require('../models');
const {
  actionType,
  publishWorkloadUpdate,
  publishWorkloadAdd,
  publishWorkloadDelete,
  publishWorkload,
  publishStartDiscovery,
  publishStopDiscovery
} = require('../routes/common/event-publisher');

const {
  getWorkloadUpdateMessage,
  getWorkloadCreateMessage,
  getWorkloadDeleteMessage,
  getSaveDiscoveryMessage,
  getStopDiscoveryMessage
} = require('../config/adminLogs.config');

const { insertIntoAdminLogs } = require('../routes/common/adminLogs');

const getWorkLoads = async (req, res, next) => {
  try {
    const { offset = 0, limit = 10, search = '' } = req.query;
    const options = {
      offset,
      limit,
      where: {},
      include: [
        {
          model: models.DeviceInventories,
          attributes: ['device_name', 'id'],
          required: false
        }
      ],
      order: [[models.sequelize.literal('createdAt'), 'desc']]
    };
    if (
      search &&
      search !== '' &&
      search !== 'undefined' &&
      search !== 'null'
    ) {
      options.where.name = {
        [Op.like]: `%${search.trim()}%`
      };
    }

    const totalRecords = await models.Workloads.count({
      where: options.where
    });

    const workLoads = await models.Workloads.paginate(options);

    const response = {
      recordsTotal: totalRecords,
      data: workLoads.data
    };

    res.status(200).send(response);
  } catch (e) {
    next(e);
  }
};

const getAvailableWorkloadDevices = async (req, res, next) => {
  try {
    const { deviceName = '', offset = 0, workloadId = null } = req.query;
    let deviceIds = [];
    const clients = await sequelize.query(
      'select distinct(device_id) from Clients;'
    );
    clients[0] &&
      clients[0].forEach(client => {
        if (client.device_id) {
          deviceIds.push(client.device_id);
        }
      });

    if (workloadId && workloadId != 'null' && workloadId != 'undefined') {
      const workloadPivots = await models.WorkloadDevicePivots.findAll({
        where: {
          workloadId: workloadId
        }
      });
      let workloadDevices = [];
      workloadPivots.forEach(workload => {
        if (!workloadDevices.includes(workload.deviceId)) {
          workloadDevices.push(workload.deviceId);
        }
      });
      deviceIds = _.difference(deviceIds, workloadDevices);
    }

    const options = {
      where: {
        device_type: ['Physical Server', 'Virtual Server'],
        id: {
          [sequelize.Op.notIn]: deviceIds
        }
      },
      // limit: 10,
      // offset,
      attributes: ['device_name', 'device_type', 'id', 'ip_address']
    };

    if (deviceName && deviceName != 'null' && deviceName != 'undefined') {
      options.where.device_name = {
        [Op.like]: `%${deviceName.trim()}%`
      };
    }

    const devices = await models.DeviceInventories.findAll(options);
    res.json(devices);
  } catch (e) {
    next(e);
  }
};

const createWorkloadClients = async (workload, dbDevices = [], transaction) => {
  const existingClients = await models.Client.findAll({
    where: {
      device_id: dbDevices
    }
  });
  if (existingClients && existingClients.length > 0) {
    throw new httpErrors.BadRequest('Invalid devices');
  }
  const [encKeys, hmacKeys] = await Promise.all([
    Promise.all(dbDevices.map(x => OpensslObj.generateEncKey())),
    Promise.all(dbDevices.map(x => OpensslObj.generateHMacKey()))
  ]);

  const clients = dbDevices.map((device, i) => ({
    Name: '{' + 'S2S Client' + '}@{' + device.device_name + '}',
    ipv4_address: device.ip_address,
    device_os: device.os,
    client_type: 'Server',
    client_status: 1,
    risk_score: device.risk_score,
    client_darkening: device.darkening_state,
    client_os_version: '',
    serial_number: device.device_id + '',
    device_hardware_id: crypto
      .createHash('md5')
      .update(device.device_id + '')
      .digest('hex'),
    agent_id: 0,
    enc_key: encKeys[i].key,
    HMacKey: hmacKeys[i].key,
    is_local: 1,
    device_id: device.id,
    auto_updates: false,
    user_auth_required: false,
  }));

  const dbClients = await models.Client.bulkCreate(clients, { transaction });

  const clientIds = await dbClients.map(client => client.id);

  const softwareInventoryData = clientIds.map(clientId => ({
    client_id: clientId
  }));

  await Promise.all(
    softwareInventoryData.map(data =>
      models.ClientSoftwareInventory.create(data, { transaction })
    )
  );

  await models.sequelize.query('UPDATE Clients SET agent_id = id;', {
    transaction
  });

  const pivots = await dbClients.map(client => ({
    workloadId: workload.id,
    deviceId: client.device_id,
    clientId: client.id
  }));
  await models.WorkloadDevicePivots.bulkCreate(pivots, { transaction });
  return;
};

const removeWorkloadClients = async (workload, clientIds = [], transaction) => {

  await models.ClientSoftwareInventory.destroy(
    { where: { client_id: { [Op.in]: clientIds } } },
    { transaction }
  );
  return models.Client.destroy({
    where: {
      id: clientIds
    },
    transaction
  });
};

const createWorkload = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();

  try {
    const { name, devices } = req.body;

    await checkDuplicateName(name);

    const dbDevices = await models.DeviceInventories.findAll({
      where: {
        id: devices
      }
    });
    if (dbDevices.length !== devices.length) {
      throw new httpErrors.BadRequest('Invalid device ids');
    }

    const uniqueIPs = new Set();
    const filteredIPsArray = dbDevices.filter(device => {
      if (uniqueIPs.has(device.ip_address)) {
        throw new httpErrors.BadRequest(
          `Duplicate ip_address found: ${device.ip_address}`
        );
      }

      uniqueIPs.add(device.ip_address);
      return true; // Include the unique entry
    });

    const workload = await models.Workloads.create(
      {
        name,
        s2s_mode: 'idle'
      },
      { transaction }
    );
    await createWorkloadClients(workload, dbDevices, transaction);
    await transaction.commit();
    await publishWorkloadAdd(actionType.ADDED, {
      ids: [parseInt(workload.id)]
    });

    insertIntoAdminLogs(getWorkloadCreateMessage(name), req);
    res.status(200).send(workload);
  } catch (e) {
    await transaction.rollback();
    next(e);
  }
};

const checkDuplicateName = (name, id) => {
  const whereClause = {
    name
  };

  if (id) {
    whereClause.id = {
      $ne: id
    };
  }

  return models.Workloads.findOne({
    where: whereClause
  }).then(workload => {
    if (!workload) {
      return Promise.resolve();
    } else {
      throw new httpErrors.NotFound('Workload with same name already exist');
    }
  });
};

const getWorkloadById = async (req, res, next) => {
  try {
    const { workloadId } = req.params;
    const workload = await models.Workloads.findOne({
      where: {
        id: workloadId
      },
      include: [
        {
          model: models.DeviceInventories,
          required: false,
          attributes: ['id', 'device_name', 'device_type']
        }
      ]
    });
    if (!workload) {
      throw new httpErrors.BadRequest('Invalid workload id');
    }
    res.status(200).send(workload);
  } catch (e) {
    next(e);
  }
};

const editWorkloadById = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();

  try {
    const { workloadId } = req.params;
    const { name, devices } = req.body;
    const workload = await models.Workloads.findOne({
      where: {
        id: workloadId
      },
      include: [
        {
          model: models.DeviceInventories,
          required: false,
          attributes: ['id']
        }
      ]
    });
    if (!workload) {
      throw new httpErrors.BadRequest('Invalid workload id');
    }
    await checkDuplicateName(name, workload.id);

    const pivots = await workload.getWDEVICESPivots();
    let deviceIds = [];
    pivots.forEach(pivot => {
      if (!deviceIds.includes(pivot.deviceId)) {
        deviceIds.push(pivot.deviceId);
      }
    });

    if (workload.name !== name) {
      workload.name = name;
      await workload.save({ transaction });
      await updateWorkloadClientsName(workload, deviceIds, transaction);
    }

    const devicesToAdd = _.difference(devices, deviceIds);
    const devicesToDelete = _.difference(deviceIds, devices);

    const microsegmentSources = await models.Microsegments.findAll({
      where: {
        deviceId: devicesToDelete
      }
    });

    const dbDevices = await models.DeviceInventories.findAll({
      where: {
        id: devicesToAdd
      }
    });

    const IPs = dbDevices.map(device => `'${device.ip_address}'`).join(', ');

    // Check if IP is not null
    if (IPs) {
      const IPsToCheck = dbDevices.map(device => `'${device.ip_address}'`);
      if (hasDuplicates(IPsToCheck))
        throw new httpErrors.BadRequest('Workload devices IP must be unique');
      const IPDuplicationcheck = `SELECT DISTINCT workloads.id, workloads.name, 
                                        deviceInventories.device_name, deviceInventories.ip_address  
                                        FROM Workloads workloads
                                        INNER JOIN WorkloadDevicePivots AS workloadDevicePivots ON workloadDevicePivots.workloadId = workloads.id 
                                        INNER JOIN DeviceInventories AS deviceInventories ON deviceInventories.id = workloadDevicePivots.deviceId 
                                        WHERE workloads.id= ${workloadId} AND deviceInventories.ip_address IN (${IPs});
                                        `;

      console.log(IPDuplicationcheck);
      const IPDuplicationcheckResult = (
        await models.sequelize.query(IPDuplicationcheck)
      )[0];

      if (IPDuplicationcheckResult.length > 0)
        throw new httpErrors.BadRequest(
          'Device with the IP already exists in the workload'
        );
    }
    await createWorkloadClients(workload, dbDevices, transaction);

    await models.WorkloadDevicePivots.destroy({
      where: {
        deviceId: devicesToDelete,
        workloadId: workload.id
      },
      transaction
    });

    const devicesToCheck = await models.DeviceInventories.findAll({
      where: {
        id: devicesToDelete
      },
      attributes: ['device_name', 'ip_address']
    });
  
    const workloadManualRule = await models.S2sManualRule.findAll({
      where: {
        [Op.or]: [
          { workloadId: workloadId },
          { src_workload_id: workloadId }
        ],
        [Op.or]: [
          { source_device: { [Op.in]: devicesToCheck.map(device => device?.device_name) } },
          { source_ip: { [Op.in]: devicesToCheck.map(device => device?.ip_address) } },
          { destination_device: { [Op.in]: devicesToCheck.map(device => device?.device_name) } },
          { destination_ip: { [Op.in]: devicesToCheck.map(device => device?.ip_address) } }
        ]
      }
    });
  
    const matchedRuleIds = workloadManualRule.map(rule => rule?.id);
  
    await models.S2sManualRule.destroy({
      where: {
        [Op.or]: [
          { workloadId: workload.id },
          { src_workload_id: workload.id }
        ],
        id: matchedRuleIds 
      }
    });

    await models.S2sDiscoveryData.destroy({
      where: {
        deviceId: devicesToDelete,
        [Op.or]: [
          { workloadId: workload.id },
          { src_workload_id: workload.id }
        ]
      },
      transaction
    });

    let clientsToDelete = [];
    pivots.forEach(pivot => {
      if (
        devicesToDelete.includes(pivot.deviceId) &&
        !clientsToDelete.includes(pivot.clientId)
      ) {
        clientsToDelete.push(pivot.clientId);
      }
    });

    await removeWorkloadClients(workload, clientsToDelete, transaction);

    // await workload.removeDeviceInventories(
    //   workload.DeviceInventories.map(device => device.id),
    //   { transaction }
    // );
    // await workload.addDeviceInventories(devices, { transaction });

    await transaction.commit();
    await publishWorkloadUpdate(actionType.UPDATED, {
      ids: [parseInt(workloadId)]
    });

    insertIntoAdminLogs(getWorkloadUpdateMessage(name), req);
    res.status(200).send(workload);
  } catch (e) {
    await transaction.rollback();
    next(e);
  }
};

const deleteWorkloadById = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();

  try {
    const { workloadId } = req.params;
    const workload = await models.Workloads.findOne({
      where: {
        id: workloadId
      },
      include: [
        {
          model: models.DeviceInventories,
          required: false,
          attributes: ['id']
        }
      ]
    });
    if (!workload) {
      throw new httpErrors.BadRequest('Invalid workload id');
    }
    const microsegment = await models.Microsegments.findAll({
      where: {
        workloadId: workload.id
      }
    });
    // if (microsegment && microsegment.length > 0) {
    //     throw new httpErrors.BadRequest("Can't delete this workload because a microsegment still exists within this workload.")
    // }
    const pivots = await workload.getWDEVICESPivots();
    let clientIds = [];
    pivots.forEach(pivot => {
      if (!clientIds.includes(pivot.clientId)) {
        clientIds.push(pivot.clientId);
      }
    });

    await models.WorkloadDevicePivots.destroy({
      where: {
        workloadId: workload.id
      },
      transaction
    });
    await removeWorkloadClients(workload, clientIds, transaction);
    await workload.destroy({ transaction });
    await transaction.commit();
    await publishWorkloadDelete(actionType.DELETED, {
      ids: [parseInt(workloadId)]
    });

    insertIntoAdminLogs(getWorkloadDeleteMessage(workload.name), req);
    res.status(200).json({ message: 'Workload deleted' });
  } catch (e) {
    await transaction.rollback();
    next(e);
  }
};

const updateWorkloadClientsName = async (
  workload,
  dbDevices = [],
  transaction
) => {
  try {
    for (const deviceId of dbDevices) {
      const client = await models.Client.findOne({
        where: {
          device_id: deviceId
        },
        transaction
      });

      const device = await models.DeviceInventories.findOne({
        where: {
          id: deviceId
        },
        transaction
      });

      if (client) {
        const updatedName = `{${device.device_name}}@{${workload.name}}`;

        // Update the client's Name field
        await models.Client.update(
          { Name: updatedName },
          {
            where: {
              id: client.id
            },
            transaction
          }
        );
      }
    }
  } catch (e) {
    await transaction.rollback();
    next(e);
  }
};

const workloadBulkAction = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    const { action, allSelected, search, workloadIds, excludeIds } = req.body;

    const options = {
      where: {},
      include: [
        {
          model: models.DeviceInventories,
          required: false,
          attributes: ['id']
        }
      ]
    };

    if (allSelected == true) {
      options.where.id = {
        [Op.not]: excludeIds
      };

      if (search.trim()) {
        options.where.name = {
          [Op.like]: `%${search.trim()}%`
        };
      }
    } else {
      options.where.id = workloadIds;
    }

    if (action === 'deleteWorkloads') {
      const workloads = await models.Workloads.findAll(options);
      const idsToDelete = workloads.map(workload => workload.id);

      const microsegment = await models.Microsegments.findAll({
        where: {
          workloadId: idsToDelete
        }
      });
      // if (microsegment && microsegment.length > 0) {
      //     throw new httpErrors.BadRequest("Can't delete workloads because a microsegment still exists within these workloads.")
      // }

      const pivots = await models.WorkloadDevicePivots.findAll(
        {
          where: {
            workloadId: idsToDelete
          }
        },
        { transaction }
      );

      let clientIds = [];
      pivots.forEach(pivot => {
        if (!clientIds.includes(pivot.clientId)) {
          clientIds.push(pivot.clientId);
        }
      });

      await models.WorkloadDevicePivots.destroy({
        where: {
          workloadId: idsToDelete
        },
        transaction
      });

      await removeWorkloadClients(null, clientIds, transaction);

      await models.Workloads.destroy({
        where: {
          id: idsToDelete
        },
        transaction
      });
      await transaction.commit();
      await publishWorkloadDelete(actionType.DELETED, {
        ids: idsToDelete
      });
      return res.status(200).json({ message: 'Workloads deleted' });
    }
  } catch (e) {
    await transaction.rollback();
    next(e);
  }
};

const saveWorkDiscovery = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    let { duration, threshold, mode } = req.body;

    const updateClause = {
      discovery_duration: duration,
      discovery_threshold: threshold,
      s2s_mode: mode
    };

    const workload = await models.Workloads.findOne({
      where: {
        id: req.params.id
      }
    });

    if (!workload) {
      throw new httpErrors.NotFound('No workload found to update');
    }

    await workload.update(updateClause, { transaction });

    let saveMode = '';

    if (mode == 'discovery') {
      saveMode = true;
    }

    if (mode == 'spa') {
      saveMode = false;
    }

    let logMessage;
    if (mode == 'discovery') {
      logMessage = getSaveDiscoveryMessage(workload.name);
    } else if (mode == 'spa' && saveMode) {
      logMessage = getStopDiscoveryMessage(workload.name);
    }

    if (mode == 'discovery') {
      const devices = await models.WorkloadDevicePivots.findAll(
        { where: { workloadId: req.params.id } },
        { transaction }
      );
      const deviceIds = devices.map(d => {
        return d.deviceId;
      });
      await models.S2sDiscoveryData.destroy(
        { where: { deviceId: { [Op.in]: deviceIds } } },
        { transaction }
      );
    }

    if (logMessage) {
      insertIntoAdminLogs(logMessage, req);
    }

    await transaction.commit();

    // await publishStopDiscovery(actionType.UPDATED, {});

    res.json('Workload updated successfully');
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

async function updateS2sData(workloadId) {
  await models.S2sManualRule.update(
    { workloadId },
    {
      where: {
        [Op.or]: [
          { workload_tag: null },
          { workload_tag: '' }
        ]
      }
    }
  );

  await models.S2sDiscoveryData.update(
    { workloadId },
    {
      where: {
        [Op.or]: [
          { workload_tag: null },
          { workload_tag: '' }
        ]
      }
    }
  );

}

async function createOrUpdateDefaultWorkload(duration, threshold, mode) {
  let workload = await models.Workloads.findOne({ where: { name: 'Default' } });

  if (!workload) {
    workload = await models.Workloads.create({
      name: 'Default',
      discovery_duration: duration,
      discovery_threshold: threshold,
      s2s_mode: mode
    });
  }

  await updateS2sData(workload.id);

  const devicesData = await models.DeviceInventories.findAll({
    where: { device_type: { [Op.or]: ['Physical Server', 'Virtual Server'] } },
    include: [{ model: models.Client, attributes: ['id'], required: true }],
  });

  // Filter out devices that are already associated with the workload
  const existingDeviceIds = (await models.WorkloadDevicePivots.findAll({
    where: { workloadId: workload.id },
    attributes: ['deviceId'],
  })).map(record => record.deviceId);

  const pivotRecords = devicesData
  .filter(device => !existingDeviceIds.includes(device.id))
  .map(device => ({
    workloadId: workload.id,
    deviceId: device.id,
    clientId: device.Client.id
  }));

  await models.WorkloadDevicePivots.bulkCreate(pivotRecords);
}

const saveAllWorkDiscovery = async (req, res, next) => {
  try {
    let { duration, threshold, mode } = req.body;

    const updateClause = {
      discovery_duration: duration,
      discovery_threshold: threshold,
      s2s_mode: mode
    };

    await createOrUpdateDefaultWorkload(duration, threshold, mode);

    const workload = await models.Workloads.findOne({
      where: { name: 'Default' }
    });

    if (!workload) {
      throw new httpErrors.NotFound('No workload found to update');
    }

    const updatedWorkload = await workload.update(updateClause);

    let saveMode = '';
    if (mode == 'discovery') {
      saveMode = true;
    } else if (mode == 'spa') {
      saveMode = false;
    }

    let logMessage;
    if (mode == 'discovery') {
      logMessage = getSaveDiscoveryMessage(updatedWorkload.name);
      
    } else if (mode == 'spa' && saveMode) {
      logMessage = getStopDiscoveryMessage(updatedWorkload.name);
    }

    if (logMessage) {
      insertIntoAdminLogs(logMessage, req);
    }

    await getRefreshPreviousWorkload();


    if(saveMode) {
      await publishStartDiscovery(actionType.UPDATED, {});
    }else {
      await publishStopDiscovery(actionType.UPDATED, {});
    }

    res.json('Workload updated successfully');
  } catch (error) {
    next(error);
  }
};

const saveMicroSegTag = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    const { edge, tagName } = req.body;
    if(edge?.type === 'auto'){
      await models.S2sDiscoveryData.update(
        {
          microsegment_tag: tagName 
        },
        {
          where: {
            id: edge?.id
          }
        }
      );
    }else if(edge?.type === 'manual'){
      await models.S2sManualRule.update(
        {
          microsegment_tag: tagName 
        },
        {
          where: {
            id: edge?.id
          }
        }
      );
    }

    return res.status(200).json({ message: 'Microsegment tag updated' });
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const saveAllMicroSegTag = async (req, res, next) => {
  try {
    const { edge, tagName, workloadTag, srcWorkloadTag } = req.body;

    let getPrevWl;
    if(edge.type === 'auto') {
      getPrevWl = await models.S2sDiscoveryData.findOne({
        where: {
          id: edge.id
        }
      })
    }else if(edge.type === 'manual') {
      getPrevWl = await models.S2sManualRule.findOne({
        where: {
          id: edge.id
        }
      })
    }

    const sourceDeviceFromEdge = await models.DeviceInventories.findOne({
      where: {
        [Op.and]: [
          {
            device_type: {
              [Op.or]: ['Physical Server', 'Virtual Server']
            }
          },
          {
            ip_address: edge?.to
          }
        ]
      },
      include: [{ model: models.Client, attributes: ['id'], required: true }]
    });

    let tag = workloadTag || 'Default';
    let srcTag = srcWorkloadTag || 'Default';

    const destDeviceFromEdge = await models.DeviceInventories.findOne({
      where: {
        [Op.and]: [
          {
            device_type: {
              [Op.or]: ['Physical Server', 'Virtual Server']
            }
          },
          {
            ip_address: edge?.from
          }
        ]
      },
      include: [{ model: models.Client, attributes: ['id'], required: true }],
    });

    let wlDevices = [sourceDeviceFromEdge, destDeviceFromEdge];
    wlDevices = wlDevices.filter(item => item !== null);

    let newWorkloadId;
    let srcNewWorkloadId;

    async function findOrCreateWorkload(tag) {
      let workload = await models.Workloads.findOne({ where: { name: tag } });
      if (!workload) {
        workload = await models.Workloads.create({ name: tag, s2s_mode: 'idle' });
      }
      
      return workload;
    }

    if ((tag === srcTag) && (tag !== 'Default' || srcTag !== 'Default') && (edge.type === 'auto')) {
      throw new httpErrors.Conflict('Source and destination workload tags can not be same');
    } else {
      const workload = await findOrCreateWorkload(tag);
      newWorkloadId = workload.id;

      const srcWorkload = await findOrCreateWorkload(srcTag);
      srcNewWorkloadId = srcWorkload.id;
    }

    if(tag != srcTag) {
      await wlDevices.map(device => {
        return models.WorkloadDevicePivots.create({
          workloadId: newWorkloadId,
          deviceId: device.id,
          clientId: device.Client.id
        });
      });
      await wlDevices.map(device => {
        return models.WorkloadDevicePivots.create({
          workloadId: srcNewWorkloadId,
          deviceId: device.id,
          clientId: device.Client.id
        });
      });
    }else {
      await wlDevices.map(device => {
        return models.WorkloadDevicePivots.create({
          workloadId: newWorkloadId,
          deviceId: device.id,
          clientId: device.Client.id
        });
      });
    }

    if(edge?.type === 'auto'){
      await models.S2sDiscoveryData.update(
        {
          workloadId: newWorkloadId,
          src_workload_id: srcNewWorkloadId
        },
        {
          where: {
            id: edge?.id
          }
        }
      );
    } else if(edge?.type === 'manual') {
      await models.S2sManualRule.update(
        {
          workloadId: newWorkloadId,
          src_workload_id: srcNewWorkloadId
        },
        {
          where: {
            id: edge?.id
          }
        }
      );
    }

    if(edge?.type === 'auto'){
      await models.S2sDiscoveryData.update(
        {
          microsegment_tag: tagName,
          workload_tag: workloadTag,
          source_workload_tag: srcWorkloadTag
        },
        {
          where: {
            id: edge?.id
          }
        }
      );
    } else if(edge?.type === 'manual') {
      await models.S2sManualRule.update(
        {
          microsegment_tag: tagName,
          workload_tag: workloadTag,
          source_workload_tag: srcWorkloadTag
        },
        {
          where: {
            id: edge?.id
          }
        }
      );
    }

    
    const allWorkloadIds = await models.Workloads.findAll({
      attributes: ['id'],
      raw: true
    });

    // Convert workload IDs to a set for easy lookup
    const allWorkloadIdSet = new Set(allWorkloadIds.map(workload => workload.id));


    //Remove Orphan Workloads from system
    for (const workloadId of allWorkloadIdSet) {
      const existsInS2sDiscoverydata = await models.S2sDiscoveryData.count({
        where: {
          [Op.or]: [
            { workloadId: workloadId },
            { src_workload_id: workloadId }
          ]
        },
      });
    
      const existsInManualRulesData = await models.S2sManualRule.count({
        where: {
          [Op.or]: [
            { workloadId: workloadId },
            { src_workload_id: workloadId }
          ]
        },
      });
    
      if (existsInS2sDiscoverydata === 0 && existsInManualRulesData === 0) {
        await models.Workloads.destroy({
          where: {
            id: workloadId
          }
        });
      }
    }

    return res.status(200).json({ message: 'Microsegment tag updated' });
  } catch (error) {
    next(error);
  }
};

const getS2SDiscoveryData = async (req, res, next) => {
  try {
    const { workloadId } = req.params;
    const workloadPivots = await models.WorkloadDevicePivots.findAll({
      where: {
        workloadId
      },
      attributes: [
        [Sequelize.fn('DISTINCT', Sequelize.col('clientId')), 'clientId']
      ]
    });
    const clientIds = workloadPivots.map(pivot => pivot.clientId);
    let allDiscovery = await models.S2sDiscoveryData.findAll({
      where: {
        [Op.or]: [
          { workloadId: workloadId },
          { src_workload_id: workloadId }
        ]
      },
      include: [
        {
          model: models.DeviceInventories,
          attributes: ['id', 'device_name'],
          as: 'destinationDevice'
        },
        {
          model: models.DeviceInventories,
          attributes: ['id', 'device_name'],
          as: 'sourceDevice'
        },
        {
          model: models.Client,
          attributes: ['id', 'Name']
        }
      ]
    });

    let toFilterQuery = `        
        SELECT DISTINCT workloads.id, workloads.name, 
        deviceInventories.device_name, deviceInventories.ip_address  
        FROM Workloads workloads
        INNER JOIN WorkloadDevicePivots AS workloadDevicePivots ON workloadDevicePivots.workloadId = workloads.id 
        INNER JOIN DeviceInventories AS deviceInventories ON deviceInventories.id = workloadDevicePivots.deviceId
        INNER JOIN S2sDiscoveryData s2sDiscoveryData ON s2sDiscoveryData.workloadId = workloads.id 
        `;

    let toFilterResult = (await models.sequelize.query(toFilterQuery))[0];

    console.log('toFilterResult ', toFilterResult);
    // res.status(200).send(toFilterResult);

    // return toFilterResult

    let manualDiscovery = await models.S2sManualRule.findAll({
      where: {
        [Op.or]: [
          { workloadId: workloadId },
          { src_workload_id: workloadId }
        ]
      },
    });

    const nodes = {};
    const edges = [];

    for (const discovery of allDiscovery) {
      let totalCount = 0;
      let totalAllowed = 0;
      let totalNotAllowed = 0;
      let manualtotalCount = 0;
      let manualtotalAllowed = 0;

      let manualdisData = await models.sequelize.query(
        `
                SELECT sum(allowed_comm) totalAllowed, count(id) as total
                FROM S2sManualRules
                where workloadId = '${discovery.workloadId}'
                  and ((source_ip = '${discovery.source_ip}' and destination_ip = '${discovery.destination_ip}')
                    or
                       (source_ip = '${discovery.destination_ip}' and destination_ip = '${discovery.source_ip}'))
            `,
        { raw: true, type: sequelize.QueryTypes.SELECT }
      );

      manualtotalCount = manualdisData[0].total;
      manualtotalAllowed = manualdisData[0].totalAllowed;

      let disData = await models.sequelize.query(
        `
                SELECT sum(allowed_comm) totalAllowed, count(id) as total
                FROM S2sDiscoveryData
                where workloadId = '${discovery.workloadId}'
                  and ((source_ip = '${discovery.source_ip}' and destination_ip = '${discovery.destination_ip}')
                    or
                       (source_ip = '${discovery.destination_ip}' and destination_ip = '${discovery.source_ip}'))

            `,
        { raw: true, type: sequelize.QueryTypes.SELECT }
      );

      if (manualtotalCount && manualtotalAllowed) {
        totalCount = parseInt(disData[0].total) + parseInt(manualtotalCount);
        totalAllowed =
          parseInt(disData[0].totalAllowed) + parseInt(manualtotalAllowed);
        totalNotAllowed = parseInt(totalCount) - parseInt(totalAllowed);
      } else {
        totalCount = disData[0].total;
        totalAllowed = disData[0].totalAllowed;
        totalNotAllowed = parseInt(totalCount) - parseInt(totalAllowed);
      }

      // console.log('discovery: ', results);
      let destinationName =
        (discovery &&
          discovery.destinationDevice &&
          `${discovery.destinationDevice.device_name}`) ||
        discovery.destination_ip;

      if (
        !toFilterResult.some(s2sdata => s2sdata.device_name == destinationName)
      ) {
        destinationName = discovery.destination_ip;
      }

      if (!nodes[discovery.destination_ip]) {
        if (
          toFilterResult.some(s2sdata => s2sdata.device_name == destinationName)
        ) {
          nodes[discovery.destination_ip] = {
            id: discovery.destination_ip,
            fill: {
              src: 'assets/images/laptop.png'
            },
            name: destinationName,
            device: discovery.deviceId
          };
        } else {
          nodes[discovery.destination_ip] = {
            id: discovery.destination_ip,
            fill: {
              src: 'assets/images/laptop-red.png'
            },
            name: destinationName,
            device: discovery.deviceId
          };
        }
      }

      let sourceName =
        (discovery &&
          discovery.sourceDevice &&
          `${discovery.sourceDevice.device_name}`) ||
        discovery.source_ip;

      if (!toFilterResult.some(s2sdata => s2sdata.device_name == sourceName)) {
        sourceName = discovery.source_ip;
      }

      if (!nodes[discovery.source_ip]) {
        if (toFilterResult.some(s2sdata => s2sdata.device_name == sourceName)) {
          nodes[discovery.source_ip] = {
            id: discovery.source_ip,
            fill: {
              src: 'assets/images/laptop.png'
            },
            name: sourceName,
            device: discovery.deviceId
          };
        } else {
          nodes[discovery.source_ip] = {
            id: discovery.source_ip,
            fill: {
              src: 'assets/images/laptop-red.png'
            },
            name: sourceName,
            device: discovery.deviceId
          };
        }
      }

      const trustedHost = await models.GlobalIps.findOne({
        where: {
          [Op.and]: [
            { address: discovery.source_ip },
            { port: discovery.destination_port },
            { protocol: discovery.protocol },
            { direction: 1 },
            { clientId: discovery.client_id },
            { parentType: 'Client' },
          ]
        }
      })

      let colourIs = 'red';
      let dashIs =
        parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed) > 0
          ? '10 5'
          : '0';
      let isDisabled = false;
      if (
        !discovery?.sourceDevice?.device_name ||
        !toFilterResult.some(s2sdata => s2sdata.device_name == sourceName)
      ) {
        (trustedHost || discovery.allowed_comm) ? (colourIs = '#32a852') : (colourIs = 'red');
        dashIs =
          parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed) > 0
            ? '10 5'
            : '0';
        isDisabled = true;
      } else if (parseInt(totalAllowed) === parseInt(totalCount)) {
        colourIs = '#32a852';
        dashIs =
          parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed) > 0
            ? '10 5'
            : '0';
      } else {
        colourIs = 'red';
        dashIs =
          parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed) > 0
            ? '10 5'
            : '0';
      }

      let stroke = {
        normal: {
          stroke: {
            color: colourIs,
            thickness: '2',
            dash: dashIs,
            lineJoin: 'round'
          }
        },
        hovered: {
          stroke: {
            color: colourIs,
            thickness: '3',
            dash: dashIs,
            lineJoin: 'round'
          }
        },

        selected: {
          stroke: {
            color: colourIs,
            thickness: '3',
            dash: dashIs,
            lineJoin: 'round'
          }
        }
      };

      edges.push({
        checkKey:
          discovery.source_ip +
          discovery.destination_ip +
          discovery.protocol +
          discovery.destination_port,
        from: discovery.source_ip,
        allowed_comm: trustedHost ? true : discovery.allowed_comm,
        to: discovery.destination_ip,
        name: sourceName,
        id: discovery.id,
        destinationName: destinationName,
        protocol: discovery.protocol.toUpperCase(),
        destinationPort: discovery.destination_port,
        microsegTag: discovery.microsegment_tag,
        workloadTag: discovery.workload_tag,
        srcWorkloadTag: discovery.source_workload_tag,
        client:
          (discovery.Client && discovery.Client.Name) || discovery.client_id,
        ...(discovery.allowed_comm = stroke),
        isDisabled,
        type: 'auto'
      });
    }

    for (const discovery of manualDiscovery) {
      let totalCount = 0;
      let totalAllowed = 0;
      let totalNotAllowed = 0;
      let s2stotalCount = 0;
      let s2stotalAllowed = 0;

      let s2sdisData = await models.sequelize.query(
        `
                SELECT sum(allowed_comm) totalAllowed, count(id) as total
                FROM S2sDiscoveryData
                where workloadId = '${discovery.workloadId}'
                  and ((source_ip = '${discovery.source_ip}' and destination_ip = '${discovery.destination_ip}')
                    or
                       (source_ip = '${discovery.destination_ip}' and destination_ip = '${discovery.source_ip}'))

            `,
        { raw: true, type: sequelize.QueryTypes.SELECT }
      );

      s2stotalCount = s2sdisData[0].total;
      s2stotalAllowed = s2sdisData[0].totalAllowed;

      let disData = await models.sequelize.query(
        `
                SELECT sum(allowed_comm) totalAllowed, count(id) as total
                FROM S2sManualRules
                where workloadId = '${discovery.workloadId}'
                  and ((source_ip = '${discovery.source_ip}' and destination_ip = '${discovery.destination_ip}')
                    or
                       (source_ip = '${discovery.destination_ip}' and destination_ip = '${discovery.source_ip}'))
            `,
        { raw: true, type: sequelize.QueryTypes.SELECT }
      );

      if (s2stotalCount && s2stotalAllowed) {
        totalCount = parseInt(disData[0].total) + parseInt(s2stotalCount);
        totalAllowed =
          parseInt(disData[0].totalAllowed) + parseInt(s2stotalAllowed);
        totalNotAllowed = parseInt(totalCount) - parseInt(totalAllowed);
      } else {
        totalCount = disData[0].total;
        totalAllowed = disData[0].totalAllowed;
        totalNotAllowed = parseInt(totalCount) - parseInt(totalAllowed);
      }

      let destinationName =
        discovery.destination_device || discovery.destination_ip;

      if (!nodes[discovery.destination_ip]) {
        nodes[discovery.destination_ip] = {
          id: discovery.destination_ip,
          fill: {
            src: 'assets/images/laptop.png'
          },
          name: destinationName
        };
      }

      let sourceName = discovery.source_device || discovery.source_ip;

      if (!toFilterResult.some(s2sdata => s2sdata.device_name == sourceName)) {
        sourceName = discovery.source_device || discovery.source_ip;
      }

      if (!nodes[discovery.source_ip]) {
        nodes[discovery.source_ip] = {
          id: discovery.source_ip,
          fill: {
            src: 'assets/images/laptop.png'
          },
          name: sourceName
        };
      }

      let colourIs = 'red';
      let dashIs =
        (parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed)) > 0
          ? '10 5'
          : '0';
      let isDisabled = false;
      if (!toFilterResult.some(s2sdata => s2sdata.device_name == sourceName)) {
        discovery.allowed_comm ? (colourIs = '#32a852') : (colourIs = 'red');
        dashIs =
          (parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed)) > 0
            ? '10 5'
            : '0';
        isDisabled = false;
      } else if (parseInt(totalAllowed) === parseInt(totalCount)) {
        colourIs = '#32a852';
        dashIs =
          (parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed)) > 0
            ? '10 5'
            : '0';
      } else {
        colourIs = 'red';
        dashIs =
          (parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed)) > 0
            ? '10 5'
            : '0';
      }

      let stroke = {
        normal: {
          stroke: {
            color: colourIs,
            thickness: '2',
            dash: dashIs,
            lineJoin: 'round'
          }
        },
        hovered: {
          stroke: {
            color: colourIs,
            thickness: '3',
            dash: dashIs,
            lineJoin: 'round'
          }
        },

        selected: {
          stroke: {
            color: colourIs,
            thickness: '3',
            dash: dashIs,
            lineJoin: 'round'
          }
        }
      };

      // let doesThatAlreadyExist = false
      // const index = edges.findIndex(edge => edge.checkKey = discovery.source_ip+discovery.destination_ip+discovery.protocol+discovery.destination_port)
      // if(index) edges.splice(index, 1)
      edges.push({
        checkKey:
          discovery.source_ip +
          discovery.destination_ip +
          discovery.protocol +
          discovery.destination_port,
        from: discovery.source_ip,
        allowed_comm: discovery.allowed_comm,
        to: discovery.destination_ip,
        name: sourceName,
        id: discovery.id,
        destinationName: destinationName,
        protocol: discovery.protocol.toUpperCase(),
        destinationPort: discovery.destination_port,
        microsegTag: discovery.microsegment_tag,
        workloadTag: discovery.workload_tag,
        srcWorkloadTag: discovery.source_workload_tag,
        client:
          (discovery.Client && discovery.Client.Name) || discovery.client_id,
        ...(discovery.allowed_comm = stroke),
        isDisabled,
        type: 'manual'
      });
    }

    const edgesUniqueByKey = [
      ...new Map(edges.map(item => [item['checkKey'], item])).values()
    ];

    const data = {
      nodes: Object.values(nodes),
      edges: edgesUniqueByKey
    };

    console.log('data :>> ', data);

    res.status(200).send(data);
  } catch (error) {
    next(error);
  }
};

const getAllS2SDiscoveryData = async (req, res, next) => {
  try {
    const workloadPivots = await models.WorkloadDevicePivots.findAll({
      attributes: [
        [Sequelize.fn('DISTINCT', Sequelize.col('clientId')), 'clientId']
      ]
    });
    const clientIds = workloadPivots.map(pivot => pivot.clientId);
    let allDiscovery = await models.S2sDiscoveryData.findAll({
      include: [
        {
          model: models.DeviceInventories,
          attributes: ['id', 'device_name'],
          as: 'destinationDevice'
        },
        {
          model: models.DeviceInventories,
          attributes: ['id', 'device_name'],
          as: 'sourceDevice'
        },
        {
          model: models.Client,
          attributes: ['id', 'Name']
        }
      ]
    });

    let toFilterQuery = `        
      SELECT DISTINCT workloads.id, workloads.name, 
      deviceInventories.device_name, deviceInventories.ip_address  
      FROM Workloads workloads
      INNER JOIN WorkloadDevicePivots AS workloadDevicePivots ON workloadDevicePivots.workloadId = workloads.id 
      INNER JOIN DeviceInventories AS deviceInventories ON deviceInventories.id = workloadDevicePivots.deviceId
      INNER JOIN S2sDiscoveryData s2sDiscoveryData ON s2sDiscoveryData.workloadId = workloads.id`;

    let toFilterResult = (await models.sequelize.query(toFilterQuery))[0];

    console.log('toFilterResult ', toFilterResult);

    let manualDiscovery = await models.S2sManualRule.findAll();

    const nodes = {};
    const edges = [];

    for (const discovery of allDiscovery) {
      let totalCount = 0;
      let totalAllowed = 0;
      let totalNotAllowed = 0;
      let manualtotalCount = 0;
      let manualtotalAllowed = 0;

      let manualdisData = await models.sequelize.query(
        `
        SELECT 
            SUM(allowed_comm) AS totalAllowed, 
            COUNT(id) AS total
        FROM 
            S2sManualRules
        WHERE 
            ((source_ip = '${discovery.source_ip}' AND destination_ip = '${discovery.destination_ip}')
            OR
            (source_ip = '${discovery.destination_ip}' AND destination_ip = '${discovery.source_ip}'))
        `,
        { raw: true, type: sequelize.QueryTypes.SELECT }
    );
    

      manualtotalCount = manualdisData[0].total;
      manualtotalAllowed = manualdisData[0].totalAllowed;

      let disData = await models.sequelize.query(
        `
                SELECT sum(allowed_comm) totalAllowed, count(id) as total
                FROM S2sDiscoveryData
                where ((source_ip = '${discovery.source_ip}' and destination_ip = '${discovery.destination_ip}')
                    or
                       (source_ip = '${discovery.destination_ip}' and destination_ip = '${discovery.source_ip}'))

            `,
        { raw: true, type: sequelize.QueryTypes.SELECT }
      );

      if (manualtotalCount && manualtotalAllowed) {
        totalCount = parseInt(disData[0].total) + parseInt(manualtotalCount);
        totalAllowed =
          parseInt(disData[0].totalAllowed) + parseInt(manualtotalAllowed);
        totalNotAllowed = parseInt(totalCount) - parseInt(totalAllowed);
      } else {
        totalCount = disData[0].total;
        totalAllowed = disData[0].totalAllowed;
        totalNotAllowed = parseInt(totalCount) - parseInt(totalAllowed);
      }

      // console.log('discovery: ', results);
      let destinationName =
        (discovery &&
          discovery.destinationDevice &&
          `${discovery.destinationDevice.device_name}`) ||
        discovery.destination_ip;

      if (
        !toFilterResult.some(s2sdata => s2sdata.device_name == destinationName)
      ) {
        destinationName = discovery.destination_ip;
      }

      if (!nodes[discovery.destination_ip]) {
        if (
          toFilterResult.some(s2sdata => s2sdata.device_name == destinationName)
        ) {
          nodes[discovery.destination_ip] = {
            id: discovery.destination_ip,
            fill: {
              src: 'assets/images/laptop.png'
            },
            name: destinationName,
            device: discovery.deviceId
          };
        } else {
          nodes[discovery.destination_ip] = {
            id: discovery.destination_ip,
            fill: {
              src: 'assets/images/laptop-red.png'
            },
            name: destinationName,
            device: discovery.deviceId
          };
        }
      }

      let sourceName =
        (discovery &&
          discovery.sourceDevice &&
          `${discovery.sourceDevice.device_name}`) ||
        discovery.source_ip;

      if (!toFilterResult.some(s2sdata => s2sdata.device_name == sourceName)) {
        sourceName = discovery.source_ip;
      }

      if (!nodes[discovery.source_ip]) {
        if (toFilterResult.some(s2sdata => s2sdata.device_name == sourceName)) {
          nodes[discovery.source_ip] = {
            id: discovery.source_ip,
            fill: {
              src: 'assets/images/laptop.png'
            },
            name: sourceName,
            device: discovery.deviceId
          };
        } else {
          nodes[discovery.source_ip] = {
            id: discovery.source_ip,
            fill: {
              src: 'assets/images/laptop-red.png'
            },
            name: sourceName,
            device: discovery.deviceId
          };
        }
      }

      const trustedHost = await models.GlobalIps.findOne({
        where: {
          [Op.and]: [
            { address: discovery.source_ip },
            { port: discovery.destination_port },
            { protocol: discovery.protocol },
            { direction: 1 },
            { clientId: discovery.client_id },
            { parentType: 'Client' },
          ]
        }
      })

      let colourIs = 'red';
      let dashIs =
        parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed) > 0
          ? '10 5'
          : '0';
      let isDisabled = false;
      if (
        !discovery?.sourceDevice?.device_name ||
        !toFilterResult.some(s2sdata => s2sdata.device_name == sourceName)
      ) {
        (trustedHost || discovery.allowed_comm) ? (colourIs = '#32a852') : (colourIs = 'red');
        dashIs =
          parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed) > 0
            ? '10 5'
            : '0';
        isDisabled = true;
      } else if (parseInt(totalAllowed) === parseInt(totalCount)) {
        colourIs = '#32a852';
        dashIs =
          parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed) > 0
            ? '10 5'
            : '0';
      } else {
        colourIs = 'red';
        dashIs =
          parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed) > 0
            ? '10 5'
            : '0';
      }

      let stroke = {
        normal: {
          stroke: {
            color: colourIs,
            thickness: '2',
            dash: dashIs,
            lineJoin: 'round'
          }
        },
        hovered: {
          stroke: {
            color: colourIs,
            thickness: '3',
            dash: dashIs,
            lineJoin: 'round'
          }
        },

        selected: {
          stroke: {
            color: colourIs,
            thickness: '3',
            dash: dashIs,
            lineJoin: 'round'
          }
        }
      };

      edges.push({
        checkKey:
          discovery.source_ip +
          discovery.destination_ip +
          discovery.protocol +
          discovery.destination_port,
        from: discovery.source_ip,
        allowed_comm: trustedHost ? true : discovery.allowed_comm,
        to: discovery.destination_ip,
        name: sourceName,
        id: discovery.id,
        destinationName: destinationName,
        protocol: discovery.protocol.toUpperCase(),
        destinationPort: discovery.destination_port,
        microsegTag: discovery.microsegment_tag,
        workloadTag: discovery.workload_tag,
        srcWorkloadTag: discovery.source_workload_tag,
        client:
          (discovery.Client && discovery.Client.Name) || discovery.client_id,
        ...(discovery.allowed_comm = stroke),
        isDisabled,
        type: 'auto'
      });
    }

    for (const discovery of manualDiscovery) {
      let totalCount = 0;
      let totalAllowed = 0;
      let totalNotAllowed = 0;
      let s2stotalCount = 0;
      let s2stotalAllowed = 0;

      let s2sdisData = await models.sequelize.query(
        `
                SELECT sum(allowed_comm) totalAllowed, count(id) as total
                FROM S2sDiscoveryData
                where ((source_ip = '${discovery.source_ip}' and destination_ip = '${discovery.destination_ip}')
                    or
                       (source_ip = '${discovery.destination_ip}' and destination_ip = '${discovery.source_ip}'))

            `,
        { raw: true, type: sequelize.QueryTypes.SELECT }
      );

      s2stotalCount = s2sdisData[0].total;
      s2stotalAllowed = s2sdisData[0].totalAllowed;

      let disData = await models.sequelize.query(
        `
                SELECT sum(allowed_comm) totalAllowed, count(id) as total
                FROM S2sManualRules
                where ((source_ip = '${discovery.source_ip}' and destination_ip = '${discovery.destination_ip}')
                    or
                       (source_ip = '${discovery.destination_ip}' and destination_ip = '${discovery.source_ip}'))
            `,
        { raw: true, type: sequelize.QueryTypes.SELECT }
      );

      if (s2stotalCount && s2stotalAllowed) {
        totalCount = parseInt(disData[0].total) + parseInt(s2stotalCount);
        totalAllowed =
          parseInt(disData[0].totalAllowed) + parseInt(s2stotalAllowed);
        totalNotAllowed = parseInt(totalCount) - parseInt(totalAllowed);
      } else {
        totalCount = disData[0].total;
        totalAllowed = disData[0].totalAllowed;
        totalNotAllowed = parseInt(totalCount) - parseInt(totalAllowed);
      }

      let destinationName =
        discovery.destination_device || discovery.destination_ip;

      if (!nodes[discovery.destination_ip]) {
        nodes[discovery.destination_ip] = {
          id: discovery.destination_ip,
          fill: {
            src: 'assets/images/laptop.png'
          },
          name: destinationName
        };
      }

      let sourceName = discovery.source_device || discovery.source_ip;

      if (!toFilterResult.some(s2sdata => s2sdata.device_name == sourceName)) {
        sourceName = discovery.source_device || discovery.source_ip;
      }

      if (!nodes[discovery.source_ip]) {
        nodes[discovery.source_ip] = {
          id: discovery.source_ip,
          fill: {
            src: 'assets/images/laptop.png'
          },
          name: sourceName
        };
      }

      let colourIs = 'red';
      let dashIs =
        (parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed)) > 0
          ? '10 5'
          : '0';
      let isDisabled = false;
      if (!toFilterResult.some(s2sdata => s2sdata.device_name == sourceName)) {
        discovery.allowed_comm ? (colourIs = '#32a852') : (colourIs = 'red');
        dashIs =
          (parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed)) > 0
            ? '10 5'
            : '0';
        isDisabled = false;
      } else if (parseInt(totalAllowed) === parseInt(totalCount)) {
        colourIs = '#32a852';
        dashIs =
          (parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed)) > 0
            ? '10 5'
            : '0';
      } else {
        colourIs = 'red';
        dashIs =
          (parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed)) > 0
            ? '10 5'
            : '0';
      }

      let stroke = {
        normal: {
          stroke: {
            color: colourIs,
            thickness: '2',
            dash: dashIs,
            lineJoin: 'round'
          }
        },
        hovered: {
          stroke: {
            color: colourIs,
            thickness: '3',
            dash: dashIs,
            lineJoin: 'round'
          }
        },

        selected: {
          stroke: {
            color: colourIs,
            thickness: '3',
            dash: dashIs,
            lineJoin: 'round'
          }
        }
      };

      // let doesThatAlreadyExist = false
      // const index = edges.findIndex(edge => edge.checkKey = discovery.source_ip+discovery.destination_ip+discovery.protocol+discovery.destination_port)
      // if(index) edges.splice(index, 1)
      edges.push({
        checkKey:
          discovery.source_ip +
          discovery.destination_ip +
          discovery.protocol +
          discovery.destination_port,
        from: discovery.source_ip,
        allowed_comm: discovery.allowed_comm,
        to: discovery.destination_ip,
        name: sourceName,
        id: discovery.id,
        destinationName: destinationName,
        protocol: discovery.protocol.toUpperCase(),
        destinationPort: discovery.destination_port,
        microsegTag: discovery.microsegment_tag,
        workloadTag: discovery.workload_tag,
        srcWorkloadTag: discovery.source_workload_tag,
        client:
          (discovery.Client && discovery.Client.Name) || discovery.client_id,
        ...(discovery.allowed_comm = stroke),
        isDisabled,
        type: 'manual'
      });
    }

    const edgesUniqueByKey = [
      ...new Map(edges.map(item => [item['checkKey'], item])).values()
    ];

    const data = {
      nodes: Object.values(nodes),
      edges: edgesUniqueByKey
    };

    res.status(200).send(data);
  } catch (error) {
    next(error);
  }
};

const getPaginatedS2SDiscoveryData = async (req, res, next) => {
  try {
    let { workloadId, limit, offset } = req.query;
    let { search = '' } = req.query;

    limit = parseInt(limit);
    offset = parseInt(offset);

    const workloadPivots = await models.WorkloadDevicePivots.findAll({
      where: {
        workloadId
      },
      attributes: [
        [Sequelize.fn('DISTINCT', Sequelize.col('clientId')), 'clientId']
      ]
    });
    const clientIds = workloadPivots.map(pivot => pivot.clientId);

    const totalDiscoveryCount = await models.S2sDiscoveryData.count({
      where: {
        workloadId: workloadId
      }
    });

    let allDiscovery;
    if (search) {
      allDiscovery = await models.S2sDiscoveryData.findAll({
        where: {
          [Op.or]: [
            { workloadId: workloadId },
            { src_workload_id: workloadId }
          ]
        },
        include: [
          {
            model: models.DeviceInventories,
            attributes: ['id', 'device_name'],
            as: 'destinationDevice'
          },
          {
            model: models.DeviceInventories,
            attributes: ['id', 'device_name'],
            as: 'sourceDevice'
          },
          {
            model: models.Client,
            attributes: ['id', 'Name']
          }
        ]
      });
    } else {
      allDiscovery = await models.S2sDiscoveryData.findAll({
        where: {
          [Op.or]: [
            { workloadId: workloadId },
            { src_workload_id: workloadId }
          ]
        },
        include: [
          {
            model: models.DeviceInventories,
            attributes: ['id', 'device_name'],
            as: 'destinationDevice'
          },
          {
            model: models.DeviceInventories,
            attributes: ['id', 'device_name'],
            as: 'sourceDevice'
          },
          {
            model: models.Client,
            attributes: ['id', 'Name']
          }
        ]
      });
    }

    let toFilterQuery = `        
        SELECT DISTINCT workloads.id, workloads.name, 
        deviceInventories.device_name, deviceInventories.ip_address  
        FROM Workloads workloads
        INNER JOIN WorkloadDevicePivots AS workloadDevicePivots ON workloadDevicePivots.workloadId = workloads.id 
        INNER JOIN DeviceInventories AS deviceInventories ON deviceInventories.id = workloadDevicePivots.deviceId
        INNER JOIN S2sDiscoveryData s2sDiscoveryData ON s2sDiscoveryData.workloadId = workloads.id 
        `;

    let toFilterResult = (await models.sequelize.query(toFilterQuery))[0];

    console.log('toFilterResult ', toFilterResult);
    // res.status(200).send(toFilterResult);

    // return toFilterResult

    const totalManualDiscoveryCount = await models.S2sManualRule.count({
      where: {
        workloadId: workloadId
      }
    });

    let manualDiscovery;
    if (search) {
      manualDiscovery = await models.S2sManualRule.findAll({
        where: {
          [Op.or]: [
            { workloadId: workloadId },
            { src_workload_id: workloadId }
          ]
        },
      });
    } else {
      manualDiscovery = await models.S2sManualRule.findAll({
        where: {
          [Op.or]: [
            { workloadId: workloadId },
            { src_workload_id: workloadId }
          ]
        },
      });
    }

    const nodes = {};
    const edges = [];

    for (const discovery of allDiscovery) {
      let totalCount = 0;
      let totalAllowed = 0;
      let totalNotAllowed = 0;
      let manualtotalCount = 0;
      let manualtotalAllowed = 0;

      let manualdisData = await models.sequelize.query(
        `
                SELECT sum(allowed_comm) totalAllowed, count(id) as total
                FROM S2sManualRules
                where workloadId = '${discovery.workloadId}'
                  and ((source_ip = '${discovery.source_ip}' and destination_ip = '${discovery.destination_ip}')
                    or
                       (source_ip = '${discovery.destination_ip}' and destination_ip = '${discovery.source_ip}'))
            `,
        { raw: true, type: sequelize.QueryTypes.SELECT }
      );

      manualtotalCount = manualdisData[0].total;
      manualtotalAllowed = manualdisData[0].totalAllowed;

      let disData = await models.sequelize.query(
        `
                SELECT sum(allowed_comm) totalAllowed, count(id) as total
                FROM S2sDiscoveryData
                where workloadId = '${discovery.workloadId}'
                  and ((source_ip = '${discovery.source_ip}' and destination_ip = '${discovery.destination_ip}')
                    or
                       (source_ip = '${discovery.destination_ip}' and destination_ip = '${discovery.source_ip}'))

            `,
        { raw: true, type: sequelize.QueryTypes.SELECT }
      );

      if (manualtotalCount && manualtotalAllowed) {
        totalCount = parseInt(disData[0].total) + parseInt(manualtotalCount);
        totalAllowed =
          parseInt(disData[0].totalAllowed) + parseInt(manualtotalAllowed);
        totalNotAllowed = parseInt(totalCount) - parseInt(totalAllowed);
      } else {
        totalCount = disData[0].total;
        totalAllowed = disData[0].totalAllowed;
        totalNotAllowed = parseInt(totalCount) - parseInt(totalAllowed);
      }

      // console.log('discovery: ', results);
      let destinationName =
        (discovery &&
          discovery.destinationDevice &&
          `${discovery.destinationDevice.device_name}`) ||
        discovery.destination_ip;

      if (
        !toFilterResult.some(s2sdata => s2sdata.device_name == destinationName)
      ) {
        destinationName = discovery.destination_ip;
      }

      if (!nodes[discovery.destination_ip]) {
        if (
          toFilterResult.some(s2sdata => s2sdata.device_name == destinationName)
        ) {
          nodes[discovery.destination_ip] = {
            id: discovery.destination_ip,
            fill: {
              src: 'assets/images/laptop.png'
            },
            name: destinationName,
            device: discovery.deviceId
          };
        } else {
          nodes[discovery.destination_ip] = {
            id: discovery.destination_ip,
            fill: {
              src: 'assets/images/laptop-red.png'
            },
            name: destinationName,
            device: discovery.deviceId
          };
        }
      }

      let sourceName =
        (discovery &&
          discovery.sourceDevice &&
          `${discovery.sourceDevice.device_name}`) ||
        discovery.source_ip;

      if (!toFilterResult.some(s2sdata => s2sdata.device_name == sourceName)) {
        sourceName = discovery.source_ip;
      }

      if (!nodes[discovery.source_ip]) {
        if (toFilterResult.some(s2sdata => s2sdata.device_name == sourceName)) {
          nodes[discovery.source_ip] = {
            id: discovery.source_ip,
            fill: {
              src: 'assets/images/laptop.png'
            },
            name: sourceName,
            device: discovery.deviceId
          };
        } else {
          nodes[discovery.source_ip] = {
            id: discovery.source_ip,
            fill: {
              src: 'assets/images/laptop-red.png'
            },
            name: sourceName,
            device: discovery.deviceId
          };
        }
      }

      let colourIs = 'red';
      let dashIs =
        parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed) > 0
          ? '10 5'
          : '0';
      let isDisabled = false;
      if (
        !discovery?.sourceDevice?.device_name ||
        !toFilterResult.some(s2sdata => s2sdata.device_name == sourceName)
      ) {
        discovery.allowed_comm ? (colourIs = '#32a852') : (colourIs = 'red');
        dashIs =
          parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed) > 0
            ? '10 5'
            : '0';
        isDisabled = true;
      } else if (parseInt(totalAllowed) === parseInt(totalCount)) {
        colourIs = '#32a852';
        dashIs =
          parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed) > 0
            ? '10 5'
            : '0';
      } else {
        colourIs = 'red';
        dashIs =
          parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed) > 0
            ? '10 5'
            : '0';
      }

      let stroke = {
        normal: {
          stroke: {
            color: colourIs,
            thickness: '2',
            dash: dashIs,
            lineJoin: 'round'
          }
        },
        hovered: {
          stroke: {
            color: colourIs,
            thickness: '3',
            dash: dashIs,
            lineJoin: 'round'
          }
        },

        selected: {
          stroke: {
            color: colourIs,
            thickness: '3',
            dash: dashIs,
            lineJoin: 'round'
          }
        }
      };

      edges.push({
        checkKey:
          discovery.source_ip +
          discovery.destination_ip +
          discovery.protocol +
          discovery.destination_port,
        from: discovery.source_ip,
        allowed_comm: discovery.allowed_comm,
        to: discovery.destination_ip,
        name: sourceName,
        id: discovery.id,
        destinationName: destinationName,
        protocol: discovery.protocol.toUpperCase(),
        destinationPort: discovery.destination_port,
        microsegTag: discovery.microsegment_tag,
        workloadTag: discovery.workload_tag,
        srcWorkloadTag: discovery.source_workload_tag,
        client:
          (discovery.Client && discovery.Client.Name) || discovery.client_id,
        ...(discovery.allowed_comm = stroke),
        isDisabled,
        type: 'auto'
      });
    }

    for (const discovery of manualDiscovery) {
      let totalCount = 0;
      let totalAllowed = 0;
      let totalNotAllowed = 0;
      let s2stotalCount = 0;
      let s2stotalAllowed = 0;

      let s2sdisData = await models.sequelize.query(
        `
                SELECT sum(allowed_comm) totalAllowed, count(id) as total
                FROM S2sDiscoveryData
                where workloadId = '${discovery.workloadId}'
                  and ((source_ip = '${discovery.source_ip}' and destination_ip = '${discovery.destination_ip}')
                    or
                       (source_ip = '${discovery.destination_ip}' and destination_ip = '${discovery.source_ip}'))

            `,
        { raw: true, type: sequelize.QueryTypes.SELECT }
      );

      s2stotalCount = s2sdisData[0].total;
      s2stotalAllowed = s2sdisData[0].totalAllowed;

      let disData = await models.sequelize.query(
        `
                SELECT sum(allowed_comm) totalAllowed, count(id) as total
                FROM S2sManualRules
                where workloadId = '${discovery.workloadId}'
                  and ((source_ip = '${discovery.source_ip}' and destination_ip = '${discovery.destination_ip}')
                    or
                       (source_ip = '${discovery.destination_ip}' and destination_ip = '${discovery.source_ip}'))
            `,
        { raw: true, type: sequelize.QueryTypes.SELECT }
      );

      if (s2stotalCount && s2stotalAllowed) {
        totalCount = parseInt(disData[0].total) + parseInt(s2stotalCount);
        totalAllowed =
          parseInt(disData[0].totalAllowed) + parseInt(s2stotalAllowed);
        totalNotAllowed = parseInt(totalCount) - parseInt(totalAllowed);
      } else {
        totalCount = disData[0].total;
        totalAllowed = disData[0].totalAllowed;
        totalNotAllowed = parseInt(totalCount) - parseInt(totalAllowed);
      }

      let destinationName =
        discovery.destination_device || discovery.destination_ip;

      if (!nodes[discovery.destination_ip]) {
        nodes[discovery.destination_ip] = {
          id: discovery.destination_ip,
          fill: {
            src: 'assets/images/laptop.png'
          },
          name: destinationName
        };
      }

      let sourceName = discovery.source_device || discovery.source_ip;

      if (!toFilterResult.some(s2sdata => s2sdata.device_name == sourceName)) {
        sourceName = discovery.source_device || discovery.source_ip;
      }

      if (!nodes[discovery.source_ip]) {
        nodes[discovery.source_ip] = {
          id: discovery.source_ip,
          fill: {
            src: 'assets/images/laptop.png'
          },
          name: sourceName
        };
      }

      let colourIs = 'red';
      let dashIs =
        (parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed)) > 0
          ? '10 5'
          : '0';
      let isDisabled = false;
      if (!toFilterResult.some(s2sdata => s2sdata.device_name == sourceName)) {
        discovery.allowed_comm ? (colourIs = '#32a852') : (colourIs = 'red');
        dashIs =
          (parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed)) > 0
            ? '10 5'
            : '0';
        isDisabled = false;
      } else if (parseInt(totalAllowed) === parseInt(totalCount)) {
        colourIs = '#32a852';
        dashIs =
          (parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed)) > 0
            ? '10 5'
            : '0';
      } else {
        colourIs = 'red';
        dashIs =
          (parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed)) > 0
            ? '10 5'
            : '0';
      }

      let stroke = {
        normal: {
          stroke: {
            color: colourIs,
            thickness: '2',
            dash: dashIs,
            lineJoin: 'round'
          }
        },
        hovered: {
          stroke: {
            color: colourIs,
            thickness: '3',
            dash: dashIs,
            lineJoin: 'round'
          }
        },

        selected: {
          stroke: {
            color: colourIs,
            thickness: '3',
            dash: dashIs,
            lineJoin: 'round'
          }
        }
      };

      // let doesThatAlreadyExist = false
      // const index = edges.findIndex(edge => edge.checkKey = discovery.source_ip+discovery.destination_ip+discovery.protocol+discovery.destination_port)
      // if(index) edges.splice(index, 1)
      edges.push({
        checkKey:
          discovery.source_ip +
          discovery.destination_ip +
          discovery.protocol +
          discovery.destination_port,
        from: discovery.source_ip,
        allowed_comm: discovery.allowed_comm,
        to: discovery.destination_ip,
        name: sourceName,
        id: discovery.id,
        destinationName: destinationName,
        protocol: discovery.protocol.toUpperCase(),
        destinationPort: discovery.destination_port,
        microsegTag: discovery.microsegment_tag,
        workloadTag: discovery.workload_tag,
        srcWorkloadTag: discovery.source_workload_tag,
        client:
          (discovery.Client && discovery.Client.Name) || discovery.client_id,
        ...(discovery.allowed_comm = stroke),
        isDisabled,
        type: 'manual'
      });
    }

    const edgesUniqueByKey = [
      ...new Map(edges.map(item => [item['checkKey'], item])).values()
    ];

    const filteredEdges = edgesUniqueByKey.filter(edge => {
      const searchLower = search.toLowerCase(); // Convert search terms to lowercase for case-insensitive matching

      // Search all relevant properties using a dynamic approach
      const propertiesToSearch = [
        'name',
        'to',
        'from',
        'destinationName',
        'protocol',
        'microsegTag',
        'workloadTag'
      ];
      const foundMatch = propertiesToSearch.some(property => {
        // Access property value dynamically and normalize for lowercase search
        const propertyValue = edge[property] || '';
        const valueLower = propertyValue.toLowerCase();

        // Match based on equality, substring search, or regular expression (flexible approach)
        return (
          valueLower.includes(searchLower) ||
          new RegExp(searchLower).test(valueLower)
        ); // Optional regular expression search
      });

      return foundMatch;
    });

    // filteredEdges now contains only the items that match the search value
    const totalItems = edgesUniqueByKey.length;
    const totalPages = Math.ceil(totalItems / limit);
    const startIndex = offset;
    const endIndex = Math.min(startIndex + limit, totalItems);
    const paginatedEdges = edgesUniqueByKey.slice(startIndex, endIndex);

    const data = {
      nodes: Object.values(nodes),
      edges: search.length > 0 ? filteredEdges : paginatedEdges,
      pagination: {
        currentPage: parseInt(offset),
        pageSize: parseInt(limit),
        totalPages: totalPages,
        totalItems: search.length > 0 ? filteredEdges.length : totalItems
      }
    };

    res.status(200).send(data);
  } catch (error) {
    next(error);
  }
};

const getAllPaginatedS2SDiscoveryData = async (req, res, next) => {
  try {
    let { limit, offset } = req.query;
    let { search = '' } = req.query;

    limit = parseInt(limit);
    offset = parseInt(offset);

    const workloadPivots = await models.WorkloadDevicePivots.findAll({
      attributes: [
        [Sequelize.fn('DISTINCT', Sequelize.col('clientId')), 'clientId']
      ]
    });
    const clientIds = workloadPivots.map(pivot => pivot.clientId);

    const totalDiscoveryCount = await models.S2sDiscoveryData.count({
      
    });

    let allDiscovery;
    if (search) {
      allDiscovery = await models.S2sDiscoveryData.findAll({
        include: [
          {
            model: models.DeviceInventories,
            attributes: ['id', 'device_name'],
            as: 'destinationDevice'
          },
          {
            model: models.DeviceInventories,
            attributes: ['id', 'device_name'],
            as: 'sourceDevice'
          },
          {
            model: models.Client,
            attributes: ['id', 'Name']
          }
        ]
      });
    } else {
      allDiscovery = await models.S2sDiscoveryData.findAll({
        include: [
          {
            model: models.DeviceInventories,
            attributes: ['id', 'device_name'],
            as: 'destinationDevice'
          },
          {
            model: models.DeviceInventories,
            attributes: ['id', 'device_name'],
            as: 'sourceDevice'
          },
          {
            model: models.Client,
            attributes: ['id', 'Name']
          }
        ]
      });
    }

    let toFilterQuery = `        
      SELECT DISTINCT workloads.id, workloads.name, 
      deviceInventories.device_name, deviceInventories.ip_address  
      FROM Workloads workloads
      INNER JOIN WorkloadDevicePivots AS workloadDevicePivots ON workloadDevicePivots.workloadId = workloads.id 
      INNER JOIN DeviceInventories AS deviceInventories ON deviceInventories.id = workloadDevicePivots.deviceId
      INNER JOIN S2sDiscoveryData s2sDiscoveryData ON s2sDiscoveryData.workloadId = workloads.id`;

    let toFilterResult = (await models.sequelize.query(toFilterQuery))[0];

    console.log('toFilterResult ', toFilterResult);
    // res.status(200).send(toFilterResult);

    // return toFilterResult

    const totalManualDiscoveryCount = await models.S2sManualRule.count({
    });

    let manualDiscovery;
    if (search) {
      manualDiscovery = await models.S2sManualRule.findAll();
    } else {
      manualDiscovery = await models.S2sManualRule.findAll();
    }

    const nodes = {};
    const edges = [];

    for (const discovery of allDiscovery) {
      let totalCount = 0;
      let totalAllowed = 0;
      let totalNotAllowed = 0;
      let manualtotalCount = 0;
      let manualtotalAllowed = 0;

      let manualdisData = await models.sequelize.query(
        `
        SELECT sum(allowed_comm) AS totalAllowed, count(id) AS total
        FROM S2sManualRules
        WHERE 
            (source_ip = '${discovery.source_ip}' AND destination_ip = '${discovery.destination_ip}')
            OR
            (source_ip = '${discovery.destination_ip}' AND destination_ip = '${discovery.source_ip}')        
            `,
        { raw: true, type: sequelize.QueryTypes.SELECT }
      );

      manualtotalCount = manualdisData[0].total;
      manualtotalAllowed = manualdisData[0].totalAllowed;

      let disData = await models.sequelize.query(
        `SELECT 
          SUM(allowed_comm) AS totalAllowed, 
          COUNT(id) AS total
          FROM 
              S2sDiscoveryData
          WHERE 
              (source_ip = '${discovery.source_ip}' AND destination_ip = '${discovery.destination_ip}')
              OR
              (source_ip = '${discovery.destination_ip}' AND destination_ip = '${discovery.source_ip}')
        `,
        { raw: true, type: sequelize.QueryTypes.SELECT }
      );

      if (manualtotalCount && manualtotalAllowed) {
        totalCount = parseInt(disData[0].total) + parseInt(manualtotalCount);
        totalAllowed =
          parseInt(disData[0].totalAllowed) + parseInt(manualtotalAllowed);
        totalNotAllowed = parseInt(totalCount) - parseInt(totalAllowed);
      } else {
        totalCount = disData[0].total;
        totalAllowed = disData[0].totalAllowed;
        totalNotAllowed = parseInt(totalCount) - parseInt(totalAllowed);
      }

      // console.log('discovery: ', results);
      let destinationName =
        (discovery &&
          discovery.destinationDevice &&
          `${discovery.destinationDevice.device_name}`) ||
        discovery.destination_ip;

      if (
        !toFilterResult.some(s2sdata => s2sdata.device_name == destinationName)
      ) {
        destinationName = discovery.destination_ip;
      }

      if (!nodes[discovery.destination_ip]) {
        if (
          toFilterResult.some(s2sdata => s2sdata.device_name == destinationName)
        ) {
          nodes[discovery.destination_ip] = {
            id: discovery.destination_ip,
            fill: {
              src: 'assets/images/laptop.png'
            },
            name: destinationName,
            device: discovery.deviceId
          };
        } else {
          nodes[discovery.destination_ip] = {
            id: discovery.destination_ip,
            fill: {
              src: 'assets/images/laptop-red.png'
            },
            name: destinationName,
            device: discovery.deviceId
          };
        }
      }

      let sourceName =
        (discovery &&
          discovery.sourceDevice &&
          `${discovery.sourceDevice.device_name}`) ||
        discovery.source_ip;

      if (!toFilterResult.some(s2sdata => s2sdata.device_name == sourceName)) {
        sourceName = discovery.source_ip;
      }

      if (!nodes[discovery.source_ip]) {
        if (toFilterResult.some(s2sdata => s2sdata.device_name == sourceName)) {
          nodes[discovery.source_ip] = {
            id: discovery.source_ip,
            fill: {
              src: 'assets/images/laptop.png'
            },
            name: sourceName,
            device: discovery.deviceId
          };
        } else {
          nodes[discovery.source_ip] = {
            id: discovery.source_ip,
            fill: {
              src: 'assets/images/laptop-red.png'
            },
            name: sourceName,
            device: discovery.deviceId
          };
        }
      }

      let colourIs = 'red';
      let dashIs =
        parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed) > 0
          ? '10 5'
          : '0';
      let isDisabled = false;
      if (
        !discovery?.sourceDevice?.device_name ||
        !toFilterResult.some(s2sdata => s2sdata.device_name == sourceName)
      ) {
        discovery.allowed_comm ? (colourIs = '#32a852') : (colourIs = 'red');
        dashIs =
          parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed) > 0
            ? '10 5'
            : '0';
        isDisabled = true;
      } else if (parseInt(totalAllowed) === parseInt(totalCount)) {
        colourIs = '#32a852';
        dashIs =
          parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed) > 0
            ? '10 5'
            : '0';
      } else {
        colourIs = 'red';
        dashIs =
          parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed) > 0
            ? '10 5'
            : '0';
      }

      let stroke = {
        normal: {
          stroke: {
            color: colourIs,
            thickness: '2',
            dash: dashIs,
            lineJoin: 'round'
          }
        },
        hovered: {
          stroke: {
            color: colourIs,
            thickness: '3',
            dash: dashIs,
            lineJoin: 'round'
          }
        },

        selected: {
          stroke: {
            color: colourIs,
            thickness: '3',
            dash: dashIs,
            lineJoin: 'round'
          }
        }
      };

      edges.push({
        checkKey:
          discovery.source_ip +
          discovery.destination_ip +
          discovery.protocol +
          discovery.destination_port,
        from: discovery.source_ip,
        allowed_comm: discovery.allowed_comm,
        to: discovery.destination_ip,
        name: sourceName,
        id: discovery.id,
        destinationName: destinationName,
        protocol: discovery.protocol.toUpperCase(),
        destinationPort: discovery.destination_port,
        microsegTag: discovery.microsegment_tag,
        workloadTag: discovery.workload_tag,
        srcWorkloadTag: discovery.source_workload_tag,
        client:
          (discovery.Client && discovery.Client.Name) || discovery.client_id,
        ...(discovery.allowed_comm = stroke),
        isDisabled,
        type: 'auto'
      });
    }

    for (const discovery of manualDiscovery) {
      let totalCount = 0;
      let totalAllowed = 0;
      let totalNotAllowed = 0;
      let s2stotalCount = 0;
      let s2stotalAllowed = 0;

      let s2sdisData = await models.sequelize.query(
        `
        SELECT 
        SUM(allowed_comm) AS totalAllowed, 
        COUNT(id) AS total
        FROM 
            S2sDiscoveryData
        WHERE 
            ((source_ip = '${discovery.source_ip}' AND destination_ip = '${discovery.destination_ip}')
            OR
            (source_ip = '${discovery.destination_ip}' AND destination_ip = '${discovery.source_ip}'))
            `,
        { raw: true, type: sequelize.QueryTypes.SELECT }
      );

      s2stotalCount = s2sdisData[0].total;
      s2stotalAllowed = s2sdisData[0].totalAllowed;

      let disData = await models.sequelize.query(
        `
        SELECT 
        SUM(allowed_comm) AS totalAllowed, 
        COUNT(id) AS total
        FROM 
            S2sManualRules
        WHERE 
            ((source_ip = '${discovery.source_ip}' AND destination_ip = '${discovery.destination_ip}')
            OR
            (source_ip = '${discovery.destination_ip}' AND destination_ip = '${discovery.source_ip}'))
            `,
        { raw: true, type: sequelize.QueryTypes.SELECT }
      );

      if (s2stotalCount && s2stotalAllowed) {
        totalCount = parseInt(disData[0].total) + parseInt(s2stotalCount);
        totalAllowed =
          parseInt(disData[0].totalAllowed) + parseInt(s2stotalAllowed);
        totalNotAllowed = parseInt(totalCount) - parseInt(totalAllowed);
      } else {
        totalCount = disData[0].total;
        totalAllowed = disData[0].totalAllowed;
        totalNotAllowed = parseInt(totalCount) - parseInt(totalAllowed);
      }

      let destinationName =
        discovery.destination_device || discovery.destination_ip;

      if (!nodes[discovery.destination_ip]) {
        nodes[discovery.destination_ip] = {
          id: discovery.destination_ip,
          fill: {
            src: 'assets/images/laptop.png'
          },
          name: destinationName
        };
      }

      let sourceName = discovery.source_device || discovery.source_ip;

      if (!toFilterResult.some(s2sdata => s2sdata.device_name == sourceName)) {
        sourceName = discovery.source_device || discovery.source_ip;
      }

      if (!nodes[discovery.source_ip]) {
        nodes[discovery.source_ip] = {
          id: discovery.source_ip,
          fill: {
            src: 'assets/images/laptop.png'
          },
          name: sourceName
        };
      }

      let colourIs = 'red';
      let dashIs =
        (parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed)) > 0
          ? '10 5'
          : '0';
      let isDisabled = false;
      if (!toFilterResult.some(s2sdata => s2sdata.device_name == sourceName)) {
        discovery.allowed_comm ? (colourIs = '#32a852') : (colourIs = 'red');
        dashIs =
          (parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed)) > 0
            ? '10 5'
            : '0';
        isDisabled = false;
      } else if (parseInt(totalAllowed) === parseInt(totalCount)) {
        colourIs = '#32a852';
        dashIs =
          (parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed)) > 0
            ? '10 5'
            : '0';
      } else {
        colourIs = 'red';
        dashIs =
          (parseInt(totalAllowed) > 0 && parseInt(totalNotAllowed)) > 0
            ? '10 5'
            : '0';
      }

      let stroke = {
        normal: {
          stroke: {
            color: colourIs,
            thickness: '2',
            dash: dashIs,
            lineJoin: 'round'
          }
        },
        hovered: {
          stroke: {
            color: colourIs,
            thickness: '3',
            dash: dashIs,
            lineJoin: 'round'
          }
        },

        selected: {
          stroke: {
            color: colourIs,
            thickness: '3',
            dash: dashIs,
            lineJoin: 'round'
          }
        }
      };

      // let doesThatAlreadyExist = false
      // const index = edges.findIndex(edge => edge.checkKey = discovery.source_ip+discovery.destination_ip+discovery.protocol+discovery.destination_port)
      // if(index) edges.splice(index, 1)
      edges.push({
        checkKey:
          discovery.source_ip +
          discovery.destination_ip +
          discovery.protocol +
          discovery.destination_port,
        from: discovery.source_ip,
        allowed_comm: discovery.allowed_comm,
        to: discovery.destination_ip,
        name: sourceName,
        id: discovery.id,
        destinationName: destinationName,
        protocol: discovery.protocol.toUpperCase(),
        destinationPort: discovery.destination_port,
        microsegTag: discovery.microsegment_tag,
        workloadTag: discovery.workload_tag,
        srcWorkloadTag: discovery.source_workload_tag,
        client:
          (discovery.Client && discovery.Client.Name) || discovery.client_id,
        ...(discovery.allowed_comm = stroke),
        isDisabled,
        type: 'manual'
      });
    }

    const edgesUniqueByKey = [
      ...new Map(edges.map(item => [item['checkKey'], item])).values()
    ];

    const filteredEdges = edgesUniqueByKey.filter(edge => {
      const searchLower = search.toLowerCase(); // Convert search terms to lowercase for case-insensitive matching

      // Search all relevant properties using a dynamic approach
      const propertiesToSearch = [
        'name',
        'to',
        'from',
        'destinationName',
        'protocol',
        'microsegTag',
        'workloadTag',
      ];
      const foundMatch = propertiesToSearch.some(property => {
        // Access property value dynamically and normalize for lowercase search
        const propertyValue = edge[property] || '';
        const valueLower = propertyValue.toLowerCase();

        // Match based on equality, substring search, or regular expression (flexible approach)
        return (
          valueLower.includes(searchLower) ||
          new RegExp(searchLower).test(valueLower)
        ); // Optional regular expression search
      });

      return foundMatch;
    });

    // filteredEdges now contains only the items that match the search value
    const totalItems = edgesUniqueByKey.length;
    const totalPages = Math.ceil(totalItems / limit);
    const startIndex = offset;
    const endIndex = Math.min(startIndex + limit, totalItems);
    const paginatedEdges = edgesUniqueByKey.slice(startIndex, endIndex);

    const data = {
      nodes: Object.values(nodes),
      edges: search.length > 0 ? filteredEdges : paginatedEdges,
      pagination: {
        currentPage: parseInt(offset),
        pageSize: parseInt(limit),
        totalPages: totalPages,
        totalItems: search.length > 0 ? filteredEdges.length : totalItems
      }
    };

    res.status(200).send(data);
  } catch (error) {
    next(error);
  }
};

const getManualRuleData = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    const manualRuleData = await models.S2sManualRule.findAll(
      {
        where: {
          [Op.or]: [
            { workloadId: req.params.workloadId },
            { src_workload_id: req.params.workloadId }
          ]
        },
      },
      { transaction }
    );

    await transaction.commit();
    res.json({ manualRuleData });
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const getAllManualRuleData = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    const manualRuleData = await models.S2sManualRule.findAll();

    await transaction.commit();
    res.json({ manualRuleData });
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const getWorkloadDevices = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    const workloadPivots = await models.S2sDiscoveryData.findAll({
      where: {
        [Op.or]: [
          { workloadId: req.params.workloadId },
          { src_workload_id: req.params.workloadId }
        ]
      },
      attributes: ['destination_ip', 'source_ip']
    });

    const manualWorkloadPivots = await models.S2sManualRule.findAll({
      where: {
        [Op.or]: [
          { workloadId: req.params.workloadId },
          { src_workload_id: req.params.workloadId }
        ]
      },
      attributes: ['source_ip', 'destination_ip']
    });

    const workloadIPs = workloadPivots.flatMap(pivot => [pivot.destination_ip, pivot.source_ip]);
    const manualWorkloadIPs = manualWorkloadPivots.flatMap(pivot => [pivot.source_ip, pivot.destination_ip]);

    const uniqueIPs = [...new Set([...workloadIPs, ...manualWorkloadIPs])];

    const devicesData = await models.DeviceInventories.findAll({
      where: {
        ip_address: {
          [Op.in]: uniqueIPs
        },
        [Op.or]: [
          { device_type: 'Physical Server' },
          { device_type: 'Virtual Server' }
        ]
      }
    });
    await transaction.commit();
    res.json({ devicesData });
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const getAllWorkloadDevices = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    const devicesData = await models.DeviceInventories.findAll({
      where: {
        device_type: {
          [Op.or]: ['Physical Server', 'Virtual Server']
        }
      }
    });
    await transaction.commit();
    res.json({ devicesData });
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const s2sDiscoveryManualData = async (req, res, next) => {
  const { manualData } = req.body;
  const transaction = await models.sequelize.transaction();
  try {
    let isSourceDeviceWorkloadDeviceQuery = `        
        SELECT DISTINCT workloads.id, workloads.name, 
        deviceInventories.device_name, deviceInventories.ip_address  
        FROM Workloads workloads
        INNER JOIN WorkloadDevicePivots AS workloadDevicePivots ON workloadDevicePivots.workloadId = workloads.id 
        INNER JOIN DeviceInventories AS deviceInventories ON deviceInventories.id = workloadDevicePivots.deviceId
        INNER JOIN S2sDiscoveryData s2sDiscoveryData ON s2sDiscoveryData.workloadId = workloads.id 
        WHERE deviceInventories.ip_address  = :selSourceIp`;

    let isSourceDeviceWorkloadDevice = (
      await models.sequelize.query(isSourceDeviceWorkloadDeviceQuery, {
        replacements: { selSourceIp: manualData.selSourceIp },
        type: models.sequelize.QueryTypes.SELECT,
      })
    );

    let isDestinationDeviceWorkloadDeviceQuery = `        
        SELECT DISTINCT workloads.id, workloads.name, 
        deviceInventories.device_name, deviceInventories.ip_address  
        FROM Workloads workloads
        INNER JOIN WorkloadDevicePivots AS workloadDevicePivots ON workloadDevicePivots.workloadId = workloads.id 
        INNER JOIN DeviceInventories AS deviceInventories ON deviceInventories.id = workloadDevicePivots.deviceId
        INNER JOIN S2sDiscoveryData s2sDiscoveryData ON s2sDiscoveryData.workloadId = workloads.id 
        WHERE deviceInventories.ip_address  = :selDestinationIp`;

    let isDestinationDeviceWorkloadDevice = (
      await models.sequelize.query(isDestinationDeviceWorkloadDeviceQuery, {
        replacements: { selDestinationIp: manualData.selDestinationIp },
        type: models.sequelize.QueryTypes.SELECT,
      })
    );

    const client = await models.Client.findOne({
      where: {
        device_id: manualData.deviceId
      }
    });

    if (!client) {
      await transaction.rollback();
      return res.status(400).json({
        success: false
      });
    }

    const validProtocols = ['tcp', 'udp'];
    
    if (!validProtocols.includes(manualData.selProtocol.toLowerCase())) {
      await transaction.rollback();
      return res.status(400).json({
        success: false,
        message: 'Invalid protocol. Only "tcp" and "udp" are allowed.',
      });
    }

    const workload = await models.S2sManualRule.findOne(
      {
        where: {
          source_ip: manualData.selSourceIp,
          destination_ip: manualData.selDestinationIp,
          protocol: manualData.selProtocol.toLowerCase(),
          destination_port: manualData.selPort,
          [Op.or]: [
            { workloadId: req.params.workloadId },
            { src_workload_id: req.params.workloadId }
          ]
        },
      },
      { transaction }
    );

    if (workload) {
      throw new httpErrors.Conflict('Manual rule already exist');
    }

    const workloadTag = await models.Workloads.findOne({
      where: {
        id: req.params.workloadId
      }
    })

    await models.S2sManualRule.create(
      {
        source_device:
          isSourceDeviceWorkloadDevice.length > 0
            ? isSourceDeviceWorkloadDevice[0].device_name
            : manualData.selSourceDevice,
        source_ip: manualData.selSourceIp,
        destination_device:
          isDestinationDeviceWorkloadDevice.length > 0
            ? isDestinationDeviceWorkloadDevice[0].device_name
            : manualData.selDestDevice,
        destination_ip: manualData.selDestinationIp,
        destination_port: manualData.selPort,
        protocol: manualData.selProtocol.toLowerCase(),
        workloadId: req.params.workloadId,
        src_workload_id: req.params.workloadId,
        source_workload_tag: workloadTag.name,
        workload_tag: workloadTag.name,
        deviceId: manualData.deviceId,
        client_id: client.dataValues.id,
        allowed_comm: isSourceDeviceWorkloadDevice.length > 0 ? true : false
      },
      { transaction }
    );
    await transaction.commit();
    res.json('Manual rule added');
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const allS2sDiscoveryManualData = async (req, res, next) => {
  const { manualData } = req.body;
  const transaction = await models.sequelize.transaction();
  try {
      let isSourceDeviceWorkloadDeviceQuery = `
        SELECT DISTINCT 
          workloads.id, 
          workloads.name, 
          deviceInventories.device_name, 
          deviceInventories.ip_address  
        FROM Workloads AS workloads
        INNER JOIN WorkloadDevicePivots AS workloadDevicePivots 
          ON workloadDevicePivots.workloadId = workloads.id 
        INNER JOIN DeviceInventories AS deviceInventories 
          ON deviceInventories.id = workloadDevicePivots.deviceId
        INNER JOIN S2sDiscoveryData AS s2sDiscoveryData 
          ON s2sDiscoveryData.workloadId = workloads.id 
        WHERE deviceInventories.ip_address = :selSourceIp`;

    let isSourceDeviceWorkloadDevice = (
      await models.sequelize.query(isSourceDeviceWorkloadDeviceQuery, {
        replacements: { selSourceIp: manualData.selSourceIp },
        type: models.sequelize.QueryTypes.SELECT,
      })
    );

    let isDestinationDeviceWorkloadDeviceQuery = `        
        SELECT DISTINCT workloads.id, workloads.name, 
        deviceInventories.device_name, deviceInventories.ip_address  
        FROM Workloads workloads
        INNER JOIN WorkloadDevicePivots AS workloadDevicePivots ON workloadDevicePivots.workloadId = workloads.id 
        INNER JOIN DeviceInventories AS deviceInventories ON deviceInventories.id = workloadDevicePivots.deviceId
        INNER JOIN S2sDiscoveryData s2sDiscoveryData ON s2sDiscoveryData.workloadId = workloads.id 
        WHERE deviceInventories.ip_address  = :selDestinationIp`;

    let isDestinationDeviceWorkloadDevice = (
      await models.sequelize.query(isDestinationDeviceWorkloadDeviceQuery, {
        replacements: { selDestinationIp: manualData.selDestinationIp },
        type: models.sequelize.QueryTypes.SELECT,
      })
    );

    const client = await models.Client.findOne({
      where: {
        device_id: manualData.deviceId
      }
    });

    if (!client) {
      await transaction.rollback();
      return res.status(400).json({
        success: false
      });
    }

    const getDefaultWorkload = await models.Workloads.findOne({
      where:{
        name: 'Default'
      }
    });

    const validProtocols = ['tcp', 'udp'];

    if (!validProtocols.includes(manualData.selProtocol.toLowerCase())) {
      return res.status(400).json({
        success: false,
        message: 'Invalid protocol. Only "tcp" and "udp" are allowed.',
      });
    }

    //Add Tag here
    const workload = await models.S2sManualRule.findOne({
      where: {
        [Op.and]: [
          { source_ip: manualData.selSourceIp },
          { destination_ip: manualData.selDestinationIp },
          { protocol: manualData.selProtocol.toLowerCase() },
          { destination_port: manualData.selPort },
          {
            [Op.or]: [
              { workloadId: getDefaultWorkload.id },
              { src_workload_id: getDefaultWorkload.id }
            ]
          }
        ]
      },
      transaction
    });

    if (workload) {
      throw new httpErrors.Conflict('Manual rule already exist');
    }

    //Add Tag here
    //Tag is hardcoded see this
    await models.S2sManualRule.create(
      {
        source_device:
          isSourceDeviceWorkloadDevice.length > 0
            ? isSourceDeviceWorkloadDevice[0].device_name
            : manualData.selSourceDevice,
        source_ip: manualData.selSourceIp,
        destination_device:
          isDestinationDeviceWorkloadDevice.length > 0
            ? isDestinationDeviceWorkloadDevice[0].device_name
            : manualData.selDestDevice,
        destination_ip: manualData.selDestinationIp,
        destination_port: manualData.selPort,
        protocol: manualData.selProtocol.toLowerCase(),
        workloadId: getDefaultWorkload.id,
        src_workload_id: getDefaultWorkload.id,
        deviceId: manualData.deviceId,
        client_id: client.dataValues.id,
        allowed_comm: isSourceDeviceWorkloadDevice.length > 0 ? true : false
      },
      { transaction }
    );
    await transaction.commit();
    res.json('Manual rule added');
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const deleteManualRule = async (req, res, next) => {
  const {
    source_device,
    source_ip,
    destination_device,
    destination_ip,
    destination_port,
    protocol,
    workloadId
  } = req.body;
  const transaction = await models.sequelize.transaction();
  try {
    await models.S2sManualRule.destroy({
      where: {
        [Op.and]: [
          { source_device: source_device },
          { source_ip: source_ip },
          { destination_device: destination_device },
          { destination_ip: destination_ip },
          { destination_port: destination_port },
          { protocol: protocol },
          {
            [Op.or]: [
              { workloadId: workloadId },
              { src_workload_id: workloadId }
            ]
          }
        ]
      },
      transaction
    });

    const allWorkloadIds = await models.Workloads.findAll({
      attributes: ['id']
    });

    //Remove Orphan Workloads from system
    for (const workload of allWorkloadIds) {
      const workloadId = workload.id
      const existsInS2sDiscoverydata = await models.S2sDiscoveryData.count({
        where: {
          [Op.or]: [
            { workloadId: workloadId },
            { src_workload_id: workloadId }
          ]
        },
      });
    
      const existsInManualRulesData = await models.S2sManualRule.count({
        where: {
          [Op.or]: [
            { workloadId: workloadId },
            { src_workload_id: workloadId }
          ]
        },
      });
    
      if (existsInS2sDiscoverydata === 0 && existsInManualRulesData === 0) {
        await models.Workloads.destroy({
          where: {
            id: workloadId
          }
        });
      }
    }

    await transaction.commit();
    res.json('Rule deleted successfully');
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const deleteFromAllManualRules = async (req, res, next) => {
  const {
    source_device,
    source_ip,
    destination_device,
    destination_ip,
    destination_port,
    protocol,
    workloadTag,
    srcWorkloadTag
  } = req.body;
  const transaction = await models.sequelize.transaction(); 
  
  try {
    await models.S2sManualRule.destroy(
      {
        where: {
          [Op.and]: [
            { source_device: source_device },
            { source_ip: source_ip },
            { destination_device: destination_device },
            { destination_ip: destination_ip },
            { destination_port: destination_port },
            { protocol: protocol },
            { workload_tag: workloadTag },
            { source_workload_tag: srcWorkloadTag}
          ]
        }
      },
      { transaction }
    );

    const allWorkloadIds = await models.Workloads.findAll({
      attributes: ['id']
    });

    //Remove Orphan Workloads from system
    for (const workload of allWorkloadIds) {
      const workloadId = workload.id
      const existsInS2sDiscoverydata = await models.S2sDiscoveryData.count({
        where: {
          [Op.or]: [
            { workloadId: workloadId },
            { src_workload_id: workloadId }
          ]
        },
      });
    
      const existsInManualRulesData = await models.S2sManualRule.count({
        where: {
          [Op.or]: [
            { workloadId: workloadId },
            { src_workload_id: workloadId }
          ]
        },
      });
    
      if (existsInS2sDiscoverydata === 0 && existsInManualRulesData === 0) {
        await models.Workloads.destroy({
          where: {
            id: workloadId
          }
        });
      }
    }

    await transaction.commit();
    res.json('Rule deleted successfully');
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const deleteFromAllAutoRules = async (req, res, next) => {
  const {
    id
  } = req.body;
  const transaction = await models.sequelize.transaction();
  
  try {
    const prevWorkloadId = await models.S2sDiscoveryData.findOne({
      where:{
        id: id
      }
    })

    const defaultWorkload = await models.Workloads.findOne({
      where: {
        name: 'Default'
      }
    })

    const defaultWorkloadId =  defaultWorkload.id;


    await models.S2sDiscoveryData.destroy(
      {
        where: {
          id: id
        }
      },
      { transaction }
    );


    const allWorkloadIds = await models.Workloads.findAll({
      attributes: ['id']
    });

    //Remove Orphan Workloads from system
    for (const workload of allWorkloadIds) {
      const workloadId = workload.id
      const existsInS2sDiscoverydata = await models.S2sDiscoveryData.count({
        where: {
          [Op.or]: [
            { workloadId: workloadId },
            { src_workload_id: workloadId }
          ]
        },
      });
    
      const existsInManualRulesData = await models.S2sManualRule.count({
        where: {
          [Op.or]: [
            { workloadId: workloadId },
            { src_workload_id: workloadId }
          ]
        },
      });
    
      if (existsInS2sDiscoverydata === 0 && existsInManualRulesData === 0) {
        await models.Workloads.destroy({
          where: {
            id: workloadId
          }
        });
      }
    }

    await transaction.commit();
    res.json('Rule deleted successfully');
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const flushDiscovery = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {

    await sequelize.query(`
      DELETE FROM S2sDiscoveryData;
    `);

    await sequelize.query(`
    DELETE FROM Workloads
    WHERE name != 'Default'
      AND id NOT IN (
          SELECT workloadId FROM S2sManualRules
      );
`);


    await transaction.commit();
    res.json('Dicovery Data flushed successfully');
  } catch (error) {
    await transaction.rollback();
    next(error);
  }
};

const getRefreshPreviousWorkload = async () => {
  try {
    const defaultWorkload = await models.Workloads.findOne({
      where: {
        name: 'Default'
      },
      attributes: ['id'],
      raw: true,
    });

    // Retrieve the default workload ID
    const defaultWorkloadId = defaultWorkload ? defaultWorkload.id : null;

    // Fetch all workloads
    const workloads = await models.Workloads.findAll({
      attributes: ['id'],
      raw: true,
    });


    // Iterate over each workload ID
    for (const workload of workloads) {
      const workloadId = workload.id;
      //COMMENTING IT FOR NOW
      // if (workloadId === defaultWorkloadId) {
      //   continue;
      // }
      
      const s2sDiscoveryDataRecords = await models.S2sDiscoveryData.findAll({
        where: {
          [Op.or]: [
            { workloadId: workloadId },
            { src_workload_id: workloadId }
          ]
        },
        attributes: ['source_ip', 'destination_ip'],
        raw: true
      });

      const s2sManualRuleRecords = await models.S2sManualRule.findAll({
        where: {
          [Op.or]: [
            { workloadId: workloadId },
            { src_workload_id: workloadId }
          ]
        },
        attributes: ['source_ip', 'destination_ip'],
        raw: true
      });

      const discoveryDataIps = s2sDiscoveryDataRecords.reduce((ips, record) => {
        ips.push(record.source_ip, record.destination_ip);
        return ips;
      }, []);

      const manualRuleIps = s2sManualRuleRecords.reduce((ips, record) => {
        ips.push(record.source_ip, record.destination_ip);
        return ips;
      }, []);

      const allIps = [...new Set([...discoveryDataIps, ...manualRuleIps])];

      const deviceRecords = await models.DeviceInventories.findAll({
        where: {
          ip_address: {
            [Op.in]: allIps
          },
          device_type: {
            [Op.or]: ['Physical Server', 'Virtual Server']
          }
        },
        attributes: ['id'],
        raw: true
      });

      const workloadsIdstoUpdate = deviceRecords.map(record => record.id);

      const existingPivots = await models.WorkloadDevicePivots.findAll({
        where: {
          workloadId: workloadId
        },
        attributes: ['deviceId'],
        raw: true
      });

      const existingDeviceIds = new Set(existingPivots.map(pivot => pivot.deviceId));

      const newDeviceIds = new Set(workloadsIdstoUpdate);

      const deviceIdsToRemove = [...existingDeviceIds].filter(deviceId => !newDeviceIds.has(deviceId));

      const defaultWL = await models.Workloads.findOne({
        where: {
          name: 'Default'
        }
      });

      if (deviceIdsToRemove.length > 0) {
        await models.WorkloadDevicePivots.destroy({
          where: {
            workloadId: workloadId,
            deviceId: deviceIdsToRemove
          }
        });
      }
    }
  } catch (e) {
    console.log('error', e);
  }
};

const getRefreshWorkloads = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  
  try {
    const { workloadId } = req.params;

    const s2sDiscoveryDataRecords = await models.S2sDiscoveryData.findAll({
      where: {
        [Op.or]: [
          { workloadId: workloadId },
          { src_workload_id: workloadId }
        ]
      },
      attributes: ['source_ip', 'destination_ip'],
      raw: true,
      transaction
    });

    const s2sManualRuleRecords = await models.S2sManualRule.findAll({
      where: {
        [Op.or]: [
          { workloadId: workloadId },
          { src_workload_id: workloadId }
        ]
      },
      attributes: ['source_ip', 'destination_ip'],
      raw: true,
      transaction
    });

    const discoveryDataIps = s2sDiscoveryDataRecords.reduce((ips, record) => {
      ips.push(record.source_ip, record.destination_ip);
      return ips;
    }, []);

    const manualRuleIps = s2sManualRuleRecords.reduce((ips, record) => {
      ips.push(record.source_ip, record.destination_ip);
      return ips;
    }, []);

    const allIps = [...new Set([...discoveryDataIps, ...manualRuleIps])];

    const deviceRecords = await models.DeviceInventories.findAll({
      where: {
        ip_address: {
          [Op.in]: allIps
        },
        device_type: {
          [Op.or]: ['Physical Server', 'Virtual Server']
        }
      },
      attributes: ['id'],
      raw: true,
      transaction
    });

    const workloadsIdstoUpdate = deviceRecords.map(record => record.id);

    const existingPivots = await models.WorkloadDevicePivots.findAll({
      where: {
        workloadId: workloadId
      },
      attributes: ['deviceId'],
      raw: true,
      transaction
    });

    const existingDeviceIds = new Set(existingPivots.map(pivot => pivot.deviceId));

    const newDeviceIds = new Set(workloadsIdstoUpdate);

    const deviceIdsToRemove = [...existingDeviceIds].filter(deviceId => !newDeviceIds.has(deviceId));

    const defaultWL = await models.Workloads.findOne({
      where: {
        name: 'Default'
      }
    });

    if (deviceIdsToRemove.length > 0) {
      await models.WorkloadDevicePivots.destroy({
        where: {
          workloadId: workloadId,
          deviceId: deviceIdsToRemove
        },
        transaction
      });
    }

    //Send Pub/Sub to controller TBD

    await transaction.commit();
    
    res.status(200).json({ message: 'refreshed' });
  } catch (e) {
    await transaction.rollback();
    next(e);
  }
};

const getWorklaodName = async (req, res, next) => {
  try {
    const { workloadId } = req.params;

    const workload = await models.Workloads.findOne({
      where: {
        id: workloadId
      },
      attributes: ['name']
    })
    
    res.status(200).json(workload);
  } catch (e) {
    next(e);
  }
};

const toggleS2sDiscovery = async (req, res, next) => {
  try {
    const { discoveryId } = req.params;
    const type = req.body.type;
    if (type === 'auto') {
      const discovery = await models.S2sDiscoveryData.findOne({
        where: {
          id: discoveryId
        }
      });
      discovery.allowed_comm = !discovery.allowed_comm;
      console.log('discovery.allowed_comm ', discovery.allowed_comm);
      await discovery.save();
      res
        .status(200)
        .json({ messag: 'updated', status: discovery.allowed_comm });
    } else if (type === 'manual') {
      const manualDiscovery = await models.S2sManualRule.findOne({
        where: {
          id: discoveryId
        }
      });

      manualDiscovery.allowed_comm = !manualDiscovery.allowed_comm;
      console.log(
        'manualDiscovery.allowed_comm ',
        manualDiscovery.allowed_comm
      );
      await manualDiscovery.save();
      res
        .status(200)
        .json({ message: 'updated', status: manualDiscovery.allowed_comm });
    } else return res.status(400).send('Invalid parameters');
  } catch (e) {
    next(e);
  }
};

const ValidateIPaddress = ipaddress => {
  if (
    /^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/.test(
      ipaddress
    )
  ) {
    return true;
  }
  return false;
};

const hasDuplicates = arrayOfIPs => {
  const seen = new Set();
  for (const item of arrayOfIPs) {
    if (seen.has(item)) {
      return true;
    }
    seen.add(item);
  }
  return false;
};

module.exports = {
  getWorkLoads,
  getAvailableWorkloadDevices,
  createWorkload,
  getWorkloadById,
  editWorkloadById,
  getRefreshWorkloads,
  deleteWorkloadById,
  workloadBulkAction,
  saveWorkDiscovery,
  saveAllWorkDiscovery,
  getS2SDiscoveryData,
  getAllS2SDiscoveryData,
  getPaginatedS2SDiscoveryData,
  getAllPaginatedS2SDiscoveryData,
  toggleS2sDiscovery,
  s2sDiscoveryManualData,
  allS2sDiscoveryManualData,
  getManualRuleData,
  getAllManualRuleData,
  getWorkloadDevices,
  getAllWorkloadDevices,
  deleteManualRule,
  saveMicroSegTag,
  flushDiscovery,
  saveAllMicroSegTag,
  deleteFromAllManualRules,
  deleteFromAllAutoRules,
  getWorklaodName
};
