const httpErrors = require('http-errors');
const redisClient = require('../config/cache');
const util = require('util');

// middleware to check user permissions on each route, incoming privilges is a string or array of strings required to access the route
const verifyPermisions = privileges => {
  return async (req, res, next) => {
    try {
      const httpMethod = req.method;

      const cachedData = await redisClient.getAsync(
        `userTemplate:${req.user.id}`
      );

      if (!cachedData) {
        throw new httpErrors.Unauthorized(
          'Session expiry time exceeded, please login again'
        );
      }

      const parsedData = JSON.parse(cachedData);

      // Check if any of the requested privileges exist in the parsedData array
      const hasPrivilege = parsedData.some(parsedPrivilege =>
        privileges.includes(parsedPrivilege.privilege_name)
      );

      if (!hasPrivilege) {
        throw new httpErrors.Unauthorized(
          'User does not have the required privileges to access the API'
        );
      }

      const checkPermission = privilege => {
        return (
          (httpMethod === 'GET' && privilege.read_permission) ||
          (httpMethod === 'POST' && privilege.create_permission) ||
          (httpMethod === 'PUT' && privilege.update_permission) ||
          (httpMethod === 'PATCH' && privilege.update_permission) ||
          (httpMethod === 'DELETE' && privilege.delete_permission)
        );
      };

      // Check if any of the requested privileges have the required permission
      const hasPermission = parsedData.some(parsedPrivilege => {
        return privileges.includes(parsedPrivilege.privilege_name) && checkPermission(parsedPrivilege);
      });

      if (!hasPermission) {
        throw new httpErrors.Unauthorized(
          'User does not have the required permission to access this API.'
        );
      }

      // User has the required privileges, proceed to the next middleware or route handler
      next();
    } catch (error) {
      next(error);
    }
  };
};

module.exports = verifyPermisions;
