'use strict';
const moment = require('moment');

/** @type {import('sequelize-cli').Migration} */
module.exports = {
  async up(queryInterface, Sequelize) {
    const timezones = getDefinedTimezones();
    const entitlements = await getEntitlements(queryInterface, Sequelize);
    
    // Helper function to get the moment time zone
    function getMomentTimeZone(selectedTimeZone) {
      const timezone = timezones.find(tz => tz.value === selectedTimeZone);
      return timezone ? timezone.moment : null;
    }

    // Helper function to convert local time to UTC
    function convertToUTC(timeString, timeZoneMomentValue) {
      const localTime = moment.tz(
        `1970-01-01 ${timeString}`,
        'YYYY-MM-DD HH:mm',
        timeZoneMomentValue
      );
      const utcTime = localTime.utc();
      return utcTime.format('HH:mm');
    }

    for (const entitlement of entitlements) {
      const timeZone = entitlement.time_zone;
      const timeZoneMoment = getMomentTimeZone(timeZone);

      if (timeZoneMoment) {
        const convertedStartTime = convertToUTC(
          entitlement.access_start_time,
          timeZoneMoment
        );
        const convertedEndTime = convertToUTC(
          entitlement.access_end_time,
          timeZoneMoment
        );

        await queryInterface.sequelize.query(
          `UPDATE Entitlements SET access_start_time = :startTime, access_end_time = :endTime WHERE id = :id`,
          {
            replacements: {
              startTime: convertedStartTime,
              endTime: convertedEndTime,
              id: entitlement.id
            },
            type: Sequelize.QueryTypes.UPDATE
          }
        );
      }
    }
  },

  async down(queryInterface, Sequelize) {
    const timezones = getDefinedTimezones();
    const entitlements = await getEntitlements(queryInterface, Sequelize);

    // Helper function to get the moment time zone
    function getMomentTimeZone(selectedTimeZone) {
      const timezone = timezones.find(tz => tz.value === selectedTimeZone);
      return timezone ? timezone.moment : null;
    }

    // Helper function to convert UTC to local time
    function convertFromUTC(timeString, timeZoneMomentValue) {
      // Create a moment object with the provided UTC time string
      const utcTime = moment.utc(
        `1970-01-01 ${timeString}`,
        'YYYY-MM-DD HH:mm'
      );
      // Convert the UTC time to the specified timezone
      const localTime = utcTime.tz(timeZoneMomentValue);
      // Return only the time part (hours:minutes:seconds) in the specified timezone
      return localTime.format('HH:mm:ss');
    }

    for (const entitlement of entitlements) {
      const timeZone = entitlement.time_zone;
      const timeZoneMoment = getMomentTimeZone(timeZone);

      if (timeZoneMoment) {
        const convertedStartTime = convertFromUTC(
          entitlement.access_start_time,
          timeZoneMoment
        );
        const convertedEndTime = convertFromUTC(
          entitlement.access_end_time,
          timeZoneMoment
        );

        await queryInterface.sequelize.query(
          `UPDATE Entitlements SET access_start_time = :startTime, access_end_time = :endTime WHERE id = :id`,
          {
            replacements: {
              startTime: convertedStartTime,
              endTime: convertedEndTime,
              id: entitlement.id
            },
            type: Sequelize.QueryTypes.UPDATE
          }
        );
      }
    }
  }
};

function getDefinedTimezones() {
  return [
    { name: 'UTC', value: 'UTC', moment: 'Etc/UCT' },
    { name: 'GMT', value: 'GMT', moment: 'Europe/London' },
    { name: 'AEDT', value: 'AEDT', moment: 'Antarctica/Macquarie' },
    { name: 'JST', value: 'JST', moment: 'Asia/Tokyo' },
    { name: 'IST', value: 'IST', moment: 'Asia/Calcutta' },
    { name: 'EET', value: 'EET', moment: 'Africa/Cairo' },
    { name: 'CAT', value: 'CAT', moment: 'Africa/Lusaka' },
    { name: 'ART', value: 'ART', moment: 'America/Argentina/Buenos_Aires' },
    { name: 'NST', value: 'NST', moment: 'Canada/Newfoundland' },
    { name: 'AST', value: 'AST', moment: 'Canada/Atlantic' },
    { name: 'CET', value: 'CET', moment: 'Europe/Amsterdam' },
    { name: 'AWST', value: 'AWST', moment: 'Australia/Perth' },
    { name: 'NZST', value: 'NZST', moment: 'Pacific/Auckland' },
    { name: 'SAST', value: 'SAST', moment: 'Africa/Johannesburg' },
    { name: 'WET', value: 'WET', moment: 'Atlantic/Canary' },
    { name: 'PST', value: 'PST', moment: 'America/Vancouver' },
    { name: 'MST', value: 'MST', moment: 'US/Mountain' },
    { name: 'CST', value: 'CST', moment: 'Mexico/General' },
    { name: 'EST', value: 'EST', moment: 'America/Cayman' },
    { name: 'HST', value: 'HST', moment: 'Pacific/Honolulu' },
    { name: 'AKST', value: 'AKST', moment: 'US/Alaska' }
  ];
}

//Helper function to get Entitlements where access_restriction = 1
async function getEntitlements(queryInterface, Sequelize) {
  return await queryInterface.sequelize.query(
    `SELECT id, access_start_time, access_end_time, time_zone FROM Entitlements WHERE access_restriction = 1`,
    { type: Sequelize.QueryTypes.SELECT }
  );
}
