const crypto = require('crypto');
const moment = require('moment');
const models = require('../models');
const Queue = require('./queue');
const helper = require('../routes/common/helper');
const { generateRandomPassword } = require('../utils/common');
const { actionType, publishLogOut} = require('../routes/common/event-publisher');
const { Op } = require('sequelize');

class BulkPasswordChangeQueue extends Queue {
  constructor(queueName, opts) {
    super(queueName, opts)
  }

  async consumer(msg) {
      const transaction = await models.sequelize.transaction();
      const {
        selectAll,
        selectedIds,
        organization,
        department,
        searchText,
        idp
      } = msg;

      let options = {
        where: {
          id: {
            ...selectAll ? {
              [Op.notIn]: selectedIds
            } : {
              [Op.in]: selectedIds
            }
          },
          [Op.or]: {
            name: {
              [Op.like]: `%${searchText.trim()}%`
            },
            email: {
              [Op.like]: `%${searchText.trim()}%`
            },
            userName: {
              [Op.like]: `%${searchText.trim()}%`
            }
          },
          // Get only Local Users to Enforce to Password Change
          identity_provider: {
            [Op.eq]:'Local User'  
          }
        },
        transaction
      };

      if (organization) {
        const myOrganization = await models.Organizations.findOne({
          where: {
            name: organization
          },
          attributes: ['id'],
        })
        if (myOrganization) {
          const departmentOptions = {
            organization_id: myOrganization.id
          }
          if (department) {
            departmentOptions.id = department
          }
          const myDepartments = await models.Departments.findAll({
            where: departmentOptions
          })
          if (myDepartments && myDepartments.length > 0) {
            options.where.department = {
              [Op.in]: myDepartments.map(department => department.id)
            }
          }
        }
      }
      const localUsers = await models.LocalUsers.findAll(options, { transaction });
      const userPasswords = [];

      await Promise.all(localUsers.map(user => {
        // If User is Gateway User Than Don't do Enforce Password Change
        if(user.name!=='Gateway User' && !user.userName.includes('gwadmin')){
          const generatePasword = generateRandomPassword();

          userPasswords.push({
              name: user.name,
              email: user.email,
              password: generatePasword,
              id: user.id,
              userName: user.userName
          });
          return user.update({
              change_pass: true,
              password: crypto.createHash('sha256').update(generatePasword).digest('hex'),
              password_updated_at: moment(),
              temporary_password_expiry: moment().add(1, 'hours')
          });
        }
      }))

      const client_ids = await models.Client.findAll({
        attributes: ['agent_id'],
        where: {
          user: {
            [Op.in]: userPasswords.map(u => u.userName)
          }
        },
        raw: true
      })


    console.log('Bulk password change Consumer: ', msg);

    console.log('Sending emails...');
    if(client_ids.length){
      await publishLogOut(actionType.UPDATED, {
        code: 63,
        ids: client_ids.map(c => c.agent_id)
      });
    }

    userPasswords.forEach(user => {
        console.log('Sending Email to:', user.email);
        helper.sendLocalUserPasswordResetMail({
        recipient: user.email,
        emailSubject: 'Invisily Password Reset',
        templateFileName: 'client-password-reset.html',
        name: user.name,
        username: user.userName,
        password: user.password,
    });
    });
  }
}

module.exports = new BulkPasswordChangeQueue('bulk-password-change-queue');
