var express = require('express');
var router = express.Router();
var policiesLayer = require('../../dao/policies');
var policiesValidator = require('../../validators/policies');
var validate = require('../../validators/validate');
const verifyPermissions = require('../../middlewares/verifyPermissions');

router.get('/access', verifyPermissions(['Device Entitlements', 'User Entitlements','Users','Devices']), policiesLayer.getPolicies);

router.get('/access/verifyPolicies',verifyPermissions(['Users','Service Groups','Gateways']), policiesLayer.verifyPolicies);

router.post(
  '/access/create',
  policiesValidator.createPolicy(),
  validate,
  verifyPermissions(['Device Entitlements', 'User Entitlements']),
  policiesLayer.createPolicy
);

router.get(
  '/access/:id(\\d+)',
  policiesValidator.getPolicy(),
  validate,
  policiesLayer.getPolicyById
);

router.get(
  '/access/user/:id(\\d+)',
  verifyPermissions('Users'),
  policiesValidator.getPolicy(),
  validate,
  policiesLayer.getUserPolicyById
);

router.put(
  '/access/:id(\\d+)',
  policiesValidator.updatePolicy(),
  validate,
  verifyPermissions(['Device Entitlements', 'User Entitlements']),
  policiesLayer.updatePolicy
);

router.delete(
  '/access/:id(\\d+)',
  policiesValidator.deletePolicy(),
  validate,
  verifyPermissions(['Device Entitlements', 'User Entitlements']),
  policiesLayer.deletePolicy
);

router.patch(
  '/access/:id(\\d+)/enable/:value(true|false)',
  policiesValidator.setEnableValue(),
  validate,
  verifyPermissions(['Device Entitlements', 'User Entitlements']),
  policiesLayer.setEnableValue
);

router.get('/access/filters', policiesLayer.getAllClientAndServices);
router.get(
  '/access/filterswithoutpagination',
  verifyPermissions(['Device Entitlements', 'User Entitlements', 'Security Policy','Software Policy','Process Policy','Data Volume Policy']),
  policiesLayer.getAllClientAndServicesWithoutPagination
);

router.get('/access/clients', 
  verifyPermissions(['Device Entitlements', 'User Entitlements','Security Policy','OS Policy','Software Policy','Process Policy','Data Volume Policy']),
  policiesLayer.getAllClientsByName);

router.patch(
  '/device/:id(\\d+)/enable/:value(true|false)',
  policiesValidator.setEnableValue(),
  validate,
  verifyPermissions('Security Policy'),
  policiesLayer.setDeviceEnableValue
);

router.get('/device',verifyPermissions(['Security Policy','Devices']),  policiesLayer.getDevicePolicies);

router.post(
  '/device/create',
  policiesValidator.createDevicePolicy(),
  validate,
  verifyPermissions('Security Policy'),
  policiesLayer.createDevicePolicy
);

router.get(
  '/device/:id(\\d+)',
  policiesValidator.getDevicePolicy(),
  validate,
  policiesLayer.getDevicePolicyById
);

router.put(
  '/device/:id(\\d+)',
  policiesValidator.updateDevicePolicy(),
  validate,
  verifyPermissions('Security Policy'),
  policiesLayer.updateDevicePolicy
);

router.delete(
  '/device/:id(\\d+)',
  policiesValidator.deleteDevicePolicy(),
  validate,
  verifyPermissions('Security Policy'),
  policiesLayer.deleteDevicePolicy
);

//SOFWARE PROCESS POLICIES

router.patch(
  '/software-process/:id(\\d+)/enable/:value(true|false)',
  policiesValidator.setEnableValue(),
  validate,
  verifyPermissions('Process Policy'),
  policiesLayer.setSoftwareProcessEnableValue
);

router.get('/software-process', verifyPermissions(['Process Policy','Devices']), policiesLayer.getSoftwareProcessPolicies);

router.get(
  '/software-process/software-process-binding-list',
  verifyPermissions('Process Policy'),
  policiesLayer.getSoftwaresProcessBindingList
);

router.get(
  '/software-process/edr-software-process-binding-list',
  verifyPermissions(['Security Policy']),
  policiesLayer.getEDRSoftwareProcessBindingList
);

router.post(
  '/software-process/create',
  policiesValidator.createSoftwareProcessPolicy(),
  validate,
  verifyPermissions('Process Policy'),
  policiesLayer.createSoftwareProcessPolicy
);

router.get(
  '/software-process/:id(\\d+)',
  policiesValidator.getSoftwareProcessPolicy(),
  validate,
  policiesLayer.getSoftwareProcessPolicyById
);

router.put(
  '/software-process/:id(\\d+)',
  policiesValidator.updateSoftwareProcessPolicy(),
  validate,
  verifyPermissions('Process Policy'),
  policiesLayer.updateSoftwareProcessPolicy
);

router.delete(
  '/software-process/:id(\\d+)',
  policiesValidator.deleteSoftwareProcessPolicy(),
  validate,
  verifyPermissions('Process Policy'),
  policiesLayer.deleteSoftwareProcessPolicy
);

//SOFTWARE POLICES

router.get('/software', verifyPermissions(['Software Policy','Devices']), policiesLayer.getSoftwarePolicies);

router.post(
  '/software/create',
  policiesValidator.createSoftwarePolicy(),
  validate,
  verifyPermissions('Software Policy'),
  policiesLayer.createSoftwarePolicy
);

router.patch(
  '/software/:id(\\d+)/enable/:value(true|false)',
  policiesValidator.setEnableValue(),
  validate,
  verifyPermissions('Software Policy'),
  policiesLayer.setSoftwareEnableValue
);

router.get(
  '/software/:id(\\d+)',
  policiesValidator.getSoftwarePolicy(),
  validate,
  policiesLayer.getSoftwarePolicyById
);

router.put(
  '/software/:id(\\d+)',
  policiesValidator.updateSoftwarePolicy(),
  validate,
  verifyPermissions('Software Policy'),
  policiesLayer.updateSoftwarePolicy
);

router.delete(
  '/software/:id(\\d+)',
  policiesValidator.deleteSoftwarePolicy(),
  validate,
  verifyPermissions('Software Policy'),
  policiesLayer.deleteSoftwarePolicy
);

// OS POLICES

router.get('/os/exists', verifyPermissions('OS Policy'), policiesLayer.getAllOs);

router.get('/os', verifyPermissions(['OS Policy','Devices']), policiesLayer.getOsPolicies);

router.patch(
  '/os/:id(\\d+)/enable/:value(true|false)',
  policiesValidator.setEnableValue(),
  validate,
  verifyPermissions('OS Policy'),
  policiesLayer.setOSEnableValue
);

router.post(
  '/os/create',
  policiesValidator.createOsPolicy(),
  validate,
  verifyPermissions('OS Policy'),
  policiesLayer.createOsPolicy
);

router.get(
  '/os/:id(\\d+)',
  policiesValidator.getOsPolicy(),
  validate,
  policiesLayer.getOsPolicyById
);

router.put(
  '/os/:id(\\d+)',
  policiesValidator.updateOsPolicy(),
  validate,
  verifyPermissions('OS Policy'),
  policiesLayer.updateOsPolicy
);

router.delete(
  '/os/:id(\\d+)',
  policiesValidator.deleteOsPolicy(),
  validate,
  verifyPermissions('OS Policy'),
  policiesLayer.deleteOsPolicy
);

// Routing Policies

router.get('/routing/filters', verifyPermissions('Routing Policy'), policiesLayer.getAllGatwayAndServices);

router.patch(
  '/routing/:id(\\d+)/enable/:value(true|false)',
  policiesValidator.setEnableValue(),
  validate,
  verifyPermissions('Routing Policy'),
  policiesLayer.setRoutingEnableValue
);

router.get('/routing', verifyPermissions('Routing Policy'), policiesLayer.getRoutingPolicies);

router.get('/routing/gateways', verifyPermissions('Routing Policy'), policiesLayer.getAllGatewaysByName);

router.post(
  '/routing/create',
  policiesValidator.createRoutingPolicy(),
  validate,
  verifyPermissions('Routing Policy'),
  policiesLayer.createRoutingPolicy
);

router.get(
  '/routing/:id(\\d+)',
  verifyPermissions('Routing Policy'),
  policiesValidator.getRoutingPolicy(),
  validate,
  policiesLayer.getRoutingPolicyById
);

router.put(
  '/routing/:id(\\d+)',
  policiesValidator.updateRoutingPolicy(),
  validate,
  verifyPermissions('Routing Policy'),
  policiesLayer.updateRoutingPolicy
);

router.delete(
  '/routing/:id(\\d+)',
  policiesValidator.deleteRoutingPolicy(),
  validate,
  verifyPermissions('Routing Policy'),
  policiesLayer.deleteRoutingPolicy
);

// Data Volume Policy

router.get('/data-volume-policy', verifyPermissions(['Data Volume Policy','Devices']), policiesLayer.getDataVolumePolicies);

router.post(
  '/data-volume-policy/create',
  policiesValidator.createDataVolumePolicy(),
  validate,
  verifyPermissions('Data Volume Policy'),
  policiesLayer.createDataVolumePolicy
);

router.get(
  '/data-volume-policy/:id(\\d+)',
  policiesValidator.getDataVolumePolicy(),
  validate,
  policiesLayer.getDataVolumePoliciesById
);

router.put(
  '/data-volume-policy/:id(\\d+)',
  policiesValidator.updateSoftwareProcessPolicy(),
  validate,
  verifyPermissions('Data Volume Policy'),
  policiesLayer.updateDataVolumePolicies
);

router.delete(
  '/data-volume-policy/:id(\\d+)',
  policiesValidator.deleteSoftwareProcessPolicy(),
  validate,
  verifyPermissions('Data Volume Policy'),
  policiesLayer.deleteDataVolumePolicy
);

router.patch(
  '/data-volume-policy/:id(\\d+)/enable/:value(true|false)',
  policiesValidator.setEnableValue(),
  validate,
  verifyPermissions('Data Volume Policy'),
  policiesLayer.setDataVolumeEnableValue
);

module.exports = router;
