#!/bin/bash

script_name="$0"

set -eE -o functrace

failure() {
    local line="$1"
    local cmd="$2"
    echo >&"[FTL] FATAL ERROR! [$script_name:$line] | $cmd"
}

trap 'failure "$LINENO" "$BASH_COMMAND"' ERR

log() {
    local timestamp="$(date +"%Y-%m-%d %T")"
    echo "$timestamp" "$@"
}

install_dependencies() {
    log "[INF] Installing dependencies"
    local -r pkgs=(ipset)
    for pkg in ${pkgs[*]}; do
        log "[INF] Validating package [$pkg]"
        if rpm -q "$pkg" >/dev/null 2>&1; then
            log "[INF] Package is installed! [$pkg]"
        else
            log "[INF] Package is not installed! [$pkg]"
            log "[INF] Installing package via yum [$pkg]"
            if ! yum install "$pkg" -y; then
                log "[ERR] Failed to install package! [$pkg]"
                exit 1
            fi
            pkg_version=$("$pkg" --version)
            log "[INF] Package installed successfully! [$pkg ($pkg_version)]"
        fi
    done
    log "[INF] Dependencies installed successfully!"
}

# script starts here

log "[INF] Running script $script_name"

rpm_pkg_name='invisily_client.rpm'
settings_file_name='settings.cfg'
installed_pkg_name='invisily-client'
sniffer_config_file_path='/opt/invisily/linuxclientd/client_data/configs/sniffer_config.cfg'
settings_config_file_path="/opt/invisily/linuxclientd/bin/$settings_file_name"
backend_service='invisily_backend.service'

log "[INF] Configuring and installing S2S client"

log "[INF] Validating root user"
if ((EUID != 0)); then
    log "[ERR] Run with root user!"
    log "[INF] Usage: sudo $script_name"
    exit 1
fi
log "[INF] Validated root user successfully!"

log "[INF] Validating rpm package [$rpm_pkg_name]"
if [[ ! -f ./"$rpm_pkg_name" ]]; then
    log "[ERR] Package not found! [$rpm_pkg_name]"
    exit 1
fi
log "[INF] Validated rpm package! [$rpm_pkg_name]"

log "[INF] Validating settings file [$settings_file_name]"
if [[ ! -f ./"$settings_file_name" ]]; then
    log "[ERR] File not found! [$settings_file_name]"
    exit 1
fi
log "[INF] Validated settings file! [$rpm_pkg_name]"

log "[INF] Validating installed package [$installed_pkg_name]"
if rpm -q "$installed_pkg_name" >/dev/null 2>&1; then
    log "[ERR] Package is already installed! [$installed_pkg_name]"
    log "[INF] Please uninstall it and then try again!"
    exit 1
fi
log "[INF] Package is not installed! [$installed_pkg_name]"

install_dependencies

log "[INF] Installing package [$rpm_pkg_name]"
if ! rpm -ivh ./"$rpm_pkg_name" --nodeps; then
    log "[ERR] Failed to install package! [$rpm_pkg_name]"
    exit 1
fi
log "[INF] Package installed successfully! [$rpm_pkg_name]"

log "[INF] Generating sniffer configuration [$sniffer_config_file_path]"

log "[INF] Select network interface:"
interfaces=$(ip -4 -br addr | tr -s '[:space:]' | grep -E '[[:print:]] UP' | cut -d ' ' -f1)
select interface in "$interfaces"; do
    if [[ -n "$interface" ]]; then
        log "[INF] Interface selected! [$interface]"
        break
    else
        log "[ERR] Invalid option!"
    fi
done

ip=$(ip -4 -br addr show "$interface" | tr -s '[:space:]' | cut -d ' ' -f3 | cut -d '/' -f1)

{
    echo "interface=\"$interface\""
    echo "ip=\"$ip\""
} >"$sniffer_config_file_path"

log "[INF] Dumping sniffer configuration [$sniffer_config_file_path]"
log "[INF] --- SNIFFER CONFIGURATION STARTS ---"
cat "$sniffer_config_file_path"
log "[INF] ---- SNIFFER CONFIGURATION ENDS ----"

log "[INF] Sniffer configuration generated successfully! [$sniffer_config_file_path]"

log "[INF] Generating settings configuration [$settings_config_file_path]"

log "[INF] Dumping settings configuration [$settings_config_file_path]"
log "[INF] --- SETTINGS CONFIGURATION STARTS ---"
cp ./"$settings_file_name" "$settings_config_file_path"
cat "$settings_config_file_path"
log "[INF] ---- SETTINGS CONFIGURATION ENDS ----"

log "[INF] Settings configuration generated successfully! [$settings_config_file_path]"

log "[INF] Restarting backend service"
systemctl restart "$backend_service"
log "[INF] Backend service restart successfully!"

log "[INF] Configured and installed S2S client successfully!"

log "[INF] Exiting $script_name"