#!/bin/bash

#set -eo pipefail
#shopt -s nullglob

################################################################################
# Installation files existance check                                           #
################################################################################
MY_PATH=$(dirname "$0")
MY_PATH=$(cd "$MY_PATH" && pwd)
DETAILED_LOGS="$MY_PATH/logs.txt"
OFFLINE_CONTROLLER_INSTALLER_NAME=controller-5.8.tar.gz
PYTHON_MODULES=controller-python-dependencies.tar.gz
PYTHON_MODULES_EXT_DIR=controller-python-dependencies
DEB_MODULES=controller-deb-dependencies.tar.gz
CTRL_INI="$MY_PATH/ctrl.ini"
REQ_TXT="/opt/invisily/sdp-controller/bin/requirements.txt"

source "$CTRL_INI" 2>/dev/null
FS_USER=$SUDO_USER

# output redirection
exec > >(tee -i "$DETAILED_LOGS")
exec 2>&1

################################################################################
# OBP vars                                                                     #
################################################################################
OBP_ZIP="$MY_PATH/obp.zip"
OBP_DIR="/tmp/obp"
OBP_CONFIG_TXT="obp-config.txt"
OBP_VER_FILE="/var/www/html/onboarding-portal/models/general/.version"

TMP_ONBOARD_PORTAL_OFFLINE_PKG_LOCATION="$OBP_DIR/invisily-onboard-portal-offline-dependencies"
OFFLINE_UNZIP_INSTALLER="unzip-6.0-21.tar.gz"

################################################################################
# MFA vars                                                                     #
################################################################################
INSTALLER_MFA=mfa_server.tar.gz
MFA_ZIP="$MY_PATH/mfa.zip"
MFA_DIR="/tmp/mfa"
OFFLINE_DEB_INSTALLER_NAME_MFA=bio-mfa.tar.gz
MFA_CONFIG_TXT="mfa-config.txt"
MFA_VER_FILE="/opt/invisily/mfa_suite/mfa_server/fidouaf/pom.xml"
UPDATE_MFA_VER_FILE="$MFA_DIR/mfa_suite/mfa_server/fidouaf/pom.xml"
TMP_MFA_OFFLINE_PKG_LOCATION="$MFA_DIR/invisily-biomfa-offline-dependencies"
APT_ARCHIVE="/var/cache/apt/archives"

################################################################################
# Logging functions                                                            #
################################################################################
curFile=$(basename "$0" | awk -F . '{print $1}')
logfile="/var/log/$curFile.log"

[[ -f "$logfile" ]] && { rm -rf "$logfile" >/dev/null 2>&1 || {
    printf '\033[0;31m%s [%s] [%s]: %s\033[0m\n' "$(date +"%Y-%m-%d %H:%M:%S%z")" "Error" "$0" "Please run script as root. Exiting..." && exit 1
    }
}
touch "$logfile" >/dev/null 2>&1 || {
    printf '\033[0;31m%s [%s] [%s]: %s\033[0m\n' "$(date +"%Y-%m-%d %H:%M:%S%z")" "Error" "$0" "Please run script as root. Exiting..." && exit 1
}

invisily_log() {
    local type="$1"; shift
    if [ "$type" == "ERROR" ]; then
        printf '\033[0;31m%s [%s] [%s]: %s\033[0m\n' "$(date +"%Y-%m-%d %H:%M:%S%z")" "$type" "$0" "$*" | tee -a "$logfile"
    elif [ "$type" == "Warn" ]; then
        printf '\033[0;34m%s [%s] [%s]: %s\033[0m\n' "$(date +"%Y-%m-%d %H:%M:%S%z")" "$type" "$0" "$*" | tee -a "$logfile"
    elif [ "$type" == "Result" ]; then
        printf '\033[0;34m%s\033[0m\n' "$*" | tee -a "$logfile"
    else
        printf '%s [%s] [%s]: %s\n' "$(date +"%Y-%m-%d %H:%M:%S%z")" "$type" "$0" "$*" | tee -a "$logfile"
    fi
}
invisily_note() {
    invisily_log Note "$@"
}
invisily_warn() {
    invisily_log Warn "$@" >&2
}
invisily_result() {
    invisily_log Result "$@" >&2
}
invisily_error_no_exit() {
    invisily_log ERROR "$@" >&2
}
invisily_error() {
    invisily_log ERROR "$@" >&2
    invisily_log ERROR "Exiting..."  >&2
    exit 1
}

if [ "$EUID" -ne 0 ]; then
    invisily_error $'Please run as root.'
fi

################################################################################
# Initial necessary verification                                               #
################################################################################
if [ -f /etc/lsb-release ] || [ -d /etc/lsb-release.d ]; then
    ubuntuVersion=$(lsb_release -r | cut -d: -f2 | sed s/'^\t'//)
    if [ "$ubuntuVersion" != "22.04" ] && [ "$ubuntuVersion" != "20.04" ]; then
        invisily_error "Supported Operating system is Ubuntu 22.04/20.04 , but it appears you are not running Ubuntu 22.04 or Ubuntu 20.04"
    fi
else
    invisily_error "Supported Operating system is Ubuntu 22.04/20.04, but it appears you are not running Ubuntu 22.04 or Ubuntu 20.04"
fi

if [ "$( (dpkg-query -W -f='${Status}' ubuntu-desktop 2>/dev/null | grep -c "ok installed") )" -eq 1 ]; then
    invisily_error "Ubuntu 22.04/20.04 Server is required, but it appears that you are running Ubuntu Desktop"
fi

if [ $# -eq 0 ]; then
    invisily_error $'No arguments supplied. Please use -h or --help to get help'
fi

################################################################################
# Validation functions                                                         #
################################################################################
__ip_validation() {
    ip=$1
    fail=0
    if expr "$ip" : '[0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*$' >/dev/null; then
        for i in 1 2 3 4; do
            if [[ $i -eq 1 ]]; then
                if [[ $(echo "$ip" | cut -d. -f$i) -gt 255 || $(echo "$ip" | cut -d. -f$i) -le 0 ]]; then
                    fail=1
                    break
                fi
            else
                if [[ $(echo "$ip" | cut -d. -f$i) -gt 255 ]]; then
                    fail=1
                    break
                fi
            fi
        done
    else
        fail=1
    fi
    if [ "$fail" == "1" ]; then
        return 1
    else
        return 0
    fi
}

apt_pkg_check(){
    if [ $# -eq 0 ]; then
        invisily_error $'No arguments supplied. Please use give apt package name in argument'
    elif [ $# -gt 1 ]; then
        invisily_error $'apt_pkg_check function accepts only one agument'
    else
        dependency=$1
        if [ "$(! dpkg-query -W -f='${Status}' "$dependency" 2>/dev/null | grep -c "ok installed")" -eq 0 ]; then
            return 1
        else
            return 0
        fi
    fi
}

installation_mode_verification() {
    if [[ $INSTALL_MODE == "online" ]]; then
        ONLINE_INSTALL=1
    elif [[ $INSTALL_MODE == "offline" ]]; then
        OFFLINE_INSTALL=1
    else
        invisily_error $'--installation-type should be either online or offline, however you have entered' "$INSTALL_MODE"
    fi
}

apt_dependency_uninstall(){
    if [ $# -eq 0 ]; then
        invisily_error $'No arguments supplied. Please give apt package name in argument'
    elif [ $# -gt 1 ]; then
        APT_DEPENDENCIES=("$@")
        INSTALLABLE_APT_DEPENDENCIES=()
        UNINSTALLABLE_APT_DEPENDENCIES=()
        for dependency in "${APT_DEPENDENCIES[@]}"; do
            if [ "$(! dpkg-query -W -f='${Status}' "$dependency" 2>/dev/null | grep -c "ok installed")" -eq 0 ]; then
                invisily_note "Not found $dependency. Adding to installables..."
                INSTALLABLE_APT_DEPENDENCIES+=("$dependency")
            else
                invisily_note "Found $dependency. Adding to uninstallables..."
                UNINSTALLABLE_APT_DEPENDENCIES+=("$dependency")
            fi
        done

        INSTALLABLE_APT_DEPENDENCIES="${INSTALLABLE_APT_DEPENDENCIES[*]}"

        UNINSTALLABLE_APT_DEPENDENCIES="${UNINSTALLABLE_APT_DEPENDENCIES[*]}"

        if [ -n "$UNINSTALLABLE_APT_DEPENDENCIES" ]; then
            invisily_note "Following dependencies are already installed: $UNINSTALLABLE_APT_DEPENDENCIES , and going to uninstall..."

            if apt-get purge --yes $UNINSTALLABLE_APT_DEPENDENCIES; then
                invisily_note "Uninstalled $UNINSTALLABLE_APT_DEPENDENCIES."
            else
                invisily_error $'Uninstallation failed for '"$UNINSTALLABLE_APT_DEPENDENCIES"'. Exiting...'
            fi
            if apt autoremove -y; then
                invisily_note "Uninstalled $UNINSTALLABLE_APT_DEPENDENCIES."
            else
                invisily_error $'Uninstallation failed for '"$UNINSTALLABLE_APT_DEPENDENCIES"'. Exiting...'
            fi
        fi

        if [ -n "$INSTALLABLE_APT_DEPENDENCIES" ]; then
            invisily_note "Following dependencies are already not installed: $INSTALLABLE_APT_DEPENDENCIES"
        fi
    else
        dependency=$1
        if [ "$(! dpkg-query -W -f='${Status}' "$dependency" 2>/dev/null | grep -c "ok installed")" -ne 0 ]; then
            if apt-get purge --yes "$dependency"; then
                invisily_note "Uninstalled $dependency."
            else
                invisily_error $'Uninstallation failed for '"$dependency"'. Exiting...'
            fi
            if apt autoremove -y; then
                invisily_note "Uninstalled $dependency."
            else
                invisily_error $'Uninstallation failed for '"$dependency"'. Exiting...'
            fi
        else
            invisily_note "$dependency is already not installed on system"
        fi
    fi
}

dns_ip_verification() {
    if ! __ip_validation "$1" ; then
        invisily_error "DNS IP you entered is not a valid dns ip"
    elif [[ "$1" == *"127.0"* ]]; then
        invisily_error "DNS IP cannot be local host IP"
    elif ! timeout 3 nslookup google.com "$1" >/dev/null 2>&1 ; then
        if [[ $disable_dns_check -ne 1 ]]; then
            invisily_error "Your DNS IP is not resolving google.com, please enter a valid dns ip which really resolves domains"
        else
            invisily_warn "DNS Server ip you have provided, is not resolving google.com, however this error is ignoring becuase you have passed --disable-dns-check"
        fi
    else
        invisily_note "DNS verification is passed for DNS IP $1 you have provided."
    fi
}

################################################################################
# Help                                                                         #
################################################################################
Help() {
    # Display Help
    echo -e "Usage: $0 [-h] [--help] [--install] [--remove] [--installation-type=xxx] [--multi-home]"
    echo -e ""
    echo -e "Help: $0 -h"
    echo -e "      OR"
    echo -e "      $0 --help"
    echo -e
    echo -e "Options:"
    echo -e "-h | --help                            Print help."
    echo -e "--install                              Install Invisily Controller"
    echo -e "--remove                               Remove Invisily Controller"
    echo -e "--installation-type=xxx                should be either online or offline"
    echo -e "--multi-home                           Multi-home installation to run on Multiple Interfaces"
    echo -e "--dns-ip                               Provide specific ip to set for dns (not recommended)"
    echo -e "--disable-dns-check                    Stop installer to do dns query to confirm availability of dns service on provided dns ip"
    echo -e ""
}

################################################################################
################################################################################
# Main program                                                                 #
################################################################################
################################################################################
for i in "$@"
do
case $i in
    --install)
    INSTALL=1
    shift # past argument=value
    ;;
    --remove)
    REMOVE=1
    shift # past argument=value
    ;;
    --installation-type=*)
    INSTALL_MODE="${i#*=}"
    installation_mode_verification
    shift # past argument=value
    ;;
    --multi-home)
    MULTI_HOME=1
    shift # past argument=value
    ;;
    --disable-dns-check)
    disable_dns_check=1
    shift # past argument=value
    ;;
    --dns-ip=*)
    dnsip="${i#*=}"
    shift # past argument=value
    ;;
    --verbose)
    VERBOSE=1
    set -x
    ;;
    -h|--help)
    Help
    exit 1
    ;;
    *)
    invisily_error "Unknown option. Exiting..."
    ;;
esac
done

arg_verification(){
    if [ "$INSTALL" == "1" ] && [ "$REMOVE" == "1" ]; then
        invisily_error $'You cannot use --install and --remove at the same time'
    fi
    if [ "$INSTALL" == "1" ] && [ -z "$INSTALL_MODE" ]; then
        invisily_error $'--installation-type is not passed, should be either online or offline'
    fi
    if [ "$REMOVE" == "1" ] && [ -n "$INSTALL_MODE" ]; then
        invisily_error $'--installation-type is not required when removing or uninstalling'
    fi
    if [[ -n $dnsip ]]; then
        dns_ip_verification "$dnsip"
    fi
}

install_controller() {
    if [[ $ONLINE_INSTALL -eq 1 ]]; then
        if [[ -f /opt/invisily/sdp-controller/bin/controller.sh ]]; then
            /opt/invisily/sdp-controller/bin/controller.sh stop
        else
            invisily_note "Ignoring to stop controller and sniffer since it seems to be a fresh installation"
        fi
        if ping -c3 -W1  github.com >/dev/null 2>&1 || ping -c3 -W1  google.com >/dev/null 2>&1 || ping -c3 -W1 archive.ubuntu.com >/dev/null 2>&1 || timeout 1 bash -c "echo &> /dev/null  < /dev/tcp/archive.ubuntu.com/80"; then
            if ! apt-get -y update; then
                invisily_error "apt-get failed due to unknown reason."
            fi
        else
            invisily_error "Your machine is not connected to Internet. Exiting..."
        fi
    elif [[ $OFFLINE_INSTALL -eq 1 ]]; then
        if [[ "$ubuntuVersion" == "20.04" ]]; then
            invisily_error "Offline installation is not supported in Ubuntu 20.04..."
        fi
        if [[ -f /root/.DO_NOT_DELETE_INVISILY ]]; then
            invisily_note "You are using Invisily Provided OS image, proceeding for Offline Install..."
        elif [[ -f $MY_PATH/$OFFLINE_CONTROLLER_INSTALLER_NAME ]]; then
            [ -d /tmp/invisily-controller-offline-dependencies ] && rm -rf /tmp/invisily-controller-offline-dependencies
            mkdir -p /tmp/invisily-controller-offline-dependencies
            if ! tar xzvf "$MY_PATH"/$OFFLINE_CONTROLLER_INSTALLER_NAME -C /tmp/invisily-controller-offline-dependencies >/dev/null 2>&1; then
                invisily_error "Extracting Offline Dependencies Failed."
            fi
        else
            invisily_error "You does not seem to be using Invisily specific Ubuntu 22.04 image nor Invisily Offline dependencies are available in the current directory."
        fi
    fi

    invisily_note "invisily-controller installation ..."
    invisily_note "invisily-controller checking previous installation ..."
    apt_pkg_check invisily-controller && apt-get purge invisily-controller -y
    [ -f /opt/invisily/sdp-controller/watchdog/conf/watchdog.cnf ] && mv /opt/invisily/sdp-controller/watchdog/conf/watchdog.cnf ~/watchdog.cnf

    if [ -d "/opt/invisily/sdp-controller" ]; then
        rm -rf  /opt/invisily/sdp-controller
    fi

    if [[ $ONLINE_INSTALL == 1 ]]; then
        invisily_note "invisily-controller deb installation ..."
        chmod a+x ./*.deb
        if ! apt-get install  ./*.deb -y; then
            invisily_error "Error invisily-controller deb installation failed.."
        fi
        apt_pkg_check systemd-timesyncd || apt-get install systemd-timesyncd -y
        #if apt-get install -y python3-pip python3 net-tools gcc-aarch64-linux-gnu && add-apt-repository universe && curl https://bootstrap.pypa.io/pip/3.6/get-pip.py --output get-pip.py && python3 get-pip.py; then
                #invisily_note "Successfully installed pip for python3"
            #else
                #invisily_error "Failed to install pip for python3"
        #fi
        if [[ -n $(pip3 show mysql-connector 2>/dev/null) ]]; then
            pip3 uninstall mysql-connector
        fi
    elif [[ $OFFLINE_INSTALL == 1 ]]; then
        cd /tmp/invisily-controller-offline-dependencies || exit
        [ -f /tmp/invisily-controller-offline-dependencies/$DEB_MODULES ] || invisily_error "Cannot find Ubuntu offline installer dependencies from extracted files!"
        mkdir -p "$APT_ARCHIVE"
        apt-get clean
        tar xzvf /tmp/invisily-controller-offline-dependencies/$DEB_MODULES -C "$APT_ARCHIVE"
        cd "$APT_ARCHIVE" || exit
        export DEBIAN_FRONTEND=noninteractive
        cd "$APT_ARCHIVE" || exit
        if ! apt-get -y install ./*.deb; then
            invisily_error "Installation of dependencies for Controller is failed!"
        else
            invisily_note "Installation of dependencies for Controller is successful!"
        fi
        apt-get clean

        # install controller package
        invisily_note "invisily-controller deb installation ..."
        chmod a+x "$MY_PATH"/*.deb
        if ! apt-get install  "$MY_PATH"/*.deb -y; then
            invisily_error "Error invisily-controller deb installation failed.."
        fi

        cd "$MY_PATH" || exit
    fi
    [ -f ~/watchdog.cnf ] && rm -rf /opt/invisily/sdp-controller/watchdog/conf/watchdog.cnf && mv ~/watchdog.cnf /opt/invisily/sdp-controller/watchdog/conf/watchdog.cnf

    if [[ -z $device_name ]]; then
        device_found=0
        while [[ $device_found -ne 1 ]]
        do
            array_test=()
            argstr=""
            declare -i count=0
            declare -i count2=0
            echo "Available Network Devices for Controller Deployment are :"
            for iface in $(ip a | awk -F ': ' '/^[0-9]+: / && !/: lo: / {print $2}')
            do
                iface_ip=$(ip a show "$iface" | awk '/inet / {split($2, a, "/"); print a[1]}')
                echo -e "\t $count. $iface [$iface_ip]"
                array_test+=("$iface")
                count=$((count+1));
            done

            echo "Enter the number of the network device for Controller deployment : "
            read -r device_index;
            for i in "${array_test[@]}";
            do
                if [ "$count2" == "$device_index" ];then
                    device_name="$i";
                    device_found=1
                fi
                count2=$((count2+1));
            done
            if [[ $device_found -eq 0 ]]; then
                invisily_warn "You did not enter correct number of available network device, please re-enter correct option below!"
            fi
        done
    fi
    device_name_ip=$(ip a show "$device_name" | awk '/inet / {split($2, a, "/"); print a[1]}')

    if [[ $MULTI_HOME -eq 1 ]]; then
        if [[ -z $device_name2 ]]; then
            device_found=0
            while [[ $device_found -ne 1 ]]
            do
                array_test=()

                argstr=""
                declare -i count=0
                declare -i count2=0
                echo "Available Network Devices Controller Deployment are :"
                for iface in $(ip a | awk -F ': ' '/^[0-9]+: / && !/: lo: / {print $2}')
                do
                    if [[ "$iface" != "$device_name" ]]; then
                        iface_ip=$(ip a show "$iface" | awk '/inet / {split($2, a, "/"); print a[1]}')
                        echo -e "\t $count. $iface [$iface_ip]"
                        array_test+=("$iface")
                        count=$((count+1));
                    fi
                done

                echo "Enter the number of the network device for Multi-Home Controller deployment : "
                read -r device_index;
                for i in "${array_test[@]}";
                do
                    if [ "$count2" == "$device_index" ];then
                        device_name2="$i";
                        device_found=1
                    fi
                    count2=$((count2+1));
                done
                if [[ $device_found -eq 0 ]]; then
                    echo "You did not enter correct number of available network device, please re-enter correct option below!"
                fi
            done
        fi
        device_name2_ip=$(ip a show "$device_name2" | awk '/inet / {split($2, a, "/"); print a[1]}')
        cp -r "$MY_PATH"/sniffer_config.cfg "$MY_PATH"/sniffer_config-"$device_name2".cfg
        sed_args="s/networkid_value_placeholder/$device_name2/g";
        sed -i "$sed_args" ./sniffer_config-"$device_name2".cfg
        sed -i "s/component_IPAddress=.*/component_IPAddress=\"$device_name2_ip\"/g"  sniffer_config-"$device_name2".cfg
        cp -r /opt/invisily/sdp-controller/bin /opt/invisily/sdp-controller/bin-"$device_name2"
        mv /opt/invisily/sdp-controller/bin-"$device_name2"/sniffer.out /opt/invisily/sdp-controller/bin-"$device_name2"/sniffer-"$device_name2".out
        mv -f sniffer_config-"$device_name2".cfg /opt/invisily/sdp-controller/bin-"$device_name2"/sniffer_config.cfg
        [[ -f /etc/systemd/system/invisily_sniffer-$device_name2.service ]] && rm -rf /etc/systemd/system/invisily_sniffer-"$device_name2".service && systemctl daemon-reload
        cp -r /etc/systemd/system/invisily_sniffer.service /etc/systemd/system/invisily_sniffer-"$device_name2".service
        sed -i "s|WorkingDirectory=.*|WorkingDirectory=/opt/invisily/sdp-controller/bin-$device_name2|g"  /etc/systemd/system/invisily_sniffer-"$device_name2".service
        sed -i "s|ExecStart=.*|ExecStart=/opt/invisily/sdp-controller/bin-$device_name2/sniffersd.sh start|g"  /etc/systemd/system/invisily_sniffer-"$device_name2".service
        sed -i "s|SyslogIdentifier=.*|SyslogIdentifier=invisily_sniffer_multi_home_$device_name2|g"  /etc/systemd/system/invisily_sniffer-"$device_name2".service
        sed -i "s|Description=.*|Description= Sniffer Service Multi Home $device_name2|g"  /etc/systemd/system/invisily_sniffer-"$device_name2".service
        sed -i "s|PIDFile=.*|PIDFile=/var/run/sniffersd_$device_name2.pid|g"  /etc/systemd/system/invisily_sniffer-"$device_name2".service

        cat > /etc/systemd/system/multi-home.service <<END
[Unit]
Description= Controller Sniffer MultiHome
Documentation= http://invisily.com/
Wants=network.target
After=network.target
After=multi-user.target
After=invisily_sniffer.service
After=invisily_sniffer-$device_name2.service
After=invisily_controller.service


[Service]
Type=oneshot
User=root
Group=root
WorkingDirectory=/opt/invisily/sdp-controller/bin
ExecStart=/opt/invisily/sdp-controller/bin/controller.sh start
RemainAfterExit=true
ExecStop=/opt/invisily/sdp-controller/bin/controller.sh stop
StandardOutput=journal

[Install]
WantedBy=multi-user.target
END
        sed -i '/After=multi-user.target/a After=invisily_sniffer.service' /etc/systemd/system/invisily_sniffer-"$device_name2".service
        sed -i '/After=invisily_sniffer.service/a After=invisily_sniffer-'"$device_name2"'.service' /etc/systemd/system/invisily_controller.service
        systemctl daemon-reload
        systemctl enable invisily_sniffer-"$device_name2".service
        systemctl enable multi-home
        sed -i "s|sniffer.out|sniffer-$device_name2.out|g" /opt/invisily/sdp-controller/bin-"$device_name2"/sniffersd.sh
        if [[ -f /etc/apparmor.d/opt.invisily.sdp-controller.bin.sniffer.$device_name2 ]]; then
            apparmor_parser -R /etc/apparmor.d/opt.invisily.sdp-controller.bin.sniffer."$device_name2"
            rm -rf /etc/apparmor.d/opt.invisily.sdp-controller.bin.sniffer."$device_name2"
        fi
        cp -r /etc/apparmor.d/opt.invisily.sdp-controller.bin.sniffer /etc/apparmor.d/opt.invisily.sdp-controller.bin.sniffer."$device_name2"
        sed -i "s|sniffer.out|sniffer-$device_name2.out|g" /etc/apparmor.d/opt.invisily.sdp-controller.bin.sniffer."$device_name2"
        sed -i "s|/opt/invisily/sdp-controller/bin|/opt/invisily/sdp-controller/bin-$device_name2|g" /etc/apparmor.d/opt.invisily.sdp-controller.bin.sniffer."$device_name2"
        apparmor_parser -r /etc/apparmor.d/opt.invisily.sdp-controller.bin.sniffer."$device_name2"
        service apparmor reload
        cat > /opt/invisily/sdp-controller/bin/controller.sh <<END
#!/bin/bash
### BEGIN INIT INFO
# Short-Description:        Start and stop sniffer & controller services.
# Description:              -
# Date-Creation:            -
# Date-Last-Modification:   -
# Author:                   -
### END INIT INFO

# Variables
PGREP=/usr/bin/pgrep
ZERO=0

# Start Sniffer & controller services.
start() {

    echo "Stopping [ CONTROLLER ] & [ SNIFFER    ] ..."
    systemctl stop invisily_controller.service
    systemctl stop invisily_sniffer.service
    systemctl stop invisily_sniffer-$device_name2.service
    systemctl stop invisily_auto_update.service

    sleep 2
    echo "Starting [ SNIFFER    ] ..."
    systemctl start invisily_sniffer.service
    sleep 2
    echo "Starting [ SNIFFER_$device_name2    ] ..."
    systemctl start invisily_sniffer-$device_name2.service
    sleep 2
    sudo iptables -A INPUT -i $device_name -j DROP
    sleep 2
    echo "Starting [ CONTROLLER ] ..."
    systemctl start invisily_controller.service

    sleep 2
    echo "Starting [ Controller Update Service ] ..."
    systemctl start invisily_auto_update.service

    echo ""
    echo " Please check the status of components by sudo ./controller.sh status "

}


# Stop Sniffer
stop() {

    systemctl stop invisily_controller.service
    echo "[ CONTROLLER ] Stopped ..."

    systemctl stop invisily_sniffer.service
    echo "[ SNIFFER    ] Stopped ..."
    systemctl stop invisily_sniffer-$device_name2.service
    echo "[ SNIFFER_$device_name2    ] Stopped ..."
    sudo iptables -F

    systemctl stop invisily_auto_update.service
    echo "[ Controller Update Service ] Stopped ..."

}

# Verify status of Sniffer
status() {

    STATUS=\`systemctl is-active invisily_controller.service\`
    if [[ \${STATUS} == 'active' ]]; then
        echo "[ CONTROLLER ] Service is active & running ..."
    else
        echo "[ CONTROLLER ] Service is stopped "
    fi

    STATUS=\`systemctl is-active invisily_sniffer.service\`
    if [[ \${STATUS} == 'active' ]]; then
        echo "[ SNIFFER    ] Service is active & running ..."
    else
        echo "[ SNIFFER    ] Service is stopped "
    fi

    STATUS=\`systemctl is-active invisily_sniffer-$device_name2.service\`
    if [[ \${STATUS} == 'active' ]]; then
        echo "[ SNIFFER2_$device_name2    ] Service is active & running ..."
    else
        echo "[ SNIFFER2_$device_name2    ] Service is stopped "
    fi

    STATUS=$(systemctl is-active invisily_auto_update.service)
    if [[ ${STATUS} == 'active' ]]; then
        echo "[ Controller Update Service ] Service is active & running ..."
    else
        echo "[ Controller Update Service ] Service is stopped "
    fi
}

# Main logic
case "\$1" in
    start)
        start
        ;;
    stop)
        stop
        ;;
    status)
        status
        ;;
    restart|reload)
        stop
        start
        ;;
  *)
    echo \$"Usage: \$0 {start|stop|status|restart|reload}"
    exit 1
esac
exit 0
END
    cp -r /opt/invisily/sdp-controller/bin/controller.sh /opt/invisily/sdp-controller/bin-"$device_name2"/controller.sh
    sed -i "s|#enable_multihome=0|enable_multihome=1|g" /opt/invisily/sdp-controller/watchdog/conf/watchdog.cnf
    sed -i "s|#multi_sniffer_iface_name=<value>|multi_sniffer_iface_name=$device_name2|g" /opt/invisily/sdp-controller/watchdog/conf/watchdog.cnf
    sed -i "/ptrace trace peer=.*\/snifer.out,/a \  ptrace trace peer=\/opt\/invisily\/sdp-controller\/bin-$device_name2\/sniffer-$device_name2.out," /etc/apparmor.d/opt.invisily.sdp-controller.controller.ctrlsd
    apparmor_parser -r /etc/apparmor.d/opt.invisily.sdp-controller.controller.ctrlsd
    fi

    sed_args="s/networkid_value_placeholder/$device_name/g";
    sed -i "$sed_args" ./sniffer_config.cfg
    sed -i "s/component_IPAddress=.*/component_IPAddress=\"$device_name_ip\"/g"  ./sniffer_config.cfg
    sed -i "$sed_args" ./ctrl.ini

    if [[ $MULTI_HOME -eq 1 ]]; then
        sed -i "s/ControllerIP=.*/ControllerIP=0.0.0.0/g"  ./ctrl.ini
    else
        sed -i "s/ControllerIP=.*/ControllerIP=$device_name_ip/g"  ./ctrl.ini
    fi

    if [[ -z $dnsip ]]; then
        dnsip=$(cat < /etc/resolv.conf | grep nameserver | awk -F ' ' '{print $2}')
        if [[ -z $dnsip || "$dnsip" == *"127.0"* ]]; then
            dnsip=$(cat /etc/netplan/*.yaml | grep -A1 nameservers | grep addresses | awk -F ' ' '{print $2}' | cut -d "[" -f2 | cut -d "]" -f1 | awk -F ',' '{print $1}')
            if [[ -z $dnsip || "$dnsip" == *"127.0"* ]]; then
                dnsip=$(systemd-resolve --status | grep -A9 eth0 | grep 'DNS Servers' | awk -F ':' '{print $2}' | cut -d ' ' -f 2)
                if [[ -z $dnsip || "$dnsip" == *"127.0"* ]]; then
                    invisily_warn "No Valid dns ip is found from your system configuration, please enter valid dns ip"
                    echo "Enter the DNS Server IP (Only one IP e.g. 8.8.8.8) : "
                    read -r dnsip
                fi
            fi
        fi

        while true;
        do
            if ! __ip_validation "$dnsip" ; then
                invisily_warn "DNS IP you entered is not a valid dns ip"
                echo "Enter the DNS Server IP (Only one IP e.g. 8.8.8.8) : "
                read -r dnsip
            elif [[ "$dnsip" == *"127.0"* ]]; then
                invisily_warn "DNS IP cannot be local host IP"
                echo "Enter the DNS Server IP (Only one IP e.g. 8.8.8.8) : "
                read -r dnsip
            elif ! timeout 3 nslookup google.com "$dnsip" >/dev/null 2>&1 ; then
                if [[ $disable_dns_check -ne 1 ]]; then
                    invisily_warn "Your DNS IP is not resolving google.com, please enter a valid dns ip which really resolves domains"
                    echo "Enter the DNS Server IP (Only one IP e.g. 8.8.8.8) : "
                    read -r dnsip
                else
                    invisily_warn "DNS Server ip you have provided, is not resolving google.com, however this error is ignoring becuase you have passed --disable-dns-check"
                    break
                fi
            else
                break
            fi
        done
    fi

    invisily_note "DNS IP $dnsip will be used in configuration"
    if [[ -z $dns_port ]]; then
        dns_port=53
        invisily_note "DNS Port $dns_port will be used in configuration"
    fi
    if [[ -z $dns_protocol ]]; then
        dns_protocol="udp"
        invisily_note "DNS Protocol $dns_protocol will be used in configuration"
    fi

    sed_args="s/DNSIP_value_placeholder/$dnsip/g";
    sed -i "$sed_args" ./ctrl.ini

    sed_args="s/DNSPort_value_placeholder/$dns_port/g";
    sed -i "$sed_args" ./ctrl.ini

    sed_args="s/DNSProtocol_value_placeholder/$dns_protocol/g";
    sed -i "$sed_args" ./ctrl.ini


    invisily_note "invisily-controller Server post install : finishing step one ..."
    if [ -d "/opt/invisily/sdp-controller/controller/certs" ]; then
        rm -rf  /opt/invisily/sdp-controller/controller/certs
    fi
    if [ -d "./certs" ]; then
        mv -f ./certs /opt/invisily/sdp-controller/controller/
    else
        invisily_error "invisily-controller Server post install : Error finishing step one failed ..."
    fi

    invisily_note "invisily-controller Server post install : finishing step two ..."
    if [ -f "/opt/invisily/sdp-controller/controller/ctrl.ini" ]; then
        rm -f /opt/invisily/sdp-controller/controller/ctrl.ini
    fi
    if [ -f "/opt/invisily/sdp-controller/controller/rup.txt" ]; then
        rm -f /opt/invisily/sdp-controller/controller/rup.txt
    fi
    if [ -f "./ctrl.ini" ]; then
        mv -f ./ctrl.ini /opt/invisily/sdp-controller/controller/
    else
        invisily_error "invisily-controller Server post install : finishing step two failed ..."
    fi

    invisily_note "invisily-controller Server post install : finishing step three ..."
    if [ -d "/opt/invisily/sdp-controller/controller/mysqlssl" ]; then
        rm -rf /opt/invisily/sdp-controller/controller/mysqlssl
    fi
    if [ -d "./mysqlssl" ]; then
        mv -f ./mysqlssl /opt/invisily/sdp-controller/controller/
    else
        invisily_error "invisily-controller Server post install : Error finishing step three failed ..."
    fi

    invisily_note "invisily-controller Server post install : finishing step four ..."
    if [ -f "/opt/invisily/sdp-controller/bin/sniffer_config.cfg" ]; then
        rm -rf /opt/invisily/sdp-controller/bin/sniffer_config.cfg
    fi
    if [ -f "./sniffer_config.cfg" ]; then
        mv -f ./sniffer_config.cfg /opt/invisily/sdp-controller/bin/
    else
        invisily_error "invisily-controller Server post install : finishing step four failed ..."
    fi

    invisily_note "invisily-controller Server post install : finishing step five ..."
    if [ -f "./rup.ec" ]; then
        mv -f ./rup.ec /opt/invisily/sdp-controller/controller/
    else
        invisily_error "invisily-controller Server post install : finishing step five failed ..."
    fi

    invisily_note "invisily-controller Server post install : finishing step six ..."
    if [ -f "./rupk.txt" ]; then
        mv -f ./rupk.txt /opt/invisily/sdp-controller/controller/
    else
        invisily_error "invisily-controller Server post install : finishing step six failed ..."
    fi

    if [ "$ubuntuVersion" == "20.04" ]; then
        echo "making essential edit in /etc/ssl/openssl.cnf for Ubuntu20.04"
        exists=$(grep -c '##Invisily_Updated##' '/etc/ssl/openssl.cnf')
        if [[ $exists -gt 0 ]]; then
           echo "Already edited"
        else
            sed -i '1s/^/openssl_conf = default_conf\n/' /etc/ssl/openssl.cnf
            echo '##Invisily_Updated##
[ default_conf ]
ssl_conf = ssl_sect

[ssl_sect]

system_default = ssl_default_sect

[ssl_default_sect]
MinProtocol = None
CipherString = DEFAULT:@SECLEVEL=1' >>  /etc/ssl/openssl.cnf
        fi
    fi

    PORTAL_NTP_IP=$(cat < /opt/invisily/sdp-controller/controller/ctrl.ini | grep NTPIP | awk -F = '{print $2}')
    if [[ -z $PORTAL_NTP_IP ]]; then
        invisily_error "Unable to find NTPIP from /opt/invisily/sdp-controller/controller/ctrl.ini, NTP setup is failed!"
    fi
    cat > /etc/systemd/timesyncd.conf <<END
[Time]
NTP=$PORTAL_NTP_IP
#FallbackNTP=ntp.ubuntu.com
#RootDistanceMaxSec=5
PollIntervalMinSec=32
PollIntervalMaxSec=2048
END
    systemctl restart systemd-timesyncd
    invisily_note "Configuring NTP, Please wait ..."
    sleep 10
    if systemctl status systemd-timesyncd | grep Status | grep "Synchronized.*$PORTAL_NTP_IP\|synchronization.*$PORTAL_NTP_IP"; then
        invisily_result "NTP Server has been configured!"
    else
        invisily_warn "NTP Configuration Failed! Your NTP Server is not listing to UDP port 123, Please make sure your NTP Server is up"
    fi
    invisily_result "Invisily-controller has been installed successfully ..."
}

remove_controller(){
    if /opt/invisily/sdp-controller/bin/controller.sh stop; then
        invisily_note "Invisily Controller Service has been stopped"
    else
        invisily_warn "Invisily controller services got error while stopping"
    fi
    if apt_pkg_check invisily-controller; then
        apt-get purge invisily-controller -y
        invisily_note "Invisily controller package has been uninstalled successfully"
    else
        invisily_warn "Invisily Installer package uninstallation got error"
    fi
    if [[ -d "/opt/invisily/sdp-controller" ]]; then
        rm -rf  /opt/invisily/sdp-controller
    fi
    rm -rf /etc/systemd/system/invisily_sniffer*
    systemctl daemon-reload
    apparmor_parser -R /etc/apparmor.d/opt.invisily.sdp-controller.*
    service apparmor reload
    rm -rf /etc/apparmor.d/opt.invisily.sdp-controller.*
    service apparmor reload
    invisily_note "invisily-controller removed successfully"
}

update_variables() {
    if [[ -f "/var/local/env/sdp" ]]; then
        RABBITMQ_PASS=$(grep RABBITMQ_PASS /var/local/env/sdp | cut -d"=" -f2)
    fi
    if [[ -f "/var/local/env/sdp" ]]; then
        RABBITMQ_USER=$(grep RABBITMQ_USER /var/local/env/sdp | cut -d"=" -f2)
    fi
    if [[ -f "/var/local/env/sdp" ]]; then
        REDIS_PASSWORD=$(grep REDIS_PASSWORD /var/local/env/sdp | cut -d"=" -f2)
    fi
}

online_status() {
    INSTALL_MODE=$1
    COMPONENT=$2
    if [[ $INSTALL_MODE == "online" ]]; then
        sudo -H -u "$FS_USER" sudo bash <<CODESETUP
[[ "$VERBOSE" == "1" ]] && set -x
$(declare -f invisily_log)
$(declare -f invisily_warn)
$(declare -f invisily_note)
$(declare -f invisily_result)
$(declare -f invisily_error)
COMPONENT=$COMPONENT
ping -c3 -W1  github.com >/dev/null 2>&1 || ping -c3 -W1  google.com >/dev/null 2>&1 || ping -c3 -W1 archive.ubuntu.com >/dev/null 2>&1 || timeout 1 bash -c "echo &> /dev/null  < /dev/tcp/archive.ubuntu.com/80"
if [[ \$? -eq 0 ]]; then
    [[ -f "/etc/apt/sources.list.d/bintray.rabbitmq.list" ]] && rm -rf /etc/apt/sources.list.d/bintray.rabbitmq.list
    apt-get -y update
    if [[ \$? -ne 0 ]]; then
        invisily_error "apt-get failed due to unknown reason."
    fi
else
    invisily_error "Your <\$COMPONENT> machine  is not connected to Internet. Exiting..."
fi
CODESETUP
    fi
}

installation_status() {
    INSTALL_MODE=$1
    MY_PATH=$2
    OFFLINE_PKG=$3
    COMPONENT=$4
    if [[ $INSTALL_MODE == "online" ]]; then
        if ping -c3 -W1  github.com >/dev/null 2>&1 || ping -c3 -W1  google.com >/dev/null 2>&1 || ping -c3 -W1 archive.ubuntu.com >/dev/null 2>&1 || timeout 1 bash -c "echo &> /dev/null  < /dev/tcp/archive.ubuntu.com/80"; then
            ONLINE_INSTALL=1
            if ! apt-get -y update && apt-get -y install zip unzip; then
                invisily_error "Online Installation of dependencies for $COMPONENT has failed!"
            fi
        else
            invisily_error "Your machine is not connected to Internet. Exiting..."
        fi
    elif [[ $INSTALL_MODE == "offline" ]]; then
        if [[ ! -f /root/.DO_NOT_DELETE_INVISILY ]]; then
            invisily_error_no_exit "You are not using Invisily Provided OS image!"
        fi
        if [[ -f $MY_PATH/$OFFLINE_PKG ]]; then
            mkdir -p "$APT_ARCHIVE"
            apt-get clean
            tar xzvf "$MY_PATH"/"$OFFLINE_PKG" -C "$APT_ARCHIVE" || invisily_error "Extracting Offline Dependencies Failed."
            if [[ $COMPONENT == "unzip" ]]; then
                cd "$APT_ARCHIVE" || exit
                export DEBIAN_FRONTEND=noninteractive
                if ! apt-get install -y ./*.deb*; then
                    invisily_error "Offline Installation of $COMPONENT has failed!"
                fi
                apt-get clean
            fi
        else
            invisily_error "You does not seem to be using Invisily specific Ubuntu 22.04 image nor $OFFLINE_PKG is available in $MY_PATH"
        fi
    else
        invisily_error "You can only enter online or offline as first argument. Exiting..."
    fi
}

is_greater(){
    UPADTE_VERSION=$1
    CUR_VERSION=$2
    for i in {1..4}; do
        ind1=$(echo "$UPADTE_VERSION" | cut -d"." -f"$i")
        ind2=$(echo "$CUR_VERSION" | cut -d"." -f"$i")
        if [[ $ind1 -gt $ind2 ]]; then
            echo 1
            return
        fi
    done
}

update_onboard_portal() {
    invisily_note "Updating Onboarding Portal..."
    FS_USER_HOME_DIR=$(getent passwd "$FS_USER" | cut -d: -f6)
    ONBOARD_BASE_DIR=/var/www/html/onboarding-portal
    ONBOARD_CONFIG_DIR="/var/local/env"
    ONBOARD_CONFIG_FILE=$ONBOARD_CONFIG_DIR/onboard
    DATE=$(date '+%Y-%m-%d%H:%M:%S')

    [ ! -d "/var/log/invisily" ] && mkdir -p /var/log/invisily
    cd "/var/www/html" || exit
    mv "./onboarding-portal" "./onboarding-portal-${DATE}"
    #removing node_modules from old installation to save space
    rm -rf "${ONBOARD_BASE_DIR}-${DATE}/node_modules"
    chown -R "$FS_USER":"$FS_USER" "${ONBOARD_BASE_DIR}-${DATE}"

    # unzip onboard Portal
    cd "$OBP_DIR" || exit
    unzip -o "$INSTALLER_ONBOARD_PORTAL" -d $ONBOARD_BASE_DIR
    cp -r ${ONBOARD_BASE_DIR}-"${DATE}"/config/config.js ${ONBOARD_BASE_DIR}/config/ || exit 1
    cp -r ${ONBOARD_BASE_DIR}-"${DATE}"/bin/www ${ONBOARD_BASE_DIR}/bin/ || exit 1
    cp -r ${ONBOARD_BASE_DIR}-"${DATE}"/env/onboard ${ONBOARD_BASE_DIR}/env/ || exit 1
    chown -R "$FS_USER":"$FS_USER" "${ONBOARD_BASE_DIR}"

    mkdir -p /var/local/installers
    cp -r $ONBOARD_BASE_DIR/scripts/copy-file-to-path.sh /var/local/installers/copy-file-to-path.sh
    cp -r $ONBOARD_BASE_DIR/scripts/generate-ssl-certs.sh /var/local/installers/generate-ssl-certs.sh
    cp -r $ONBOARD_BASE_DIR/scripts/change-certs-permission.sh /var/local/installers/change-certs-permission.sh
    chown -R "$FS_USER":"$FS_USER" /var/local/installers

    chmod 755 /var/local/installers/copy-file-to-path.sh
    chmod 755 /var/local/installers/generate-ssl-certs.sh
    chmod 755 /var/local/installers/change-certs-permission.sh

    if grep REDIS_HOST $ONBOARD_CONFIG_FILE; then
        sed -e "s|REDIS_HOST=.*|REDIS_HOST=$REDIS_HOST|g" -i $ONBOARD_CONFIG_FILE
    else
        {
            echo "REDIS_HOST=\${REDIS_HOST}" >> $ONBOARD_CONFIG_FILE && {
                grep REDIS_HOST $ONBOARD_CONFIG_FILE && sed -e "s/\${REDIS_HOST}/$REDIS_HOST/" -i $ONBOARD_CONFIG_FILE
            }
        }
    fi
    if grep REDIS_PORT $ONBOARD_CONFIG_FILE; then
        sed -e "s|REDIS_PORT=.*|REDIS_PORT=$REDIS_PORT|g" -i $ONBOARD_CONFIG_FILE
    else
        {
            echo "REDIS_PORT=\${REDIS_PORT}" >> $ONBOARD_CONFIG_FILE && {
                grep REDIS_PORT $ONBOARD_CONFIG_FILE && sed -e "s/\${REDIS_PORT}/$REDIS_PORT/" -i $ONBOARD_CONFIG_FILE
            }
        }
    fi
    if grep REDIS_PASSWORD $ONBOARD_CONFIG_FILE; then
        sed -e "s|REDIS_PASSWORD=.*|REDIS_PASSWORD=$REDIS_PASSWORD|g" -i $ONBOARD_CONFIG_FILE
    else
        {
            echo "REDIS_PASSWORD=\${REDIS_PASSWORD}" >> $ONBOARD_CONFIG_FILE && {
                grep REDIS_PASSWORD $ONBOARD_CONFIG_FILE && sed -e "s/\${REDIS_PASSWORD}/$REDIS_PASSWORD/" -i $ONBOARD_CONFIG_FILE
            }
        }
    fi
    if ! grep RABBITMQ_USER $ONBOARD_CONFIG_FILE; then
        {
            echo "RABBITMQ_USER=\${RABBITMQ_USER}" >> $ONBOARD_CONFIG_FILE && {
                grep RABBITMQ_USER $ONBOARD_CONFIG_FILE && sed -e "s/\${RABBITMQ_USER}/$RABBITMQ_USER/" -i $ONBOARD_CONFIG_FILE
            }
        }
    fi
    if ! grep RABBITMQ_PASS $ONBOARD_CONFIG_FILE; then
        {
            echo "RABBITMQ_PASS=\${RABBITMQ_PASS}" >> $ONBOARD_CONFIG_FILE && {
                grep RABBITMQ_PASS $ONBOARD_CONFIG_FILE && sed -e "s/\${RABBITMQ_PASS}/$RABBITMQ_PASS/" -i $ONBOARD_CONFIG_FILE
            }
        }
    fi
    if grep RABBITMQ_HOST $ONBOARD_CONFIG_FILE; then
        sed -e "s|RABBITMQ_HOST=.*|RABBITMQ_HOST=$RABBITMQ_HOST|g" -i $ONBOARD_CONFIG_FILE
    else
        {
            echo "RABBITMQ_HOST=\${RABBITMQ_HOST}" >> $ONBOARD_CONFIG_FILE && {
                grep RABBITMQ_HOST $ONBOARD_CONFIG_FILE && sed -e "s/\${RABBITMQ_HOST}/$RABBITMQ_HOST/" -i $ONBOARD_CONFIG_FILE
            }
        }
    fi
    if ! grep RABBITMQ_PORT $ONBOARD_CONFIG_FILE; then
        {
            echo "RABBITMQ_PORT=\${RABBITMQ_PORT}" >> $ONBOARD_CONFIG_FILE && {
                grep RABBITMQ_PORT $ONBOARD_CONFIG_FILE && sed -e "s/\${RABBITMQ_PORT}/$RABBITMQ_PORT/" -i $ONBOARD_CONFIG_FILE
            }
        }
    fi

    cp -r $ONBOARD_BASE_DIR/certs_invisily /var/local/installers/
    cd $ONBOARD_BASE_DIR || exit

    if ! grep "inv  ALL=(ALL) NOPASSWD: /var/local/installers/generate-ssl-certs.sh" /etc/sudoers > /dev/null ; then
        echo "inv  ALL=(ALL) NOPASSWD: /var/local/installers/generate-ssl-certs.sh" | sudo tee -a /etc/sudoers > /dev/null
    fi

    [ ! -f ~/.rnd ] && touch ~/.rnd
    mkdir -p /var/local/ssl
    mv "$OBP_DIR"/sdp-key.pem  /var/local/ssl
    mv "$OBP_DIR"/sdp-cert.crt   /var/local/ssl
    if [ ! -d "/root/ca" ]; then
        tar xzvf "$OBP_DIR"/ca.tar.gz -C /root/
    fi
    chown -R "$FS_USER":"$FS_USER" /var/local/ssl

    if [[ $OFFLINE_INSTALL -eq 1 ]]; then
        cd "$OBP_DIR" || exit
        tar xzvf "$OFFLINE_NODE_INSTALLER_NAME"
        [[ ! -f ./node.tar.gz ]] && invisily_error $"Offline Node Installer seems corrupted since it did not extract required file node.tar.gz"
        [[ ! -f ./onboarding_portal_node_modules.tar.gz ]] && invisily_error $"Offline Node Installer seems corrupted since it did not extract required file onboarding_portal_node_modules.tar.gz"
        tar xzvf node.tar.gz -C "$FS_USER_HOME_DIR"
        chown -R "$FS_USER":"$FS_USER" "$FS_USER_HOME_DIR"/.nvm "$FS_USER_HOME_DIR"/.npm "$FS_USER_HOME_DIR"/.pm2
        tar xzvf onboarding_portal_node_modules.tar.gz -C $ONBOARD_BASE_DIR
        chown -R "$FS_USER":"$FS_USER" $ONBOARD_BASE_DIR
        if grep '.*nvm.*' "$FS_USER_HOME_DIR"/.bashrc; then
            sed -e 's/.*nvm.*//g' -i "$FS_USER_HOME_DIR"/.bashrc
        else
            cat >> "$FS_USER_HOME_DIR"/.bashrc <<EOF
export NVM_DIR="\$HOME/.nvm"
[ -s "\$NVM_DIR/nvm.sh" ] && \\. "\$NVM_DIR/nvm.sh"  # This loads nvm
[ -s "\$NVM_DIR/bash_completion" ] && \\. "\$NVM_DIR/bash_completion"  # This loads nvm bash_completion
EOF
        fi
    fi

    if [[ $ONLINE_INSTALL -eq 1 ]]; then
        sudo -H -u "$FS_USER" bash <<CODESETUP
cd ~
[ ! -f ~/.rnd ] && touch ~/.rnd
if [ ! -d ~/.nvm ]; then
  # NVM installation
  curl -o- https://raw.githubusercontent.com/creationix/nvm/v0.33.11/install.sh | bash

  export NVM_DIR="\$HOME/.nvm"
  [ -s "\$NVM_DIR/nvm.sh" ] && \\. "\$NVM_DIR/nvm.sh"  # This loads nvm
  [ -s "\$NVM_DIR/bash_completion" ] && \\. "\$NVM_DIR/bash_completion"  # This loads nvm bash_completion
fi

# Make NVM available in current shell
. ~/.nvm/nvm.sh >/dev/null 2>&1
ONBOARD_BASE_DIR=/var/www/html/onboarding-portal
cd \$ONBOARD_BASE_DIR
export NODE_ENV='production'
[[ -d node_modules ]] && rm -rf node_modules
npm install
npm install -g pm2 sequelize-cli
npm install
CODESETUP

        if [ $? != 0 ]; then
            invisily_error "Onboard Portal setup failed."
        fi
    fi
    sudo -H -u "$FS_USER" bash <<CODESETUP
. ~/.nvm/nvm.sh >/dev/null 2>&1
ONBOARD_BASE_DIR=/var/www/html/onboarding-portal
ONBOARD_CONFIG_DIR="/var/local/env"
ONBOARD_CONFIG_FILE=$ONBOARD_CONFIG_DIR/onboard
cd \$ONBOARD_BASE_DIR
. \$ONBOARD_CONFIG_FILE
pm2 delete inv
pm2 start \$ONBOARD_BASE_DIR/bin/www --name inv -i 0 --log-date-format "YYYY-MM-DD HH:mm Z"
pm2 save
pm2 startup systemd > /tmp/pm2istcode.txt
pm2 logrotate 2>&1 | tee /tmp/pm2logrotatecode.txt
CODESETUP

    if [ $? != 0 ]; then
        invisily_error "Onboard Portal update failed."
    fi

    [ ! -d "/var/log/invisily/onboard-log" ] && ln -s /home/"$FS_USER"/.pm2/logs /var/log/invisily/onboard-log
    cat < /tmp/pm2istcode.txt | grep sudo > /tmp/pm2fstcode.txt && bash /tmp/pm2fstcode.txt
    cat < /tmp/pm2logrotatecode.txt | grep sudo | cut -d']' -f2 > /tmp/pm2logrotatecode2.txt && bash /tmp/pm2logrotatecode2.txt
    [[ -f /etc/logrotate.d/pm2-$FS_USER ]] && sed -ie 's/weekly/hourly/g' /etc/logrotate.d/pm2-"$FS_USER"
    [[ ! -f /etc/cron.hourly/logrotate ]] && cp -r /etc/cron.daily/logrotate /etc/cron.hourly/logrotate
    rm -rf /tmp/pm2fstcode.txt /tmp/pm2istcode.txt /tmp/pm2logrotatecode.txt /tmp/pm2logrotatecode2.txt
    rm -rf ${ONBOARD_BASE_DIR}-"${DATE}"
}

onboard_setup() {
    installation_status "$INSTALL_MODE" "$MY_PATH" "$OFFLINE_UNZIP_INSTALLER" "unzip"
    rm -rf "$OBP_DIR"
    if [ -f "$OBP_ZIP" ]; then
        unzip -o "$OBP_ZIP" -d "$OBP_DIR"
    else
        invisily_error "$OBP_ZIP doesn't exists."
    fi

    source "$OBP_DIR/$OBP_CONFIG_TXT"
    cd "$OBP_DIR" || exit
    # fresh installation
    ONBOARD_PM2_START_FILE=$(sudo ls /etc/systemd/system/pm2-*)
    if [[ -z $ONBOARD_PM2_START_FILE ]]; then
        invisily_note "Starting fresh deployment of Onboarding Portal..."
        ONBOARD_BASE_DIR=/var/www/html/onboarding-portal
        ONBOARD_CONFIG_DIR="/var/local/env"
        ONBOARD_CONFIG_FILE=$ONBOARD_CONFIG_DIR/onboard
        [ -d "$ONBOARD_CONFIG_DIR" ] && rm -rf "$ONBOARD_CONFIG_DIR"
        [ -d "/var/www/html" ] && rm -rf /var/www/html
        mkdir -p /var/log/invisily/
        installation_status "$INSTALL_MODE" "$MY_PATH" "$OFFLINE_ONBOARD_INSTALLER_NAME" "onboard-portal"
        if [[ $ONLINE_INSTALL == 1 ]]; then
            export DEBIAN_FRONTEND=noninteractive
            if ! apt-get -y update && apt-get -y install gcc curl build-essential software-properties-common openssl libxml2-dev zlib1g-dev; then
                invisily_error "Online Installation of dependencies for onboarding portal has failed!"
            fi
        fi
        if [[ $OFFLINE_INSTALL == 1 ]]; then
            cd "$APT_ARCHIVE" || exit
            export DEBIAN_FRONTEND=noninteractive
            if ! apt-get install -y ./*.deb*; then
                invisily_error "Offline Installation of dependencies for onboarding portal has failed!"
            fi
            apt-get clean
        fi

        mkdir -p $ONBOARD_BASE_DIR
        chown -R "$FS_USER":"$FS_USER" $ONBOARD_BASE_DIR
        cd "$OBP_DIR" || exit
        unzip -o "$INSTALLER_ONBOARD_PORTAL" -d $ONBOARD_BASE_DIR
        mkdir -p $ONBOARD_CONFIG_DIR
        mkdir -p /var/local/installers
        chown -R "$FS_USER":"$FS_USER" /var/local/installers
        cp $ONBOARD_BASE_DIR/env/onboard $ONBOARD_CONFIG_FILE
        chown -R "$FS_USER":"$FS_USER" $ONBOARD_CONFIG_DIR
        cp -r $ONBOARD_BASE_DIR/scripts/copy-file-to-path.sh /var/local/installers/copy-file-to-path.sh
        cp -r $ONBOARD_BASE_DIR/scripts/generate-ssl-certs.sh /var/local/installers/generate-ssl-certs.sh
        cp -r $ONBOARD_BASE_DIR/scripts/change-certs-permission.sh /var/local/installers/change-certs-permission.sh

        chmod 755 /var/local/installers/copy-file-to-path.sh
        chmod 755 /var/local/installers/generate-ssl-certs.sh
        chmod 755 /var/local/installers/change-certs-permission.sh

        cp -r $ONBOARD_BASE_DIR/certs_invisily /var/local/installers/
        if ! sed -e "/\${INV_DB_USER}/{s//$MYSQL_USER/;h};\${x;/./{x;q0};x;q1}" -i $ONBOARD_CONFIG_FILE; then
            invisily_error $'Updating MySQL Username in ' $ONBOARD_CONFIG_FILE  'is failed'
        fi
        if ! sed -e "/\${INV_DB_HOST}/{s//$MYSQL_HOST/;h};\${x;/./{x;q0};x;q1}" -i $ONBOARD_CONFIG_FILE; then
            invisily_error $'Updating MySQL Hostname in ' $ONBOARD_CONFIG_FILE  'is failed'
        fi
        if ! sed -e "/\${INV_DB_PASSWORD}/{s//$MYSQL_PASSWORD/;h};\${x;/./{x;q0};x;q1}" -i $ONBOARD_CONFIG_FILE; then
            invisily_error $'Updating MySQL Password in ' $ONBOARD_CONFIG_FILE  'is failed'
        fi

        if ! sed -e "/{ONBOARDING_PORT}/{s//$OnboardingPort/;h};\${x;/./{x;q0};x;q1}" -i $ONBOARD_CONFIG_FILE; then
            invisily_error $'Updating OnBoarding portal port number in ' $ONBOARD_CONFIG_FILE  'is failed'
        fi

        if ! sed -e "/\${MFA_PORT}/{s//$MfaServerPort/;h};\${x;/./{x;q0};x;q1}" -i $ONBOARD_CONFIG_FILE; then
            invisily_error $'Updating MFA port number in ' $ONBOARD_CONFIG_FILE  'is failed'
        fi
        if ! sed -e "/{ONBOARDING_ENV}/{s//$ONBOARDING_ENV/;h};\${x;/./{x;q0};x;q1}" -i $ONBOARD_CONFIG_FILE; then
            invisily_error $'Updating OnBoarding environment var in ' $ONBOARD_CONFIG_FILE  'is failed'
        fi
        if ! sed -e "/\${CONTENT_URL}/{s//$CONTENT_URL/;h};\${x;/./{x;q0};x;q1}" -i $ONBOARD_CONFIG_FILE; then
            invisily_error $'Updating Content URL in ' $ONBOARD_CONFIG_FILE  'is failed'
        fi
        if ! sed -e "/\${JWT_SECRET_KEY}/{s//$JWT_SECRET_KEY/;h};\${x;/./{x;q0};x;q1}" -i $ONBOARD_CONFIG_FILE; then
            invisily_error $'Updating JWT SECRET in ' $ONBOARD_CONFIG_FILE  'is failed'
        fi
        if ! sed -e "/\${REDIS_HOST}/{s//$REDIS_HOST/;h};\${x;/./{x;q0};x;q1}" -i $ONBOARD_CONFIG_FILE; then
            invisily_error $'Updating Redis Server IP in ' $ONBOARD_CONFIG_FILE  'is failed'
        fi
        if ! sed -e "/\${REDIS_PORT}/{s//$REDIS_PORT/;h};\${x;/./{x;q0};x;q1}" -i $ONBOARD_CONFIG_FILE; then
            invisily_error $'Updating Redis Server Port in ' $ONBOARD_CONFIG_FILE  'is failed'
        fi
        if ! sed -e "/\${REDIS_PASSWORD}/{s//$REDIS_PASSWORD/;h};\${x;/./{x;q0};x;q1}" -i $ONBOARD_CONFIG_FILE; then
            invisily_error $'Updating Redis Server Password in ' $ONBOARD_CONFIG_FILE  'is failed'
        fi
        if ! sed -e "/\${RABBITMQ_HOST}/{s//$RABBITMQ_HOST/;h};\${x;/./{x;q0};x;q1}" -i $ONBOARD_CONFIG_FILE; then
            invisily_error $'Updating RabbitMQ IP in ' $ONBOARD_CONFIG_FILE  'is failed'
        fi

        if ! sed -e "/\${RABBITMQ_PORT}/{s//$RabbitMQPort/;h};\${x;/./{x;q0};x;q1}" -i $ONBOARD_CONFIG_FILE; then
            invisily_error $'Updating RabbitMQ Port in ' $ONBOARD_CONFIG_FILE  'is failed'
        fi
        if ! sed -e "/\${RABBITMQ_USER}/{s//$RABBITMQ_USER/;h};\${x;/./{x;q0};x;q1}" -i $ONBOARD_CONFIG_FILE; then
            invisily_error $'Updating RabbitMQ Username in ' $ONBOARD_CONFIG_FILE  'is failed'
        fi
        if ! sed -e "/\${RABBITMQ_PASS}/{s//$RABBITMQ_PASS/;h};\${x;/./{x;q0};x;q1}" -i $ONBOARD_CONFIG_FILE; then
            invisily_error $'Updating RabbitMQ Password in ' $ONBOARD_CONFIG_FILE  'is failed'
        fi

        FS_USER_HOME_DIR=$(getent passwd "$FS_USER" | cut -d: -f6)
        invisily_note $'Generating SSL certificates...'
        [ ! -f ~/.rnd ] && touch ~/.rnd
        mkdir -p /var/local/ssl
        mv "$OBP_DIR"/sdp-key.pem  /var/local/ssl
        mv "$OBP_DIR"/sdp-cert.crt   /var/local/ssl
        tar xzvf "$OBP_DIR"/ca.tar.gz -C /root/
        chown -R "$FS_USER":"$FS_USER" /var/local/ssl

        if [[ $OFFLINE_INSTALL -eq 1 ]]; then
            cd "$OBP_DIR" || exit
            tar xzvf "$OFFLINE_NODE_INSTALLER_NAME"
            [[ ! -f ./node.tar.gz ]] && invisily_error $"Offline Node Installer seems corrupted since it did not extract required file node.tar.gz"
            [[ ! -f ./onboarding_portal_node_modules.tar.gz ]] && invisily_error $"Offline Node Installer seems corrupted since it did not extract required file onboarding_portal_node_modules.tar.gz"
            tar xzvf node.tar.gz -C "$FS_USER_HOME_DIR"
            chown -R "$FS_USER":"$FS_USER" "$FS_USER_HOME_DIR"/.nvm "$FS_USER_HOME_DIR"/.npm "$FS_USER_HOME_DIR"/.pm2
            tar xzvf onboarding_portal_node_modules.tar.gz -C $ONBOARD_BASE_DIR
            chown -R "$FS_USER":"$FS_USER" $ONBOARD_BASE_DIR
            if grep '.*nvm.*' "$FS_USER_HOME_DIR"/.bashrc; then
                sed -e 's/.*nvm.*//g' -i "$FS_USER_HOME_DIR"/.bashrc
            else
                cat >> "$FS_USER_HOME_DIR"/.bashrc <<EOF
export NVM_DIR="\$HOME/.nvm"
[ -s "\$NVM_DIR/nvm.sh" ] && \\. "\$NVM_DIR/nvm.sh"  # This loads nvm
[ -s "\$NVM_DIR/bash_completion" ] && \\. "\$NVM_DIR/bash_completion"  # This loads nvm bash_completion
EOF
            fi
        fi

        if [[ $ONLINE_INSTALL -eq 1 ]]; then
            sudo -H -u "$FS_USER" bash <<CODESETUP
cd ~
[ ! -f ~/.rnd ] && touch ~/.rnd
if [ ! -d ~/.nvm ]; then
  # NVM installation
  curl -o- https://raw.githubusercontent.com/creationix/nvm/v0.33.11/install.sh | bash

  export NVM_DIR="\$HOME/.nvm"
  [ -s "\$NVM_DIR/nvm.sh" ] && \\. "\$NVM_DIR/nvm.sh"  # This loads nvm
  [ -s "\$NVM_DIR/bash_completion" ] && \\. "\$NVM_DIR/bash_completion"  # This loads nvm bash_completion
fi

# Make NVM available in current shell
. ~/.nvm/nvm.sh >/dev/null 2>&1

nvm install 18.19.0
nvm alias default 18.19.0
nvm use 18.19.0
ONBOARD_BASE_DIR=/var/www/html/onboarding-portal
cd \$ONBOARD_BASE_DIR

export NODE_ENV='production'

npm install
npm install -g pm2 sequelize-cli
npm install
CODESETUP

            if [ $? != 0 ]; then
                invisily_error "Onboard Portal setup failed."
            fi
        fi

        sudo -H -u "$FS_USER" bash <<CODESETUP
. ~/.nvm/nvm.sh >/dev/null 2>&1
ONBOARD_BASE_DIR=/var/www/html/onboarding-portal
ONBOARD_CONFIG_DIR="/var/local/env"
ONBOARD_CONFIG_FILE=\$ONBOARD_CONFIG_DIR/onboard
cd \$ONBOARD_BASE_DIR
. \$ONBOARD_CONFIG_FILE
pm2 start \$ONBOARD_BASE_DIR/bin/www --name inv -i 0 --log-date-format "YYYY-MM-DD HH:mm Z"
pm2 save
pm2 startup systemd > /tmp/pm2istcode.txt
pm2 logrotate 2>&1 | tee /tmp/pm2logrotatecode.txt
CODESETUP

        if [ $? != 0 ]; then
            invisily_error "Onboard Portal setup failed."
        fi

        cat < /tmp/pm2istcode.txt | grep sudo > /tmp/pm2fstcode.txt
        bash /tmp/pm2fstcode.txt
        ln -s /home/"$FS_USER"/.pm2/logs /var/log/invisily/onboard-log
        cat < /tmp/pm2logrotatecode.txt | grep sudo | cut -d']' -f2 > /tmp/pm2logrotatecode2.txt
        bash /tmp/pm2logrotatecode2.txt
        [[ -f /etc/logrotate.d/pm2-$FS_USER ]] && sed -ie 's/weekly/hourly/g' /etc/logrotate.d/pm2-"$FS_USER"
        [[ ! -f /etc/cron.hourly/logrotate ]] && cp -r /etc/cron.daily/logrotate /etc/cron.hourly/logrotate
        rm -rf /tmp/pm2fstcode.txt /tmp/pm2istcode.txt /tmp/pm2logrotatecode.txt /tmp/pm2logrotatecode2.txt
        if ps aux | pgrep pm2 | xargs -r kill -9; then
            invisily_warn "Existing pm2 processes are killed"
        else
            invisily_warn "There was no pm2 process running already"
        fi
        systemctl enable pm2-"${FS_USER}".service
        systemctl restart pm2-"${FS_USER}".service
        chown -R "$FS_USER":"$FS_USER" "$ONBOARD_BASE_DIR"

        invisily_result "========================================================"
        invisily_result "Invisily On-Board Portal has been successfully installed"
        invisily_result "========================================================"
    else
        # update obp
        INSTALLER_ONBOARD_PORTAL_ZIP="$OBP_DIR/$INSTALLER_ONBOARD_PORTAL"
        INSTALLER_ONBOARD_PORTAL_DIR="${INSTALLER_ONBOARD_PORTAL%.*}"

        if ! unzip -o "$INSTALLER_ONBOARD_PORTAL_ZIP" -d "$INSTALLER_ONBOARD_PORTAL_DIR"; then
            invisily_error "$INSTALLER_ONBOARD_PORTAL_ZIP not found"
        fi


        source "$OBP_VER_FILE"
        CUR_OBP_VER="$MAJOR_RELEASE.$MINOR_RELEASE.$PATCH_NUMBER.$BUILD_NUMBER"
        UPDATE_OBP_VER=$(echo "$INSTALLER_ONBOARD_PORTAL_DIR" | grep -Eo "([[:digit:]]+\.){3}[[:digit:]]+")
        if [[ $(is_greater "$UPDATE_OBP_VER" "$CUR_OBP_VER") ]]; then
            invisily_note "Onboarding portal update Detected. Updating Onboarding portal..."
            # stop controller
            /opt/invisily/sdp-controller/bin/controller.sh stop
            online_status "$INSTALL_MODE" "onboard-portal"
            update_variables
            # update starts here
            update_onboard_portal
            invisily_note "============================================";
            invisily_note "OnBoard Portal has been successfully updated";
            invisily_note "============================================";
            chown -R "$FS_USER":"$FS_USER" /var/www/html/onboarding-portal
        else
            invisily_note "Latest Onboarding portal is already installed!"
        fi
    fi
}

mfa_setup(){
    installation_status "$INSTALL_MODE" "$MY_PATH" "$OFFLINE_UNZIP_INSTALLER" "unzip"
    rm -rf "$MFA_DIR"
    MFA_START_FILE=$(sudo ls /etc/systemd/system/mfa.service)
    if [ -f "$MFA_ZIP" ]; then
        unzip -o "$MFA_ZIP" -d "$MFA_DIR"
    else
        invisily_error "$MFA_ZIP doesn't exists"
    fi

    source "$MFA_DIR/$MFA_CONFIG_TXT"

    cd "$MFA_DIR" || exit
    tar xzvf "$MFA_DIR"/"$INSTALLER_MFA"
    [[ ! -f $MFA_DIR/mfa_server.zip ]] && invisily_error "MFA Setup is not available in current directory, please copy setup in current directory and run this script again!" && exit 1
    [[ ! -f $MFA_DIR/m2.tar.gz ]] && invisily_error "MFA Offline Setup is not available in current directory, please copy setup in current directory and run this script again!" && exit 1
    unzip -o "$MFA_DIR"/mfa_server.zip -d "./mfa_suite"
    UPDATE_MFA_VERSION=$(grep -Eo "([[:digit:]]+\.){3}[[:digit:]]+-SNAPSHOT" "$UPDATE_MFA_VER_FILE" | cut -d"-" -f1)
    # fresh installation
    if [[ -z $MFA_START_FILE ]]; then
        installation_status "$INSTALL_MODE" "$MY_PATH" "$OFFLINE_DEB_INSTALLER_NAME_MFA" "mfa"
        if [[ $ONLINE_INSTALL == 1 ]]; then
            apt-get -y update
            if ! apt-get install -y openjdk-8-jdk  openjdk-8-jre  maven; then
                invisily_error "Offline Installation of dependencies for mfa has failed!"
            fi
        elif [[ $OFFLINE_INSTALL == 1 ]]; then
            cd "$APT_ARCHIVE" || exit
            export DEBIAN_FRONTEND=noninteractive
            if ! apt-get install -y ./*.deb*; then
                invisily_error "Offline Installation of dependencies for mfa has failed!"
            fi
            apt-get clean
        fi

        if [ ! -f "$MFA_DIR"/"$INSTALLER_MFA" ]; then
            invisily_error "MFA setup is not available in $MFA_DIR, please copy setup in $MFA_DIR directory and run this script again!"
        fi

        if [ ! -d /opt/invisily/mfa_suite ]; then
            mkdir -p /opt/invisily/mfa_suite/
        fi

        if [ ! -d /var/log/invisily/mfa_suite/mfa_server ]; then
            mkdir -p /var/log/invisily/mfa_suite/mfa_server
            chown -R "$FS_USER":"$FS_USER" /var/log/invisily/mfa_suite/mfa_server
        fi

        if [ -d /opt/invisily/mfa_suite/mfa_server ]; then
            rm -rf /opt/invisily/mfa_suite/mfa_server
        fi

        unzip -o "$MFA_DIR"/mfa_server.zip -d /opt/invisily/mfa_suite/
        if [[ $OFFLINE_INSTALL == 1 ]]; then
            tar xzvf "$MFA_DIR"/m2.tar.gz -C /home/"$FS_USER"
            chown -R "$FS_USER":"$FS_USER" /home/"$FS_USER"/.m2
        fi

        if [ -d /opt/invisily/mfa_suite/__MACOSX ]; then
            rm -rf /opt/invisily/mfa_suite/__MACOSX
        fi

        sed -i -e 's/development.username=.*/development.username='"$MFA_DB_USER"'/g' /opt/invisily/mfa_suite/mfa_server/fido-uaf-core/src/main/resources/database.properties
        sed -i -e 's/development.password.*/development.password='"$MFA_DB_PASS"'/g' /opt/invisily/mfa_suite/mfa_server/fido-uaf-core/src/main/resources/database.properties
        DB_IP_ADDR=$MYSQL_HOST
        sed -i -e 's/development.url=.*/development.url=jdbc:mysql:\/\/'"$DB_IP_ADDR"'\/mfa/g' /opt/invisily/mfa_suite/mfa_server/fido-uaf-core/src/main/resources/database.properties

        chown -R "$FS_USER":"$FS_USER" /opt/invisily/mfa_suite/
        sudo -H -u "$FS_USER" bash <<CODESETUP
cd /opt/invisily/mfa_suite/mfa_server/fido-uaf-core
mvn db-migrator:create
mvn db-migrator:migrate
mvn process-classes
mvn clean install
CODESETUP

        cp /opt/invisily/mfa_suite/mfa_server/fidouaf/pom.xml /opt/invisily/mfa_suite/mfa_server/fidouaf/pom.xml-back
        sed  -i  -e   's/9091/'"$MfaServerPort"'/g'     /opt/invisily/mfa_suite/mfa_server/fidouaf/pom.xml

        #generate SSL certificate
        cd /opt/invisily/mfa_suite/mfa_server/fidouaf/src/main/webapp || exit
        openssl req -new -newkey rsa:2048 -days 3650 -nodes -x509 -subj "/C=NA/ST=NA/L=NA/O=NA/CN=invisily.com" -keyout mfacert.key -out mfacert.crt
        openssl pkcs12 -export -in mfacert.crt -inkey mfacert.key -out keystore.p12 -name localhost  -passout pass:ebryx123
        { echo "ebryx123"; echo "ebryx123"; echo "ebryx123"; } | keytool -importkeystore -noprompt -destkeystore keystore.jks -srckeystore keystore.p12 -srcstoretype pkcs12
        echo "ebryx123" | keytool -importkeystore -srckeystore keystore.jks -destkeystore keystore.jks -deststoretype pkcs12
        mv keystore.jks .keystore
        chown -R "$FS_USER":"$FS_USER" .keystore
        rm -rf mfacert.key mfacert.crt keystore.p12 keystore.jks.old

        if [[ $ONLINE_INSTALL == 1 ]]; then
            sudo -H -u "$FS_USER" bash <<CODESETUP
cd /opt/invisily/mfa_suite/mfa_server/fidouaf
timeout 3m /usr/bin/mvn package tomcat7:run
CODESETUP
        fi

        if [ -f /etc/systemd/system/mfa.service ]; then
            rm -rf /etc/systemd/system/mfa.service
            echo 'File removed!'
            systemctl daemon-reload
        fi

        cat > /tmp/mfa.service <<EOF
[Unit]
Description=MFA fidouaf
After=network.target

[Service]
Type=simple
User=$FS_USER
WorkingDirectory=/opt/invisily/mfa_suite/mfa_server/fidouaf
ExecStart=/usr/bin/mvn package -o tomcat7:run
RemainAfterExit=no
Restart=on-failure
RestartSec=20s
KillSignal=SIGQUIT
StandardOutput=file:/var/log/invisily/mfa_suite/mfa_server/mfa_server.log
StandardError=file:/var/log/invisily/mfa_suite/mfa_server/mfa_server_error.log

[Install]
WantedBy=multi-user.target
EOF
        mv /tmp/mfa.service /etc/systemd/system/mfa.service
        systemctl daemon-reload
        systemctl start mfa
        systemctl enable mfa
        invisily_note "Starting MFA server, Please wait as it may take upto 3-4 minute..."
        systemctl restart mfa.service
        sleep 20
        STATUS=$(systemctl is-active mfa.service)
        if [[ ${STATUS} == 'active' ]]; then
            invisily_note "MFA server has been started successfully"
        else
            invisily_warn "MFA server has not started properly, trying to compile it again!"
            sudo -H -u "$FS_USER" bash <<CODESETUP
cd /opt/invisily/mfa_suite/mfa_server/fido-uaf-core
mvn db-migrator:create
mvn db-migrator:migrate
mvn process-classes
mvn clean install
CODESETUP
            if [[ ${STATUS} == 'active' ]]; then
                invisily_note "MFA server has been started successfully"
            else
                invisily_error_no_exit "MFA is not started properly even after multiple attempts, a manual checkup is required!"
            fi
        fi

        [ -f /etc/logrotate.d/invisily-mfa-server ] && rm -rf /etc/logrotate.d/invisily-mfa-server

        cat > /etc/logrotate.d/invisily-mfa-server <<EOF
/var/log/invisily/mfa_suite/mfa_server/mfa_server.log
/var/log/invisily/mfa_suite/mfa_server/mfa_server_error.log
{
    daily
    missingok
    rotate 3
    maxsize 5M
    minsize 5M
    compress
    delaycompress
    copytruncate
    postrotate
        /usr/lib/rsyslog/rsyslog-rotate
    endscript
}
EOF
    # update mfa
    else
        CUR_MFA_VER=$(grep -Eo "([[:digit:]]+\.){3}[[:digit:]]+-SNAPSHOT" "$MFA_VER_FILE" | cut -d"-" -f1)
        if [[ $(is_greater "$UPDATE_MFA_VERSION" "$CUR_MFA_VER") ]]; then
            invisily_note "MFA server update detected. Now updating MFA server"
            MFA_PKG_UPDATE_DATE=$(date '+%Y-%m-%d%H:%M:%S')
            systemctl stop mfa.service

            if [ ! -f "$MFA_DIR"/"$INSTALLER_MFA" ]; then
                invisily_error "$MFA_DIR/$INSTALLER_MFA is not available, please copy setup in $MFA_DIR!"
            fi

            if [ -d /opt/invisily/mfa_suite ]; then
                cd /opt/invisily/mfa_suite/ || exit
                mv mfa_server mfa_server."$MFA_PKG_UPDATE_DATE"
            else
                invisily_error "/opt/invisily/mfa_suite/ directory does not exists, apprently you do not have MFA installed already hence upgrade is not possible!"
            fi

            if [ ! -d /var/log/invisily/mfa_suite/mfa_server ]; then
                mkdir -p /var/log/invisily/mfa_suite/mfa_server
                chown -R "$FS_USER":"$FS_USER" /var/log/invisily/mfa_suite/mfa_server
            else
                chown -R "$FS_USER":"$FS_USER" /var/log/invisily/mfa_suite/mfa_server
            fi

            cd "$MFA_DIR" || exit
            tar xzvf "$MFA_DIR"/"$INSTALLER_MFA"
            [[ ! -f $MFA_DIR/mfa_server.zip ]] && invisily_error "mfa_server.zip is not available in $MFA_DIR, please copy setup and run this script again!"
            [[ ! -f $MFA_DIR/m2.tar.gz ]] && invisily_error "m2.tar.gz is not available in $MFA_DIR, please copy setup and run this script again!"

            unzip -o "$MY_PATH"/mfa_server.zip -d /opt/invisily/mfa_suite/

            if [ -d /opt/invisily/mfa_suite/__MACOSX ]; then
                rm -rf /opt/invisily/mfa_suite/__MACOSX
            fi

            cp -rf /opt/invisily/mfa_suite/mfa_server."$MFA_PKG_UPDATE_DATE"/fido-uaf-core/src/main/resources/database.properties /opt/invisily/mfa_suite/mfa_server/fido-uaf-core/src/main/resources/database.properties

            # MFA_PORT=$(cat < /opt/invisily/mfa_suite/mfa_server."$MFA_PKG_UPDATE_DATE"/fidouaf/pom.xml | grep "httpsPort" | sed 's/<httpsPort>//' | sed 's/<\/httpsPort>//' | sed "s/ //g")
            sed  -i  -e   's/9091/'"$MfaServerPort"'/g'     /opt/invisily/mfa_suite/mfa_server/fidouaf/pom.xml
            # cp -rf /opt/invisily/mfa_suite/mfa_server.$MFA_PKG_UPDATE_DATE/fidouaf/pom.xml /opt/invisily/mfa_suite/mfa_server/fidouaf/pom.xml
            cp -rf /opt/invisily/mfa_suite/mfa_server."$MFA_PKG_UPDATE_DATE"/fidouaf/src/main/webapp/.keystore /opt/invisily/mfa_suite/mfa_server/fidouaf/src/main/webapp/.keystore
            chown -R "$FS_USER":"$FS_USER" /opt/invisily/mfa_suite/

            sudo -H -u "$FS_USER" bash <<CODESETUP
cd /opt/invisily/mfa_suite/mfa_server/fido-uaf-core
mvn db-migrator:migrate
mvn process-classes
mvn clean install
CODESETUP

            if [[ $ONLINE_INSTALL == 1 ]]; then
                sudo -H -u "$FS_USER" bash <<CODESETUP
cd /opt/invisily/mfa_suite/mfa_server/fidouaf
timeout 3m /usr/bin/mvn package tomcat7:run
CODESETUP
            fi

            if [ -f /etc/systemd/system/mfa.service ]; then
                rm -rf /etc/systemd/system/mfa.service
                echo 'File removed!'
                systemctl daemon-reload
            fi

            cat > /tmp/mfa.service <<EOF
[Unit]
Description=MFA fidouaf
After=network.target

[Service]
Type=simple
User=$FS_USER
WorkingDirectory=/opt/invisily/mfa_suite/mfa_server/fidouaf
ExecStart=/usr/bin/mvn package -o tomcat7:run
RemainAfterExit=no
Restart=on-failure
RestartSec=20s
KillSignal=SIGQUIT
StandardOutput=file:/var/log/invisily/mfa_suite/mfa_server/mfa_server.log
StandardError=file:/var/log/invisily/mfa_suite/mfa_server/mfa_server_error.log

[Install]
WantedBy=multi-user.target
EOF
            mv /tmp/mfa.service /etc/systemd/system/mfa.service
            systemctl daemon-reload
            systemctl start mfa
            systemctl enable mfa
            invisily_note "Starting MFA, Please wait as it may take upto 3-4 minute..."
            systemctl restart mfa.service
            sleep 20
            STATUS=$(systemctl is-active mfa.service)
            if [[ ${STATUS} == 'active' ]]; then
                invisily_note "MFA server updated to latest version and started successfully!"
            else
                invisily_warn "MFA is not started properly, trying to compile it again!"
                sudo -H -u "$FS_USER" bash <<CODESETUP
cd /opt/invisily/mfa_suite/mfa_server/fido-uaf-core
mvn db-migrator:migrate
mvn process-classes
mvn clean install
CODESETUP
                invisily_note "Starting MFA server, Please wait as it may take upto 3-4 minute..."
                systemctl restart mfa.service
                sleep 20
                STATUS=$(systemctl is-active mfa.service)
                if [[ ${STATUS} == 'active' ]]; then
                    invisily_note "MFA server updated to latest version and started successfully!"
                else
                    invisily_error_no_exit "MFA server has not started properly even after multiple attempts, a manual checkup is required!"
                fi
            fi
            rm -rf /opt/invisily/mfa_suite/mfa_server."$MFA_PKG_UPDATE_DATE"
        else
            invisily_note "Latest MFA server is already installed!"
        fi
    fi
}

## Onboarding Portal uninstall
portal_uninstall() {
    if [ $# -eq 0 ]; then
        comp="Admin"
    else
        comp=$1
    fi

    if [ ! -f "/etc/systemd/system/pm2-$FS_USER.service" ]; then
        invisily_error_no_exit $"$comp Portal does not seems to be installed in this machine."
    fi
    FS_USER_HOME_DIR=$(getent passwd "$FS_USER" | cut -d: -f6)
    sudo -H -u "$FS_USER" bash <<CODESETUP
[[ "$VERBOSE" == "1" ]] && set -x
$(declare -f invisily_log)
$(declare -f invisily_warn)
$(declare -f invisily_note)
$(declare -f invisily_error)
$(declare -f invisily_result)
$(declare -f invisily_error_no_exit)
cd ~
. ~/.nvm/nvm.sh >/dev/null 2>&1
pm2 uninstall pm2-logrotate && invisily_note $'pm2 logrotate uninstalled' || invisily_warn $'Could not uninstall pm2 logrotate process'
pm2 kill && invisily_note $'pm2 process killed' || invisily_warn $'Could not kill pm2 process'
grep .*nvm.* $FS_USER_HOME_DIR/.bashrc && sed -e 's/.*nvm.*//g' -i $FS_USER_HOME_DIR/.bashrc || invisily_warn $'Cant find .bashrc environment variables for nvm, nvm doesnt seems installed'
if [[ -d .nvm ]]; then
    rm -rf .nvm
else
    invisily_warn $'.nvm directory does not exists'
fi
if [[ -d .pm2 ]]; then
    rm -rf .pm2
else
    invisily_warn $'.pm2 directory does not exists'
fi
if [[ -d .pm2 ]]; then
    rm -rf .pm2
else
    invisily_warn $'.pm2 directory does not exists'
fi
source $FS_USER_HOME_DIR/.bashrc
pm status && echo "yes" || echo "no"
CODESETUP
    if [ -d "/var/local/installers" ]; then
        rm -rf /var/local/installers
    else
        invisily_warn $'/var/local/installers directory does not exists'
    fi
    if [ -d "/var/local/env" ]; then
        rm -rf /var/local/env
    else
        invisily_warn $'/var/local/env directory does not exists'
    fi
    if [ -d "/var/local/ssl" ]; then
        rm -rf /var/local/ssl
    else
        invisily_warn $'/var/local/ssl directory does not exists'
    fi
    if [ -d "/var/local/mysqlssl" ]; then
        rm -rf /var/local/mysqlssl
    else
        invisily_warn $'/var/local/mysqlssl directory does not exists'
    fi
    if [ -f "/var/local/.RABBITMQ_ADMIN_CREDS" ]; then
        rm -rf /var/local/.RABBITMQ_ADMIN_CREDS
    else
        invisily_warn $'/var/local/.RABBITMQ_ADMIN_CREDS file does not exists'
    fi
    if [ -d "/var/www/html" ]; then
        rm -rf /var/www/html
    else
        invisily_warn $'/var/www/html directory does not exists'
    fi
    if [ -d "/var/log/invisily" ]; then
        rm -rf /var/log/invisily
    else
        invisily_warn $'/var/log/invisily directory does not exists'
    fi
    if [ -d "/opt/invisily/watchdog/" ]; then
        rm -rf /opt/invisily/watchdog/
    else
        invisily_warn $'/opt/invisily/watchdog/ directory does not exists'
    fi
    if [[ -f /etc/systemd/system/pm2-$FS_USER.service ]]; then
        rm -rf /etc/systemd/system/pm2-"$FS_USER".service
        systemctl daemon-reload
    fi
    invisily_note "Invisily $comp Portal is successfully removed"
}

##MFA Uninstall
mfa_uninstall() {
    systemctl stop mfa.service
    rm -rf /etc/systemd/system/mfa.service
    [ -d /opt/invisily/mfa_suite ] && rm -rf /opt/invisily/mfa_suite
    [ -d /home/"$FS_USER"/.m2 ] && rm -rf /home/"$FS_USER"/.m2
    systemctl daemon-reload
    apt_dependency_uninstall openjdk-8-jdk openjdk-8-jre maven
}

remove_assets() {
    rm -rf "$DETAILED_LOGS" "$logfile" "$OBP_ZIP" "$OBP_DIR" "/tmp/invisily-backup-logs" "/tmp/sdp-controller-update"
}

################################################################################
# main function                                                                #
################################################################################

cd "$MY_PATH" || exit
arg_verification
if [ "$INSTALL" == "1" ]; then
    install_controller
    onboard_setup
    mfa_setup
    invisily_result "All components installed successfully!"
elif [ "$REMOVE" == "1" ]; then
    remove_controller
    portal_uninstall "Onboard"
    mfa_uninstall
    remove_assets
    invisily_result "All components removed successfully!"
else
    invisily_error "No proper argument is passed, please use --help to find which arguments you can pass to this script"
fi