#!/bin/bash

#set -eo pipefail
#shopt -s nullglob
[[ -f $MY_PATH/.result ]] && rm -rf "$MY_PATH"/.result

################################################################################
# Installation files existance check                                           #
################################################################################

MY_PATH=$(dirname "$0")
MY_PATH=$(cd "$MY_PATH" && pwd)
OFFLINE_GATEWAY_INSTALLER_NAME=gateway-5.8.tar.gz
DEVICE_PROFILER_INSTALLATION_DIR="/opt/invisily/device-profiler"
DEVICE_PROFILER_LOG_DIR="/var/log/invisily/device-profiler"
DEVICE_PROFILER_LOG_FILE="$DEVICE_PROFILER_LOG_DIR/client.log"
DEVICE_PROFILER_SYSTEMD_SERVICE_NAME="invisily_device_profiler.service"
DEVICE_PROFILER_SYSTEMD_SERVICE_FILE="/etc/systemd/system/$DEVICE_PROFILER_SYSTEMD_SERVICE_NAME"
NGINX_SSL_DIR="/etc/nginx/ssl"
DEB_MODULES=gateway-deb-dependencies.tar.gz
KAFKA_MODULES=gateway-kafka-dependencies.tar.gz
PYTHON_MODULES_EXT_DIR=gateway-python-dependencies
APT_ARCHIVE="/var/cache/apt/archives"

################################################################################
# Logging functions                                                            #
################################################################################
curFile=$(basename "$0" | awk -F . '{print $1}')
logfile="/var/log/$curFile-install.log"

[[ -f "$logfile" ]] && { rm -rf "$logfile" >/dev/null 2>&1 || {
    printf '\033[0;31m%s [%s] [%s]: %s\033[0m\n' "$(date +"%Y-%m-%d %H:%M:%S%z")" "Error" "$0" "Please run script as root. Exiting..." && exit 1
    }
}
touch "$logfile" >/dev/null 2>&1 || {
    printf '\033[0;31m%s [%s] [%s]: %s\033[0m\n' "$(date +"%Y-%m-%d %H:%M:%S%z")" "Error" "$0" "Please run script as root. Exiting..." && exit 1
}

invisily_log() {
    local type="$1"; shift
    if [ "$type" == "ERROR" ]; then
        printf '\033[0;31m%s [%s] [%s]: %s\033[0m\n' "$(date +"%Y-%m-%d %H:%M:%S%z")" "$type" "$0" "$*" | tee -a "$logfile"
    elif [ "$type" == "Warn" ]; then
        printf '\033[0;34m%s [%s] [%s]: %s\033[0m\n' "$(date +"%Y-%m-%d %H:%M:%S%z")" "$type" "$0" "$*" | tee -a "$logfile"
    elif [ "$type" == "Result" ]; then
        printf '\033[0;34m%s\033[0m\n' "$*" | tee -a "$logfile"
    else
        printf '%s [%s] [%s]: %s\n' "$(date +"%Y-%m-%d %H:%M:%S%z")" "$type" "$0" "$*" | tee -a "$logfile"
    fi
}
invisily_note() {
    invisily_log Note "$@"
}
invisily_warn() {
    invisily_log Warn "$@" >&2
}
invisily_result() {
    invisily_log Result "$@" >&2
}
invisily_error_no_exit() {
    invisily_log ERROR "$@" >&2
}
invisily_error() {
    invisily_log ERROR "$@" >&2
    invisily_log ERROR "Exiting..."  >&2
    exit 1
}

if [ "$EUID" -ne 0 ]; then
    invisily_error $'Please run as root.'
fi

################################################################################
# Initial necessary verification                                               #
################################################################################
if [ -f /etc/lsb-release ] || [ -d /etc/lsb-release.d ]; then
    ubuntuVersion=$(lsb_release -r | cut -d: -f2 | sed s/'^\t'//)
    if [ "$ubuntuVersion" != "22.04" ] && [ "$ubuntuVersion" != "20.04" ] ; then
        invisily_error "Supported Operating system is Ubuntu 22.04 or 20.04, but it appears you are not running Ubuntu 22.04 or 20.04"
    fi
else
    invisily_error "Supported Operating system is Ubuntu 22.04 or 20.04, but it appears you are not running Ubuntu 22.04 or 20.04"
fi

if [ $( (dpkg-query -W -f='${Status}' ubuntu-desktop 2>/dev/null | grep -c "ok installed") ) -eq 1 ]; then
    invisily_error "Ubuntu 22.04 or 20.04 Server is required, but it appears that you are running Ubuntu Desktop"
fi

if [ $# -eq 0 ]; then
    invisily_error $'No arguments supplied. Please use -h or --help to get help'
fi

################################################################################
# Validation functions                                                         #
################################################################################
__ip_validation() {
    ip=$1
    fail=0
    if expr "$ip" : '[0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*$' >/dev/null; then
        for i in 1 2 3 4; do
            if [[ $i -eq 1 ]]; then
                if [[ $(echo "$ip" | cut -d. -f$i) -gt 255 || $(echo "$ip" | cut -d. -f$i) -le 0 ]]; then
                    fail=1
                    break
                fi
            else
                if [[ $(echo "$ip" | cut -d. -f$i) -gt 255 ]]; then
                    fail=1
                    break
                fi
            fi
        done
    else
        fail=1
    fi
    if [ "$fail" == "1" ]; then
        return 1
    else
        return 0
    fi
}

apt_pkg_check(){
    if [ $# -eq 0 ]; then
        invisily_error $'No arguments supplied. Please use give apt package name in argument'
    elif [ $# -gt 1 ]; then
        invisily_error $'apt_pkg_check function accepts only one agument'
    else
        dependency=$1
        if [ "$(! dpkg-query -W -f='${Status}' "$dependency" 2>/dev/null | grep -c "ok installed")" -eq 0 ]; then
            return 1
        else
            return 0
        fi
    fi
}

installation_mode_verification() {
    if [[ $INSTALL_MODE == "online" ]]; then
        ONLINE_INSTALL=1
    elif [[ $INSTALL_MODE == "offline" ]]; then
        OFFLINE_INSTALL=1
    else
        invisily_error "--installation-type should be either online or offline, however you have entered $INSTALL_MODE"
    fi
}

dns_ip_verification() {
    if ! __ip_validation "$1" ; then
        invisily_error "DNS IP you entered is not a valid dns ip"
    elif [[ "$1" == *"127.0"* ]]; then
        invisily_error "DNS IP cannot be local host IP"
    elif ! timeout 3 nslookup google.com "$1" >/dev/null 2>&1 ; then
        if [[ $disable_dns_check -ne 1 ]]; then
            invisily_error "Your DNS IP is not resolving google.com, please enter a valid dns ip which really resolves domains"
        else
            invisily_warn "DNS Server ip you have provided, is not resolving google.com, however this error is ignoring becuase you have passed --disable-dns-check"
        fi
    else
        invisily_note "DNS verification is passed for DNS IP $1 you have provided."
    fi
}

_ip_list_validation(){
    IFS="," read -ra ip_list <<< "$1"
    for ip in "${ip_list[@]}"; do
        __ip_validation $ip
    done
}
################################################################################
# Help                                                                         #
################################################################################
Help() {
    # Display Help
    echo -e "Usage: $0 [-h] [--help] [--install] [--remove] [--installation-type=xxx] [--multi-home]"
    echo -e ""
    echo -e "Help: $0 -h"
    echo -e "      OR"
    echo -e "      $0 --help"
    echo -e
    echo -e "Options:"
    echo -e "-h | --help                                            Print help."
    echo -e "--install                                              Install Invisily Gateway"
    echo -e "--remove                                               Remove Invisily Gateway"
    echo -e "--installation-type=xxx                                should be either online or offline"
    echo -e "--multi-home                                           Multi-home installation to run on Multiple Interfaces"
    echo -e "--dns-ip                                               Provide specific ip to set for dns (not recommended)"
    echo -e "--disable-dns-check                                    Stop installer to do dns query to confirm availability of dns service on provided dns ip"
    echo -e "--kafka-servers-ips=xxx.xxx.xxx.xxx,xxx.xxx.xxx.xxx    Input Kafka server IPs, supports multiple with comma separated, Ex. 192.168.0.0,192.168.0.1. Skip this if you don't want to configure DP"
    echo -e "--kafka-port=xxx                                       Input Kafka server port, Ex. 9092. Skip this if you don't want to configure DP"
    echo -e "--kafka-topic=xxxxx                                    Input Kafka topic, Ex. invisily-profiler-topic. Skip this if you don't want to configure DP"
    echo -e ""
}
################################################################################
################################################################################
# Main program                                                                 #
################################################################################
################################################################################
for i in "$@"
do
case $i in
    --install)
    INSTALL=1
    shift # past argument=value
    ;;
    --remove)
    REMOVE=1
    shift # past argument=value
    ;;
    --installation-type=*)
    INSTALL_MODE="${i#*=}"
    installation_mode_verification
    shift # past argument=value
    ;;
    --multi-home)
    MULTI_HOME=1
    shift # past argument=value
    ;;
    --disable-dns-check)
    disable_dns_check=1
    shift # past argument=value
    ;;
    --dns-ip=*)
    dnsip="${i#*=}"
    shift # past argument=value
    ;;
    --kafka-servers-ips*)
    KAFKA_SERVERS_IPS="${i#*=}"
    _ip_list_validation $KAFKA_SERVERS_IPS
    shift # past argument=value
    ;;
    --kafka-port*)
    KAFKA_PORT="${i#*=}"
    shift # past argument=value
    ;;
    --kafka-topic*)
    KAFKA_TOPIC="${i#*=}"
    shift # past argument=value
    ;;
    -h|--help)
    Help
    exit 1
    ;;
    *)
    invisily_error "Unknown option. Exiting..."
    ;;
esac
done
arg_verification(){
    if [ "$INSTALL" == "1" ] && [ "$REMOVE" == "1" ]; then
        invisily_error $'You cannot use --install and --remove at the same time'
    fi
    if [ "$INSTALL" == "1" ] && [ -z "$INSTALL_MODE" ]; then
        invisily_error $'--installation-type is not passed, should be either online or offline'
    fi
    if [ "$REMOVE" == "1" ] && [ -n "$INSTALL_MODE" ]; then
        invisily_error $'--installation-type is not required when removing or uninstalling'
    fi
    if [[ -n $dnsip ]]; then
        dns_ip_verification "$dnsip"
    fi
    if [ -n "$KAFKA_SERVERS_IPS" ] && [ -z $KAFKA_PORT ]; then
        invisily_error $'Kafka server port is not specified when running the script'
    fi
    if [ -n "$KAFKA_SERVERS_IPS" ] && [ -z $KAFKA_TOPIC ]; then
        invisily_error $'Kafka topic for device profiler is not specified when running the script'
    fi
    if [ -n "$KAFKA_TOPIC" ] && [ -z $KAFKA_SERVERS_IPS ]; then
        invisily_error $'Kafka server IPs are not specified when running the script'
    fi
    if [ -n "$KAFKA_PORT" ] && [ -z $KAFKA_SERVERS_IPS ]; then
        invisily_error $'Kafka server IPs are not specified when running the script'
    fi
}
remote_server_tcp_port_connection_check() {
    REMOTE_IP_TO_CHECK=$1
    REMOTE_PORT_TO_CHECK=$2
    if timeout 1 bash -c 'echo &> /dev/null  < /dev/tcp/'"$REMOTE_IP_TO_CHECK"'/'"$REMOTE_PORT_TO_CHECK"; then
        return 0
    else
        return 1
    fi
}
device_profiler_install(){
    [[ ! -d $DEVICE_PROFILER_INSTALLATION_DIR ]] && mkdir -p $DEVICE_PROFILER_INSTALLATION_DIR
    [[ ! -d  $DEVICE_PROFILER_LOG_DIR ]]  && mkdir -p $DEVICE_PROFILER_LOG_DIR
    [[ ! -f $DEVICE_PROFILER_LOG_FILE ]] && touch $DEVICE_PROFILER_LOG_FILE
    #Creating configs.json
    cd $DEVICE_PROFILER_INSTALLATION_DIR || exit
    [[ -f configs.json ]] && rm -rf configs.json
    IFS="," read -ra kafka_ip_list <<< "$KAFKA_SERVERS_IPS"

    for ip in "${kafka_ip_list[@]}"; do
        if ! remote_server_tcp_port_connection_check "$ip" "$KAFKA_PORT"; then
            invisily_error "Invalid Kafka server IP, not able to connect to kafka server."
        fi
    done

    kafka_ip_list=( "${kafka_ip_list[@]/%/:$KAFKA_PORT}" )
    JSON_KAFKA_SERVERS_IPS="["
    for ip in "${kafka_ip_list[@]}"; do
        JSON_KAFKA_SERVERS_IPS+="\"$ip\","
    done
    JSON_KAFKA_SERVERS_IPS="${JSON_KAFKA_SERVERS_IPS%,}"
    JSON_KAFKA_SERVERS_IPS+="]"
    if [[ $ONLINE_INSTALL == 1 ]]; then
        if apt-get -y update && apt-get -y install python3 && wget https://bootstrap.pypa.io/get-pip.py && python3 get-pip.py --root-user-action=ignore && apt-get -y install python3-setuptools && python3 -m pip install -U pip --root-user-action=ignore && python3 -m pip install -U setuptools --root-user-action=ignore && rm -rf get-pip.py; then
            invisily_note "Successfully installed pip for python3"
        else
            invisily_error "Failed to install pip for python3"
        fi
    fi
    if [[ $OFFLINE_INSTALL == 1 ]]; then
        cd /tmp/invisily-gateway-offline-dependencies || exit
        mkdir -p "$APT_ARCHIVE"
        apt-get clean
        tar xzvf /tmp/invisily-gateway-offline-dependencies/$DEB_MODULES -C "$APT_ARCHIVE"
        cd "$APT_ARCHIVE" || exit
        export DEBIAN_FRONTEND=noninteractive
        if ! dpkg -i ./*.deb; then
            if ! dpkg -i ./*.deb; then
                invisily_error "Installation of dependencies for Gateway is failed!"
            fi
        fi
        cd "$MY_PATH" || exit
    fi
    cat > $DEVICE_PROFILER_INSTALLATION_DIR/configs.json <<END
{
    "bootstrap_servers": $JSON_KAFKA_SERVERS_IPS,
    "topic": "$KAFKA_TOPIC",
    "interface": "",
    "log_file": "$DEVICE_PROFILER_LOG_FILE",
    "agent_bindings": "/opt/invisily/device-profiler/clients.data",
    "identifier": "src"
}
END
    if [[ $ONLINE_INSTALL == 1 ]]; then
    pip install -r requirements.txt --root-user-action=ignore
    elif [[ $OFFLINE_INSTALL == 1 ]]; then
    [ -f /tmp/invisily-gateway-offline-dependencies/$KAFKA_MODULES ] || invisily_error "Cannot find Python offline installer dependencies from extracted files!"
        mkdir -p /tmp/invisily-gateway-offline-dependencies/gateway-python-dependencies
        tar xzvf /tmp/invisily-gateway-offline-dependencies/$KAFKA_MODULES -C /tmp/invisily-gateway-offline-dependencies/$PYTHON_MODULES_EXT_DIR
        if ! sudo su -c 'cd /tmp/invisily-gateway-offline-dependencies/gateway-python-dependencies \
        && cd /tmp/invisily-gateway-offline-dependencies/gateway-python-dependencies \
        && pip3 install --no-index --ignore-installed --find-links=./ *.whl \
        && pip3 list'; then
            invisily_error "Installation of python pip modules for Controller is failed"
        fi
    fi
    [[ -f requirements.txt ]] && rm -rf requirements.txt
    systemctl daemon-reload

    invisily_result "Invisily device-profiler has been installed successfully ..."
}
install_gateway() {
    if [[ $ONLINE_INSTALL -eq 1 ]]; then
        if [[ -f /opt/invisily/linuxgateway/bin/gateway.sh ]]; then
            /opt/invisily/linuxgateway/bin/gateway.sh stop
        else
            invisily_note "Ignoring to stop gateway and sniffer since it seems to be a fresh installation"
        fi
        if ping -c3 -W1  github.com >/dev/null 2>&1 || ping -c3 -W1  google.com >/dev/null 2>&1 || ping -c3 -W1 archive.ubuntu.com >/dev/null 2>&1 || timeout 1 bash -c "echo &> /dev/null  < /dev/tcp/archive.ubuntu.com/80"; then
            if ! apt-get -y update; then
                invisily_error "apt-get failed due to unknown reason."
            fi
            apt_pkg_check systemd-timesyncd || apt-get install systemd-timesyncd -y
        else
            invisily_error "Your machine is not connected to Internet. Exiting..."
        fi
    elif [[ $OFFLINE_INSTALL -eq 1 ]]; then
        if [[ "$ubuntuVersion" == "20.04" ]]; then
            invisily_error "Offline installation is not supported in Ubuntu 20.04..."
        fi
        if [[ -f /root/.DO_NOT_DELETE_INVISILY ]]; then
            invisily_note "You are using Invisily Provided OS image, proceeding for Offline Install..."
        elif [[ -f $MY_PATH/$OFFLINE_GATEWAY_INSTALLER_NAME ]]; then
            [ -d /tmp/invisily-gateway-offline-dependencies ] && rm -rf /tmp/invisily-gateway-offline-dependencies
            mkdir -p /tmp/invisily-gateway-offline-dependencies
            if ! tar xzvf "$MY_PATH"/$OFFLINE_GATEWAY_INSTALLER_NAME -C /tmp/invisily-gateway-offline-dependencies >/dev/null 2>&1; then
                invisily_error "Installing Offline Dependencies failed."
            fi
        else
            invisily_error "You does not seem to be using Invisily specific Ubuntu 22.04 image nor Invisily Offline dependencies are available in the current directory."
        fi
    fi

    invisily_note "invisily-Gateway Server installation ..."
    invisily_note "invisily-Gateway Server checking previous installation ..."
    apt_pkg_check invisily-gateway && apt-get purge invisily-gateway -y
    [ -f /opt/invisily/sdp-gateway/watchdog/conf/watchdog.cnf ] && mv /opt/invisily/sdp-gateway/watchdog/conf/watchdog.cnf ~/watchdog.cnf

    if [ -d "/opt/invisily/linuxgateway" ]; then
        rm -rf  /opt/invisily/linuxgateway
    fi

    if [[ $OFFLINE_INSTALL == 1 ]]; then
        cd /tmp/invisily-gateway-offline-dependencies || exit
        mkdir -p "$APT_ARCHIVE"
        apt-get clean
        tar xzvf /tmp/invisily-gateway-offline-dependencies/$DEB_MODULES -C "$APT_ARCHIVE"
        cd "$APT_ARCHIVE" || exit
        export DEBIAN_FRONTEND=noninteractive
        if ! dpkg -i ./*.deb; then
            if ! dpkg -i ./*.deb; then
                invisily_error "Installation of dependencies for Gateway is failed!"
            fi
        fi
        cd "$MY_PATH" || exit
    fi

    invisily_note "invisily-gateway deb installation ..."
    chmod a+x ./*
    if ! apt-get install  ./*.deb -y; then
        invisily_error "Error invisily-gateway deb installation failed.."
    fi
    [ -f ~/watchdog.cnf ] && rm -rf /opt/invisily/sdp-gateway/watchdog/conf/watchdog.cnf && mv ~/watchdog.cnf /opt/invisily/sdp-gateway/watchdog/conf/watchdog.cnf

    if [[ -z $device_name ]]; then
        device_found=0
        while [[ $device_found -ne 1 ]]
        do
            array_test=()
            argstr=""
            declare -i count=0
            declare -i count2=0
            echo "Available Network Devices for Gateway Deployment are :"
            for iface in $(ip a | awk -F ': ' '/^[0-9]+: / && !/: lo: / {print $2}')
            do
                iface_ip=$(ip a show "$iface" | awk '/inet / {split($2, a, "/"); print a[1]}')
                echo -e "\t $count. $iface [$iface_ip]"
                array_test+=("$iface")
                count=$((count+1));
            done

            echo "Enter the number of the network device for Gateway deployment : "
            read -r device_index;
            for i in "${array_test[@]}";
            do
                if [ "$count2" == "$device_index" ];then
                    device_name="$i";
                    device_found=1
                fi
                count2=$((count2+1));
            done
            if [[ $device_found -eq 0 ]]; then
                invisily_warn "You did not enter correct number of available network device, please re-enter correct option below!"
            fi
        done
    fi
    device_name_ip=$(ip a show "$device_name" | awk '/inet / {split($2, a, "/"); print a[1]}')

    if [[ $MULTI_HOME -eq 1 ]]; then
        if [[ -z $device_name2 ]]; then
            device_found=0
            while [[ $device_found -ne 1 ]]
            do
                array_test=()
                argstr=""
                declare -i count=0
                declare -i count2=0
                echo "Available Network Devices Gateway Deployment are :"
                for iface in $(ip a | awk -F ': ' '/^[0-9]+: / && !/: lo: / {print $2}')
                do
                    if [[ "$iface" != "$device_name" ]]; then
                        iface_ip=$(ip a show "$iface" | awk '/inet / {split($2, a, "/"); print a[1]}')
                        echo -e "\t $count. $iface [$iface_ip]"
                        array_test+=("$iface")
                        count=$((count+1));
                    fi
                done

                echo "Enter the number of the network device for Multi-Home Gateway deployment : "
                read -r device_index;
                for i in "${array_test[@]}";
                do
                    if [ "$count2" == "$device_index" ];then
                        device_name2="$i";
                        device_found=1
                    fi
                    count2=$((count2+1));
                done
                if [[ $device_found -eq 0 ]]; then
                    echo "You did not enter correct number of available network device, please re-enter correct option below!"
                fi
            done
        fi
        device_name2_ip=$(ip a show "$device_name2" | awk '/inet / {split($2, a, "/"); print a[1]}')
        cp -r "$MY_PATH"/sniffer_config.cfg "$MY_PATH"/sniffer_config-"$device_name2".cfg
        sed_args="s/networkid_value_placeholder/$device_name2/g";
        sed -i "$sed_args" ./sniffer_config-"$device_name2".cfg
        sed -i "s/component_IPAddress=.*/component_IPAddress=\"$device_name2_ip\"/g"  sniffer_config-"$device_name2".cfg
        sed -i "s/serviceMessagesFilePath=.*/serviceMessagesFilePath=\"..\/bin\/msg\/\"/g"  sniffer_config-"$device_name2".cfg
        cp -r /opt/invisily/linuxgateway/bin /opt/invisily/linuxgateway/bin-"$device_name2"
        mv /opt/invisily/linuxgateway/bin-"$device_name2"/sniffer.out /opt/invisily/linuxgateway/bin-"$device_name2"/sniffer-"$device_name2".out
        mv -f sniffer_config-"$device_name2".cfg /opt/invisily/linuxgateway/bin-"$device_name2"/sniffer_config.cfg
        [[ -f /etc/systemd/system/invisily_sniffer-$device_name2.service ]] && rm -rf /etc/systemd/system/invisily_sniffer-"$device_name2".service && systemctl daemon-reload
        cp -r /etc/systemd/system/invisily_sniffer.service /etc/systemd/system/invisily_sniffer-"$device_name2".service
        sed -i "s|WorkingDirectory=.*|WorkingDirectory=/opt/invisily/linuxgateway/bin-$device_name2|g"  /etc/systemd/system/invisily_sniffer-"$device_name2".service
        sed -i "s|ExecStart=.*|ExecStart=/opt/invisily/linuxgateway/bin-$device_name2/sniffersd.sh start|g"  /etc/systemd/system/invisily_sniffer-"$device_name2".service
        sed -i "s|SyslogIdentifier=.*|SyslogIdentifier=invisily_sniffer_multi_home_$device_name2|g"  /etc/systemd/system/invisily_sniffer-"$device_name2".service
        sed -i "s|Description=.*|Description= Sniffer Service Multi Home $device_name2|g"  /etc/systemd/system/invisily_sniffer-"$device_name2".service
        sed -i "s|PIDFile=.*|PIDFile=/var/run/sniffersd_$device_name2.pid|g"  /etc/systemd/system/invisily_sniffer-"$device_name2".service
        cat > /etc/systemd/system/multi-home.service <<END
[Unit]
Description= Gateway Sniffer MultiHome
Documentation= http://invisily.com/
Wants=network.target
After=network.target
After=multi-user.target
After=invisily_sniffer.service
After=invisily_sniffer-$device_name2.service
After=invisily_gateway.service


[Service]
Type=oneshot
User=root
Group=root
WorkingDirectory=/opt/invisily/linuxgateway/bin
ExecStart=/opt/invisily/linuxgateway/bin/gateway.sh start
RemainAfterExit=true
ExecStop=/opt/invisily/linuxgateway/bin/gateway.sh stop
StandardOutput=journal

[Install]
WantedBy=multi-user.target
END
        sed -i '/After=multi-user.target/a After=invisily_sniffer.service' /etc/systemd/system/invisily_sniffer-"$device_name2".service
        sed -i '/After=invisily_sniffer.service/a After=invisily_sniffer-'"$device_name2"'.service' /etc/systemd/system/invisily_gateway.service
        systemctl daemon-reload
        systemctl enable invisily_sniffer-"$device_name2".service
        systemctl enable multi-home
        sed -i "s|sniffer.out|sniffer-$device_name2.out|g" /opt/invisily/linuxgateway/bin-"$device_name2"/sniffersd.sh
        [[ -f /etc/apparmor.d/opt.invisily.linuxgateway.bin.sniffer.$device_name2 ]] && apparmor_parser -R /etc/apparmor.d/opt.invisily.linuxgateway.bin.sniffer."$device_name2" && rm -rf /etc/apparmor.d/opt.invisily.linuxgateway.bin.sniffer."$device_name2"
        cp -r /etc/apparmor.d/opt.invisily.linuxgateway.bin.sniffer /etc/apparmor.d/opt.invisily.linuxgateway.bin.sniffer."$device_name2"
        sed -i "s|sniffer.out|sniffer-$device_name2.out|g" /etc/apparmor.d/opt.invisily.linuxgateway.bin.sniffer."$device_name2"
        sed -i "s|/opt/invisily/linuxgateway/bin|/opt/invisily/linuxgateway/bin-$device_name2|g" /etc/apparmor.d/opt.invisily.linuxgateway.bin.sniffer."$device_name2"
        sed -i "s|/opt/invisily/linuxgateway/bin-$device_name2/msg|/opt/invisily/linuxgateway/bin/msg|g" /etc/apparmor.d/opt.invisily.linuxgateway.bin.sniffer."$device_name2"
        apparmor_parser -r /etc/apparmor.d/opt.invisily.linuxgateway.bin.sniffer."$device_name2"
        service apparmor reload
        cat > /opt/invisily/linuxgateway/bin/gateway.sh <<END
#!/bin/bash
### BEGIN INIT INFO
# Short-Description:        Start and stop sniffer & gateway services.
# Description:              -
# Date-Creation:            -
# Date-Last-Modification:   -
# Author:                   -
### END INIT INFO

# Variables
PGREP=/usr/bin/pgrep
ZERO=0

# Start Sniffer & gateway services.
start() {

    echo "Stopping [ GATEWAY ] & [ SNIFFER    ] ..."
    sudo systemctl stop invisily_gateway.service
    sudo systemctl stop invisily_sniffer.service
    sudo systemctl stop invisily_sniffer-$device_name2.service

    sleep 2
    echo "Starting [ SNIFFER    ] ..."
    sudo systemctl start invisily_sniffer.service
    sleep 2
    echo "Starting [ SNIFFER_$device_name2    ] ..."
    sudo systemctl start invisily_sniffer-$device_name2.service
    sleep 2
    sudo iptables -A INPUT -i $device_name2 -j DROP
    sleep 2
    echo "Starting [ GATEWAY ] ..."
    sudo systemctl start invisily_gateway.service

    sleep 2
    echo "Starting [ Gateway Update Service ] ..."
    sudo systemctl start invisily_auto_update.service

    echo ""
    echo " Please check the status of components by sudo ./gateway.sh status "

}


# Stop Sniffer
stop() {

    sudo systemctl stop invisily_gateway.service
    echo "[ GATEWAY ] Stopped ..."

    sudo systemctl stop invisily_sniffer.service
    echo "[ SNIFFER    ] Stopped ..."
    sudo systemctl stop invisily_sniffer-$device_name2.service
    echo "[ SNIFFER_$device_name2    ] Stopped ..."
    sudo iptables -F

}

# Verify status of Sniffer
status() {

    STATUS=\`systemctl is-active invisily_gateway.service\`
    if [[ \${STATUS} == 'active' ]]; then
        echo "[ GATEWAY ] Service is active & running ..."
    else
        echo "[ GATEWAY ] Service is stopped "
    fi

    STATUS=\`systemctl is-active invisily_sniffer.service\`
    if [[ \${STATUS} == 'active' ]]; then
        echo "[ SNIFFER    ] Service is active & running ..."
    else
        echo "[ SNIFFER    ] Service is stopped "
    fi

    STATUS=\`systemctl is-active invisily_sniffer-$device_name2.service\`
    if [[ \${STATUS} == 'active' ]]; then
        echo "[ SNIFFER2_$device_name2    ] Service is active & running ..."
    else
        echo "[ SNIFFER2_$device_name2    ] Service is stopped "
    fi

    STATUS=$(systemctl is-active invisily_auto_update.service)
    if [[ ${STATUS} == 'active' ]]; then
        echo "[ Gateway Update Service ] Service is active & running ..."
    else
        echo "[ Gateway Update Service ] Service is stopped "
    fi
}

# Main logic
case "\$1" in
    start)
        start
        ;;
    stop)
        stop
        ;;
    status)
        status
        ;;
    restart|reload)
        stop
        start
        ;;
  *)
    echo \$"Usage: \$0 {start|stop|status|restart|reload}"
    exit 1
esac
exit 0
END
    cp -r /opt/invisily/linuxgateway/bin/gateway.sh /opt/invisily/linuxgateway/bin-"$device_name2"/gateway.sh
    sed -i "s|#enable_multihome=0|enable_multihome=1|g" /opt/invisily/linuxgateway/watchdog/conf/watchdog.cnf
    sed -i "s|#multi_sniffer_iface_name=<value>|multi_sniffer_iface_name=$device_name2|g" /opt/invisily/linuxgateway/watchdog/conf/watchdog.cnf
    fi
    #Service side network configuration
    if [[ -z $svc_device_name ]]; then
        device_found=0
        while [[ $device_found -ne 1 ]]
        do
            array_test=()

            argstr=""
            declare -i count=0
            declare -i count2=0
            echo "Available Network Devices to use for SERVICE side Network in Gateway Deployment are :"
            for iface in $(ip a | awk -F ': ' '/^[0-9]+: / && !/: lo: / {print $2}')
            do
                iface_ip=$(ip a show "$iface" | awk '/inet / {split($2, a, "/"); print a[1]}')
                echo -e "\t $count. $iface [$iface_ip]"
                array_test+=("$iface")
                count=$((count+1));
            done

            echo "Enter the number of the network device to use for SERVICE side Network in Gateway deployment : "
            read -r device_index;
            for i in "${array_test[@]}";
            do
                if [ "$count2" == "$device_index" ];then
                    svc_device_name="$i";
                    device_found=1
                fi
                count2=$((count2+1));
            done
            if [[ $device_found -eq 0 ]]; then
                invisily_warn "You did not enter correct number of available network device, please re-enter correct option below!"
            fi
        done
    fi

    svc_device_name_ip=$(ip a show "$svc_device_name" | awk '/inet / {split($2, a, "/"); print a[1]}')

    sed_args="s/networkid_value_placeholder/$device_name/g";
    sed -i "$sed_args" ./sniffer_config.cfg
    sed -i "s/component_IPAddress=.*/component_IPAddress=\"$device_name_ip\"/g"  ./sniffer_config.cfg

    sed_args="s/networkid_value_placeholder/$svc_device_name/g";
    sed -i "$sed_args" ./gateway_config.cfg

    if [[ $MULTI_HOME -eq 1 ]]; then
        sed -i "s/gateIPAddress=.*/gateIPAddress=\"0.0.0.0\";/g"  ./gateway_config.cfg
    else
        sed -i "s/gateIPAddress=.*/gateIPAddress=\"$device_name_ip\";/g"  ./gateway_config.cfg
    fi

    if [[ -z $dnsip ]]; then
        dnsip=$(cat < /etc/resolv.conf | grep nameserver | awk -F ' ' '{print $2}')
        if [[ -z $dnsip || "$dnsip" == *"127.0"* ]]; then
            dnsip=$(cat /etc/netplan/*.yaml | grep -A1 nameservers | grep addresses | awk -F ' ' '{print $2}' | cut -d "[" -f2 | cut -d "]" -f1 | awk -F ',' '{print $1}')
            if [[ -z $dnsip || "$dnsip" == *"127.0"* ]]; then
                dnsip=$(systemd-resolve --status | grep -A9 eth0 | grep 'DNS Servers' | awk -F ':' '{print $2}' | cut -d ' ' -f 2)
                if [[ -z $dnsip || "$dnsip" == *"127.0"* ]]; then
                    invisily_warn "No Valid dns ip is found from your system configuration, please enter valid dns ip"
                    echo "Enter the DNS Server IP (Only one IP e.g. 8.8.8.8) : "
                    read -r dnsip
                fi
            fi
        fi

        while true;
        do
            if ! __ip_validation "$dnsip" ; then
                invisily_warn "DNS IP you entered is not a valid dns ip"
                echo "Enter the DNS Server IP (Only one IP e.g. 8.8.8.8) : "
                read -r dnsip
            elif [[ "$dnsip" == *"127.0"* ]]; then
                invisily_warn "DNS IP cannot be local host IP"
                echo "Enter the DNS Server IP (Only one IP e.g. 8.8.8.8) : "
                read -r dnsip
            elif ! timeout 3 nslookup google.com "$dnsip" >/dev/null 2>&1 ; then
                if [[ $disable_dns_check -ne 1 ]]; then
                    invisily_warn "Your DNS IP is not resolving google.com, please enter a valid dns ip which really resolves domains"
                    echo "Enter the DNS Server IP (Only one IP e.g. 8.8.8.8) : "
                    read -r dnsip
                else
                    invisily_warn "DNS Server ip you have provided, is not resolving google.com, however this error is ignoring becuase you have passed --disable-dns-check"
                    break
                fi
            else
                break
            fi
        done
    fi

    invisily_note "DNS IP $dnsip will be used in configuration"

    if [[ -z $dns_port ]]; then
        dns_port=53
        invisily_note "DNS Port $dns_port will be used in configuration"
    fi
    if [[ -z $dns_protocol ]]; then
        dns_protocol="udp"
        invisily_note "DNS Protocol $dns_protocol will be used in configuration"
    fi

    sed_args="dns_ip=\"$dnsip\"";

    sed -i -e '$a\' -e "$sed_args" ./gateway_config.cfg

    sed_args="dns_port=\"$dns_port\"";

    sed -i -e '$a\' -e "$sed_args" ./gateway_config.cfg

    sed_args="dns_protocol=\"$dns_protocol\"";

    sed -i -e '$a\' -e "$sed_args" ./gateway_config.cfg

    echo -e "\nlog_level=\"info\"" >> ./gateway_config.cfg


    invisily_note "invisily-Gateway Server post install : finishing step one ..."

    if [ -d "/opt/invisily/linuxgateway/bin/certs" ]; then
        rm -rf  /opt/invisily/linuxgateway/bin/certs
    fi

    if [ -f "/opt/invisily/linuxgateway/bin/rlup.txt" ]; then
        rm -rf  /opt/invisily/linuxgateway/bin/rlup.txt
    fi

    if [ -d "./certs" ]; then
        mv -f ./certs /opt/invisily/linuxgateway/bin/
    else
        invisily_error "invisily-Gateway Servers post install : finishing step one failed ..."
    fi

    invisily_note "invisily-Gateway Server post install : finishing step two ..."

    if [ -f "/opt/invisily/linuxgateway/bin/gateway_config.cfg" ]; then
        rm -rf /opt/invisily/linuxgateway/bin/gateway_config.cfg
    fi

    if [ -f "./gateway_config.cfg" ]; then
        mv -f ./gateway_config.cfg /opt/invisily/linuxgateway/bin/
    else
        invisily_error "invisily-Gateway Server post install : finishing step two failed ..."
    fi

    invisily_note "invisily-Gateway Server post install : finishing step three ..."

    if [ -f "/opt/invisily/linuxgateway/bin/sniffer_config.cfg" ]; then
        rm -rf /opt/invisily/linuxgateway/bin/sniffer_config.cfg
    fi

    if [ -f "./sniffer_config.cfg" ]; then
        mv -f ./sniffer_config.cfg /opt/invisily/linuxgateway/bin/
    else
        invisily_error "invisily-Gateway Server post install : finishing step three failed ..."
    fi

    invisily_note "invisily-Gateway Server post install : finishing step four ..."

    if [ -f "./rlup.enc" ]; then
        mv -f ./rlup.enc /opt/invisily/linuxgateway/bin/
    else
        invisily_error "invisily-Gateway Server post install : finishing step four failed ..."
    fi

    invisily_note "invisily-Gateway Server post install : finishing step five ..."

    if [ -f "/opt/invisily/linuxgateway/bin/controllers.data" ]; then
        rm -rf /opt/invisily/linuxgateway/bin/controllers.data
    fi

    if [ -f "./controllers.data" ]; then
        mv -f ./controllers.data /opt/invisily/linuxgateway/bin/
    else
        invisily_error "invisily-Gateway Server post install : finishing step five failed ..."
    fi

    PORTAL_NTP_IP=$(cat < /opt/invisily/linuxgateway/bin/gateway_config.cfg | grep NTPIP | awk -F = '{print $2}' | cut -d ';' -f1 | tr -d '"')
    if [[ -z $PORTAL_NTP_IP ]]; then
        invisily_error "Unable to find NTPIP from /opt/invisily/linuxgateway/bin/gateway_config.cfg, NTP setup is failed!"
    fi
    cat > /etc/systemd/timesyncd.conf <<END
[Time]
NTP=$PORTAL_NTP_IP
#FallbackNTP=ntp.ubuntu.com
#RootDistanceMaxSec=5
PollIntervalMinSec=32
PollIntervalMaxSec=2048
END
    systemctl restart systemd-timesyncd
    invisily_note "Configuring NTP, Please wait ..."
    sleep 10
    if ! systemctl status systemd-timesyncd | grep Status | grep "Synchronized.*$PORTAL_NTP_IP\|synchronization.*$PORTAL_NTP_IP"; then
        invisily_warn "NTP Configuration Failed! Your NTP Server $PORTAL_NTP_IP is not listing to UDP port 123, Please make sure your NTP Server is up"
    else
        invisily_result "NTP Server $PORTAL_NTP_IP has been configured!"
    fi

    source /opt/invisily/linuxgateway/bin/gateway_config.cfg

    if [[ "$proxy_enabled" == "1" ]] && [[ $ONLINE_INSTALL -eq 1 ]]; then
        apt-get install nginx -y
        [[ ! -d  $NGINX_SSL_DIR ]] && mkdir -p $NGINX_SSL_DIR
        if openssl req -new -newkey rsa:2048 -days 3650 -nodes -x509 -subj "/C=NA/ST=NA/L=NA/O=NA/CN=invisily.com" -keyout $NGINX_SSL_DIR/nginx.key -out $NGINX_SSL_DIR/nginx.crt; then
            invisily_note "Successfully created ssl certificates for nginx"
        else
            invisily_warn "Failed to create ssl certificate"
        fi
    fi

    if [[ "$proxy_enabled" == "0" ]] && [[ $OFFLINE_INSTALL -eq 1 ]]; then
        systemctl stop nginx.service && systemctl disable nginx.service || echo

    fi

    if [[ "$proxy_enabled" == "1" ]]; then
    chmod a+x /opt/invisily/linuxgateway/bin/proxy_service.sh
    fi

    invisily_result "Invisily-gateway has been installed successfully ..."
}

device_profiler_remove(){
    systemctl stop $DEVICE_PROFILER_SYSTEMD_SERVICE_NAME  || invisily_warn $''$DEVICE_PROFILER_SYSTEMD_SERVICE_NAME ' service cannot find'
    rm -rf $DEVICE_PROFILER_SYSTEMD_SERVICE_FILE || invisily_warn $''$DEVICE_PROFILER_SYSTEMD_SERVICE_FILE ' could not be deleted!'
    rm -rf $DEVICE_PROFILER_INSTALLATION_DIR || invisily_warn $'''$DEVICE_PROFILER_INSTALLATION_DIR' ' could not be deleted!'
    invisily_warn $'Invisily Device Profiler has been successfully Uninstalled!'
    systemctl daemon-reload
}

remove_gateway(){
    if /opt/invisily/linuxgateway/bin/gateway.sh stop; then
        invisily_note "Invisily Gateway Service has been stopped"
    else
        invisily_warn "Invisily Gateway services got error while stopping"
    fi
    if apt_pkg_check invisily-gateway; then
        apt-get purge invisily-gateway -y
        invisily_note "Invisily Gateway package has been uninstalled successfully"
    else
        invisily_warn "Invisily Gateway package uninstallation got error"
    fi
    [[ -d "/opt/invisily/linuxgateway" ]] && rm -rf  /opt/invisily/linuxgateway
    rm -rf /etc/systemd/system/invisily_sniffer*
    systemctl daemon-reload
    apparmor_parser -R /etc/apparmor.d/opt.invisily.linuxgateway.*
    service apparmor reload
    rm -rf /etc/apparmor.d/opt.invisily.linuxgateway.*
    service apparmor reload
    invisily_note "invisily-gateway removed successfully"
}

cd "$MY_PATH" || exit
arg_verification
if [ "$INSTALL" == "1" ]; then
    install_gateway
    if [ -n "$KAFKA_SERVERS_IPS" ] &&  [ -n "$KAFKA_PORT" ] && [ -n "$KAFKA_TOPIC" ]; then
        device_profiler_install
    fi
elif [ "$REMOVE" == "1" ]; then
    remove_gateway
    device_profiler_remove
else
    invisily_error "No proper argument is passed, please use --help to find which arguments you can pass to this script"
fi