#!/bin/bash

set -eo pipefail
shopt -s nullglob
[[ -f "$MY_PATH"/.result ]] && rm -rf "$MY_PATH"/.result
MY_PATH=$(dirname "$0")
MY_PATH=$(cd "$MY_PATH" && pwd)
FS_USER=$SUDO_USER
DETAILED_LOGS="$MY_PATH/logs.txt"
RDI_PORTAL_ZIP_NAME=RdiBridge.zip
RDI_PORTAL_SETTING_FILE_NAME="settings.cfg"
PUBLIC_IP=$(curl -s ifconfig.me)

# output redirection
exec > >(tee -i "$DETAILED_LOGS")
exec 2>&1

################################################################################
# Logging functions                                                            #
################################################################################
invisily_log() {
    local type="$1"; shift
    if [ "$type" == "ERROR" ]; then
        printf '\033[0;31m%s [%s] [%s]: %s\033[0m\n' "$(date +"%Y-%m-%d %H:%M:%S%z")" "$type" "$0" "$*"
    elif [ "$type" == "Warn" ]; then
        printf '\033[0;34m%s [%s] [%s]: %s\033[0m\n' "$(date +"%Y-%m-%d %H:%M:%S%z")" "$type" "$0" "$*"
    elif [ "$type" == "Result" ]; then
        printf '\033[0;34m%s\033[0m\n' "$*"
    else
        printf '%s [%s] [%s]: %s\n' "$(date +"%Y-%m-%d %H:%M:%S%z")" "$type" "$0" "$*"
    fi
}
invisily_note() {
    invisily_log Note "$@"
}
invisily_warn() {
    invisily_log Warn "$@" >&2
}
invisily_result() {
    invisily_log Result "$@" >&2
}
invisily_error_no_exit() {
    invisily_log ERROR "$@" >&2
}
invisily_error() {
    invisily_log ERROR "$@" >&2
    invisily_log ERROR "Exiting..."  >&2
    exit 1
}

if [ "$EUID" -ne 0 ]; then
    invisily_error $'Please run as root.'
fi


################################################################################
# Dependency installation functions                                            #
################################################################################


apt_dependency_install(){
    if [ $# -eq 0 ]; then
        invisily_error $'No arguments supplied. Please use give apt package name in argument'
    elif [ $# -gt 1 ]; then
        APT_DEPENDENCIES=( "$@" )
        INSTALLABLE_APT_DEPENDENCIES=()
        UNINSTALLABLE_APT_DEPENDENCIES=()
        for dependency in "${APT_DEPENDENCIES[@]}"; do
            if [ "$(! dpkg-query -W -f='${Status}' "$dependency" 2>/dev/null | grep -c "ok installed")" -eq 0 ]; then
                invisily_note "Not found $dependency. Adding to installables..."
                INSTALLABLE_APT_DEPENDENCIES+=( "$dependency" )
            else
                invisily_note "Found $dependency. Adding to uninstallables..."
                UNINSTALLABLE_APT_DEPENDENCIES+=( "$dependency" )
            fi
        done

        INSTALLABLE_APT_DEPENDENCIES=( "${INSTALLABLE_APT_DEPENDENCIES[*]}" )
        UNINSTALLABLE_APT_DEPENDENCIES=( "${UNINSTALLABLE_APT_DEPENDENCIES[*]}" )
        if [ ${#UNINSTALLABLE_APT_DEPENDENCIES[@]} -gt 0 ]; then
            invisily_note "Following dependencies are already installed: ${UNINSTALLABLE_APT_DEPENDENCIES[*]}"
        fi
        if [ ${#INSTALLABLE_APT_DEPENDENCIES[@]} -gt 0 ]; then
            invisily_note "Installable dependencies are: ${INSTALLABLE_APT_DEPENDENCIES[*]}"
            invisily_note "Installing APT dependencies..."
            if apt-get install --yes "${INSTALLABLE_APT_DEPENDENCIES[*]}"; then
                invisily_note "Installed ${INSTALLABLE_APT_DEPENDENCIES[*]}."
            else
                invisily_error $'Installation failed for '"${INSTALLABLE_APT_DEPENDENCIES[*]}"'. Exiting...'
            fi
        fi
    else
        dependency=$1
        if [ "$(! dpkg-query -W -f='${Status}' "$dependency" 2>/dev/null | grep -c "ok installed")" -eq 0 ]; then

            if ! apt-get install --yes "$dependency"; then
                invisily_error $'Installation failed for '"$dependency"'. Exiting...'
            else
                invisily_note "Installed $dependency."
            fi
        fi
    fi
}

apt_dependency_uninstall(){
    if [ $# -eq 0 ]; then
        invisily_error $'No arguments supplied. Please use give apt package name in argument'
    elif [ $# -gt 1 ]; then
        APT_DEPENDENCIES=( "$@" )
        INSTALLABLE_APT_DEPENDENCIES=()
        UNINSTALLABLE_APT_DEPENDENCIES=()
        for dependency in "${APT_DEPENDENCIES[@]}"; do
            if [ "$(! dpkg-query -W -f='${Status}' "$dependency" 2>/dev/null | grep -c "ok installed")" -eq 0 ]; then
                invisily_note "Not found $dependency. Adding to installables..."
                INSTALLABLE_APT_DEPENDENCIES+=( "$dependency" )
            else
                invisily_note "Found $dependency. Adding to uninstallables..."
                UNINSTALLABLE_APT_DEPENDENCIES+=( "$dependency" )
            fi
        done

        INSTALLABLE_APT_DEPENDENCIES=( "${INSTALLABLE_APT_DEPENDENCIES[*]}" )
        UNINSTALLABLE_APT_DEPENDENCIES=( "${UNINSTALLABLE_APT_DEPENDENCIES[*]}" )
        if [ ${#UNINSTALLABLE_APT_DEPENDENCIES[@]} -gt 0 ]; then
            invisily_note "Following dependencies are already installed: ${UNINSTALLABLE_APT_DEPENDENCIES[*]} , and going to uninstall..."
            if apt-get purge --yes "${UNINSTALLABLE_APT_DEPENDENCIES[*]}"; then
                invisily_note "Uninstalled ${UNINSTALLABLE_APT_DEPENDENCIES[*]}."
            else
                invisily_error $'Uninstallation failed for '"${UNINSTALLABLE_APT_DEPENDENCIES[*]}"'. Exiting...'
            fi
            if apt autoremove -y; then
                invisily_note "Uninstalled ${UNINSTALLABLE_APT_DEPENDENCIES[*]}."
            else
                invisily_error $'Uninstallation failed for '"${UNINSTALLABLE_APT_DEPENDENCIES[*]}"'. Exiting...'
            fi
        fi
        if [ ${#INSTALLABLE_APT_DEPENDENCIES[@]} -gt 0 ]; then
            invisily_note "Following dependencies are already not installed: ${INSTALLABLE_APT_DEPENDENCIES[*]}"
        fi
    else
        dependency=$1
        if [ "$(! dpkg-query -W -f='${Status}' "$dependency" 2>/dev/null | grep -c "ok installed")" -ne 0 ]; then
            if apt-get purge --yes "$dependency"; then
                invisily_note "Uninstalled $dependency."
            else
                invisily_error $'Uninstallation failed for '"$dependency"'. Exiting...'
            fi
            if apt autoremove -y; then
                invisily_note "Uninstalled $dependency."
            else
                invisily_error $'Uninstallation failed for '"$dependency"'. Exiting...'
            fi
        else
            invisily_note "$dependency is already not installed on system"
        fi
    fi
}

apt_dependency_check(){
    dependency=$1
    if [ "$(! dpkg-query -W -f='${Status}' "$dependency" 2>/dev/null | grep -c "ok installed")" -eq 0 ]; then
        return 0
    else
        return 1
    fi
}


################################################################################
# Initial necessary verification                                               #
################################################################################
BUILD_ARCH=$(uname -p)

apt_dependency_check lsb-release && apt_dependency_install lsb-release

if [ -f /etc/lsb-release ] || [ -d /etc/lsb-release.d ]; then
    ubuntuVersion=$(lsb_release -r | cut -d: -f2 | sed s/'^\t'//)
    if [[ "$(arch)" =~ ^x86_64$ ]] ; then
        if [[ "$ubuntuVersion" != "20.04" && "$ubuntuVersion" != "22.04" ]]; then
            invisily_error "Supported Operating system is Ubuntu 20.04 or greater version, but it appears you are not running Ubuntu 20.04 or greater version"
        fi
    elif [[ "${BUILD_ARCH}" =~ ^aarch64$ ]] ; then
        if [[ "$ubuntuVersion" != "20.04" && "$ubuntuVersion" != "22.04" ]]; then
            invisily_error "Supported Operating system is Ubuntu 20.04 or greater version, but it appears you are not running Ubuntu 20.04 or greater version"
        fi
    else
        invisily_error "Supported Operating system for x86_64 and arm64 is Ubuntu 20.04 or greater version, but it appears you are not running either of these."
    fi
else
    invisily_error "It appears that you are not running Ubuntu Server Operating system."
fi
if [ "$(dpkg-query -W -f='${Status}' ubuntu-desktop 2>/dev/null | grep -c "ok installed")" -eq 1 ]; then
    invisily_error "Ubuntu 20.04 Server or greater version is required, but it appears that you are running Ubuntu Desktop"
fi

if [ $# -eq 0 ]; then
    invisily_error $'No arguments supplied. Please use -h or --help to get help'
fi

if [[ ! -f $RDI_PORTAL_ZIP_NAME ]]; then
    invisily_error "Setup files for RDI Bridge Portal are not found."
fi
################################################################################
# Validation functions                                                         #
################################################################################
cidr_to_netmask() {
    value=$(( 0xffffffff ^ ((1 << (32 - $1)) - 1) ))
    echo "$(( (value >> 24) & 0xff )).$(( (value >> 16) & 0xff )).$(( (value >> 8) & 0xff )).$(( value & 0xff ))"
}
__ip_validation() {
    ip=$1
    fail=0
    if expr "$ip" : '[0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*$' >/dev/null; then
        for i in 1 2 3 4; do
            if [ "$(echo "$ip" | cut -d. -f$i)" -gt 255 ]; then
            fail=1
            break
            fi
        done
    else
        fail=1
    fi
    if [ "$fail" == "1" ]; then
        invisily_error "IP Address ${ip} is not a valid IPv4 Address"  1>&2
        # exit 1
    fi
}

################################################################################
# Help                                                                         #
################################################################################
Help()
{
    # Display Help
    echo -e "Usage: sudo ./$0 [-h] [--help] [--install] [--remove] [--rdi-portal-port=<xxxx>] [--api-portal-url=<https://xxx.xxx.xxx.xxx>] [--api-portal-port=<xxxx>]"
    echo -e ""
    echo -e "Help: sudo ./$0 -h"
    echo -e "      OR"
    echo -e "      sudo ./$0 --help"
    echo -e
    echo -e "Options:"
    echo -e "-h | --help                            Print help."
    echo -e "--install                              Install Invisily Server Bridge"
    echo -e "--remove                               Remove Invisily Server Bridge"
    echo -e "--update                               Update Invisily Server Bridge"
    echo -e "--rdi-portal-port                      RDI Bridge Portal Port"
    echo -e ""
}
################################################################################
################################################################################
# Main program                                                                 #
################################################################################
################################################################################
for i in "$@"
do
case $i in
    --install)
    INSTALL=1
    shift # past argument=value
    ;;
    --remove)
    REMOVE=1
    shift # past argument=value
    ;;
    --rdi-portal-port=*)
    RDI_PORT="${i#*=}"
    shift # past argument=value
    ;;
    --update)
    UPDATE=1
    shift # past argument=value
    ;;
    -h|--help)
    Help
    exit 1
    ;;
    *)
    invisily_error "Unknown option. Exiting..."
    ;;
esac
done

arg_verification(){
    if [ "$INSTALL" == "1" ] && [ "$REMOVE" == "1" ]; then
        invisily_error $'You cannot select --install and --remove at the same time'
    fi
    if [[ -z $RDI_PORT ]] && [[ "$INSTALL" == "1" ]]; then
        RDI_PORT=8443
    fi
    if [[ -z $REDIS_IP ]] && [[ "$INSTALL" == "1" ]]; then
        REDIS_IP="127.0.0.1"
    fi
    if [[ -z $REDIS_PORT ]] && [[ "$INSTALL" == "1" ]]; then
        REDIS_PORT=6379
    fi
    if [[ -z $REDIS_PASS ]] && [[ "$INSTALL" == "1" ]]; then
        REDIS_PASS=$(openssl rand -base64 29 | tr -d "=+/" | cut -c1-14)
    fi

    if [[ -z $API_PORTAL_URL ]]; then
        API_PORTAL_URL="https://api.invisily"
    fi
    if [[ -z $API_PORTAL_PORT ]]; then
        API_PORTAL_PORT="443"
    fi
}

################################################################################
# Required variable declaration                                                #
################################################################################
MY_PATH=$(dirname "$0")
MY_PATH=$(cd "$MY_PATH" && pwd)
INV_SRV_CLIENT="invisily-client"
INV_SRV_CLIENT_INSTALLER="installer.deb"
INV_SRV_CLIENT_INSTALLER_PATH="$MY_PATH/$INV_SRV_CLIENT_INSTALLER"



docker_install() {
    echo 'net.ipv4.ip_forward=1' > /etc/sysctl.d/30-invisily-server-bridge-forward.conf
    # Enable without waiting for a reboot or service restart
    echo 1 > /proc/sys/net/ipv4/ip_forward
    if apt_dependency_check docker.io; then
        apt_dependency_install docker.io
    else
        invisily_error $'Docker seems already installed on this system, please uninstall it first using argumen --remove, Exiting...'
    fi
    cd "$MY_PATH"
    cat >> /etc/docker/daemon.json <<END
{
"log-driver": "journald",
"log-opts": {
"tag": "{{.Name}}"
},
"bip": "172.26.0.1/16"
}
END
    usermod -aG docker "$SUDO_USER"
    systemctl restart docker
}

invisily_client_install() {
    if [[ ! -f $INV_SRV_CLIENT_INSTALLER_PATH ]]; then
        invisily_error $'Invisily Client Installer is not present in current Directory.'
    fi
    if apt_dependency_check invisily-client; then
        apt_dependency_install "$INV_SRV_CLIENT_INSTALLER_PATH"
    else
        invisily_error $'Invisily Client seems already installed on this system, please update using --update or uninstall it first using argumen --remove, Exiting...'
    fi
}

invisily_client_update() {
    if [[ ! -f $INV_SRV_CLIENT_INSTALLER_PATH ]]; then
        invisily_error $'Invisily Client Installer is not present in current Directory.'
    fi
    if ! apt_dependency_check invisily-client; then
        AVAIL_VERSION=$(dpkg-deb -I $INV_SRV_CLIENT_INSTALLER_PATH | grep Version | cut -d' ' -f3)
        CUR_INSTALLED_VERSION=$(dpkg -s invisily-client  | grep -i '^Version' | cut -d' ' -f2)
        if [[ "$AVAIL_VERSION" != "$CUR_INSTALLED_VERSION" ]];then
            apt_dependency_install "$INV_SRV_CLIENT_INSTALLER_PATH"
            invisily_result $'Invisily RDI Bridge is successfully installed'
            {
                echo "================================================="
                echo "Invisily Client is successfully installed    "
                echo "================================================="
                echo "Go to /opt/invisily/linuxclientd/bin/ and Run    "
                echo "sudo ./Invisilyd  to first time login to Invisily"
                echo "After that Run sudo systemctl start invisily_backend    "
                echo "to run Invisily Service via Linux Service        "
                echo "================================================="
            } >> "$MY_PATH"/.result
        else
            invisily_warn "Same version of Invisily client is already installed"
        fi
    else
        invisily_error $'Invisily Client is not already installed on this system, If this is a new system then please use --install to newly install Invisily Client'
    fi
}

rdi_bridge_portal_install() {
    RDI_ENV="production"
    [[ ! -f "$MY_PATH/$RDI_PORTAL_SETTING_FILE_NAME" ]] && invisily_error "RDI Bridge settings.cfg file does not exists."
    [ -d "/var/local/env" ] && rm -rf /var/local/env
    [ -d "/var/www/html" ] && rm -rf /var/www/html
    #[ -d "/var/log/invisily" ] && rm -rf /var/log/invisily
    mkdir -p /var/log/invisily/rdi-bridge-portal/
    chown -R "$FS_USER":"$FS_USER" /var/log/invisily/
    RDI_PORTAL_BASE_DIR=/var/www/html/rdi-bridge-portal
    RDI_PORTAL_CONFIG_DIR="/var/local/env"
    RDI_PORTAL_SSL_DIR="/var/local/ssl"
    RDI_PORTAL_CONFIG_FILE_NAME="rdi"
    RDI_PORTAL_CONFIG_FILE=$RDI_PORTAL_CONFIG_DIR/$RDI_PORTAL_CONFIG_FILE_NAME
    export DEBIAN_FRONTEND=noninteractive
    apt-get -y update && apt-get -y install zip gcc nginx curl build-essential software-properties-common openssl libxml2-dev zlib1g-dev
    apt_dependency_check python3 && apt_dependency_install python3
    apt_dependency_check python3-pip && apt_dependency_install python3-pip
    pip install gunicorn
    mkdir -p $RDI_PORTAL_BASE_DIR
    chown -R "$FS_USER":"$FS_USER" $RDI_PORTAL_BASE_DIR
    cd "$MY_PATH" || exit
    unzip -o "$MY_PATH/$RDI_PORTAL_ZIP_NAME" -d $RDI_PORTAL_BASE_DIR
    mkdir -p $RDI_PORTAL_CONFIG_DIR $RDI_PORTAL_SSL_DIR
    chown -R "$FS_USER":"$FS_USER" $RDI_PORTAL_CONFIG_DIR $RDI_PORTAL_SSL_DIR $RDI_PORTAL_BASE_DIR
    cp $RDI_PORTAL_BASE_DIR/env/$RDI_PORTAL_CONFIG_FILE_NAME $RDI_PORTAL_CONFIG_FILE
    cp -rf "$MY_PATH"/$RDI_PORTAL_SETTING_FILE_NAME $RDI_PORTAL_CONFIG_DIR
    chown -R "$FS_USER":"$FS_USER" $RDI_PORTAL_CONFIG_DIR

    if ! sed -e "/{RDI_PORT}/{s//$RDI_PORT/;h};\${x;/./{x;q0};x;q1}" -i $RDI_PORTAL_CONFIG_FILE; then
        invisily_error $'Updating RDI portal port number in ' $RDI_PORTAL_CONFIG_FILE  'is failed'
    fi
    if ! sed -e "/{RDI_ENV}/{s//$RDI_ENV/;h};\${x;/./{x;q0};x;q1}" -i $RDI_PORTAL_CONFIG_FILE; then
        invisily_error $'Updating RDI Portal environment var in ' $RDI_PORTAL_CONFIG_FILE  'is failed'
    fi
    if ! sed -i "s|{API_PORTAL_URL}|$API_PORTAL_URL|g" $RDI_PORTAL_CONFIG_FILE; then
        invisily_error $'Updating API Portal URL in ' $RDI_PORTAL_CONFIG_FILE  'is failed'
    fi
    if ! sed -e "/{API_PORTAL_PORT}/{s//$API_PORTAL_PORT/;h};\${x;/./{x;q0};x;q1}" -i $RDI_PORTAL_CONFIG_FILE; then
        invisily_error $'Updating API Portal Port var in ' $RDI_PORTAL_CONFIG_FILE  'is failed'
    fi
    if ! sed -e "/\${REDIS_HOST}/{s//$REDIS_IP/;h};\${x;/./{x;q0};x;q1}" -i $RDI_PORTAL_CONFIG_FILE; then
        invisily_error $'Updating Redis Server IP in ' $RDI_PORTAL_CONFIG_FILE  'is failed'
    fi
    if ! sed -e "/\${REDIS_PORT}/{s//$REDIS_PORT/;h};\${x;/./{x;q0};x;q1}" -i $RDI_PORTAL_CONFIG_FILE; then
        invisily_error $'Updating Redis Server Port in ' $RDI_PORTAL_CONFIG_FILE  'is failed'
    fi
    if ! sed -e "/\${REDIS_PASSWORD}/{s//$REDIS_PASS/;h};\${x;/./{x;q0};x;q1}" -i $RDI_PORTAL_CONFIG_FILE; then
        invisily_error $'Updating Redis Server Password in ' $RDI_PORTAL_CONFIG_FILE  'is failed'
    fi
    cd $RDI_PORTAL_SSL_DIR
    openssl req -new -newkey rsa:2048 -days 3650 -nodes -x509 -subj "/C=NA/ST=NA/L=NA/O=NA/CN=invisily.com" -keyout sdp-key.pem -out sdp-cert.crt
    chown -R "$SUDO_USER":"$SUDO_USER" $RDI_PORTAL_SSL_DIR

    # creating python systemd service
    cd ~
    cd /var/www/html/rdi-bridge-portal
    while read line; do
        if [[ ! $line =~ ^#.* ]]; then
            pip3 install --no-compile $line || true
        fi
    done < requirements.txt
    cd ~
    cat > /etc/systemd/system/gunicorn.socket <<END
[Unit]
Description=gunicorn socket

[Socket]
ListenStream=/run/gunicorn.sock

[Install]
WantedBy=sockets.target
END

    cat > /etc/systemd/system/gunicorn.service <<END
[Unit]
Description=gunicorn daemon
Requires=gunicorn.socket
After=network.target gunicorn.socket

[Service]
User=$FS_USER
Group=www-data
WorkingDirectory=/var/www/html/rdi-bridge-portal
ExecStart=/usr/local/bin/gunicorn -w 4 -b unix:/run/gunicorn.sock app:app
Restart=always
Environment=PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin

[Install]
WantedBy=multi-user.target
END

    systemctl daemon-reload
    systemctl start gunicorn.socket
    systemctl enable gunicorn.socket
    
    if [ ! -S /run/gunicorn.sock ]; then
    invisily_error "gunicorn socket creation failed."
    fi

    if ! curl --unix-socket /run/gunicorn.sock localhost; then
    invisily_error "Curl to gunicorn socket failed."
    fi

    systemctl daemon-reload
    systemctl restart gunicorn

    cd ~

    cd /var/www/html/rdi-bridge-portal

    export FLASK_ENV='production'

    systemctl daemon-reload || invisily_warn "Systemctl daemon reloading is failed"
    systemctl restart gunicorn.socket || invisily_warn "systemctl gunicorn.socket service restart is failed"
    
    if ! apt_dependency_check nginx; then
    apt_dependency_install nginx
    fi
    cat > /etc/nginx/sites-available/default <<END
server {
  listen ${RDI_PORT} ssl;
  server_name ${PUBLIC_IP};

  ssl_certificate ${RDI_PORTAL_SSL_DIR}/sdp-cert.crt;
  ssl_certificate_key ${RDI_PORTAL_SSL_DIR}/sdp-key.pem;

  location / {
    proxy_pass http://unix:/run/gunicorn.sock;
    proxy_set_header Host \$host;
    proxy_set_header X-Real-IP \$remote_addr;
    proxy_set_header X-Forwarded-For \$proxy_add_x_forwarded_for;
    proxy_set_header X-Forwarded-Proto \$scheme;
  }
}
END
    if ! nginx -t >/dev/null 2>&1; then
    echo "Nginx configuration test failed. Exiting..."
    exit 1
    fi

    systemctl restart nginx

    echo "====================================================="
    echo "Invisily RDI Portal has been successfully installed"
    echo "====================================================="
}

rdi_bridge_portal_update() {
    [[ ! -f "$MY_PATH/$RDI_PORTAL_SETTING_FILE_NAME" ]] && invisily_error "RDI Bridge settings.cfg file does not exists."
    [ -d "/var/www/html" ] && rm -rf /var/www/html
    RDI_PORTAL_BASE_DIR=/var/www/html/rdi-bridge-portal
    RDI_PORTAL_CONFIG_DIR="/var/local/env"
    RDI_PORTAL_SSL_DIR="/var/local/ssl"
    RDI_PORTAL_CONFIG_FILE_NAME="rdi"
    DATE=$(date '+%Y-%m-%d%H:%M:%S')
    RDI_PORTAL_CONFIG_FILE=$RDI_PORTAL_CONFIG_DIR/$RDI_PORTAL_CONFIG_FILE_NAME
    apt_dependency_check python3 && apt_dependency_install python3
    apt_dependency_check python3-pip && apt_dependency_install python3-pip
    mkdir -p $RDI_PORTAL_BASE_DIR
    cd "$MY_PATH" || exit
    unzip -o "$MY_PATH/$RDI_PORTAL_ZIP_NAME" -d $RDI_PORTAL_BASE_DIR

    [[ ! -d $RDI_PORTAL_SSL_DIR ]] && mkdir -p $RDI_PORTAL_SSL_DIR
    if [[ ! -f $RDI_PORTAL_SSL_DIR/sdp-key.pem ]] || [[ ! -f $RDI_PORTAL_SSL_DIR/sdp-cert.crt ]]; then
        rm -rf "${RDI_PORTAL_SSL_DIR:?}/"*
        openssl req -new -newkey rsa:2048 -days 3650 -nodes -x509 -subj "/C=NA/ST=NA/L=NA/O=NA/CN=invisily.com" -keyout sdp-key.pem -out sdp-cert.crt
    fi
    chown -R "$FS_USER":"$FS_USER" $RDI_PORTAL_CONFIG_DIR $RDI_PORTAL_SSL_DIR $RDI_PORTAL_BASE_DIR

    systemctl stop rdi_portal.service || invisily_warn "systemctl rdi_portal stopping is failed"
    systemctl disable rdi_portal.service || invisily_warn "systemctl rdi_portal disabling failed"
    systemctl daemon-reload || invisily_warn "systemctl daemon-reloading is not reloading"
    rm -rf /lib/systemd/system/rdi_portal.service

cd ~

    #B2B_PORTAL_BASE_DIR=/var/www/html/rdi-bridge-portal
    cd /var/www/html/rdi-bridge-portal

    export FLASK_ENV='production'

    while read line; do
        if [[ ! $line =~ ^#.* ]]; then
            pip3 install --no-compile $line || true
        fi
    done < requirements.txt

    # creating python systemd service
    cd ~

    cat > /lib/systemd/system/rdi_portal.service <<EOF
[Unit]
Description=RDI Application
After=network.target

[Service]
Type=simple
ExecStart=/usr/bin/python3 /var/www/html/rdi-bridge-portal/app.py
Restart=always
User=$FS_USER
Environment=PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin
WorkingDirectory=/var/www/html/rdi-bridge-portal/flask-portal

[Install]
WantedBy=multi-user.target
EOF

    systemctl daemon-reload || invisily_warn "Systemctl daemon reloading is failed"
    systemctl enable rdi_portal.service || invisily_warn "systemctl rdi_portal service enabling is failed"
    systemctl start rdi_portal.service || invisily_warn "systemctl rdi_portal service failed to start"

    echo "====================================================="
    echo "Invisily RDI Portal has been successfully updated"
    echo "====================================================="
}

rdi_bridge_install(){
    PUBLIC_IP=$(curl -s ifconfig.me)
    if apt-get -y update; then
        invisily_note "OS repo cache updated successfully"
    else
        invisily_error "OS repo cache failed to update. Please make sure you have internet working inside your server. Please run apt-get -y update manually first and then re-run installation script"
    fi
    invisily_client_install
    docker_install
    docker_image_creation

    invisily_result $'Invisily RDI Bridge is successfully installed'
    {
        echo "=============================================================="
        echo "Invisily RDI Bridge is successfully installed                 "
        echo "=============================================================="
        echo "Go to /opt/invisily/linuxclientd/bin/ and Run                 "
        echo "sudo ./invisily_backend to first time login to Invisily       "
        echo "After that Run sudo systemctl start invisily_backend.service  "
        echo "to run Invisily Service via Linux Service                     "
        echo "=============================================================="
        echo "=============================================================="
        echo "Open browser, and go to URL:-                                 "
        echo "https://$PUBLIC_IP:$RDI_PORT"
        echo "=============================================================="
        echo "=============================================================="
    } >> "$MY_PATH"/.result
}

install_redis_server() {
    REDIS_DEPENDECY_NAME=redis-server
    if [ "$(! dpkg-query -W -f='${Status}' $REDIS_DEPENDECY_NAME 2>/dev/null | grep -c "ok installed")" -eq 0 ]; then
        invisily_note "Not found redis-server, installing redis-server"
        ###############################################################################
        # Install redis-server dependency and check for failure / success
        ###############################################################################

        export DEBIAN_FRONTEND=noninteractive
        if ! apt-get install --yes $REDIS_DEPENDECY_NAME; then
            invisily_error "Installation failed for redis-server"
        fi
        echo "Setting up Redis Server"
        [ -f /etc/redis/redis.conf ] && sed -e "s/# requirepass foobared/requirepass $REDIS_PASS/g" -i /etc/redis/redis.conf
        [ -f /etc/redis/redis.conf ] && sed -i "s|bind 127.0.0.1.*|bind 0.0.0.0 ::1|g" /etc/redis/redis.conf
        #<<<<configuration for persistent cache>>>
        sed -i "s|appendonly.*no|appendonly yes|g" /etc/redis/redis.conf
        if systemctl is-active --quiet redis-server.service; then
            kill -9 "$(lsof -t -i:6379)" &> /dev/null
            if [[ -f "/var/run/redis/redis-server.pid" ]]; then
                if ! kill -9 "$(pgrep -x redis-server)"; then
                    invisily_warn "Failed to kill redis-server process"
                fi
            fi
        else
            systemctl restart redis-server.service
        fi
        #<<<<configuration for persistent cache>>>
        systemctl enable redis-server
    fi
}



docker_image_creation() {
    DOCKER_CONTAINER_IMAGE_NAME="invisily-rdi-bridge-desktop"
    cat > custom_startup_desktop.sh << 'END'
#!/usr/bin/env bash
set -ex
START_COMMAND="google-chrome"
PGREP="chrome"
PGREP_SUPERVISORD="supervisord"
MAXIMIZE="true"
DEFAULT_ARGS=""
LOG_FILE="/var/log/custom_start_log.txt"
# output redirection
exec > >(tee -i "$LOG_FILE")
exec 2>&1
if [[ $MAXIMIZE == 'true' ]] ; then
    DEFAULT_ARGS+=" --start-maximized"
fi
ARGS=${APP_ARGS:-$DEFAULT_ARGS}

options=$(getopt -o gau: -l go,assign,url: -n "$0" -- "$@") || exit
eval set -- "$options"

while [[ $1 != -- ]]; do
    case $1 in
        -g|--go) GO='true'; shift 1;;
        -a|--assign) ASSIGN='true'; shift 1;;
        -u|--url) OPT_URL=$2; shift 2;;
        *) echo "bad option: $1" >&2; exit 1;;
    esac
done
shift

# Process non-option arguments.
for arg; do
    echo "arg! $arg"
done

FORCE=$2

kasm_exec() {
    if [ -n "$OPT_URL" ] ; then
        URL=$OPT_URL
    elif [ -n "$1" ] ; then
        URL=$1
    fi

    # Since we are execing into a container that already has the browser running from startup,
    #  when we don't have a URL to open we want to do nothing. Otherwise a second browser instance would open.
    if [ -n "$URL" ] ; then
        /usr/bin/filter_ready
        /usr/bin/desktop_ready
        $START_COMMAND $ARGS $OPT_URL
    else
        echo "No URL specified for exec command. Doing nothing."
    fi
}

kasm_startup() {
    if [ -n "$KASM_URL" ] ; then
        URL=$KASM_URL
    elif [ -z "$URL" ] ; then
        URL=$LAUNCH_URL
    fi
    if [[ ! -f /home/kasm-user/Desktop/xfce4-terminal.desktop ]]; then
        cp -rf /usr/share/applications/xfce4-terminal.desktop /home/kasm-user/Desktop/xfce4-terminal.desktop
        chmod u+x /home/kasm-user/Desktop/xfce4-terminal.desktop
        chown 1000:1000 /home/kasm-user/Desktop/xfce4-terminal.desktop
    fi

    if [ -z "$DISABLE_CUSTOM_STARTUP" ] ||  [ -n "$FORCE" ] ; then

        echo "Entering process startup loop"
        set +x
        while true
        do
            if ! pgrep -x $PGREP_SUPERVISORD > /dev/null
            then
                set +e
                supervisord_startup
                set -e
            fi
            # if ! pgrep -x $PGREP > /dev/null
            # then
            #     /usr/bin/filter_ready
            #     /usr/bin/desktop_ready
            #     set +e
            #     $START_COMMAND $ARGS $URL &
            #     set -e
            # fi
            sleep 1
        done
        set -x

    fi

}
###custom start####
apt_dependency_check(){
    dependency=$1
    if [ "$(! dpkg-query -W -f='${Status}' $dependency 2>/dev/null | grep -c "ok installed")" -eq 0 ]; then
        return 0
    else
        return 1
    fi
}
supervisord_startup() {
    if ! apt_dependency_check supervisor; then
        sudo -E "DEVICE_UUID=$DEVICE_UUID" "INV_USERNAME=$INV_USERNAME" "INV_PASSWORD=$INV_PASSWORD" /usr/bin/supervisord
    fi
}
####custom end####
if [ -n "$GO" ] || [ -n "$ASSIGN" ] ; then
    kasm_exec
else
#    lighttpd_startup
    kasm_startup
fi
END
    cat > supervisord.conf << 'END'
[supervisord]
nodaemon=false
logfile = /var/log/supervisord.log
logfile_maxbytes = 10MB
logfile_backups = 10
loglevel = info

[include]
files=/etc/supervisor/conf.d/supervisord-invisily-backend.conf  /etc/supervisor/conf.d/supervisord-invisily-firewall.conf  /etc/supervisor/conf.d/supervisord-rdi-bridge.conf

; https://github.com/Supervisor/supervisor/issues/376#issuecomment-404385767

[unix_http_server]
file=/var/run/supervisor.sock
chmod=0700

[rpcinterface:supervisor]
supervisor.rpcinterface_factory = supervisor.rpcinterface:make_main_rpcinterface

[supervisorctl]
serverurl=unix:///var/run/supervisor.sock
END
    cat > supervisord-invisily-backend.conf << 'END'
[program:invisily_backend]
command=/supervisord-invisily-backend.sh
priority=100
startsecs=0
autorestart=true
autostart=true
stopasgroup=true
killasgroup=true
stopsignal=INT
END
    cat > supervisord-rdi-bridge.conf << 'END'
[program:rdi-bridge]
command=/supervisord-rdi-bridge.sh
priority=400
startsecs=0
autorestart=true
autostart=true
stopasgroup=true
killasgroup=true
stopsignal=INT
END
    cat > supervisord-invisily-firewall.conf << 'END'
[program:invisily_firewall]
command=/supervisord-invisily-firewall.sh
priority=200
startsecs=0
autorestart=true
autostart=true
stopasgroup=true
killasgroup=true
stopsignal=INT
END
    cat > supervisord-invisily-firewall.sh << 'END'
#!/bin/bash
cd /opt/invisily/linuxclientd/bin
/bin/bash -ce "exec /opt/invisily/linuxclientd/bin/invisily_firewall"
END
    cat > supervisord-invisily-backend.sh << 'END'
#!/bin/bash
CLIENT_USERNAME="$INV_USERNAME"
CLIENT_PASSWORD="$INV_PASSWORD"
cd /opt/invisily/linuxclientd/bin
./invisily_backend -i $DEVICE_UUID -u "$CLIENT_USERNAME" -p "$CLIENT_PASSWORD" -t "slave_bridge" -m background
END
    cat > supervisord-rdi-bridge.sh << 'END'
#!/bin/bash
cd /
/bin/bash /invisily-rdi-bridge-container-daemon.sh
END
    cat > invisily-rdi-bridge-container-daemon.sh << 'END'
#!/bin/bash
PID=/var/run/invisily-rdi-bridge-container-daemon.pid
echo "My pid is: $$"
echo $$ > $PID
finish=0
trap 'finish=1' SIGUSR1
LOG_FILE_PATH=/var/log/invisily/invisily-rdi-bridge
LOG_FILE=$LOG_FILE_PATH/invisily-rdi-bridge-container-daemon.log
# INV_PROVIDED_SVC_JSON_FILE=/opt/invisily/linuxclientd/client_data/configs/services.json
INV_PROVIDED_SVC_JSON_FILE=/tmp/services.json
WEB_SVC_JSON_FILE=/var/www/html/services.json
DATE=$(date +%Y-%m-%d-%H:%M:%S)
[[ ! -d $LOG_FILE_PATH ]] && mkdir -p $LOG_FILE_PATH
#####################################################
#### LOG FUNCTIONS ##################################
#####################################################
invisily_log() {
    local type="$1"; shift
    if [ "$type" == "ERROR" ]; then
        printf '\033[0;31m%s [%s] [%s]: %s\033[0m\n' "$(date +"%Y-%m-%d %H:%M:%S%z")" "$type" "$0" "$*" | tee -a $LOG_FILE
    elif [ "$type" == "Warn" ]; then
        printf '\033[0;34m%s [%s] [%s]: %s\033[0m\n' "$(date +"%Y-%m-%d %H:%M:%S%z")" "$type" "$0" "$*" | tee -a $LOG_FILE
    elif [ "$type" == "Result" ]; then
        printf '\033[0;34m%s\033[0m\n' "$*" | tee -a $LOG_FILE
    else
        printf '%s [%s] [%s]: %s\n' "$(date +"%Y-%m-%d %H:%M:%S%z")" "$type" "$0" "$*"  | tee -a $LOG_FILE
    fi
}
invisily_note() {
    invisily_log Note "$@"
}
invisily_warn() {
    invisily_log Warn "$@" >&2
}
invisily_result() {
    invisily_log Result "$@" >&2
}
invisily_error() {
    invisily_log ERROR "$@" >&2
    invisily_log ERROR "Exiting..."  >&2
    exit 1
}
invisily_error_no_exit() {
    invisily_log ERROR "$@" >&2
}
###############################################################

invisily_note "Bridge monitoring started/restarted"
sleep 1
while (( finish != 1 ))
do
    DATE=$(date +%Y-%m-%d-%H:%M:%S)
    #services html file code
    if curl -s -w "%{http_code}" -o $INV_PROVIDED_SVC_JSON_FILE "http://localhost:9174/services" | grep -q "^2"; then
        CURRENT_FILE_MD5=$(md5sum $INV_PROVIDED_SVC_JSON_FILE | awk -F ' ' '{print $1}')
        USED_FILE_MD5=$(md5sum $WEB_SVC_JSON_FILE | awk -F ' ' '{print $1}')
        if [[ "$CURRENT_FILE_MD5" != "$USED_FILE_MD5" ]]; then
            invisily_note "$DATE - Invisily provided Services JSON file is not matching with currently inused file, Updating it..."
            if cp -r $INV_PROVIDED_SVC_JSON_FILE $WEB_SVC_JSON_FILE; then
                invisily_note "Invisily provided Services JSON file is updated to web server location"
            else
                invisily_error "Invisily provided Services JSON file is failed to update to web server location"
            fi
        fi
        rm -rf $INV_PROVIDED_SVC_JSON_FILE
    else
        invisily_error "Curl request failed! Error: Failed to connect to services API url"
    fi

    pids=$(pidof invisily_backend)
    if [ "$pids" == "" ]
    then
        if supervisorctl restart invisily_backend; then
            invisily_note "Invisily Backend service was not running and has been restarted!"
        else
            invisily_error_no_exit "Invisily Backend service was not running, attempt to restart is failed"
        fi
    fi
    pids=$(pidof invisily_firewall)
    if [ "$pids" == "" ]
    then
        if supervisorctl restart invisily_firewall; then
            invisily_note "Invisily Firewall service was not running and has been restarted!"
        else
            invisily_error_no_exit "Invisily Firewall service was not running, attempt to restart is failed"
        fi
    fi
    REFRESH_TIME=$((10 + RANDOM % 15))
    invisily_note "Recheck will now run after $REFRESH_TIME seconds"
    sleep $REFRESH_TIME
done
END
cat > invisily.desktop << 'END'
[Desktop Entry]
Version=1.0
Name=Invisily Client Services
# Only KDE 4 seems to use GenericName, so we reuse the KDE strings.
# From Ubuntu's language-pack-kde-XX-base packages, version 9.04-20090413.
GenericName=Web Browser
# Gnome and KDE 3 uses Comment.
Comment=Access the Internet
Exec=/usr/bin/firefox http://services.inv
StartupNotify=true
Terminal=false
Icon=/home/kasm-user/invisily-icon.png
Type=Application
Categories=Network;WebBrowser;
MimeType=application/pdf;application/rdf+xml;application/rss+xml;application/xhtml+xml;application/xhtml_xml;application/xml;image/gif;image/jpeg;image/png;image/webp;text/html;text/xml;x-scheme-handler/http;x-scheme-handler/https;
Actions=new-window;new-private-window;

[Desktop Action new-window]
Name=New Window
Exec=/usr/bin/firefox http://services.inv

[Desktop Action new-private-window]
Name=New Incognito Window
Exec=/usr/bin/firefox --incognito
END
    cat > dockerfile-invisily << 'END'
ARG BASE_TAG="latest"
ARG BASE_IMAGE="ubuntu-desktop"
FROM invisily/$BASE_IMAGE:$BASE_TAG
USER root

ENV HOME /home/kasm-default-profile
ENV STARTUPDIR /dockerstartup
ENV INST_SCRIPTS $STARTUPDIR/install
WORKDIR $HOME

######### Start Invisily customization ############

COPY ./custom_startup_desktop.sh $STARTUPDIR/custom_startup.sh
RUN chmod +x $STARTUPDIR/custom_startup.sh

ADD ./supervisord.conf /etc/supervisor/supervisord.conf
ADD ./supervisord-invisily-backend.conf /etc/supervisor/conf.d/supervisord-invisily-backend.conf
ADD ./supervisord-rdi-bridge.conf /etc/supervisor/conf.d/supervisord-rdi-bridge.conf
ADD ./supervisord-invisily-firewall.conf /etc/supervisor/conf.d/supervisord-invisily-firewall.conf
ADD ./supervisord-invisily-firewall.sh /supervisord-invisily-firewall.sh
RUN chmod 755 /supervisord-invisily-firewall.sh
ADD ./supervisord-invisily-backend.sh /supervisord-invisily-backend.sh
RUN chmod 755 /supervisord-invisily-backend.sh
ADD ./supervisord-rdi-bridge.sh /supervisord-rdi-bridge.sh
ADD ./invisily-rdi-bridge-container-daemon.sh /invisily-rdi-bridge-container-daemon.sh
RUN chmod 755 /supervisord-rdi-bridge.sh
RUN chmod 755 /invisily-rdi-bridge-container-daemon.sh
ADD ./service_list.html /var/www/html/index.html

RUN apt-get update \
    && apt-get install -y systemd systemd-sysv apt-utils \
    sudo dmidecode iptables curl wget gnupg iputils-ping \
    network-manager iproute2 supervisor zip net-tools \
    net-tools lsb-release ntp libbsd-dev psmisc lsof jq \
    && useradd -m -d /home/kasm-user -s /bin/bash kasm-user \
    && echo 'kasm-user ALL=(ALL) NOPASSWD:SETENV: /usr/bin/supervisord' >> /etc/sudoers \
    && rm -rf /var/lib/apt/list/*
COPY ./installer.deb /root/
RUN apt-get install -y /root/installer.deb
RUN rm -rf /root/installer.deb
ADD ./invisily.desktop /home/kasm-user/Desktop/invisily.desktop
RUN chmod u+x /home/kasm-user/Desktop/invisily.desktop
######### END Invisily customization ############

RUN chown 1000:0 $HOME
RUN $STARTUPDIR/set_user_permission.sh $HOME

ENV HOME /home/kasm-user
WORKDIR $HOME
RUN mkdir -p $HOME && chown -R 1000:0 $HOME



USER 1000
END
    if docker build -t $DOCKER_CONTAINER_IMAGE_NAME -f dockerfile-invisily .; then
        if docker images | grep $DOCKER_CONTAINER_IMAGE_NAME; then
            invisily_log "Docker image creation is sccessfull"
        fi
    else
        invisily_error_no_exit "Docker image creationis failed"
    fi
    if [[ ! -d /var/local/scripts/ ]]; then
        mkdir -p /var/local/scripts/
        chown -R "$FS_USER":"$FS_USER" /var/local/scripts/
    fi
    if [[ -d /var/local/scripts/ ]]; then
        cat > /var/local/scripts/b2b_container_create.sh << 'END'
#!/bin/bash
CONTAINER_NAME=$1
CONTAINER_PORT=$2
DEVICE_UUID=$3
INV_USERNAME=$4
INV_PASSWORD=$5
VNC_USR=$6
VNC_PW=$7
docker run -d --rm -it --name "$CONTAINER_NAME" --shm-size=512m -p "$CONTAINER_PORT":6901 --cap-add=NET_ADMIN --device=/dev/net/tun -e DEVICE_UUID="$DEVICE_UUID" -e INV_USERNAME="$INV_USERNAME" -e INV_PASSWORD="$INV_PASSWORD" -e VNC_USR="$VNC_USR" -e VNC_PW="$VNC_PW" --add-host services.inv:127.0.0.1 -v /sys/fs/cgroup:/sys/fs/cgroup:ro invisily-rdi-bridge-desktop:latest
END
    chown -R "$FS_USER":"$FS_USER" /var/local/scripts/
    chmod 755 /var/local/scripts/b2b_container_create.sh
    fi
    rm -rf custom_startup_desktop.sh supervisord-invisily-backend.conf supervisord-rdi-bridge.conf supervisord-invisily-firewall.conf  supervisord-invisily-firewall.sh supervisord-invisily-backend.sh supervisord-rdi-bridge.sh invisily-rdi-bridge-container-daemon.sh invisily.desktop dockerfile-invisily supervisord.conf || invisily_warn "Failed to delete temporary files"
    docker rmi "$(docker images -a|grep "<none>"|awk '$1=="<none>" {print $3}')" 2>/dev/null || invisily_warn "Failed to delete zoombie images, this is ignorable"
    chmod 666 /var/run/docker.sock

}

rdi_bridge_remove(){
    systemctl stop invisily_backend.service || invisily_warn $'invisily_backend service cannot find'
    apt_dependency_check $INV_SRV_CLIENT || apt_dependency_uninstall $INV_SRV_CLIENT
    systemctl stop redis-server  || invisily_warn $'redis service cannot find'
    apt_dependency_check redis-server || apt_dependency_uninstall redis-server
    apt_dependency_check redis-tools || apt_dependency_uninstall redis-tools
    # apt_dependency_uninstall redis*
    systemctl stop docker ||  invisily_warn $'docker service cannot find'
    apt_dependency_check docker.io || apt_dependency_uninstall docker-engine docker docker.io docker-ce
    apt_dependency_check lsb-release || apt_dependency_uninstall lsb-release
    apt autoremove -y   || invisily_warn $'apt autoremove failed due to unknown reasons'
    rm -rf /var/lib/docker /etc/docker   || invisily_warn $'deleting docker directories failed due to unknown reasons'
    rm -rf /etc/apparmor.d/docker    || invisily_warn $'deleting docker directories failed due to unknown reasons'
    groupdel docker    || invisily_warn $'deleting docker group failed due to unknown reasons'
    rm -rf /var/run/docker.sock    || invisily_warn $'deleting docker directories failed due to unknown reasons'
    ip link delete docker0     || invisily_warn $'deleting docker0 interface failed due to unknown reasons'
    rm -rf /opt/invisily || invisily_warn $'/opt/invisily could not be deleted!'
    rm -rf /opt/invisily_updates || invisily_warn $'/opt/invisily_updates could not be deleted!'
    systemctl daemon-reload
    #rdi Portal Uninstall
    FS_USER_HOME_DIR=$(getent passwd "$FS_USER" | cut -d: -f6)
    sudo -H -u "$FS_USER" bash <<CODESETUP
[[ "$VERBOSE" == "1" ]] && set -x
$(declare -f invisily_log)
$(declare -f invisily_warn)
$(declare -f invisily_note)
$(declare -f invisily_error)
$(declare -f invisily_result)
$(declare -f invisily_error_no_exit)

CODESETUP

cd ~

systemctl stop rdi_portal.service || invisily_warn "systemctl rdi_portal stopping is failed"
systemctl disable rdi_portal.service || invisily_warn "systemctl rdi_portal disabling failed"
systemctl daemon-reload || invisily_warn "systemctl daemon-reloading is not reloading"
rm -rf /lib/systemd/system/rdi_portal.service


    if [ -d "/var/local/env" ]; then
        rm -rf /var/local/env
    else
        invisily_warn $'/var/local/env directory does not exists'
    fi
    if [ -d "/var/local/ssl" ]; then
        rm -rf /var/local/ssl
    else
        invisily_warn $'/var/local/ssl directory does not exists'
    fi
    if [ -d "/var/www/html" ]; then
        rm -rf /var/www/html
    else
        invisily_warn $'/var/www/html directory does not exists'
    fi
    if [ -d "/var/log/invisily" ]; then
        rm -rf /var/log/invisily
    else
        invisily_warn $'/var/log/invisily directory does not exists'
    fi
    if [[ -f /etc/systemd/system/pm2-$FS_USER.service ]]; then
        rm -rf /etc/systemd/system/pm2-"$FS_USER".service
        systemctl daemon-reload
    fi
    invisily_warn $"Invisily RDI bridge has been uninstalled successfully and now your server will be rebooted."
    read -p "Press Enter to reboot" </dev/tty
    reboot
}


arg_verification
if [[ "$INSTALL" == "1" ]]; then
    install_redis_server
    rdi_bridge_portal_install
    rdi_bridge_install
elif [[ "$REMOVE" == "1" ]]; then
    rdi_bridge_remove
elif [[ "$UPDATE" == "1" ]]; then
    rdi_bridge_portal_update
    docker_image_creation
    invisily_client_update
else
    invisily_error $'You did not provide any argument to any process'
fi

if [[ -f "$MY_PATH"/.result ]]; then
    while IFS= read -r line; do
        invisily_result $''"$line"
    done < "$MY_PATH"/.result
    rm -rf "$MY_PATH"/.result
else
    invisily_warn $'Cannot find Result file to display'
fi
