#!/bin/bash

set -eo pipefail
shopt -s nullglob

################################################################################
# Logging functions                                                            #
################################################################################
invisily_log() {
    local type="$1"; shift
    if [ "$type" == "ERROR" ]; then
        printf '\033[0;31m%s [%s] [%s]: %s\033[0m\n' "$(date +"%Y-%m-%d %H:%M:%S%z")" "$type" "$0" "$*"
    elif [ "$type" == "Warn" ]; then
        printf '\033[0;34m%s [%s] [%s]: %s\033[0m\n' "$(date +"%Y-%m-%d %H:%M:%S%z")" "$type" "$0" "$*"
    elif [ "$type" == "Result" ]; then
        printf '\033[0;34m%s\033[0m\n' "$*"
    else
        printf '%s [%s] [%s]: %s\n' "$(date +"%Y-%m-%d %H:%M:%S%z")" "$type" "$0" "$*"
    fi
}
invisily_note() {
    invisily_log Note "$@"
}
invisily_warn() {
    invisily_log Warn "$@" >&2
}
invisily_result() {
    invisily_log Result "$@" >&2
}
invisily_error() {
    invisily_log ERROR "$@" >&2
    invisily_log ERROR "Exiting..."  >&2
    exit 1
}

if [ "$EUID" -ne 0 ]; then
    invisily_error $'Please run as root.'
fi

################################################################################
# Initial necessary verification                                               #
################################################################################
if [ -f /etc/lsb-release ] || [ -d /etc/lsb-release.d ]; then
    ubuntuVersion=$(lsb_release -r | cut -d: -f2 | sed s/'^\t'//)
    if [[ "$ubuntuVersion" != "20.04" && "$ubuntuVersion" != "22.04" ]]; then
        invisily_error "Supported Operating system is Ubuntu 20.04, but it appears you are not running Ubuntu 20.04"
    fi
else
    invisily_error "Supported Operating system is Ubuntu 20.04, but it appears you are not running Ubuntu 20.04"
fi
if [ `(dpkg-query -W -f='${Status}' ubuntu-desktop 2>/dev/null | grep -c "ok installed")` -eq 1 ]; then
    invisily_error "Ubuntu 20.04 Server is required, but it appears that you are running Ubuntu Desktop"
fi

if [ $# -eq 0 ]; then
    invisily_error $'No arguments supplied. Please use -h or --help to get help'
fi

################################################################################
# Help                                                                         #
################################################################################
Help()
{
    # Display Help
    echo -e "Usage: $0 [-h] [--help] [--install] [--remove] [--reconfigure] [--client-username] [--client-password]"
    echo -e ""
    echo -e "Help: $0 -h"
    echo -e "      OR"
    echo -e "      $0 --help"
    echo -e
    echo -e "Options:"
    echo -e "-h | --help                            Print help."
    echo -e "--install                              Install Invisily SAAS Bridge"
    echo -e "--remove                               Remove Invisily SAAS Bridge"
    echo -e "--reconfigure                          Reconfigure Invisily SAAS Bridge"
    echo -e "--client-username                      Invisily client username"
    echo -e "--client-password                      Invisily client password"
    echo -e ""
}
################################################################################
################################################################################
# Main program                                                                 #
################################################################################
################################################################################
for i in "$@"
do
case $i in
    --install)
    INSTALL=1
    shift # past argument=value
    ;;
    --client-username=*)
    CLIENT_USERNAME="${i#*=}"
    shift # past argument=value
    ;;
    --client-password=*)
    CLIENT_PASSWORD="${i#*=}"
    shift # past argument=value
    ;;
    --reconfigure)
    RECONFIG=1
    shift # past argument=value
    ;;
    --remove)
    REMOVE=1
    shift # past argument=value
    ;;
    -h|--help)
    Help
    exit 1
    ;;
    *)
    invisily_error "Unknown option. Exiting..."
    ;;
esac
done

arg_verification(){
    if [ "$INSTALL" == "1" ] && [ "$REMOVE" == "1" ]; then
        invisily_error $'You cannot select --install and --remove at the same time'
    fi
    if [[ "$INSTALL" == "1" ]] && [[ -z "$CLIENT_USERNAME" ]]; then
        invisily_error $'You have selected --install but did not pass client username, please pass it using --client-username'
    fi
    if [[ "$INSTALL" == "1" ]] && [[ -z "$CLIENT_PASSWORD" ]]; then
        invisily_error $'You have selected --install but did not pass client password, please pass it using --client-password'
    fi
}

################################################################################
# Dependency installation functions                                            #
################################################################################
apt_dependency_install(){
    if [ $# -eq 0 ]; then
        invisily_error $'No arguments supplied. Please use give apt package name in argument'
    elif [ $# -gt 1 ]; then
        APT_DEPENDENCIES=("$@")
        INSTALLABLE_APT_DEPENDENCIES=()
        UNINSTALLABLE_APT_DEPENDENCIES=()
        for dependency in "${APT_DEPENDENCIES[@]}"; do
            if [ "$(! dpkg-query -W -f='${Status}' "$dependency" 2>/dev/null | grep -c "ok installed")" -eq 0 ]; then
                invisily_note "Not found $dependency. Adding to installables..."
                INSTALLABLE_APT_DEPENDENCIES+=("$dependency")
            else
                invisily_note "Found $dependency. Adding to uninstallables..."
                UNINSTALLABLE_APT_DEPENDENCIES+=("$dependency")
            fi
        done

        INSTALLABLE_APT_DEPENDENCIES="${INSTALLABLE_APT_DEPENDENCIES[*]}"
        UNINSTALLABLE_APT_DEPENDENCIES="${UNINSTALLABLE_APT_DEPENDENCIES[*]}"
        if [ -n "$UNINSTALLABLE_APT_DEPENDENCIES" ]; then
            invisily_note "Following dependencies are already installed: $UNINSTALLABLE_APT_DEPENDENCIES"
        fi
        if [ -n "$INSTALLABLE_APT_DEPENDENCIES" ]; then
            invisily_note "Installable dependencies are: $INSTALLABLE_APT_DEPENDENCIES"
            invisily_note "Installing APT dependencies..."
            apt-get install --yes "$INSTALLABLE_APT_DEPENDENCIES"
            if [[ $? -gt 0 ]]; then
                invisily_error $'Installation failed for '"$INSTALLABLE_APT_DEPENDENCIES"'. Exiting...'
            else
                invisily_note "Installed $INSTALLABLE_APT_DEPENDENCIES."
            fi
        fi
    else
        dependency=$1
        if [ "$(! dpkg-query -W -f='${Status}' "$dependency" 2>/dev/null | grep -c "ok installed")" -eq 0 ]; then
            apt-get install --yes "$dependency"
            if [[ $? -gt 0 ]]; then
                invisily_error $'Installation failed for '"$dependency"'. Exiting...'
            else
                invisily_note "Installed $dependency."
            fi
        fi
    fi
}
apt_dependency_uninstall(){
    if [ $# -eq 0 ]; then
        invisily_error $'No arguments supplied. Please use give apt package name in argument'
    elif [ $# -gt 1 ]; then
        APT_DEPENDENCIES=($@)
        INSTALLABLE_APT_DEPENDENCIES=()
        UNINSTALLABLE_APT_DEPENDENCIES=()
        for dependency in "${APT_DEPENDENCIES[@]}"; do
            if [ $(! dpkg-query -W -f='${Status}' $dependency 2>/dev/null | grep -c "ok installed") -eq 0 ]; then
                invisily_note "Not found $dependency. Adding to installables..."
                INSTALLABLE_APT_DEPENDENCIES+=($dependency)
            else
                invisily_note "Found $dependency. Adding to uninstallables..."
                UNINSTALLABLE_APT_DEPENDENCIES+=($dependency)
            fi
        done

        INSTALLABLE_APT_DEPENDENCIES="${INSTALLABLE_APT_DEPENDENCIES[*]}"
        UNINSTALLABLE_APT_DEPENDENCIES="${UNINSTALLABLE_APT_DEPENDENCIES[*]}"
        if [ -n "$UNINSTALLABLE_APT_DEPENDENCIES" ]; then
            invisily_note "Following dependencies are already installed: $UNINSTALLABLE_APT_DEPENDENCIES , and going to uninstall..."
            apt-get purge --yes $UNINSTALLABLE_APT_DEPENDENCIES && invisily_note "Uninstalled $UNINSTALLABLE_APT_DEPENDENCIES." || invisily_error $'Uninstallation failed for '$UNINSTALLABLE_APT_DEPENDENCIES'. Exiting...'
            apt autoremove -y && invisily_note "Uninstalled $UNINSTALLABLE_APT_DEPENDENCIES." || invisily_error $'Uninstallation failed for '$UNINSTALLABLE_APT_DEPENDENCIES'. Exiting...'
        fi
        if [ -n "$INSTALLABLE_APT_DEPENDENCIES" ]; then
            invisily_note "Following dependencies are already not installed: $INSTALLABLE_APT_DEPENDENCIES"
        fi
    else
        dependency=$1
        if [ $(! dpkg-query -W -f='${Status}' $dependency 2>/dev/null | grep -c "ok installed") -ne 0 ]; then
            apt-get purge --yes "$dependency" && invisily_note "Uninstalled $dependency." || invisily_error $'Uninstallation failed for '$dependency'. Exiting...'
            apt autoremove -y && invisily_note "Uninstalled $dependency." || invisily_error $'Uninstallation failed for '$dependency'. Exiting...'
        else
            invisily_note "$dependency is already not installed on system"
        fi
    fi
}

apt_dependency_check(){
    dependency=$1
    if [ "$(! dpkg-query -W -f='${Status}' "$dependency" 2>/dev/null | grep -c "ok installed")" -eq 0 ]; then
        return 0
    else
        return 1
    fi
}

MY_PATH=$(dirname "$0")
MY_PATH=$(cd "$MY_PATH" && pwd)
INV_SRV_CLIENT="invisily-client"
INV_SRV_CLIENT_INSTALLER="installer.deb"
INV_SRV_CLIENT_INSTALLER_PATH="$MY_PATH/$INV_SRV_CLIENT_INSTALLER"
CONF_FILE_NAME="saas_bridge.conf"
GO_PKG="go1.22.2.linux-amd64.tar.gz"
SAAS_LOG_DIR="/opt/vouch-proxy/log"
SAAS_LOG_FILE="$SAAS_LOG_DIR/vouch-proxy.log"
JWT_KEY=$(openssl rand -base64 44 | tr -d "=+/" | tr -d "\n ")
VOUCH_SER_NAME="vouch-proxy.service"
HOST_SER_NAME="monitor-hosts.service"
CHECK_HOST_DIR="/opt/invisily/invisily-saas-bridge"

configure_hosts_watchdog(){
    echo "old_service_ip=$(getent hosts $Service | awk '{print $1}')" > $CHECK_HOST_DIR/.hosts_config
    sed -i "s|\$Service|$Service|g" $CHECK_HOST_DIR/monitor-hosts.sh
    sed -i "s|\$CHECK_HOST_DIR|$CHECK_HOST_DIR|g" $CHECK_HOST_DIR/monitor-hosts.sh
    systemctl start $HOST_SER_NAME && systemctl enable $HOST_SER_NAME || invisily_error $'Host file monitor service start failed. Exiting...'
}

saas_bridge_install(){

    [[ -f $MY_PATH/$CONF_FILE_NAME ]] && echo || invisily_error $'Apparently SAAS configuration file is not present in current directory.'
    [[ -f $INV_SRV_CLIENT_INSTALLER_PATH ]] && echo || invisily_error $'Invisily Client Installer is not present in current Directory.'
    cd $MY_PATH
    apt_dependency_check invisily-client && apt_dependency_install $INV_SRV_CLIENT_INSTALLER_PATH || invisily_error $'Invisily Client seems already installed on this system, please uninstall it first using argument --remove, Exiting...'
    apt_dependency_check nginx && apt_dependency_install nginx || invisily_warn $'nginx seems already installed on this system.'
    apt_dependency_check zip && apt_dependency_install zip || invisily_warn $'zip seems to be already installed on this system.'
    apt_dependency_check git && apt_dependency_install git || invisily_warn $'git seems to be already installed on this system.'
    wget https://golang.org/dl/${GO_PKG} &> /dev/null || invisily_error $'Golang package download Failed. Exiting....'
    git clone https://github.com/vouch/vouch-proxy.git || invisily_error $'Dowloading of vouch-proxy failed. Exiting...'
    tar -C /usr/local -xzf ${GO_PKG} || invisily_error $'Extraction of Golang package failed. Exiting....'
    grep -q "^export GOROOT" ~/.profile || echo "export GOROOT=/usr/local/go" >> ~/.profile
    grep -q "^export PATH=/usr/local/go/bin" ~/.profile || echo "export PATH=/usr/local/go/bin:$PATH" >> ~/.profile
    source $MY_PATH/$CONF_FILE_NAME || invisily_error $'Sourcing SAAS config file failed. Exiting....'
    source ~/.profile || invisily_error $'Sourcing profile file failed. Exiting......'
    RootDomain=`echo "$AuthAgentDomain" | cut -d"." -f2-`
    Service=`echo "$Service" | cut -d"/" -f3`
    MappedDomain=`echo "$MappedDomain" | cut -d"/" -f3`
    go version &> /dev/null && rm -rf $MY_PATH/${GO_PKG} || invisily_error $'Looks like Golang is not properly installed. Exiting...'
    mv $MY_PATH/vouch-proxy /opt || invisily_error $'copying vouch proxy dir to /opt failed. Exiting...'
    cd /opt/vouch-proxy
    go build && invisily_note $'vouch proxy moduled compiled successfully' || invisily_error $'Failure in compiling vouch proxy go module. Exiting...'
    cd $MY_PATH
    mkdir -p $SAAS_LOG_DIR
    mkdir -p $CHECK_HOST_DIR
    [[ ! -f "$SAAS_LOG_FILE" ]] && touch $SAAS_LOG_FILE

    cat > /etc/systemd/system/$VOUCH_SER_NAME<<END
[Unit]
Description=Vouch Proxy
After=network.target

[Service]
Type=simple
User=root
WorkingDirectory=/opt/vouch-proxy
ExecStart=/opt/vouch-proxy/vouch-proxy
Restart=on-failure
RestartSec=5
StartLimitInterval=60s
StartLimitBurst=3
StandardOutput=file:$SAAS_LOG_FILE
StandardError=file:$SAAS_LOG_FILE

[Install]
WantedBy=default.target
END

# Generate SSL certificate to be used in nginx conf file of service

    mkdir -p /opt/ssl/$Service
    openssl req -x509 -nodes -days 365 -newkey rsa:2048 -subj "/C=US/ST=CA/O=Invisily, Inc./CN=$Service" \
-keyout /opt/ssl/$Service/${Service}.key \
-out /opt/ssl/$Service/${Service}.crt

cat > /opt/vouch-proxy/config/config.yml<<END
vouch:
  # logLevel: debug # VOUCH_LOGLEVEL
  logLevel: debug
  testing: false
  listen: 0.0.0.0  # VOUCH_LISTEN
  port: 9090       # VOUCH_PORT

  domains:
  - $RootDomain

  jwt:
    secret: $JWT_KEY
    issuer: Vouch # VOUCH_JWT_ISSUER
    maxAge: 240
    compress: true

  cookie:
    name: VouchCookie
    secure: true
    httpOnly: false # VOUCH_COOKIE_HTTPONLY
    maxAge: 14400

oauth:
  # Generic OpenID Connect
  provider: oidc
  client_id: $AuthenticationServiceClientID
  client_secret: $AuthenticationServiceClientSecret
  auth_url: $AuthenticationServiceURL
  token_url: $AuthenticationServiceTokenURL
  user_info_url: $AuthenticationServiceUserInfoURL
  scopes:
    - openid
    - email
    - profile
  callback_url: https://$AuthAgentDomain/auth
END

cat > /etc/nginx/sites-available/$Service<<END
upstream service {
  server $Service:$ServicePort fail_timeout=0;
}

server {
  listen 80;
  server_name $MappedDomain;
  return 301 https://\$host\$request_uri;
}

server {
  listen 443 ssl;
  server_name $MappedDomain;

  ssl_certificate /opt/ssl/$Service/$Service.crt;
  ssl_certificate_key /opt/ssl/$Service/$Service.key;
  ssl_prefer_server_ciphers on;

  auth_request /validate;

  location = /validate {
    proxy_pass http://127.0.0.1:9090/validate;
    proxy_set_header Host \$http_host;
    proxy_pass_request_body off; # no need to send the POST body
    proxy_set_header Content-Length "";
    auth_request_set \$auth_resp_x_vouch_user \$upstream_http_x_vouch_user;
    auth_request_set \$auth_resp_jwt \$upstream_http_x_vouch_jwt;
    auth_request_set \$auth_resp_err \$upstream_http_x_vouch_err;
    auth_request_set \$auth_resp_failcount \$upstream_http_x_vouch_failcount;
  }

  error_page 401 = @error401;

  location @error401 {
    return 302 https://$AuthAgentDomain/login?url=https://\$http_host\$request_uri&vouch-failcount=\$auth_resp_failcount&X-Vouch-Token=\$auth_resp_jwt&error=\$auth_resp_err;
  }

  location / {
    proxy_pass              http://service;
    proxy_set_header X-Vouch-User \$auth_resp_x_vouch_user;

  }
}
END
    [[ -f "/etc/nginx/sites-available/$Service" ]] && ln -s /etc/nginx/sites-available/$Service /etc/nginx/sites-enabled/

# Generate SSL certificate to be used in nginx conf file of vouch proxy
    mkdir -p /opt/ssl/$AuthAgentDomain
    openssl req -x509 -nodes -days 365 -newkey rsa:2048 -subj "/C=US/ST=CA/O=Invisily, Inc./CN=$AuthAgentDomain" \
-keyout /opt/ssl/$AuthAgentDomain/$AuthAgentDomain.key \
-out /opt/ssl/$AuthAgentDomain/$AuthAgentDomain.crt

cat > /etc/nginx/sites-available/$AuthAgentDomain<<END
server {
  listen 443 ssl;
  server_name $AuthAgentDomain;

  ssl_certificate /opt/ssl/$AuthAgentDomain/$AuthAgentDomain.crt;
  ssl_certificate_key /opt/ssl/$AuthAgentDomain/$AuthAgentDomain.key;

  # Proxy to your Vouch instance
  location / {
    proxy_set_header  Host  $AuthAgentDomain;
    proxy_pass        http://127.0.0.1:9090;
  }
}
END

cat > /etc/systemd/system/$HOST_SER_NAME<<END
[Unit]
Description=Monitor Hosts File
After=network.target

[Service]
Type=simple
User=root
WorkingDirectory=$CHECK_HOST_DIR
ExecStart=$CHECK_HOST_DIR/monitor-hosts.sh
Restart=on-failure
RestartSec=20
StartLimitInterval=60s
StartLimitBurst=3

[Install]
WantedBy=default.target
END

cat > $CHECK_HOST_DIR/monitor-hosts.sh<<'END'
#!/bin/bash

set -eo pipefail
shopt -s nullglob

MY_PATH=$(dirname "$0")
MY_PATH=$(cd "$MY_PATH" && pwd)
CHECK_HOST_LOG_DIR="/var/log/invisily/invisily-saas-bridge"
CHECK_HOST_LOG_FILE="$CHECK_HOST_LOG_DIR/monitor-hosts.log"

if [ "$EUID" -ne 0 ]; then
    invisily_error $'Please run as root.'
fi

mkdir -p $CHECK_HOST_LOG_DIR
[ ! -f "$CHECK_HOST_LOG_FILE" ] && touch $CHECK_HOST_LOG_FILE
DATE=`date '+%Y-%m-%d%H:%M:%S'`
script_name=`basename $0`
wc=`ps -ef | grep -v grep | grep -v nano | grep $script_name | wc -l`
if [ $wc -gt 2 ]; then
  echo "$DATE - Instance of watchdog is already running. Please check..." >> $CHECK_HOST_LOG_FILE
  exit 0
fi

################################################################################
# Logging functions                                                            #
################################################################################
invisily_log() {
    local type="$1"; shift
    if [ "$type" == "ERROR" ]; then
        printf '\033[0;31m%s [%s] [%s]: %s\033[0m\n' "$(date +"%Y-%m-%d %H:%M:%S%z")" "$type" "$0" "$*" | tee -a "$CHECK_HOST_LOG_FILE"
    elif [ "$type" == "Warn" ]; then
        printf '\033[0;34m%s [%s] [%s]: %s\033[0m\n' "$(date +"%Y-%m-%d %H:%M:%S%z")" "$type" "$0" "$*" | tee -a "$CHECK_HOST_LOG_FILE"
    elif [ "$type" == "Result" ]; then
        printf '\033[0;34m%s\033[0m\n' "$*" | tee -a "$CHECK_HOST_LOG_FILE"
    else
        printf '%s [%s] [%s]: %s\n' "$(date +"%Y-%m-%d %H:%M:%S%z")" "$type" "$0" "$*" | tee -a "$CHECK_HOST_LOG_FILE"
    fi
}
invisily_note() {
    invisily_log Note "$@"
}
invisily_warn() {
    invisily_log Warn "$@" >&2
}
invisily_result() {
    invisily_log Result "$@" >&2
}
invisily_error() {
    invisily_log ERROR "$@" >&2
    invisily_log ERROR "Exiting..."  >&2
    exit 1
}

cd $MY_PATH

monitor_hosts_file(){

    new_service_ip=`getent hosts "$Service" | awk '{print $1}'` || { invisily_warn $"${DATE} - Looks like client is not running. The script will again try after 10 seconds..."; return; }
    [ -f "$CHECK_HOST_DIR/.hosts_config" ] && source $CHECK_HOST_DIR/.hosts_config || { invisily_warn $"${DATE} - <${CHECK_HOST_DIR}/.hosts_config> File not present. Unable to compare old and new IPs...."; return; }
    if [[ "$new_service_ip" != "$old_service_ip" ]]; then
        invisily_log $"${DATE} - service ip changed. Restarting nginx service....."
        echo "old_service_ip=`getent hosts $Service | awk '{print $1}'`" > $CHECK_HOST_DIR/.hosts_config
        systemctl restart nginx.service
    fi
}

while true; do
    monitor_hosts_file
    sleep 10
done
END


    [[ -f "/etc/nginx/sites-available/$AuthAgentDomain" ]] && ln -s /etc/nginx/sites-available/$AuthAgentDomain /etc/nginx/sites-enabled/

    #Start of different components

    cd /opt/invisily/linuxclientd/bin/
    chmod +x $CHECK_HOST_DIR/monitor-hosts.sh
    sudo ./invisily_backend -u $CLIENT_USERNAME -p $CLIENT_PASSWORD || invisily_error $'invisily_backend service not started properly. Exiting....'
    sudo systemctl start invisily_backend.service || invisily_error $'invisily_backend systemd service not started properly. Exiting....'
    sleep 2m
    grep -q "$Service" /etc/hosts || invisily_error $'Looks like client is not started yet, please start the client service and pass <--reconfigure> for further steps.'
    systemctl daemon-reload
    configure_hosts_watchdog
    systemctl start $VOUCH_SER_NAME && systemctl enable $VOUCH_SER_NAME || invisily_error $'vouch proxy service start failed. Exiting...'
    nginx -t || invisily_error $'nginx conf test failed. Exiting...'
    systemctl restart nginx || invisily_error $'nginx restart failed. Exiting...'

}

saas_bridge_reconfigure(){

    invisily_note $'Re-configuring SAAS Bridge......'
    source $MY_PATH/$CONF_FILE_NAME || invisily_error $'Sourcing SAAS config file failed. Exiting....'
    Service=$(echo "$Service" | cut -d"/" -f3)
    grep -q "$Service" /etc/hosts || invisily_error $'Looks like client is not started yet, please start the client service and pass <--reconfigure> for further steps.'
    systemctl daemon-reload
    configure_hosts_watchdog
    systemctl start $VOUCH_SER_NAME && systemctl enable $VOUCH_SER_NAME || invisily_error $'vouch proxy service start failed. Exiting...'
    nginx -t || invisily_error $'nginx conf test failed. Exiting...'
    systemctl restart nginx || invisily_error $'nginx restart failed. Exiting...'

}

saas_bridge_remove(){

    invisily_note $'Uninstalling SAAS Bridge......'
    systemctl stop $VOUCH_SER_NAME  || invisily_warn $''$VOUCH_SER_NAME ' service not found'
    systemctl stop $HOST_SER_NAME  || invisily_warn $''$HOST_SER_NAME ' service not found'
    systemctl stop invisily_backend  || invisily_warn $'invisilidy service not found'
    systemctl stop nginx || invisily_warn $'nginx service cannot found'
    apt_dependency_check $INV_SRV_CLIENT || apt_dependency_uninstall $INV_SRV_CLIENT
    apt_dependency_check nginx || apt_dependency_uninstall nginx nginx-common
    rm -rf /opt/invisily || invisily_warn $'/opt/invisily could not be deleted!'
    rm -rf /var/log/invisily || invisily_warn $'/var/log/invisily could not be deleted!'
    rm -rf /opt/invisily_updates || invisily_warn $'/opt/invisily_updates could not be deleted!'
    rm -rf /opt/ssl || invisily_warn $'/opt/ssl could not be deleted!'
    rm -rf /opt/vouch-proxy || invisily_warn $'/opt/vouch-proxy could not be deleted!'
    rm -rf /usr/local/go || invisily_warn $'/usr/local/go could not be deleted!'
    [[ -d "/etc/nginx" ]] && rm -rf /etc/nginx
    [[ -f "/etc/systemd/system/$VOUCH_SER_NAME" ]] && rm -rf /etc/systemd/system/$VOUCH_SER_NAME
    [[ -f "/etc/systemd/system/$HOST_SER_NAME" ]] && rm -rf /etc/systemd/system/$HOST_SER_NAME
}

arg_verification
if [[ "$INSTALL" == "1" ]]; then
    saas_bridge_install
elif [[ "$REMOVE" == "1" ]]; then
    saas_bridge_remove
elif [[ "$RECONFIG" == "1" ]]; then
    saas_bridge_reconfigure
else
    invisily_error $'You did not provide any argument to any process'
fi