#!/bin/bash

set -eo pipefail
shopt -s nullglob
MY_PATH=$(dirname "$0")
MY_PATH=$(cd "$MY_PATH" && pwd)
[[ -f "$MY_PATH"/.result ]] && rm -rf "$MY_PATH"/.result
################################################################################
# Logging functions                                                            #
################################################################################
invisily_log() {
    local type="$1"; shift
    if [ "$type" == "ERROR" ]; then
        printf '\033[0;31m%s [%s] [%s]: %s\033[0m\n' "$(date +"%Y-%m-%d %H:%M:%S%z")" "$type" "$0" "$*"
    elif [ "$type" == "Warn" ]; then
        printf '\033[0;34m%s [%s] [%s]: %s\033[0m\n' "$(date +"%Y-%m-%d %H:%M:%S%z")" "$type" "$0" "$*"
    elif [ "$type" == "Result" ]; then
        printf '\033[0;34m%s\033[0m\n' "$*"
    else
        printf '%s [%s] [%s]: %s\n' "$(date +"%Y-%m-%d %H:%M:%S%z")" "$type" "$0" "$*"
    fi
}
invisily_note() {
    invisily_log Note "$@"
}
invisily_warn() {
    invisily_log Warn "$@" >&2
}
invisily_result() {
    invisily_log Result "$@" >&2
}
invisily_error() {
    invisily_log ERROR "$@" >&2
    invisily_log ERROR "Exiting..."  >&2
    exit 1
}

if [ "$EUID" -ne 0 ]; then
    invisily_error $'Please run as root.'
fi

################################################################################
# Initial necessary verification                                               #
################################################################################
if [ -f /etc/lsb-release ] || [ -d /etc/lsb-release.d ]; then
    ubuntuVersion=$(lsb_release -r | cut -d: -f2 | sed s/'^\t'//)
    if [[ "$ubuntuVersion" != "22.04" ]]; then
        invisily_error "Supported Operating system is Ubuntu 22.04, but it appears you are not running Ubuntu 22.04"
    fi
else
    invisily_error "Supported Operating system is Ubuntu 22.04, but it appears you are not running Ubuntu 22.04"
fi
if [ "$(dpkg-query -W -f='${Status}' ubuntu-desktop 2>/dev/null | grep -c "ok installed")" -eq 1 ]; then
    invisily_error "Ubuntu 22.04 Server is required, but it appears that you are running Ubuntu Desktop"
fi

if [ $# -eq 0 ]; then
    invisily_error $'No arguments supplied. Please use -h or --help to get help'
fi
################################################################################
# Validation functions                                                         #
################################################################################
cidr_to_netmask() {
    value=$(( 0xffffffff ^ ((1 << (32 - $1)) - 1) ))
    echo "$(( (value >> 24) & 0xff )).$(( (value >> 16) & 0xff )).$(( (value >> 8) & 0xff )).$(( value & 0xff ))"
}
__ip_validation() {
    ip=$1
    fail=0
    if expr "$ip" : '[0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*$' >/dev/null; then
        for i in 1 2 3 4; do
            if [ "$(echo "$ip" | cut -d. -f$i)" -gt 255 ]; then
            fail=1
            break
            fi
        done
    else
        fail=1
    fi
    if [ "$fail" == "1" ]; then
        invisily_error "IP Address ${ip} is not a valid IPv4 Address"  1>&2
        # exit 1
    fi
}

__cidr_validation() {
    if [ "$1" -eq "$1" ] 2>/dev/null
    then
        port=$1
        if (( $port < 8|| $port > 30 )) ; then
            invisily_error "CIDR $1 is not a valid CIDR" 1>&2
        fi
    else
        invisily_error "CIDR $1 is not a valid CIDR" 1>&2
    fi
}

__ip_calc(){
    ip=$1; mask=$2
    IFS=. read -r i1 i2 i3 i4 <<< "$ip"
    IFS=. read -r m1 m2 m3 m4 <<< "$mask"
    NETWORK=$((i1 & m1)).$((i2 & m2)).$((i3 & m3)).$((i4 & m4))
    # BROADCAST=$((i1 & m1 | 255-m1)).$((i2 & m2 | 255-m2)).$((i3 & m3 | 255-m3)).$((i4 & m4 | 255-m4))
    FIRST_IP=$((i1 & m1)).$((i2 & m2)).$((i3 & m3)).$(((i4 & m4)+1))
    LAST_IP=$((i1 & m1 | 255-m1)).$((i2 & m2 | 255-m2)).$((i3 & m3 | 255-m3)).$(((i4 & m4 | 255-m4)-1))
    FIFTIEH_IP=$((i1 & m1)).$((i2 & m2)).$((i3 & m3)).$(((i4 & m4)+50))
    HUNDREDITEH_IP=$((i1 & m1)).$((i2 & m2)).$((i3 & m3)).$(((i4 & m4)+100))
}

__ip_cidr_validation(){
    CIDR=$1
    CIDR_SUBNET=$(echo $CIDR | awk -F '/' '{print $1}')
    CIDR_CIDR=$(echo $CIDR | awk -F '/' '{print $2}')
    __ip_validation $CIDR_SUBNET
    __cidr_validation $CIDR_CIDR
    SUBNET_MASK=$(cidr_to_netmask $CIDR_CIDR)
    __ip_calc $CIDR_SUBNET $SUBNET_MASK
    DHCP_SUBNET=$NETWORK
    # DHCP_BROADCAST=$BROADCAST
    DHCP_FIRST_IP=$FIRST_IP
    DHCP_LAST_IP=$LAST_IP
    DHCP_LOW_RANGE=$FIFTIEH_IP
    DHCP_HIGH_RANGE=$HUNDREDITEH_IP
    DHCP_IP_RANGE=$DHCP_SUBNET/$CIDR_CIDR
    if [[ -n $DHCP_GW_IP ]]; then
        __gw_ip_validation $DHCP_GW_IP
    fi
}

int_ip() {
    OIFS=$IFS
    IFS='.'
    ip=($1)
    IFS=$OIFS
    echo "${ip[0]} * 256 ^ 3 + ${ip[1]} * 256 ^2 + ${ip[2]} * 256 ^1 + ${ip[3]} * 256 ^ 0" | bc
}
__gw_ip_validation(){
    if [[ -n $DHCP_FIRST_IP && -n $DHCP_LAST_IP ]]; then
        HOSTMIN=$(int_ip ${DHCP_FIRST_IP})
        HOSTMAX=$(int_ip ${DHCP_LAST_IP})
        IPINT=$(int_ip ${1})
        if ! [[ ${IPINT} -le ${HOSTMAX} && ${IPINT} -ge ${HOSTMIN} ]]; then
            invisily_error $'Gateway IP does not belong to the subnet supplied, please re-enter Correct Gateway IP using --dhcp-gw-ip, Exiting...'
        fi
    fi
}

_ip_list_validation(){
    IFS="," read -ra ip_list <<< "$1"
    for ip in "${ip_list[@]}"; do
        __ip_validation $ip
    done
}

################################################################################
# Help                                                                         #
################################################################################
Help()
{
    # Display Help
    echo -e "Usage: $0 [-h] [--help] [--install] [--remove] [--dhcp-ip-range=<xx.xxx.xxx.xxx/xx>] [--dhcp-gw-ip=<xxx.xxx.xxx.xxx>]"
    echo -e ""
    echo -e "Help: $0 -h"
    echo -e "      OR"
    echo -e "      $0 --help"
    echo -e
    echo -e "Options:"
    echo -e "-h | --help                                            Print help."
    echo -e "--install                                              Install Invisily Server Bridge"
    echo -e "--remove                                               Remove Invisily Server Bridge"
    echo -e "--dhcp-ip-range=xxx.xxx.xxx.xxx/xx                     Input Subnet/CIDR of DHCP Server, Ex. 192.168.0.0/24"
    echo -e "--dhcp-gw-ip=xxx.xxx.xxx.xxx                           Input Gateway IP Address, Ex. 192.168.0.1"
    echo -e "--kafka-servers-ips=xxx.xxx.xxx.xxx,xxx.xxx.xxx.xxx    Input Kafka server IPs, supports multiple with comma separated, Ex. 192.168.0.0,192.168.0.1"
    echo -e "--kafka-port=xxx                                       Input Kafka server port, Ex. 9092"
    echo -e "--kafka-topic=xxxxx                                    Input Kafka topic, Ex. invisily-profiler-topic"
    echo -e ""
}
################################################################################
################################################################################
# Main program                                                                 #
################################################################################
################################################################################
for i in "$@"
do
case $i in
    --install)
    INSTALL=1
    shift # past argument=value
    ;;
    --remove)
    REMOVE=1
    shift # past argument=value
    ;;
    --dhcp-ip-range=*)
    DHCP_IP_RANGE="${i#*=}"
    __ip_cidr_validation $DHCP_IP_RANGE
    shift # past argument=value
    ;;
    --dhcp-gw-ip*)
    DHCP_GW_IP="${i#*=}"
    __ip_validation $DHCP_GW_IP
    __gw_ip_validation $DHCP_GW_IP
    shift # past argument=value
    ;;
    --kafka-servers-ips*)
    KAFKA_SERVERS_IPS="${i#*=}"
    _ip_list_validation $KAFKA_SERVERS_IPS
    shift # past argument=value
    ;;
    --kafka-port*)
    KAFKA_PORT="${i#*=}"
    shift # past argument=value
    ;;
    --kafka-topic*)
    KAFKA_TOPIC="${i#*=}"
    shift # past argument=value
    ;;
    -h|--help)
    Help
    exit 1
    ;;
    *)
    invisily_error "Unknown option. Exiting..."
    ;;
esac
done

arg_verification(){
    if [ "$INSTALL" == "1" ] && [ "$REMOVE" == "1" ]; then
        invisily_error $'You cannot select --install and --remove at the same time'
    fi
    if [[ -z $DHCP_IP_RANGE ]] && [[ -n $DHCP_GW_IP ]]; then
        invisily_error $'You cannot specify DHCP Gateway IP without providing DHCP IP Range, please use --dhcp-ip-range to provide DHCP IP Range, Ex. 192.168.0.0/24'
    fi
    if [[ -z $DHCP_IP_RANGE ]]; then
        DHCP_IP_RANGE="192.168.0.0/24"
        __ip_cidr_validation $DHCP_IP_RANGE
    fi
    if [[ -z $DHCP_GW_IP ]]; then
        DHCP_GW_IP=$DHCP_FIRST_IP
    fi
    if [ -n "$KAFKA_SERVERS_IPS" ] && [ -z $KAFKA_PORT ]; then
        invisily_error $'Kafka server port is not specified when running the script'
    fi
    if [ -n "$KAFKA_SERVERS_IPS" ] && [ -z $KAFKA_TOPIC ]; then
        invisily_error $'Kafka topic for device profiler is not specified when running the script'
    fi
    if [ -n "$KAFKA_TOPIC" ] && [ -z $KAFKA_SERVERS_IPS ]; then
        invisily_error $'Kafka server IPs are not specified when running the script'
    fi
    if [ -n "$KAFKA_PORT" ] && [ -z $KAFKA_SERVERS_IPS ]; then
        invisily_error $'Kafka server IPs are not specified when running the script'
    fi
}

################################################################################
# Required variable declaration                                                #
################################################################################

MAIN_DIR="/opt/invisily/invisily-server-bridge"
CONF_DIR="$MAIN_DIR/conf"
# DOCKER_DAEMON_SCRIPT_NAME="install_invisily_server_bridge_docker_daemon.sh"
# DOCKER_DAEMON_SCRIPT="$MAIN_DIR/$DOCKER_DAEMON_SCRIPT_NAME"
WATCHDOG_FILE_NAME="invisily-server-bridge-watchdog.sh"
WATCHDOG_FILE_PATH="$MAIN_DIR/$WATCHDOG_FILE_NAME"
WATCHDOG_CRON_FILE="/etc/cron.d/invisily-watchdog"
LOG_DIR="/var/log/invisily/invisily-server-bridge"
SNIFFER_LOG_DIR="/var/log/invisily/sniffer"
SNIFFER_LOG_FILE="$SNIFFER_LOG_DIR/sniffer.log"
DOCKER_DAEMON_PID=/var/run/invisily-server-bridge-daemon.pid
SYSTEMD_PID=/var/run/invisily-server-bridge.pid
DOCKER_DAEMON_LOG_FILE="$LOG_DIR/invisily-server-bridge-daemon.log"
DOCKER_DAEMON_PNAME="invisily-server-bridge-daemon.sh"
DOCKER_DAEMON_PNAME_PATH="/opt/invisily/invisily-server-bridge"
DOCKER_DAEMON_FILE="$DOCKER_DAEMON_PNAME_PATH/$DOCKER_DAEMON_PNAME"
DOCKER_FILE="$MY_PATH"/Dockerfile
DOCKER_SUPPORTING_DIR="$MY_PATH"/supporting_files
DOCKER_SUPPORTING_SUPERVISORD_CONF=$DOCKER_SUPPORTING_DIR/supervisord.conf
DOCKER_SUPPORTING_SUPERVISORD_INVISILY_CONF=$DOCKER_SUPPORTING_DIR/supervisord-invisily.conf
DOCKER_SUPPORTING_SUPERVISORD_DNSMASQ_CONF=$DOCKER_SUPPORTING_DIR/supervisord-dnsmasq.conf
DOCKER_SUPPORTING_SUPERVISORD_SERVER_BRIDGE_CONF=$DOCKER_SUPPORTING_DIR/supervisord-server-bridge.conf
DOCKER_SUPPORTING_SUPERVISORD_INVISILY_SH=$DOCKER_SUPPORTING_DIR/supervisord-invisily.sh
DOCKER_SUPPORTING_SUPERVISORD_DNSMASQ_SH=$DOCKER_SUPPORTING_DIR/supervisord-dnsmasq.sh
DOCKER_SUPPORTING_SUPERVISORD_SERVER_BRIDGE_SH=$DOCKER_SUPPORTING_DIR/supervisord-server-bridge.sh
DOCKER_SUPPORTING_SUPERVISORD_SERVER_BRIDGE_CONTAINER_DEEMON_SH=$DOCKER_SUPPORTING_DIR/invisily-server-bridge-container-daemon.sh
DEVICE_PROFILER_INSTALLATION_DIR="/opt/invisily/device-profiler"
# NET_RULES_NAME="iptables_nat_rules.sh"
# NET_RULES="$PNAME_PATH/$NET_RULES_NAME"
# NET_RULES_CHECK_NAME="iptables_nat_rules_check.sh"
# NET_RULES_CHECK="$PNAME_PATH/$NET_RULES_CHECK_NAME"
# FORWARD_RULES_NAME="iptables_forward_rules.sh"  #new addition
# FORWARD_RULES="$PNAME_PATH/$FORWARD_RULES_NAME"  #new addition
# FORWARD_RULES_CHECK_NAME="iptables_forward_rules_check.sh"    #new addition
# FORWARD_RULES_CHECK="$PNAME_PATH/$FORWARD_RULES_CHECK_NAME"    #new addition
NET_SERVICE_FILE_NAME="invisily-server-bridge-service.sh"
NET_SERVICE_FILE="$DOCKER_DAEMON_PNAME_PATH/$NET_SERVICE_FILE_NAME"
LOG_ROTATE_FILE="/etc/logrotate.d/invisily-server-bridge"
SYSTEMD_SERVICE_NAME="invisily-server-bridge.service"
SYSTEMD_SERVICE_FILE="/etc/systemd/system/$SYSTEMD_SERVICE_NAME"
DEVICE_PROFILER_SYSTEMD_SERVICE_NAME="device-profiler.service"
DEVICE_PROFILER_SYSTEMD_SERVICE_FILE="/etc/systemd/system/$DEVICE_PROFILER_SYSTEMD_SERVICE_NAME"
# REFRESH_TIME=20
INV_SRV_CLIENT="invisily-client"
INV_SRV_CLIENT_INSTALLER="installer.deb"
INV_SRV_CLIENT_INSTALLER_PATH="$MY_PATH/$INV_SRV_CLIENT_INSTALLER"
DEVICE_BINDING_CONF_FILE_NAME="device_binding.conf"
# DEVICE_BINDING_CONF_FILE_FROM_GENERATED="/opt/invisily/linuxclientd/client_data/configs/$DEVICE_BINDING_CONF_FILE_NAME"
# DEVICE_BINDING_CONF_FILE_IN_DHCP="/etc/dhcp/$DEVICE_BINDING_CONF_FILE_NAME"
DEVICE_PROFILER_PKG="$MY_PATH/device-profiler.zip"
# SRV_JSON_FILE_NAME="services.json"
# INV_PROVIDED_SVC_JSON_FILE="/opt/invisily/linuxclientd/client_data/configs/$SRV_JSON_FILE_NAME"
# WEB_SVC_JSON_FILE="/var/www/html/$SRV_JSON_FILE_NAME"
################################################################################
# Dependency installation functions                                            #
################################################################################
apt_dependency_install(){
    if [ $# -eq 0 ]; then
        invisily_error $'No arguments supplied. Please use give apt package name in argument'
    elif [ $# -gt 1 ]; then
        APT_DEPENDENCIES=($@)
        INSTALLABLE_APT_DEPENDENCIES=()
        UNINSTALLABLE_APT_DEPENDENCIES=()
        for dependency in "${APT_DEPENDENCIES[@]}"; do
            if [ "$(! dpkg-query -W -f='${Status}' "$dependency" 2>/dev/null | grep -c "ok installed")" -eq 0 ]; then
                invisily_note "Not found $dependency. Adding to installables..."
                INSTALLABLE_APT_DEPENDENCIES+=($dependency)
            else
                invisily_note "Found $dependency. Adding to uninstallables..."
                UNINSTALLABLE_APT_DEPENDENCIES+=($dependency)
            fi
        done

        INSTALLABLE_APT_DEPENDENCIES=( "${INSTALLABLE_APT_DEPENDENCIES[*]}" )
        UNINSTALLABLE_APT_DEPENDENCIES=( "${UNINSTALLABLE_APT_DEPENDENCIES[*]}" )
        if [ ${#UNINSTALLABLE_APT_DEPENDENCIES[@]} -gt 0 ]; then
            invisily_note "Following dependencies are already installed: ${UNINSTALLABLE_APT_DEPENDENCIES[*]}"
        fi
        if [ ${#INSTALLABLE_APT_DEPENDENCIES[@]} -gt 0 ]; then
            invisily_note "Installable dependencies are: ${INSTALLABLE_APT_DEPENDENCIES[*]}"
            invisily_note "Installing APT dependencies..."
            if apt-get install --yes "${INSTALLABLE_APT_DEPENDENCIES[*]}"; then
                invisily_note "Installed ${INSTALLABLE_APT_DEPENDENCIES[*]}."
            else
                invisily_error $'Installation failed for '"${INSTALLABLE_APT_DEPENDENCIES[*]}"'. Exiting...'
            fi
        fi
    else
        dependency=$1
        if [ "$(! dpkg-query -W -f='${Status}' $dependency 2>/dev/null | grep -c "ok installed")" -eq 0 ]; then
            apt-get install --yes $dependency
            if [[ $? -gt 0 ]]; then
                invisily_error $'Installation failed for '$dependency'. Exiting...'
            else
                invisily_note "Installed $dependency."
            fi
        fi
    fi
}

apt_dependency_uninstall(){
    if [ $# -eq 0 ]; then
        invisily_error $'No arguments supplied. Please use give apt package name in argument'
    elif [ $# -gt 1 ]; then
        APT_DEPENDENCIES=($@)
        INSTALLABLE_APT_DEPENDENCIES=()
        UNINSTALLABLE_APT_DEPENDENCIES=()
        for dependency in "${APT_DEPENDENCIES[@]}"; do
            if [ "$(! dpkg-query -W -f='${Status}' "$dependency" 2>/dev/null | grep -c "ok installed")" -eq 0 ]; then
                invisily_note "Not found $dependency. Adding to installables..."
                INSTALLABLE_APT_DEPENDENCIES+=($dependency)
            else
                invisily_note "Found $dependency. Adding to uninstallables..."
                UNINSTALLABLE_APT_DEPENDENCIES+=($dependency)
            fi
        done

        INSTALLABLE_APT_DEPENDENCIES=( "${INSTALLABLE_APT_DEPENDENCIES[*]}" )
        UNINSTALLABLE_APT_DEPENDENCIES=( "${UNINSTALLABLE_APT_DEPENDENCIES[*]}" )
        if [ ${#UNINSTALLABLE_APT_DEPENDENCIES[@]} -gt 0 ]; then
            invisily_note "Following dependencies are already installed: ${UNINSTALLABLE_APT_DEPENDENCIES[*]} , and going to uninstall..."
            if apt-get purge --yes "${UNINSTALLABLE_APT_DEPENDENCIES[*]}"; then
                invisily_note "Uninstalled ${UNINSTALLABLE_APT_DEPENDENCIES[*]}."
            else
                invisily_error $'Uninstallation failed for '"${UNINSTALLABLE_APT_DEPENDENCIES[*]}"'. Exiting...'
            fi
            if apt autoremove -y; then
                invisily_note "Uninstalled ${UNINSTALLABLE_APT_DEPENDENCIES[*]}."
            else
                invisily_error $'Uninstallation failed for '"${UNINSTALLABLE_APT_DEPENDENCIES[*]}"'. Exiting...'
            fi
        fi
        if [ ${#INSTALLABLE_APT_DEPENDENCIES[@]} -gt 0 ]; then
            invisily_note "Following dependencies are already not installed: ${INSTALLABLE_APT_DEPENDENCIES[*]}"
        fi
    else
        dependency=$1
        if [ "$(! dpkg-query -W -f='${Status}' "$dependency" 2>/dev/null | grep -c "ok installed")" -ne 0 ]; then
            if apt-get purge --yes "$dependency"; then
                invisily_note "Uninstalled $dependency."
            else
                invisily_error $'Uninstallation failed for '"$dependency"'. Exiting...'
            fi
            if apt autoremove -y; then
                invisily_note "Uninstalled $dependency."
            else
                invisily_error $'Uninstallation failed for '"$dependency"'. Exiting...'
            fi
        else
            invisily_note "$dependency is already not installed on system"
        fi
    fi
}

apt_dependency_check(){
    dependency=$1
    if [ "$(! dpkg-query -W -f='${Status}' $dependency 2>/dev/null | grep -c "ok installed")" -eq 0 ]; then
        return 0
    else
        return 1
    fi
}

remote_server_tcp_port_connection_check() {
    REMOTE_IP_TO_CHECK=$1
    REMOTE_PORT_TO_CHECK=$2
    if timeout 1 bash -c 'echo &> /dev/null  < /dev/tcp/'"$REMOTE_IP_TO_CHECK"'/'"$REMOTE_PORT_TO_CHECK"; then
        return 0
    else
        return 1
    fi
}

device_profiler_install(){
    [[ -f $DEVICE_PROFILER_PKG ]] && echo || invisily_error $'Device Profiler Installer Package is not present in current Directory.'
    [[ ! -d $DEVICE_PROFILER_INSTALLATION_DIR ]] && mkdir -p $DEVICE_PROFILER_INSTALLATION_DIR
    unzip -o $DEVICE_PROFILER_PKG -d $DEVICE_PROFILER_INSTALLATION_DIR
    [[ ! -d  $SNIFFER_LOG_DIR ]]  && mkdir -p $SNIFFER_LOG_DIR
    [[ ! -f $SNIFFER_LOG_FILE ]] && touch $SNIFFER_LOG_FILE
    [[ ! -d  /opt/invisily/linuxclientd/client_data/configs/ ]]  && mkdir -p /opt/invisily/linuxclientd/client_data/configs/
    [[ ! -f /opt/invisily/linuxclientd/client_data/configs/dev_bridge.data ]] && touch /opt/invisily/linuxclientd/client_data/configs/dev_bridge.data
    #Creating configs.json
    cd $DEVICE_PROFILER_INSTALLATION_DIR
    [[ -f configs.json ]] && rm -rf configs.json
    BRIDGE_INTERFACE=$(ip route | grep $DHCP_GW_IP | awk -F"dev " 'NR==1{split($2,a," ");print a[1]}')

    IFS="," read -ra kafka_ip_list <<< "$KAFKA_SERVERS_IPS"

    for ip in "${kafka_ip_list[@]}"; do
        if ! remote_server_tcp_port_connection_check "$ip" "$KAFKA_PORT"; then
            invisily_error "Invalid Kafka server IP, not able to connect to kafka server."
        fi
    done

    kafka_ip_list=( "${kafka_ip_list[@]/%/:$KAFKA_PORT}" )
    JSON_KAFKA_SERVERS_IPS="["
    for ip in "${kafka_ip_list[@]}"; do
        JSON_KAFKA_SERVERS_IPS+="\"$ip\","
    done
    JSON_KAFKA_SERVERS_IPS="${JSON_KAFKA_SERVERS_IPS%,}"
    JSON_KAFKA_SERVERS_IPS+="]"
    if apt-get -y install python3 python3-pip python3-setuptools && python3 -m pip install -U pip && python3 -m pip install -U setuptools; then
        invisily_note "Successfully installed pip for python3"
    else
        invisily_error "Failed to install pip for python3"
    fi

cat > $DEVICE_PROFILER_INSTALLATION_DIR/configs.json <<END
{
    "bootstrap_servers": $JSON_KAFKA_SERVERS_IPS,
    "topic": "$KAFKA_TOPIC",
    "interface": "$BRIDGE_INTERFACE",
    "log_file": "$SNIFFER_LOG_FILE",
    "agent_bindings": "/opt/invisily/linuxclientd/client_data/configs/dev_bridge.data",
    "identifier": "src_mac"
}
END
    pip install -r requirements.txt

    cat > $DEVICE_PROFILER_SYSTEMD_SERVICE_FILE <<END
[Unit]
Description=hit service
After=network-online.target
[Service]
WorkingDirectory=$DEVICE_PROFILER_INSTALLATION_DIR
ExecStart=$DEVICE_PROFILER_INSTALLATION_DIR/sniffer
[Install]
WantedBy=multi-user.target
END

    systemctl daemon-reload
}

server_bridge_install(){
    ################################################################################
    # Initial variable declaration                                                 #
    ################################################################################
    DHCP_RANGE="$DHCP_IP_RANGE"
    # [[ -f $MY_PATH/$DEVICE_BINDING_CONF_FILE_NAME ]] && echo || invisily_error $'Apparently DEVICE Binding configuration file is not present in current directory.'
    [[ -f $INV_SRV_CLIENT_INSTALLER_PATH ]] && echo || invisily_error $'Invisily Client Installer is not present in current Directory.'
    # Enable net.ipv4.ip_forward for the system
    echo 'net.ipv4.ip_forward=1' > /etc/sysctl.d/30-invisily-server-bridge-forward.conf
    # Enable without waiting for a reboot or service restart
    echo 1 > /proc/sys/net/ipv4/ip_forward
    if ! ifconfig | grep $DHCP_GW_IP; then
        invisily_error $'DHCP Gateway IP '$DHCP_GW_IP ' is not configured on any interface in this system, Exiting...'
    fi
    if apt-get -y update; then
        invisily_note "OS repo cache updated successfully"
    else
        invisily_error "OS repo cache failed to update. Please make sure you have internet working inside your server. Please run apt-get -y update manually first and then re-run installation script"
    fi
    if apt_dependency_check invisily-client; then
        apt_dependency_install "$INV_SRV_CLIENT_INSTALLER_PATH"
    else
        invisily_error $'Invisily Client seems already installed on this system, please uninstall it first using argumen --remove, Exiting...'
    fi
    if apt_dependency_check docker.io; then
        apt_dependency_install docker.io
    else
        invisily_error $'Docker seems already installed on this system, please uninstall it first using argumen --remove, Exiting...'
    fi
    if apt_dependency_check jq; then
        apt_dependency_install jq
    else
        invisily_error $'jq already installed on this system, please uninstall it first using argumen --remove, Exiting...'
    fi
    if apt_dependency_check isc-dhcp-server; then
        apt_dependency_install isc-dhcp-server
    else
        invisily_error $'DHCP Server isc-dhcp-server seems already installed on this system, please uninstall it first using argumen --remove, Exiting...'
    fi


    [[ -f /etc/dhcp/dhcpd.conf ]] && cp -r /etc/dhcp/dhcpd.conf /etc/dhcp/dhcpd.conf.orig
    cat > /etc/dhcp/dhcpd.conf <<END
option domain-name "invisily.lab";
option domain-name-servers $DHCP_GW_IP;
default-lease-time 600;
max-lease-time 7200;
ddns-update-style none;
authoritative;
deny unknown-clients;

subnet $DHCP_SUBNET netmask $SUBNET_MASK {
range $DHCP_LOW_RANGE $DHCP_HIGH_RANGE;
option subnet-mask $SUBNET_MASK;
option routers $DHCP_GW_IP;
}
include "/etc/dhcp/$DEVICE_BINDING_CONF_FILE_NAME";
END
    # mv $MY_PATH/$DEVICE_BINDING_CONF_FILE_NAME /etc/dhcp/ && echo || invisily_error $'Moving configuration file to dhcp is failed, Exiting...'
    touch /etc/dhcp/"$DEVICE_BINDING_CONF_FILE_NAME"
    cd /etc/dhcp/
    if dhcpd -t -cf dhcpd.conf; then
        invisily_note $'DHCP configuration successfully verified...'
    else
        invisily_error $'DHCP configuration is corrupted, Exiting...'
    fi
    cd "$MY_PATH"
    systemctl restart isc-dhcp-server.service
    mkdir -p $MAIN_DIR $LOG_DIR $CONF_DIR
    touch $DOCKER_DAEMON_LOG_FILE

    cat > "$DOCKER_FILE"  <<END
FROM ubuntu:18.04

ENV container docker
ENV LC_ALL C
ENV DEBIAN_FRONTEND noninteractive

RUN sed -i 's/# deb/deb/g' /etc/apt/sources.list

RUN apt-get -y update \\
    && apt-get -y upgrade \\
    && apt-get install -y systemd systemd-sysv apt-utils \\
    sudo dmidecode iptables curl wget gnupg iputils-ping \\
    network-manager iproute2 supervisor zip net-tools \\
    net-tools lsb-release ntp libbsd-dev psmisc dnsmasq jq

# Add image configuration and scripts
ADD supporting_files/supervisord-invisily.sh /supervisord-invisily.sh
ADD supporting_files/supervisord-dnsmasq.sh /supervisord-dnsmasq.sh
ADD supporting_files/supervisord-server-bridge.sh /supervisord-server-bridge.sh
ADD supporting_files/invisily-server-bridge-container-daemon.sh /invisily-server-bridge-container-daemon.sh
RUN chmod 755 /*.sh
ADD supporting_files/supervisord-invisily.conf /etc/supervisor/conf.d/supervisord-invisily.conf
ADD supporting_files/supervisord-dnsmasq.conf /etc/supervisor/conf.d/supervisord-dnsmasq.conf
ADD supporting_files/supervisord-server-bridge.conf /etc/supervisor/conf.d/supervisord-server-bridge.conf
ADD supporting_files/supervisord.conf /etc/supervisor/supervisord.conf
COPY $INV_SRV_CLIENT_INSTALLER /root/
RUN apt-get install -y /root/$INV_SRV_CLIENT_INSTALLER
RUN rm -rf /root/$INV_SRV_CLIENT_INSTALLER


VOLUME [ "/invisily-client" ]

ENTRYPOINT ["supervisord", "-n"]
END
    mkdir -p "$DOCKER_SUPPORTING_DIR"
    cat > "$DOCKER_SUPPORTING_SUPERVISORD_CONF" <<END
[supervisord]
nodaemon=true
logfile = /var/log/supervisord.log
logfile_maxbytes = 10MB
logfile_backups = 10
loglevel = info

[include]
files=/etc/supervisor/conf.d/supervisord-invisily.conf  /etc/supervisor/conf.d/supervisord-dnsmasq.conf  /etc/supervisor/conf.d/supervisord-server-bridge.conf

; https://github.com/Supervisor/supervisor/issues/376#issuecomment-404385767

[unix_http_server]
file=/var/run/supervisor.sock
chmod=0700

[rpcinterface:supervisor]
supervisor.rpcinterface_factory = supervisor.rpcinterface:make_main_rpcinterface

[supervisorctl]
serverurl=unix:///var/run/supervisor.sock
END
    cat > "$DOCKER_SUPPORTING_SUPERVISORD_INVISILY_CONF" <<END
[program:invisily]
command=/supervisord-invisily.sh
priority=100
startsecs=0
autorestart=true
autostart=true
stopasgroup=true
killasgroup=true
stopsignal=INT
END
    cat > "$DOCKER_SUPPORTING_SUPERVISORD_DNSMASQ_CONF" <<END
[program:dnsmasq]
command=/supervisord-dnsmasq.sh
priority=200
startsecs=0
autorestart=true
autostart=true
stopasgroup=true
killasgroup=true
stopsignal=INT
END
    cat > "$DOCKER_SUPPORTING_SUPERVISORD_INVISILY_SH" <<END
#!/bin/bash
CLIENT_USERNAME="\$WORKER_AGENT_ID"_"\$MASTER_AGENT_ID"
CLIENT_PASSWORD=\$(openssl rand -base64 29 | tr -d "=+/" | cut -c1-14)
cd /opt/invisily/linuxclientd/bin
./invisily_backend -i \$DEVICE_UUID -u \$CLIENT_USERNAME -p \$CLIENT_PASSWORD -t slave_bridge -m background
END
    cat > "$DOCKER_SUPPORTING_SUPERVISORD_DNSMASQ_SH" <<END
#!/bin/bash
CONTAINER_IP=\$(ip addr show eth0 | grep "inet\b" | awk '{print \$2}' | cut -d/ -f1)
cat > /etc/dnsmasq.conf <<EOF
listen-address=::1,127.0.0.1,\$CONTAINER_IP
address=/services.inv/\$CONTAINER_IP
EOF
while [ -z "\$(ifconfig | grep inv)" ]; do
  echo "Waiting for Invisily to start ..."
  sleep 1
done
echo "Invisily started."
sleep 5
dnsmasq --no-daemon
END
    cat > "$DOCKER_SUPPORTING_SUPERVISORD_SERVER_BRIDGE_CONF" <<END
[program:serverbridge]
command=/supervisord-server-bridge.sh
priority=400
startsecs=0
autorestart=true
autostart=true
stopasgroup=true
killasgroup=true
stopsignal=INT
END
    cat > "$DOCKER_SUPPORTING_SUPERVISORD_SERVER_BRIDGE_SH" <<END
#!/bin/bash
cd /
/bin/bash /invisily-server-bridge-container-daemon.sh
END
    cat > "$DOCKER_SUPPORTING_SUPERVISORD_SERVER_BRIDGE_CONTAINER_DEEMON_SH" <<END
#!/bin/bash
PID=/var/run/invisily-server-bridge-container-daemon.pid
echo "My pid is: \$\$"
echo \$\$ > \$PID
finish=0
trap 'finish=1' SIGUSR1
LOG_FILE_PATH=/var/log/invisily/invisily-server-bridge
LOG_FILE=\$LOG_FILE_PATH/invisily-server-bridge-container-daemon.log
SERVER_BRIDGE_MAIN_DIR=/opt/invisily/invisily-server-bridge
NET_RULES=/opt/invisily/invisily-server-bridge/iptables_nat_rules.sh
NET_RULES_CHECK=/opt/invisily/invisily-server-bridge/iptables_nat_rules_check.sh
#INV_PROVIDED_SVC_JSON_FILE=/opt/invisily/linuxclientd/client_data/configs/services.json
INV_PROVIDED_SVC_JSON_FILE=/tmp/services.json
WEB_SVC_JSON_FILE=/var/www/html/services.json
DHCP_RANGE="$DHCP_RANGE"
DATE=\$(date +%Y-%m-%d-%H:%M:%S)
[[ ! -d \$LOG_FILE_PATH ]] && mkdir -p \$LOG_FILE_PATH
#####################################################
#### LOG FUNCTIONS ##################################
#####################################################
invisily_log() {
    local type="\$1"; shift
    if [ "\$type" == "ERROR" ]; then
        printf '\033[0;31m%s [%s] [%s]: %s\033[0m\n' "\$(date +"%Y-%m-%d %H:%M:%S%z")" "\$type" "\$0" "\$*" | tee -a \$LOG_FILE
    elif [ "\$type" == "Warn" ]; then
        printf '\033[0;34m%s [%s] [%s]: %s\033[0m\n' "\$(date +"%Y-%m-%d %H:%M:%S%z")" "\$type" "\$0" "\$*" | tee -a \$LOG_FILE
    elif [ "\$type" == "Result" ]; then
        printf '\033[0;34m%s\033[0m\n' "\$*" | tee -a \$LOG_FILE
    else
        printf '%s [%s] [%s]: %s\n' "\$(date +"%Y-%m-%d %H:%M:%S%z")" "\$type" "\$0" "\$*"  | tee -a \$LOG_FILE
    fi
}
invisily_note() {
    invisily_log Note "\$@"
}
invisily_warn() {
    invisily_log Warn "\$@" >&2
}
invisily_result() {
    invisily_log Result "\$@" >&2
}
invisily_error() {
    invisily_log ERROR "\$@" >&2
    invisily_log ERROR "Exiting..."  >&2
    exit 1
}
invisily_error_no_exit() {
    invisily_log ERROR "\$@" >&2
}
###############################################################

invisily_note "Bridge monitoring started/restarted"
rm -rf \$SERVER_BRIDGE_MAIN_DIR/routes.*
cidr_to_netmask() {
    value=\$(( 0xffffffff ^ ((1 << (32 - \$1)) - 1) ))
    echo "\$(( (value >> 24) & 0xff )).\$(( (value >> 16) & 0xff )).\$(( (value >> 8) & 0xff )).\$(( value & 0xff ))"
}
route_cidr_calculator() {
    CIDR=\$1
    CIDR_SUBNET=\$(echo "\$CIDR" | awk -F '/' '{print \$1}')
    CIDR_CIDR=\$(echo "\$CIDR" | awk -F '/' '{print \$2}')
    SUBNET_MASK=\$(cidr_to_netmask "\$CIDR_CIDR")
    IFS=. read -r i1 i2 i3 i4 <<< "\$CIDR_SUBNET"
    IFS=. read -r m1 m2 m3 m4 <<< "\$SUBNET_MASK"
    CIDR_NETWORK=\$((i1 & m1)).\$((i2 & m2)).\$((i3 & m3)).\$((i4 & m4))
    echo \$CIDR_NETWORK/"\$CIDR_CIDR"
}
sleep 1
while (( finish != 1 ))
do
    OLD_DATE=\$DATE
    DATE=\$(date +%Y-%m-%d-%H:%M:%S)
    ip -4 addr | grep inet | grep inv | grep -vE '127(\.[0-9]{1,3}){3}' > \$SERVER_BRIDGE_MAIN_DIR/routes."\$DATE"

    rm -rf \$NET_RULES_CHECK  \$NET_RULES
    ip -4 addr | grep inet | grep inv | grep -vE '127(\.[0-9]{1,3}){3}' |
    while IFS= read -r line
    do
        echo "\$line"
        ip_range=\$(echo "\$line" | awk -F ' ' '{print \$2}')
        ip_range=\$(route_cidr_calculator "\$ip_range")
        int_name=\$(echo "\$line" | awk -F ' ' '{print \$5}')
        echo "/sbin/iptables -t nat -A POSTROUTING -s \$DHCP_RANGE -d \$ip_range -o \$int_name -j MASQUERADE" >> \$NET_RULES
        echo "/sbin/iptables -t nat -C POSTROUTING -s \$DHCP_RANGE -d \$ip_range -o \$int_name -j MASQUERADE" >> \$NET_RULES_CHECK
    done

    if [[ -f \$NET_RULES_CHECK && -f \$NET_RULES ]]; then
        if [[ -s \$NET_RULES_CHECK ]]; then
            input="\$NET_RULES_CHECK"
            while IFS= read -r line
            do
                echo "\$line"
                if \$line; then
                    invisily_note "[ \$line ] iptables rules already exists"
                else
                    invisily_warn "[ \$line ] iptables rules did not exists"
                    invisily_note "Cleaning old NAT rules"
                    /sbin/iptables -t nat -F;
                    invisily_note "Setting up new NAT rules"
                    /bin/bash -x \$NET_RULES
                    invisily_note "New NAT rules are inserted to table"
                    if supervisorctl restart dnsmasq; then
                        invisily_note "DNS service has been restarted after NAT rules are changed"
                    else
                        invisily_error_no_exit "DNS service is failed to restart after NAT rules are changed"
                    fi
                fi
            done < "\$input"
        else
            invisily_warn "iptables files are empty"
            invisily_note "Cleaning NAT Rules"
            /sbin/iptables -t nat -F;
        fi
    else
        invisily_warn "iptables files do not exists"
        invisily_note "Cleaning NAT Rules"
        /sbin/iptables -t nat -F;
    fi

    ### Mac filtering code starterd
    if [[ -f \$SERVER_BRIDGE_MAIN_DIR/ip.addr ]]; then
        if [[ -s \$SERVER_BRIDGE_MAIN_DIR/ip.addr ]]; then
            IP_ADDR=\$(cat \$SERVER_BRIDGE_MAIN_DIR/ip.addr)
            if /sbin/iptables -C FORWARD -s "\$IP_ADDR"/32 -j ACCEPT; then
                invisily_note "[ /sbin/iptables -C FORWARD -s \$IP_ADDR/32 -j ACCEPT ] iptables forward rules already exists"
            else
                invisily_warn "[ /sbin/iptables -C FORWARD -s \$IP_ADDR/32 -j ACCEPT ] iptables forward rules did not exists"
                invisily_note "Cleaning old forward rules"
                /sbin/iptables -F FORWARD;
                invisily_note "Setting up new FORWARD rules"
                /sbin/iptables -A FORWARD -m state --state RELATED,ESTABLISHED -j ACCEPT
                /sbin/iptables -A FORWARD -s "\$IP_ADDR"/32 -j ACCEPT
                /sbin/iptables -P FORWARD DROP
                invisily_note "New FORWARD rules are inserted to table"
            fi
        else
            invisily_warn "ip addr file is empty"
            invisily_note "Rule for full subnet will be inserted"
            cmd="/sbin/iptables -C FORWARD -s \$DHCP_RANGE -j ACCEPT"
            if \$cmd; then
                invisily_note "[ \$cmd ] iptables forward rules for subnet already exists"
            else
                invisily_warn "[ \$cmd ] iptables forward rules for subnet did not exists"
                invisily_note "Cleaning old forward rules"
                /sbin/iptables -F FORWARD;
                invisily_note "Setting up new FORWARD rules"
                /sbin/iptables -A FORWARD -m state --state RELATED,ESTABLISHED -j ACCEPT
                /sbin/iptables -A FORWARD -s "\$DHCP_RANGE" -j ACCEPT
                /sbin/iptables -P FORWARD DROP
                invisily_note "New FORWARD rules for subnet are inserted to table"
            fi
        fi
    else
        invisily_warn "ip addr file doesnt exists"
        invisily_note "Rule for full subnet will be inserted"
        cmd="/sbin/iptables -C FORWARD -s \$DHCP_RANGE -j ACCEPT"
        if \$cmd; then
            invisily_note "[ \$cmd ] iptables forward rules for subnet already exists"
        else
            invisily_warn "[ \$cmd ] iptables forward rules for subnet did not exists"
            invisily_note "Cleaning old forward rules"
            /sbin/iptables -F FORWARD;
            invisily_note "Setting up new FORWARD rules"
            /sbin/iptables -A FORWARD -m state --state RELATED,ESTABLISHED -j ACCEPT
            /sbin/iptables -A FORWARD -s "\$DHCP_RANGE" -j ACCEPT
            /sbin/iptables -P FORWARD DROP
            invisily_note "New FORWARD rules for subnet are inserted to table"
        fi
    fi
    #services html file code
    # if curl -s -w "%{http_code}" -o \$INV_PROVIDED_SVC_JSON_FILE "http://localhost:9174/services" | grep -q "^2"; then
    #     CURRENT_FILE_MD5=\$(md5sum \$INV_PROVIDED_SVC_JSON_FILE | awk -F ' ' '{print \$1}')
    #     USED_FILE_MD5=\$(md5sum \$WEB_SVC_JSON_FILE | awk -F ' ' '{print \$1}')
    #     if [[ "\$CURRENT_FILE_MD5" != "\$USED_FILE_MD5" ]]; then
    #         invisily_note "\$DATE - Invisily provided Services JSON file is not matching with currently inused file, Updating it..."
    #         if cp -r \$INV_PROVIDED_SVC_JSON_FILE \$WEB_SVC_JSON_FILE; then
    #             invisily_note "Invisily provided Services JSON file is updated to web server location"
    #         else
    #             invisily_error "Invisily provided Services JSON file is failed to update to web server location"
    #         fi
    #     fi
    #     rm -rf \$INV_PROVIDED_SVC_JSON_FILE
    # else
    #     invisily_error "Curl request failed! Error: Failed to connect to services API url"
    # fi

    if [[ -f \$SERVER_BRIDGE_MAIN_DIR/routes.\$OLD_DATE ]]; then
        OLD_ROUTES_FILE_MD5=\$(md5sum \$SERVER_BRIDGE_MAIN_DIR/routes."\$OLD_DATE" | awk -F ' ' '{print \$1}')
        CUR_ROUTES_FILE_MD5=\$(md5sum \$SERVER_BRIDGE_MAIN_DIR/routes."\$DATE" | awk -F ' ' '{print \$1}')
        if [[ "\$CUR_ROUTES_FILE_MD5" != "\$OLD_ROUTES_FILE_MD5" ]]; then
            invisily_warn "Current routing file does not match with previous routing file, which means tunnels were re-established by Invisily client"
            invisily_note "Restarting Dnsmasq so that hostnames can resolve to new ips if required"
            if supervisorctl restart dnsmasq; then
                invisily_note "DNSMASQ service is successfully restarted"
            else
                invisily_error_no_exit "DNSMASQ service is failed to restart"
            fi
            if rm -rf \$SERVER_BRIDGE_MAIN_DIR/routes."\$OLD_DATE"; then
                invisily_note "\$SERVER_BRIDGE_MAIN_DIR/routes.\$OLD_DATE - file successfully deleted"
            else
                invisily_error_no_exit "\$SERVER_BRIDGE_MAIN_DIR/routes.\$OLD_DATE - file delete failed"
            fi
        else
            invisily_note "Old and current routing files are same"
            if rm -rf \$SERVER_BRIDGE_MAIN_DIR/routes."\$OLD_DATE"; then
                invisily_note "\$SERVER_BRIDGE_MAIN_DIR/routes.\$OLD_DATE - file successfully deleted"
            else
                invisily_error_no_exit "\$DATE - \$SERVER_BRIDGE_MAIN_DIR/routes.\$OLD_DATE - file delete failed"
            fi
        fi
    else
        invisily_warn "\$SERVER_BRIDGE_MAIN_DIR/routes.\$OLD_DATE - file does not exists, most probably its the first time this check is running"
    fi

    pids=\$(pidof invisily_backend)
    if [ "\$pids" == "" ]
    then
        if supervisorctl restart invisily; then
            invisily_note "Invisily service was not running and has been restarted!"
        else
            invisily_error_no_exit "Invisily service was not running, attempt to restart is failed"
        fi
    fi
    pids=\$(pidof dnsmasq)
    if [ "\$pids" == "" ]
    then
        if supervisorctl restart dnsmasq; then
            invisily_note "dnsmasq service was not running and has been restarted!"
        else
            invisily_error_no_exit "dnsmasq service was not running, attempt to restart is failed"
        fi
    fi
    REFRESH_TIME=\$((10 + RANDOM % 15))
    invisily_note "Recheck will now run after \$REFRESH_TIME seconds"
    sleep \$REFRESH_TIME
done
END
    cd "$MY_PATH"
    cat >> /etc/docker/daemon.json <<END
{
"log-driver": "journald",
"log-opts": {
"tag": "{{.Name}}"
},
"bip": "172.26.0.1/16"
}
END
    systemctl restart docker
    docker build -t invisily_client .


    cat > $DOCKER_DAEMON_FILE <<END
#!/bin/bash
PID=$DOCKER_DAEMON_PID
echo "My pid is: \$\$"
echo \$\$ > \$PID
finish=0
trap 'finish=1' SIGUSR1
LOG_FILE_PATH=/var/log/invisily/invisily-server-bridge
LOG_FILE=\$LOG_FILE_PATH/invisily-server-bridge-daemon.log
SERVER_BRIDGE_MAIN_DIR=/opt/invisily/invisily-server-bridge
BRIDGE_CLIENTS_INFO_FROM_INV=/opt/invisily/linuxclientd/client_data/configs/dev_bridge.data
# BRIDGE_CLIENTS_INFO_TO_BRIDGE="\$SERVER_BRIDGE_MAIN_DIR"/conf/dev_bridge.json
DOCKER_ACTIVE_ROUTES="\$SERVER_BRIDGE_MAIN_DIR"/conf/.activeroutes
DOCKER_SHARED_DIR="\$SERVER_BRIDGE_MAIN_DIR"/conf/share
DEVICE_BINDING_CONF_FILE_NAME="device_binding.conf"
DEVICE_BINDING_CONF_FILE_FROM_GENERATED="\$SERVER_BRIDGE_MAIN_DIR/conf/\$DEVICE_BINDING_CONF_FILE_NAME"
DEVICE_BINDING_CONF_FILE_IN_DHCP="/etc/dhcp/\$DEVICE_BINDING_CONF_FILE_NAME"
DHCP_RANGE="$DHCP_RANGE"
DATE=\$(date +%Y-%m-%d-%H:%M:%S)
[[ ! -d \$LOG_FILE_PATH ]] && mkdir -p \$LOG_FILE_PATH
[[ ! -f \$LOG_FILE ]] && touch \$LOG_FILE
[[ ! -d \$DOCKER_ACTIVE_ROUTES ]] && mkdir -p \$DOCKER_ACTIVE_ROUTES
[[ ! -d \$DOCKER_SHARED_DIR ]] && mkdir -p \$DOCKER_SHARED_DIR
#####################################################
#### LOG FUNCTIONS ##################################
#####################################################
invisily_log() {
    local type="\$1"; shift
    if [ "\$type" == "ERROR" ]; then
        printf '\033[0;31m%s [%s] [%s]: %s\033[0m\n' "\$(date +"%Y-%m-%d %H:%M:%S%z")" "\$type" "\$0" "\$*" | tee -a \$LOG_FILE
    elif [ "\$type" == "Warn" ]; then
        printf '\033[0;34m%s [%s] [%s]: %s\033[0m\n' "\$(date +"%Y-%m-%d %H:%M:%S%z")" "\$type" "\$0" "\$*" | tee -a \$LOG_FILE
    elif [ "\$type" == "Result" ]; then
        printf '\033[0;34m%s\033[0m\n' "\$*" | tee -a \$LOG_FILE
    else
        printf '%s [%s] [%s]: %s\n' "\$(date +"%Y-%m-%d %H:%M:%S%z")" "\$type" "\$0" "\$*"  | tee -a \$LOG_FILE
    fi
}
invisily_note() {
    invisily_log Note "\$@"
}
invisily_warn() {
    invisily_log Warn "\$@" >&2
}
invisily_result() {
    invisily_log Result "\$@" >&2
}
invisily_error() {
    invisily_log ERROR "\$@" >&2
    invisily_log ERROR "Exiting..."  >&2
    exit 1
}
invisily_error_no_exit() {
    invisily_log ERROR "\$@" >&2
}
###############################################################
cidr_to_netmask() {
    value=\$(( 0xffffffff ^ ((1 << (32 - \$1)) - 1) ))
    echo "\$(( (value >> 24) & 0xff )).\$(( (value >> 16) & 0xff )).\$(( (value >> 8) & 0xff )).\$(( value & 0xff ))"
}

docker_zoombie_containers_cleanup() {
    cmd=\$(docker ps -a -q | wc -l)
    if [[ \$cmd -ne 0 ]]; then
        invisily_note "Docker zoombie containers cleanup started"
        if docker stop "\$(docker ps -a -q)"; then
            invisily_note "Docker zoombie containers are successfully stopped"
        else
            invisily_note "Docker zoombie containers are failed to stopped"
        fi
        if docker rm "\$(docker ps -a -q)"; then
            invisily_note "Docker zoombie containers are successfully deleted"
        else
            invisily_note "Docker zoombie containers are failed to delete"
        fi
    else
        invisily_note "No docker container exists in the system"
    fi
}
route_cidr_calculator() {
    CIDR=\$1
    CIDR_SUBNET=\$(echo "\$CIDR" | awk -F '/' '{print \$1}')
    CIDR_CIDR=\$(echo "\$CIDR" | awk -F '/' '{print \$2}')
    SUBNET_MASK=\$(cidr_to_netmask "\$CIDR_CIDR")
    IFS=. read -r i1 i2 i3 i4 <<< "\$CIDR_SUBNET"
    IFS=. read -r m1 m2 m3 m4 <<< "\$SUBNET_MASK"
    CIDR_NETWORK=\$((i1 & m1)).\$((i2 & m2)).\$((i3 & m3)).\$((i4 & m4))
    echo \$CIDR_NETWORK/"\$CIDR_CIDR"
}

delete_container_func() {
    DOCKER_CONTAINER_TO_DELETE=\$1
    invisily_note "[Container ID \$DOCKER_CONTAINER_TO_DELETE ] stopping and deleting started"
    if docker stop "\$DOCKER_CONTAINER_TO_DELETE"; then
        invisily_note "[Container ID \$DOCKER_CONTAINER_TO_DELETE ] is stopped successfully"
    else
        invisily_warn "[Container ID \$DOCKER_CONTAINER_TO_DELETE ] is failed to stop"
    fi
    if docker rm "\$DOCKER_CONTAINER_TO_DELETE"; then
        invisily_note "[Container ID \$DOCKER_CONTAINER_TO_DELETE ] is deleted successfully"
    else
        invisily_warn "[Container ID \$DOCKER_CONTAINER_TO_DELETE ] is failed to delete"
    fi
    if [[ -d \$DOCKER_SHARED_DIR/\$DOCKER_CONTAINER_TO_DELETE ]]; then
        rm -rf "\${DOCKER_SHARED_DIR:?}/\$DOCKER_CONTAINER_TO_DELETE"
        invisily_note "[Container ID \$DOCKER_CONTAINER_TO_DELETE ] Shared directory of container is sucessfully deleted"
    else
        invisily_warn "[Container ID \$DOCKER_CONTAINER_TO_DELETE ] Deleting Shared directory of container is failed"
    fi
    if [[ -f "\$DOCKER_ACTIVE_ROUTES/\$DOCKER_CONTAINER_TO_DELETE.routes" ]]; then
        mv "\$DOCKER_ACTIVE_ROUTES/\$DOCKER_CONTAINER_TO_DELETE.routes" "\$DOCKER_ACTIVE_ROUTES/\$DOCKER_CONTAINER_TO_DELETE.routes.TOBEDELETED"
        invisily_note "[Container ID \$DOCKER_CONTAINER_TO_DELETE ] Routes for container are added to delete queue successfully"
    else
        invisily_warn "[Container ID \$DOCKER_CONTAINER_TO_DELETE ] Adding container routes to delete queue is failed"
    fi
}

check_docker_default_forward_rules() {
    if ! /sbin/iptables -C FORWARD -j DOCKER-USER; then
        /sbin/iptables -A FORWARD -j DOCKER-USER
    fi
    if ! /sbin/iptables -C FORWARD -j DOCKER-ISOLATION-STAGE-1; then
        /sbin/iptables -A FORWARD -j DOCKER-ISOLATION-STAGE-1
    fi
    if ! /sbin/iptables -C FORWARD -o docker0 -m conntrack --ctstate RELATED,ESTABLISHED -j ACCEPT; then
        /sbin/iptables -A FORWARD -o docker0 -m conntrack --ctstate RELATED,ESTABLISHED -j ACCEPT
    fi
    if ! /sbin/iptables -C FORWARD -o docker0 -j DOCKER; then
        /sbin/iptables -A FORWARD -o docker0 -j DOCKER
    fi
    if ! /sbin/iptables -C FORWARD -i docker0 ! -o docker0 -j ACCEPT; then
        /sbin/iptables -A FORWARD -i docker0 ! -o docker0 -j ACCEPT
    fi
    if ! /sbin/iptables -C FORWARD -i docker0 -o docker0 -j ACCEPT; then
        /sbin/iptables -A FORWARD -i docker0 -o docker0 -j ACCEPT
    fi
}
create_container_func() {
    WORKER_AGENT_ID=\$1
    NAME_OF_CONTAINER_TO_CREATE=\$WORKER_AGENT_ID
    DEVICE_ID_OF_CONTAINER_TO_CREATE=\$2
    MASTER_AGENT_ID=\$3
    CONTAINER_SHARED_DIR="\$DOCKER_SHARED_DIR/\$NAME_OF_CONTAINER_TO_CREATE"
    mkdir -p "\$CONTAINER_SHARED_DIR"
    if docker run -d --name "\$NAME_OF_CONTAINER_TO_CREATE" \\
                --privileged -e DEVICE_UUID="\$DEVICE_ID_OF_CONTAINER_TO_CREATE" \\
                -e MASTER_AGENT_ID="\$MASTER_AGENT_ID" \\
                -e WORKER_AGENT_ID="\$WORKER_AGENT_ID" \\
                -v "\$CONTAINER_SHARED_DIR":\$SERVER_BRIDGE_MAIN_DIR \\
                -v /sys/fs/cgroup:/sys/fs/cgroup:ro invisily_client; then
        invisily_note "[Container ID \$NAME_OF_CONTAINER_TO_CREATE ] Container created successfully"
    else
        invisily_error_no_exit "[Container ID \$NAME_OF_CONTAINER_TO_CREATE ] Container creation is failed"
    fi
}
delete_container_routes() {
    # deleing routes from TOBEDELETE files
    if ls "\$DOCKER_ACTIVE_ROUTES"/*.routes.TOBEDELETED; then
        invisily_note "Routing cleaning Started"
        for DOCKER_ROUTES_TO_BE_DELETE_FILE in "\$DOCKER_ACTIVE_ROUTES"/*.routes.TOBEDELETED
        do
            DOCKER_CONTAINER_TO_DELETE_ROUTE=\${DOCKER_ROUTES_TO_BE_DELETE_FILE##*/}
            DOCKER_CONTAINER_TO_DELETE_ROUTE=\${DOCKER_CONTAINER_TO_DELETE%.routes.TOBEDELETED}
            invisily_note "[Container ID \$DOCKER_CONTAINER_TO_DELETE_ROUTE] Deleting old unused routes"
            while IFS= read -r DOCKER_ROUTES_TO_BE_DELETE
            do
                ip_range=\$(echo "\$DOCKER_ROUTES_TO_BE_DELETE" | awk -F ' ' '{print \$2}')
                ip_range=\$(route_cidr_calculator "\$ip_range")
                cmd=\$(ip route show "\$ip_range" | wc -l)
                if [[ \$cmd -eq 0 ]]; then
                    invisily_note "[Container ID \$DOCKER_CONTAINER_TO_DELETE_ROUTE] ip route for \$ip_range does not exists"
                elif [[ \$cmd -eq 1 ]]; then
                    invisily_note "[Container ID \$DOCKER_CONTAINER_TO_DELETE_ROUTE] ip route for \$ip_range already exists"
                    invisily_note "[Container ID \$DOCKER_CONTAINER_TO_DELETE_ROUTE] Deleting ip route for \$ip_range"
                    if ip route del "\$ip_range"; then
                        invisily_note "[Container ID \$DOCKER_CONTAINER_TO_DELETE_ROUTE] ip route for \$ip_range is successfully deleted from the system"
                    else
                        invisily_note "[Container ID \$DOCKER_CONTAINER_TO_DELETE_ROUTE] Deleting ip route for \$ip_range from the system is failed"
                    fi
                fi
            done < "\$DOCKER_ROUTES_TO_BE_DELETE_FILE"
            rm -rf "\$DOCKER_ROUTES_TO_BE_DELETE_FILE"
        done
        invisily_note "[Container ID \$DOCKER_CONTAINER_TO_DELETE_ROUTE] Routing cleaning Ended"
    else
        invisily_note "No Routing entries to be cleaned"
    fi
}

handle_container_forward_rules() {
    if [[ "\$1" == "create" ]]; then
        while IFS= read -r container_mac_address
        do
            if /sbin/iptables -C FORWARD -s "\$DHCP_RANGE" -m mac --mac-source "\$container_mac_address" -j ACCEPT; then
                invisily_note "Iptables forward rule for \$container_mac_address already exists"
            else
                invisily_warn "Iptables forward rule for \$container_mac_address does not exists, inserting it"
                if /sbin/iptables -A FORWARD -s "\$DHCP_RANGE" -m mac --mac-source "\$container_mac_address" -j ACCEPT; then
                    invisily_note "Iptables forward rule for \$container_mac_address successfully inserted"
                else
                    invisily_error_no_exit "Failed to insert Iptables forward rule for \$container_mac_address"
                fi
            fi
        done < <(curl -s http://localhost/bridge/slaves | jq -r '.slaves[].mac')
        check_docker_default_forward_rules
        /sbin/iptables -P FORWARD DROP
    elif [[ "\$1" == "delete" ]]; then
        /sbin/iptables -F FORWARD
        check_docker_default_forward_rules
        /sbin/iptables -P FORWARD DROP
    else
        invisily_warn "Unknown argument is provided"
    fi
}

handle_dhcp_config() {
    #generating dhcp config
    cmd=\$(docker ps -q | wc -l)
    if [[ \$cmd -ne 0 ]]; then
        if rm -rf \$DEVICE_BINDING_CONF_FILE_FROM_GENERATED; then
            invisily_note "Last genereated DHCP config file is successfully deleted"
        else
            invisily_warn "Failed to delete last generated DHCP config file"
        fi
        docker ps -q | xargs -n 1 docker inspect --format '{{range .NetworkSettings.Networks}}{{.IPAddress}}{{end}} {{ .Name }}' | sed 's/ \// /' |
        while IFS= read -r DOCKER_CONTAINER_LIST_LINE
        do
            DOCKER_CONTAINER_IP=\$(echo "\$DOCKER_CONTAINER_LIST_LINE" | awk -F ' ' '{print \$1}')
            DOCKER_CONTAINER_ID=\$(echo "\$DOCKER_CONTAINER_LIST_LINE" | awk -F ' ' '{print \$2}')
            DOCKER_CONTAINER_UUID=\$(docker inspect "\$DOCKER_CONTAINER_ID" | grep DEVICE_UUID |  cut -d'"' -f 2 | awk -F '=' '{print \$2}')
            cat >> \$DEVICE_BINDING_CONF_FILE_FROM_GENERATED <<EOF
host \$DOCKER_CONTAINER_ID{
    option domain-name-servers \$DOCKER_CONTAINER_IP;
    hardware ethernet \$DOCKER_CONTAINER_UUID;
}
EOF
        done
        if [[ -f \$DEVICE_BINDING_CONF_FILE_FROM_GENERATED ]]; then
            invisily_note "\$DEVICE_BINDING_CONF_FILE_FROM_GENERATED file is successfully generated with DHCP configuration"
        else
            invisily_error_no_exit "\$DEVICE_BINDING_CONF_FILE_FROM_GENERATED does not exists after creation process, there may be some problem in generating DHCP configuration"
        fi
    else
        invisily_warn "No container is running in system to generate DHCP config"
    fi
    #comparing running dhcp config with generated dhcp config
    if [[ -f \$DEVICE_BINDING_CONF_FILE_FROM_GENERATED && -f \$DEVICE_BINDING_CONF_FILE_IN_DHCP ]]; then
        MD5_OF_DEVICE_BINDING_CONF_FILE_FROM_GENERATED=\$(md5sum \$DEVICE_BINDING_CONF_FILE_FROM_GENERATED | awk -F ' ' '{print \$1}')
        MD5_OF_DEVICE_BINDING_CONF_FILE_IN_DHCP=\$(md5sum \$DEVICE_BINDING_CONF_FILE_IN_DHCP | awk -F ' ' '{print \$1}')
        if [[ "\$MD5_OF_DEVICE_BINDING_CONF_FILE_FROM_GENERATED" != "\$MD5_OF_DEVICE_BINDING_CONF_FILE_IN_DHCP" ]]; then
            if cp -r \$DEVICE_BINDING_CONF_FILE_FROM_GENERATED \$DEVICE_BINDING_CONF_FILE_IN_DHCP; then
                invisily_note "Newly generated DHCP configuration is successlly copied to DHCP configuration directory"
            else
                invisily_error_no_exit "Newly generated DHCP configuration is failed to copy to DHCP configuration directory"
            fi
            cd /etc/dhcp/ || exit
            if dhcpd -t -cf dhcpd.conf; then
                invisily_note "DHCP configuration successfully verified..."
                if systemctl restart isc-dhcp-server.service; then
                    invisily_note "DHCP Service has been restarted successfully"
                else
                    invisily_error_no_exit "DHCP Service is failed to restart"
                fi
            else
                invisily_error_no_exit "DHCP configuration is corrupted"
            fi
        else
            invisily_note "DHCP generated config and running config are same"
        fi
    else
        invisily_warn "DHCP config generated or used file does not exists"
    fi
}

verification_of_container_routes_in_hosts() {
    cmd=\$(docker ps -q | wc -l)
    if [[ \$cmd -ne 0 ]]; then
        docker ps -q | xargs -n 1 docker inspect --format '{{range .NetworkSettings.Networks}}{{.IPAddress}}{{end}} {{ .Name }}' | sed 's/ \// /' |
        while IFS= read -r DOCKER_CONTAINER_LIST_LINE
        do
            DOCKER_CONTAINER_IP=\$(echo "\$DOCKER_CONTAINER_LIST_LINE" | awk -F ' ' '{print \$1}')
            DOCKER_CONTAINER_ID=\$(echo "\$DOCKER_CONTAINER_LIST_LINE" | awk -F ' ' '{print \$2}')
            if ls \$DOCKER_ACTIVE_ROUTES/"\$DOCKER_CONTAINER_ID".routes; then
                invisily_note "[Container ID \$DOCKER_CONTAINER_ID] Route file exists for Container"
                invisily_note "[Container ID \$DOCKER_CONTAINER_ID] Checking route file contains updated routes for Container"
                if ls \$DOCKER_SHARED_DIR/"\$DOCKER_CONTAINER_ID"/routes.*; then
                    CONTAINER_ROUTE_FILE=\$(ls \$DOCKER_SHARED_DIR/"\$DOCKER_CONTAINER_ID"/routes.*)
                    MD5_OF_CONTAINER_ROUTE_FILE=\$(md5sum "\$CONTAINER_ROUTE_FILE" | awk -F ' ' '{print \$1}')
                    MD5_OF_ACTIVE_ROUTE_FILE=\$(md5sum \$DOCKER_ACTIVE_ROUTES/"\$DOCKER_CONTAINER_ID".routes | awk -F ' ' '{print \$1}')
                    if [[ "\$MD5_OF_CONTAINER_ROUTE_FILE" == "\$MD5_OF_ACTIVE_ROUTE_FILE" ]]; then
                        invisily_note "[Container ID \$DOCKER_CONTAINER_ID] Active route and container routes are matching"
                    else
                        invisily_warn "[Container ID \$DOCKER_CONTAINER_ID] Active route and container routes are not matching"
                        invisily_note "[Container ID \$DOCKER_CONTAINER_ID] Copying container routes to active routes and adding existing routes to delete queue"
                        if mv \$DOCKER_ACTIVE_ROUTES/"\$DOCKER_CONTAINER_ID".routes \$DOCKER_ACTIVE_ROUTES/"\$DOCKER_CONTAINER_ID".routes.TOBEDELETED; then
                            invisily_note "[Container ID \$DOCKER_CONTAINER_ID] Existing routes successfully added to delete queue"
                        else
                            invisily_error_no_exit "[Container ID \$DOCKER_CONTAINER_ID] Existing routes failed to add to delete queue"
                        fi
                        if cp -rf "\$CONTAINER_ROUTE_FILE" \$DOCKER_ACTIVE_ROUTES/"\$DOCKER_CONTAINER_ID".routes; then
                            invisily_note "[Container ID \$DOCKER_CONTAINER_ID] Container routes successfully added to active routes"
                        else
                            invisily_error_no_exit "[Container ID \$DOCKER_CONTAINER_ID] Container routes failed to add to active routes"
                        fi
                    fi
                else
                    invisily_warn "[Container ID \$DOCKER_CONTAINER_ID] Container routing file of does not exists, perhaps Invisily client inside container is not running or there is no services assigned to invisily user running inside container"
                    invisily_note "[Container ID \$DOCKER_CONTAINER_ID] Marking active routes to be deleted"
                    if mv \$DOCKER_ACTIVE_ROUTES/"\$DOCKER_CONTAINER_ID".routes \$DOCKER_ACTIVE_ROUTES/"\$DOCKER_CONTAINER_ID".routes.TOBEDELETED; then
                        invisily_note "[Container ID \$DOCKER_CONTAINER_ID] Existing routes successfully added to delete queue"
                    else
                        invisily_error_no_exit "[Container ID \$DOCKER_CONTAINER_ID] Existing routes failed to add to delete queue"
                    fi
                fi
            else
                invisily_warn "[Container ID \$DOCKER_CONTAINER_ID] Route file does not exists"
                invisily_note "[Container ID \$DOCKER_CONTAINER_ID] Trying to find routing file"
                if ls \$DOCKER_SHARED_DIR/"\$DOCKER_CONTAINER_ID"/routes.*; then
                    invisily_note "[Container ID \$DOCKER_CONTAINER_ID] Routing file is found, copying it"
                    CONTAINER_ROUTE_FILE=\$(ls \$DOCKER_SHARED_DIR/"\$DOCKER_CONTAINER_ID"/routes.*)
                    if cp -rf "\$CONTAINER_ROUTE_FILE" \$DOCKER_ACTIVE_ROUTES/"\$DOCKER_CONTAINER_ID".routes; then
                        invisily_note "[Container ID \$DOCKER_CONTAINER_ID] Container routes successfully added to active routes"
                    else
                        invisily_error_no_exit "[Container ID \$DOCKER_CONTAINER_ID] Container routes failed to add to active routes"
                    fi
                else
                    invisily_warn "[Container ID \$DOCKER_CONTAINER_ID] Current routing file does not exists, perhaps Invisily client inside container is not running or there is no services assigned to invisily user running inside container"
                fi
            fi
            #end matching routing information of containers to invisily
        done
    else
        invisily_warn "No container is running in system to check routing"
    fi
}

active_routing_rule_check() {
    cmd=\$(docker ps -q | wc -l)
    if [[ \$cmd -ne 0 ]]; then
        invisily_note "Routing setup is Started"
        docker ps -q | xargs -n 1 docker inspect --format '{{range .NetworkSettings.Networks}}{{.IPAddress}}{{end}} {{ .Name }}' | sed 's/ \// /'  |
        while IFS= read -r DOCKER_CONTAINER_LIST_LINE
        do
            DOCKER_CONTAINER_IP=\$(echo "\$DOCKER_CONTAINER_LIST_LINE" | awk -F ' ' '{print \$1}')
            DOCKER_CONTAINER_ID=\$(echo "\$DOCKER_CONTAINER_LIST_LINE" | awk -F ' ' '{print \$2}')
            if [[ -f \$DOCKER_ACTIVE_ROUTES/\$DOCKER_CONTAINER_ID.routes ]]; then
                while IFS= read -r SINGLE_DOCKER_CONTAINER_LIST
                do
                    ip_range=\$(echo "\$SINGLE_DOCKER_CONTAINER_LIST" | awk -F ' ' '{print \$2}')
                    ip_range=\$(route_cidr_calculator "\$ip_range")
                    cmd=\$(ip route show "\$ip_range" via "\$DOCKER_CONTAINER_IP" dev docker0 | wc -l)
                    if [[ \$cmd -eq 0 ]]; then
                        invisily_warn "[Container ID \$DOCKER_CONTAINER_ID] ip route for \$ip_range via \$DOCKER_CONTAINER_IP  does not exists"
                        invisily_note "[Container ID \$DOCKER_CONTAINER_ID] Adding ip route for \$ip_range via \$DOCKER_CONTAINER_IP"
                        if ip route add "\$ip_range" via "\$DOCKER_CONTAINER_IP" dev docker0; then
                            invisily_note "[Container ID \$DOCKER_CONTAINER_ID] ip route for \$ip_range via \$DOCKER_CONTAINER_IP is successfully added to system"
                        else
                            invisily_error_no_exit "[Container ID \$DOCKER_CONTAINER_ID] Adding ip route for \$ip_range via \$DOCKER_CONTAINER_IP to system is failed"
                        fi
                    elif [[ \$cmd -eq 1 ]]; then
                        invisily_note "[Container ID \$DOCKER_CONTAINER_ID] ip route for \$ip_range via \$DOCKER_CONTAINER_IP  already exists"
                    fi
                done < "\$DOCKER_ACTIVE_ROUTES/\$DOCKER_CONTAINER_ID.routes"
            else
                invisily_note "[Container ID \$DOCKER_CONTAINER_ID] Active routes file for \$DOCKER_CONTAINER_ID does not exists"
            fi
        done
        invisily_note "Routing monitoring is completed"
    else
        invisily_warn "No container is running to monitor routing"
    fi
}

updating_bridge_info_received_from_controller() {
    if response=\$(curl -s "http://localhost/bridge/slaves"); then
        master_agent_id=\$(echo "\$response" | jq -r '.agent_id')

        # Store worker agent IDs and MAC addresses in associative arrays
        declare -A worker_agent_ids
        declare -A worker_mac_addresses

        while IFS= read -r agent; do
            worker_agent_id=\$(jq -r '.agent_id' <<< "\$agent")
            worker_mac_address=\$(jq -r '.mac' <<< "\$agent")
            worker_agent_ids["\$worker_agent_id"]=\$worker_mac_address
            worker_mac_addresses["\$worker_mac_address"]=\$worker_agent_id
        done < <(echo "\$response" | jq -c '.slaves[]')

        # Retrieve list of running Docker containers and compare environment variables
        docker ps -a -q | xargs -n 1 docker inspect --format '{{range .NetworkSettings.Networks}}{{.IPAddress}}{{end}} {{ .Name }}  {{.Config.Env}}' | sed 's/ \// /' | while read -r container_info; do
            container_ip=\$(awk '{print \$1}' <<< "\$container_info")
            container_name=\$(awk '{print \$2}' <<< "\$container_info")
            container_env=\$(awk '{\$1=\$2=""; print substr(\$0, 3)}' <<< "\$container_info")

            # Extract DEVICE_UUID and WORKER_AGENT_ID from container environment variables
            device_uuid=\$(grep -oP '(?<=DEVICE_UUID=)[^ ]+' <<< "\$container_env")
            worker_agent_id=\$(grep -oP '(?<=WORKER_AGENT_ID=)[^ ]+' <<< "\$container_env")

            # Check if worker_agent_id and device_uuid exist in the arrays
            if [[ \${worker_mac_addresses["\$device_uuid"]} == "\$worker_agent_id" && \${worker_agent_ids["\$worker_agent_id"]} == "\$device_uuid" ]]; then
                invisily_note "Container \$container_name matches required configuration. Agent ID: \$worker_agent_id, MAC: \$device_uuid"
            else
                invisily_war "Container \$container_name does not match required configuration. Agent ID: \$worker_agent_id, MAC: \$device_uuid"
                delete_container_func "\$worker_agent_id"
                delete_container_routes
                invisily_note "Workers information received from Invisily is not matching with currently inused file"
                invisily_note "Cleaning Forward rules for worker containers"
                handle_container_forward_rules delete
                invisily_note "Cleaning docker containers, and will re-spawn as per new information received from Controller"
        fi
        done
    else
        invisily_error_no_exit "Invisily Slave client information API is not responding, Invisily master client might not be running"
    fi
}

creation_of_running_containers() {
    if response=\$(curl -s  "http://localhost/bridge/slaves"); then
        master_agent_id=\$(echo "\$response" | jq -r '.agent_id')
        for agent in \$(echo "\$response" | jq -c '.slaves[]'); do
            worker_agent_id=\$(echo "\$agent" | jq -r '.agent_id')
            worker_mac_address=\$(echo "\$agent" | jq -r '.mac')
            if docker inspect "\$worker_agent_id" > /dev/null; then
                invisily_note "[Container ID \$worker_agent_id] exists already"
                invisily_note "[Container ID \$worker_agent_id] Checking if container is running or not"
                DOCKER_CONTIANER_STATUS=\$(docker inspect --format ' {{ .State.Status }}' "\$worker_agent_id" | cut -d ' ' -f 2)
                if [[ "\$DOCKER_CONTIANER_STATUS" == "running" ]]; then
                    invisily_note "[Container ID \$worker_agent_id] Container is in running state"
                else
                    invisily_error_no_exit "[Container ID \$worker_agent_id] Container is not in running state"
                    invisily_note "[Container ID \$worker_agent_id] deleteing container and recreating it"
                    delete_container_func "\$worker_agent_id"
                    delete_container_routes
                    invisily_note "[Container ID \$worker_agent_id] Trying to create new container"
                    create_container_func "\$worker_agent_id" "\$worker_mac_address" "\$master_agent_id"
                fi
            else
                invisily_warn "[Container ID \$worker_agent_id] does not exists, creating it"
                create_container_func "\$worker_agent_id" "\$worker_mac_address" "\$master_agent_id"
            fi
        done
    else
        invisily_error_no_exit "Connection to Worker information API is failed!"
        invisily_note "Cleaining all running containers, routes and forwarding rules"
        handle_container_forward_rules delete
        docker ps -a -q | xargs -n 1 docker inspect --format '{{range .NetworkSettings.Networks}}{{.IPAddress}}{{end}} {{ .Name }}' | sed 's/ \// /'  |
        while IFS= read -r DOCKER_CONTAINER_LIST_LINE
        do
            DOCKER_CONTAINER_IP=\$(echo "\$DOCKER_CONTAINER_LIST_LINE" | awk -F ' ' '{print \$1}')
            DOCKER_CONTAINER_ID=\$(echo "\$DOCKER_CONTAINER_LIST_LINE" | awk -F ' ' '{print \$2}')
            delete_container_func "\$DOCKER_CONTAINER_ID"
        done
        delete_container_routes
        docker_zoombie_containers_cleanup
    fi
}

fetching_ip_from_dhcp_lease_for_forwarding_rules_in_containers() {
    cmd=\$(docker ps -q | wc -l)
    if [[ \$cmd -ne 0 ]]; then
        if [[ -f /tmp/ip.dhcp.lease ]]; then
            if rm -rf /tmp/ip.dhcp.lease; then
                invisily_note "DHCP lease temporary file delete successfully"
            else
                invisily_error_no_exit "DHCP lease temporary file failed to delete"
            fi
        else
            invisily_note "DHCP temporary lease file does not exists"
        fi
        dhcp-lease-list --parsable  | tee -a /tmp/ip.dhcp.lease > /dev/null
        if [[ -f /tmp/ip.dhcp.lease ]]; then
            invisily_note "Updated DHCP temporary lease file is created successfully"
            docker ps -q | xargs -n 1 docker inspect --format '{{range .NetworkSettings.Networks}}{{.IPAddress}}{{end}} {{ .Name }}' | sed 's/ \// /' |
            while IFS= read -r DOCKER_CONTAINER_LIST_LINE
            do
                DOCKER_CONTAINER_IP=\$(echo "\$DOCKER_CONTAINER_LIST_LINE" | awk -F ' ' '{print \$1}')
                DOCKER_CONTAINER_ID=\$(echo "\$DOCKER_CONTAINER_LIST_LINE" | awk -F ' ' '{print \$2}')
                DOCKER_CONTAINER_UUID=\$(docker inspect "\$DOCKER_CONTAINER_ID" | grep DEVICE_UUID |  cut -d'"' -f 2 | awk -F '=' '{print \$2}')
                if < /tmp/ip.dhcp.lease tr ' ' _ | nl | grep "\$(echo "\$DOCKER_CONTAINER_UUID" | tr '[:upper:]' '[:lower:]')"; then
                    DHCP_LEASED_IP_OF_CONTAINER_CLIENT=\$(< /tmp/ip.dhcp.lease tr ' ' _ | nl | grep "\$(echo "\$DOCKER_CONTAINER_UUID" | tr '[:upper:]' '[:lower:]')" | awk -F '_' '{print \$4}')
                    if [[ -d \$DOCKER_SHARED_DIR/\$DOCKER_CONTAINER_ID ]]; then
                        if echo "\$DHCP_LEASED_IP_OF_CONTAINER_CLIENT" > "\$DOCKER_SHARED_DIR/\$DOCKER_CONTAINER_ID/ip.addr"; then
                            invisily_note "[Container ID \$DOCKER_CONTAINER_ID] DHCP leased ip for client with mac address \$DOCKER_CONTAINER_UUID is successfully passed to container"
                        else
                            invisily_error_no_exit "[Container ID \$DOCKER_CONTAINER_ID] DHCP leased ip for client with mac address \$DOCKER_CONTAINER_UUID is failed passed to container"
                        fi
                    else
                        invisily_error_no_exit "[Container ID \$DOCKER_CONTAINER_ID] Shared directory [\$DOCKER_SHARED_DIR/\$DOCKER_CONTAINER_ID] for container does not exists"
                    fi
                else
                    invisily_warn "[Container ID \$DOCKER_CONTAINER_ID] DHCP lease for mac address \$DOCKER_CONTAINER_UUID does not exists, either client is not connected to machine or client has static ip assigned"
                fi
            done
        else
            invisily_error_no_exit "Updated DHCP temporary file is failed to create"
        fi
    else
        invisily_warn "No container is running in system to generate DHCP lease config for forwarding rules inside containers"
    fi
}

invisily_note "Bridge monitoring started/restarted"

while (( finish != 1 ))
do
    #updating bridge info received from controller
    updating_bridge_info_received_from_controller
    #creation of containers
    creation_of_running_containers
    #verification of routes in host for tunnels inside containers
    verification_of_container_routes_in_hosts
    ### checking for active routing rules
    active_routing_rule_check
    # deleing routes from TOBEDELETE files
    delete_container_routes
    #check forward rules
    handle_container_forward_rules create
    handle_dhcp_config
    #Fecthing ips assigned to clients from dhcp lease
    fetching_ip_from_dhcp_lease_for_forwarding_rules_in_containers

    REFRESH_TIME=\$((10 + RANDOM % 15))
    invisily_note "Recheck will now run after \$REFRESH_TIME seconds"
    sleep \$REFRESH_TIME
done
END
    chmod 755 $DOCKER_DAEMON_FILE
    cat > $NET_SERVICE_FILE <<END
#!/bin/bash
### BEGIN INIT INFO
# Short-Description:        Start and stop bridge gateway service.
# Description:              -
# Date-Creation:            -
# Date-Last-Modification:   -
# Author:                   -
### END INIT INFO

# Variables
PGREP=/usr/bin/pgrep
ZERO=0
DHCP_RANGE="$DHCP_RANGE"
PNAME=$DOCKER_DAEMON_PNAME
PNAME_PATH=$DOCKER_DAEMON_PNAME_PATH
PID=$PID
# Start bridge Gateway services.
start() {

    \$PGREP -f \$PNAME > /dev/null
    VERIFIER=\$?
    if [ \$ZERO = \$VERIFIER ]
    then
        echo "Bridge Gateway is already running, going to kill and run again."

        #Kill the pid of Server with the service name
        kill -TERM \$(\$PGREP -f \$PNAME)

        sleep 4
        #Verify if the service is running
        \$PGREP -f \$PNAME > /dev/null
        VERIFIER=\$?
        if [ \$ZERO = \$VERIFIER ]
        then
            echo "Error : Failed to stop Bridge Gateway ..."
        else
            echo "Bridge Gateway stopped successfully."
        fi
    fi

    #Run the Gateway
    echo "Starting Bridge Gateway..."
    /bin/bash \$PNAME_PATH/\$PNAME
}


# Stop Sniffer
stop() {


    echo "Stopping Bridge Gateway ..."
    #Verify if the service is running
    \$PGREP -f \$PNAME > /dev/null
    VERIFIER=\$?
    if [ \$ZERO = \$VERIFIER ]
    then

        #Kill the pid of server with the service name
        # kill -TERM \$(\$PGREP -f \$PNAME)
        kill -SIGUSR1 \$(cat \$PID)

        #Sleep time before the service verification
        sleep 4

        #Verify if the service is running
        \$PGREP -f \$PNAME > /dev/null
        VERIFIER=\$?
        if [ \$ZERO = \$VERIFIER ]
        then
            echo "Error : Failed to stop Bridge Gateway ..."
        else
            echo "Bridge Gateway stopped successfully ..."
        fi
    else
        echo "Bridge Gateway is already stopped ..."
    fi
    docker stop \$(docker ps -q)
    docker rm \$(docker ps -q)
}

# Verify status of Bridge Gateway
status() {


    echo "Checking status of Bridge Gateway ..."
    #Verify if the service is running
    \$PGREP -f \$PNAME > /dev/null
    VERIFIER=\$?
    if [ \$ZERO = \$VERIFIER ]
    then
        echo "[ Bridge Gateway is Running ]"
    else
        echo "[ Bridge Gateway is Stopped ]"
    fi
}

# Main logic
case "\$1" in
    start)
        start
        ;;
    stop)
        stop
        ;;
    status)
        status
        ;;
    restart|reload)
        stop
        start
        ;;
*)
    echo \$"Usage: \$0 {start|stop|status|restart|reload}"
    exit 1
esac
exit 0
END
    chmod 755 $NET_SERVICE_FILE
    cat > $SYSTEMD_SERVICE_FILE <<END
[Unit]
Description= Invisily Server Bridge Service
Documentation= http://invisily.com/
After=network.target
After=network-online.target
After=multi-user.target


[Service]
Type=simple
PIDFile=$SYSTEMD_PID
User=root
Group=root
WorkingDirectory=$DOCKER_DAEMON_PNAME_PATH
ExecStart=$NET_SERVICE_FILE start
ExecStop=$NET_SERVICE_FILE stop
KillSignal=SIGTERM
SyslogIdentifier=invisily_server_bridge

[Install]
WantedBy=multi-user.target
END

    cat > $LOG_ROTATE_FILE <<END
$DOCKER_DAEMON_LOG_FILE
{
    daily
    missingok
    rotate 3
    maxsize 5M
    minsize 5M
    compress
    delaycompress
    copytruncate
    postrotate
        /usr/lib/rsyslog/rsyslog-rotate
    endscript
}
END
    systemctl daemon-reload
    systemctl start $SYSTEMD_SERVICE_NAME
    systemctl enable $SYSTEMD_SERVICE_NAME
    systemctl enable invisily_backend
    #setting up watchdog
    cat > $WATCHDOG_FILE_PATH <<END
#!/bin/bash
PNAME=$DOCKER_DAEMON_PNAME
NET_SERVICE_FILE_NAME=$NET_SERVICE_FILE_NAME
SYSTEMD_SERVICE_NAME=$SYSTEMD_SERVICE_NAME
LOG_FILE=$DOCKER_DAEMON_LOG_FILE

#####################################################
#### LOG FUNCTIONS ##################################
#####################################################
invisily_log() {
    local type="\$1"; shift
    if [ "\$type" == "ERROR" ]; then
        printf '\033[0;31m%s [%s] [%s]: %s\033[0m\n' "\$(date +"%Y-%m-%d %H:%M:%S%z")" "\$type" "\$0" "\$*" | tee -a "\$LOG_FILE"
    elif [ "\$type" == "Warn" ]; then
        printf '\033[0;34m%s [%s] [%s]: %s\033[0m\n' "\$(date +"%Y-%m-%d %H:%M:%S%z")" "\$type" "\$0" "\$*" | tee -a "\$LOG_FILE"
    elif [ "\$type" == "Result" ]; then
        printf '\033[0;34m%s\033[0m\n' "\$*" | tee -a "\$LOG_FILE"
    else
        printf '%s [%s] [%s]: %s\n' "\$(date +"%Y-%m-%d %H:%M:%S%z")" "\$type" "\$0" "\$*"  | tee -a "\$LOG_FILE"
    fi
}
invisily_note() {
    invisily_log Note "\$@"
}
invisily_warn() {
    invisily_log Warn "\$@" >&2
}
invisily_result() {
    invisily_log Result "\$@" >&2
}
invisily_error() {
    invisily_log ERROR "\$@" >&2
    invisily_log ERROR "Exiting..."  >&2
    exit 1
}
invisily_error_no_exit() {
    invisily_log ERROR "\$@" >&2
}
###############################################################

PID_NET_SERVICE_FILE_NAME=\$(ps aux | grep \$NET_SERVICE_FILE_NAME | grep -v grep | awk '{print \$2}')

if [[ -z \$PID_NET_SERVICE_FILE_NAME ]]; then
    invisily_warn "\$SYSTEMD_SERVICE_NAME was not running and has been restarted!"
    if systemctl restart "\$SYSTEMD_SERVICE_NAME"; then
        invisily_note "\$SYSTEMD_SERVICE_NAME is successfully restarted"
    else
        invisily_error_no_exit "\$SYSTEMD_SERVICE_NAME is failed restart"
    fi
fi
sleep 5
PID_PNAME=\$(ps aux | grep "\$PNAME" | grep -v grep | awk '{print \$2}')
if [[ -z \$PID_PNAME ]]; then
    invisily_warn "\$SYSTEMD_SERVICE_NAME was not running and has been restarted!"
    if systemctl restart "\$SYSTEMD_SERVICE_NAME"; then
        invisily_note "\$SYSTEMD_SERVICE_NAME is successfully restarted"
    else
        invisily_error_no_exit "\$SYSTEMD_SERVICE_NAME is failed restart"
    fi
fi
END
cat > $WATCHDOG_CRON_FILE <<END
*/2 * * * * root /bin/bash $WATCHDOG_FILE_PATH  > /dev/null 2&>1
END
    invisily_result $'Invisily Server Bridge is successfully installed'
    echo "" >> "$MY_PATH"/.result
    echo "=================================================" >> "$MY_PATH"/.result
    echo "Invisily Server Bridge is successfully installed " >> "$MY_PATH"/.result
    echo "=================================================" >> "$MY_PATH"/.result
    echo "Go to /opt/invisily/linuxclientd/bin/ and Run    " >> "$MY_PATH"/.result
    echo "sudo ./invisily_backend  to first time login to Invisily" >> "$MY_PATH"/.result
    echo "After that Run sudo systemctl start invisily_backend    " >> "$MY_PATH"/.result
    echo "to run Invisily Service via Linux Service        " >> "$MY_PATH"/.result
    echo "=================================================" >> "$MY_PATH"/.result
    echo "=================================================" >> "$MY_PATH"/.result
    echo "DHCP Network Information:-                       " >> "$MY_PATH"/.result
    echo "DHCP CIDR:            $DHCP_RANGE" >> "$MY_PATH"/.result
    echo "Gateway IP:           $DHCP_GW_IP" >> "$MY_PATH"/.result
    echo "Subnet Mask:          $SUBNET_MASK" >> "$MY_PATH"/.result
    echo "=================================================" >> "$MY_PATH"/.result
    echo "=================================================" >> "$MY_PATH"/.result
}

device_profiler_remove(){
    systemctl stop $DEVICE_PROFILER_SYSTEMD_SERVICE_NAME  || invisily_warn $''$DEVICE_PROFILER_SYSTEMD_SERVICE_NAME ' service cannot find'
    rm -rf $DEVICE_PROFILER_SYSTEMD_SERVICE_FILE || invisily_warn $''$DEVICE_PROFILER_SYSTEMD_SERVICE_FILE ' could not be deleted!'
    rm -rf $DEVICE_PROFILER_INSTALLATION_DIR || invisily_warn $'''$DEVICE_PROFILER_INSTALLATION_DIR' ' could not be deleted!'
    invisily_warn $'Invisily Device Profiler has been successfully Uninstalled!'
    systemctl daemon-reload
}

server_bridge_remove(){
    systemctl stop $SYSTEMD_SERVICE_NAME  || invisily_warn $''$SYSTEMD_SERVICE_NAME ' service cannot find'
    systemctl stop invisily_backend  || invisily_warn $'invisilidy service cannot find'
    apt_dependency_check $INV_SRV_CLIENT || apt_dependency_uninstall $INV_SRV_CLIENT
    systemctl stop docker
    apt_dependency_check docker.io || apt_dependency_uninstall docker-engine docker docker.io docker-ce
    apt_dependency_check isc-dhcp-server || apt_dependency_uninstall isc-dhcp-server
    apt_dependency_check jq || apt_dependency_uninstall jq
    apt autoremove -y   || invisily_warn $'apt autoremove failed due to unknown reasons'
    rm -rf /var/lib/docker /etc/docker   || invisily_warn $'deleting docker directories failed due to unknown reasons'
    rm -rf /etc/apparmor.d/docker    || invisily_warn $'deleting docker directories failed due to unknown reasons'
    groupdel docker    || invisily_warn $'deleting docker group failed due to unknown reasons'
    rm -rf /var/run/docker.sock    || invisily_warn $'deleting docker directories failed due to unknown reasons'
    ip link delete docker0     || invisily_warn $'deleting docker0 interface failed due to unknown reasons'
    rm -rf "$LOG_ROTATE_FILE" || invisily_warn $''"$LOG_ROTATE_FILE" ' could not be deleted!'
    rm -rf "$DOCKER_FILE" || invisily_warn $''"$DOCKER_FILE" ' could not be deleted!'
    rm -rf "$DOCKER_SUPPORTING_DIR" || invisily_warn $''"$DOCKER_SUPPORTING_DIR" ' could not be deleted!'
    rm -rf $SYSTEMD_SERVICE_FILE || invisily_warn $''$SYSTEMD_SERVICE_FILE ' could not be deleted!'
    rm -rf $NET_SERVICE_FILE || invisily_warn $''$NET_SERVICE_FILE ' could not be deleted!'
    # rm -rf $NET_FILE || invisily_warn $''$NET_FILE ' could not be deleted!'
    rm -rf /etc/dhcp/ || invisily_warn $'/etc/dhcp/ could not be deleted!'
    # rm -rf /etc/dnsmasq.conf || invisily_warn $'/etc/dnsmasq.conf could not be deleted!'
    rm -rf /opt/invisily || invisily_warn $'/opt/invisily could not be deleted!'
    rm -rf /opt/invisily_updates || invisily_warn $'/opt/invisily_updates could not be deleted!'
    systemctl daemon-reload
    invisily_warn $'Invisily Server Bridge has been successfully Uninstalled!'

}


arg_verification
if [[ "$INSTALL" == "1" ]]; then
    server_bridge_install
    if [ -n "$KAFKA_SERVERS_IPS" ] &&  [ -n "$KAFKA_PORT" ] && [ -n "$KAFKA_TOPIC" ]; then
        device_profiler_install
    fi
elif [[ "$REMOVE" == "1" ]]; then
    server_bridge_remove
    device_profiler_remove
else
    invisily_error $'You did not provide any argument to any process'
fi

if [[ -f "$MY_PATH"/.result ]]; then
    while IFS= read -r line; do
        invisily_result $''"$line"
    done < "$MY_PATH"/.result
    rm -rf "$MY_PATH"/.result
else
    invisily_warn $'Cannot find Result file to display'
fi