const moment = require('moment');
const { trimEnd } = require('lodash');
const httpErrors = require('http-errors');
const fsExtra = require('fs-extra');
const { exec } = require('child_process');
const { generateKeyPair } = require('crypto');
const client = require('../config/cache');


/**
 * Convert integer ip to actual ip address
 * @param {number} value
 * @return {string} actual ip address
 */

module.exports.LICENSE_EXPIRY_EMAIL = 'LICENSE_EXPIRY_EMAIL'

module.exports.intToIp = function (value) {
  if (!value || isNaN(value)) return '0.0.0.0';
  const part1 = value & 255;
  const part2 = (value >> 8) & 255;
  const part3 = (value >> 16) & 255;
  const part4 = (value >> 24) & 255;

  return part1 + '.' + part2 + '.' + part3 + '.' + part4;
};


/**
 * Get random number
 * @param {number} min minimum value
 * @param {number} max maximum value
 * @return {string} random number
 */
module.exports.randomNumber = function (min = 100000, max = 999999) {
  return (Math.random() * (max - min + 1)) << 0;
};


/**
 * Convert date to epoch
 * @param {date} date  
 * @return {number} Unix timestamp
 */
module.exports.dateToEpoch = function (date) {
  if (!date) return null;

  if (!(date instanceof moment)) {
    date = moment(date);
  }

  if (!date.isValid())
    throw new Error('Invalid date');

  return moment(date).unix();
};

/**
 * Get the date ranges
 * @param {object} options 
 * @param {string} options.format
 * @param {('M'|'Y'|'D'|'LM'|'W'|'L7D')} options.rangeType
 * @param {boolean} options.epoch
 * @param {boolean} options.fromCurrentDate
 * @param {Date} options.date
 * @typedef {object} DateRange
 * @property {(string|number)} DateRange.startDate
 * @property {string|number} DateRange.endDate
 * @return {DateRange} range returns the date range
 */
module.exports.getDateRange = function (options) {

  const _default = {
    format: 'YYYY-MM-DD HH:mm:ss',
    rangeType: 'M',
    fromCurrentDate: true,
    epoch: false,
    date: moment()
  };

  const opts = { ..._default, ...options };

  const year = moment().year();
  const month = moment().month() + 1;

  let startDate, endDate;

  if (opts.rangeType == 'M') {
    if (opts.fromCurrentDate) {
      startDate = moment().subtract(1, 'month').startOf('day');
      endDate = moment().endOf('day');
    } else {
      startDate = moment([year, month - 1]);
      endDate = moment(startDate).endOf('month');
    }
  } else if (opts.rangeType == 'Y') {
    if (opts.fromCurrentDate) {
      startDate = moment().subtract(1, 'year').startOf('day');
      endDate = moment().endOf('day');
    } else {
      startDate = moment([year, 0]);
      endDate = moment(startDate).endOf('year');
    }
  } else if (opts.rangeType == 'D') {
    startDate = moment(opts.date).startOf('day');
    endDate = moment(opts.date).endOf('day');
  } else if (opts.rangeType == 'LM') { // Get date of last month
    startDate = moment([year, month - 2]);
    endDate = moment(startDate).endOf('month');
  } else if (opts.rangeType == 'W') {
    startDate = moment().startOf('week');
    endDate = moment(startDate).endOf('week');
  } else if (opts.rangeType == 'L7D') { // Last seven days
    startDate = moment().subtract(7, 'day').startOf('day');
    endDate = moment().endOf('day');
  } else {
    throw new Error('Invalid range type.')
  }

  if (opts.epoch) {
    startDate = module.exports.dateToEpoch(startDate);
    endDate = module.exports.dateToEpoch(endDate);
  } else {
    startDate = startDate.format(opts.format);
    endDate = endDate.format(opts.format);
  }

  return {
    startDate,
    endDate
  }
};

/**
 * Convert value into array
 * @param {any} value Value to split
 * @param {string} splitter Split string based on character
 * @return {Array}
 */
module.exports.convertToArray = function (value = [], splitter = ',') {
  let result = [];
  let isJSON = true;

  if (!value || !value.length) return null;

  try {

    result = Array.isArray(value) ? value : JSON.parse(value);

  } catch (ex) {
    isJSON = false;
  }

  if (typeof value === 'string' && !result.length) {
    if (value.includes(splitter))
      result = value.split(splitter);
    else
      result = [value];
  }

  return result;
}


/**
 * Covert os itno display os name
  * @param {String} os
  */

module.exports.getOsDsipalyName = (os) => {

  let osHashMap = {
    ubuntu18: 'Ubuntu Desktop',
    ubuntuserver18: 'Ubuntu Server',
    macintosh: 'Macintosh',
    'macintosh10.14': 'macOS Mojave',
    'macintosh10.15': 'macOS Catalina',
    'macintosh11': 'macOS Big Sur',
    'macintosh12': 'macOS Monterey',
    'macintosh13': 'macOS Ventura',
    'macintosh14': 'macOS Sonoma',
    windows: 'Windows',
    'windows6.1': 'Windows 7',
    'windows6.2': 'Windows 8',
    'windows6.3': 'Windows 8.1',
    'windows10.0': 'Windows 10',
    'windows11.0': 'Windows 11',
    ubuntu: 'Ubuntu Desktop',
    ubuntuserver: 'Ubuntu Server',
    ubuntuserver20armv8: 'Ubuntu Server (ARMv8)',
    windowsserver: 'Windows Server',
    'windowsserver6.1': 'Windows Server 2008 R2',
    'windowsserver6.2': 'Windows Server 2012',
    'windowsserver6.3': 'Windows Server 2012 R2',
    'windowsserver10.0': 'Windows Server 2016+',
    ios: 'iOS',
    android: 'Android',
    'centosserver7': 'CentOS 7 Server',
    'centosserver8': 'CentOS 8 Server',
    'centosserver': 'CentOS Server'
  };

  if (osHashMap[os]) return osHashMap[os];

  return os;

}


/**
 * get package name of all components
 * @param {String} os 
 * @param {String} component
 * @return {String}
 */

module.exports.getPackageName = async (os = 'ubuntuserver18', component, gatewayId = '', controllerId = '') => {

  console.log(component, gatewayId, controllerId)
  let componentName = {
    client: 'invisily-client',
    controller: 'invisily-controller',
    gateway: 'invisily-gateway',
    's2s-client': 'invisily-s2s-client'
  }

  let clientsOs = {
    ubuntu: 'linux64-desktop-16.zip',
    ubuntuserver: 'linux64-server-16.zip',
    ubuntu18: 'linux64-desktop-18.zip',
    ubuntuserver18: 'linux64-server-18.zip',
    ubuntuserver20armv8: 'linux64-server-20-armv8.zip',
    macintosh: 'mac.zip',
    windows: 'window.zip',
    windowsserver: 'window-server.zip',
    android: 'installer.apk',
    ios: 'installer.ipa',
    centosserver: 'linux64-centosserver.zip'
  }

  let controllerInstallerType = {
    ubuntu18: 'linux64',
    ubuntuserver: 'linux64',
    ubuntuserver18: 'linux64',
    ubuntuserver20armv8: 'linux64',
    ubuntu: 'linux64',
    windows: 'win64',
    windowsserver: 'win64',
    macintosh: 'mac',
    android: 'android',
    ios: 'ios',
    centosserver: 'linux64'
  };

  let isComponentName = componentName[component];
  let osName = clientsOs[os];
  let controllerType = controllerInstallerType[os];


  if (isComponentName && osName && controllerType) {

    const models = require('../models');

    const update = await models.Update.findOne({
      where: {
        buildOs: os,
        buildComponent: component
      },
      attributes: ['buildVersion']
    })

    if (!update) {
      throw new httpErrors.NotFound('Package not found');
    }

    if (component === 'client') {
      return `${isComponentName}-${update.buildVersion}-${osName}`;
    } else if (component === 'controller') {
      return `${isComponentName}-${update.buildVersion}-${controllerType}-${controllerId}.zip`;
    } else if (component === 'gateway') {
      return `${isComponentName}-${update.buildVersion}-${controllerType}-${gatewayId}.zip`;
    }else if (component === 's2s-client') {
      return `${isComponentName}-${update.buildVersion}-${osName}`;
    } 

  }

  return 'invisily.zip';

}

/**
 * get ip or hostname from url
 * @param { String } url
 * @return { String }
 */

module.exports.getHostOrIpFromUrl = (url) => {
  let webUIUrl = url.split(':');
  if (webUIUrl.length === 3) {
    let webUIHost = webUIUrl[1].split('//');
    if (webUIHost.length === 2) {
      return webUIHost[1];
    }
  }
  return url;
}


/**
 * make signature file of specific file
 * @param { String } privateKeyPath
 * @param {String} inputFilePath
 * @param {String} outputFilePath
 * @param {String} sigFileName
 * @return { String } signature file path
 */
module.exports.getSignatureFile = async (privateKeyPath, inputFilePath, outputFilePath, sigFileName) => {

  await fsExtra.ensureDir(outputFilePath);

  return new Promise((resolve, reject) => {

    exec(`openssl dgst -sign ${privateKeyPath} -keyform PEM -sha256 -out ${outputFilePath}/${sigFileName}.sign -binary ${inputFilePath}`, (error, stdout, sterr) => {
      if (error) reject(error);
      resolve(`${outputFilePath}/${sigFileName}.sign`);
    })
  })
}


/**
 * generate private public key pair
 * @return { Object }
 */

module.exports.generateKeyValuePair = () => {

  return new Promise((resolve, reject) => {
    generateKeyPair('rsa', {
      modulusLength: 530,    // options 
      publicKeyEncoding: {
        type: 'spki',
        format: 'pem'
      },
      privateKeyEncoding: {
        type: 'pkcs8',
        format: 'pem',
        cipher: 'aes-192-cbc',
        passphrase: 'Invisily is very good security product!'
      }
    }, (err, publicKey, privateKey) => {
      if (!err) {
        publicKey = publicKey.replace('-----BEGIN PUBLIC KEY-----', '').replace('-----END PUBLIC KEY-----', '').replace(/(?:\r\n|\r|\n)/g, '').trim();
        privateKey = privateKey.replace('-----BEGIN ENCRYPTED PRIVATE KEY-----', '').replace('-----END ENCRYPTED PRIVATE KEY-----', '').replace(/(?:\r\n|\r|\n)/g, '').trim();
        resolve({
          publicKey,
          privateKey
        });
      } else {
        reject(err);
      }
    });
  })
}

const serverStatusKeys = {
  CONTROLLERS: 'Controllers:Online',
  GATEWAYS: 'Gateways:Online',
  CLIENTS: 'Clients:Online',
  PORTALS: 'Portals:Status',
  MFA_SERVER: 'mfaServerStatus',
  ONBOARDING_PORTAL: 'onboardingPotalStatus',
  CONTENT_PORTAL: 'contentPortalStatus'
}

module.exports.getOnlineServers = (key, time = 2) => {
  return client.zRangeByScoreAsync(
    key,
    moment().subtract(time, 'minutes').unix(),
    moment().unix()
  );
}

module.exports.getOnlineCount = async (key = this.serverStatusKeys.CONTROLLERS, time = 2) => {
  return client.zCountAsync(
    key,
    moment().subtract(time, 'minutes').unix(),
    moment().unix()
  )
}

module.exports.generateRandomPassword = () => {
  return Math.random().toString(36).toUpperCase().replace(/[0-9O]/g, '').substring(1, 3) +
    Math.random().toString(36).toLowerCase().replace(/[0-9O]/g, '').substring(1, 3) +
    Math.floor(Math.random() * (999 - 100 + 1) + 100) +
    "!@#$&".substr(Math.floor(5 * Math.random()), 1) +
    Math.floor(Math.random() * 10);
}

module.exports.serverStatusKeys = serverStatusKeys;

