const axios = require('axios');
const msal = require('@azure/msal-node');
const qrcode = require('qrcode');
const models = require('../models');

const { authenticator } = require('otplib');

const graphResource = 'https://graph.microsoft.com';

// sending email using OAuth token
async function sendEmail(accessToken, mailOptions) {
  const headers = {
    'Authorization': `Bearer ${accessToken}`,
    'Content-Type': 'application/json',
    'Accept': '*/*'
  };

  const OAuthConfig = await models.Settings.findOne({
    where: {
      id: 1
    }
  });

  const senderEmail = OAuthConfig?.oauth_from;

  const emailEndpoint = `${graphResource}/v1.0/users/${senderEmail}/sendMail`;

  let qrCodeDataUrl;

  if(mailOptions?.client){
    let otp_secret = authenticator.generateSecret();
    const otpauth = authenticator.keyuri(mailOptions.to, 'INVISILY', otp_secret);
    qrCodeDataUrl = await qrcode.toDataURL(otpauth);
  }else {
    let otp_secret = authenticator.generateSecret();
    const otpauth = authenticator.keyuri(mailOptions.to, 'INVISILY ADMIN', otp_secret);
    qrCodeDataUrl = await qrcode.toDataURL(otpauth);
  }
  
  const attachments = mailOptions?.attachments ? mailOptions.attachments.map(attachment => ({
    "@odata.type": "#microsoft.graph.fileAttachment",
    "name": `${attachment.filename}`,
    "contentBytes": attachment.filename === 'code.png' ?  qrCodeDataUrl.split(';base64,').pop() : attachment.content
  })) : [];

  const emailData = {
    message: {
      subject: mailOptions?.subject,
      body: {
        contentType: 'HTML',
        content: mailOptions?.html,
      },
      toRecipients: [{ emailAddress: { address: mailOptions?.to } }],
      attachments: attachments,
    },
  };

  try {
    const response = await axios.post(emailEndpoint, emailData, { headers });
    if (response.status === 202) {
      console.log(`Email sent to ${mailOptions.to} using OAuth...`);
      return
    } else {
      console.error(`Failed to send email. Status Code: ${response.status}`);
    }
  } catch (error) {
    console.error('Error sending email:', error.message);
    if (error.response) {
      console.error('Error details:', error.response.data);
    }
  }
}

// Get OAuth token
async function getToken() {
  //Get OAuth configs from Settings
  const OAuthConfig = await models.Settings.findOne({
    where: {
      id: 1
    }
  });

  if(OAuthConfig && 
    OAuthConfig?.oauth_config &&
    OAuthConfig?.oauth_client_id && 
    OAuthConfig?.oauth_client_secret && 
    OAuthConfig?.oauth_tenant_id) {

    //Pre-requisites for configuring OAuth
    const clientId = OAuthConfig?.oauth_client_id;
    const clientSecret = OAuthConfig?.oauth_client_secret;
    const tenantId = OAuthConfig?.oauth_tenant_id;

    // Microsoft 365 API endpoints
    const tokenUrl = `https://login.microsoftonline.com/${tenantId}/oauth2/v2.0/token`;
    const scopes = [`${graphResource}/.default`];

    // MSAL configuration
    const msalConfig = {
      auth: {
        clientId,
        authority: tokenUrl,
        clientSecret,
        grant_type: 'client_credentials'
      },
    };

    const cca = new msal.ConfidentialClientApplication(msalConfig);
    const authResult = await cca.acquireTokenByClientCredential({
      scopes,
    });

    if (authResult && authResult.accessToken) {
      return authResult.accessToken;
    } else {
      console.error('Failed to obtain access token.');
      return null;
    }
  }
}

module.exports = {
  getToken,
  sendEmail
}
