const { body } = require('express-validator');
var httpErrors = require('http-errors');

const createService = () => {
  const validations = [
    ...name(body),
    ...host(body),
    ...port(body),
    ...type(body),
    ...mappedURL(body)
  ];

  // Check if endpoints array is present in the request body
  if (body.service && body.service.endpoints) {
    validations.push(...mappedEndpointsValidation(body));
  }

  return validations;
};

const publicCreateService = () => {
  return [
    ...name(body),
    ...host(body),
    ...port(body),
    ...type(body),
    ...mappedURL(body),
    ...isPersistent(body)
  ];
};

const isPersistent = (check) => [
  check('is_persistent')
  .exists()
  .not()
  .isEmpty()
  .withMessage('is_persistent is required.')
  .custom((value, { req }) => {
    if (value && !(typeof value === 'boolean') && !(value === 'true' || value === 'false'))
      throw new Error('Value should be boolean');
    return true;
  })
]

const updateService = () => {
  const validations = [
    ...name(body),
    ...host(body),
    ...port(body),
    ...type(body),
    ...id(body),
    ...mappedURL(body)
  ];

  // Check if endpoints array is present in the request body
  if (body.service && body.service.endpoints) {
    validations.push(...mappedEndpointsValidation(body));
  }

  return validations;
};


  const id = check => [
    check('service.service_id')
      .exists()
      .not()
      .isEmpty()
      .isNumeric()
      .withMessage('Invalid Id.')
  ];

  const name = check => [
    check('service.Name')
      .exists()
      .not()
      .isEmpty()
      .trim()
      .custom((value) => {
        const regex = /^[a-zA-Z\d\-_\s]+$/;
        if(!regex.test(value)){
            throw new httpErrors.UnprocessableEntity("Invalid Name.");
        }
        return true;
      })
      .isLength({min: 1, max: 150})
      .withMessage('Invalid Name.')
  ];

  const host = check => [
      check('service.ipv4_address')
      .exists()
      .not()
      .isEmpty()
      .custom( value => {
        const validIpExpression = /^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$/;
        const validHostnameExpression = /^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])\.)*([A-Za-z0-9]|[A-Za-z0-9][A-Za-z0-9\-]*[A-Za-z0-9])$/;
        if(validIpExpression.test(value) || validHostnameExpression.test(value)){

            return true;

        }else{

            throw new Error("Invalid Host");

        }
        return true;
      }).withMessage("Invalid hostname")
  ]

  const port = check => [
    check('service')
    .custom((value, { req }) => {

        let ports = value.port.split(',');
        let dummyPorts = value.dummy_port.split(',');

        const validPortExpress = /^([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])$/;
        
        let validPorts = ports.filter(port => {
          if(port.includes(':')) {
            let rangePorts = port.split(':');
            return validPortExpress.test(rangePorts[0]) && validPortExpress.test(rangePorts[1]);
          }
          else {
            return validPortExpress.test(port);
          }
        });
        let validDummyPorts = dummyPorts.filter(port => {
          if(port.includes(':')) {
            let rangePorts = port.split(':');
            return validPortExpress.test(rangePorts[0]) && validPortExpress.test(rangePorts[1]);
          }
          else {
            return validPortExpress.test(port);
          }
        });

        if(ports.length != validPorts.length || dummyPorts.length != validDummyPorts.length){

            throw new Error('Invalid port.');

        }else{

            return true;

        }

      })
  ];

  const type = check => [
    check('service.type')
    .custom((value, { req }) => {

        const validTypeExpress = /^\S*$/;

        if(!validTypeExpress.test(value)){
            
            throw new Error('Invalid Type.');

        }else{

            return true;

        }

      })
  ];

  const mappedURL = check => [
    check('service.mappedurl')
    .exists()
    .not()
    .isEmpty()
    .custom((value, { req }) => {
      const { proxy_enabled } = req.body.service
      const regex = {
        proxyEnabled: /^(?:([a-z]{1,9}):\/\/)?(?:[a-zA-Z0-9.-]+?\.(?:[a-zA-Z]{1,}(:([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5]))?(\/.*)?$)|(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(:([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5]))?(\/.*)?$)/,
        proxyDisabled: /^(?:([a-z]{1,9}):\/\/)?(?:[a-zA-Z0-9.-]+?\.(?:[a-zA-Z]{1,}(:([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5]))?(\/.*)?$)|(^(N|n)one$)|(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(:([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5]))?(\/.*)?$)/
      }
      if(!regex[proxy_enabled? 'proxyEnabled': 'proxyDisabled'].test(value)){

        throw new Error('Invalid Mapped URL.');

      }
      return true;

    })
  ];

  const mappedEndpointsValidation = check => [
    check('service.endpoints')
      .isArray({ min: 1 })
      .custom((endpoints, { req }) => {
        const regex = /^\/\w+(?:[.:~-]\w+)*(?:\/\w+(?:[.:~-]\w+)*)*$/;
        for (const endpoint of endpoints) {
          if (!regex.test(endpoint)) {
            throw new Error('Invalid endpoint format.');
          }
        }
        return true;
      })
  ];


  module.exports = {
    createService,
    updateService,
    publicCreateService
  };