const { body, param } = require('express-validator');
const httpErrors = require('http-errors');
let language = require('../routes/common/language');


const updateSettings = () => {
    return [
        ...auto_purge_days(body),
        ...admin_mfa(body),
        ...biometrics_state(body),
        ...date_format(body),
        ...dirsync_enabled(body),
        ...dirsync_time(body),
        ...gapiKey(body),
        ...recaptcha_site_key(body),
        ...session_expiry(body),
        ...password_expiry_days(body),
        ...smtp_from(body),
        ...smtp_user(body),
        ...smtp_port(body),
        ...smtp_service(body),
        ...package_link_expiry(body),
        ...id(param)
    ];
}

const updateCoreSettings = () => [
    ...siem_host(body),
    ...core_auto_update(body),
    ...siem_port(body),
    ...siem_protocol(body),
    ...max_clients_per_controller(body),
    ...health_status(body),
    ...data_reporting_interval(body),
    ...load_stats(body),
    ...in_activity_timeout(body),
    ...mtls_blocking_timeout(body),
    ...mtls_frequency(body),
    ...load_stats_interval(body),
    ...health_interval(body),
    ...device_validation_freq(body)
]


const updateClientSettings = () => {
    return [
        ...auto_update_state(body),
        ...global_darkening(body),
        ...mfa_server_state(body),
        ...otp_state(body),
        ...user_pass_validity_period(body),
        ...process_reporting_interval(body),
        ...android_link(body),
        ...ios_link(body)
    ];
}


const mfaSettings = () =>[
    ...clientID(body),
    ...deviceID(body)
]


const clientID = check => [
    check('clientID')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .custom((value) => {
        const regex = /^[0-9]{1,}$/;
        if(!regex.test(value)){
            throw new httpErrors.BadRequest(language.get['INVALID_DATA_PROVIDED'])
        }
        return true;
    })
]

const android_link = (check) => [
    check('setting.android_link')
        .optional()
]

const ios_link = (check) => [
    check('setting.ios_link')
        .optional()
]

const process_reporting_interval = check => [
    check('setting.process_reporting_interval')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .custom((value, { req }) =>{
        return true;
    })
]


const deviceID = check => [
    check('deviceID')
    .exists()
    .not()
    .isEmpty()
    .custom((value) => {
        const regex = /^[a-zA-Z0-9]{32}$/;
        if(!regex.test(value)){
            throw new httpErrors.BadRequest(language.get['INVALID_DATA_PROVIDED'])
        }
        return true;
    })
]

const user_pass_validity_period = check => [
    check('setting.user_pass_validity_period')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .custom((value, { req }) =>{
        if(+value<5){
            throw new Error('Password expiry time must be greator or equal to 5.');
        }
        return true;
    })
]

const auto_purge_days = check => [
    check('setting.auto_purge_days')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .custom((value, { req }) =>{
        if(+value<1){
            throw new Error('Auto purge time must be greator or equal to 1.');
        }
        return true;
    })
]


const admin_mfa = check => [
    check('setting.admin_mfa')
    .exists()
    .not()
    .isEmpty()
]


const biometrics_state = check => [
    check('setting.biometrics_state')
    .exists()
    .not()
    .isEmpty()
]


const date_format = check => [
    check('setting.date_format')
    .exists()
    .not()
    .isEmpty()
]


const dirsync_enabled = check => [
    check('setting.dirclean_time')
    .exists()
    .not()
    .isEmpty()
]


const dirsync_time = check => [
    check('setting.dirsync_time')
    .custom((value, { req }) => {
        if(req.body.setting.ldap){
            if(+value < 30){
                throw new Error('Auto sync time should not be less than 30 minutes.');
            }
        }
        return true;
    })
]

const max_clients_per_controller = check => [
    check('setting.max_clients_per_controller')
    .exists()
    .isNumeric()
    .custom((value) => {
        if(+value < 1){
            throw new Error('Max clients per controller should be greater than zero.');
        }
        return true;
    })
]


const gapiKey = check => [
    check('setting.gapiKey')
    .exists()
    .not()
    .isEmpty()
]

const recaptcha_site_key = check => [
    check('setting.recaptcha_site_key')
    .exists()
    .not()
    .isEmpty()
]

const session_expiry = check => [
    check('setting.session_expiry')
    .exists()
    .isNumeric()
    .custom((value, {req}) => {
        if(+value < 30){
            throw new Error('Session expiry time should not be less than 30 minutes');
        }
        return true;
    })
]


const password_expiry_days = check => [
    check('setting.password_expiry_days')
    .exists()
    .isNumeric()
    .custom((value, {req}) => {
        if(+value < 5){
            throw new Error('Password expiry time should not be less than 5 days');
        }
        return true;
    })
]


const smtp_from = check => [
    check('setting.smtp_from')
    .exists()
    .not()
    .isEmpty()
]


const smtp_user = check => [
    check('setting.smtp_user')
    .exists()
    .not()
    .isEmpty()
]


const smtp_port = check => [
    check('setting.smtp_port')
    .exists()
    .isNumeric()
    .custom((value, { req }) => {
        const validPort = /^([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])$/;
        if(!validPort.test(value)){
            throw new Error('Invalid SMTP port.');
        }
        return true;
    })
]


const smtp_service = check => [
    check('setting.smtp_service')
    .exists()
    .not()
    .isEmpty()
]


const package_link_expiry = check => [
    check('setting.package_link_expiry')
    .exists()
    .isNumeric()
    .custom((value, {req}) => {
        if(+value < 1){
            throw new Error('Package expiry should not be less or equal to zero.');
        }
        if(+value >  10080){
            throw new Error('Package expiry should be less than 10080 minutes (7 days)');
        }
          
        return true;
    })
]


const id = check => [
    check('id')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
]


const auto_update_state = check => [
    check('setting.auto_update_state')
    .exists()
]

const core_auto_update = check => [
    check('setting.core_auto_update')
    .exists()
]



const global_darkening = check => [
    check('setting.global_darkening')
    .exists()
    .not()
    .isEmpty()
]


const mfa_server_state = check => [
    check('setting.mfa_server_state')
    .exists()
    .not()
    .isEmpty()
]


const otp_state = check => [
    check('setting.otp_state')
    .exists()
    .not()
    .isEmpty()
]

const siem_host = check => [
    check('setting.siem_host')
    .exists()
    .not()
    .isEmpty()
    .custom(value => {
      const validIpExpression = /^(\d|[1-9]\d|1\d\d|2([0-4]\d|5[0-5]))\.(\d|[1-9]\d|1\d\d|2([0-4]\d|5[0-5]))\.(\d|[1-9]\d|1\d\d|2([0-4]\d|5[0-5]))\.(\d|[1-9]\d|1\d\d|2([0-4]\d|5[0-5]))$/;
      const validHostnameExpression = /^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])\.)*([A-Za-z0-9]|[A-Za-z0-9][A-Za-z0-9\-]*[A-Za-z0-9])$/;

      if (
        !(validIpExpression.test(value) || validHostnameExpression.test(value))
      ) {
        throw new httpErrors.UnprocessableEntity('Invalid SIEM host.');
      }
      return true;
    })
    .withMessage('Invalid SIEM hostname.')
];

const siem_protocol = check => [
    check('setting.siem_protocol')
    .exists()
    .not()
    .isEmpty()
    .withMessage('Invalid SIEM protocol.')
];

const siem_port = check => [
    check('setting.siem_port')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .custom(value => {
        const validPortExpression = /^([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])$/;
        if (!validPortExpression.test(value)) {
            throw new Error('Invalid SIEM port.');
        }
        return true;
    })
    .withMessage('Invalid SIEM port.')
]

const health_status = check => [
    check('setting.health_status')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .custom(value => {
        if (value < 1) {
            throw new Error('Value must be greater than 0.');
        }
        return true;
    })
    .withMessage('Invalid Health Reporting Interval only interger allowed.')
];

const data_reporting_interval = check => [
    check('setting.data_reporting_interval')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .custom(value => {
        if (value < 1) {
            throw new Error('Value must be greater than 0.');
        }
        return true;
    })
    .withMessage('Invalid Data Reporting Interval only interger allowed.')
];

const load_stats = check => [
    check('setting.load_stats')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .custom(value => {
        if (value < 5 || value > 100) {
            throw new Error('Value must be between 5 and 100.');
        }
        return true;
    })
    .withMessage('Invalid Load Stats Interval value only interger allowed.')
];


const in_activity_timeout = check => [
    check('setting.in_activity_timeout')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .withMessage('Invalid in activity timeout value only interger allowed.')
];

const mtls_blocking_timeout = check => [
    check('setting.mtls_blocking_timeout')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .withMessage('Invalid mtls blocking timeout value only interger allowed.')
];


const mtls_frequency = check => [
    check('setting.mtls_frequency')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .withMessage('Invalid mtls frequency value only interger allowed.')
];

const load_stats_interval = check => [
    check('setting.load_stats_interval')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .withMessage('Invalid Load Stats Interval value only interger allowed.')
];

const health_interval = check => [
    check('setting.health_interval')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .withMessage('Invalid health interval value only interger allowed.')
];

const device_validation_freq = check => [
    check('setting.device_validation_freq')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .withMessage('Invalid device validation frequency value only interger allowed.')
];

module.exports = {
    updateSettings,
    updateClientSettings,
    mfaSettings,
    updateCoreSettings
}