import { Component } from '@angular/core';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';
import { Router } from '@angular/router';
import { Subject, debounceTime, distinctUntilChanged, switchMap, tap } from 'rxjs';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { Pagination } from '../../models/pagination';
import { AdditionalPermissionsService } from '../additional-permissions.service';
import { ConfirmModalComponent } from '../../shared/confirm-modal.component';
import { ApprovalModalComponent } from '../approval-modal/approval-modal.component';

@Component({
  selector: 'app-view-additional-permissions',
  templateUrl: './view-additional-permissions.component.html',
  styleUrls: ['./view-additional-permissions.component.css']
})
export class ViewAdditionalPermissionsComponent {
  templates = [];

  pagination: Pagination = new Pagination();
  pageSize = [10, 25, 50, 100, 200];

  searchKeyword;
  $searchUsers = new Subject();
  showKeys: boolean = false;
  loading: boolean = false;

  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;
  tempId: number;

  sort: string;
  sortDir: string;

  public dateFormat: string = 'dd/MM/yyyy HH:mm';

  additionalPrivilges: Array<Object> = [];

  constructor(
    private router: Router,
    private titleService: Title,
    private additionalPermissionsService: AdditionalPermissionsService,
    private globalMessageService: GlobalMessageService,
    public dialog: MatDialog
  ) {
    var localDateFormat = localStorage.getItem('dateFormat');
    if (typeof localDateFormat != 'undefined' || localDateFormat != null) {
      this.dateFormat = localDateFormat;
    }
  }

  ngOnInit() {
    this.titleService.setTitle('Additional Permissions');
    this.getPage(this.pagination.currentPage);
    this.getCRUDdata(36);
    this.subscibeToSearch();
  }

  ngOnDestroy() {}

  handleBack() {
    this.router.navigate(['/view_roles_and_permissions']);
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getPage(this.pagination.currentPage);
  }

  subscibeToSearch() {
    this.$searchUsers
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap(data => {
          this.loading = true;
          this.searchKeyword = data;
          this.pagination.currentPage = 1;
          return this.additionalPermissionsService.getAdditionalPermissions(
            this.pagination.currentPage,
            this.pagination.itemsPerPage,
            this.sort,
            this.sortDir,
            this.searchKeyword
          );
        })
      )
      .subscribe({
        next: (templates: any) => {
          this.loading = false;
          const recordsTotal = templates.recordsTotal;
          this.templates = templates.paginatedData;
          this.pagination.totalItems = recordsTotal;
        },
        error: err => {
          this.loading = false;
        }
      });
  }

  sortRecord(term: string) {
    this.sortDir = this.sortDir == 'asc' ? 'desc' : 'asc';
    this.sort = 'email';
    this.getPage(this.pagination.currentPage);
  }

  getPage(page: number) {
    this.loading = true;
    let prevPage = this.pagination.currentPage;
    this.pagination.currentPage = page;
    this.additionalPermissionsService
      .getAdditionalPermissions(
        this.pagination.currentPage,
        this.pagination.itemsPerPage,
        this.sort,
        this.sortDir,
        this.searchKeyword
      )
      .pipe(
        tap((res: any) => {
          this.loading = false;
          this.templates = res.paginatedData;          
          this.pagination.totalItems = res.recordsTotal;
          this.pagination.currentPage = page;
          if (prevPage != this.pagination.currentPage) {
            let scrollToTop = window.setInterval(() => {
              let pos = window.pageYOffset;
              if (pos > 0) {
                window.scrollTo(0, pos - 20); // how far to scroll on each step
              } else {
                window.clearInterval(scrollToTop);
              }
            }, 16);
          }
        })
      )
      .subscribe({
        next: next => {
        },
        error: err => {
          this.loading = false;
        }
      });
  }

  clearfun() {
    this.searchKeyword = '';
    this.getPage(this.pagination.currentPage);
  }

  getCRUDdata(privId) {
    this.additionalPermissionsService.getCrudData(privId).subscribe((res: any) => {
      this.tempId = res?.temp_id;
      this.createPermission = res?.create_permission === 1
      this.readPermission = res?.read_permission === 1
      this.updatePermission = res?.update_permission === 1
      this.deletePermission = res?.delete_permission === 1
    });
  }

  getAllTemplates() {
    this.additionalPermissionsService
    .getAdditionalPermissions()
    .subscribe((res: any) => {
      this.templates = res.paginatedData;
    });
  }

  onView(template) {
    this.router.navigate(['/additional_permissions_details', template?.id]);
  }
  
  onEdit(template) {
    // this.router.navigate(['/edit_roles_and_permissions', template?.temp_id]);
    const dialogRef = this.dialog.open(ApprovalModalComponent, {
      width: '425px',
      height: '259px'
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.additionalPermissionsService
          .updateAdditionalTemplate(template?.id, result)
          .subscribe(res => {
            if (res) {
              this.globalMessageService.pushAlert({
                closed: false,
                type: 'success',
                message: 'Additional privilges updated successfully.'
              });
              this.getAllTemplates();
            }
          });
      }
    });
  }

  onDelete(template) {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      data: {
        title: 'Confirmation',
        message: 'Are you sure you want to delete these additional privilges?'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.additionalPermissionsService
          .deleteAdditionalTemplate(template?.id)
          .subscribe(res => {
            if (res) {
              this.globalMessageService.pushAlert({
                closed: false,
                type: 'success',
                message: 'Additional privilges deleted successfully.'
              });
              this.getAllTemplates();
            }
          });
      }
    });
  }

  handleCreateAdditionalPermissions() {
    this.router.navigate(['/create_additional_permissions']);
  }

}
