import { Component, OnInit } from '@angular/core';
import { BackupService } from './backup.service';
import { Backups } from '../models/backups';
import * as moment from 'moment';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { GlobalMessageClass } from '../global-messages/global-messages';
import { Pagination } from '../models/pagination';
import { AuthenticationService } from '../login/authentication.service';
import { Router } from '@angular/router';
import { ErrorCodeEnum } from '../shared/emuns/error.enum';
import { LicenseErrorEnum } from '../shared/emuns/license.enum';

import { Popup } from '../Popup';
import { VersionService } from '../version/version.service';
import { ConfirmModalComponent } from '../shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { RolesAndPermissionService } from '../roles-and-permission/roles-and-permission.service';

@Component({
  selector: 'app-backups',
  templateUrl: './backups.component.html',
  styleUrls: ['./backups.component.css']
})
export class BackupsComponent implements OnInit {
  backupsList: any = {};
  takingBackup = false;
  pagination: Pagination = new Pagination();
  private dateFormat: string = 'dd/MM/yyyy HH:mm';

  loading: boolean = true;
  getAuth: Popup = new Popup();
  authPassword: string = '';
  selectedBackup: any = null;
  restoringBackup: boolean = false;
  showAuthError: boolean = false;
  reSubmit: boolean = false;
  backupObjectId: number;
  portalVersionInfo: string;
  pageSize = [10, 25, 50, 100, 200];
  sort: string;
  sortDir: string;

  error;
  isDarkMode : any;
  public alert: GlobalMessageClass;
  timeZone = '';
  disableSaveButton = false;

  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;

  constructor(
    private backUpService: BackupService,
    private globalMessageService: GlobalMessageService,
    private authenticationService: AuthenticationService,
    private versionService: VersionService,
    private router: Router,
    public dialog: MatDialog,
    private rolesPermissionService: RolesAndPermissionService
  ) {
    var localDateFormat = localStorage.getItem('dateFormat');
    if (typeof localDateFormat != 'undefined' || localDateFormat != null) {
      this.dateFormat = localDateFormat;
    }

    let timeZoneOffset = new Date().getTimezoneOffset();
    this.timeZone =
      'UTC' + (timeZoneOffset > 0 ? '-0' : '+0') + (timeZoneOffset / 60) * -1;
  }

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.getCRUDdata(60);
    this.disableSaveButton = false;
    this.getBackups();
    this.portalVersionInfo = this.versionService.portalVersion;
  }

  getCRUDdata(privId){
    this.rolesPermissionService.getCrudData(privId).subscribe((res: any) => {
      this.createPermission = res?.create_permission === 1 
      this.readPermission = res?.read_permission === 1 
      this.updatePermission = res?.update_permission === 1 
      this.deletePermission = res?.delete_permission === 1 
    });
  }

  private getBackups() {
    this.loading = true;
    this.backUpService
      .getBackupsList(
        this.pagination.itemsPerPage,
        this.pagination.currentPage,
        this.sort,
        this.sortDir
      )
      .subscribe(
        (backups: any) => {
          this.loading = false;
          if (backups.recordsTotal) {
            this.pagination.totalItems = backups.recordsTotal;
            this.backupsList = backups;
          } else {
            this.backupsList = backups;
          }
        },
        err => {
          this.loading = false;
          if (err.status === ErrorCodeEnum.Forbidden) {
            if (err._body == LicenseErrorEnum.Invalid) {
              this.authenticationService.showLicenseOnly(true);
              this.router.navigate(['/license']);
            } else {
              // this.showAlert(
              //   'error',
              //   'You are not authorized to access this page.',
              //   false
              // );
            }
          } else {
            // this.showAlert('danger', err._body, false);
          }
        }
      );
  }

  public showPurgePopup(backupObject) {
    this.getAuth.body = 'Please enter your current password for verification.';
    this.getAuth.heading = 'Verification';
    this.getAuth.action = 'Confirm';
    this.getAuth.show = true;
    this.showAuthError = false;
    this.backupObjectId = backupObject.id;
  }

  backUpHandler() {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: 'Are you sure you want to take backup?'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.disableSaveButton = true;
        this.takingBackup = true;
        this.backUpService.takeBackup(this.portalVersionInfo).subscribe(
          (response: any) => {
            // backup.createdAt = moment(backup.createdAt)
            //   .local()
            //   .format('Do MMMM YYYY, h:m A');
            // this.backupsList.push(backup);
            this.downloadHandler(response);
            this.pagination.currentPage = 1;
            this.getBackups();
            this.takingBackup = false;
          },
          error => {
            this.takingBackup = false;
            this.disableSaveButton = false;
            // this.showAlert('danger', 'Error in taking backup', false);
          }
        );
      }
    });
  }

  downloadHandler(backupObject: Backups) {
    console.log('backupObject ', backupObject);
    this.backUpService.downloadSpecificBackup(backupObject.id);
  }

  restoreHandler(backupObject: Backups) {
    this.showPurgePopup(backupObject);
  }

  cancelPopup() {
    this.getAuth.show = false;
  }

  notifyUser(action, type, message) {
    this.restoringBackup = action;
    this.alert = {
      closed: false,
      type: type,
      message: message
    };
    this.globalMessageService.pushAlert(this.alert);
  }

  authAction(authPassword) {
    this.authPassword = authPassword;
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message:
          'Warning: The DB will be restored to a previous state. Changes after that backup will be lost'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.getAuth.show = false;
        this.notifyUser(true, 'warning', 'Restoring Backup, Please Wait!');
        this.backUpService
          .restoreBackup(this.backupObjectId, this.authPassword)
          .subscribe(
            response => {
              this.notifyUser(
                false,
                'success',
                'Backup has been restored successfully.'
              );
              this.authPassword = '';
            },
            error => {
              this.restoringBackup = false;
              this.authPassword = '';
              this.showAlert('danger', 'Error in restoring backup', false);
            }
          );
      }
    });
  }

  deleteHandler(backupObject: Backups) {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: 'Are you sure you want to delete this backup?'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.backUpService.deleteBackup(backupObject.id).subscribe(
          response => {
            this.alert = {
              closed: false,
              type: 'success',
              message: 'Backup has been deleted successfully.'
            };
            this.globalMessageService.pushAlert(this.alert);
            this.getBackups();
          },
          error => {
            this.globalMessageService.pushAlert({
              type: 'danger',
              closed: false,
              message: 'Error in deleting backup'
            });
          }
        );
      }
    });
  }

  pageChange(page) {
    this.pagination.currentPage = page;
    this.getBackups();
  }

  private showAlert(type: string, message: string, sticky: boolean) {
    const alert = {
      closed: false,
      type: type,
      message: message,
      sticky: sticky
    };
    this.globalMessageService.pushAlert(alert);
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getBackups();
  }

  sortRecords(term: string) {
    this.sortDir = this.sortDir == 'asc' ? 'desc' : 'asc';
    this.sort = term;
    this.getBackups();
  }
}
