import { Injectable } from '@angular/core';
import { HttpClient as AuthHttp } from '@angular/common/http';
import { ConfigService } from '../services/config.service';
import * as moment from 'moment';
import { AuthShortlivedCookieService } from '../auth-shortlived-cookie.service';
import { Pagination } from '../models/pagination';
import { map } from 'rxjs/operators';

interface date {
  year: string;
  month: string;
  day: string;
}
@Injectable()
export class ClientsReportsService {
  private apiBaseUrl: string;
  private reportingApiUrl: string = '/reports';

  constructor(
    private authHttp: AuthHttp,
    private congigService: ConfigService,
    private authShortlivedCookie: AuthShortlivedCookieService
  ) {
    this.apiBaseUrl = this.congigService.getConfiguration().apiBaseUrl;
    this.reportingApiUrl = this.apiBaseUrl + this.reportingApiUrl;
  }

  getServicesSummary(
    pagination: Pagination,
    startDate: Date,
    endDate: Date,
    services: Array<any>,
    clients: Array<any>,
    dir: string,
    sort: string
  ): any {
    const currentPage = pagination.currentPage - 1 || 0;
    const itemsPerPage = pagination.itemsPerPage || 10;

    let _startDate = '';
    let _endDate = '';
    let _services = '';
    let _clients = '';

    if (services && services.length) {
      _services = JSON.stringify(services.map(v => v.service_id));
    }

    if (clients && clients.length) {
      _clients = JSON.stringify(clients.map(v => v.agent_id));
    }
    if (startDate) {
      _startDate = moment(startDate).format('YYYY-MM-DD');
    }
    if (endDate) {
      _endDate = moment(endDate).format('YYYY-MM-DD');
    }

    return this.authHttp
      .get(
        this.reportingApiUrl +
          '/services/summary?offset=' +
          currentPage +
          '&limit=' +
          itemsPerPage +
          '&serviceId=' +
          _services +
          '&clientId=' +
          _clients +
          '&startDate=' +
          _startDate +
          '&endDate=' +
          _endDate +
          '&dir=' +
          dir +
          '&sort=' +
          sort
      )
      .pipe(map(response => response));
  }

  getServicesReports(
    pagination: Pagination,
    startDate: Date,
    endDate: Date,
    dir: string,
    sort: string
  ): any {
    const currentPage = pagination.currentPage - 1 || 0;
    const itemsPerPage = pagination.itemsPerPage || 10;
    let _startDate = '';
    let _endDate = '';

    if (startDate) {
      _startDate = moment(startDate).format('YYYY-MM-DD');
    }
    if (endDate) {
      _endDate = moment(endDate).format('YYYY-MM-DD');
    }

    return this.authHttp
      .get(
        this.reportingApiUrl +
          '/services/reports?offset=' +
          currentPage +
          '&limit=' +
          itemsPerPage +
          '&startDate=' +
          _startDate +
          '&endDate=' +
          _endDate +
          '&dir=' +
          dir +
          '&sort=' +
          sort
      )
      .pipe(map(response => response));
  }

  getServicesSummaryGraph(
    startDate: Date,
    endDate: Date,
    services: Array<any>,
    clients: Array<any>
  ): any {
    let _startDate = '';
    let _endDate = '';
    let _services = '';
    let _clients = '';

    if (services && services.length) {
      _services = JSON.stringify(services.map(v => v.service_id));
    }

    if (clients && clients.length) {
      _clients = JSON.stringify(clients.map(v => v.agent_id));
    }
    if (startDate) {
      _startDate = moment(startDate).format('YYYY-MM-DD');
    }
    if (endDate) {
      _endDate = moment(endDate).format('YYYY-MM-DD');
    }

    return this.authHttp
      .get(
        this.reportingApiUrl +
          '/services/summary/graph?graph=true' +
          '&serviceId=' +
          _services +
          '&clientId=' +
          _clients +
          '&startDate=' +
          _startDate +
          '&endDate=' +
          _endDate
      )
      .pipe(
        map(response => response),
        map((result: any) =>
          result.map(data => ({
            name: data['Service.Name'] || 'Unknown',
            total_users: data.total_users,
            value: Number(data.total_volume).toFixed(2)
          }))
        )
      );
  }

  getUsersSummaryGraph(
    startDate: Date,
    services: Array<any>,
    clients: Array<any>
  ): any {
    let _startDate = '';
    let _services = '';
    let _clients = '';

    if (services && services.length) {
      _services = JSON.stringify(services.map(v => v.service_id));
    }

    if (clients && clients.length) {
      _clients = JSON.stringify(clients.map(v => v.agent_id));
    }
    if (startDate) {
      _startDate = moment(startDate).format('YYYY-MM-DD');
    }

    return this.authHttp
      .get(
        this.reportingApiUrl +
          '/users/services/summary/graph?' +
          'serviceId=' +
          _services +
          '&clientId=' +
          _clients +
          '&startDate=' +
          _startDate
      )
      .pipe(map(response => response));
  }

  getUsersSummary(
    pagination: Pagination,
    startDate: Date,
    endDate: Date,
    services: Array<any>,
    clients: Array<any>,
    dir: string,
    sort: string
  ): any {
    const currentPage = pagination.currentPage - 1 || 0;
    const itemsPerPage = pagination.itemsPerPage || 10;

    let _startDate = '';
    let _endDate = '';
    let _services = '';
    let _clients = '';
    if (services && services.length) {
      _services = JSON.stringify(services.map(v => v.service_id));
    }

    if (clients && clients.length) {
      _clients = JSON.stringify(clients.map(v => v.agent_id));
    }
    if (startDate) {
      _startDate = moment(startDate).format('YYYY-MM-DD');
    }
    if (endDate) {
      _endDate = moment(endDate).format('YYYY-MM-DD');
    }

    return this.authHttp
      .get(
        this.reportingApiUrl +
          '/users/services/summary?offset=' +
          currentPage +
          '&limit=' +
          itemsPerPage +
          '&serviceId=' +
          _services +
          '&clientId=' +
          _clients +
          '&startDate=' +
          _startDate +
          '&endDate=' +
          _endDate +
          '&dir=' +
          dir +
          '&sort=' +
          sort
      )
      .pipe(map(response => response));
  }

  getAllServices() {
    return this.authHttp
      .get(this.reportingApiUrl + '/services')
      .pipe(map(response => response));
  }

  getAllClients() {
    return this.authHttp
      .get(this.reportingApiUrl + '/clients')
      .pipe(map(response => response));
  }
}
