import { Component, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { ControllerService } from '../controller.service';
import { Controller } from '../../models/controller';
import { GlobalMessageClass } from '../../global-messages/global-messages';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { DashboardService } from '../../dashboard/dashboard.service';
import { AuthenticationService } from '../../login/authentication.service';
import { tap } from 'rxjs/operators';
import { ErrorCodeEnum } from '../../shared/emuns/error.enum';
import { DeviceOSEnum } from '../../shared/emuns/client.enum';
import { ContextHelpEnum } from '../../shared/emuns/global.enum';
import { LicenseErrorEnum } from '../../shared/emuns/license.enum';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-create-controller',
  templateUrl: './create-controller.component.html',
  styleUrls: []
})
export class CreateControllerComponent implements OnInit {
  public alert: GlobalMessageClass;
  controller = new Controller();
  selectedProtocol: string = 'tcp';
  protocols = [
    { value: 'tcp', display: 'TCP' },
    { value: 'udp', display: 'UDP' }
  ];
  public pageTitle: string = 'Add Controller';

  //Contextual Help
  public contextualHelp: Array<any>;
  public showContext: boolean = false;
  //Contextual Help
  public numberOfControllers: number = 0;

  public ipLengthError: boolean = false;
  public submitting: boolean = false;

  loading: boolean = false;
  editMode: boolean = false;
  disableSaveButton = false;

  controllerOs = [
    { value: DeviceOSEnum.UbuntuServer18, display: ' Ubuntu Server' },
    { value: DeviceOSEnum.UbuntuServer20armv8, display: 'Ubuntu Server (ARMv8)' }
    // { value: DeviceOSEnum.CentosServer, display: 'CentOS Server'}
  ];

  constructor(
    private router: Router,
    private route: ActivatedRoute,
    private controllerService: ControllerService,
    private globalMessageService: GlobalMessageService,
    private dashboardService: DashboardService,
    private authenticationService: AuthenticationService,
    private titleService: Title
  ) {
    this.contextualHelp = ['', ''];
  }

  showContextHelp(name: any): void {
    if (name == ContextHelpEnum.Name) {
      this.contextualHelp = ['Name', 'Descriptive name of the controller.'];
      this.showContext = true;
    } else if (name == ContextHelpEnum.email) {
      this.contextualHelp = [
        'email',
        'Email address of admin user responsible for controller deployment. Controller package will be sent to this user.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.ipv4_address) {
      this.contextualHelp = [
        'ipv4_address',
        'IP of the machine where controller will be deployed.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.port) {
      this.contextualHelp = [
        'port',
        'Port where the controller will be listening for requests.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.public_ipv4_address) {
      this.contextualHelp = [
        'public_ipv4_address',
        'List of public IP separated with comma of the controller.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.public_ipv4_port) {
      this.contextualHelp = [
        'public_ipv4_port',
        'Forwarded public port of the controller.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.spa_port) {
      this.contextualHelp = [
        'spa_port',
        'UDP port where controller will be listening for SPA requests.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.device_os) {
      this.contextualHelp = [
        'device_os',
        'OS of the machine where the controller is to be deployed.'
      ];
      this.showContext = true;
    }
  }

  closeContextualHelp(): void {
    this.showContext = false;
  }

  getPage() {
    this.loading = true;
    if (typeof this.route.params['_value']['id'] != 'undefined') {
      this.pageTitle = 'Edit Controller';
      this.editMode = true;
      this.controllerService
        .getcontrollerdetail(this.route.params['_value']['id'])
        .subscribe(
          controller => {
            this.loading = false;
            if (!controller[0]) {
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'No controller found with the specified ID.'
              };
              this.globalMessageService.pushAlert(this.alert);
              this.router.navigate(['/controllers']);
            }
            this.controller = controller[0];
            this.selectedProtocol = this.controller.protocol;
          },
          err => {
            this.loading = false;
          }
        );
    } else {
      this.dashboardService
        .countHost()
        .pipe(
          tap((res: any) => {
            this.loading = false;
            if (res.status) {
              this.numberOfControllers = res.data;
              if (this.numberOfControllers > 0) {
                this.alert = {
                  closed: false,
                  type: 'danger',
                  message: 'You cannot create more than one controller.'
                };
                this.globalMessageService.pushAlert(this.alert);
                this.router.navigate(['/controllers']);
              }
            } else {
              this.numberOfControllers = 0;
            }
          })
        )
        .subscribe({
          next: next => {},
          error: err => {
            this.loading = false;
            if (err.status === ErrorCodeEnum.Forbidden) {
              if (err._body == LicenseErrorEnum.Invalid) {
                this.authenticationService.showLicenseOnly(true);
                this.router.navigate(['/license']);
              } else {
                this.alert = {
                  closed: false,
                  type: 'danger',
                  message: 'You are not authorized to access this page.'
                };
                this.globalMessageService.pushAlert(this.alert);
              }
            }
          },
          complete: () => {}
        });
    }
  }

  ngOnInit() {
    this.disableSaveButton = false;
    this.getPage();
    this.titleService.setTitle('Infrastructure | Controllers | ' + this.pageTitle);
  }

  publicIPAdded() {
    if (this.controller.public_ipv4_address.split(',').length > 5) {
      this.ipLengthError = true;
    } else {
      this.ipLengthError = false;
    }
  }

  onsubmit() {
    this.loading = true;
    this.submitting = true;
    // this.controller.device_os = DeviceOSEnum.Ubuntu;
    this.controller.device_hardware_id = 'thisistestandfakeid';
    this.controller.port = this.controller.public_ipv4_port;
    this.controller.protocol = this.selectedProtocol;
    if (this.controller.public_ipv4_address.split(',').length > 5) {
      this.ipLengthError = true;
    } else {
      this.ipLengthError = false;
    }

    if (this.ipLengthError) {
      this.loading = false;
      this.submitting = false;
      this.alert = {
        closed: false,
        type: 'danger',
        message: 'Public IP address must not be greater than 5.'
      };
      this.globalMessageService.pushAlert(this.alert);
    } else {
      this.disableSaveButton = true;
      this.controllerService.add_controller(this.controller).subscribe(
        response => {
          this.loading = false;
          if (response.success) {
            this.alert = {
              closed: false,
              type: 'success',
              message: 'Controller added/updated successfully.'
            };
            this.globalMessageService.pushAlert(this.alert);
            this.router.navigate(['/controllers']);
          } else {
            // this.alert = {
            //   closed: false,
            //   type: 'danger',
            //   message: 'Failed to add/update the controller.'
            // };
            // this.globalMessageService.pushAlert(this.alert);
          }
        },
        err => {
          this.loading = false;
          this.disableSaveButton = false;
          this.submitting = false;
          // this.alert = {
          //   closed: false,
          //   type: 'danger',
          //   message: 'Failed to add/update the controller.'
          // };
          // this.globalMessageService.pushAlert(this.alert);
        },
        () => {
          this.submitting = false;
        }
      );
    }
  }

  onCancel(): void {
    this.router.navigate(['/controllers']);
  }

  validateUrl() {
    if (!this.controller.onboarding_url) return true;
    const parts = this.controller.onboarding_url;
    return parts.split(':').length <= 2;
  }
}
