import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { Pagination } from '../../models/pagination';
import { WhitelistedHost } from '../../models/whitelisted-host';
import { ControllerService } from '../controller.service';
import { ConfirmModalComponent } from '../../../app/shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';

@Component({
  selector: 'app-whitelisted-hosts',
  templateUrl: './whitelisted-hosts.component.html',
  styleUrls: ['./whitelisted-hosts.component.css']
})
export class WhitelistedHostsComponent implements OnInit {
  hosts: WhitelistedHost[] = [];
  allHosts: WhitelistedHost[] = [];
  pagination: Pagination = new Pagination();
  addFormShown: boolean = false;
  isLoading: boolean = false;
  formLoading: boolean = false;

  error: string;
  success: string;
  editError: string;

  // Form related state
  hostForm: WhitelistedHost = new WhitelistedHost();
  hostEditForm: WhitelistedHost;

  controllerId = null;
  editing: boolean = false;
  editHostId = null;
  disableSaveButton = false;

  protocols = [
    { value: 'tcp', display: 'TCP' },
    { value: 'udp', display: 'UDP' }
  ];
  isDarkMode : boolean;
  constructor(
    private router: Router,
    private route: ActivatedRoute,
    private services: ControllerService,
    private messageService: GlobalMessageService,
    public dialog: MatDialog
  ) {
    this.pagination.itemsPerPage = 5;
  }

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.disableSaveButton = false;
    this.getAllHosts();
  }

  getAllHosts() {
    this.route.params.subscribe(param => {
      if (param.id) {
        this.controllerId = param.id;
        this.isLoading = true;
        this.services.getWhiteListedHostes(param.id).subscribe({
          next: (response: any) => {
            this.isLoading = false;
            this.editing = true;
            this.allHosts = response.data;
            this.pagination.currentPage = 1;
            this.pagination.totalItems = this.allHosts.length;
            this.getPaginatedHosts();
          },
          error: () => {
            this.isLoading = false;
          }
        });
      }
    });
  }

  getPaginatedHosts = () => {
    const skip =
      this.pagination.itemsPerPage * (this.pagination.currentPage - 1);
    this.hosts = this.allHosts.slice(skip, skip + this.pagination.itemsPerPage);
  };

  showAddForm() {
    this.editHostId = null;
    this.addFormShown = true;
  }

  hideAddForm() {
    this.addFormShown = false;
  }

  showEditHostForm(hostId) {
    this.addFormShown = true;
    this.editHostId = hostId;
    const index = this.allHosts.findIndex(host => host.id === hostId);
    if (index > -1) {
      this.hostForm = JSON.parse(JSON.stringify(this.allHosts[index]));
    }
  }

  resetAddFormMessages() {
    this.success = '';
    this.error = '';
  }

  resetAddFormFields() {
    this.hostForm = new WhitelistedHost();
  }

  resetAddForm() {
    this.resetAddFormFields();
    this.resetAddFormMessages();
    this.hideAddForm();
  }

  changePage(currentPage) {
    this.pagination.currentPage = currentPage;
    this.getPaginatedHosts();
  }

  handleSave() {
    this.disableSaveButton = true;
    if (this.editHostId) {
      this.editHost();
    } else {
      this.createNewHost();
    }
  }

  editHost() {
    if (this.controllerId) {
      const { port, protocol, host } = this.hostForm;
      this.formLoading = true;
      this.services
        .updateWhiteListedHostes(
          port,
          protocol,
          host,
          this.editHostId,
          this.controllerId
        )
        .subscribe({
          next: _response => {
            this.formLoading = false;
            const alert = {
              closed: false,
              type: 'success',
              message: 'WhiteListed host updated successfully.'
            };
            this.messageService.pushAlert(alert);
            this.addFormShown = false;
            this.updateHostLocally();
            this.hostForm = new WhitelistedHost();
            this.getPaginatedHosts();
            this.disableSaveButton = false;
          },
          error: err => {
            this.disableSaveButton = false;
            this.formLoading = false;
          }
        });
    } else {
      this.formLoading = true;
      this.updateHostLocally();
      this.formLoading = false;
      this.addFormShown = false;
      this.hostForm = new WhitelistedHost();
      this.getPaginatedHosts();
    }
  }

  updateHostLocally() {
    const index = this.allHosts.findIndex(host => host.id === this.editHostId);
    if (index > -1) {
      this.allHosts[index] = this.hostForm;
    }
  }

  createNewHost() {
    if (this.controllerId) {
      const { port, protocol, host } = this.hostForm;
      this.formLoading = true;
      this.services
        .createWhiteListedHostes(port, protocol, host, this.controllerId)
        .subscribe({
          next: _response => {
            this.formLoading = false;
            const alert = {
              closed: false,
              type: 'success',
              message: 'WhiteListed host created successfully.'
            };
            this.messageService.pushAlert(alert);
            this.pagination.currentPage = 1;
            this.addFormShown = false;
            this.getAllHosts();
            this.hostForm = new WhitelistedHost();
            this.disableSaveButton = false;
          },
          error: () => {
            this.disableSaveButton = false;
            this.formLoading = false;
          }
        });
    } else {
      this.addToExistingHosts();
    }
  }

  addToExistingHosts() {
    const { port, protocol, host } = this.hostForm;
    const index = this.allHosts.findIndex(host => host.port === port);
    if (index > -1) {
      const alert = {
        closed: false,
        type: 'danger',
        message: 'Port already occupied.'
      };
      this.messageService.pushAlert(alert);
      return;
    }
    const tempWhiteListedHost = new WhitelistedHost();
    tempWhiteListedHost.id = this.allHosts.length + 1;
    tempWhiteListedHost.port = port;
    tempWhiteListedHost.host = host;
    tempWhiteListedHost.protocol = protocol;
    this.allHosts = [...this.allHosts, tempWhiteListedHost];
    this.getPaginatedHosts();
    this.addFormShown = false;
    this.hostForm = new WhitelistedHost();
  }

  deleteWhiteListedHost(hostId) {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: `Are you confirm want to delete?`,
      },
    });

    dialogRef.afterClosed().subscribe((result) => {
      if (result){
        if (this.controllerId) {
          this.services.deleteWhiteListedHostes(hostId).subscribe(_response => {
            this.messageService.pushAlert({
              closed: false,
              type: 'success',
              message: 'Host deleted successfully.'
            });
            this.deleteLocalHost(hostId);
            this.getPaginatedHosts();
          });
        } else {
          this.deleteLocalHost(hostId);
          this.getPaginatedHosts();
        }
      }
    });
  }

  deleteLocalHost(hostId) {
    this.allHosts = this.allHosts.filter(
      whiteListHost => whiteListHost.id !== hostId
    );
    this.pagination.totalItems = this.allHosts.length;
  }
}
