import { Injectable } from '@angular/core';
import {
  HttpClient as AuthHttp,
  HttpHeaders,
  HttpRequest
} from '@angular/common/http';
import { map } from 'rxjs/operators';
import { ConfigService } from '../services/config.service';

@Injectable()
export class LocalUserService {
  private apiBaseUrl: string;
  private localUserUrl = '/localUsers';
  private serviceUrl = '/services';
  httpOptions = {
    headers: new HttpHeaders({
      'Content-Type': 'application/json'
    })
  };

  constructor(
    private authHttp: AuthHttp,
    private configService: ConfigService
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.localUserUrl = this.apiBaseUrl + this.localUserUrl;
  }

  getLocalUsers(
    pagination,
    sort: string = 'id',
    dir: string = 'desc',
    searchKey = '',
    identityProvider = '',
    organization = '',
    department = ''
  ) {
    let query =
      `?limit=${pagination.itemsPerPage}` +
      `&offset=${pagination.currentPage - 1}` +
      `&sort=${sort}` +
      `&dir=${dir}` +
      `&searchTerm=${searchKey}` +
      `&identityProvider=${identityProvider}` +
      `&organization=${organization}` +
      `&department=${department}`;

    return this.authHttp
      .get(`${this.localUserUrl}${query}`)
      .pipe(map(response => response));
  }

  getLocalUsersFromDropDown(
    pagination,
    sort: string = 'id',
    dir: string = 'desc',
    searchKey = '',
    identityProvider = '',
    organization = '',
    department = '',
    allUser?
  ) {
    let query =
      `?limit=${pagination.itemsPerPage}` +
      `&offset=${pagination.currentPage - 1}` +
      `&sort=${sort}` +
      `&dir=${dir}` +
      `&searchTerm=${searchKey}` +
      `&identityProvider=${identityProvider}` +
      `&organization=${organization}` +
      `&department=${department}` +
      `&allUser=${allUser}`;

    return this.authHttp
      .get(`${this.localUserUrl}/getLocalUsersFromDropDown${query}`)
      .pipe(map(response => response));
  }

  getUserGroupsPagination(
    pagination,
    sort: string = 'id',
    dir: string = 'desc'
  ) {
    let query =
      `?limit=${pagination.itemsPerPage}` +
      `&offset=${pagination.currentPage - 1}` +
      `&sort=${sort}` +
      `&dir=${dir}`;

    return this.authHttp
      .get(`${this.localUserUrl}/paginationGroups/${query}`)
      .pipe(map(response => response));
  }

  getAvailableLocalUsers(departmentId) {
    return this.authHttp
      .get(`${this.localUserUrl}/available/${departmentId}`)
      .pipe(map(response => response));
  }

  getLocalUserServices(id) {
    return this.authHttp.get(this.localUserUrl + `/${id}` + this.serviceUrl);
  }

  getCustomUserGroups() {
    return this.authHttp
      .get(`${this.localUserUrl}/getCustomUserGroups`)
      .pipe(map(response => response));
  }

  handleBulkdelete(groupData) {
    return this.authHttp
      .delete(`${this.localUserUrl}/handleBulkdelete`, { body: groupData })
      .pipe(map(response => response));
  }

  handleBulkdeleteCurrent(groupData) {
    return this.authHttp
      .delete(`${this.localUserUrl}/handleBulkdeleteCurrentPage`, { body: groupData })
      .pipe(map(response => response));
  }

  addCustomUserGroups(data) {
    return this.authHttp
      .post(`${this.localUserUrl}/customUserGroups`, { localUser: data })
      .pipe(map(response => response));
  }

  updateUserGroup(data, id) {
    return this.authHttp
      .put(`${this.localUserUrl}/updateUserGroup/${id}`, { localUser: data })
      .pipe(map(response => response));
  }

  getCustomGroupData() {
    return this.authHttp
      .get(`${this.localUserUrl}/customGroupData`)
      .pipe(map(response => response));
  }

  deleteCustomGroupData(id) {
    return this.authHttp
      .delete(`${this.localUserUrl}/deleteCustomGroup/${id}`)
      .pipe(map(response => response));
  }

  deleteUserFromCustomGoup(userId, groupId) {
    return this.authHttp
      .delete(
        `${this.localUserUrl}/deleteUserFromCustomGoup/${userId}/${groupId}`
      )
      .pipe(map(response => response));
  }

  updateCustomGroupData(data, id) {
    return this.authHttp
      .put(`${this.localUserUrl}/updateCustomGroup/${id}`, { data })
      .pipe(map(response => response));
  }

  getCustomUserGroupData(id) {
    return this.authHttp
      .get(`${this.localUserUrl}/getCustomUserGroupData/${id}`)
      .pipe(map(response => response));
  }

  searchUsers(
    searchQuery: string,
    pageSize: number = 10,
    page: number = 1,
    sort: string = 'id',
    dir: string = 'desc',
    identityProvider = '',
    organization = '',
    department = ''
  ) {
    const encodedUrlComponent = encodeURIComponent(searchQuery);
    let query =
      `?limit=${pageSize}` +
      `&offset=${page - 1}` +
      `&sort=${sort}` +
      `&dir=${dir}` +
      `&searchTerm=${encodedUrlComponent}` +
      `&identityProvider=${identityProvider}` +
      `&organization=${organization}` +
      `&department=${department}`;
    return this.authHttp
      .get(`${this.localUserUrl}${query}`)
      .pipe(map(response => response));
  }

  searchUsersJQL(
    searchQuery: string = '',
    pageSize: number = 10,
    page: number = 1,
    sort: string = 'id',
    dir: string = 'desc',
    groupId?: number
  ) {
    const encodedUrlComponent = encodeURIComponent(searchQuery);
    let query =
      `?limit=${pageSize}` +
      `&offset=${page - 1}` +
      `&sort=${sort}` +
      `&dir=${dir}` +
      `&searchTerm=${encodedUrlComponent}` +
      `&groupId=${groupId}`;
    return this.authHttp
      .get(`${this.localUserUrl}/jql${query}`)
      .pipe(map(response => response));
  }

  getSingleLocalUser(id) {
    return this.authHttp
      .get(`${this.localUserUrl}/${id}`)
      .pipe(map(response => response));
  }

  getUserDeviceRelatedPolicies(userId, deviceId) {
    return this.authHttp
      .get(`${this.localUserUrl}/device/policies/${userId}/${deviceId}`)
      .pipe(map(response => response));
  }

  deleteUser(id, userName, verificationPassword) {
    this.httpOptions['body'] = { verificationPassword };
    return this.authHttp
      .delete(`${this.localUserUrl}/${id}/${userName}`, this.httpOptions)
      .pipe(map(response => response));
  }

  deleteUserOtp(id, userName) {
    return this.authHttp
      .delete(`${this.localUserUrl}/${id}/${userName}/otp`)
      .pipe(map(response => response));
  }

  addLocalUser(localUser, editing, domainName,reachableDomainName) {
    let newLocalUser 

    if(localUser.identity_provider == 'Active Directory'){
       newLocalUser = {
        ...localUser,
        reachable_email : localUser.reachable_email + reachableDomainName
      };

    }else{
       newLocalUser = {
        ...localUser,
        email: localUser.email + domainName,
      };
    }
    if (!editing) {
      return this.authHttp
        .post(`${this.localUserUrl}`, { localUser: newLocalUser })
        .pipe(map(response => response));
    } else {
      if (!newLocalUser.password) {
        delete newLocalUser.password;
      }
      return this.authHttp
        .put(`${this.localUserUrl}/${newLocalUser.id}`, {
          localUser: newLocalUser
        })
        .pipe(map(response => response));
    }
  }

  addDeviceToUser(localUserId, deviceName) {
    return this.authHttp
      .put(`${this.localUserUrl}/${localUserId}/device`, {
        deviceName,
        operation: 'add'
      })
      .pipe(map(response => response));
  }

  removeDeviceFromUser(localUserId, deviceId) {
    return this.authHttp
      .put(`${this.localUserUrl}/${localUserId}/device`, {
        deviceId,
        operation: 'remove'
      })
      .pipe(map(response => response));
  }

  sendResetPasswordLink(user) {
    const { id, name, email } = user;
    return this.authHttp
      .post(`${this.localUserUrl}/sendResetLink`, { id, name, email })
      .pipe(map(response => response));
  }

  bulkPasswordChange(
    selectedUsers,
    selectAllExcept,
    selectAll,
    search,
    verificationPassword
  ) {
    return this.authHttp.put(`${this.localUserUrl}/bulk/password-change`, {
      selectAllExcept: Array.from(selectAllExcept),
      selectedUsers: Array.from(selectedUsers),
      selectAll,
      verificationPassword,
      search
    });
  }

  bulkDeleteUsers(
    selectAll: boolean = false,
    selectedIds,
    idp: string = '',
    verificationPassword
  ) {
    return this.authHttp
      .put(`${this.localUserUrl}/bulk/delete-users`, {
        selectAll,
        selectedIds: Array.from(selectedIds),
        verificationPassword,
        idp
      })
      .pipe(map(response => response));
  }

  bulkActions(
    actionType: string = '',
    status: boolean = false,
    selectAll: boolean = false,
    selectedIds,
    organization: string = '',
    department: string,
    searchText: string = '',
    idp: string = '',
    verificationPassword,
    mfa
  ) {
    return this.authHttp
      .put(`${this.localUserUrl}/bulk-actions`, {
        actionType,
        status,
        selectAll,
        selectedIds: Array.from(selectedIds),
        department,
        searchText,
        idp,
        organization,
        verificationPassword,
        mfa
      })
      .pipe(map(response => response));
  }


  deleteFromCurrentPage(selectedIds,verificationPassword) {
    return this.authHttp
      .put(`${this.localUserUrl}/bulk-actions-delete-current`, {
        selectedIds: Array.from(selectedIds),
        verificationPassword
      })
      .pipe(map(response => response));
  }

  deleteFromCurrentPageOtp(selectedIds) {
    return this.authHttp
      .put(`${this.localUserUrl}/bulk-actions-delete-current-otp`, {
        selectedIds: Array.from(selectedIds),
      })
      .pipe(map(response => response));
  }

  searchUsersGroup(
    searchQuery: string,
    pageSize: number = 10,
    page: number = 1,
    sort: string = 'id',
    dir: string = 'asc'
  ) {
    const encodedUrlComponent = encodeURIComponent(searchQuery);
    let query =
      `?limit=${pageSize}` +
      `&offset=${page - 1}` +
      `&sort=${sort}` +
      `&dir=${dir}` +
      `&searchTerm=${encodedUrlComponent}`;
    return this.authHttp
      .get(`${this.localUserUrl}/search${query}`)
      .pipe(map(response => response));
  }

  uploadCsvFile(fileData) {
    return this.authHttp
      .post(`${this.localUserUrl}/csv/upload`, { fileData: fileData })
      .pipe(map(response => response));
  }
}
