import { Component, OnInit, ViewChild } from '@angular/core';
import { OrganizationService } from '../organization-list-view/organization.service';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { GlobalMessageClass } from '../global-messages/global-messages';
import { Pagination } from '../models/pagination';
import { Subject, Observable } from 'rxjs';
import { NgModel } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { debounceTime, distinctUntilChanged, switchMap } from 'rxjs/operators';
import { ConfirmModalComponent } from '../shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';
import { RolesAndPermissionService } from '../roles-and-permission/roles-and-permission.service';

@Component({
  selector: 'app-create-organization',
  templateUrl: './create-organization.component.html',
  styleUrls: ['./create-organization.component.css']
})
export class CreateOrganizationComponent implements OnInit {
  fileData = {};
  isFileValid = false;
  departments: any = {};
  pagination = new Pagination();
  isUploading = false;
  $searchDepartment = new Subject();
  departmentEditForm;
  departmentAddForm;
  saveButtonDisabled = false;
  addDepartmentSaveButtonDisabled = true;
  editOrganizationButtonDisabled = false;
  addDepartment = false;
  searchTerm: String = null;

  loading: Boolean = false;

  sort: string;
  sortDir: string = 'desc';

  organizationId;
  organizationName;
  originalValue;
  name: string;
  editMode = false;
  viewMode = false;

  @ViewChild('editName', { static: false }) editName: NgModel;
  @ViewChild('addName', { static: false }) addName: NgModel;
  @ViewChild('organizationNameText', { static: false })
  organizationNameText: NgModel;

  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;

  pageSize = [10, 25, 50, 100, 200];

  public alert: GlobalMessageClass;
  isDarkMode : boolean;
  constructor(
    private organizationService: OrganizationService,
    private globalMessageService: GlobalMessageService,
    private route: ActivatedRoute,
    private router: Router,
    public dialog: MatDialog,
    private titleService: Title,
    private rolesPermissionService: RolesAndPermissionService
  ) {}

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.viewMode = this.router.url.includes('/view-organization');
    this.getCRUDdata(47);
    if (this.route.snapshot.params.id) {
      this.organizationId = +this.route.snapshot.params.id;
      this.editMode = true;
      this.organizationService
        .getOrganization(this.organizationId)
        .subscribe((organization: any) => {
          this.organizationName = organization.name;
          this.originalValue = this.organizationName;
        });
      this.getDepartments();
      this.subscribeToSearch();
    } else {
      this.editMode = false;
      this.organizationId = null;
      this.organizationName = null;
      this.originalValue = null;
    }
    if (this.viewMode) {
      this.titleService.setTitle('Organization | View Organization');
    }
    else if (this.editMode) {
      this.titleService.setTitle('Organization | Edit Organization');
    }
    else {
      this.titleService.setTitle('Organization | Add Organization');
    }
  }

  getCRUDdata(privId){
    this.rolesPermissionService.getCrudData(privId).subscribe((res: any) => {
      this.createPermission = res?.create_permission === 1 ? true : false
      this.readPermission = res?.read_permission === 1 ? true : false
      this.updatePermission = res?.update_permission === 1 ? true : false
      this.deletePermission = res?.delete_permission === 1 ? true : false
    });
  }

  subscribeToSearch() {
    this.$searchDepartment
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((search: { searchWord: string; typing: Boolean }) => {
          if (search.typing) {
            this.pagination.currentPage = 1;
          }
          return this.organizationService.searchDepartment(
            this.organizationId,
            search.searchWord,
            this.pagination.itemsPerPage,
            this.pagination.currentPage,
            this.sort,
            this.sortDir
          );
        })
      )
      .subscribe((departments: any) => {
        this.loading = false;
        this.departments = departments;
        this.pagination.totalItems = departments.recordsTotal;
      });
  }

  getDepartments() {
    this.loading = true;
    this.organizationService
      .getDepartments(
        this.organizationId,
        this.pagination.itemsPerPage,
        this.pagination.currentPage,
        this.sort,
        this.sortDir
      )
      .subscribe(
        (departments: any) => {
          this.loading = false;
          this.departments = departments;
          this.pagination.totalItems = departments.recordsTotal;
        },
        error => {
          this.loading = false;
        }
      );
  }

  pageChange(event) {
    this.pagination.currentPage = event;
    if (this.searchTerm) {
      this.$searchDepartment.next({
        searchWord: this.searchTerm,
        typing: false
      });
    } else {
      this.getDepartments();
    }
  }

  deleteDepartment(department) {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: 'Do you really want to delete this department?',
      },
    });

    dialogRef.afterClosed().subscribe((result) => {
      if (result){
        if (this.editMode) {
          this.organizationService.deleteDepartment(department.id).subscribe(
            data => {
              this.departments.data = this.departments.data.filter(
                singleDepartment => {
                  return singleDepartment.id !== department.id;
                }
              );
  
              this.alert = {
                closed: false,
                type: 'success',
                message: 'Department has been deleted successfully.'
              };
              this.globalMessageService.pushAlert(this.alert);
  
              this.getDepartments();
            },
            error => {}
          );
        } else {
          this.departments.data = this.departments.data.filter(myDepartment => {
            return department.id !== myDepartment.id;
          });
        }
      }
    });
  }

  cancelEditDepartment() {
    this.departmentEditForm = null;
    this.saveButtonDisabled = false;
  }

  addDepartmentCancelHandler() {
    this.addDepartmentSaveButtonDisabled = true;
    this.addDepartment = false;
    this.departmentAddForm = null;
  }

  editHandler(department) {
    this.departmentEditForm = { ...department };
  }

  addDepartmentHandler() {
    this.addDepartment = true;
    this.departmentAddForm = {
      name: ''
    };
  }

  addDepartmentSaveHandler() {
    const { name } = this.departmentAddForm;
    let addedDepartment = {
      name: name.trim(),
      organizationId: this.organizationId
    };
    if (this.editMode) {
      this.organizationService.addNewDepartment(addedDepartment).subscribe(
        response => {
          this.addDepartmentSaveButtonDisabled = true;
          this.addDepartment = false;
          this.departmentAddForm = null;
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'success',
            message: 'Department added successfully.'
          });
          this.getDepartments();
        },
        error => {}
      );
    } else {
      let recordsTotal = this.departments.recordsTotal || 0;
      let data = this.departments.data || [];

      if (data.some(department => department.name === addedDepartment.name)) {
        this.globalMessageService.pushAlert({
          closed: false,
          type: 'danger',
          message: 'Department already exists in this organization.'
        });
        return;
      }
      recordsTotal = recordsTotal + 1;
      addedDepartment['id'] = recordsTotal;
      data = [...data, addedDepartment];
      this.departments = { recordsTotal, data };
      this.addDepartmentSaveButtonDisabled = true;
      this.addDepartment = false;
      this.departmentAddForm = null;
      this.globalMessageService.pushAlert({
        closed: false,
        type: 'success',
        message: 'Department added successfully.'
      });
    }
  }

  editInputChanged() {
    this.saveButtonDisabled = this.editName.invalid;
  }

  addInputChanged() {
    this.addDepartmentSaveButtonDisabled = this.addName.invalid;
  }

  updateDepartment(departmentParam) {
    if (this.editMode) {
      this.organizationService
        .updateDepartment(
          this.departmentEditForm.id,
          this.departmentEditForm.name
        )
        .subscribe(
          response => {
            const editedIndex = this.departments.data
              .map(department => {
                return department.id;
              })
              .indexOf(departmentParam.id);
            if (editedIndex != -1) {
              this.departments.data[editedIndex] = {
                ...this.departmentEditForm
              };
            }
            if (this.departmentEditForm.id === departmentParam.id) {
              this.saveButtonDisabled = false;
              this.departmentEditForm = null;
            }
          },
          error => {}
        );
    } else {
      const editedDepartment = this.departmentEditForm;
      this.departments.data = this.departments.data.map(department => {
        return department.id === editedDepartment.id
          ? editedDepartment
          : department;
      });
      this.saveButtonDisabled = false;
      this.departmentEditForm = null;
    }
  }

  searchDepartment(searchTerm: String) {
    this.loading = true;
    if (searchTerm) {
      this.$searchDepartment.next({ searchWord: searchTerm, typing: true });
    } else {
      this.pagination.currentPage = 1;
      this.getDepartments();
    }
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getDepartments();
  }

  sortRecords(term: string) {
    if (this.editMode) {
      this.sortDir = this.sortDir == 'asc' ? 'desc' : 'asc';
      this.sort = term;
      this.searchTerm == '' || this.searchTerm == null
        ? this.getDepartments()
        : this.$searchDepartment.next({
            searchWord: this.searchTerm,
            typing: false
          });
    }
  }

  goToOrganizationListing() {
    this.router.navigate(['/organizations']);
  }

  addOrganizationSaveHandler() {
    let addedOrganization = {
      name: this.organizationName.trim(),
      department_count: this.departments.data ? this.departments.data.length : 0
    };
    this.organizationService
      .addNewOrganization(addedOrganization, this.departments.data)
      .subscribe(response => {
        this.alert = {
          closed: false,
          type: 'success',
          message: 'Organization created successfully.'
        };
        this.globalMessageService.pushAlert(this.alert);
        this.goToOrganizationListing();
      });
  }

  editOrganizationHandler() {
    // if(this.originalValue == this.organizationName) {
    //   this.alert = {
    //     closed: false,
    //     type: 'danger',
    //     message: 'The Organization name has not been modified.'
    //   };
    //   this.globalMessageService.pushAlert(this.alert);
    // }
    // else {
    this.organizationService
      .updateOrganization(this.organizationId, this.organizationName)
      .subscribe(response => {
        this.alert = {
          closed: false,
          type: 'success',
          message: 'Organization name has been modified successfully.'
        };
        this.goToOrganizationListing();
        this.globalMessageService.pushAlert(this.alert);
      });
    // }
  }

  navigate(){}
}
