import { Injectable } from '@angular/core';
import { ConfigService } from '../services/config.service';
import { HttpClient as AuthHttp } from '@angular/common/http';
import { Pagination } from '../models/pagination';
import { map } from 'rxjs/operators';
import { DeviceOSEnum } from '../shared/emuns/client.enum';

const windowsOsVersion = [
  { value: 'any', display: 'Any' },
  { value: '6.1.7601', display: '6.1.7601 Windows 7' },
  { value: '6.2.9200', display: '6.2.9200 Windows 8' },
  { value: '6.3.9600', display: '6.3.9600 Windows 8.1' },
  { value: '10.0.18363', display: '10.0.18363 Windows 10' },
  { value: '10.0.22000', display: '10.0.22000 Windows 11' }
];

const macOsVersions = [
  { value: 'any', display: 'Any' },
  { value: '10.13.6', display: '10.13.6 macOS High Sierra' },
  { value: '10.14.6', display: '10.14.6 macOS Mojave' },
  { value: '10.15.4', display: '10.15.4 macOS Catalina' },
  { value: '10.15.5', display: '10.15.5 macOS Catalina' },
  { value: '10.15.6', display: '10.15.6 macOS Catalina' },
  { value: '11.0.0', display: '11.0.0 macOS Big Sur' }
];

const androidOsVersions = [
  { value: 'any', display: 'Any' },
  { value: '6.0', display: '6.0' },
  { value: '7.0', display: '7.0' },
  { value: '8.0', display: '8.0' },
  { value: '9.0', display: '9.0' },
  { value: '10.0', display: '10.0' },
  { value: '11.0', display: '11.0' }
];

const iOsOsVersions = [
  { value: 'any', display: 'Any' },
  { value: '8.0', display: '8.0' },
  { value: '9.0', display: '9.0' },
  { value: '10.0', display: '10.0' },
  { value: '11.0', display: '11.0' },
  { value: '12.0', display: '12.0' },
  { value: '13.0', display: '13.0' },
  { value: '14.0', display: '14.0' }
];

const ubuntu18OsVersions = [
  { value: 'any', display: 'Any' },
  { value: '20.10.0', display: '20.10.0' },
  { value: '20.04.2', display: '20.04.2' },
  { value: '20.04.1', display: '20.04.1' },
  { value: '20.04.0', display: '20.04.0' },
  { value: '18.04.5', display: '18.04.5' },
  { value: '18.04.4', display: '18.04.4' },
  { value: '18.04.3', display: '18.04.3' },
  { value: '18.04.2', display: '18.04.2' },
  { value: '18.04.1', display: '18.04.1' },
  { value: '18.04', display: '18.04.0' }
];

const ubuntu16OsVersions = [
  { value: 'any', display: 'Any' },
  { value: '16.04.6', display: '16.04.6' },
  { value: '16.04.5', display: '16.04.5' },
  { value: '16.04.4', display: '16.04.4' },
  { value: '16.04.3', display: '16.04.3' },
  { value: '16.04.2', display: '16.04.2' },
  { value: '16.04.1', display: '16.04.1' },
  { value: '16.04', display: '16.04' }
];

const ubuntuserver20armv8Versions = [
  { value: 'any', display: 'Any' },
  { value: '20.10.0', display: '20.10.0' },
  { value: '20.04.2', display: '20.04.2' },
  { value: '20.04.1', display: '20.04.1' },
  { value: '20.04.0', display: '20.04.0' }
];

@Injectable()
export class OsPolicyService {
  private policyApiUrl: string;
  constructor(
    private authHttp: AuthHttp,
    private configService: ConfigService
  ) {
    const baseUrl = configService.getConfiguration().apiBaseUrl;
    this.policyApiUrl = baseUrl + '/policies/os';
  }

  getAllOs(id) {
    return this.authHttp
      .get(this.policyApiUrl + '/exists' + `?id=${id}`)
      .pipe(map(response => response));
  }

  getAllPolicies(
    pagination: Pagination,
    name: string = '',
    selectedClientIds = [],
    sort: string = 'os',
    dir: string = 'asc',
    searchKey = '',
    allClientPolicies: boolean = false
  ) {
    const currentPage = pagination.currentPage - 1 || 0;
    const itemsPerPage = pagination.itemsPerPage || 10;

    let _name: string = searchKey;

    if (name.trim()) {
      _name = name.trim();
    }
    let _clients = selectedClientIds.join(',');
    return this.authHttp
      .get(
        this.policyApiUrl +
          `?offset=${currentPage}&limit=${itemsPerPage}&os=${_name}&sort=${sort}&dir=${dir}&clients=${_clients}&allClientPolicies=${allClientPolicies}`
      )
      .pipe(map(response => response));
  }

  deletePolicy(id) {
    return this.authHttp
      .delete(this.policyApiUrl + `/${id}`)
      .pipe(map(response => response));
  }

  enablePolicy(id, value) {
    return this.authHttp
      .patch(this.policyApiUrl + `/${id}/enable/${value}`, {})
      .pipe(map(response => response));
  }

  getSinglePolicy(id) {
    return this.authHttp
      .get(`${this.policyApiUrl}/${id}`)
      .pipe(map(response => response));
  }

  createService(policyOption) {
    return this.authHttp
      .post(`${this.policyApiUrl}/create`, policyOption)
      .pipe(map(response => response));
  }

  updateService(policyOption, id) {
    return this.authHttp.put(`${this.policyApiUrl}/${id}`, policyOption);
  }

  getAllOsVersions(device_os) {
    let clientOsVersion = [];

    if (device_os === DeviceOSEnum.Windows) {
      clientOsVersion = windowsOsVersion;
    } else if (device_os === DeviceOSEnum.Macintosh) {
      clientOsVersion = macOsVersions;
      // this.client.client_os_version = this.clientOsVersion[0].value;
    } else if (device_os === DeviceOSEnum.IOS) {
      clientOsVersion = iOsOsVersions;
    } else if (device_os === DeviceOSEnum.Android) {
      clientOsVersion = androidOsVersions;
    } else {
      if (device_os === DeviceOSEnum.Ubuntu || device_os === DeviceOSEnum.UbuntuServer) {
        clientOsVersion = ubuntu16OsVersions;
      } else if (device_os === DeviceOSEnum.Ubuntu18 || device_os === DeviceOSEnum.UbuntuServer18) {
        clientOsVersion = ubuntu18OsVersions;
      } else if (device_os === DeviceOSEnum.UbuntuServer20armv8) {
        clientOsVersion = ubuntuserver20armv8Versions;
      }
    }
    return clientOsVersion;
  }
}
