import { Injectable, Injector } from '@angular/core';
import {
  HttpClient,
  HttpHandler,
  HttpInterceptor,
  HttpErrorResponse,
  HttpResponse,
  HttpHeaders,
  HttpParams
} from '@angular/common/http';
import { Observable, throwError } from 'rxjs';
import { catchError, tap } from 'rxjs/operators';
import { GlobalMessageService } from './global-messages/global-messages.service';
import { AuthenticationService } from './login/authentication.service';
import { Router } from '@angular/router';
import { LicenseDaysEnum, LicenseViolationEnum } from './shared/emuns/license.enum'

@Injectable()
export class CustomHttpClient extends HttpClient {
  constructor(
    private httpHandler: HttpHandler,
    private globalMessageService: GlobalMessageService,
    private authenticationService: AuthenticationService,
    private injector: Injector
  ) {
    super(httpHandler);
  }

  private hideNavItems(_licenseDays = '-1') {
    if (
      _licenseDays !== LicenseDaysEnum.Processing &&
      _licenseDays !== LicenseDaysEnum.NoLicense &&
      _licenseDays !== LicenseDaysEnum.Invalid
    ) {
      const licenseDays = parseFloat(_licenseDays);
      const licenseDaysApprox = Math.floor(licenseDays);

      if (licenseDaysApprox === -1) {
        const authenticationService = this.injector.get(AuthenticationService);
        authenticationService.showLicenseOnly(true);
      }
    } else {
      const authenticationService = this.injector.get(AuthenticationService);
      authenticationService.showLicenseOnly(true);
    }
  }

  private showInformativeLicenseNotification(
    _licenseDays = '-1',
    expiringToday,
    expiringTomorrow,
    violated,
    controllerCount
  ) {
    if (
      _licenseDays !== LicenseDaysEnum.Processing &&
      _licenseDays !== LicenseDaysEnum.NoLicense &&
      _licenseDays != LicenseDaysEnum.Invalid
    ) {
      const licenseDays = parseFloat(_licenseDays);
      const licenseDaysApprox = Math.floor(licenseDays);

      if (licenseDays > 0 && licenseDays <= 30) {
        this.globalMessageService.removeAlertByName(
          'LICENSE_NOTIFICATION_PROCESSING'
        );
        this.globalMessageService.removeAlertByName(
          'LICENSE_NOTIFICATION_NOLICENSE'
        );
        this.globalMessageService.removeAlertByName(
          'LICENSE_NOTIFICATION_NOLICENSE_CONTROLLER'
        );
        this.globalMessageService.pushAlertIfNotPresent({
          name: 'LICENSE_NOTIFICATION',
          sticky: true,
          type: 'info',
          message:
            expiringToday === 'true'
              ? 'Your license is going to expire today.'
              : expiringTomorrow === 'true'
              ? 'Your license is going to expire tomorrow.'
              : `Your license is about to expire in ${licenseDaysApprox} days.`
        });
      } else if (licenseDays <= 0) {
        this.globalMessageService.removeAlertByName(
          'LICENSE_NOTIFICATION_PROCESSING'
        );
        this.globalMessageService.removeAlertByName(
          'LICENSE_NOTIFICATION_NOLICENSE'
        );
        this.globalMessageService.removeAlertByName(
          'LICENSE_NOTIFICATION_NOLICENSE_CONTROLLER'
        );

        this.globalMessageService.pushAlertIfNotPresent({
          name: 'LICENSE_NOTIFICATION',
          sticky: true,
          type: 'danger',
          message: 'Your license has expired.'
        });
      }
    } else if (_licenseDays === LicenseDaysEnum.Processing) {
      this.globalMessageService.removeAlertByName(
        'LICENSE_NOTIFICATION_NOLICENSE'
      );
      this.globalMessageService.removeAlertByName(
        'LICENSE_NOTIFICATION_NOLICENSE_CONTROLLER'
      );
      this.globalMessageService.removeAlertByName('LICENSE_NOTIFICATION');
      if (controllerCount == 0) {
        this.globalMessageService.pushAlertIfNotPresent({
          name: 'LICENSE_NOTIFICATION_PROCESSING_Controller_Count',
          sticky: true,
          type: 'danger',
          message:
            'Your license process is waiting for controller deployment.please deploy the Controller for processing of license.'
        });
      } else {
        this.globalMessageService.pushAlertIfNotPresent({
          name: 'LICENSE_NOTIFICATION_PROCESSING',
          sticky: true,
          type: 'info',
          message:
            'Your license is being processed and you will have access to limited features of your Invisily installation.'
        });
      }
    } else if (_licenseDays === LicenseDaysEnum.Invalid) {
      this.showViolationLicenseNotification(violated);
    } else {
      this.globalMessageService.removeAlertByName(
        'LICENSE_NOTIFICATION_PROCESSING'
      );
      this.globalMessageService.pushAlertIfNotPresent({
        name: 'LICENSE_NOTIFICATION_NOLICENSE',
        sticky: true,
        type: 'info',
        message:
          'Your Invisily installation does not have a valid license and requires a license from you to enable all features.'
      });
      this.globalMessageService.pushAlertIfNotPresent({
        name: 'LICENSE_NOTIFICATION_NOLICENSE_CONTROLLER',
        sticky: true,
        type: 'info',
        message: 'You need to upload a license before deploying Controller.'
      });
    }
  }

  private showViolationLicenseNotification(violated) {
    this.globalMessageService.pushAlertIfNotPresent({
      name: 'LICENSE_NOTIFICATION_INVALID',
      sticky: true,
      type: 'danger',
      message:
        violated === LicenseViolationEnum.ServiceLimit.toString()
          ? 'License violation detected. Max Number of Services limit exceeded.'
          : violated === LicenseViolationEnum.ClientLimit.toString()
          ? 'License violation detected. Max Number of Clients limit exceeded.'
          : 'Invalid license. Something wrong has been detected with your license.'
    });
  }

  private showExpiryLicenseNotification() {
    this.globalMessageService.pushAlertIfNotPresent({
      name: 'EXPIRY_LICENSE_NOTIFICATION',
      sticky: false,
      type: 'danger',
      message: 'Your license has expired.'
    });
  }

  private showProcessingRestrictedLicenseNotification() {
    this.globalMessageService.pushAlertIfNotPresent({
      name: 'LICENSE_NOTIFICATION_PROCESSING_RESTRICTED',
      sticky: false,
      type: 'info',
      message:
        'You cannot access this feature until the license has finished processing.'
    });
  }

  // public request(method: string | any, url: string): Observable<any> {
  //   return super.request(method, url).pipe(
  //     tap(
  //       (res: Response) => {
  //         this.showInformativeLicenseNotification(
  //           res.headers.get('X-License-Days'),
  //           res.headers.get('X-License-Expiring-Today'),
  //           res.headers.get('X-License-Expiring-Tomorrow'),
  //           res.headers.get('X-License-Violation-Reason'),
  //           res.headers.get('X-Controller-Count')
  //         );
  //         this.hideNavItems(res.headers.get('X-License-Days'));
  //       },
  //       (err: HttpErrorResponse) => {
  //         if (err.status === 403) {
  //           if (
  //             err.error == LicenseErrorEnum.Invalid ||
  //             err.error == 'No License' ||
  //             err.error == 'ProcessingRestricted' ||
  //             err.error.indexOf('Violation-Reason::') > -1
  //           ) {
  //             if (err.error == LicenseErrorEnum.Invalid) {
  //               this.showExpiryLicenseNotification();
  //             } else if (err.error.indexOf('Violation-Reason::') > -1) {
  //               this.showViolationLicenseNotification(err.error.split('::')[1]);
  //             } else if (err.error !== 'No License') {
  //               this.showProcessingRestrictedLicenseNotification();
  //             }
  //             this.hideNavItems('-1');

  //             const router = this.injector.get(Router);
  //             router.navigate(['/license']);
  //           } else {
  //             this.globalMessageService.pushAlert({
  //               closed: false,
  //               type: 'danger',
  //               message: 'You are not authorized to perform this operation.'
  //             });
  //           }
  //         } else if (err.status === 401) {
  //           const authenticationService = this.injector.get(
  //             AuthenticationService
  //           );
  //           authenticationService.logout(true);
  //         } else {
  //           const errors = Array.isArray(err.error.message)
  //             ? err.error.message
  //             : [{ error: err.error.message }];
  //           const errorMessges = errors.map(e => e[Object.keys(e)[0]]);

  //           errorMessges.forEach(error => {
  //             if (error) {
  //               this.globalMessageService.pushAlert({
  //                 closed: false,
  //                 type: 'danger',
  //                 message: error
  //               });
  //             }
  //           });
  //         }
  //       }
  //     )
  //   );
  // }
}
