import { Component, OnInit, ViewChild } from '@angular/core';
import { DeviceInventoryService } from './device-inventory.service';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { GlobalMessageClass } from '../global-messages/global-messages';
import { Pagination } from '../models/pagination';
import { Subject, Observable } from 'rxjs';
import { NgModel } from '@angular/forms';
import { debounceTime, distinctUntilChanged, switchMap } from 'rxjs/operators';
import { ConfirmModalComponent } from '../shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-device-inventory',
  templateUrl: './device-inventory.component.html',
  styleUrls: ['./device-inventory.component.css']
})
export class DeviceInventoryComponent implements OnInit {
  uploadCsvFile = false;
  fileData = {};
  isFileValid = false;
  deviceInventories: any = {};
  allDeviceInventories: any = {};
  pagination = new Pagination();
  isUploading = false;
  $searchInventory = new Subject();
  deviceEditForm;
  deviceAddForm;
  saveButtonDisabled = false;
  addSaveButtonDsiabled = true;
  addInventory = false;
  searchTerm: string = null;

  loading: boolean = true;

  sort: string;
  sortDir: string = 'desc';

  @ViewChild('editUserName', { static: false }) editUserName: NgModel;
  @ViewChild('editDeviceTag', { static: false }) editDeviceTag: NgModel;
  @ViewChild('editDeviceNumber', { static: false }) editDeviceNumber: NgModel;
  @ViewChild('addUserName', { static: false }) addUserName: NgModel;
  @ViewChild('addDeviceNumber', { static: false }) addDeviceNumber: NgModel;
  @ViewChild('addDeviceTag', { static: false }) addDeviceTag: NgModel;

  pageSize = [10, 25, 50, 100, 200];

  public alert: GlobalMessageClass;
  isDarkMode : boolean;
  
  constructor(
    private inventoryService: DeviceInventoryService,
    private globalMessageService: GlobalMessageService,
    public dialog: MatDialog,
    private titleService: Title
  ) {}

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.titleService.setTitle('Device Management | Device Inventory');
    this.getDevicInventories();
    this.subscribeToSearch();
  }

  subscribeToSearch() {
    this.$searchInventory
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((search: { searchWord: string; typing: boolean }) => {
          if (search.typing) {
            this.pagination.currentPage = 1;
          }
          return this.inventoryService.searchInventory(
            search.searchWord,
            this.pagination.itemsPerPage,
            this.pagination.currentPage,
            this.sort,
            this.sortDir
          );
          // if (searchWord && /\S/.test(searchWord)) {
          //   return this.inventoryService.searchInventory(searchWord);
          // } else {
          //   this.pagination.currentPage = 1;
          //   this.getDevicInventories();
          //   return Observable.of(this.deviceInventories);
          // }
        })
      )
      .subscribe((deviceInventoies: any) => {
        this.loading = false;
        this.deviceInventories = deviceInventoies;
        this.pagination.totalItems = deviceInventoies.recordsTotal;
      });
  }

  getDevicInventories() {
    this.loading = true;
    this.inventoryService
      .getDeviceInventories(
        this.pagination.itemsPerPage,
        this.pagination.currentPage,
        this.sort,
        this.sortDir
      )
      .subscribe(
        (inventories: any) => {
          this.loading = false;
          this.deviceInventories = inventories;
          this.pagination.totalItems = inventories.recordsTotal;
        },
        error => {
          this.loading = false;
          // this.showAlert(
          //   'error',
          //   'Error in fetching device inventories',
          //   false
          // );
        }
      );
  }

  uploadHandler() {
    this.uploadCsvFile = !this.uploadCsvFile;
  }

  handleFileUpload(event) {
    if (event.target.files && event.target.files.length > 0) {
      let file = event.target.files[0];
      let name = file.name;
      let type = file.type;
      let localName = name.split('.');
      let extension = localName[localName.length - 1];
      if (extension === 'csv' || type === 'text/csv') {
        let fileRedaer = new FileReader();
        fileRedaer.readAsDataURL(file);
        fileRedaer.onload = () => {
          let data = fileRedaer.result;
          this.fileData = {
            data: data,
            name: name,
            type: type
          };
          this.isFileValid = true;
        };
      } else {
        this.isFileValid = false;
        alert('Please select a valid file type, these are allowed types `csv`');
      }
    }
  }

  pageChange(event) {
    this.pagination.currentPage = event;
    if (this.searchTerm) {
      this.$searchInventory.next({
        searchWord: this.searchTerm,
        typing: false
      });
    } else {
      this.getDevicInventories();
    }
  }

  uploadCsvFileHandler() {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: `Uploading this file will overwrite existing data.`,
      },
    });

    dialogRef.afterClosed().subscribe((result) => {
      if (result){
        this.isUploading = true;
        this.inventoryService.uploadCsvFile(this.fileData).subscribe(
        data => {
          this.getDevicInventories();
          this.isUploading = false;
          this.uploadCsvFile = false;
          this.isFileValid = false;

          this.alert = {
            closed: false,
            type: 'success',
            message: 'File has been uploaded successfully.'
          };
          this.globalMessageService.pushAlert(this.alert);
        },
        error => {
          this.isUploading = false;
          // this.showAlert('danger',JSON.parse(error._body).message, false);
        }
      );
      }
    });
  }

  downloadHandler() {
    this.inventoryService.downloadCsv();
  }

  deleteInventory(inventory) {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: `Do you really want to delete this inventory item?`,
      },
    });

    dialogRef.afterClosed().subscribe((result) => {
      if (result){
        this.inventoryService.deleteInventory(inventory).subscribe(
          data => {
            this.deviceInventories.data = this.deviceInventories.data.filter(
              singleInventory => {
                return singleInventory.id !== inventory.id;
              }
            );
            this.allDeviceInventories = this.deviceInventories;
  
            this.alert = {
              closed: false,
              type: 'success',
              message: 'Device has been deleted successfully.'
            };
            this.globalMessageService.pushAlert(this.alert);
  
            this.getDevicInventories();
          },
          error => {
            // this.showAlert('danger', 'Error in deleting inventory', false);
          }
        );
      }
    });
  }

  cancelEditInventory() {
    this.deviceEditForm = null;
    this.saveButtonDisabled = false;
  }

  addCancelHandler() {
    this.addSaveButtonDsiabled = true;
    this.addInventory = false;
    this.deviceAddForm = null;
  }

  editHandler(inventory) {
    this.deviceEditForm = { ...inventory };
  }

  addInventoryHandler() {
    this.addInventory = true;
    this.deviceAddForm = {
      userName: '',
      deviceSerialNumber: ''
    };
  }

  addSaveHandler() {
    const { userName, deviceSerialNumber, deviceTag } = this.deviceAddForm;
    let addedDevice = {
      userName: userName.trim(),
      deviceSerialNumber: deviceSerialNumber.trim(),
      deviceTag: deviceTag.trim()
    };
    this.inventoryService.addNewInventory(addedDevice).subscribe(
      response => {
        this.deviceInventories.data = [
          response,
          ...this.deviceInventories.data
        ];
        this.addSaveButtonDsiabled = true;
        this.addInventory = false;
        this.deviceAddForm = null;
        this.getDevicInventories();
      },
      error => {
        // this.showAlert('danger', error._body, false);
      }
    );
  }

  editInputChanged() {
    this.saveButtonDisabled =
      this.editDeviceNumber.invalid ||
      this.editUserName.invalid ||
      this.editDeviceTag.invalid;
  }

  addInputChanged() {
    this.addSaveButtonDsiabled =
      this.addDeviceNumber.invalid ||
      this.addUserName.invalid ||
      this.addDeviceTag.invalid;
  }

  updateInventory(inventoryParam) {
    this.inventoryService.updateInventory(this.deviceEditForm).subscribe(
      response => {
        const editedIndex = this.deviceInventories.data
          .map(inventory => {
            return inventory.id;
          })
          .indexOf(inventoryParam.id);
        if (editedIndex != -1) {
          this.deviceInventories.data[editedIndex] = { ...this.deviceEditForm };
        }
        if (this.deviceEditForm.id === inventoryParam.id) {
          this.saveButtonDisabled = false;
          this.deviceEditForm = null;
        }
      },
      error => {
        // this.showAlert('danger', error._body, false);
      }
    );
  }

  private showAlert(type: string, message: string, sticky: boolean) {
    const alert = {
      closed: false,
      type: type,
      message: message,
      sticky: sticky
    };
    this.globalMessageService.pushAlert(alert);
  }

  searchInventory(searchTerm: string) {
    this.loading = true;
    if (searchTerm) {
      this.$searchInventory.next({ searchWord: searchTerm, typing: true });
    } else {
      this.pagination.currentPage = 1;
      this.getDevicInventories();
    }
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getDevicInventories();
  }

  sortRecords(term: string) {
    this.sortDir = this.sortDir == 'asc' ? 'desc' : 'asc';
    this.sort = term;
    this.searchTerm == '' || this.searchTerm == null
      ? this.getDevicInventories()
      : this.$searchInventory.next({
          searchWord: this.searchTerm,
          typing: false
        });
  }
}
