import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { Pagination } from '../models/pagination';
import { DeviceService } from './device.service';
import { OrganizationService } from '../organization-list-view/organization.service';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { Observable, Subject, Subscription } from 'rxjs';
import { PreserveSearch } from '../models/preserve-search';
import { debounceTime, distinctUntilChanged, switchMap } from 'rxjs/operators';
import { DeviceOSEnum, DeviceTypeEnum } from '../shared/emuns/client.enum';
import { ConfirmModalComponent } from '../shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';
import { OtpAuthenticationService } from '../otp-verification/otp-authentication.service';
import { RolesAndPermissionService } from '../roles-and-permission/roles-and-permission.service';

@Component({
  selector: 'app-devices-list',
  templateUrl: './devices-list.component.html',
  styleUrls: ['./devices-list.component.css']
})
export class DevicesListComponent implements OnInit {
  showHelpText: boolean = false;
  pagination = new Pagination();
  devices: any = {};
  public actionsDropdown: number = 0;
  $searchDevices = new Subject();
  showPasswordConfimation = false;
  deviceTobeDeleted: any;
  isNameSort: boolean = false;
  searchWord;
  preserveSearch = new PreserveSearch();
  navigationToEditDevice: boolean = false;
  DeviceOS = DeviceOSEnum;
  DeviceType = DeviceTypeEnum;
  loading: Boolean = true;

  pageSize = [10, 25, 50, 100, 200];

  deviceTypes = [
    DeviceTypeEnum.Desktop,
    DeviceTypeEnum.IoTBridgeClient,
    DeviceTypeEnum.Laptop,
    DeviceTypeEnum.VirtualDesktop,
    DeviceTypeEnum.PhysicalServer,
    DeviceTypeEnum.Smartphone,
    DeviceTypeEnum.Tablet,
    DeviceTypeEnum.ThinBridgeClient,
    DeviceTypeEnum.ServerBridgeClient,
    DeviceTypeEnum.VirtualServer,
    DeviceTypeEnum.ARMPlatform
  ];
  organizations = [];
  departments = [];
  locations = [];
  osVersions = [];

  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;

  selectedDeviceType: string;
  selectedOrganization: string;
  selectedOs: string;
  selectedDepartment;
  selectedLocation: string;
  selectedOsVersion: string;
  referenceMachineCheck = false;

  sort: string;
  sortDir: string;

  selectAll = false;

  selectAllExcept: any = new Set();
  selectedDevices: any = new Set();

  uploadCsvFile = false;
  fileData = {};
  isFileValid = false;
  isUploading = false;
  showFailedUpload: boolean = false;
  failedDevices = [];

  devicesOs = [
    { value: DeviceOSEnum.Macintosh, display: 'Macintosh' },
    { value: DeviceOSEnum.Ubuntu18, display: 'Ubuntu Desktop' },
    { value: DeviceOSEnum.Windows, display: 'Windows' },
    {
      value: DeviceOSEnum.UbuntuServer20armv8,
      display: 'Ubuntu Server (ARMv8)'
    },
    { value: DeviceOSEnum.CentosServer, display: 'CentOS Server' },
    { value: DeviceOSEnum.UbuntuServer18, display: 'Ubuntu Server' },
    { value: DeviceOSEnum.WindowsServer, display: 'Windows Server' },
    { value: DeviceOSEnum.IOS, display: 'iOS' },
    { value: DeviceOSEnum.Android, display: 'Android' }
  ];

  isDarkMode: boolean;
  showOtpDialog: boolean = false;
  otptoken: string = '';
  option: string;
  private otpStatusSubscription: Subscription;

  constructor(
    private router: Router,
    private deviceService: DeviceService,
    private organizationService: OrganizationService,
    private globalMessageService: GlobalMessageService,
    public dialog: MatDialog,
    private titleService: Title,
    private OtpAuthenticationService: OtpAuthenticationService,
    private rolesPermissionService: RolesAndPermissionService
  ) {}

  sortByNameHandler() {
    this.isNameSort = !this.isNameSort;
    if (this.searchWord) {
      this.$searchDevices.next({ searchWord: this.searchWord, typing: false });
    } else {
      this.getDevices();
    }
  }

  getOSName(osName) {
    switch (osName) {
      case DeviceOSEnum.Ubuntu18:
        return 'Ubuntu Desktop';
      case DeviceOSEnum.UbuntuServer18:
        return 'Ubuntu Server';
      case DeviceOSEnum.UbuntuServer20armv8:
        return 'Ubuntu Server (ARMv8)';
      case DeviceOSEnum.Windows:
        return 'Windows';
      case DeviceOSEnum.WindowsServer:
        return 'Windows Server';
      case DeviceOSEnum.Macintosh:
        return 'Macintosh';
      case DeviceOSEnum.IOS:
        return 'iOS';
      case DeviceOSEnum.Android:
        return 'Android';
      case DeviceOSEnum.CentosServer:
        return 'CentOS Server';
      default:
        return osName;
    }
  }

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.titleService.setTitle('Device Management | Devices');
    this.getCRUDdata(40);
    const _saveSearch = this.preserveSearch.getSearch('deviceSearch');
    if (!_saveSearch) {
      this.getDevices();
    } else {
      this.searchWord = _saveSearch.searchKey;
      this.pagination = _saveSearch.pagination;
      this.selectedDeviceType = _saveSearch.selectedDeviceType;
      this.selectedOsVersion = _saveSearch.selectedOsVersion;
      this.selectedOrganization = _saveSearch.selectedOrganization;
      this.selectedDepartment = parseInt(_saveSearch.selectedDepartment);
      this.selectedLocation = _saveSearch.selectedLocation;
      this.selectedOs = _saveSearch.selectedOs;
      if (this.selectedOrganization) {
        this.fetchOrganizationDepartments();
      } else {
        this.selectedDepartment = null;
      }
      this.getDevices();
    }
    this.organizationService.getOrganizationNames().subscribe((orgs: any) => {
      this.organizations = orgs;
    });
    this.deviceService.getAllLocations().subscribe((locations: any) => {
      this.locations = locations.filter(location => {
        return location != null && location != '';
      });
    });
    this.subscribeToSearch();
  }

  ngOnDestroy() {
    if (!this.navigationToEditDevice) {
      this.preserveSearch.saveSearch('deviceSearch', '');
    }

    if (this.otpStatusSubscription) {
      this.otpStatusSubscription.unsubscribe();
    }
  }

  getCRUDdata(privId){
    this.rolesPermissionService.getCrudData(privId).subscribe((res: any) => {
      this.createPermission = res?.create_permission === 1 ? true : false
      this.readPermission = res?.read_permission === 1 ? true : false
      this.updatePermission = res?.update_permission === 1 ? true : false
      this.deletePermission = res?.delete_permission === 1 ? true : false
    });
  }

  selectAllAction() {
    this.selectAllExcept.clear();
    this.selectedDevices.clear();
    this.changeCheckStatusOfAllDevices(this.selectAll);
  }

  private changeCheckStatusOfAllDevices(check) {
    this.devices.data = this.devices.data.map(_device => {
      return {
        ..._device,
        isChecked:
          ((!this.selectAllExcept.has(_device.id) && this.selectAll) ||
            (this.selectedDevices.has(_device.id) && !this.selectAll)) &&
          ![
            DeviceTypeEnum.IoTBridgeClient,
            DeviceTypeEnum.ServerBridgeClient,
            DeviceTypeEnum.ThinBridgeClient
          ].includes(_device.device_type)
      };
    });
    if (this.selectAll) {
      for (let j = 0; j < this.devices.data.length; j++) {
        if (
          [
            DeviceTypeEnum.IoTBridgeClient,
            DeviceTypeEnum.ServerBridgeClient,
            DeviceTypeEnum.ThinBridgeClient
          ].includes(this.devices.data[j].device_type)
        ) {
          this.selectAllExcept.add(this.devices.data[j].id);
        }
      }
    }
  }

  subscribeToSearch() {
    this.$searchDevices
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((search: { searchWord: string; typing: boolean }) => {
          this.loading = true;
          this.clearSelection();
          if (search.typing) {
            this.pagination.currentPage = 1;
          }
          return this.deviceService.searchDevices(
            search.searchWord,
            this.pagination.itemsPerPage,
            this.pagination.currentPage,
            this.sort,
            this.sortDir,
            this.selectedDeviceType,
            this.selectedOrganization,
            this.selectedDepartment,
            this.selectedLocation,
            this.selectedOsVersion,
            this.selectedOs
          );
        })
      )
      .subscribe((devices: any) => {
        this.devices = devices;
        this.dropDownUpdates();

        this.pagination.totalItems = devices.recordsTotal;
        this.loading = false;
      });
  }

  createDevice() {
    this.router.navigate(['/create-device']);
  }

  async dropDownUpdates() {
    if (this.devices.data) {
      for (let i = 0; i < this.devices.data.length; i++) {
        let osVersion = this.devices.data[i].os_version;
        if (osVersion) {
          if (this.osVersions.indexOf(osVersion) == -1) {
            this.osVersions.push(osVersion);
          }
        }
      }
      this.osVersions = [...this.osVersions];
    }
  }

  getDevices() {
    this.loading = true;
    this.deviceService
      .getDevices(
        this.pagination,
        this.sort,
        this.sortDir,
        this.searchWord,
        this.selectedDeviceType ? this.selectedDeviceType : '',
        this.selectedOrganization ? this.selectedOrganization : '',
        this.selectedDepartment ? this.selectedDepartment : '',
        this.selectedLocation ? this.selectedLocation : '',
        this.selectedOsVersion ? this.selectedOsVersion : '',
        this.selectedOs ? this.selectedOs : '',
        this.referenceMachineCheck
      )
      .subscribe(
        (response: any) => {
          this.loading = false;
          response.data = response.data.map(_device => {
            return { ..._device, isChecked: false };
          });

          this.devices = response;
          this.dropDownUpdates();

          setTimeout(() => {
            this.changeCheckStatusOfAllDevices(this.selectAll);
          }, 10);

          this.pagination.totalItems = response.recordsTotal;
        },
        error => {
          this.loading = false;
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'danger',
            message: 'Error in getting devices',
            sticky: false
          });
        }
      );
    this.preserveCurrentSerarch();
  }

  clearfun() {
    this.searchWord = '';
    this.getDevices();
  }

  changeCheckBox(device) {
    const { isChecked, id } = device;
    if (this.selectAll) {
      if (isChecked) {
        this.selectAllExcept.delete(id);
      } else {
        this.selectAllExcept.add(id);
      }
    } else {
      if (!isChecked) {
        this.selectedDevices.delete(id);
      } else {
        this.selectedDevices.add(id);
      }
    }
  }

  editDevice(device) {
    let mfa = localStorage.getItem('adminMfaValue');
    if (mfa === 'true') {
      this.showOtpDialog = true;
      this.option = 'edit';
      this.otpStatusSubscription = this.OtpAuthenticationService.otpStatusCheckEdit$.subscribe(
        (res: any) => {
          if (res == true) {
            this.preserveCurrentSerarch();
            this.router.navigate(['/device', device.id]);
            this.OtpAuthenticationService.setOtpStatusValueEdit(false);
          }
        }
      );
    } else {
      this.preserveCurrentSerarch();
      this.router.navigate(['/device', device.id]);
    }
  }

  preserveCurrentSerarch() {
    const save = {
      pagination: this.pagination,
      searchKey: this.searchWord,
      selectedDeviceType: this.selectedDeviceType,
      selectedOsVersion: this.selectedOsVersion,
      selectedOrganization: this.selectedOrganization,
      selectedDepartment: this.selectedDepartment,
      selectedLocation: this.selectedLocation,
      selectedOs: this.selectedOs
    };
    this.preserveSearch.saveSearch('deviceSearch', save);
    this.navigationToEditDevice = true;
  }

  viewDevice(device) {
    this.preserveCurrentSerarch();
    this.router.navigate(['/view-device', device.id]);
  }

  deleteDevice(device) {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: `This will delete device. Proceed?`
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        let mfa = localStorage.getItem('adminMfaValue');
        if (mfa === 'true') {
          this.showOtpDialog = true;
          this.option = 'delete';
          this.otpStatusSubscription = this.OtpAuthenticationService.otpStatusCheckDel$.subscribe(
            (res: any) => {
              if (res == true) {
                this.deviceTobeDeleted = device;
                this.deviceService
                  .deleteDevice(this.deviceTobeDeleted.id)
                  .subscribe(
                    response => {
                      this.getDevices();
                      this.globalMessageService.pushAlert({
                        closed: false,
                        type: 'success',
                        message: 'Device has been deleted successfully.',
                        sticky: false
                      });
                    },
                    error => {
                      this.loading = false;
                    }
                  );
                this.showOtpDialog = false;
                this.OtpAuthenticationService.setOtpStatusValueDel(false);
              }
            }
          );
        } else {
          this.deviceTobeDeleted = device;
          this.deviceService.deleteDevice(this.deviceTobeDeleted.id).subscribe(
            response => {
              this.getDevices();
              this.globalMessageService.pushAlert({
                closed: false,
                type: 'success',
                message: 'Device has been deleted successfully.',
                sticky: false
              });
            },
            error => {
              this.loading = false;
            }
          );
        }
      }
    });
  }

  pageChange(page) {
    this.pagination.currentPage = page;
    if (this.searchWord) {
      this.$searchDevices.next({ searchWord: this.searchWord, typing: false });
    } else {
      this.getDevices();
    }
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getDevices();
  }

  deviceTypeChange() {
    this.clearSelection();
    this.pagination.currentPage = 1;
    if (!this.selectedDeviceType) this.selectedDeviceType = null;
    this.getDevices();
  }

  clearSelection() {
    this.selectAllExcept.clear();
    this.selectAll = false;
  }

  async fetchOrganizationDepartments() {
    this.pagination.currentPage = 1;
    this.departments = [];
    if (this.selectedOrganization) {
      const org = (await this.organizationService
        .getOrganizationByName(this.selectedOrganization)
        .toPromise()) as any;
      const departments = (await this.organizationService
        .getAllDepartments(org.id)
        .toPromise()) as any;
      for (let j = 0; j < departments.data.length; j++) {
        this.departments.push({
          deptId: departments.data[j].id,
          deptName: departments.data[j].name
        });
      }
      this.departments = [...this.departments];
    } else {
      this.selectedOrganization = null;
    }
  }

  async organizationChange() {
    this.clearSelection();
    this.departments = [];
    this.selectedDepartment = undefined;
    if (!this.selectedOrganization) {
      this.selectedOrganization = undefined;
    } else {
      this.fetchOrganizationDepartments();
    }
    this.getDevices();
  }

  departmentChange() {
    this.pagination.currentPage = 1;
    if (!this.selectedDepartment) this.selectedDepartment = null;
    this.clearSelection();
    this.getDevices();
  }

  locationChange() {
    this.pagination.currentPage = 1;
    if (!this.selectedLocation) this.selectedLocation = null;
    this.clearSelection();
    this.getDevices();
  }

  osVersionChange() {
    if (!this.selectedOsVersion) this.selectedOsVersion = null;
    this.clearSelection();
    this.getDevices();
  }

  referenceMachineFilter() {
    this.getDevices();
  }

  devicesValidation() {
    this.loading = true;
    this.deviceService.devicesValidation().subscribe(
      response => {
        this.loading = false;
        this.getDevices();
        this.globalMessageService.pushAlert({
          closed: false,
          type: 'success',
          message: 'Devices Validation has been performed successfully.',
          sticky: false
        });
      },
      error => {
        this.loading = false;
        this.globalMessageService.pushAlert({
          closed: false,
          type: 'danger',
          message: 'Could not perform Devices Validation task.',
          sticky: false
        });
      }
    );
  }

  sortRecord(term: string) {
    this.sortDir = this.sortDir == 'asc' ? 'desc' : 'asc';
    this.sort = term;
    this.getDevices();
  }

  uploadHandler() {
    this.uploadCsvFile = !this.uploadCsvFile;
  }

  handleFileUpload(event) {
    if (event.target.files && event.target.files.length > 0) {
      let file = event.target.files[0];
      let name = file.name;
      let type = file.type;
      let localName = name.split('.');
      let extension = localName[localName.length - 1];
      if (extension === 'csv' || type === 'text/csv') {
        let fileRedaer = new FileReader();
        fileRedaer.readAsDataURL(file);
        fileRedaer.onload = () => {
          let data = fileRedaer.result;
          this.fileData = {
            data: data,
            name: name,
            type: type
          };
          this.isFileValid = true;
        };
      } else {
        this.isFileValid = false;
        alert('Please select a valid file type, these are allowed types `csv`');
      }
    }
  }

  uploadCsvFileHandler() {
    this.isUploading = true;
    this.deviceService.uploadCsvFile(this.fileData).subscribe(
      (data: any) => {
        this.failedDevices = data;
        this.showFailedUpload = true;
        this.uploadCsvFile = false;
        this.isUploading = false;
        this.getDevices();
        // this.isFileValid = false;

        // this.globalMessageService.pushAlert({
        //   closed: false,
        //   type: 'success',
        //   message: 'File has been uploaded successfully.'
        // });
      },
      error => {
        this.isUploading = false;
        // this.showAlert('danger',JSON.parse(error._body).message, false);
      }
    );
  }

  downloadHandler() {
    this.deviceService.downloadCsv();
  }

  closeDialog() {
    this.showFailedUpload = false;
    this.failedDevices = [];
  }

  bulkActions(type, status) {
    let selectedIds = this.selectAll
      ? this.selectAllExcept
      : this.selectedDevices;
    this.deviceService
      .bulkActions(
        type,
        status,
        this.selectAll,
        selectedIds,
        this.selectedOrganization,
        this.selectedDepartment,
        this.selectedLocation,
        this.searchWord,
        this.selectedDeviceType,
        this.selectedOs
      )
      .subscribe({
        next: res => {
          this.getDevices();
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'success',
            message: 'Device(s) has been updated successfully.',
            sticky: false
          });
        },
        error: err => {
          this.loading = false;
        }
      });
  }

  osFilterChange() {
    this.clearSelection();
    this.getDevices();
  }

  cancelOtpVerificationPassword(event: any) {
    this.showOtpDialog = false;
    this.OtpAuthenticationService.setOtpStatusValueEdit(false);
    this.OtpAuthenticationService.setOtpStatusValueDel(false);
  }

  submitOtpVerificationPassword(event: any) {
    this.otptoken = event;
    // this.otptoken = event
  }

  getClosePopup(event: any) {
    this.showOtpDialog = false;
  }
}
