import { Component, OnDestroy, OnInit } from '@angular/core';
import { ForbiddenSoftwareServices } from './forbidden-software.service';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { Pagination } from '../models/pagination';
import { Subject } from 'rxjs';
import { debounceTime, distinctUntilChanged, switchMap } from 'rxjs/operators';
import { GlobalMessageClass } from '../global-messages/global-messages';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-forbidden-software-list',
  templateUrl: './forbidden-software-list.component.html',
  styleUrls: ['./forbidden-software-list.component.css']
})
export class ForbiddenSoftwareListComponent implements OnInit, OnDestroy {
  showHelpText: boolean = false;

  pagination = new Pagination();
  forbiddenSoftwares: any = {};
  $searchForbiddenSoftwares = new Subject();
  searchKeyword;
  sort = 'id';
  sortDir = 'desc';

  selectAll = false;
  selectAllExcept: any = new Set();
  selectedSoftwares: any = new Set();

  pageSize = [10, 25, 50, 100, 200];

  loading: boolean = true;

  public alert: GlobalMessageClass;

  constructor(
    private forbiddenSoftwareService: ForbiddenSoftwareServices,
    private globalMessageService: GlobalMessageService,
    private titleService: Title
  ) {}

  ngOnDestroy(): void {
    this.$searchForbiddenSoftwares.unsubscribe();
  }

  ngOnInit() {
    this.refreshData();
    this.subscibeToSearch();
    this.titleService.setTitle('Device Health Check | Unauthorized Software Installations');
  }

  public refreshData(): void {
    this.getForbiddenSoftwares();
  }

  getForbiddenSoftwares() {
    this.loading = true;
    this.forbiddenSoftwareService
      .getForbiddenSoftwares(
        this.pagination.itemsPerPage,
        this.pagination.currentPage,
        this.searchKeyword,
        this.sort,
        this.sortDir
      )
      .subscribe({
        next: res => {
          this.loading = false;

          res.data = res.data.map(_software => {
            return { ..._software, isChecked: false };
          });

          this.forbiddenSoftwares = res;

          setTimeout(() => {
            this.changeCheckStatusOfAllSoftwares();
          }, 10);
          this.pagination.totalItems = res.recordsTotal;
        },
        error: err => {
          this.loading = false;
        }
      });
  }

  clearfun() {
    this.searchKeyword = '';
    this.getForbiddenSoftwares();
  }

  sortData() {
    this.sortDir = this.sortDir === 'desc' ? 'asc' : 'desc';
    this.refreshData();
  }

  subscibeToSearch() {
    this.$searchForbiddenSoftwares
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap(data => {
          this.loading = true;
          this.pagination.currentPage = 1;
          return this.forbiddenSoftwareService.searchForbiddenSoftwares(
            this.searchKeyword,
            this.pagination.itemsPerPage,
            this.pagination.currentPage,
            this.sort,
            this.sortDir
          );
        })
      )
      .subscribe({
        next: forbiddenSoftwares => {
          this.loading = false;
          const {recordsTotal } = forbiddenSoftwares as any;
          this.forbiddenSoftwares = forbiddenSoftwares;
          this.pagination.totalItems = recordsTotal;
        },
        error: err => {
          this.loading = false;
        }
      });
  }

  selectAllAction() {
    this.selectAllExcept.clear();
    this.selectedSoftwares.clear();
    this.changeCheckStatusOfAllSoftwares();
  }

  private changeCheckStatusOfAllSoftwares() {
    this.forbiddenSoftwares.data = this.forbiddenSoftwares.data.map(
      _software => {
        return {
          ..._software,
          isChecked:
            (!this.selectAllExcept.has(_software.id) && this.selectAll) ||
            (this.selectedSoftwares.has(_software.id) && !this.selectAll)
        };
      }
    );
  }

  changeCheckBox(software) {
    const { isChecked, id } = software;
    if (this.selectAll) {
      if (isChecked) {
        this.selectAllExcept.delete(id);
      } else {
        this.selectAllExcept.add(id);
      }
    } else {
      if (!isChecked) {
        this.selectedSoftwares.delete(id);
      } else {
        this.selectedSoftwares.add(id);
      }
    }
  }

  bulkActions() {
    let selectedIds = this.selectAll
      ? this.selectAllExcept
      : this.selectedSoftwares;
    this.forbiddenSoftwareService
      .bulkActions(this.selectAll, selectedIds, this.searchKeyword)
      .subscribe({
        next: res => {
          this.getForbiddenSoftwares();
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'success',
            message: 'Software(s) has been updated successfully.',
            sticky: false
          });
        },
        error: err => {
          this.loading = false;
        }
      });
  }

  pageChange(event) {
    this.pagination.currentPage = event;
    this.getForbiddenSoftwares();
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getForbiddenSoftwares();
  }

  sortRecords(term: string) {
    this.sortDir = this.sortDir == 'asc' ? 'desc' : 'asc';
    this.sort = term;
    this.getForbiddenSoftwares();
  }
}
