import { Component, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { Subject } from 'rxjs';
import { GatewayGroup } from '../../models/gatewayGroup';
import { Pagination } from '../../models/pagination';
import { GatewayGroupsService } from '../gateway-groups.service';
import { debounceTime, distinctUntilChanged, switchMap } from 'rxjs/operators';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { ConfirmModalComponent } from '../../../app/shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';
import { AccessPolicyService } from '../../create-access-policy/access-policy.service';
import { RolesAndPermissionService } from '../../roles-and-permission/roles-and-permission.service';

@Component({
  selector: 'app-gateway-groups',
  templateUrl: './gateway-groups.component.html',
  styleUrls: ['./gateway-groups.component.css']
})
export class GatewayGroupsComponent implements OnInit {
  showHelpText: boolean = false;
  gatewayGroups: GatewayGroup[] = [];
  pagination: Pagination = new Pagination();
  searchGatewayGroup = new Subject();
  searchKeyword: any = '';
  loading: boolean = false;
  pageSize = [10, 25, 50, 100, 200];
  isDarkMode : any;
  sorting = [
    {
      name: 'Name',
      value: 'name'
    },
    {
      name: 'Created at',
      value: 'createdAt'
    },
    {
      name: 'No. of Gateways',
      value: 'totalGateways'
    }
  ];

  selectedSorting = 'name';
  ascending: boolean = true;

  listView: Boolean = true;
  selectAll: boolean = false;

  selectAllExcept: any = new Set();
  selectedGroups: any = new Set();

  selectedGroup = {};
  showConfirmation = false;

  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;

  constructor(
    private router: Router,
    private gateWayGroupServices: GatewayGroupsService,
    private globalMessageService: GlobalMessageService,
    public dialog: MatDialog,
    private titleService: Title,
    private accessService: AccessPolicyService,
    private rolesPermissionService : RolesAndPermissionService
  ) {
    const gatewayGroupView = localStorage.getItem('gatewayGroupView');

    if (gatewayGroupView) {
      this.listView = gatewayGroupView == 'list' ? true : false;
    }
  }

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.getCRUDdata(26);
    this.titleService.setTitle('Infrastructure | Gateway | Gateways Groups');
    this.getGatewayGroups();
    this.subscibeToSearch();
  }

  getCRUDdata(privId){
    this.rolesPermissionService.getCrudData(privId).subscribe((res: any) => {
      this.createPermission = res?.create_permission === 1 ? true : false
      this.readPermission = res?.read_permission === 1 ? true : false
      this.updatePermission = res?.update_permission === 1 ? true : false
      this.deletePermission = res?.delete_permission === 1 ? true : false
    });
  }
  getGatewayGroups() {
    this.loading = true;
    this.gateWayGroupServices
      .getGatewaysGroups(
        this.pagination.currentPage,
        this.pagination.itemsPerPage,
        this.selectedSorting,
        this.ascending,
        this.searchKeyword
      )
      .subscribe(
        result => {
          this.gatewayGroups = result.data;
          this.pagination.totalItems = result.recordsTotal;
          this.loading = false;
          this.changeCheckStatusOfAllGroups();
        },
        err => {
          this.loading = false;
        }
      );
  }

  clearfun() {
    this.searchKeyword = '';
    this.getGatewayGroups();
  }

  createNewGatewayGroup() {
    this.router.navigate(['/create-gateway-group']);
  }

  pageChange(event) {
    this.pagination.currentPage = event;
    this.getGatewayGroups();
  }

  editHandler(group) {
    this.router.navigate([`/create-gateway-group/${group.id}`]);
  }

  deleteGatewayGroup() {
    const group: any = this.selectedGroup;
    this.accessService.verifyPolicies('', '', [], group.id).subscribe(res => {
      if(res){
        this.globalMessageService.pushAlert({
          closed: false,
          type: 'danger',
          message: 'Remove gateway group from Entitlement(s) first',
          sticky: false
        });
      }else{
        this.gateWayGroupServices.deleteGatewayGroup(group.id).subscribe(
          response => {
            this.pagination.totalItems = this.pagination.totalItems - 1;
            this.gatewayGroups = this.gatewayGroups.filter(
              gatewayGroup => gatewayGroup.id !== group.id
            );
            this.globalMessageService.pushAlert({
              message: 'Gateway Group deleted',
              type: 'success'
            });
            this.showConfirmation = false;
            this.selectedGroup = {};
            this.selectedGroups.clear();
          },
          err => {
            this.globalMessageService.pushAlert({
              message: JSON.parse(err._body).message,
              type: 'danger'
            });
            this.showConfirmation = false;
            this.selectedGroup = {};
            this.selectedGroups.clear();
          }
        );
      }
    })
    
  }

  confirmDelete(group) {
    this.selectedGroup = group;
    // this.showConfirmation = true;
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: `Are you sure want to delete "${group.name}"`,
      },
    });

    dialogRef.afterClosed().subscribe((result) => {
      if (result){
        this.deleteGatewayGroup()
      }else {
        this.cancelConfirmation();
      }
    });
  }

  cancelConfirmation() {
    this.showConfirmation = false;
    this.selectedGroup = {};
  }

  subscibeToSearch() {
    this.searchGatewayGroup
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap(data => {
          this.loading = true;
          this.searchKeyword = data;
          this.selectAllExcept.clear();
          this.selectAll = false;
          this.pagination.currentPage = 1;
          return this.gateWayGroupServices.getGatewaysGroups(
            this.pagination.currentPage,
            this.pagination.itemsPerPage,
            this.selectedSorting,
            this.ascending,
            this.searchKeyword
          );
        })
      )
      .subscribe({
        next: (result: any) => {
          this.loading = false;
          this.gatewayGroups = result.data;
          this.pagination.totalItems = result.recordsTotal;
          this.changeCheckStatusOfAllGroups();
        },
        error: err => {
          this.loading = false;
        }
      });
  }

  gatewayCountClick(group) {
    // if(group.all_gateways)
    // {
    //   this.router.navigate([`/gateways`]);
    // }
    // else if(group.totalGateways !== 0)
    // {
    //   this.router.navigate([`/gateways`], { queryParams: { gatewayGroup: group.id }});
    // }
  }

  handleSortingChange() {
    this.getGatewayGroups();
  }

  handleDirectionChange() {
    this.ascending = !this.ascending;
    this.getGatewayGroups();
  }

  pageSizeChange() {
    this.getGatewayGroups();
  }

  changeView(view) {
    localStorage.setItem('gatewayGroupView', view);
    this.listView = view == 'list' ? true : false;
  }

  goToDetailPage(gateway) {
    this.router.navigate([`/gatewayGroups/${gateway.id}`]);
  }

  selectAllAction() {
    if (!this.selectAll) {
      this.selectAllExcept.clear();
      this.selectedGroups.clear();
    } else {
      this.selectedGroups.clear();
    }
    this.changeCheckStatusOfAllGroups();
  }

  private changeCheckStatusOfAllGroups() {
    this.gatewayGroups = this.gatewayGroups.map(group => {
      return {
        ...group,
        isChecked:
          (!this.selectAllExcept.has(group.id) && this.selectAll) ||
          (this.selectedGroups.has(group.id) && !this.selectAll)
      };
    });
  }

  changeCheckBox(group) {
    const { isChecked, id } = group;
    if (this.selectAll) {
      if (isChecked) {
        this.selectAllExcept.delete(id);
      } else {
        this.selectAllExcept.add(id);
      }
    } else {
      if (!isChecked) {
        this.selectedGroups.delete(id);
      } else {
        this.selectedGroups.add(id);
      }
    }
  }

  handleDeleteGroups() {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      data: {
        title: 'Confirmation',
        message: `Are you sure want to delete group(s)`,
      },
    });

    dialogRef.afterClosed().subscribe((result) => {
      if (result){
        this.gateWayGroupServices
        .bulkDeleteGroup(
          this.selectedGroups,
          this.selectAll,
          this.selectAllExcept,
          this.searchKeyword
        )
        .subscribe(response => {
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'success',
            message: 'Gateway groups has been deleted successfully'
          });
          this.selectAll = false;
          this.selectAllExcept = new Set();
          this.selectedGroups = new Set();
          this.getGatewayGroups();
        });
      }
    });
  }
}
