import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { GroupService } from '../group.service';
import { Group } from '../../models/group';

import { GlobalMessageClass } from '../../global-messages/global-messages';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { AuthenticationService } from '../../login/authentication.service';
import { Popup } from '../../Popup';
import { Client } from '../../models/client';
import { Subject } from 'rxjs';
import { PreserveSearch } from '../../models/preserve-search';

import { Pagination } from '../../models/pagination';
import { debounceTime, distinctUntilChanged, switchMap } from 'rxjs/operators';
import { ErrorCodeEnum } from '../../shared/emuns/error.enum';
import { LicenseErrorEnum } from '../../shared/emuns/license.enum';
import { ConfirmModalComponent } from '../../../app/shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';
import { AccessPolicyService } from '../../create-access-policy/access-policy.service';
import { RolesAndPermissionService } from '../../roles-and-permission/roles-and-permission.service';

@Component({
  selector: 'app-group-list',
  templateUrl: './group-list.component.html',
  styleUrls: ['./group-list.component.css']
})
export class GroupListComponent implements OnInit {
  showHelpText: boolean = false;
  groupList: any = {};
  clientList: Client[];

  $searchTerm = new Subject();
  searchTerm$ = new Subject();
  searchTerm;
  searching: boolean = false;
  groupId: number;
  nextPage: number;
  shouldReload: boolean = true;
  pageSize = [10, 25, 50, 100];
  selectAll: boolean = false;

  sort = 'name';
  sortDir = 'asc';

  preserveSearch = new PreserveSearch();
  navigationToEditGroup: boolean = false;
  searchText: string = '';

  loading: Boolean = true;
  pagination: Pagination = new Pagination();
  $searchAlerts = new Subject();

  selectAllExcept: any = new Set();
  selectedGroups: any = new Set();

  listView = true;
  isDarkMode : boolean; 

  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;

  
  constructor(
    private router: Router,
    private globalMessageService: GlobalMessageService,
    private groupService: GroupService,
    private authenticationService: AuthenticationService,
    public dialog: MatDialog,
    private titleService: Title,
    private accessService: AccessPolicyService,
    private rolesPermissionService: RolesAndPermissionService
  ) {}

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.titleService.setTitle('Client Management | Groups');
    this.getCRUDdata(46);
    const groupView = localStorage.getItem('groupView');
    this.listView = groupView ? (groupView === 'grid' ? false : true) : true;
    const saveWord = this.preserveSearch.getSearch('GroupSearch');
    this.searchTerm$
      .pipe(
        debounceTime(1000),
        switchMap((term: any) => {
          if (term && term.length) this.shouldReload = true;
          this.searchTerm = term;
          return this.getGroupClients(this.groupId, term);
        })
      )
      .subscribe({
        next: (res: any) => {
          this.loading = false;
          this.clientList = res.clients;
          this.nextPage = res.nextPage;
          this.searching = false;
        },
        error: err => {
          this.loading = false;
          this.clientList = [];
          this.searching = false;
        }
      });

    this.getGroups();
    this.subscibeToSearch();

    if (!saveWord) {
      this.groupService.getGroups(this.pagination).subscribe(
        groups => {
          this.loading = false;
          this.groupList = groups;
        },
        err => {
          this.loading = false;
          if (err.status === ErrorCodeEnum.Forbidden) {
            if (err._body == LicenseErrorEnum.Invalid) {
              this.authenticationService.showLicenseOnly(true);
              this.router.navigate(['/license']);
            } else {
              this.globalMessageService.pushAlert({
                closed: false,
                type: 'error',
                message: 'You are not authorized to access this page.'
              });
            }
          }
        }
      );
    } else {
      this.searchText = saveWord;
      this.groupService.getGroups(this.pagination, saveWord).subscribe(
        groups => {
          this.loading = false;
          this.groupList = groups;
        },
        err => {
          if (err.status === ErrorCodeEnum.Forbidden) {
            if (err._body == LicenseErrorEnum.Invalid) {
              this.authenticationService.showLicenseOnly(true);
              this.router.navigate(['/license']);
            } else {
              this.globalMessageService.pushAlert({
                closed: false,
                type: 'error',
                message: 'You are not authorized to access this page.'
              });
            }
          }
        }
      );
    }
  }

  handleDirectionChange() {
    this.sortDir = this.sortDir == 'desc' ? 'asc' : 'desc';
    this.pagination.currentPage = 1;
    this.getGroups();
  }

  getCRUDdata(privId){
    this.rolesPermissionService.getCrudData(privId).subscribe((res: any) => {
      this.createPermission = res?.create_permission === 1 ? true : false
      this.readPermission = res?.read_permission === 1 ? true : false
      this.updatePermission = res?.update_permission === 1 ? true : false
      this.deletePermission = res?.delete_permission === 1 ? true : false
    });
  }

  getGroups() {
    this.loading = true;
    this.groupService
      .getGroups(this.pagination, this.searchTerm, this.sort, this.sortDir)
      .subscribe(
        groups => {
          this.loading = false;
          this.groupList = groups;
          this.pagination.totalItems = groups.recordsTotal;
          this.changeCheckStatusOfAllGroups();
        },
        err => {
          this.loading = false;
          if (err.status === ErrorCodeEnum.Forbidden) {
            if (err._body == LicenseErrorEnum.Invalid) {
              this.authenticationService.showLicenseOnly(true);
              this.router.navigate(['/license']);
            } else {
              this.globalMessageService.pushAlert({
                closed: false,
                type: 'danger',
                message: 'You are not authorized to access this page.'
              });
            }
          }
        }
      );
  }

  clearfun() {
    this.searchText = '';
    this.getGroups();
  }

  createGroup(): void {
    this.router.navigate(['/create-group']);
  }

  getGroupClients(groupId, term) {
    return this.groupService.getGroupClients(groupId, btoa(term));
  }

  switchGroup(groupId) {
    if (groupId !== this.groupId || this.shouldReload) {
      this.groupId = groupId;
      this.clientList = [];
      this.shouldReload = false;
      this.$searchTerm.next('');
      this.searchTerm$.next('');
    }
  }

  deleteGroup(group) {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: `Do you want to delete group '${group.name}'?`,
      },
    });

    dialogRef.afterClosed().subscribe((result) => {
      if (result){
        this.accessService.verifyPolicies('', '', [], '', '', group.id).subscribe(res => {
          if(res){
            this.globalMessageService.pushAlert({
              closed: false,
              type: 'danger',
              message: 'Remove client group from Entitlement(s) first',
              sticky: false
            });
          }else{
            this.groupService.deleteGroup(group.id).subscribe(
              response => {
                this.groupList.data = this.groupList.data.filter(
                  _group => _group.id !== group.id
                );
                this.selectAll = false
                this.selectAllExcept = new Set()
                this.selectedGroups = new Set()
                this.getGroups();
                this.globalMessageService.pushAlert({
                  message: response,
                  type: 'success'
                });
              },
              err => {
                this.globalMessageService.pushAlert({
                  message: JSON.parse(err._body),
                  type: 'danger'
                });
              }
            );
          }
        })
      }
    });
  }

  addToGroup(group, client) {
    group.GroupDetail.client_count++;
    client.is_member = true;

    this.groupService.addToGroup(group.id, client.id).subscribe(
      () => {
        this.globalMessageService.pushAlert({
          message: `${client.Name} added to ${group.name}`,
          type: 'success'
        });
      },
      () => {
        this.globalMessageService.pushAlert({
          message: `Failed to add ${client.Name} to ${group.name}`,
          type: 'danger'
        });
      }
    );
  }

  removeFromGroup(group, client) {
    group.GroupDetail.client_count--;
    client.is_member = false;

    this.groupService.removeFromGroup(group.id, client.id).subscribe(
      () => {
        this.globalMessageService.pushAlert({
          message: `${client.Name} removed from ${group.name}`,
          type: 'success'
        });
      },
      () => {
        this.globalMessageService.pushAlert({
          message: `Failed to remove ${client.Name} from ${group.name}`,
          type: 'danger'
        });
      }
    );
  }

  editGroup(group: Group): void {
    this.preserveSearch.saveSearch('GroupSearch', this.searchText);
    this.navigationToEditGroup = true;
    this.router.navigate(['/create-group', group.id]);
  }

  groupDetail(group: Group): void {
    this.router.navigate(['/group', group.id]);
  }

  pageChange(event) {
    this.pagination.currentPage = event;
    this.getGroups();
  }

  subscibeToSearch() {
    this.$searchTerm
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap(data => {
          this.loading = true;
          this.searchTerm = data;
          this.pagination.currentPage = 1;
          this.loading = true;
          this.selectAllExcept.clear();
          this.selectAll = false;
          return this.groupService.getGroups(
            this.pagination,
            this.searchTerm,
            this.sort,
            this.sortDir
          );
        })
      )
      .subscribe({
        next: (groups: any) => {
          this.loading = false;
          this.groupList = groups;
          this.pagination.totalItems = groups.recordsTotal;
        },
        error: err => {
          this.loading = false;
        }
      });
  }

  ngOnDestroy() {
    if (!this.navigationToEditGroup) {
      this.preserveSearch.saveSearch('GroupSearch', '');
    }
  }

  changeView(view) {
    localStorage.setItem('groupView', view);
    this.listView = view == 'list' ? true : false;
  }

  pageSizeChange(total) {
    this.pagination.itemsPerPage = total;
    this.pagination.currentPage = 1;
    this.getGroups();
  }

  selectAllAction() {
    if (!this.selectAll) {
      this.selectAllExcept.clear();
      this.selectedGroups.clear();
    } else {
      this.selectedGroups.clear();
    }
    this.changeCheckStatusOfAllGroups();
  }

  changeCheckBox(group) {
    const { isChecked, id } = group;
    if (this.selectAll) {
      if (isChecked) {
        this.selectAllExcept.delete(id);
      } else {
        this.selectAllExcept.add(id);
      }
    } else {
      if (!isChecked) {
        this.selectedGroups.delete(id);
      } else {
        this.selectedGroups.add(id);
      }
    }
  }

  private changeCheckStatusOfAllGroups() {
    this.groupList.data =
      this.groupList && this.groupList.data && this.groupList.data.length
        ? this.groupList.data.map(group => {
            return {
              ...group,
              isChecked:
                (!this.selectAllExcept.has(group.id) && this.selectAll) ||
                (this.selectedGroups.has(group.id) && !this.selectAll)
            };
          })
        : [];
  }

  handleDeleteGroups() {
    const count = this.selectAll
      ? this.pagination.totalItems - this.selectAllExcept.size
      : this.selectedGroups.size;
      const dialogRef = this.dialog.open(ConfirmModalComponent, {
        panelClass: this.isDarkMode ? 'dark-theme' : '',
        data: {
          title: 'Confirmation',
          message: `Are you sure want to delete ${count} group(s)`,
        },
      });
  
      dialogRef.afterClosed().subscribe((result) => {
        if (result){
          this.groupService
          .bulkDeleteGroup(
            this.selectedGroups,
            this.selectAll,
            this.selectAllExcept,
            this.searchTerm
          )
          .subscribe(response => {
            this.globalMessageService.pushAlert({
              closed: false,
              type: 'success',
              message: 'Groups has been deleted successfully'
            });
            this.selectAll = false;
            this.selectAllExcept = new Set();
            this.selectedGroups = new Set();
            this.getGroups();
          });
        }
      });
  }
}
