import { Injectable } from '@angular/core';
import { HttpClient as AuthHttp, HttpHeaders } from '@angular/common/http';

import { ConfigService } from '../services/config.service';
import { Group } from '../models/group';
import { TrustedHost } from '../models/trusted-host';
import { Pagination } from '../models/pagination';
import { map } from 'rxjs/operators';
import { ClientTypeEnum } from '../shared/emuns/client.enum';

@Injectable()
export class GroupService {
  private apiBaseUrl: string;
  private groupsApiUrl = '/groups';
  httpOptions = {
    headers: new HttpHeaders({
      'Content-Type': 'application/json'
    })
  };

  constructor(
    private authHttp: AuthHttp,
    private configService: ConfigService
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.groupsApiUrl = this.apiBaseUrl + this.groupsApiUrl;
  }

  getGroups(
    pagination: Pagination,
    searchTerm = '',
    sort = 'name',
    dir = 'asc'
  ): any {
    const query = `?limit=${
      pagination.itemsPerPage
    }&offset=${pagination.currentPage -
      1}&searchTerm=${searchTerm}&sort=${sort}&dir=${dir}`;
    return this.authHttp
      .get(this.groupsApiUrl + query)
      .pipe(map(response => response));
  }

  getAvailableGroups(): any {
    return this.authHttp
      .get(this.groupsApiUrl + '/available')
      .pipe(map(response => response));
  }

  getGroupClientsWithPaginatin(
    groupId,
    offset,
    limit,
    searchTerm = '',
    sort = 'name',
    dir = 'asc',
    type = '',
    os = ''
  ): any {
    const query = `?limit=${limit}&offset=${offset}&searchTerm=${searchTerm}&sort=${sort}&dir=${dir}&type=${type}&os=${os}`;
    return this.authHttp
      .get(this.groupsApiUrl + '/clients/' + groupId + query)
      .pipe(map(response => response));
  }

  getGroupClients(groupId, _term): any {
    const term = _term || '__MEMBERS__';

    return this.authHttp
      .get(`${this.groupsApiUrl}/${groupId}/${term}`)
      .pipe(map(response => response));
  }

  deleteGroup(groupId): any {
    return this.authHttp
      .delete(`${this.groupsApiUrl}/${groupId}`)
      .pipe(map(response => response));
  }

  bulkDeleteGroup(
    groupIds = [],
    allSelected = false,
    excludeGroups = [],
    search = ''
  ): any {
    const query = `?search=${search}&allSelected=${allSelected}`;
    this.httpOptions['body'] = {
      groupIds: Array.from(groupIds),
      excludeGroups: Array.from(excludeGroups)
    };
    return this.authHttp
      .delete(`${this.groupsApiUrl}/bulk${query}`, this.httpOptions)
      .pipe(map(response => response));
  }

  addToGroup(groupId, clientId): any {
    return this.authHttp.post(
      `${this.groupsApiUrl}/membership/${groupId}/${clientId}`,
      {}
    );
  }

  removeFromGroup(groupId, clientId): any {
    return this.authHttp.delete(
      `${this.groupsApiUrl}/membership/${groupId}/${clientId}`,
      {}
    );
  }

  addGroup(
    group: Group,
    hosts: Array<TrustedHost>,
    clientIds = [],
    addAllClients = false,
    type = ClientTypeEnum.Server
  ): any {
    let headers = new HttpHeaders({ 'Content-Type': 'application/json' });
    let options = { headers: headers };

    return this.authHttp
      .post(
        this.groupsApiUrl,
        { ...group, hosts, clientIds, addAllClients, clientType: type },
        options
      )
      .pipe(map(response => response));
  }

  getCurrentGroupClients(groupId): any {
    return this.authHttp
      .get(this.groupsApiUrl + '/clients/all/' + groupId)
      .pipe(map(response => response));
  }

  getGroupDetails(groupId): any {
    return this.authHttp
      .get(`${this.groupsApiUrl}/${groupId}`)
      .pipe(map(response => response));
  }

  editGroup(
    groupId,
    group: Group,
    servicesChanged: boolean,
    clientIds = [],
    appendClients = false,
    addAllClients = false,
    type = ClientTypeEnum.Server
  ): any {
    return this.authHttp
      .put(
        `${this.groupsApiUrl}/${groupId}?services_changed=${
          servicesChanged ? '1' : '0'
        }`,
        { ...group, clientIds, appendClients, addAllClients, clientType: type }
      )
      .pipe(map(response => response));
  }
}
