import { Component, OnInit, Output, EventEmitter, Input } from '@angular/core';
import { AmChart, AmChartsService } from '@amcharts/amcharts3-angular';
import { GatewayService } from '../gateway/gateway.service';
import { ControllerService } from '../controller/controller.service';
import { Subscription, timer } from 'rxjs';
import { Observable } from 'rxjs';
import { first } from 'rxjs/operators';
import { GraphComponentEnum } from '../shared/emuns/health-status.enum';

@Component({
  selector: 'app-health-status',
  templateUrl: './health-status.component.html',
  styleUrls: ['./health-status.component.css']
})
export class HealthStatusComponent implements OnInit {
  @Input('controllerId') controllerId: number;
  @Output('cancelHealth') cancelHealth = new EventEmitter();

  private memoryGraph: AmChart;
  private storageGraph: AmChart;
  private cpuGraph: AmChart;

  private timerSubscription: Subscription;

  loading: boolean = true;
  selectedInterval: number = 1;
  lastUpdated = null;
  version = null;
  os = null;
  lastErrorLog = null;
  lastSnifferErrorLog = null;
  cpu = null;

  dataFormats = [
    {
      period: 'fff',
      format: 'JJ:NN:SS'
    },
    {
      period: 'ss',
      format: 'JJ:NN:SS'
    },
    {
      period: 'mm',
      format: 'JJ:NN'
    },
    {
      period: 'hh',
      format: 'JJ:NN'
    },
    {
      period: 'DD',
      format: 'MMM DD'
    },
    {
      period: 'WW',
      format: 'MMM DD'
    },
    {
      period: 'MM',
      format: 'MMM DD'
    },
    {
      period: 'YYYY',
      format: 'YYYY'
    }
  ];

  showCpuGraph: boolean = false;
  showStorageGraph: boolean = false;
  showMemoryGraph: boolean = false;

  selectedGraph: number = 1;

  GraphComponent = GraphComponentEnum;
  isDarkMode : any;
  constructor(
    private AmCharts: AmChartsService,
    private gatewayService: GatewayService,
    private controllerService: ControllerService
  ) {}

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'))

    this.setAmChartFont();
    this.memoryGraph = this.AmCharts.makeChart('memory-graph', {
      type: 'serial',
      theme: 'light',
      startDuration: 1,
      sequencedAnimation: false,
      zoomOutText: '',
      hideCredits: true,
      marginRight: 20,
      marginLeft: 50,
      autoMarginOffset: 20,
      mouseWheelZoomEnabled: false,
      dataDateFormat: 'YYYY-MM-DD HH',
      valueAxes: [
        {
          id: 'v1',
          axisAlpha: 0,
          position: 'left',
          title: 'MB Utilized',
          color: this.isDarkMode ? '#eee' : '#353d4a',
          titleColor: this.isDarkMode ? '#eee' : '#353d4a',
        }
      ],
      balloon: {
        borderThickness: 1,
        shadowAlpha: 0
      },
      graphs: [
        {
          id: 'g1',
          balloon: {
            adjustBorderColor: false,
            color: '#ffffff'
            // horizontalPadding: 20,
            // verticalPadding: 20
          },
          lineColor: '#9460FF',
          bullet: 'none',
          bulletBorderAlpha: 1,

          bulletSize: 6,
          hideBulletsCount: 100,
          lineThickness: 2,
          title: 'Total memory',
          color: this.isDarkMode ? '#eee' : '#353d4a',
          titleColor: this.isDarkMode ? '#eee' : '#353d4a',
          useLineColorForBulletBorder: true,
          valueField: 'memTotal',
          balloonText: "<span style='font-size:16px;'>[[memTotal]]</span>"
        },
        {
          id: 'g3',
          balloon: {
            adjustBorderColor: false,
            color: '#ffffff'
          },
          lineColor: '#2CD9C5',
          bullet: 'none',
          bulletBorderAlpha: 1,
          bulletColor: '#FFFFFF',
          bulletSize: 5,
          hideBulletsCount: 50,
          lineThickness: 2,
          title: 'Available memory',
          color: this.isDarkMode ? '#eee' : '#353d4a',
          titleColor: this.isDarkMode ? '#eee' : '#353d4a',
          useLineColorForBulletBorder: true,
          valueField: 'memFree',
          balloonText: "<span style='font-size:12x;'>[[memFree]]</span>"
        }
      ],

      categoryField: 'time',
      categoryAxis: {
        parseDates: true,
        minPeriod: 'hh',
        dashLength: 1,
        minorGridEnabled: true,
        dateFormats: this.dataFormats,
        color: this.isDarkMode ? '#eee' : '#353d4a',
      },
      dataProvider: [],
      legend: {
        position: 'bottom',
        useMarkerColorForLabels: true
      }
    });

    this.storageGraph = this.AmCharts.makeChart('storage-graph', {
      type: 'serial',
      theme: 'light',
      startDuration: 1,
      sequencedAnimation: false,
      zoomOutText: '',
      hideCredits: true,
      marginRight: 20,
      marginLeft: 50,
      autoMarginOffset: 20,
      mouseWheelZoomEnabled: false,
      dataDateFormat: 'YYYY-MM-DD',
      valueAxes: [
        {
          id: 'v1',
          axisAlpha: 0,
          position: 'left',
          title: 'MB Utilized',
          color: this.isDarkMode ? '#eee' : '#353d4a',
          titleColor: this.isDarkMode ? '#eee' : '#353d4a',
        }
      ],
      balloon: {
        borderThickness: 1,
        shadowAlpha: 0
      },
      graphs: [
        {
          id: 'g1',
          balloon: {
            adjustBorderColor: false,
            color: '#ffffff'
            // horizontalPadding: 20,
            // verticalPadding: 20
          },
          lineColor: '#9460FF',
          bullet: 'none',
          bulletBorderAlpha: 1,

          bulletSize: 6,
          hideBulletsCount: 100,
          lineThickness: 2,
          title: 'Total storage',
          useLineColorForBulletBorder: true,
          valueField: 'storageTotal',
          balloonText: "<span style='font-size:16px;'>[[storageTotal]]</span>",
          color: this.isDarkMode ? '#eee' : '#353d4a',
          titleColor: this.isDarkMode ? '#eee' : '#353d4a',
        },
        {
          id: 'g3',
          balloon: {
            adjustBorderColor: false,
            color: '#ffffff'
          },
          lineColor: '#2CD9C5',
          bullet: 'none',
          bulletBorderAlpha: 1,
          bulletColor: '#FFFFFF',
          bulletSize: 5,
          hideBulletsCount: 50,
          lineThickness: 2,
          title: 'Available storage',
          useLineColorForBulletBorder: true,
          valueField: 'storageFree',
          balloonText: "<span style='font-size:12x;'>[[storageFree]]</span>",
          color: this.isDarkMode ? '#eee' : '#353d4a',
          titleColor: this.isDarkMode ? '#eee' : '#353d4a',
        }
      ],

      categoryField: 'time',
      categoryAxis: {
        parseDates: true,
        dashLength: 1,
        minPeriod: 'hh',
        minorGridEnabled: true,
        color: this.isDarkMode ? '#eee' : '#353d4a',
        dateFormats: this.dataFormats
      },
      dataProvider: [],
      legend: {
        position: 'bottom',
        useMarkerColorForLabels: true
      }
    });

    this.cpuGraph = this.AmCharts.makeChart('cpu-graph', {
      type: 'serial',
      theme: 'light',
      startDuration: 1,
      sequencedAnimation: false,
      zoomOutText: '',
      hideCredits: true,
      marginRight: 20,
      marginLeft: 50,
      autoMarginOffset: 20,
      mouseWheelZoomEnabled: false,
      dataDateFormat: 'YYYY-MM-DD',
      valueAxes: [
        {
          id: 'v1',
          axisAlpha: 0,
          position: 'left'
          // "title": "CPU Utilization",
        }
      ],
      balloon: {
        borderThickness: 1,
        shadowAlpha: 0
      },
      graphs: [
        {
          id: 'g1',
          balloon: {
            adjustBorderColor: false,
            color: '#ffffff'
            // horizontalPadding: 20,
            // verticalPadding: 20
          },
          lineColor: '#9460FF',
          bullet: 'none',
          bulletBorderAlpha: 1,

          bulletSize: 6,
          hideBulletsCount: 100,
          lineThickness: 2,
          title: 'CPU utilization',
          useLineColorForBulletBorder: true,
          valueField: 'cpuUtilization',
          balloonText:
            "<span style='font-size:16px;'>[[cpuUtillization]]</span>",
            color: this.isDarkMode ? '#eee' : '#353d4a',
          titleColor: this.isDarkMode ? '#eee' : '#353d4a',
        },
        {
          id: 'g2',
          balloon: {
            adjustBorderColor: false,
            color: '#ffffff'
          },
          lineColor: '#2CD9C5',
          bullet: 'none',
          bulletBorderAlpha: 1,
          bulletColor: '#FFFFFF',
          bulletSize: 5,
          hideBulletsCount: 50,
          lineThickness: 2,
          title: 'Main Process CPU utilization',
          useLineColorForBulletBorder: true,
          valueField: 'mainProcessCPUUtilization',
          balloonText:
            "<span style='font-size:12x;'>[[mainProcessCPUUtilization ]]</span>",
            color: this.isDarkMode ? '#eee' : '#353d4a',
          titleColor: this.isDarkMode ? '#eee' : '#353d4a',
        },
        {
          id: 'g3',
          balloon: {
            adjustBorderColor: false,
            color: '#ffffff'
          },
          lineColor: '#E32D2A',
          bullet: 'none',
          bulletBorderAlpha: 1,
          bulletColor: '#FFFFFF',
          bulletSize: 5,
          hideBulletsCount: 50,
          lineThickness: 2,
          title: 'Sniffer Process CPU utilization',
          useLineColorForBulletBorder: true,
          valueField: 'snifferProcessCPUUtilization',
          balloonText:
            "<span style='font-size:12px;'>[[snifferProcessCPUUtilization]]</span>"
        }
      ],

      categoryField: 'time',
      categoryAxis: {
        parseDates: true,
        dashLength: 1,
        minorGridEnabled: true,
        minPeriod: 'hh',
        color: this.isDarkMode ? '#eee' : '#353d4a',
        dateFormats: this.dataFormats
      },
      dataProvider: [],
      legend: {
        position: 'bottom',
        useMarkerColorForLabels: true
      }
    });

    this.refreshData();
  }

  setAmChartFont() {
    this.AmCharts.addInitHandler(function(chart) {
      // Set the default fontFamily for the chart
      chart.fontFamily = 'Poppins';
    }, ["serial", "pie", "xy"]);
  }

  public refreshData(): void {
    if (this.timerSubscription) {
      this.timerSubscription.unsubscribe();
    }
    this.getPage();
    this.subscribeToData();
  }

  private subscribeToData(): void {
    this.timerSubscription = timer(20000)
      .pipe(first())
      .subscribe(() => this.refreshData());
  }

  getPage() {
    this.loading = true;
    this.controllerService
      .getcontrollerGraphs(this.controllerId, this.selectedInterval)
      .subscribe({
        next: (data: any) => {
          this.lastUpdated = data.lastUpdated || null;
          this.version = data.version || null;
          this.os = data.os || null;
          this.cpu = data.cpuDetail || null;
          this.lastErrorLog = data.lastErrorLog || null;
          this.lastSnifferErrorLog = data.lastSnifferErrorLog || null;

          this.AmCharts.updateChart(this.memoryGraph, () => {
            this.showMemoryGraph = data.memory.length > 0;
            this.memoryGraph.dataProvider = data.memory;
            this.memoryGraph.categoryAxis.minPeriod =
              this.selectedInterval === 1 ? 'hh' : 'DD';
          });

          this.AmCharts.updateChart(this.storageGraph, () => {
            this.showStorageGraph = data.storage.length > 0;
            this.storageGraph.categoryAxis.minPeriod =
              this.selectedInterval === 1 ? 'hh' : 'DD';
            this.storageGraph.dataProvider = data.storage;
          });

          this.AmCharts.updateChart(this.cpuGraph, () => {
            this.showCpuGraph = data.cpu.length > 0;
            this.cpuGraph.categoryAxis.minPeriod =
              this.selectedInterval === 1 ? 'hh' : 'DD';
            this.cpuGraph.dataProvider = data.cpu;
          });
        },
        complete: () => {
          this.loading = false;
        }
      });
  }

  cancelHandler() {
    this.cancelHealth.emit('cancel');
  }

  updateGraphs(interval: number) {
    this.selectedInterval = interval;
    this.refreshData();
  }

  changeGraph(graphIndex: number) {
    if (graphIndex !== this.selectedGraph) {
      this.refreshData();
      this.selectedGraph = graphIndex;
    }
  }

  public ngOnDestroy(): void {
    if (this.memoryGraph) {
      this.memoryGraph.unsubscribe && this.memoryGraph.unsubscribe();
    }
    if (this.storageGraph) {
      this.storageGraph.unsubscribe && this.storageGraph.unsubscribe();
    }
    if (this.cpuGraph) {
      this.cpuGraph.unsubscribe && this.cpuGraph.unsubscribe();
    }
    if (this.timerSubscription) {
      this.timerSubscription.unsubscribe();
    }
  }
}
