import { Component, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { IdpService } from '../idp.service';
import { ServiceProvider } from '../../models/service_provider';
import { Pagination } from '../../models/pagination';
import { PreserveSearch } from '../../models/preserve-search';
import * as _ from 'lodash';
import { AuthenticationService } from '../../login/authentication.service';
import { GlobalMessageClass } from '../../global-messages/global-messages';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { Subject, Subscription } from 'rxjs';
import {
  debounceTime,
  distinctUntilChanged,
  switchMap,
  tap
} from 'rxjs/operators';
import { ErrorCodeEnum } from '../../shared/emuns/error.enum';
import { LicenseErrorEnum } from '../../shared/emuns/license.enum';
import { Title } from '@angular/platform-browser';
import { OtpAuthenticationService } from '../../otp-verification/otp-authentication.service';
import { RolesAndPermissionService } from '../../roles-and-permission/roles-and-permission.service';

@Component({
  selector: 'app-idp-list',
  templateUrl: './idp-list.component.html',
  styleUrls: ['./idp-list.component.css']
})
export class IdpListComponent implements OnInit {
  showHelpText: boolean = false;
  public totalCount: any = 0;
  public alert: GlobalMessageClass;
  providersData: any = {};
  private selectedIdp: ServiceProvider;
  pagination: Pagination = new Pagination();
  showPasswordConfimation = false;
  preserveSearch = new PreserveSearch();
  navigationToEdit: boolean = false;

  loading: Boolean = true;

  pageSize = [10, 25, 50, 100, 200];

  sort: string;
  sortDir: string;

  $searchIdp = new Subject();
  searchKeyword;

  showOtpDialog: boolean = false;
  otptoken: string = '';
  option: string;
  private otpStatusSubscription: Subscription;
  
  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;

  constructor(
    private router: Router,
    private idpService: IdpService,
    private route: ActivatedRoute,
    private globalMessageService: GlobalMessageService,
    private authenticationService: AuthenticationService,
    private titleService: Title,
    private OtpAuthenticationService: OtpAuthenticationService,
    private rolesPermissionService : RolesAndPermissionService
  ) {}

  ngOnInit() {
    this.titleService.setTitle('Infrastructure | IDP Management');
    this.getCRUDdata(33);
    let providerSearch = this.preserveSearch.getSearch('idpSearch');
    if (providerSearch) {
      this.pagination = providerSearch.pagination;
      this.searchKeyword = providerSearch.searchKeyword;
    }
    this.getPage(this.pagination.currentPage);
    this.subscibeToSearch();
  }

  getCRUDdata(privId){
    this.rolesPermissionService.getCrudData(privId).subscribe((res: any) => {
      this.createPermission = res?.create_permission === 1 ? true : false
      this.readPermission = res?.read_permission === 1 ? true : false
      this.updatePermission = res?.update_permission === 1 ? true : false
      this.deletePermission = res?.delete_permission === 1 ? true : false
    });
  }
  
  subscibeToSearch() {
    this.$searchIdp
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap(data => {
          this.loading = true;
          this.searchKeyword = data;
          this.pagination.currentPage = 1;
          return this.idpService.getIdp(
            this.pagination.currentPage,
            this.pagination.itemsPerPage,
            'all',
            this.sort,
            this.sortDir,
            this.searchKeyword
          );
        })
      )
      .subscribe({
        next: (idps: any) => {
          this.loading = false;
          const { recordsTotal } = idps;
          this.providersData = idps;
          this.pagination.totalItems = recordsTotal;
        },
        error: err => {
          this.loading = false;
        }
      });
  }

  getPage(page: number) {
    this.loading = true;
    this.pagination.currentPage = page;
    this.idpService
      .getIdp(
        this.pagination.currentPage,
        this.pagination.itemsPerPage,
        'all',
        this.sort,
        this.sortDir,
        this.searchKeyword
      )
      .pipe(
        tap((res: any) => {
          this.loading = false;
          this.providersData = res;
          this.totalCount = res.recordsTotal;
          this.pagination.totalItems = res.recordsTotal;
          this.pagination.currentPage = page;
        })
      )
      .subscribe({
        next: next => {},
        error: err => {
          this.loading = false;
          if (err.status === ErrorCodeEnum.Forbidden) {
            if (err._body == LicenseErrorEnum.Invalid) {
              this.authenticationService.showLicenseOnly(true);
              this.router.navigate(['/license']);
            } else {
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'You are not authorized to access this page.'
              };
              this.globalMessageService.pushAlert(this.alert);
            }
          }
        },
        complete: () => {}
      });
  }

  clearfun() {
    this.searchKeyword = '';
    this.getPage(this.pagination.currentPage);
  }

  private onSelect(idp: ServiceProvider): void {
    let mfa = localStorage.getItem('adminMfaValue');
    if (mfa === 'true') {
      this.showOtpDialog = true;
      this.option = 'edit';
      this.otpStatusSubscription = this.OtpAuthenticationService.otpStatusCheckEdit$.subscribe(
        (res: any) => {
          if (res == true) {
            let save = {
              pagination: this.pagination,
              searchKeyword: this.searchKeyword
            };
            this.preserveSearch.saveSearch('idpSearch', save);
            this.navigationToEdit = true;
            this.selectedIdp = idp;
            this.router.navigate(['/idps', this.selectedIdp.id]);
            this.OtpAuthenticationService.setOtpStatusValueEdit(false);
          }
        }
      );
    } else {
      let save = {
        pagination: this.pagination,
        searchKeyword: this.searchKeyword
      };
      this.preserveSearch.saveSearch('idpSearch', save);
      this.navigationToEdit = true;
      this.selectedIdp = idp;
      this.router.navigate(['/idps', this.selectedIdp.id]);
    }
  }

  ngOnDestroy(): void {
    if (!this.navigationToEdit) {
      this.preserveSearch.saveSearch('idpSearch', '');
    }

    if (this.otpStatusSubscription) {
      this.otpStatusSubscription.unsubscribe();
    }
  }

  cancelVerificationPassword(event) {
    this.showPasswordConfimation = false;
  }

  submitVerificationPassword(event) {
    this.idpService
      .deleteIdp(this.selectedIdp.id, event)
      .subscribe(response => {
        this.showPasswordConfimation = false;
        if (response.success) {
          this.alert = {
            closed: false,
            type: 'success',
            message: 'IDP has been deleted successfully.'
          };
          this.globalMessageService.pushAlert(this.alert);
          this.getPage(this.pagination.currentPage);
        }
      });
  }

  private onDelete(idp: ServiceProvider): void {
    let mfa = localStorage.getItem('adminMfaValue');
    if (mfa === 'true') {
      this.showOtpDialog = true;
      this.option = 'delete';
      this.showPasswordConfimation = false;
      this.otpStatusSubscription = this.OtpAuthenticationService.otpStatusCheckDel$.subscribe(
        (res: any) => {
          if (res == true) {
            this.idpService.deleteIdpByOtp(idp.id).subscribe(response => {
              this.showPasswordConfimation = false;
              if (response.success) {
                this.alert = {
                  closed: false,
                  type: 'success',
                  message: 'IDP has been deleted successfully.'
                };
                this.globalMessageService.pushAlert(this.alert);
                this.getPage(this.pagination.currentPage);
              }
            });
            this.showOtpDialog = false;
            this.OtpAuthenticationService.setOtpStatusValueDel(false);
          }
        }
      );
    } else {
      this.selectedIdp = idp;
      this.showPasswordConfimation = true;
    }
  }

  add_idp(): void {
    this.router.navigate(['/create-idp']);
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getPage(this.pagination.currentPage);
  }

  sortRecord(term: string) {
    this.sortDir = this.sortDir == 'asc' ? 'desc' : 'asc';
    this.sort = term;
    this.getPage(this.pagination.currentPage);
  }

  cancelOtpVerificationPassword(event: any) {
    this.showOtpDialog = false;
    this.OtpAuthenticationService.setOtpStatusValueEdit(false);
    this.OtpAuthenticationService.setOtpStatusValueDel(false);
  }

  submitOtpVerificationPassword(event: any) {
    this.otptoken = event;
    // this.otptoken = event
  }

  getClosePopup(event: any) {
    this.showOtpDialog = false;
  }
}
