import { Injectable } from '@angular/core';
import { HttpClient as AuthHttp, HttpHeaders } from '@angular/common/http';
import { ConfigService } from './services/config.service';
import { Pagination } from './models/pagination';

import { AuthShortlivedCookieService } from './auth-shortlived-cookie.service';
import { map } from 'rxjs/operators';
import { DeviceTypeEnum } from './shared/emuns/client.enum'

@Injectable()
export class IotService {
  private bridgeUrl: string;

  constructor(
    private authHttp: AuthHttp,
    private authShortlivedCookie: AuthShortlivedCookieService,
    private configService: ConfigService
  ) {
    const baseUrl = configService.getConfiguration().apiBaseUrl;
    this.bridgeUrl = baseUrl + '/serverBridge';
  }

  getAllBridgess(
    pagination: Pagination,
    name: string = '',
    type: string = DeviceTypeEnum.ServerBridge
  ) {
    const currentPage = pagination.currentPage - 1 || 0;
    const itemsPerPage = pagination.itemsPerPage || 10;

    let _name: string = '';

    if (name.trim()) {
      _name = name.trim();
    }
    return this.authHttp
      .get(
        this.bridgeUrl +
          `?type=${type}&offset=${currentPage}&limit=${itemsPerPage}&name=${_name}`
      )
      .pipe(map(response => response));
  }

  deletePolicy(id, verificationPassword) {
    const options = {
      headers: new HttpHeaders({
        'Content-Type': 'application/json'
      }),
      body: { verificationPassword }
    };
    return this.authHttp
      .delete(this.bridgeUrl + `/bridge/${id}`, options)
      .pipe(map(response => response));
  }

  enablePolicy(id, value) {
    return this.authHttp
      .patch(this.bridgeUrl + `/bridge/${id}/enable/${value}`, {})
      .pipe(map(response => response));
  }

  getSinglePolicy(id) {
    return this.authHttp
      .get(`${this.bridgeUrl}/bridge/${id}`)
      .pipe(map(response => response));
  }

  getAllDropDownData(clientOffset, clientName, type: string = DeviceTypeEnum.ServerBridge) {
    return this.authHttp
      .get(
        `${this.bridgeUrl}/getServerClients?type=${type}&offset=${clientOffset}&name=${clientName}`
      )
      .pipe(map(response => response));
  }

  getPhysicalDevices(deviceOffset, deviceName, type: string = DeviceTypeEnum.ServerBridge) {
    return this.authHttp
      .get(
        `${this.bridgeUrl}/physical_devices?type=${type}&offset=${deviceOffset}&search=${deviceName}`
      )
      .pipe(map(response => response));
  }

  getDevices(deviceOffset, deviceName, type: string = DeviceTypeEnum.IOT) {
    return this.authHttp
      .get(
        `${this.bridgeUrl}/devices?type=${type}&offset=${deviceOffset}&search=${deviceName}`
      )
      .pipe(map(response => response));
  }

  createService(policyOption, type: string = DeviceTypeEnum.ServerBridge) {
    return this.authHttp
      .post(`${this.bridgeUrl}?type=${type}`, policyOption)
      .pipe(map(response => response));
  }

  createB2BBridge(
    name,
    clientId = null,
    userIds = [],
    portalUrl,
    portalPort,
    minVncPort,
    maxVncPort
  ) {
    return this.authHttp
      .post(`${this.bridgeUrl}/b2b-bridge`, {
        name,
        clientId,
        userIds,
        portalUrl,
        portalPort,
        minVncPort,
        maxVncPort
      })
      .pipe(map(response => response));
  }

  createRBIBridge(
    name,
    clientId = null,
    userIds = [],
    portalUrl,
    portalPort,
    minVncPort,
    maxVncPort
  ) {
    return this.authHttp
      .post(`${this.bridgeUrl}/rbi-bridge`, {
        name,
        clientId,
        userIds,
        portalUrl,
        portalPort,
        minVncPort,
        maxVncPort
      })
      .pipe(map(response => response));
  }

  updateService(policyOption, id) {
    return this.authHttp.put(`${this.bridgeUrl}/bridge/${id}`, policyOption);
  }

  updateB2BBridge(
    id,
    name,
    clientId = null,
    userIds = [],
    portalUrl,
    portalPort,
    minVncPort,
    maxVncPort
  ) {
    return this.authHttp.put(`${this.bridgeUrl}/bridge/b2b-bridge/${id}`, {
      name,
      userIds,
      clientId,
      portalUrl,
      portalPort,
      minVncPort,
      maxVncPort
    });
  }

  updateRBIBridge(
    id,
    name,
    clientId = null,
    userIds = [],
    portalUrl,
    portalPort,
    minVncPort,
    maxVncPort
  ) {
    return this.authHttp.put(`${this.bridgeUrl}/bridge/rbi-bridge/${id}`, {
      name,
      userIds,
      clientId,
      portalUrl,
      portalPort,
      minVncPort,
      maxVncPort
    });
  }

  sendPackage(id) {
    return this.authHttp
      .get(`${this.bridgeUrl}/bridge/sendBridge/${id}`)
      .pipe(map(response => response));
  }

  downloadPackage(id) {
    this.authShortlivedCookie.setFor(5);
    // window.open(`/api/serverBridge/bridge/download/${id}`, '_blank');
    window.location.href = `/api/serverBridge/bridge/download/${id}`;
  }
}
