import { Component, OnInit } from '@angular/core';
import { IotService } from '../iot.service';
import { Pagination } from '../models/pagination';
import { debounceTime, distinctUntilChanged, switchMap } from 'rxjs/operators';
import { Observable, of, Subject } from 'rxjs';
import { Router } from '@angular/router';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { GlobalMessageClass } from '../global-messages/global-messages';
import { PreserveSearch } from '../models/preserve-search';
import { HttpClient } from '@angular/common/http';
import { ErrorCodeEnum } from '../shared/emuns/error.enum';
import { Title } from '@angular/platform-browser';
import { RolesAndPermissionService } from '../roles-and-permission/roles-and-permission.service';

@Component({
  selector: 'app-iot',
  templateUrl: './iot.component.html',
  styleUrls: ['./iot.component.css']
})
export class IotComponent implements OnInit {
  showHelpText: boolean = false;
  type: string = 'iot';
  title: string = 'Server Bridges';
  buttonTitle: string = 'Server Bridge';
  helpText: string = '';
  pagination = new Pagination();
  searchKeyword: any = '';
  isSearching: boolean = false;
  bridges: any = {};
  $searchPolicy = new Subject();
  loading: boolean = true;
  serverBridgeToDelete;

  showPasswordConfimation: Boolean = false;

  pageSize = [10, 25, 50, 100, 200];

  preserveSearch = new PreserveSearch();
  navigationToEdit: boolean = false;

  public alert: GlobalMessageClass;

  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;

  constructor(
    private router: Router,
    private iotService: IotService,
    private globalMessageService: GlobalMessageService,
    private http: HttpClient,
    private titleService: Title,
    private rolesPermissionService: RolesAndPermissionService
  ) {
    this.type = router.url.replace('/', '');
    if(this.type == 'rdi-bridge'){
      this.type = 'b2b-bridge'
    }
    switch (this.type) {
      case 'iot-bridge':
        this.title = 'Device Bridges (ARM)';
        this.buttonTitle = 'Device Bridge (ARM)'
        this.helpText =
          'Create a bridge to allow IoT devices to connect to services behind Invisily gateways.';
        this.getCRUDdata(29);

        break;
      case 'client-bridge':
        this.title = 'Device Bridges (x86)';
        this.buttonTitle = 'Device Bridge (x86)'
        this.helpText =
          'Create a bridge to allow thin clients access to a VDI server behind an Invisily gateway.';
        this.getCRUDdata(28);
        break;
      case 'client-bridge':
        this.title = 'Device Bridges (x86)';
        this.buttonTitle = 'Device Bridge (x86)'
        this.helpText =
          'Create a bridge to allow thin clients access to a VDI server behind an Invisily gateway.';
        this.getCRUDdata(28);
        break;
      case 'b2b-bridge':
        this.title = 'RDI Bridges';
        this.buttonTitle = 'RDI Bridge'
        this.helpText =
          'Create Remote Desktop Isolation bridge to allow users to access Invisily protected services from within a remote virtual desktop without installing Invisily client on their devices.';
        this.getCRUDdata(30);
        break;
      case 'rbi-bridge':
        this.title = 'RBI Bridges';
        this.buttonTitle = 'RBI Bridge'
        this.helpText =
          'Create a bridge to access Invisily protected services from a remote browser.';
        this.getCRUDdata(67);
        break;
      default:
        this.title = 'Server Bridges';
        this.buttonTitle = 'Server Bridge'
        this.helpText =
          'Create a bridge to allow a VDI server access to services behind Invisily gateways.';
        this.getCRUDdata(27);
        break;
    }
    this.titleService.setTitle('Infrastructure | ' + this.title);
  }

  ngOnInit() {
    let saveSearch = this.preserveSearch.getSearch('serverBridgeSaveSearch');
    if (saveSearch) {
      this.pagination = saveSearch.pagination;
      this.searchKeyword = saveSearch.searchKeyword;
    }
    this.subscibeToSearch();
    this.pagination.id = 'bridges';
    this.getAllBridges(this.pagination);
  }

  getCRUDdata(privId) {
    this.rolesPermissionService.getCrudData(privId).subscribe((res: any) => {
      this.createPermission = res?.create_permission === 1 ? true : false;
      this.readPermission = res?.read_permission === 1 ? true : false;
      this.updatePermission = res?.update_permission === 1 ? true : false;
      this.deletePermission = res?.delete_permission === 1 ? true : false;
    });
  }

  getAllBridges(pagination: Pagination) {
    this.loading = true;
    this.iotService
      .getAllBridgess(pagination, this.searchKeyword, this.type)
      .subscribe({
        next: (bridges: any) => {
          this.loading = false;
          this.pagination.totalItems = bridges.recordsTotal;
          this.bridges = bridges;
        },
        error: err => {
          this.loading = false;
        }
      });
  }

  clearfun() {
    this.searchKeyword = '';
    this.getAllBridges(this.pagination);
  }

  subscibeToSearch() {
    this.$searchPolicy
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap(data => {
          this.loading = true;
          this.isSearching = true;
          this.searchKeyword = data;
          this.pagination.currentPage = 1;
          this.getAllBridges(this.pagination);
          return of('Fetching data...');
        })
      )
      .subscribe(() => {
        this.isSearching = false;
      });
  }

  enablePolicy(id: any, value: any) {
    this.iotService.enablePolicy(id, value).subscribe(res => {
      this.getAllBridges(this.pagination);
    });
  }

  editHandler(policy) {
    const search = {
      pagination: this.pagination,
      searchKeyword: this.searchKeyword
    };
    this.preserveSearch.saveSearch('serverBridgeSaveSearch', search);
    this.navigationToEdit = true;

    if(this.type === 'b2b-bridge'){
      this.type = 'rdi-bridge';
    }
    this.router.navigate([`edit/${this.type}/${policy.id}`]);
  }

  ngOnDestroy(): void {
    if (!this.navigationToEdit) {
      this.preserveSearch.saveSearch('serverBridgeSaveSearch', '');
    }
  }

  pageChange(page: any) {
    this.pagination.currentPage = page;
    this.getAllBridges(this.pagination);
  }

  addAccessPolicyHandler() {
    if(this.type === 'b2b-bridge'){
      this.type = 'rdi-bridge';
    }
    this.router.navigate(['/create/' + this.type]);
  }

  onSend(bridge) {
    this.iotService.sendPackage(bridge.id).subscribe(
      data => {
        this.globalMessageService.pushAlertIfNotPresent({
          closed: false,
          type: 'success',
          message:
            'You wil receive bridge server package within two to three minutes.',
          sticky: false
        });
      },
      error => {
        this.globalMessageService.pushAlertIfNotPresent({
          closed: false,
          type: 'danger',
          message: JSON.parse(error._body).message,
          sticky: false
        });
      }
    );
  }

  bridgeDownload(id) {
    this.globalMessageService.pushAlert({
      closed: false,
      type: 'success',
      message: 'Starting download...',
      sticky: false
    });

    this.http
      .get(`/api/serverBridge/bridge/download/${id}`, { responseType: 'blob' })
      .subscribe(
        data => {
          this.globalMessageService.pushAlertIfNotPresent({
            closed: false,
            type: 'success',
            message: 'Download in progress...',
            sticky: false
          });

          this.iotService.downloadPackage(id);
        },
        error => {
          this.globalMessageService.pushAlertIfNotPresent({
            closed: false,
            type: 'danger',
            message:
              error.status === ErrorCodeEnum.NotFound
                ? 'Client installer does not exist'
                : error.message,
            sticky: false
          });
        }
      );
  }

  onDownload(bridge) {
    this.iotService.downloadPackage(bridge.id);
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getAllBridges(this.pagination);
  }

  deleteServerBridge(server) {
    this.showPasswordConfimation = true;
    this.serverBridgeToDelete = server;
  }

  cancelVerificationPassword(event) {
    this.showPasswordConfimation = false;
  }

  submitVerificationPassword(event) {
    this.loading = true;
    this.iotService.deletePolicy(this.serverBridgeToDelete.id, event).subscribe(
      res => {
        this.serverBridgeToDelete = null;
        this.loading = false;
        this.showPasswordConfimation = false;
        this.alert = {
          closed: false,
          type: 'success',
          message: 'Server Bridge has been deleted successfully.'
        };
        this.globalMessageService.pushAlert(this.alert);
        this.getAllBridges(this.pagination);
      },
      err => {
        this.loading = false;
      }
    );
  }
}
