import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { LdapService } from '../ldap.service';
import { Ldap } from '../../models/ldap';
import { Pagination } from '../../models/pagination';
import { PreserveSearch } from '../../models/preserve-search';
import { GlobalMessageClass } from '../../global-messages/global-messages';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { AuthenticationService } from '../../login/authentication.service';
import { Subject, Subscription } from 'rxjs';
import {
  debounceTime,
  distinctUntilChanged,
  switchMap,
  tap
} from 'rxjs/operators';
import { ErrorCodeEnum } from '../../shared/emuns/error.enum';
import { LicenseErrorEnum } from '../../shared/emuns/license.enum';
import { Title } from '@angular/platform-browser';
import { OtpAuthenticationService } from '../../otp-verification/otp-authentication.service';
import { RolesAndPermissionService } from '../../roles-and-permission/roles-and-permission.service';

@Component({
  selector: 'app-ldapuser-list',
  templateUrl: './ldapuser-list.component.html',
  styleUrls: ['./ldapuser-list.component.css']
})
export class LdapuserListComponent implements OnInit {
  showHelpText: boolean = false;
  public alert: GlobalMessageClass;
  ldaps: any = {};
  private ldapsList = new Array();
  private selectedLdap: Ldap;
  pagination: Pagination = new Pagination();
  showPasswordConfimation = false;

  loading: Boolean = true;

  pageSize = [10, 25, 50, 100, 200];

  $searchLdap = new Subject();
  searchKeyword;

  preserveSearch = new PreserveSearch();
  navigationToEdit: boolean = false;
  
  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;

  showOtpDialog: boolean = false;
  otptoken: string = '';
  option: string;
  private otpStatusSubscription: Subscription;

  constructor(
    private router: Router,
    private ldapService: LdapService,
    private route: ActivatedRoute,
    private globalMessageService: GlobalMessageService,
    private authenticationService: AuthenticationService,
    private titleService: Title,
    private OtpAuthenticationService: OtpAuthenticationService,
    private rolesPermissionService : RolesAndPermissionService
  ) {}

  subscibeToSearch() {
    this.$searchLdap
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap(data => {
          this.loading = true;
          this.searchKeyword = data;
          this.pagination.currentPage = 1;
          return this.ldapService.getLdaps(
            this.pagination.currentPage,
            this.pagination.itemsPerPage,
            this.searchKeyword
          );
        })
      )
      .subscribe({
        next: (ldaps: any) => {
          this.loading = false;
          const { recordsTotal } = ldaps;
          this.ldaps = ldaps;
          this.pagination.totalItems = recordsTotal;
        },
        error: err => {
          this.loading = false;
        }
      });
  }

  getPage(page: number) {
    this.loading = true;
    this.pagination.currentPage = page;
    this.ldapService
      .getLdaps(
        this.pagination.currentPage,
        this.pagination.itemsPerPage,
        this.searchKeyword
      )
      .pipe(
        tap((res: any) => {
          this.loading = false;
          this.ldaps = res;
          this.pagination.totalItems = res.recordsTotal;
          this.pagination.currentPage = page;
        })
      )
      .subscribe({
        next: next => {},
        error: err => {
          this.loading = false;
          if (err.status === ErrorCodeEnum.Forbidden) {
            if (err._body == LicenseErrorEnum.Invalid) {
              this.authenticationService.showLicenseOnly(true);
              this.router.navigate(['/license']);
            } else {
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'You are not authorized to access this page.'
              };
              this.globalMessageService.pushAlert(this.alert);
            }
          }
        },
        complete: () => {}
      });
  }

  clearfun() {
    this.searchKeyword = '';
    this.getPage(this.pagination.currentPage);
  }

  public ngOnInit(): void {
    let ldapSearch = this.preserveSearch.getSearch('ldapSearch');
    this.getCRUDdata(32);
    if (ldapSearch) {
      this.pagination = ldapSearch.pagination;
      this.searchKeyword = ldapSearch.searchKeyword;
    }
    this.getPage(this.pagination.currentPage);
    this.subscibeToSearch();
    this.titleService.setTitle('Infrastructure | LDAP Configuration');
  }

  getCRUDdata(privId){
    this.rolesPermissionService.getCrudData(privId).subscribe((res: any) => {
      this.createPermission = res?.create_permission === 1 ? true : false
      this.readPermission = res?.read_permission === 1 ? true : false
      this.updatePermission = res?.update_permission === 1 ? true : false
      this.deletePermission = res?.delete_permission === 1 ? true : false
    });
  }

  private onSelect(ldap: Ldap): void {
    let mfa = localStorage.getItem('adminMfaValue');
    if (mfa === 'true') {
      this.showOtpDialog = true;
      this.option = 'edit';
      this.otpStatusSubscription = this.OtpAuthenticationService.otpStatusCheckEdit$.subscribe(
        (res: any) => {
          if (res == true) {
            let ldapSearch = {
              pagination: this.pagination,
              searchKeyword: this.searchKeyword
            };
            this.preserveSearch.saveSearch('ldapSearch', ldapSearch);
            this.navigationToEdit = true;
            this.selectedLdap = ldap;
            this.router.navigate(['/ldap', this.selectedLdap.id]);
            this.OtpAuthenticationService.setOtpStatusValueEdit(false);
          }
        }
      );
    } else {
      let ldapSearch = {
        pagination: this.pagination,
        searchKeyword: this.searchKeyword
      };
      this.preserveSearch.saveSearch('ldapSearch', ldapSearch);
      this.navigationToEdit = true;
      this.selectedLdap = ldap;
      this.router.navigate(['/ldap', this.selectedLdap.id]);
    }
  }

  ngOnDestroy(): void {
    if (!this.navigationToEdit) {
      this.preserveSearch.saveSearch('ldapSearch', '');
    }

    if (this.otpStatusSubscription) {
      this.otpStatusSubscription.unsubscribe();
    }
  }

  cancelVerificationPassword(event) {
    this.showPasswordConfimation = false;
  }

  submitVerificationPassword(event) {
    this.ldapService
      .delete_ldap(this.selectedLdap.id, event)
      .subscribe(response => {
        this.showPasswordConfimation = false;
        if (response.success) {
          this.alert = {
            closed: false,
            type: 'success',
            message: 'LDAP User has been deleted successfully.'
          };
          this.globalMessageService.pushAlert(this.alert);
          this.getPage(this.pagination.currentPage);
        }
      });
  }

  private onDelete(ldap: Ldap): void {
    let mfa = localStorage.getItem('adminMfaValue');
    if (mfa === 'true') {
      this.showOtpDialog = true;
      this.option = 'delete';
      this.otpStatusSubscription = this.OtpAuthenticationService.otpStatusCheckDel$.subscribe(
        (res: any) => {
          if (res == true) {
            this.showPasswordConfimation = false;
            this.ldapService.delete_ldap_by_otp(ldap.id).subscribe(response => {
              this.showPasswordConfimation = false;
              if (response.success) {
                this.alert = {
                  closed: false,
                  type: 'success',
                  message: 'LDAP User has been deleted successfully.'
                };
                this.globalMessageService.pushAlert(this.alert);
                this.getPage(this.pagination.currentPage);
              }
            });
            this.showOtpDialog = false;
            this.OtpAuthenticationService.setOtpStatusValueDel(false);
          }
        }
      );
    } else {
      this.selectedLdap = ldap;
      this.showPasswordConfimation = true;
    }
  }

  add_new_user(): void {
    this.router.navigate(['/create-ldapuser']);
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getPage(this.pagination.currentPage);
  }

  cancelOtpVerificationPassword(event: any) {
    this.showOtpDialog = false;
    this.OtpAuthenticationService.setOtpStatusValueEdit(false);
    this.OtpAuthenticationService.setOtpStatusValueDel(false);
  }

  submitOtpVerificationPassword(event: any) {
    this.otptoken = event;
    // this.otptoken = event
  }

  getClosePopup(event: any) {
    this.showOtpDialog = false;
  }
}
