import { Injectable } from '@angular/core';
import { Router } from '@angular/router';
import { ConfigService } from '../services/config.service';
import { BehaviorSubject } from 'rxjs';
import { Observable } from 'rxjs';
import * as moment from 'moment';
import { GlobalMessageClass } from '../global-messages/global-messages';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { map } from 'rxjs/operators';

@Injectable()
export class AuthenticationService {
  public alert: GlobalMessageClass;
  private authenticationUrl = '/authentication';
  public token: string;
  private tokenExpirationTimer: any;
  private _showNavBar: BehaviorSubject<boolean> = new BehaviorSubject<boolean>(
    null
  );
  private _showLicenseOnly: BehaviorSubject<boolean> = new BehaviorSubject<
    boolean
  >(null);
  private _checkUserRole: BehaviorSubject<boolean> = new BehaviorSubject<
    boolean
  >(null);
  private _logout: BehaviorSubject<boolean> = new BehaviorSubject<boolean>(
    null
  );
  public showNavBarEmitter: Observable<
    boolean
  > = this._showNavBar.asObservable();
  public showLicenseOnlyEmitter: Observable<
    boolean
  > = this._showLicenseOnly.asObservable();
  private _firstName: BehaviorSubject<string> = new BehaviorSubject<string>(
    null
  );

  public checkUserRoleEmitter: Observable<
    boolean
  > = this._checkUserRole.asObservable();

  public showLogOutEmitter: Observable<boolean> = this._logout.asObservable();
  public showFirstNameEmitter: Observable<
    string
  > = this._firstName.asObservable();
  private apiBaseUrl: string;

  public authStateChange: BehaviorSubject<boolean> = new BehaviorSubject<
    boolean
  >(null);

  httpOptions = {
    headers: new HttpHeaders({ 'Content-Type': 'application/json' })
  };
  constructor(
    private authHttp: HttpClient,
    private router: Router,
    private configService: ConfigService,
    private globalMessageService: GlobalMessageService
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.authenticationUrl = this.apiBaseUrl + this.authenticationUrl;
  }

  showNavBar(ifShow: boolean) {
    this._showNavBar.next(ifShow);
  }

  showLicenseOnly(ifShow: boolean) {
    this._showLicenseOnly.next(ifShow);
  }

  checkUserRole(ifShow: boolean) {
    this._checkUserRole.next(ifShow);
  }

  showLogOut(logout: boolean) {
    this._logout.next(logout);
  }

  showFirstName(name: string) {
    this._firstName.next(name);
  }

  createLinToken(email: string) {
    return this.authHttp
      .post(`${this.authenticationUrl}/createLinToken`, { email })
      .pipe(map(response => response));
  }

  validateLinToken(email, password, token) {
    return this.authHttp
      .post(`${this.authenticationUrl}/validateLinToken`, {
        email,
        password,
        token
      })
      .pipe(map(response => response));
  }
  validateLinTokenByModal(email, token) {
    return this.authHttp
      .post(`${this.authenticationUrl}/validateLinTokenModal`, {
        email,
        token
      })
      .pipe(map(response => response));
  }

  validateOTPToken(email, token) {
    return this.authHttp
      .post(`${this.authenticationUrl}/validateOTPToken`, { email, token })
      .pipe(map(response => response));
  }

  passRecaptcha(email) {
    return this.authHttp
      .post(`${this.authenticationUrl}/passRecaptchaStatus`, { email })
      .pipe(map(response => response));
  }

  getRecaptchaSiteKey() {
    return this.authHttp
      .get(`${this.authenticationUrl}/getRecaptchaSiteKey`)
      .pipe(map(response => response));
  }

  login(username: string, password: string): Observable<any> {
    return this.authHttp
      .post(
        this.authenticationUrl,
        JSON.stringify({ username: username, password: password }),
        this.httpOptions
      )
      .pipe(
        map((response: Response | any) => {
          // login successful if there's a jwt token in the response
          let res = response;
          let token = res.token;
          let user = res.user;
          if (token) {
            // set token property
            this.token = token;

            // return true to indicate successful login
            return {
              status: true,
              message: res.message,
              user: user,
              loginResponse: res,
              admin_mfa: res.admin_mfa
            };
          } else if (res.admin_mfa) {
            return {
              status: true,
              message: res.message,
              user: user,
              admin_mfa: res.admin_mfa
            };
          } else {
            // return false to indicate failed login
            let returnObj: any = {
              status: false,
              message: res.message
            };
            if (res.question1 && res.question2 && res.question3) {
              returnObj = {
                ...returnObj,
                question1: res.question1,
                question2: res.question2,
                question3: res.question3,
                recaptchaConfigured: res.recaptchaConfigured
              };
            }
            return returnObj;
          }
        })
      );
  }

  changePassword(password: string): Observable<any> {
    return this.authHttp
      .post(
        this.authenticationUrl + '/changePassword',
        { password: password },
        this.httpOptions
      )
      .pipe(response => response);
  }

  updateToken(token) {
    let currentUser = JSON.parse(localStorage.getItem('currentUser'));

    if (currentUser) {
      currentUser.token = token;
      localStorage.setItem('currentUser', JSON.stringify(currentUser));
    }
  }

  fakeLogOut() {
    this.showNavBar(false);
    //this.checkUserRole(false);
    this.showLogOut(true);
  }

  logout(isSessionExpired: boolean) {
    this.authHttp
      .post(this.authenticationUrl + '/logout', {})
      .pipe(map(response => response))
      .subscribe(response => {
        this.globalMessageService.removeAlertByName('LICENSE_NOTIFICATION');
      });

    this.token = null;
    localStorage.removeItem('currentUser');
    localStorage.removeItem('token');
    localStorage.removeItem('tokenExpiry');
    sessionStorage.clear();
    // Just clear the messages without animating them when logging out
    this.globalMessageService.disableAnimatedAlerts();
    // Clearing alerts right after disabling animations doesn't give UI state
    // updates enough time to actually disable it, so we queue using timeout
    setTimeout(() => {
      this.globalMessageService.clearAlerts();
      this.globalMessageService.enableAnimatedAlerts();
    }, 0);

    this.showNavBar(false);
    //this.checkUserRole(false);
    this.showLogOut(true);
    if (isSessionExpired) {
      this.router.navigate(['login', { sessionExpired: true }]);
    } else {
      this.router.navigate(['login']);
    }
  }

  autologout(time: any) {
    const logoutAfter = moment
      .duration(moment(time).diff(moment()))
      .asMilliseconds();
    this.tokenExpirationTimer = setTimeout(() => {
      // this.logout(true);
    }, logoutAfter - 60 * 1000);
  }

  getToken(): string {
    let currentUser = JSON.parse(localStorage.getItem('currentUser'));
    return currentUser.token;
  }

  updateTokenExpiry() {
    let userFirstName = this.getUserFirstName();
    let token = this.getToken();
    let expiryTime = parseInt(localStorage.getItem('expiryTime'));
    let tokenExpiry: moment.Moment = moment().add(expiryTime, 'minutes');
    let userName = this.getUserName();

    // store username and jwt token in local storage to keep user logged in between page refreshes
    localStorage.setItem(
      'currentUser',
      JSON.stringify({
        userFirstName: userFirstName,
        userName: userName,
        token: token,
        tokenExpiry: tokenExpiry
      })
    );
  }

  getUserInfo(): any {
    return JSON.parse(localStorage.getItem('currentUser'));
  }

  getUserFirstName(): any {
    return JSON.parse(localStorage.getItem('currentUser')).userFirstName;
  }

  getUserName(): any {
    return JSON.parse(localStorage.getItem('currentUser')).userName;
  }

  isTokenExpired(): boolean {
    return false; //!tokenNotExpired();
  }

  getTokenExpirationDate(): moment.Moment {
    return moment(this.getUserInfo().tokenExpiry);
  }

  resendOtp(email: string) {
    return this.authHttp
      .post(`${this.authenticationUrl}/createLinToken`, {
        email,
        resendQR: true
      })
      .pipe(map(response => response));
  }

  validateAnswers(
    email: string,
    answer1: string,
    answer2: string,
    answer3: string
  ) {
    return this.authHttp
      .post(`${this.authenticationUrl}/validateAnswers`, {
        email,
        answer1,
        answer2,
        answer3
      })
      .pipe(map(response => response));
  }
}
