import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { Pagination } from '../models/pagination';
import { OrganizationService } from './organization.service';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { Observable, Subject } from 'rxjs';
import { PreserveSearch } from '../models/preserve-search';
import { debounceTime, distinctUntilChanged, switchMap } from 'rxjs/operators';
import { ConfirmModalComponent } from '../shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';
import { RolesAndPermissionService } from '../roles-and-permission/roles-and-permission.service';

@Component({
  selector: 'app-organization-list-view',
  templateUrl: './organization-list-view.component.html',
  styleUrls: ['./organization-list-view.component.css']
})
export class OrganizationListViewComponent implements OnInit {
  showHelpText: boolean = false;
  pagination = new Pagination();
  organizations: any = {};
  allOrganizations: any = {};
  public actionsDropdown: number = 0;
  $searchOrganizations = new Subject();
  organizationTobeDeleted: any;
  isNameSort: boolean = false;
  searchWord;
  organizationAddForm = { name: '' };
  preserveSearch = new PreserveSearch();
  navigationToEditOrganization: boolean = false;
  addOrganization = false;

  loading: boolean = true;

  pageSize = [10, 25, 50, 100, 200];

  sort: string;
  sortDir: string;

  selectAll = false;

  selectAllExcept: any = new Set();
  selectedOrganizations: any = new Set();

  isDarkMode : boolean;

  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;


  constructor(
    private router: Router,
    private organizationService: OrganizationService,
    private globalMessageService: GlobalMessageService,
    public dialog: MatDialog,
    private titleService: Title,
    private rolesPermissionService: RolesAndPermissionService
  ) {}

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.titleService.setTitle('Organizations');
    this.getCRUDdata(47);
    const _saveSearch = this.preserveSearch.getSearch('organizationSearch');
    if (!_saveSearch) {
      this.getOrganizations();
    } else {
      this.searchWord = _saveSearch.searchKey;
      this.pagination = _saveSearch.pagination;
      this.getOrganizations();
    }
    this.subscribeToSearch();
  }

  ngOnDestroy() {
    if (!this.navigationToEditOrganization) {
      this.preserveSearch.saveSearch('organizationSearch', '');
    }
  }

  subscribeToSearch() {
    this.$searchOrganizations
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((search: { searchWord: string; typing: boolean }) => {
          this.loading = true;
          if (search.typing) {
            this.pagination.currentPage = 1;
          }
          return this.organizationService.searchOrganizations(
            search.searchWord,
            this.pagination.itemsPerPage,
            this.pagination.currentPage,
            this.sort,
            this.sortDir
          );
        })
      )
      .subscribe((organizations: any) => {
        this.organizations = organizations;
        this.pagination.totalItems = organizations.recordsTotal;
        this.loading = false;
      });
  }

  createOrganization() {
    this.router.navigate(['/create-organization']);
  }

  addOrganizationCancelHandler() {
    this.addOrganization = false;
  }

  getCRUDdata(privId){
    this.rolesPermissionService.getCrudData(privId).subscribe((res: any) => {
      this.createPermission = res?.create_permission === 1
      this.readPermission = res?.read_permission === 1
      this.updatePermission = res?.update_permission === 1
      this.deletePermission = res?.delete_permission === 1
    });
  }

  getOrganizations() {
    this.loading = true;
    this.organizationService
      .getOrganizations(
        this.pagination,
        this.sort,
        this.sortDir,
        this.searchWord
      )
      .subscribe(
        (response: any) => {
          this.loading = false;
          response.data = response.data.map(_organization => {
            return { ..._organization, isChecked: false };
          });
          this.organizations = response;
          this.pagination.totalItems = response.recordsTotal;
          this.allOrganizations = this.organizations;
        },
        error => {
          this.loading = false;
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'danger',
            message: 'Error in getting Organizations',
            sticky: false
          });
        }
      );
  }

  clearfun() {
    this.searchWord = '';
    this.getOrganizations();
  }

  addOrganizationHandler() {
    this.addOrganization = true;
    this.organizationAddForm = {
      name: ''
    };
  }

  addOrganizationSaveHandler() {
    const { name } = this.organizationAddForm;
    let addedOrganization = {
      name: name.trim(),
      department_count: 0
    };
    this.organizationService.addNewOrganization(addedOrganization).subscribe(
      response => {
        this.addOrganization = false;
        this.organizationAddForm = null;
        this.getOrganizations();
      },
      error => {}
    );
  }

  editOrganization(organization) {
    this.saveFilterState();
    this.router.navigate(['/create-organization', organization.id]);
  }

  viewOrganization(organization) {
    this.saveFilterState();
    this.router.navigate(['/view-organization', organization.id]);
  }

  saveFilterState() {
    const save = {
      pagination: this.pagination,
      searchKey: this.searchWord
    };
    this.preserveSearch.saveSearch('organizationSearch', save);
    this.navigationToEditOrganization = true;
  }

  deleteOrganization(organization) {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: `This will delete associated Departments as well. Proceed?`,
      },
    });

    dialogRef.afterClosed().subscribe((result) => {
      if (result){
        this.organizationTobeDeleted = organization;
        this.organizationService
          .deleteOrganization(this.organizationTobeDeleted.id)
          .subscribe(
            response => {
              this.getOrganizations();
              this.globalMessageService.pushAlert({
                closed: false,
                type: 'success',
                message: 'Organization has been deleted successfully.',
                sticky: false
              });
            },
            error => {
              this.loading = false;
            }
          );
      }
    });
  }

  pageChange(page) {
    this.pagination.currentPage = page;
    if (this.searchWord) {
      this.$searchOrganizations.next({
        searchWord: this.searchWord,
        typing: false
      });
    } else {
      this.getOrganizations();
    }
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getOrganizations();
  }

  sortRecord(term: string) {
    this.sortDir = this.sortDir == 'asc' ? 'desc' : 'asc';
    this.sort = term;
    this.getOrganizations();
  }
}
