import { Injectable } from '@angular/core';
import { HttpClient as AuthHttp } from '@angular/common/http';
import { map } from 'rxjs/operators';
import { ConfigService } from '../services/config.service';

@Injectable()
export class OrganizationService {
  private apiBaseUrl: string;
  private organizationsUrl = '/organizations';
  private departmentsUrl = '/departments';

  constructor(
    private authHttp: AuthHttp,
    private configService: ConfigService
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.organizationsUrl = this.apiBaseUrl + this.organizationsUrl;
  }

  getOrganizations(
    pagination,
    sort: string = 'id',
    dir: string = 'desc',
    searchKey = ''
  ) {
    let query =
      `?limit=${pagination.itemsPerPage}` +
      `&offset=${pagination.currentPage - 1}` +
      `&sort=${sort}` +
      `&dir=${dir}` +
      `&searchTerm=${searchKey}`;

    return this.authHttp
      .get(`${this.organizationsUrl}${query}`)
      .pipe(map(response => response));
  }

  getOrganization(id) {
    return this.authHttp
      .get(`${this.organizationsUrl}/${id}`)
      .pipe(map(response => response));
  }

  getOrganizationByName(name) {
    return this.authHttp
      .get(`${this.organizationsUrl}/names/${name}`)
      .pipe(map(response => response));
  }

  getOrganizationNames() {
    return this.authHttp
      .get(`${this.organizationsUrl}/names`)
      .pipe(map(response => response));
  }

  getDepartments(
    organizationId = 0,
    pageSize: number = 10,
    page: number = 1,
    sort: string = 'id',
    dir: string = 'desc'
  ) {
    let query =
      `?organization=${organizationId}` +
      `&limit=${pageSize}` +
      `&offset=${page - 1}` +
      `&sort=${sort}` +
      `&dir=${dir}`;

    return this.authHttp
      .get(`${this.organizationsUrl}${this.departmentsUrl}${query}`)
      .pipe(map(response => response));
  }

  getAllDepartments(organizationId = 0) {
    let query = `?organization=${organizationId}`

    return this.authHttp
      .get(`${this.organizationsUrl}/all-depts${query}`)
      .pipe(map(response => response));
  }

  getDepartment(departmentId) {
    return this.authHttp
      .get(`${this.organizationsUrl}${this.departmentsUrl}/${departmentId}`)
      .pipe(map(response => response));
  }

  searchOrganizations(
    searchQuery: string,
    pageSize: number = 10,
    page: number = 1,
    sort: string = 'id',
    dir: string = 'desc'
  ) {
    const encodedUrlComponent = encodeURIComponent(searchQuery);
    let query =
      `?limit=${pageSize}` +
      `&offset=${page - 1}` +
      `&sort=${sort}` +
      `&dir=${dir}` +
      `&searchTerm=${encodedUrlComponent}`;

    return this.authHttp
      .get(`${this.organizationsUrl + query}`)
      .pipe(map(response => response));
  }

  searchDepartment(
    organizationId = 0,
    searchQuery: string,
    pageSize: number = 10,
    page: number = 1,
    sort: string = 'id',
    dir: string = 'desc'
  ) {
    const encodedUrlComponent = encodeURIComponent(searchQuery);
    let query =
      `?organization=${organizationId}` +
      `&limit=${pageSize}` +
      `&offset=${page - 1}` +
      `&sort=${sort}` +
      `&dir=${dir}` +
      `&searchTerm=${encodedUrlComponent}`;

    return this.authHttp
      .get(`${this.organizationsUrl + this.departmentsUrl + query}`)
      .pipe(map(response => response));
  }

  deleteOrganization(id) {
    return this.authHttp
      .delete(`${this.organizationsUrl}/${id}`)
      .pipe(map(response => response));
  }

  deleteDepartment(id) {
    return this.authHttp
      .delete(`${this.organizationsUrl}${this.departmentsUrl}/${id}`)
      .pipe(map(response => response));
  }

  updateDepartment(id, name) {
    return this.authHttp
      .put(`${this.organizationsUrl}${this.departmentsUrl}/${id}`, { name })
      .pipe(map(response => response));
  }

  addNewDepartment(departmentObj) {
    return this.authHttp
      .post(`${this.organizationsUrl}${this.departmentsUrl}`, {
        department: departmentObj
      })
      .pipe(map(response => response));
  }

  updateOrganization(id, organizationName) {
    return this.authHttp
      .put(`${this.organizationsUrl}/${id}`, {
        name: organizationName
      })
      .pipe(map(response => response));
  }

  addNewOrganization(organizationObj, departments = []) {
    return this.authHttp
      .post(`${this.organizationsUrl}`, {
        organization: organizationObj,
        departments
      })
      .pipe(map(response => response));
  }
}
