import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { Pagination } from '../models/pagination';
import { OsPolicyService } from '../create-os-policy/os-policy.service';
import { Subject, Subscription } from 'rxjs';
import { debounceTime, distinctUntilChanged, switchMap } from 'rxjs/operators';
import { PreserveSearch } from '../models/preserve-search';
import { GlobalMessageClass } from '../global-messages/global-messages';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { AccessPolicyService } from '../create-access-policy/access-policy.service';
import { DeviceOSEnum } from '../shared/emuns/client.enum';
import { ConfirmModalComponent } from '../shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';
import { OtpAuthenticationService } from '../otp-verification/otp-authentication.service';

@Component({
  selector: 'app-os-policy',
  templateUrl: './os-policy.component.html',
  styleUrls: ['./os-policy.component.css']
})
export class OsPolicyComponent implements OnInit {
  pagination = new Pagination();
  spaces = 0;
  showHelpText: boolean = false;
  $searchPolicy = new Subject();
  policies: any = {};
  searchKeyword;
  clientAllOs = {
    ubuntu: 'Ubuntu Desktop',
    ubuntuserver: 'Ubuntu Server',
    ubuntu18: 'Ubuntu Desktop',
    ubuntuserver18: 'Ubuntu Server',
    ubuntuserver20armv8: 'Ubuntu Server (ARMv8)',
    'windows6.1': 'Windows 7',
    'windows6.2': 'Windows 8',
    'windows6.3': 'Windows 8.1',
    'windows10.0': 'Windows 10',
    'windows11.0': 'Windows 11',
    'windowsserver6.1': 'Windows Server 2008 R2',
    'windowsserver6.2': 'Windows Server 2012',
    'windowsserver6.3': 'Windows Server 2012 R2',
    'windowsserver10.0': 'Windows Server 2016+',
    macintosh: 'macOS',
    centosserver7: 'CentOS 7 Server',
    centosserver8: 'CentOS 8 Server'
  };

  pageSize = [10, 25, 50, 100, 200];

  preserveSearch = new PreserveSearch();
  navigationToEdit: boolean = false;

  clientOffset = 0;
  selectedClientIds = [];
  currentClientSearch = '';
  clients: any = [];
  $clientSearch = new Subject();
  isClientSearching = false;
  allClientPolicies: boolean = false;

  loading: Boolean = true;

  sort = 'createdAt';
  sortDir = 'desc';

  public alert: GlobalMessageClass;
  isDarkMode: boolean;
  showOtpDialog: boolean = false;
  otptoken: string = '';
  option: string;
  private otpStatusSubscription: Subscription;
  constructor(
    private router: Router,
    private osService: OsPolicyService,
    private accessService: AccessPolicyService,
    private globalMessageService: GlobalMessageService,
    public dialog: MatDialog,
    private titleService: Title,
    private OtpAuthenticationService: OtpAuthenticationService
  ) {}

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.titleService.setTitle('Policy Management | Device Policy | OS Policy');
    let saveSearch = this.preserveSearch.getSearch('OSPolicySaveSearch');
    if (saveSearch) {
      this.pagination = saveSearch.pagination;
      this.searchKeyword = saveSearch.searchKeyword;
    }
    this.pagination.id = 'os';
    this.getOsPolicies();
    this.subscibeToSearch();
    this.getClients();
    this.subscribeClientSearch();
  }

  getClients() {
    this.accessService
      .getClients(this.clientOffset, this.currentClientSearch)
      .subscribe((data: any) => {
        if (this.spaces < 2 && data.length === 0) {
          this.spaces += 1;
          this.clients = [
            ...this.clients,
            { type: 'clientGroup', id: '', name: '', disabled: true }
          ];
        }
        this.clients = [...this.clients, ...data];
      });
  }

  fetchMoreClients() {
    this.clientOffset += 1;
    this.getClients();
  }

  clientFocus() {
    this.currentClientSearch = '';
    this.clients = [];
    this.getClients();
  }

  searchPolicyByClientName() {
    this.getOsPolicies();
  }

  enablePolicy(policy: any, value: any) {
    this.osService.enablePolicy(policy.id, value).subscribe(res => {
      policy.is_enabled = !policy.is_enabled;
    });
  }

  subscribeClientSearch() {
    this.$clientSearch
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((term: string) => {
          this.isClientSearching = true;
          this.spaces = 0;
          this.currentClientSearch = term;
          this.clientOffset = 0;
          return this.accessService.getClients(
            this.clientOffset,
            this.currentClientSearch
          );
        })
      )
      .subscribe(data => {
        this.isClientSearching = false;
        this.clients = data;
      });
  }

  subscibeToSearch() {
    this.$searchPolicy
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap(data => {
          this.loading = true;
          this.searchKeyword = data;
          this.pagination.currentPage = 1;
          return this.osService.getAllPolicies(
            this.pagination,
            this.searchKeyword,
            this.selectedClientIds
          );
        })
      )
      .subscribe({
        next: policies => {
          this.loading = false;
          const { data, recordsTotal } = policies as any;
          this.policies = policies;
          this.pagination.totalItems = recordsTotal;
        },
        error: error => {
          this.loading = false;
        }
      });
  }

  private getOsPolicies() {
    this.loading = true;
    this.osService
      .getAllPolicies(
        this.pagination,
        '',
        this.selectedClientIds,
        this.sort,
        this.sortDir,
        this.searchKeyword,
        this.allClientPolicies
      )
      .subscribe({
        next: (policies: any) => {
          this.loading = false;
          const { data, recordsTotal } = policies;
          this.policies = policies;
          this.pagination.totalItems = recordsTotal;
        },
        error: err => {
          this.loading = false;
        }
      });
  }

  clearfun() {
    this.searchKeyword = '';
    this.getOsPolicies();
  }

  deleteHandler(id) {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: `Are you sure you want to delete?`
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        let mfa = localStorage.getItem('adminMfaValue');
        if (mfa === 'true') {
          this.showOtpDialog = true;
          this.option = 'delete';
          this.otpStatusSubscription = this.OtpAuthenticationService.otpStatusCheckDel$.subscribe(
            (res: any) => {
              console.log('OTPST', res);
              if (res == true) {
                this.osService.deletePolicy(id).subscribe(res => {
                  if (res) {
                    this.alert = {
                      closed: false,
                      type: 'success',
                      message: 'Policy has been deleted successfully.'
                    };
                    this.globalMessageService.pushAlert(this.alert);
                  }
                  this.getOsPolicies();
                });

                this.showOtpDialog = false;
                this.OtpAuthenticationService.setOtpStatusValueDel(false);
              }
            }
          );
        } else {
          this.osService.deletePolicy(id).subscribe(res => {
            if (res) {
              this.alert = {
                closed: false,
                type: 'success',
                message: 'Policy has been deleted successfully.'
              };
              this.globalMessageService.pushAlert(this.alert);
            }
            this.getOsPolicies();
          });
        }
      }
    });
  }
  addDevicePolicy() {
    this.router.navigate(['/create-os-policy']);
  }

  editPolicy(id) {
    let mfa = localStorage.getItem('adminMfaValue');
    if (mfa === 'true') {
      this.showOtpDialog = true;
      this.option = 'edit';
      this.otpStatusSubscription = this.OtpAuthenticationService.otpStatusCheckEdit$.subscribe(
        (res: any) => {
          if (res == true) {
            const search = {
              pagination: this.pagination,
              searchKeyword: this.searchKeyword
            };
            this.preserveSearch.saveSearch('OSPolicySaveSearch', search);
            this.navigationToEdit = true;
            this.router.navigate([`/osPolicy/${id}`]);
          }
        }
      );
      this.OtpAuthenticationService.setOtpStatusValueEdit(false);
    } else {
      const search = {
        pagination: this.pagination,
        searchKeyword: this.searchKeyword
      };
      this.preserveSearch.saveSearch('OSPolicySaveSearch', search);
      this.navigationToEdit = true;
      this.router.navigate([`/osPolicy/${id}`]);
    }
  }

  ngOnDestroy(): void {
    if (!this.navigationToEdit) {
      this.preserveSearch.saveSearch('OSPolicySaveSearch', '');
    }

    if (this.otpStatusSubscription) {
      this.otpStatusSubscription.unsubscribe();
    }
  }

  pageChange(event) {
    this.pagination.currentPage = event;
    this.getOsPolicies();
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getOsPolicies();
  }

  allClientPoliciesToggle(check) {
    this.allClientPolicies = check;
    this.getOsPolicies();
  }

  sortRecords(term: string) {
    this.sortDir = this.sortDir == 'asc' ? 'desc' : 'asc';
    this.sort = term;

    this.getOsPolicies();
  }

  getOsName(os) {
    switch (os) {
      case DeviceOSEnum.Ubuntu18:
        return 'Ubuntu Desktop';
      case DeviceOSEnum.UbuntuServer18:
        return 'Ubuntu Server';
      case DeviceOSEnum.Macintosh:
        return 'Macintosh';
      case DeviceOSEnum.Windows:
        return 'Windows';
      case DeviceOSEnum.Ubuntu:
        return 'Ubuntu Desktop';
      case DeviceOSEnum.UbuntuServer:
        return 'Ubuntu Server';
      case DeviceOSEnum.WindowsServer:
        return 'Windows Server';
      case DeviceOSEnum.IOS:
        return 'iOS';
      case DeviceOSEnum.Android:
        return 'Android';
      case DeviceOSEnum.UbuntuServer20armv8:
        return 'Ubuntu Server (ARMv8)';
      case DeviceOSEnum.CentosServer:
        return 'CentOS Server';
      case DeviceOSEnum.Unknown:
        return 'Unknown';
      default:
        return os;
    }
  }

  cancelVerificationPassword(event: any) {
    this.showOtpDialog = false;
    this.OtpAuthenticationService.setOtpStatusValueEdit(false);
    this.OtpAuthenticationService.setOtpStatusValueDel(false);
  }

  submitVerificationPassword(event: any) {
    console.log('EVENR', event);
    this.otptoken = event;
    // this.otptoken = event
  }

  getClosePopup(event: any) {
    this.showOtpDialog = false;
  }
}
