import { Component, OnInit, ViewChild } from '@angular/core';
import { Router } from '@angular/router';
import { NgModel } from '@angular/forms';
import { Group } from '../models/group';
import { GroupService } from '../group/group.service';
import { PolicyService } from './policy.service';
import { Subject, Observable } from 'rxjs';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { Pagination } from '../models/pagination';
import { debounceTime, switchMap, tap } from 'rxjs/operators';
import { ConfirmModalComponent } from '../shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';
import { AccessPolicyService } from '../create-access-policy/access-policy.service';
import { RolesAndPermissionService } from '../roles-and-permission/roles-and-permission.service';

@Component({
  selector: 'app-policy',
  templateUrl: './policy.component.html',
  styleUrls: ['./policy.component.css']
})
export class PolicyComponent implements OnInit {
  showHelpText: boolean = false;

  @ViewChild('submitForm', { static: false }) submitForm: NgModel;
  saveButtonDisabled = false;
  editServiceGroup;
  currentEditingName;
  groupList = [];
  serviceList = [];
  searchTerm$ = new Subject();
  searching = false;
  groupId: number;
  addGroupName;
  isGroupName = false;
  loading: Boolean = true;

  searchKeyword: any = '';

  pagination: Pagination = new Pagination();

  pageSize = [10, 25, 50, 100, 200];

  listView: Boolean = true;

  selectedGroup: any = {};
  showConfirmation: Boolean = false;

  sort = 'serviceCount';
  sortDir = 'desc';

  selectAll: boolean = false;

  selectAllExcept: any = new Set();
  selectedGroups: any = new Set();
  isDarkMode : boolean;

  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;

  constructor(
    private router: Router,
    private groupService: GroupService,
    private policyService: PolicyService,
    private globalMessageService: GlobalMessageService,
    public dialog: MatDialog,
    private titleService: Title,
    private accessService: AccessPolicyService,
    private rolesPermissionService : RolesAndPermissionService

  ) {
    const serviceGroupView = localStorage.getItem('serviceGroupView');

    if (serviceGroupView) {
      this.listView = serviceGroupView == 'list' ? true : false;
    }
  }

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.getCRUDdata(23);
    this.getAllGroups();
    this.subscribeToSearch();
    this.titleService.setTitle('Infrastructure | Service Groups');
  }

  getCRUDdata(privId){
    this.rolesPermissionService.getCrudData(privId).subscribe((res: any) => {
      this.createPermission = res?.create_permission === 1 ? true : false
      this.readPermission = res?.read_permission === 1 ? true : false
      this.updatePermission = res?.update_permission === 1 ? true : false
      this.deletePermission = res?.delete_permission === 1 ? true : false
    });
  }

  private subscribeToSearch() {
    this.searchTerm$
      .pipe(
        tap(() => {
          this.searching = true;
        }),
        debounceTime(1000),
        switchMap((term: any) => {
          if (!term) {
            term = '__SERVICES__';
          }
          return this.policyService.searchService(this.groupId, term);
        })
      )
      .subscribe({
        next: (res: any) => {
          this.addIsMemberStatus(res);
          this.searching = false;
        },
        error: err => {
          this.serviceList = [];
          this.searching = false;
        }
      });
  }

  private getAllGroups() {
    this.policyService
      .getAllServiceGroups(
        this.pagination,
        this.searchKeyword,
        this.sort,
        this.sortDir
      )
      .subscribe({
        next: (groups: any) => {
          this.loading = false;
          this.groupList = groups.data;
          this.pagination.totalItems = groups.recordsTotal;
          this.changeCheckStatusOfAllGroups();
        },
        error: err => {
          this.loading = false;
        }
      });
  }

  clearfun() {
    this.searchKeyword = '';
    this.getAllGroups();
  }

  editHandler(serviceGroup) {
    this.router.navigate([`/create-service-group/${serviceGroup.id}`]);
    // this.currentEditingName = sericeGroup.name;
    // this.editServiceGroup = { ...sericeGroup };
  }
  addPolicyHandler() {
    this.router.navigate(['/create-service-group']);
    // this.isGroupName = !this.isGroupName;
    // this.addGroupName = '';
  }
  cancelHandler() {
    this.editServiceGroup = null;
    this.currentEditingName = '';
    this.addGroupName = '';
  }

  inputHandler() {
    this.saveButtonDisabled = this.submitForm.invalid;
  }

  saveHandler(group) {
    if (this.currentEditingName !== group.name) {
      // this.policyService.editGroup(group.id, this.currentEditingName).subscribe(
      //   response => {
      //     this.globalMessageService.pushAlert({
      //       type: 'success',
      //       message: response
      //     });
      //     group.name = this.currentEditingName;
      //     this.cancelHandler();
      //   },
      //   err => {
      //     // this.globalMessageService.pushAlert({
      //     //   type: 'danger',
      //     //   message: JSON.parse(err._body).message
      //     // });
      //   }
      // );
    } else {
      this.cancelHandler();
    }
  }

  searchGroup(searchName) {
    this.searchKeyword = searchName.trim();
    this.selectAll = false;
    this.selectAllExcept.clear();
    this.getAllGroups();
  }

  searchService(groupId, search = '__SERVICES__') {
    this.groupId = groupId;
    this.serviceList = [];
    this.searching = true;
    this.policyService.searchService(groupId, search).subscribe(data => {
      this.searching = false;
      this.addIsMemberStatus(data);
    });
  }

  addIsMemberStatus(services) {
    this.serviceList = services.map(service => {
      const groupServices = service.GroupServices;
      if (groupServices && groupServices.length !== 0) {
        return {
          ...service,
          is_member: 1
        };
      }
      return {
        ...service,
        is_member: 0
      };
    });
  }

  removeFromGroup(group, item) {
    this.policyService.removeService(group.id, item.id).subscribe(
      data => {
        group.serviceCount--;
        item.is_member = false;
        this.globalMessageService.pushAlert({
          message: `${item.Name} removed from ${group.name}`,
          type: 'success'
        });
      },
      () => {
        // this.globalMessageService.pushAlert({
        //   message: `Failed to remove ${item.Name} to ${group.name}`,
        //   type: 'danger'
        // });
      }
    );
  }

  deleteGroup() {
    const group = this.selectedGroup;
    this.accessService.verifyPolicies('', group.id, []).subscribe(res => {
      if(res){
        this.globalMessageService.pushAlert({
          closed: false,
          type: 'danger',
          message: 'Remove Service Group from Entitlement(s) first',
          sticky: false
        });
      }else{
        this.policyService.deleteGroup(group.id).subscribe(
          response => {
            this.getAllGroups();
            this.globalMessageService.pushAlert({
              message: response,
              type: 'success'
            });
            this.showConfirmation = false;
            this.selectedGroup = {};
          },
          err => {
            this.showConfirmation = false;
            this.selectedGroup = {};
            // this.globalMessageService.pushAlert({
            //   message: JSON.parse(err._body).message,
            //   type: 'danger'
            // });
          }
        );
      }
    })
  }

  addToGroup(group, item) {
    this.policyService.addService(group.id, item.id).subscribe(
      data => {
        group.serviceCount++;
        item.is_member = true;
        this.globalMessageService.pushAlert({
          message: `${item.Name} added to ${group.name}`,
          type: 'success'
        });
        this.addGroupName = '';
        this.currentEditingName = '';
      },
      () => {
        // this.globalMessageService.pushAlert({
        //   message: `Failed to add ${item.Name} to ${group.name}`,
        //   type: 'danger'
        // });
      }
    );
  }

  addNewGroupHandler() {
    // this.policyService.addNewGroup(this.addGroupName).subscribe(
    //   data => {
    //     const newGroup = Object.assign(
    //       {},
    //       { id: data.id, name: data.name, serviceCount: 0 }
    //     );
    //     this.isGroupName = false;
    //     this.groupList.unshift(newGroup);
    //     this.allGroups = this.groupList;
    //     this.addGroupName = '';
    //     this.currentEditingName = '';
    //   },
    //   error => {
    //     // this.globalMessageService.pushAlert({
    //     //   message: JSON.parse(error._body).message,
    //     //   type: 'danger'
    //     // });
    //   }
    // );
  }

  changeView(view) {
    localStorage.setItem('serviceGroupView', view);
    this.listView = view == 'list' ? true : false;
  }

  pageSizeChange() {
    this.getAllGroups();
  }

  pageChange(event) {
    this.pagination.currentPage = event;
    this.getAllGroups();
  }

  deleteConfirmation(group) {
    this.selectedGroup = group;
    // this.showConfirmation = true;
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: `Are you sure want to delete "${group.name}"`,
      },
    });

    dialogRef.afterClosed().subscribe((result) => {
      if (result){
        this.deleteGroup()
      }else{
        this.cancelConfirmation();
      }
    });
  }

  cancelConfirmation() {
    // this.showConfirmation = false;
    this.selectedGroup = {};
  }

  showGroupDetail(groupId) {
    this.router.navigate([`/serviceGroups/${groupId}`]);
  }

  handleDirectionChange(sort) {
    this.sort = sort;
    this.sortDir = this.sortDir == 'desc' ? 'asc' : 'desc';
    this.pagination.currentPage = 1;
    this.getAllGroups();
  }

  selectAllAction() {
    if (!this.selectAll) {
      this.selectAllExcept.clear();
      this.selectedGroups.clear();
    } else {
      this.selectedGroups.clear();
    }
    this.changeCheckStatusOfAllGroups();
  }

  private changeCheckStatusOfAllGroups() {
    this.groupList = this.groupList.map(group => {
      return {
        ...group,
        isChecked:
          (!this.selectAllExcept.has(group.id) && this.selectAll) ||
          (this.selectedGroups.has(group.id) && !this.selectAll)
      };
    });
  }

  changeCheckBox(group) {
    const { isChecked, id } = group;
    if (this.selectAll) {
      if (isChecked) {
        this.selectAllExcept.delete(id);
      } else {
        this.selectAllExcept.add(id);
      }
    } else {
      if (!isChecked) {
        this.selectedGroups.delete(id);
      } else {
        this.selectedGroups.add(id);
      }
    }
  }

  handleDeleteGroups() {
    const count = this.selectAll
      ? this.pagination.totalItems - this.selectAllExcept.size
      : this.selectedGroups.size;
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: `Are you sure want to delete ${count} group(s)`,
      },
    });

    dialogRef.afterClosed().subscribe((result) => {
      if (result){
        this.policyService
        .bulkDeleteGroup(
          this.selectedGroups,
          this.selectAll,
          this.selectAllExcept,
          this.searchKeyword
        )
        .subscribe(response => {
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'success',
            message: 'Service groups has been deleted successfully'
          });
          this.selectAll = false;
          this.selectAllExcept = new Set();
          this.selectedGroups = new Set();
          this.getAllGroups();
        });
      }
    });
  }
}
