import { Injectable } from '@angular/core';
import { ConfigService } from '../services/config.service';
import { HttpClient as AuthHttp, HttpHeaders } from '@angular/common/http';
import { Pagination } from '../models/pagination';
import { map } from 'rxjs/operators';

@Injectable()
export class PolicyService {
  groupsApiUrl;
  searchServiceUrl;
  addServiceUrl;
  deleteSericeUrl;
  httpOptions = {
    headers: new HttpHeaders({
      'Content-Type': 'application/json'
    })
  };
  constructor(
    private configService: ConfigService,
    private authHttp: AuthHttp
  ) {
    const apiBaseUrl = configService.getConfiguration().apiBaseUrl;
    this.groupsApiUrl = apiBaseUrl + '/serviceGroups';
    this.searchServiceUrl = this.groupsApiUrl + '/getAllServices';
    this.addServiceUrl = this.groupsApiUrl + '/addService';
    this.deleteSericeUrl = this.groupsApiUrl + '/deleteService';
  }

  getAllServiceGroups(
    pagination: Pagination,
    search = '',
    sort = 'serviceCount',
    sortDir = 'desc'
  ) {
    return this.authHttp
      .get(
        `${this.groupsApiUrl}?limit=${
          pagination.itemsPerPage
        }&offset=${(pagination.currentPage - 1) *
          pagination.itemsPerPage}&search=${search}&sort=${sort}&sortDir=${sortDir}`
      )
      .pipe(map(response => response));
  }

  getAvailableServiceGroups() {
    return this.authHttp
      .get(`${this.groupsApiUrl}/available`)
      .pipe(map(response => response));
  }

  addNewGroup(name, description, serviceIds = [], addAllServices = false) {
    return this.authHttp
      .post(`${this.groupsApiUrl}`, {
        name,
        description,
        serviceIds,
        addAllServices
      })
      .pipe(map(response => response));
  }

  searchService(groupId, searchTerm) {
    searchTerm = searchTerm === '__SERVICES__' ? searchTerm : btoa(searchTerm);
    return this.authHttp
      .get(`${this.searchServiceUrl}/${groupId}/${searchTerm}`)
      .pipe(map(response => response));
  }

  getGroupDetailById(groupId) {
    return this.authHttp
      .get(`${this.groupsApiUrl}/${groupId}`)
      .pipe(map(response => response));
  }

  addService(groupId, serviceId) {
    return this.authHttp.post(
      `${this.addServiceUrl}/${groupId}/${serviceId}`,
      {}
    );
  }
  removeService(groupId, serviceId) {
    return this.authHttp.delete(
      `${this.deleteSericeUrl}/${groupId}/${serviceId}`,
      {}
    );
  }
  editGroup(
    groupId,
    name,
    description,
    serviceIds = [],
    appendServices = false,
    addAllServices = false
  ): any {
    return this.authHttp
      .put(`${this.groupsApiUrl}/${groupId}`, {
        name,
        description,
        serviceIds,
        appendServices,
        addAllServices
      })
      .pipe(map(response => response));
  }
  deleteGroup(groupId): any {
    return this.authHttp
      .delete(`${this.groupsApiUrl}/${groupId}`)
      .pipe(map(response => response));
  }
  getGroupServicesWithPagination(
    groupId,
    offset = 0,
    limit = 10,
    sort = 'Name',
    dir = 'asc',
    searchKey = ''
  ): any {
    const query = `?limit=${limit}&offset=${offset}&sort=${sort}&dir=${dir}&searchKey=${searchKey}`;

    return this.authHttp
      .get(`${this.groupsApiUrl}/services/${groupId}${query}`)
      .pipe(map(response => response));
  }

  bulkDeleteGroup(
    groupIds = [],
    allSelected = false,
    excludeGroups = [],
    search = ''
  ): any {
    const query = `?search=${search}&allSelected=${allSelected}`;
    this.httpOptions['body'] = {
      groupIds: Array.from(groupIds),
      excludeGroups: Array.from(excludeGroups)
    };
    return this.authHttp
      .delete(`${this.groupsApiUrl}/bulk${query}`, this.httpOptions)
      .pipe(map(response => response));
  }
}
