import { Component } from '@angular/core';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';
import { Router } from '@angular/router';
import {
  Subject,
  debounceTime,
  distinctUntilChanged,
  switchMap,
  tap
} from 'rxjs';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { Pagination } from '../../models/pagination';
import { ConfirmModalComponent } from '../../shared/confirm-modal.component';
import { RolesAndPermissionService } from '../roles-and-permission.service';

@Component({
  selector: 'app-view-roles-and-permission',
  templateUrl: './view-roles-and-permission.component.html',
  styleUrls: ['./view-roles-and-permission.component.css']
})
export class ViewRolesAndPermissionComponent {
  templates = [];

  pagination: Pagination = new Pagination();
  pageSize = [10, 25, 50, 100, 200];

  searchKeyword;
  $searchUsers = new Subject();
  showKeys: Boolean = false;
  loading: Boolean = false;

  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;
  tempId: boolean;

  count: number;

  sort: string;
  sortDir: string;

  constructor(
    private globalMessageService: GlobalMessageService,
    private rolesPermissionService: RolesAndPermissionService,
    private router: Router,
    public dialog: MatDialog,
    private titleService: Title
  ) {}

  ngOnInit() {
    this.titleService.setTitle('Roles And Permissions');    
    this.getPage(this.pagination.currentPage);
    this.getCRUDdata(36);
    this.subscibeToSearch();
  }

  ngOnDestroy() {}

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getPage(this.pagination.currentPage);
  }

  subscibeToSearch() {
    this.$searchUsers
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap(data => {
          this.loading = true;
          this.searchKeyword = data;
          this.pagination.currentPage = 1;
          return this.rolesPermissionService.getAssignedCreatedTemplates(
            this.pagination.currentPage,
            this.pagination.itemsPerPage,
            this.sort,
            this.sortDir,
            this.searchKeyword
          );
        })
      )
      .subscribe({
        next: (templates: any) => {
          this.loading = false;
          const recordsTotal = templates.recordsTotal;
          this.templates = templates.paginatedData;
          this.pagination.totalItems = recordsTotal;
        },
        error: err => {
          this.loading = false;
        }
      });
  }

  sortRecord(term: string) {
    this.sortDir = this.sortDir == 'asc' ? 'desc' : 'asc';
    this.sort = 'name';
    this.getPage(this.pagination.currentPage);
  }

  onDelete(template) {
    this.count = 0;
    this.rolesPermissionService
      .getTemplateUsersCount(template?.temp_id)
      .subscribe((res: any) => {
        if (res) {
          this.count = res?.count
          const dialogRef = this.dialog.open(ConfirmModalComponent, {
            data: {
              title: 'Confirmation',
              message: this.count > 0 ? `This template is currently assigned to ${this.count} users. Removing it will not affect the functionality for existing users, but new users utilizing this template will not be created.` : 'Are you sure you want to delete this template?'
            }
          });
      
          dialogRef.afterClosed().subscribe(result => {
            if (result) {
              this.rolesPermissionService
                .deleteTemplate(template?.temp_id)
                .subscribe(res => {
                  if (res) {
                    this.globalMessageService.pushAlert({
                      closed: false,
                      type: 'success',
                      message: 'Template deleted successfully.'
                    });
                    this.getAllTemplates();
                  }
                });
            }
          });
        }
      })
  }

  getPage(page: number) {
    this.loading = true;
    let prevPage = this.pagination.currentPage;
    this.pagination.currentPage = page;
    this.rolesPermissionService
      .getAssignedCreatedTemplates(
        this.pagination.currentPage,
        this.pagination.itemsPerPage,
        this.sort,
        this.sortDir,
        this.searchKeyword
      )
      .pipe(
        tap((res: any) => {
          this.loading = false;
          this.templates = res.paginatedData;          
          this.pagination.totalItems = res.recordsTotal;
          this.pagination.currentPage = page;
          if (prevPage != this.pagination.currentPage) {
            let scrollToTop = window.setInterval(() => {
              let pos = window.pageYOffset;
              if (pos > 0) {
                window.scrollTo(0, pos - 20); // how far to scroll on each step
              } else {
                window.clearInterval(scrollToTop);
              }
            }, 16);
          }
        })
      )
      .subscribe({
        next: next => {},
        error: err => {
          this.loading = false;
        }
      });
  }

  clearfun() {
    this.searchKeyword = '';
    this.getPage(this.pagination.currentPage);
  }

  getCRUDdata(privId) {
    this.rolesPermissionService.getCrudData(privId).subscribe((res: any) => {
      this.tempId = res?.temp_id === 1 || res?.temp_id === 2 ? true : false;
      this.createPermission = res?.create_permission === 1 ? true : false;
      this.readPermission = res?.read_permission === 1 ? true : false;
      this.updatePermission = res?.update_permission === 1 ? true : false;
      this.deletePermission = res?.delete_permission === 1 ? true : false;
    });
  }

  handleAddTemplate() {
    this.router.navigate(['/create_roles_and_permissions']);
  }

  handleAdditionalPermissions() {
    this.router.navigate(['/view_additional_permissions']);
  }

  onEdit(template) {
    this.router.navigate(['/edit_roles_and_permissions', template?.temp_id]);
  }

  getAllTemplates() {
    this.rolesPermissionService
      .getAssignedCreatedTemplates()
      .subscribe((res: any) => {
        this.templates = res.paginatedData;
      });
  }
}
