import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { Pagination } from '../models/pagination';
import { RoutingPolicyService } from './routing-policy.service';
import { PreserveSearch } from '../models/preserve-search';
import { Subject, Subscription } from 'rxjs';
import { debounceTime, distinctUntilChanged, switchMap } from 'rxjs/operators';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { GlobalMessageClass } from '../global-messages/global-messages';
import { AccessPolicyService } from '../create-access-policy/access-policy.service';
import { ConfirmModalComponent } from '../shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';
import { OtpAuthenticationService } from '../otp-verification/otp-authentication.service';

@Component({
  selector: 'app-routing-policy',
  templateUrl: './routing-policy.component.html',
  styleUrls: ['./routing-policy.component.css']
})
export class RoutingPolicyComponent implements OnInit {
  showHelpText: boolean = false;
  pagination = new Pagination();
  $searchPolicy = new Subject();
  policies: any = {};
  searchKeyword;
  private Search: PreserveSearch = new PreserveSearch();
  isNavigatingToSeach: boolean = false;
  loading: Boolean = true;

  pageSize = [10, 25, 50, 100, 200];

  preserveSearch = new PreserveSearch();
  navigationToEdit: boolean = false;

  gateways: any = [];
  $gatewaySearch = new Subject();
  isGatewaySearching = false;
  gatewayOffset = 0;
  selectedGatwayIds = [];
  currentGatewaySearch = '';

  public alert: GlobalMessageClass;
  isDarkMode: boolean;
  showOtpDialog: boolean = false;
  otptoken: string = '';
  option:string;
  private otpStatusSubscription: Subscription;

  constructor(
    private router: Router,
    private routingService: RoutingPolicyService,
    private accessService: AccessPolicyService,
    private globalMessageService: GlobalMessageService,
    public dialog: MatDialog,
    private titleService: Title,
    private OtpAuthenticationService: OtpAuthenticationService
  ) {}

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.titleService.setTitle('Policy Management | Routing Policy');
    let saveSearch = this.preserveSearch.getSearch('routingPolicySaveSearch');
    if (saveSearch) {
      this.pagination = saveSearch.pagination;
      this.searchKeyword = saveSearch.searchKeyword;
    }
    this.subscibeToSearch();
    this.pagination.id = 'routing';
    let routingPolicySearch = this.Search.getSearch('routingPolicySearch');
    if (routingPolicySearch) {
      this.pagination.currentPage = routingPolicySearch.currentPage;
      this.searchKeyword = routingPolicySearch.searchKey;
      this.$searchPolicy.next(this.searchKeyword);
    } else {
      this.getRoutingPolicies();
    }
    this.getGateways();
    this.subscribeGatewaySearch();
  }

  getGateways() {
    this.routingService
      .getGateways(this.gatewayOffset, this.currentGatewaySearch)
      .subscribe((data: any) => {
        this.gateways = [...this.gateways, ...data];
      });
  }

  fetchMoreGateways() {
    this.gatewayOffset += 1;
    this.getGateways();
  }

  gatewayFocus() {
    this.currentGatewaySearch = '';
    this.gateways = [];
    this.getGateways();
  }

  searchPolicyByClientName() {
    this.getRoutingPolicies();
  }

  subscribeGatewaySearch() {
    this.$gatewaySearch
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((term: string) => {
          this.isGatewaySearching = true;
          this.currentGatewaySearch = term;
          this.gatewayOffset = 0;
          return this.routingService.getGateways(
            this.gatewayOffset,
            this.currentGatewaySearch
          );
        })
      )
      .subscribe(data => {
        this.isGatewaySearching = false;
        this.gateways = data;
      });
  }

  subscibeToSearch() {
    this.$searchPolicy
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap(data => {
          this.loading = true;
          this.searchKeyword = data;
          this.pagination.currentPage = 1;
          return this.routingService.getAllPolicies(
            this.pagination,
            this.searchKeyword,
            this.selectedGatwayIds
          );
        })
      )
      .subscribe(policies => {
        const { data, recordsTotal } = policies as any;
        this.policies = policies;
        this.pagination.totalItems = recordsTotal;
        this.loading = false;
      });
  }

  private getRoutingPolicies() {
    this.loading = true;
    this.routingService
      .getAllPolicies(
        this.pagination,
        this.searchKeyword,
        this.selectedGatwayIds
      )
      .subscribe(policies => {
        this.loading = false;
        const { data, recordsTotal } = policies;
        this.policies = policies || {};
        this.pagination.totalItems = recordsTotal || 0;
      });
  }

  clearfun() {
    this.searchKeyword = '';
    this.getRoutingPolicies();
  }

  deleteHandler(id) {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: 'Are you sure you want to delete?'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        let mfa = localStorage.getItem('adminMfaValue');
        if (mfa === 'true') {
          this.showOtpDialog = true;
          this.option = 'delete'
          this.otpStatusSubscription = this.OtpAuthenticationService.otpStatusCheckDel$.subscribe(
            (res: any) => {
              if (res == true) {
                this.routingService.deletePolicy(id).subscribe(res => {
                  this.alert = {
                    closed: false,
                    type: 'success',
                    message: 'Policy has been deleted successfully.'
                  };
                  this.globalMessageService.pushAlert(this.alert);
                  this.getRoutingPolicies();
                });
                this.showOtpDialog = false;
                this.OtpAuthenticationService.setOtpStatusValueDel(false);
              }
            }
            );
        } else {
          this.routingService.deletePolicy(id).subscribe(res => {
            this.alert = {
              closed: false,
              type: 'success',
              message: 'Policy has been deleted successfully.'
            };
            this.globalMessageService.pushAlert(this.alert);
            this.getRoutingPolicies();
          });
        }
      }
    });
  }

  enablePolicy(policy: any, value: any) {
    this.routingService.enablePolicy(policy.id, value).subscribe(res => {
      policy.is_enabled = !policy.is_enabled;
    });
  }

  addRoutingPolicy() {
    this.router.navigate(['/routing-policy/create']);
  }

  editPolicy(id) {
    let mfa = localStorage.getItem('adminMfaValue');
    if (mfa === 'true') {
      this.showOtpDialog = true;
      this.option = 'edit'
      this.otpStatusSubscription = this.OtpAuthenticationService.otpStatusCheckEdit$.subscribe((res: any) => {
        if (res == true) {
          const search = {
            pagination: this.pagination,
            searchKeyword: this.searchKeyword
          };
          this.preserveSearch.saveSearch('routingPolicySaveSearch', search);
          this.navigationToEdit = true;
          this.router.navigate([`/routing-policy/edit/${id}`]);
          
        }
      });
      this.OtpAuthenticationService.setOtpStatusValueEdit(false)
    } else {
      const search = {
        pagination: this.pagination,
        searchKeyword: this.searchKeyword
      };
      this.preserveSearch.saveSearch('routingPolicySaveSearch', search);
      this.navigationToEdit = true;
      this.router.navigate([`/routing-policy/edit/${id}`]);
    }
  }

  ngOnDestroy(): void {
    if (!this.navigationToEdit) {
      this.preserveSearch.saveSearch('routingPolicySaveSearch', '');
    }

    if (this.otpStatusSubscription) {
      this.otpStatusSubscription.unsubscribe();
    }
  }

  pageChange(event) {
    this.pagination.currentPage = event;
    this.getRoutingPolicies();
  }

  // ngOnDestroy(): void {
  //   if (!this.isNavigatingToSeach) {
  //     this.Search.saveSearch('routingPolicySearch', null);
  //   }
  // }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getRoutingPolicies();
  }

  cancelVerificationPassword(event: any) {
    this.showOtpDialog = false;
    this.OtpAuthenticationService.setOtpStatusValueEdit(false);
    this.OtpAuthenticationService.setOtpStatusValueDel(false);
  }

  submitVerificationPassword(event: any) {
    this.otptoken = event;
    // this.otptoken = event
  }

  getClosePopup(event:any){
    this.showOtpDialog = false;
  }
}
