import { Component, OnInit, ViewChild, ElementRef } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { Subject } from 'rxjs';
import { debounceTime, distinctUntilChanged, switchMap } from 'rxjs/operators';
import * as psl from 'psl';
import { SaasBridgeService } from '../saas-bridge.service';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-create-saas-bridge',
  templateUrl: './create-saas-bridge.component.html',
  styleUrls: ['./create-saas-bridge.component.css']
})
export class CreateSaasBridgeComponent implements OnInit {
  title: string = 'Add';

  tab: string = '1';

  @ViewChild('clientGroups', { static: false }) clientDropdown: ElementRef;

  clients: any = [];
  selectedClientIds;
  clientOffset = 0;
  currentClientSearch = '';
  $clientSearch = new Subject();
  isClientSearching = false;
  selectedDeviceIds = [];
  isEditing = false;
  currentEditingId;

  loading: Boolean = false;

  saasBridge = {
    name: '',
    clientId: 0,
    serviceId: 0,
    mappedDomain: '',
    authAgentDomain: '',
    authService: '',
    authServiceClientId: '',
    authServiceClientSecret: '',
    authServiceUrl: '',
    authServiceTokenUrl: '',
    authServiceUserInfoUrl: ''
  };

  authServices = [
    {
      display: 'Aws Congnito',
      value: 'aws cognito'
    },
    {
      display: 'OKTA',
      value: 'okta'
    },
    {
      display: 'KEYCLOAK',
      value: 'keycloak'
    }
  ];

  clientServices = [];

  editBridgeId: Number = 0;
  disableSaveButton = false;

  constructor(
    private globalMessageService: GlobalMessageService,
    private activatedRoute: ActivatedRoute,
    private saasBridgeService: SaasBridgeService,
    private router: Router,
    private titleService: Title
  ) {
    this.activatedRoute.params.subscribe(data => {
      if (data.id) {
        this.title = 'Edit';
        this.editBridgeId = data.id;
      } else {
        this.title = 'Add';
        this.getClients(this.currentClientSearch);
      }
      this.titleService.setTitle('Infrastructure | SaaS Bridges | ' + this.title + ' SaaS Bridge');
    });
  }

  ngOnInit() {
    this.disableSaveButton = false;
    if (this.editBridgeId) {
      this.getBridge(this.editBridgeId);
    }
    this.subscribeClientSearch();
  }

  continue() {
    this.tab = '2';
  }

  back() {
    this.tab = '1';
  }

  getBridge(bridgeId) {
    this.loading = true;
    this.saasBridgeService.getBridge(bridgeId).subscribe({
      next: (bridge: any) => {
        this.saasBridge = bridge;
        this.getClients(this.saasBridge.clientId);
        this.selectedClientIds = this.saasBridge.clientId;
        this.getClientServices(this.saasBridge.clientId);

        this.loading = false;
      },
      error: err => {
        this.loading = false;
      }
    });
  }

  fetchMoreClients() {
    this.clientOffset += 1;
    this.getClients(this.currentClientSearch);
  }

  clientChangeHandler(event) {
    if (event) {
      this.getClientServices(event.id);
    }
  }

  getClients(name) {
    this.loading = true;
    this.saasBridgeService.getClients(name, this.clientOffset).subscribe({
      next: (res: any) => {
        this.clients = [...this.clients, ...res];
        this.loading = false;
      },
      error: err => {
        this.loading = false;
      }
    });
  }

  getClientServices(clientId) {
    this.loading = true;
    this.saasBridgeService.getClientService(clientId).subscribe({
      next: (res: any) => {
        this.clientServices = res;
        this.loading = false;
      },
      error: err => {
        this.loading = false;
      }
    });
  }

  private subscribeClientSearch() {
    this.$clientSearch
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((term: string) => {
          this.isClientSearching = true;
          this.currentClientSearch = term;
          this.clientOffset = 0;
          return this.saasBridgeService.getClients(
            this.currentClientSearch,
            this.clientOffset
          );
        })
      )
      .subscribe(data => {
        this.isClientSearching = false;
        this.clients = data;
      });
  }

  saveBridge(event) {
    this.loading = true;
    this.disableSaveButton = true;
    if (this.title == 'Edit') {
      event.stopPropagation();
      this.saasBridge.clientId = this.selectedClientIds;
      this.saasBridgeService
        .updateBridge(this.saasBridge, this.editBridgeId)
        .subscribe({
          next: res => {
            this.showAlert('success', 'SaaS Bridge Updated', false);
            this.loading = false;
            this.router.navigate(['/saas-bridge']);
          },
          error: err => {
            this.disableSaveButton = false;
            this.loading = false;
          }
        });
    } else {
      event.stopPropagation();
      this.saasBridge.clientId = this.selectedClientIds;
      this.saasBridgeService.addBridge(this.saasBridge).subscribe({
        next: res => {
          this.showAlert('success', 'SaaS Bridge Created', false);
          this.loading = false;
          this.router.navigate(['/saas-bridge']);
        },
        error: err => {
          this.disableSaveButton = false;
          this.loading = false;
        }
      });
    }
  }

  onChangeDomain(domain) {
    this.saasBridge.authAgentDomain = `login.${this.getDomainName(domain)}`;
  }

  private showAlert(type: string, message: string, sticky: boolean) {
    const alert = {
      closed: false,
      type: type,
      message: message,
      sticky: sticky
    };
    this.globalMessageService.pushAlert(alert);
  }

  getDomainName(url) {
    const _url = url.replace(/(^\w+:|^)\/\//g, '');
    const parsed = psl.parse(_url);
    if (!parsed.domain) {
      return '';
    }
    return parsed.domain;
  }

  cancelHandler() {
    this.router.navigate(['/saas-bridge']);
  }
}
