import { Injectable } from '@angular/core';
import { BehaviorSubject, Observable, of } from 'rxjs';
import { TrustedHost } from '../models/trusted-host';
import { throwError } from 'rxjs';
import { HostLimitEnum } from '../shared/emuns/global.enum';

@Injectable()
export class LocalTrustedHostsService {
  private _localModeHosts$ = new BehaviorSubject([]);
  public hosts$ = this._localModeHosts$.asObservable();

  private HOST_LIMIT = HostLimitEnum.Limit;

  getPage(page: number = 1, pageSize: number = 10): Observable<any> {
    const localModeHosts = this._localModeHosts$.value;
    const pageStart = (page - 1) * pageSize;
    const pageEnd = pageStart + pageSize;

    return of({
      status: true,
      message: 'Data Fetched',
      data: localModeHosts.slice(pageStart, pageEnd),
      total: localModeHosts.length
    });
  }

  setInitialalue(hosts) {
    this._localModeHosts$.next(hosts);
  }

  hostExists(newHost: TrustedHost): boolean {
    const localModeHosts = this._localModeHosts$.value;

    return (
      localModeHosts.filter(
        host =>
          host.address == newHost.address &&
          host.port == newHost.port &&
          host.protocol == newHost.protocol &&
          host.direction == newHost.direction &&
          host.id !== newHost.id
      ).length !== 0
    );
  }

  addHost(host: TrustedHost): Observable<any> {
    const localModeHosts = this._localModeHosts$.value;

    if (localModeHosts.length >= this.HOST_LIMIT) {
      return this.simulateHttpError({
        status: false,
        message: `You have reached the limit of ${this.HOST_LIMIT} host records`
      });
    } else if (this.hostExists(host)) {
      return this.simulateHttpError({
        status: false,
        message: 'Host with the given rules already exists'
      });
    } else {
      const newHost = {
        ...host,
        id:
          '_' +
          Math.random()
            .toString(36)
            .substr(2, 9)
      };

      this._localModeHosts$.next([newHost, ...this._localModeHosts$.value]);

      return of({
        status: true,
        message: 'Trusted Host record added successfully.'
      });
    }
  }

  editHost(id: number | string, editedHost: TrustedHost): Observable<any> {
    const localModeHosts = this._localModeHosts$.value;

    if (this.hostExists(editedHost)) {
      return this.simulateHttpError({
        status: false,
        message: 'Host with the given rules already exists'
      });
    } else {
      this._localModeHosts$.next(
        localModeHosts.map(host => (host.id === id ? { ...editedHost } : host))
      );

      return of({
        status: true,
        message: 'Trusted Host record updated successfully.'
      });
    }
  }

  deleteHost(id: number | string): Observable<any> {
    this._localModeHosts$.next(
      this._localModeHosts$.value.filter(host => host.id !== id)
    );

    return of({
      status: true,
      message: 'Trusted Host record deleted successfully.'
    });
  }

  clearHosts() {
    this._localModeHosts$.next([]);
  }

  simulateHttpError(err) {
    return throwError({
      _body: JSON.stringify(err)
    });
  }
}
