import {
  Component,
  OnInit,
  Input,
  ViewChild,
  Output,
  EventEmitter
} from '@angular/core';
import { GlobalMessageClass } from '../global-messages/global-messages';
import { GlobalMessageService } from '../global-messages/global-messages.service';

import { TrustedHost } from '../models/trusted-host';
import { Pagination } from '../models/pagination';
import { TrustedHostsService } from './trusted-hosts.service';
import { Router } from '@angular/router';
import { AuthenticationService } from '../login/authentication.service';
import { HttpErrorResponse } from '@angular/common/http';
import { NgModel } from '@angular/forms';
import { LocalTrustedHostsService } from './local-trusted-hosts.service';
import { Observable, of, Subject, Subscription } from 'rxjs';
import { debounceTime, distinctUntilChanged, switchMap } from 'rxjs/operators';
import { ErrorCodeEnum } from '../shared/emuns/error.enum';
import { LicenseErrorEnum } from '../shared/emuns/license.enum';
import { ContextHelpEnum } from '../shared/emuns/global.enum';
import { ClientService } from '../client/client.service';
import { ConfirmModalComponent } from '../shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { OtpAuthenticationService } from '../otp-verification/otp-authentication.service';
// import { start } from 'repl';

@Component({
  selector: 'app-trusted-hosts',
  templateUrl: './trusted-hosts.component.html',
  styleUrls: ['./trusted-hosts.component.css']
})
export class TrustedHostsComponent implements OnInit {
  @Input('parentId') parentId: number;
  @Input('parentType') parentType: string;
  @Input('isClone') isClone: boolean;
  @Input('disableAddHostButton') disableAddHostButton: boolean;

  @Output() localModeHostsChanged = new EventEmitter();

  @ViewChild('editAddress', { static: false }) editAddress: NgModel;
  @ViewChild('editLabel', { static: false }) editLabel: NgModel;
  @ViewChild('editPort', { static: false }) editPort: NgModel;
  @ViewChild('startPort', { static: false }) startPort: NgModel;
  @ViewChild('endPort', { static: false }) endPort: NgModel;

  localMode: boolean = false;
  showHelpText: boolean = false;
  helpTextContent: string = '';

  hosts: any = {};

  localModeHostsSubscription;
  clientIdSubscription;

  public alert: GlobalMessageClass;

  // Form related state
  hostForm: TrustedHost = new TrustedHost();
  hostEditForm: TrustedHost;
  isLoading: boolean = true;
  addFormShown: boolean = false;

  error: string;
  success: string;
  editError: string;

  recentlyEdited: Array<number | string> = [];
  deleting: Array<number | string> = [];

  editButtonDisabled: boolean = false;

  contextualHelp: Array<any> = ['', ''];
  showContext: boolean = false;

  pagination: Pagination = new Pagination();

  protocols = [
    { value: 'tcp', display: 'TCP' },
    { value: 'udp', display: 'UDP' },
    { value: 'icmp', display: 'ICMP' },
  ];

  directions = [
    { value: 1, display: 'Inbound' },
    { value: 0, display: 'Outbound' }
  ];

  service: any;
  searchTerm$ = new Subject();
  hostsSearchArray: any = [];
  totalTrustedHosts: any;
  search = false;
  searchTerm: String = null;
  ipRegex: RegExp = /^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/;
  subnetRegex: RegExp = /^(((255\.){3}(255|254|252|248|240|224|192|128|0+))|((255\.){2}(255|254|252|248|240|224|192|128|0+)\.0)|((255\.)(255|254|252|248|240|224|192|128|0+)(\.0+){2})|((255|254|252|248|240|224|192|128|0+)(\.0+){3}))$/;
  combineRegex = '';
  isPortRangeChecked = false;

  pageSize = [10, 25, 50, 100, 200];

  sort: string;
  sortDir: string;
  isDarkMode: boolean;

  showOtpDialog: boolean = false;
  otptoken: string = '';
  option: string;
  disablePorts : boolean = false;
  disablePortsEditing : boolean = false;
  private otpStatusSubscription: Subscription;

  constructor(
    private globalMessageService: GlobalMessageService,
    private trustedHostsService: TrustedHostsService,
    private localTrustedHostsService: LocalTrustedHostsService,
    private authenticationService: AuthenticationService,
    private router: Router,
    private clientService: ClientService,
    public dialog: MatDialog,
    private OtpAuthenticationService: OtpAuthenticationService
  ) {}

  ngOnChanges() {
    if (this.parentType === 'Client') {
      this.clientService.clientType$.subscribe(clientType => {
        if (clientType === 'server') {
          this.addTextHelp('Applicable to clients with sniffer or darkening');
        } else if (clientType === 'client') {
          this.addTextHelp('Applicable when darkening is enabled');
        }
      });
    } else {
      this.addTextHelp('Applicable to clients with sniffer or darkening');
    }
  }

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.switchMode(this.parentId);
    this.searchTerm$
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((search: { searchTerm: any; typing: boolean }) => {
          this.isLoading = true;
          if (search.typing) {
            this.pagination.currentPage = 1;
          }
          if (search.searchTerm) {
            this.search = true;
            return this.service.searchHost(
              search.searchTerm,
              this.parentType,
              this.parentId,
              this.pagination.itemsPerPage,
              this.pagination.currentPage,
              this.sort,
              this.sortDir
            );
          }
          this.search = false;
          this.pagination.totalItems = this.totalTrustedHosts;
          this.pagination.currentPage = 1;
          this.hosts.data = this.hostsSearchArray;
          console.log('this.hosts.data :>> ', this.hosts.data);
          this.hosts.data.forEach(host => {
            if(host.protocol === 'icmp'){
              this.disablePorts = true;
            }else{
              this.disablePorts = false;
            }
          })
          return of({
            data: this.hosts.data,
            recordsTotal: this.totalTrustedHosts
          });
        })
      )
      .subscribe(
        (response: any) => {
          this.isLoading = false;
          this.hosts = response;
          console.log('this.hosts :>> ', response.data);
          this.mapHosts(response.data || []);
          this.pagination.totalItems = response.recordsTotal;
        },
        error => {
          this.isLoading = false;
        }
      );
    this.hostForm.direction = this.directions[0].value;
    this.hostForm.protocol = this.protocols[0].value;
  }

  onProtocolChange(protocol:string){
    this.disablePorts = protocol === 'icmp';
    if(this.disablePorts){
      this.hostForm.port = null
    }
    console.log('this.disablePorts :>> ', this.disablePorts);
  }

  validateHostForm() {
    const protocol = this.hostEditForm.protocol;
    const isPortInRange = this.hostEditForm.isPortInRange;
  
    // Check for protocol 'icmp' (no port validation required)
    if (protocol === 'icmp') {
      this.editButtonDisabled = false;
      return;
    }
  
    // For 'tcp' or 'udp', ensure port or port range is provided
    if (protocol === 'tcp' || protocol === 'udp') {
      if (isPortInRange) {
        // If in range mode, both startPort and endPort must be filled
        const validRange = this.hostEditForm.startPort && this.hostEditForm.endPort;
        this.editButtonDisabled = !validRange;
      } else {
        // If not in range mode, the single port must be filled
        this.editButtonDisabled = !this.hostEditForm.port;
      }
      return;
    }
  
    // Default case: if none of the above matches, disable the button
    this.editButtonDisabled = true;
  }

  onProtocolChangeTemp(protocol:string){
   console.log('protocol :>> ', protocol);
   this.validateHostForm(); 
   if(protocol === 'icmp'){
    this.disablePortsEditing = true;
    this.hostEditForm.port = null;
   }else{
    this.disablePortsEditing = false;
   }
  }

  portRangeSelected() {
    this.isPortRangeChecked = !this.isPortRangeChecked;
    if (!this.isPortRangeChecked) {
      this.hostForm.port = this.hostForm.startPort;
    } else {
      this.hostForm.startPort = this.hostForm.port;
    }
  }

  startPortChanged() {
    const { startPort, endPort } = this.hostForm;
    if (!endPort) {
      this.hostForm.port = startPort;
    } else {
      if (startPort) {
        this.hostForm.port = startPort + '-' + endPort;
      } else {
        this.hostForm.port = endPort;
      }
    }
  }

  endPortChanged() {
    const { startPort, endPort } = this.hostForm;
    if (startPort) {
      if (endPort) {
        this.hostForm.port = startPort + '-' + endPort;
      } else {
        this.hostForm.port = startPort;
      }
    } else {
      this.hostForm.port = endPort;
    }
  }

  hostEditFormStartPort() {
    this.editInputChanged();
  }

  hostEditFormEndPort() {
    this.editInputChanged();
  }

  editPortRangeSelected(host) {
    console.log('host in edit PORT :>> ', host);
    host.protocol = this.hostForm.protocol
    if (host.protocol === 'icmp') {
      // If the protocol is 'icmp', reset ports or handle it according to your needs
      host.isPortInRange = false;
      host.startPort = null;
      host.endPort = null;
      host.port = null;
      return; // Skip the rest of the logic since it's not applicable for 'icmp'
    }
    host.isPortInRange = !host.isPortInRange;
    const { isPortInRange } = host;
    if (!isPortInRange) {
      host.port = host.startPort;
    } else {
      host.startPort = host.port;
    }
    this.hostEditForm = host;
  }

  mapHosts(data) {
    console.log('data :>> ', data);
    this.hosts.data = data.map(host => {
      if (host.port && host.port.includes('-') && host.protocol !== 'icmp') {
        return {
          ...host,
          isPortInRange: true,
          startPort: host.port.split('-')[0],
          endPort: host.port.split('-')[1],
          previousPort: host.port
        };
      }
      return {
        ...host,
        isPortInRange: false,
        previousPort: host.port
      };
    });
  }

  ngOnDestroy() {
    if (this.localMode) {
      this.service.clearHosts();
    }

    if (this.otpStatusSubscription) {
      this.otpStatusSubscription.unsubscribe();
    }

    this.clearSubscriptions();
  }

  localChangeService() {
    this.localMode = true;

    this.service = this.localTrustedHostsService;

    this.localModeHostsSubscription = this.service.hosts$.subscribe(
      localModeHosts => {
        this.localModeHostsChanged.emit(
          // Remove temporary id from the items
          localModeHosts.map(({ id, ...host }) => host)
        );
      }
    );
  }

  switchMode(parentId) {
    if (typeof parentId === 'undefined') {
      this.localChangeService();
    } else {
      this.localMode = false;

      this.service = this.trustedHostsService;

      this.clearSubscriptions();
    }

    this.getPage(this.pagination.currentPage);
  }

  clearSubscriptions() {
    if (this.localModeHostsSubscription) {
      this.localModeHostsSubscription.unsubscribe();
    }

    if (this.clientIdSubscription) {
      this.clientIdSubscription.unsubscribe();
    }
  }

  closeContextualHelp(): void {
    this.showContext = false;
  }

  chagePage(page: number) {
    if (!this.search) {
      this.getPage(page);
    } else {
      this.pagination.currentPage = page;
      this.searchTerm$.next({ searchTerm: this.searchTerm, typing: false });
    }
  }

  getPage(page: number) {
    this.isLoading = true;
    this.pagination.currentPage = page;
    if (this.search == false) {
      this.service
        .getPage(
          this.pagination.currentPage,
          this.pagination.itemsPerPage,
          this.parentId,
          this.parentType,
          this.sort,
          this.sortDir
        )
        .subscribe({
          next: response => {
            this.hosts = response;
            console.log('this.hosts IN GET PAGE :>> ', this.hosts);

            this.hosts.data.forEach(host => {
              if(host.protocol === 'icmp'){
                this.disablePorts = true;
              }else{
                this.disablePorts = false;
              }
            })

            this.isLoading = false;
            this.pagination.totalItems = response.recordsTotal;
            this.mapHosts(response.data || []);
            if (page === 1) {
              this.hostsSearchArray = this.hosts.data;
            }
            this.totalTrustedHosts = response.recordsTotal;
            if (this.isClone) {
              this.hosts.data = this.hosts.data.map(host => {
                return {
                  ...host,
                  id:
                    '_' +
                    Math.random()
                      .toString(36)
                      .substr(2, 9)
                };
              });
              this.localMode = true;
              this.service = this.localTrustedHostsService;
              this.service.setInitialalue(this.hosts);
              this.localModeHostsSubscription = this.service.hosts$.subscribe(
                localModeHosts => {
                  this.localModeHostsChanged.emit(
                    // Remove temporary id from the items
                    localModeHosts.map(({ id, ...host }) => host)
                  );
                }
              );
            }
          },
          error: err => {
            this.isLoading = false;
            if (err.status === ErrorCodeEnum.Forbidden) {
              if (err._body == LicenseErrorEnum.Invalid) {
                this.authenticationService.showLicenseOnly(true);
                this.router.navigate(['/license']);
              } else {
                const alert = {
                  closed: false,
                  type: 'danger',
                  message: 'You are not authorized to access this page.'
                };
                this.globalMessageService.pushAlert(alert);
              }
            } else {
              const alert = {
                closed: false,
                type: 'danger',
                message:
                  'Something went wrong while fetching Trusted Hosts list.'
              };
              this.globalMessageService.pushAlert(alert);
            }
          }
        });
    } else {
      this.searchTerm$.next({ searchTerm: this.searchTerm, typing: false });
    }
  }

  clearfun() {
    this.searchTerm = '';
    this.getPage(this.pagination.currentPage);
  }

  showContextHelp(name: any): void {
    if (name == ContextHelpEnum.host) {
      this.contextualHelp = [
        'host',
        'Hostname, IP address or subnet (in CIDR notation) to allow.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.port) {
      this.contextualHelp = ['port', 'Port or port range to allow.'];
      this.showContext = true;
    } else if (name == ContextHelpEnum.label) {
      this.contextualHelp = ['label', 'Short description.'];
      this.showContext = true;
    } else if (name == ContextHelpEnum.protocol) {
      this.contextualHelp = [
        'protocol',
        'Select protocol to allow on specified port.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.direction) {
      this.contextualHelp = [
        'direction',
        'Select direction to allow on specified IP/HostLimitEnum.'
      ];
      this.showContext = true;
    }
  }

  showAddForm() {
    this.addFormShown = true;

    // If there is success message from previous addition, remove it
    this.success = '';
  }

  hideAddForm() {
    this.addFormShown = false;
  }

  resetAddFormMessages() {
    this.isPortRangeChecked = false;
    this.success = '';
    this.error = '';
  }

  resetAddFormFields() {
    this.hostForm = new TrustedHost();
  }

  addTextHelp(string) {
    this.helpTextContent = string;
  }

  resetAddForm() {
    this.resetAddFormFields();
    this.resetAddFormMessages();
    this.hideAddForm();
  }

  addHost() {
    this.hostForm.isPortInRange = this.isPortRangeChecked;
    if (this.isPortRangeChecked) {
      const { startPort, endPort } = this.hostForm;
      if (parseInt(startPort) < parseInt(endPort)) {
        this.addTrustHost();
      } else {
        this.globalMessageService.pushAlert({
          type: 'danger',
          message: 'End port must be greater than start port'
        });
      }
    } else {
      this.addTrustHost();
    }
  }

  addTrustHost() {
    this.isLoading = true;
    this.error = '';
    this.success = '';

    this.service
      .addHost(this.hostForm, this.parentId, this.parentType)
      .subscribe({
        next: response => {
          this.globalMessageService.pushAlert({
            type: 'success',
            message: response.message
          });
          this.isLoading = false;
          this.isPortRangeChecked = false;
          this.resetAddFormFields();
          this.hideAddForm();
          this.refresh();
        },
        error: err => {
          // this.globalMessageService.pushAlert({
          //   type: 'danger',
          //   message: JSON.parse((err as any)._body).message
          // });
          this.isLoading = false;
        }
      });
  }

  refresh() {
    this.pagination.currentPage = 1;
    this.getPage(this.pagination.currentPage);
  }

  refreshCurrentPage() {
    this.getPage(this.pagination.currentPage);
  }

  /*
  addIpAddress() {
    let length = 0;
    this.error = '';
    let regex = /^(?=.*[^\.]$)((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.?){4}$/;
    let hostname = /^(http:\/\/www\.|https:\/\/www\.|http:\/\/|https:\/\/)?[a-z0-9]+([\-\.]{1}[a-z0-9]+)*\.[a-z]{2,5}(:[0-9]{1,5})?(\/.*)?$/;
    var result1 = regex.test(this.addGlobalIp);
    var result2 = hostname.test(this.addGlobalIp);
    if (this.hosts.length < 1000) {
      if (
        this.addGlobalIp.length &&
        this.addGlobalIp != null &&
        (this.addGlobalIpPort.length && this.addGlobalIpPort != null) &&
        (this.addGlobalIpProtocol.length && this.addGlobalIpProtocol != null) &&
        (this.addGlobalIpLabel.length && this.addGlobalIpLabel != null)
      ) {
        if (this.hosts) {
          if (this.hosts.length) {
            length = this.hosts.length;
          } else {
            length = 0;
          }
        }
        if (this.addGlobalIp.length >= 250) {
          this.error = 'Host/IP must be less than 250 characters in length';
        } else if (result1 == false && result2 == false) {
          this.error = 'Invalid format of Host/IP';
        } else if (
          isNaN(this.addGlobalIpPort) ||
          (parseInt(this.addGlobalIpPort) < 0 ||
            parseInt(this.addGlobalIpPort) > 65535)
        ) {
          this.error =
            'Port must be a valid number between 0 to 65535 both inclusive.';
        } else {
          let alreadyExist = false;
          var currentIndex: any = '';
          var ref = this;
          this.hosts.forEach(function(ip, index) {
            if (ref.editHost) {
              /if(typeof ip.id === 'undefined' && typeof ref.selectedHost.id === 'undefined'){
                currentIndex = ref.hosts.indexOf(ref.selectedHost);
              }else{
                index = ref.selectedHost.id;
              }/
              currentIndex = ref.hosts.indexOf(ref.selectedHost);
              if (
                ip.address === ref.addGlobalIp &&
                ip.direction === ref.addGlobalIpDirection &&
                ip.port === ref.addGlobalIpPort &&
                ip.protocol === ref.addGlobalIpProtocol &&
                index != currentIndex
              ) {
                alreadyExist = true;
              }
            } else {
              if (
                ip.address === ref.addGlobalIp &&
                ip.direction === ref.addGlobalIpDirection &&
                ip.port === ref.addGlobalIpPort &&
                ip.protocol === ref.addGlobalIpProtocol
              ) {
                alreadyExist = true;
              }
            }
          });
          if (alreadyExist) {
            this.error = 'This IP already exist.';
          } else {
            if (this.editHost) {
              var index = this.hosts.indexOf(this.selectedHost);
              this.hosts[index].address = this.addGlobalIp;
              this.hosts[index].port = this.addGlobalIpPort;
              this.hosts[index].protocol = this.addGlobalIpProtocol;
              this.hosts[index].label = this.selectedHost.label;
              this.hosts[index].direction = this.addGlobalIpDirection;
              this.editHost = false;
              this.selectedHost = {};
            } else {
              this.hosts[length] = new TrustedHost();
              this.hosts[length].address = this.addGlobalIp;
              this.hosts[length].port = this.addGlobalIpPort;
              this.hosts[length].protocol = this.addGlobalIpProtocol;
              this.hosts[length].label = this.addGlobalIpLabel;
              this.hosts[length].direction = this.addGlobalIpDirection;
            }
            this.addGlobalIp = '';
            this.addGlobalIpPort = '';
            this.addGlobalIpProtocol = 'tcp';
            this.addGlobalIpLabel = '';
          }
        }
      } else {
        this.error =
          'You must provide a valid Host, port, protocol and label.';
      }
    } else {
      this.error = 'You can only add upto 1000 Host/IP.';
    }
  }
  */

  deleteHost(deletedHost: TrustedHost) {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: `Do you really want to delete trusted host: ${deletedHost.label}?`
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        let mfa = localStorage.getItem('adminMfaValue');
        if (mfa === 'true') {
          this.showOtpDialog = true;
          this.option = 'delete';
          this.otpStatusSubscription = this.OtpAuthenticationService.otpStatusCheckDel$.subscribe(
            (res: any) => {
              if (res == true) {
                this.deleting.push(deletedHost.id);

                this.service
                  .deleteHost(
                    deletedHost.id,
                    this.parentId,
                    this.parentType,
                    deletedHost.label,
                    deletedHost.port,
                    deletedHost.protocol
                  )
                  .subscribe({
                    next: () => {
                      this.alert = {
                        closed: false,
                        type: 'success',
                        message: 'Host has been deleted successfully.'
                      };
                      this.globalMessageService.pushAlert(this.alert);

                      this.deleting = this.deleting.filter(
                        id => id !== deletedHost.id
                      );

                      // If we are not on the last page then, refresh to fill the new space
                      // otherwise, just remove the item from the list
                      if (
                        this.pagination.currentPage !== this.pagination.lastPage
                      ) {
                        this.refreshCurrentPage();
                      } else {
                        this.pagination.totalItems--;

                        // We are on the last page and we have run out of items, goto previous page
                        // and only if we're not on the first page
                        if (
                          this.hosts.data.length === 1 &&
                          this.hosts.data
                            .map(host => host.id)
                            .indexOf(deletedHost.id) > -1 &&
                          // No need to goto previous page if current page is the first
                          this.pagination.currentPage !== 1
                        ) {
                          this.pagination.currentPage--;
                          this.refreshCurrentPage();
                        } else {
                          this.hosts.data = this.hosts.data.filter(
                            host => host.id !== deletedHost.id
                          );
                        }
                      }
                    },
                    error: err => {
                      this.globalMessageService.pushAlert({
                        type: 'danger',
                        message: JSON.parse((err as any)._body).message
                      });
                    }
                  });
                this.showOtpDialog = false;
                this.OtpAuthenticationService.setOtpStatusValueDel(false);
              }
            }
          );
        } else {
          this.deleting.push(deletedHost.id);

          this.service
            .deleteHost(
              deletedHost.id,
              this.parentId,
              this.parentType,
              deletedHost.label,
              deletedHost.port,
              deletedHost.protocol
            )
            .subscribe({
              next: () => {
                this.alert = {
                  closed: false,
                  type: 'success',
                  message: 'Host has been deleted successfully.'
                };
                this.globalMessageService.pushAlert(this.alert);

                this.deleting = this.deleting.filter(
                  id => id !== deletedHost.id
                );

                // If we are not on the last page then, refresh to fill the new space
                // otherwise, just remove the item from the list
                if (this.pagination.currentPage !== this.pagination.lastPage) {
                  this.refreshCurrentPage();
                } else {
                  this.pagination.totalItems--;

                  // We are on the last page and we have run out of items, goto previous page
                  // and only if we're not on the first page
                  if (
                    this.hosts.data.length === 1 &&
                    this.hosts.data
                      .map(host => host.id)
                      .indexOf(deletedHost.id) > -1 &&
                    // No need to goto previous page if current page is the first
                    this.pagination.currentPage !== 1
                  ) {
                    this.pagination.currentPage--;
                    this.refreshCurrentPage();
                  } else {
                    this.hosts.data = this.hosts.data.filter(
                      host => host.id !== deletedHost.id
                    );
                  }
                }
              },
              error: err => {
                this.globalMessageService.pushAlert({
                  type: 'danger',
                  message: JSON.parse((err as any)._body).message
                });
              }
            });
        }
      }
    });
  }

  showEditForm(host: TrustedHost) {
    this.recentlyEdited = this.recentlyEdited.filter(id => host.id !== id);
    this.hostEditForm = { ...host };
    this.hostEditForm.direction = host.direction;
    this.hostEditForm.protocol = host.protocol;
  }

  resetEditForm(host) {
    console.log('host :>> ', host);
    host.port = host.previousPort;
    this.hostEditForm = null;
    this.editButtonDisabled = false;
    this.editError = '';
    this.disablePortsEditing = false
  }

  editHost(editedHost: TrustedHost) {
    console.log('editedHost :>> ', editedHost);
    let mfa = localStorage.getItem('adminMfaValue');
    if (mfa === 'true') {
      this.showOtpDialog = true;
      this.option = 'edit';
      this.otpStatusSubscription = this.OtpAuthenticationService.otpStatusCheckEdit$.subscribe(
        (res: any) => {
          if (res == true) {
            // Handle ICMP case: skip port validation
            if (this.hostEditForm.protocol === 'icmp') {
              this.hostEditForm.port = null; // Ensure port is cleared for ICMP
              this.editHostForm(editedHost);
            } else if (this.hostEditForm.isPortInRange) {
              const { startPort, endPort } = this.hostEditForm;
              if (+startPort < +endPort) {
                this.hostEditForm.port =
                  this.hostEditForm.startPort + '-' + this.hostEditForm.endPort;
                this.editHostForm(editedHost);
              } else {
                this.globalMessageService.pushAlert({
                  type: 'danger',
                  message: 'End port must be greater than start port'
                });
              }
            } else {
              if (!this.hostEditForm.port) {
                this.globalMessageService.pushAlert({
                  type: 'danger',
                  message: 'Port is required for TCP/UDP protocols'
                });
              } else {
                this.editHostForm(editedHost);
              }
            }

            this.showOtpDialog = false;
            this.OtpAuthenticationService.setOtpStatusValueEdit(false);
          }
        }
      );
    } else {
      // Handle ICMP case: skip port validation
      if (this.hostEditForm.protocol === 'icmp') {
        this.hostEditForm.port = null; // Ensure port is cleared for ICMP
        this.editHostForm(editedHost);
      } else if (this.hostEditForm.isPortInRange) {
        const { startPort, endPort } = this.hostEditForm;
        if (+startPort < +endPort) {
          this.hostEditForm.port =
            this.hostEditForm.startPort + '-' + this.hostEditForm.endPort;
          this.editHostForm(editedHost);
        } else {
          this.globalMessageService.pushAlert({
            type: 'danger',
            message: 'End port must be greater than start port'
          });
        }
      } else {
        if (!this.hostEditForm.port) {
          this.globalMessageService.pushAlert({
            type: 'danger',
            message: 'Port is required for TCP/UDP protocols'
          });
        } else {
          this.editHostForm(editedHost);
        }
      }
    }
  }


  editHostForm(editedHost: TrustedHost) {
    this.editButtonDisabled = true;
    this.editError = '';
    this.service
      .editHost(
        editedHost.id,
        this.hostEditForm,
        this.parentId,
        this.parentType
      )
      .subscribe({
        next: () => {
          this.recentlyEdited.push(this.hostEditForm.id);

          const editData = this.hostEditForm;
          const editedHostIndex = this.hosts.data
            .map(host => host.id)
            .indexOf(editedHost.id);

          // Make sure the record that was edited is still present by the time
          // the request completes, only then update record at stored index
          if (editedHostIndex > -1) {
            this.hosts.data[editedHostIndex] = {
              ...editData
            };
          }

          // Make sure the user is not editing some other record during the
          // time we waited for the request to complete
          if (editData.id === editedHost.id) {
            this.editButtonDisabled = false;
            this.hostEditForm = null;
          }
        },
        error: err => {
          this.editButtonDisabled = false;
        }
      });
  }

  editInputChanged() {
    this.validateHostForm();
    this.editError = '';

    this.editButtonDisabled =
      this.editAddress.invalid ||
      this.editLabel.invalid ||
      (this.startPort && this.startPort.invalid) ||
      (this.endPort && this.endPort.invalid) ||
      (this.editPort && this.editPort.invalid);
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getPage(this.pagination.currentPage);
  }

  sortRecord(term: string) {
    this.sortDir = this.sortDir == 'asc' ? 'desc' : 'asc';
    this.sort = term;
    this.getPage(this.pagination.currentPage);
  }

  cancelVerificationPassword(event: any) {
    this.showOtpDialog = false;
    this.OtpAuthenticationService.setOtpStatusValueEdit(false);
    this.OtpAuthenticationService.setOtpStatusValueDel(false);
  }

  submitVerificationPassword(event: any) {
    this.otptoken = event;
    // this.otptoken = event
  }

  getClosePopup(event: any) {
    this.showOtpDialog = false;
  }
}
