import { Component, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { UpdateService } from '../update.service';
import { PreserveSearch } from '../../models/preserve-search';
import { Pagination } from '../../models/pagination';
import { Update } from '../../models/update';
import { GlobalMessageClass } from '../../global-messages/global-messages';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { AuthenticationService } from '../../login/authentication.service';
import { Subject } from 'rxjs';
import {
  debounceTime,
  distinctUntilChanged,
  switchMap,
  tap
} from 'rxjs/operators';

import { ErrorCodeEnum } from '../../shared/emuns/error.enum';
import { DeviceOSEnum } from '../../shared/emuns/client.enum';
import { LicenseErrorEnum } from '../../shared/emuns/license.enum';
import { APIDataTypeEnum } from '../../shared/emuns/global.enum';
import { ConfirmModalComponent } from '../../../app/shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';
import { RolesAndPermissionService } from '../../roles-and-permission/roles-and-permission.service';

@Component({
  selector: 'app-updates-list',
  templateUrl: './updates-list.component.html',
  styleUrls: ['./updates-list.component.css']
})
export class UpdatesListComponent implements OnInit {
  preserveSearch = new PreserveSearch();
  public pagination: Pagination = new Pagination();
  public updates: any = {};
  public usersList = new Array();
  public selectedUpdate: Update;
  public alert: GlobalMessageClass;
  public addEditPageTitle: string;

  public loading: boolean = true;
  public dateFormat: string = 'dd/MM/yyyy HH:mm';
  pageSize = [10, 25, 50, 100, 200];
  sort: string;
  sortDir: string;
  $searchUpdates = new Subject();
  searchKeyword;
  navigationToEdit: boolean = false;

  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;

  timeZone = '';

  DeviceOS = DeviceOSEnum;
  DataType = APIDataTypeEnum;
  isDarkMode : boolean;
  constructor(
    private router: Router,
    private route: ActivatedRoute,
    private updateService: UpdateService,
    private globalMessageService: GlobalMessageService,
    private authenticationService: AuthenticationService,
    public dialog: MatDialog,
    private titleService: Title,
    private rolesPermissionService: RolesAndPermissionService
  ) {
    var localDateFormat = localStorage.getItem('dateFormat');
    if (typeof localDateFormat != 'undefined' || localDateFormat != null) {
      this.dateFormat = localDateFormat;
    }

    let timeZoneOffset = new Date().getTimezoneOffset();
    this.timeZone =
      'UTC' + (timeZoneOffset > 0 ? '-0' : '+0') + (timeZoneOffset / 60) * -1;
  }

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.titleService.setTitle('Settings | Software Updates');
    this.getCRUDdata(61);
    let updateSearch = this.preserveSearch.getSearch('backupSaveSearch');
    if (updateSearch) {
      this.pagination = updateSearch.pagination;
      this.searchKeyword = updateSearch.searchKeyword;
    }
    this.getPage(this.pagination.currentPage);
    this.subscibeToSearch();
  }

  getCRUDdata(privId){
    this.rolesPermissionService.getCrudData(privId).subscribe((res: any) => {
      this.createPermission = res?.create_permission === 1
      this.readPermission = res?.read_permission === 1
      this.updatePermission = res?.update_permission === 1
      this.deletePermission = res?.delete_permission === 1
    });
  }

  subscibeToSearch() {
    this.$searchUpdates
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap(data => {
          this.loading = true;
          this.searchKeyword = data;
          this.pagination.currentPage = 1;
          return this.updateService.getUpdates(
            this.pagination.currentPage,
            this.pagination.itemsPerPage,
            this.sort,
            this.sortDir,
            this.searchKeyword
          );
        })
      )
      .subscribe({
        next: (updates: any) => {
          this.loading = false;
          const { recordsTotal } = updates;
          this.updates = updates;
          this.pagination.totalItems = recordsTotal;
        },
        error: err => {
          this.loading = false;
        }
      });
  }

  private onSelect(update: Update): void {
    const search = {
      pagination: this.pagination,
      searchKeyword: this.searchKeyword
    };
    this.selectedUpdate = update;

    this.preserveSearch.saveSearch('backupSaveSearch', search);
    this.navigationToEdit = true;
    this.router.navigate(['/updates', this.selectedUpdate.id]);
  }

  ngOnDestroy(): void {
    if (!this.navigationToEdit) {
      this.preserveSearch.saveSearch('backupSaveSearch', '');
    }
  }

  getPage(page: number) {
    this.loading = true;
    this.pagination.currentPage = page;

    this.updateService
      .getUpdates(
        this.pagination.currentPage,
        this.pagination.itemsPerPage,
        this.sort,
        this.sortDir,
        this.searchKeyword
      )
      .pipe(
        tap((res: any) => {
          this.loading = false;
          this.pagination.totalItems = res.recordsTotal;
          this.pagination.currentPage = page;
          this.updates = res;
          // if (prevPage != this.pagination.currentPage) {
          //   let scrollToTop = window.setInterval(() => {
          //     let pos = window.pageYOffset;
          //     if (pos > 0) {
          //       window.scrollTo(0, pos - 20); // how far to scroll on each step
          //     } else {
          //       window.clearInterval(scrollToTop);
          //     }
          //   }, 16);
          // }
        })
      )
      .subscribe({
        next: next => {},
        error: err => {
          this.loading = false;
          if (err.status === ErrorCodeEnum.Forbidden) {
            if (err._body == LicenseErrorEnum.Invalid) {
              this.authenticationService.showLicenseOnly(true);
              this.router.navigate(['/license']);
            } else {
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'You are not authorized to access this page.'
              };
              this.globalMessageService.pushAlert(this.alert);
            }
          }
        },
        complete: () => {}
      });
  }

  clearfun() {
    this.searchKeyword = '';
    this.getPage(this.pagination.currentPage);
  }

  addUpdate(): void {
    this.addEditPageTitle = 'Add Update';
    this.router.navigate(['/create-update']);
  }

  private onDelete(update: Update): void {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: 'Are you sure to delete it?',
      },
    });

    dialogRef.afterClosed().subscribe((result) => {
      if (result){
        this.selectedUpdate = update;
        this.updateService.deleteUpdate(update.id).subscribe(response => {
          if (response.success) {
            this.getPage(this.pagination.currentPage);
          }
        });
      }
    });
  }

  downloadAction(update) {
    this.updateService.downloadPackage(update);
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getPage(this.pagination.currentPage);
  }

  sortRecords(term: string) {
    this.sortDir = this.sortDir == 'asc' ? 'desc' : 'asc';
    this.sort = term;
    this.getPage(this.pagination.currentPage);
  }
}
