import { Injectable } from '@angular/core';
import { ConfigService } from '../services/config.service';
import {
  HttpClient as AuthHttp,
  HttpErrorResponse,
  HttpHeaders
} from '@angular/common/http';
import { AuthenticationService } from '../login/authentication.service';
import { User } from '../models/user';
import { Role } from '../models/role';
import { LoginResponse } from '../models/login_response';
import { Observable, throwError } from 'rxjs';
import { map } from 'rxjs/operators';

@Injectable()
export class UserService {
  private usersUrl = 'http://localhost:3000/users';
  private usersApiUrl = '/users';
  private authApiUrl = '/authentication/';
  private rolesApiUrl = '/roles';
  private securityQuestionsApiUrl = '/securityQuestions';
  private checkvalidity = '/users/duplicationCheck';
  private users: User[];
  private apiBaseUrl: string;
  httpOptions = {
    headers: new HttpHeaders({ 'Content-Type': 'application/json' })
  };

  constructor(
    private authenticationService: AuthenticationService,
    private authHttp: AuthHttp,
    private configService: ConfigService
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.usersApiUrl = this.apiBaseUrl + this.usersApiUrl;
    this.authApiUrl = this.apiBaseUrl + this.authApiUrl;
    this.rolesApiUrl = this.apiBaseUrl + this.rolesApiUrl;
    this.securityQuestionsApiUrl =
      this.apiBaseUrl + this.securityQuestionsApiUrl;
    this.checkvalidity = this.apiBaseUrl + this.checkvalidity;
  }

  /** GET users from the server */

  getUsers(
    page: number = 1,
    pageSize: number = 10,
    sort: string = 'id',
    dir: string = 'desc',
    searchKey = ''
  ): any {
    let queryString = `?limit=${pageSize}&offset=${page -
      1}&sort=${sort}&dir=${dir}&searchKey=${searchKey}`;
    return this.authHttp
      .get(`${this.usersApiUrl}${queryString}`)
      .pipe(map((response: Response) => response));
  }

  getroles(): Observable<Role[] | any> {
    return this.authHttp
      .get(this.rolesApiUrl)
      .pipe(map((response: Response) => response));
  }

  getSecurityQuestions(): Observable<any> {
    return this.authHttp
      .get(this.securityQuestionsApiUrl)
      .pipe(map((response: Response) => response));
  }

  add_user(user: User, verificationPassword): Observable<LoginResponse | any> {
    let queryString = '';
    if (user.id != 0 && typeof user.id != 'undefined') {
      queryString = '/' + user.id;
      return this.authHttp
        .put(
          this.usersApiUrl + queryString,
          { user: user, verificationPassword },
          this.httpOptions
        )
        .pipe(map(this.extractData));
      //.catch(this.handleError);
    } else {
      return this.authHttp
        .post(
          this.usersApiUrl,
          { user: user, verificationPassword },
          this.httpOptions
        )
        .pipe(map((response: Response) => response));
    }
  }

  checkEmail(email: string): Observable<LoginResponse | any> {
    let queryString = '';
    return this.authHttp
      .post(this.usersApiUrl + 'checkMail', { email: email }, this.httpOptions)
      .pipe(map((response: Response) => response));
  }

  checkPassword(password: string, id: number): Observable<LoginResponse | any> {
    let queryString = '';
    return this.authHttp
      .post(
        this.authApiUrl + 'checkPassword',
        { password: password, id: id },
        this.httpOptions
      )
      .pipe(map((response: Response) => response));
  }

  checkPasswordbyEmail(
    password: string,
    email: string
  ): Observable<LoginResponse | any> {
    let queryString = '';
    return this.authHttp
      .post(
        this.authApiUrl + 'checkPasswordWithEmail',
        { password: password, email: email },
        this.httpOptions
      )
      .pipe(map((response: Response) => response));
  }

  resetMfa(id: number, verificationPassword) {
    return this.authHttp
      .post(this.usersApiUrl + '/resetMfa', { id: id, verificationPassword })
      .pipe(map(response => response));
  }

  delete_user(
    id: number,
    verificationPassword
  ): Observable<LoginResponse | any> {
    var queryString = '';

    if (id != 0) {
      queryString = '/delete/' + id;
    }
    this.httpOptions['body'] = { verificationPassword };
    return this.authHttp.delete(
      this.usersApiUrl + queryString,
      this.httpOptions
    );
    //.pipe(map(this.extractData));
    //.catch(this.handleError);
  }

  lock_user(id: number, verificationPassword): Observable<any> {
    return this.authHttp
      .post(
        this.usersApiUrl + '/lockUser',
        { id: id, verificationPassword },
        this.httpOptions
      )
      .pipe(map((response: Response) => response));
  }

  send_verification_email(id: number): Observable<any> {
    return this.authHttp
      .post(
        this.usersApiUrl + '/resend-verification/',
        { id: id },
        this.httpOptions
      )
      .pipe(map((response: Response) => response));
  }

  un_lock_user(id: number, verificationPassword): Observable<any> {
    return this.authHttp
      .post(
        this.usersApiUrl + '/unlockUser',
        { id: id, verificationPassword },
        this.httpOptions
      )
      .pipe(map((response: Response) => response));
  }

  updateUser(user: User): Observable<any> {
    return this.authHttp
      .put(this.usersUrl, user, this.httpOptions)
      .pipe(map((response: Response) => response));
  }

  updateUserTheme(isDark: boolean): Observable<any> {
    return this.authHttp
      .put(
        this.usersApiUrl + '/updateTheme',
        { isDark: isDark },
        this.httpOptions
      )
      .pipe(map((response: Response) => response));
  }

  regenerateKeys(userId): Observable<any> {
    return this.authHttp
      .put(this.usersApiUrl + '/keys/' + userId, {})
      .pipe(map((response: Response) => response));
  }

  getuserdetail(id): Observable<User | any> {
    var queryString = '';

    if (id != '') {
      queryString = '/' + id;
    }

    return this.authHttp
      .get(this.usersApiUrl + queryString)
      .pipe(map((response: Response) => response));
  }

  updateSecurityQuestions(id, user) {
    return this.authHttp
      .post(this.usersApiUrl + '/updateSecurityQuestions/' + id, { user: user })
      .pipe(map((response: Response) => response));
  }

  private extractData(res: Response) {
    let body = res;
    return body || {};
  }

  // private handleError(error: Response | any) {
  //   // In a real world app, we might use a remote logging infrastructure
  //   let errMsg: string;
  //   if (error) {
  //     const body = error || '';
  //     const err = body.error || JSON.stringify(body);
  //     errMsg = `${error.status} - ${error.statusText || ''} ${err}`;
  //   } else {
  //     errMsg = error.message ? error.message : error.toString();
  //   }
  //   console.error(errMsg);
  //   return Observable.throw(errMsg);
  // }
  private handleError(error: HttpErrorResponse | any) {
    let errMsg: string;
    if (error instanceof HttpErrorResponse) {
      const body = error.error || '';
      const err = body.error || JSON.stringify(body);
      errMsg = `${error.status} - ${error.statusText || ''} ${err}`;
    } else {
      errMsg = error.message ? error.message : error.toString();
    }
    console.error(errMsg);
    return throwError(() => new Error(errMsg));
  }
}
