import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { Subject } from 'rxjs';
import { debounceTime, distinctUntilChanged, switchMap } from 'rxjs/operators';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { Device } from '../../models/device';
import { WorkloadService } from '../workload.service';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-create-workload',
  templateUrl: './create-workload.component.html',
  styleUrls: ['./create-workload.component.css']
})
export class CreateWorkloadComponent implements OnInit {
  deviceOffset: number = 0;
  deviceName: string;
  devicesList = [];
  selectedDeviceIds = [];
  workloadName: string = '';
  loading: boolean = false;
  isEditing: boolean = false;
  editingId: string;

  $deviceSearch = new Subject();
  isDeviceSearching = false;

  constructor(
    public router: Router,
    private workloadService: WorkloadService,
    private globalMessageService: GlobalMessageService,
    private activatedRoute: ActivatedRoute,
    private titleService: Title
  ) {}

  ngOnInit() {
    this.activatedRoute.params.subscribe(data => {
      if (data.id) {
        this.isEditing = true;
        this.editingId = data.id;
        this.workloadService
          .getWorkloadById(data.id)
          .subscribe((response: any) => {
            this.workloadName = response.name;
            this.selectedDeviceIds =
              response.DeviceInventories.map(device => device.id || '') || [];
          });
      }
      
      if (this.isEditing) {
        this.titleService.setTitle('S2S Microsegmentations | Workloads Management | Edit Workload');
      } else {
        this.titleService.setTitle('S2S Microsegmentations | Workloads Management | Create Workload');
      }
      
      this.getDevices();
      this.subscribeDeviceSearch();
    });
  }

  getDevices() {
    this.workloadService
      .getDevicesForWorkload(
        this.deviceOffset,
        this.deviceName,
        this.editingId || null
      )
      .subscribe(response => {
        this.devicesList = response;
      });
  }

  subscribeDeviceSearch() {
    this.$deviceSearch
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((term: string) => {
          this.isDeviceSearching = true;
          this.deviceOffset = 0;
          this.deviceName = term;
          return this.workloadService.getDevicesForWorkload(
            this.deviceOffset,
            this.deviceName,
            this.editingId || null
          );
        })
      )
      .subscribe((response: any) => {
        this.isDeviceSearching = false;
        this.devicesList = response;
      });
  }

  fetchMoreDevices() {
    this.deviceOffset += 1;
    this.getDevices();
  }

  submitHandler() {
    this.loading = true;
    if (this.isEditing) {
      this.workloadService
        .editWorkload(this.editingId, this.workloadName, this.selectedDeviceIds)
        .subscribe(
          _response => {
            this.globalMessageService.pushAlert({
              closed: true,
              type: 'success',
              message: 'Workload saved successfully'
            });
            this.router.navigate(['/workloads']);
          },
          () => {
            this.loading = false;
          },
          () => {
            this.loading = false;
          }
        );
    } else {
      this.workloadService
        .createWorkload(this.workloadName, this.selectedDeviceIds)
        .subscribe(
          _response => {
            this.globalMessageService.pushAlert({
              closed: true,
              type: 'success',
              message: 'Workload created successfully'
            });
            this.router.navigate(['/workloads']);
          },
          () => {
            this.loading = false;
          },
          () => {
            this.loading = false;
          }
        );
    }
  }

  deviceFocus() {
    this.deviceOffset = 0;
    this.deviceName = '';
    this.$deviceSearch.next('');
  }
}
