import { Component } from '@angular/core';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';
import { Router } from '@angular/router';
import {
  Subject,
  Subscription,
  debounceTime,
  distinctUntilChanged,
  switchMap
} from 'rxjs';
import { DeviceService } from '../../devices-list/device.service';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { Pagination } from '../../models/pagination';
import { PreserveSearch } from '../../models/preserve-search';
import { OrganizationService } from '../../organization-list-view/organization.service';
import { OtpAuthenticationService } from '../../otp-verification/otp-authentication.service';
import { RolesAndPermissionService } from '../../roles-and-permission/roles-and-permission.service';
import { ConfirmModalComponent } from '../../shared/confirm-modal.component';
import { DeviceOSEnum, DeviceTypeEnum } from '../../shared/emuns/client.enum';

@Component({
  selector: 'app-workload-devices',
  templateUrl: './workload-devices.component.html',
  styleUrls: ['./workload-devices.component.css']
})
export class WorkloadDevicesComponent {
  showHelpText: boolean = false;
  pagination = new Pagination();
  devices: any = {};
  public actionsDropdown: number = 0;
  $searchDevices = new Subject();
  showPasswordConfimation = false;
  deviceTobeDeleted: any;
  isNameSort: boolean = false;
  searchWord;
  preserveSearch = new PreserveSearch();
  navigationToEditDevice: boolean = false;
  DeviceOS = DeviceOSEnum;
  DeviceType = DeviceTypeEnum;
  loading: Boolean = true;

  pageSize = [10, 25, 50, 100, 200];

  deviceTypes = [
    DeviceTypeEnum.Desktop,
    DeviceTypeEnum.IoTBridgeClient,
    DeviceTypeEnum.Laptop,
    DeviceTypeEnum.VirtualDesktop,
    DeviceTypeEnum.PhysicalServer,
    DeviceTypeEnum.Smartphone,
    DeviceTypeEnum.Tablet,
    DeviceTypeEnum.ThinBridgeClient,
    DeviceTypeEnum.ServerBridgeClient,
    DeviceTypeEnum.VirtualServer,
    DeviceTypeEnum.ARMPlatform
  ];
  organizations = [];
  departments = [];
  locations = [];
  osVersions = [];

  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;

  selectedDeviceType: string;
  selectedOrganization: string;
  selectedOs: string;
  selectedDepartment;
  selectedLocation: string;
  selectedOsVersion: string;
  referenceMachineCheck = false;

  sort: string;
  sortDir: string;

  selectAll = false;

  selectAllExcept: any = new Set();
  selectedDevices: any = new Set();

  uploadCsvFile = false;
  fileData = {};
  isFileValid = false;
  isUploading = false;
  showFailedUpload: boolean = false;
  failedDevices = [];

  devicesOs = [
    { value: DeviceOSEnum.CentosServer, display: 'CentOS Server' },
    { value: DeviceOSEnum.UbuntuServer18, display: 'Ubuntu Server' },
    { value: DeviceOSEnum.WindowsServer, display: 'Windows Server' },
  ];

  isDarkMode: boolean;
  showOtpDialog: boolean = false;
  otptoken: string = '';
  option: string;
  private otpStatusSubscription: Subscription;

  constructor(
    private router: Router,
    private deviceService: DeviceService,
    private organizationService: OrganizationService,
    private globalMessageService: GlobalMessageService,
    public dialog: MatDialog,
    private titleService: Title,
    private OtpAuthenticationService: OtpAuthenticationService,
    private rolesPermissionService: RolesAndPermissionService
  ) {}

  sortByNameHandler() {
    this.isNameSort = !this.isNameSort;
    if (this.searchWord) {
      this.$searchDevices.next({ searchWord: this.searchWord, typing: false });
    } else {
      this.getDevices();
    }
  }

  getOSName(osName) {
    switch (osName) {
      case DeviceOSEnum.Ubuntu18:
        return 'Ubuntu Desktop';
      case DeviceOSEnum.UbuntuServer18:
        return 'Ubuntu Server';
      case DeviceOSEnum.UbuntuServer20armv8:
        return 'Ubuntu Server (ARMv8)';
      case DeviceOSEnum.Windows:
        return 'Windows';
      case DeviceOSEnum.WindowsServer:
        return 'Windows Server';
      case DeviceOSEnum.Macintosh:
        return 'Macintosh';
      case DeviceOSEnum.IOS:
        return 'iOS';
      case DeviceOSEnum.Android:
        return 'Android';
      case DeviceOSEnum.CentosServer:
        return 'CentOS Server';
      default:
        return osName;
    }
  }

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.titleService.setTitle('Device Management | Devices');
    this.getCRUDdata(64);
    const _saveSearch = this.preserveSearch.getSearch('deviceSearch');
    if (!_saveSearch) {
      this.getDevices();
    } else {
      this.searchWord = _saveSearch.searchKey;
      this.pagination = _saveSearch.pagination;
      this.selectedDeviceType = _saveSearch.selectedDeviceType;
      this.selectedOsVersion = _saveSearch.selectedOsVersion;
      this.selectedOrganization = _saveSearch.selectedOrganization;
      this.selectedDepartment = parseInt(_saveSearch.selectedDepartment);
      this.selectedLocation = _saveSearch.selectedLocation;
      this.selectedOs = _saveSearch.selectedOs;
      if (this.selectedOrganization) {
        this.fetchOrganizationDepartments();
      } else {
        this.selectedDepartment = null;
      }
      this.getDevices();
    }
    this.organizationService.getOrganizationNames().subscribe((orgs: any) => {
      this.organizations = orgs;
    });
    this.deviceService.getAllLocations().subscribe((locations: any) => {
      this.locations = locations.filter(location => {
        return location != null && location != '';
      });
    });
    this.subscribeToSearch();
  }

  ngOnDestroy() {
    if (!this.navigationToEditDevice) {
      this.preserveSearch.saveSearch('deviceSearch', '');
    }

    if (this.otpStatusSubscription) {
      this.otpStatusSubscription.unsubscribe();
    }
  }

  getCRUDdata(privId) {
    this.rolesPermissionService.getCrudData(privId).subscribe((res: any) => {
      this.createPermission = res?.create_permission === 1 ? true : false;
      this.readPermission = res?.read_permission === 1 ? true : false;
      this.updatePermission = res?.update_permission === 1 ? true : false;
      this.deletePermission = res?.delete_permission === 1 ? true : false;
    });
  }

  private changeCheckStatusOfAllDevices(check) {
    this.devices.data = this.devices.data.map(_device => {
      return {
        ..._device,
        isChecked:
          ((!this.selectAllExcept.has(_device.id) && this.selectAll) ||
            (this.selectedDevices.has(_device.id) && !this.selectAll)) &&
          ![
            DeviceTypeEnum.IoTBridgeClient,
            DeviceTypeEnum.ServerBridgeClient,
            DeviceTypeEnum.ThinBridgeClient,
            DeviceTypeEnum.Smartphone
          ].includes(_device.device_type)
      };
    });
    if (this.selectAll) {
      for (let j = 0; j < this.devices.data.length; j++) {
        if (
          [
            DeviceTypeEnum.IoTBridgeClient,
            DeviceTypeEnum.ServerBridgeClient,
            DeviceTypeEnum.ThinBridgeClient,
            DeviceTypeEnum.Smartphone
          ].includes(this.devices.data[j].device_type)
        ) {
          this.selectAllExcept.add(this.devices.data[j].id);
        }
      }
    }
  }

  subscribeToSearch() {
    this.$searchDevices
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((search: { searchWord: string; typing: boolean }) => {
          this.loading = true;
          this.clearSelection();
          if (search.typing) {
            this.pagination.currentPage = 1;
          }
          return this.deviceService.searchDevices(
            search.searchWord,
            this.pagination.itemsPerPage,
            this.pagination.currentPage,
            this.sort,
            this.sortDir,
            this.selectedDeviceType,
            this.selectedOrganization,
            this.selectedDepartment,
            this.selectedLocation,
            this.selectedOsVersion,
            this.selectedOs
          );
        })
      )
      .subscribe((devices: any) => {
        this.devices = devices;
        this.dropDownUpdates();

        this.pagination.totalItems = devices.recordsTotal;
        this.loading = false;
      });
  }

  createDevice() {
    this.router.navigate(['/create-device']);
  }

  async dropDownUpdates() {
    if (this.devices.data) {
      for (let i = 0; i < this.devices.data.length; i++) {
        let osVersion = this.devices.data[i].os_version;
        if (osVersion) {
          if (this.osVersions.indexOf(osVersion) == -1) {
            this.osVersions.push(osVersion);
          }
        }
      }
      this.osVersions = [...this.osVersions];
    }
  }

  getDevices() {
    this.loading = true;
    this.deviceService
      .getWorkloadDevices(
        this.pagination,
        this.sort,
        this.sortDir,
        this.searchWord,
        this.selectedDeviceType ? this.selectedDeviceType : '',
        this.selectedOrganization ? this.selectedOrganization : '',
        this.selectedDepartment ? this.selectedDepartment : '',
        this.selectedLocation ? this.selectedLocation : '',
        this.selectedOsVersion ? this.selectedOsVersion : '',
        this.selectedOs ? this.selectedOs : '',
        this.referenceMachineCheck
      )
      .subscribe(
        (response: any) => {
          this.loading = false;
          response.data = response.data.map(_device => {
            return { ..._device, isChecked: false };
          });

          this.devices = response;
          this.dropDownUpdates();

          setTimeout(() => {
            this.changeCheckStatusOfAllDevices(this.selectAll);
          }, 10);

          this.pagination.totalItems = response.recordsTotal;
        },
        error => {
          this.loading = false;
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'danger',
            message: 'Error in getting devices',
            sticky: false
          });
        }
      );
    this.preserveCurrentSerarch();
  }

  clearfun() {
    this.searchWord = '';
    this.getDevices();
  }

  editDevice(device) {
    let mfa = localStorage.getItem('adminMfaValue');
    if (mfa === 'true') {
      this.showOtpDialog = true;
      this.option = 'edit';
      this.otpStatusSubscription = this.OtpAuthenticationService.otpStatusCheckEdit$.subscribe(
        (res: any) => {
          if (res == true) {
            this.preserveCurrentSerarch();
            const queryParams = { workload: 'false' };
            this.router.navigate(['/device', device.id], { queryParams });
            this.OtpAuthenticationService.setOtpStatusValueEdit(false);
          }
        }
      );
    } else {
      this.preserveCurrentSerarch();
      const queryParams = { workload: 'false' };
      this.router.navigate(['/device', device.id], { queryParams });
    }
  }

  preserveCurrentSerarch() {
    const save = {
      pagination: this.pagination,
      searchKey: this.searchWord,
      selectedDeviceType: this.selectedDeviceType,
      selectedOsVersion: this.selectedOsVersion,
      selectedOrganization: this.selectedOrganization,
      selectedDepartment: this.selectedDepartment,
      selectedLocation: this.selectedLocation,
      selectedOs: this.selectedOs
    };
    this.preserveSearch.saveSearch('deviceSearch', save);
    this.navigationToEditDevice = true;
  }

  viewDevice(device) {
    this.preserveCurrentSerarch();
    this.router.navigate(['/view-workload-device', device.id]);
  }

  pageChange(page) {
    this.pagination.currentPage = page;
    if (this.searchWord) {
      this.$searchDevices.next({ searchWord: this.searchWord, typing: false });
    } else {
      this.getDevices();
    }
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getDevices();
  }

  deviceTypeChange() {
    this.clearSelection();
    this.pagination.currentPage = 1;
    if (!this.selectedDeviceType) this.selectedDeviceType = null;
    this.getDevices();
  }

  clearSelection() {
    this.selectAllExcept.clear();
    this.selectAll = false;
  }

  async fetchOrganizationDepartments() {
    this.pagination.currentPage = 1;
    this.departments = [];
    if (this.selectedOrganization) {
      const org = (await this.organizationService
        .getOrganizationByName(this.selectedOrganization)
        .toPromise()) as any;
      const departments = (await this.organizationService
        .getDepartments(org.id)
        .toPromise()) as any;
      for (let j = 0; j < departments.data.length; j++) {
        this.departments.push({
          deptId: departments.data[j].id,
          deptName: departments.data[j].name
        });
      }
      this.departments = [...this.departments];
    } else {
      this.selectedOrganization = null;
    }
  }

  async organizationChange() {
    this.clearSelection();
    this.departments = [];
    this.selectedDepartment = undefined;
    if (!this.selectedOrganization) {
      this.selectedOrganization = undefined;
    } else {
      this.fetchOrganizationDepartments();
    }
    this.getDevices();
  }

  departmentChange() {
    this.pagination.currentPage = 1;
    if (!this.selectedDepartment) this.selectedDepartment = null;
    this.clearSelection();
    this.getDevices();
  }

  locationChange() {
    this.pagination.currentPage = 1;
    if (!this.selectedLocation) this.selectedLocation = null;
    this.clearSelection();
    this.getDevices();
  }

  osVersionChange() {
    if (!this.selectedOsVersion) this.selectedOsVersion = null;
    this.clearSelection();
    this.getDevices();
  }

  referenceMachineFilter() {
    this.getDevices();
  }

  sortRecord(term: string) {
    this.sortDir = this.sortDir == 'asc' ? 'desc' : 'asc';
    this.sort = term;
    this.getDevices();
  }

  osFilterChange() {
    this.clearSelection();
    this.getDevices();
  }
}
