import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { Subject } from 'rxjs';
import { debounceTime, switchMap, distinctUntilChanged } from 'rxjs/operators';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { Pagination } from '../../models/pagination';
import { PreserveSearch } from '../../models/preserve-search';
import { Workload } from '../../models/workload';
import { WorkloadService } from '../workload.service';
import { ConfirmModalComponent } from '../../../app/shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';
import { RolesAndPermissionService } from '../../roles-and-permission/roles-and-permission.service';

@Component({
  selector: 'app-workload-list',
  templateUrl: './workload-list.component.html',
  styleUrls: ['./workload-list.component.css']
})
export class WorkloadListComponent implements OnInit {
  showHelpText: boolean = false;
  pagination = new Pagination();
  searchKeyword;
  loading: boolean = false;
  workloads = { data: [] };
  private preserveSearch: PreserveSearch = new PreserveSearch();
  saveSearchName = 'workloadSaveSearch';
  pageSize = [10, 25, 50, 100, 200];
  $workloadSearch = new Subject();

  allSelected: boolean = false;
  selectAllExcept: any = new Set();
  selectedWorkloads: any = new Set();
  isDarkMode : boolean;

  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;
  
  constructor(
    private router: Router,
    private workloadService: WorkloadService,
    private globalMessageService: GlobalMessageService,
    public dialog: MatDialog,
    private titleService: Title,
    private rolesPermissionService: RolesAndPermissionService
  ) {}

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.titleService.setTitle('S2S Microsegmentations | Workloads Management');
    this.getCRUDdata(66);
    let saveSearch = this.preserveSearch.getSearch(this.saveSearchName);
    if (saveSearch) {
      this.pagination = saveSearch.pagination;
      this.searchKeyword = saveSearch.searchKeyword;
    }
    let workLoadSearch = this.preserveSearch.getSearch('softwarePolicySearch');
    if (workLoadSearch) {
      this.pagination.currentPage = workLoadSearch.currentPage;
      this.searchKeyword = workLoadSearch.searchKey;
    }
    this.subscribeWorkloadSearch();
    this.getWorkloads();
  }

  editWorkload(workloadId) {
    const search = {
      pagination: this.pagination,
      searchKeyword: this.searchKeyword
    };
    this.preserveSearch.saveSearch(this.saveSearchName, search);
    this.router.navigate([`/workloads/${workloadId}`]);
  }

  manageMicrosegment(workloadId) {
    const search = {
      pagination: this.pagination,
      searchKeyword: this.searchKeyword
    };
    this.preserveSearch.saveSearch(this.saveSearchName, search);
    this.router.navigate([`/workload/${workloadId}/discovery`]);
  }

  getCRUDdata(privId){
    this.rolesPermissionService.getCrudData(privId).subscribe((res: any) => {
      this.createPermission = res?.create_permission === 1 
      this.readPermission = res?.read_permission === 1 
      this.updatePermission = res?.update_permission === 1 
      this.deletePermission = res?.delete_permission === 1 
    });
  }

  subscribeWorkloadSearch() {
    this.$workloadSearch
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((term: string) => {
          this.pagination.currentPage = 1;
          return this.workloadService.getWorkloads(
            this.pagination.currentPage - 1,
            this.pagination.itemsPerPage,
            this.searchKeyword
          );
        })
      )
      .subscribe(
        (response: any) => {
          this.pagination.totalItems = response.recordsTotal;
          this.workloads = response;
        },
        null,
        () => {
          this.loading = false;
        }
      );
  }
  getWorkloads() {
    this.loading = true;
    this.workloadService
      .getWorkloads(
        this.pagination.currentPage - 1,
        this.pagination.itemsPerPage,
        this.searchKeyword
      )
      .subscribe(
        response => {
          this.pagination.totalItems = response.recordsTotal;
          this.workloads = response;
        },
        null,
        () => {
          this.loading = false;
        }
      );
  }

  pageChange(event) {
    this.pagination.currentPage = event;
    this.getWorkloads();
  }

  addWorkload() {
    this.router.navigate([`/create-workload`]);
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getWorkloads();
  }

  deleteHandler(workloadId) {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: `Are you sure want to delete? it will also delete all microsegments related to this workload.`,
      },
    });

    dialogRef.afterClosed().subscribe((result) => {
      if (result){
          this.workloadService.deleteWorkload(workloadId).subscribe(_response => {
          this.globalMessageService.pushAlert({
            closed: true,
            type: 'success',
            message: 'Workload deleted successfully'
          });
          this.getWorkloads();
        });
      }
    });
  }

  private changeCheckStatusOfAllGroups() {
    this.workloads.data = this.workloads.data.map(workload => {
      return {
        ...workload,
        isChecked:
          (!this.selectAllExcept.has(workload.id) && this.allSelected) ||
          (this.selectedWorkloads.has(workload.id) && !this.allSelected)
      };
    });
  }

  selectAllWorkloadChange() {
    if (!this.allSelected) {
      this.selectAllExcept.clear();
      this.selectedWorkloads.clear();
    } else {
      this.selectedWorkloads.clear();
    }
    this.changeCheckStatusOfAllGroups();
  }

  changeCurrentCheckBox(workload) {
    const { isChecked, id } = workload;
    if (this.allSelected) {
      if (isChecked) {
        this.selectAllExcept.delete(id);
      } else {
        this.selectAllExcept.add(id);
      }
    } else {
      if (!isChecked) {
        this.selectedWorkloads.delete(id);
      } else {
        this.selectedWorkloads.add(id);
      }
    }
  }

  handleBulkDelete() {
    const count = this.allSelected
      ? this.pagination.totalItems - this.selectAllExcept.size
      : this.selectedWorkloads.size;
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: `Are you sure want to delete ${count} workload(s)`,
      },
    });

    dialogRef.afterClosed().subscribe((result) => {
      if (result){
        this.workloadService
        .performBulkaction(
          'deleteWorkloads',
          this.selectedWorkloads,
          this.allSelected,
          this.selectAllExcept,
          this.searchKeyword
        )
        .subscribe(_response => {
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'success',
            message: 'Workloads has been deleted successfully'
          });
          this.allSelected = false;
          this.selectAllExcept = new Set();
          this.selectedWorkloads = new Set();
          this.getWorkloads();
        });
      }
    });
  }
}
