import { Injectable } from '@angular/core';
import { ConfigService } from '../services/config.service';
import { HttpClient as AuthHttp } from '@angular/common/http';
import { AuthenticationService } from '../login/authentication.service';
import { AuthShortlivedCookieService } from '../auth-shortlived-cookie.service';
import { map } from 'rxjs/operators';

@Injectable()
export class WorkloadService {
  private workloadsApiUrl = '/workloads';
  private apiBaseUrl: string;

  constructor(
    private authShortlivedCookie: AuthShortlivedCookieService,
    private authenticationService: AuthenticationService,
    private authHttp: AuthHttp,
    private configService: ConfigService
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.workloadsApiUrl = this.apiBaseUrl + this.workloadsApiUrl;
  }

  /** GET from the server */

  getWorkloads(page: number = 1, pageSize: number = 10, search = ''): any {
    let query = `?limit=${pageSize}&offset=${page}&search=${search}`;

    return this.authHttp
      .get(this.workloadsApiUrl + query)
      .pipe(map((response: Response) => response));
  }

  getDevicesForWorkload(offset, deviceName = '', workloadId = null): any {
    let query = `?offset=${offset}&deviceName=${deviceName}&workloadId=${workloadId}`;
    return this.authHttp
      .get(this.workloadsApiUrl + '/devices/available' + query)
      .pipe(map((response: Response) => response));
  }

  createWorkload(name, devices = []) {
    return this.authHttp
      .post(this.workloadsApiUrl + '/', { name, devices })
      .pipe(map((response: Response) => response));
  }

  editWorkload(workloadId, name, devices = []) {
    return this.authHttp
      .put(this.workloadsApiUrl + '/detail/' + workloadId, { name, devices })
      .pipe(map((response: Response) => response));
  }

  deleteWorkload(workloadId) {
    return this.authHttp
      .delete(this.workloadsApiUrl + '/detail/' + workloadId)
      .pipe(map((response: Response) => response));
  }

  getWorkloadById(workloadId) {
    return this.authHttp
      .get(this.workloadsApiUrl + '/detail/' + workloadId)
      .pipe(map((response: Response) => response));
  }

  performBulkaction(
    action = 'deleteWorkloads',
    workloadIds = [],
    allSelected = false,
    excludeIds = [],
    search = ''
  ): any {
    return this.authHttp
      .post(`${this.workloadsApiUrl}/bulk-action`, {
        workloadIds: Array.from(workloadIds),
        excludeIds: Array.from(excludeIds),
        allSelected,
        search,
        action
      })
      .pipe(map((response: Response) => response));
  }

  saveWorkDiscovery(workloadId, mode, duration, threshold) {
    return this.authHttp
      .put(this.workloadsApiUrl + '/save-work-discovery/' + workloadId, {
        duration,
        threshold,
        mode
      })
      .pipe(map((response: Response) => response));
  }

  refreshWorkloads(workloadId) {
    return this.authHttp
      .get(this.workloadsApiUrl + '/refresh/' + workloadId)
      .pipe(map((response: Response) => response));
  }

  saveAllWorkDiscovery(mode, duration, threshold) {
    return this.authHttp
      .put(this.workloadsApiUrl + '/save-work-discovery/', {
        duration,
        threshold,
        mode
      })
      .pipe(map((response: Response) => response));
  }

  flushDicovery() {
    return this.authHttp
      .delete(this.workloadsApiUrl + '/flush-discovery/', {})
      .pipe(map((response: Response) => response));
  }
  
  saveMicroSegmentTag(workloadId, edge, tagName) {
    return this.authHttp
      .put(this.workloadsApiUrl + '/save-microseg-tag/' + workloadId, {
        edge: edge,
        tagName: tagName
      })
      .pipe(map((response: Response) => response));
  }

  saveAllMicroSegmentTag(edge, tagName, workloadTag, srcWorkloadTag) {
    return this.authHttp
      .put(this.workloadsApiUrl + '/save-microseg-tag/', {
        edge: edge,
        tagName: tagName,
        workloadTag: workloadTag,
        srcWorkloadTag: srcWorkloadTag
      })
      .pipe(map((response: Response) => response));
  }

  getWorkLoadDevices(workloadId){
    return this.authHttp
    .get(this.workloadsApiUrl + '/workload-devices/' + workloadId)
    .pipe(map((response: Response) => response));
  }

  getAllWorkLoadDevices(){
    return this.authHttp
    .get(this.workloadsApiUrl + '/all-workload-devices/')
    .pipe(map((response: Response) => response));
  }

  s2sDiscoveryManualData(workloadId, manualData){
    return this.authHttp
    .post(this.workloadsApiUrl + '/work-discovery-manual/' + workloadId, { manualData })
    .pipe(map((response: Response) => response));
  }

  allS2sDiscoveryManualData(manualData){
    return this.authHttp
    .post(this.workloadsApiUrl + '/work-discovery-manual/', { manualData })
    .pipe(map((response: Response) => response));
  }

  getManualRuleData(workloadId){
    return this.authHttp
    .get(this.workloadsApiUrl + '/work-discovery-manual/' + workloadId)
    .pipe(map((response: Response) => response));
  }

  getAllManualRuleData(){
    return this.authHttp
    .get(this.workloadsApiUrl + '/work-discovery-manual/')
    .pipe(map((response: Response) => response));
  }

  deleteManualRule(workloadId, item){
    return this.authHttp
    .delete(this.workloadsApiUrl + '/work-discovery-manual/' + workloadId, { body: item })
    .pipe(map((response: Response) => response));
  }

  deleteFromAllManualRules(item){
    return this.authHttp
    .delete(this.workloadsApiUrl + '/work-discovery-manual', { body: item })
    .pipe(map((response: Response) => response));
  }

  deleteFromAllAutoRules(item){
    return this.authHttp
    .delete(this.workloadsApiUrl + '/work-discovery-auto', { body: item })
    .pipe(map((response: Response) => response));
  }

  getS2sDiscovery(workloadId) {
    return this.authHttp
      .get(this.workloadsApiUrl + '/work-discovery/' + workloadId)
      .pipe(map((response: Response) => response));
  }

  getWorklaodName(workloadId) {
    return this.authHttp
      .get(this.workloadsApiUrl + '/get-worklaod-name/' + workloadId)
      .pipe(map((response: Response) => response));
  }

  getAllS2sDiscovery() {
    return this.authHttp
      .get(this.workloadsApiUrl + '/all-work-discovery/')
      .pipe(map((response: Response) => response));
  }

  getPaginatedS2sDiscovery(workloadId: string, page: number = 1, pageSize: number = 10,searchKeyword?:string) {
    const offset = (page - 1) * pageSize;
    const queryParams = `workloadId=${workloadId}&limit=${pageSize}&offset=${offset}&search=${searchKeyword}`;
    const query = queryParams ? `?${queryParams}` : '';
  
    return this.authHttp
      .get(`${this.workloadsApiUrl}/work-paginated-discovery${query}`)
      .pipe(map((response: Response) => response));
  }

  getAllPaginatedS2sDiscovery(page: number = 1, pageSize: number = 10,searchKeyword?:string) {
    const offset = (page - 1) * pageSize;
    const queryParams = `limit=${pageSize}&offset=${offset}&search=${searchKeyword}`;
    const query = queryParams ? `?${queryParams}` : '';
  
    return this.authHttp
      .get(`${this.workloadsApiUrl}/all-work-paginated-discovery${query}`)
      .pipe(map((response: Response) => response));
  } 

  toggleS2sDiscovery(discoveryId, type) {
    return this.authHttp
      .patch(this.workloadsApiUrl + '/work-discovery/toggle/' + discoveryId, {type})
      .pipe(map((response: Response) => response));
  }
}
