import { Injectable, Injector } from '@angular/core';
import {
  HttpInterceptor,
  HttpRequest,
  HttpHandler,
  HttpEvent,
  HttpResponse
} from '@angular/common/http';
import { Observable } from 'rxjs';
import { AuthenticationService } from './app/login/authentication.service';
import { GlobalMessageService } from './app/global-messages/global-messages.service';
import { tap } from 'rxjs/operators';
import { Router } from '@angular/router';
import { LicenseErrorEnum } from './app/shared/emuns/license.enum';

@Injectable()
export class JwtInterceptor implements HttpInterceptor {
  isControllerUploaded: boolean = false;
  private isLicenseNotificationShown = false;
  constructor(
    private globalMessageService: GlobalMessageService,
    private injector: Injector
  ) {}

  private hideNavItems(_licenseDays = '-1') {
    if (
      _licenseDays !== 'Processing' &&
      _licenseDays !== 'NoLicense' &&
      _licenseDays !== 'Invalid'
    ) {
      const licenseDays = parseFloat(_licenseDays);
      const licenseDaysApprox = Math.floor(licenseDays);

      if (licenseDaysApprox === -1) {
        const authenticationService = this.injector.get(AuthenticationService);
        authenticationService.showLicenseOnly(true);
      }
    } else {
      const authenticationService = this.injector.get(AuthenticationService);
      authenticationService.showLicenseOnly(true);
    }
  }

  private showInformativeLicenseNotification(
    _licenseDays = '-1',
    expiringToday,
    expiringTomorrow,
    violated,
    controllerCount
  ) {
    if (
      _licenseDays !== 'Processing' &&
      _licenseDays !== 'NoLicense' &&
      _licenseDays != 'Invalid'
    ) {
      const licenseDays = parseFloat(_licenseDays);
      const licenseDaysApprox = Math.floor(licenseDays);

      const flagSet = sessionStorage.getItem('licenseNotificationShown');

      if (!flagSet && licenseDays > 0 && licenseDays <= 30) {
        sessionStorage.setItem('licenseNotificationShown', 'true');

        this.globalMessageService.removeAlertByName(
          'LICENSE_NOTIFICATION_PROCESSING'
        );
        this.globalMessageService.removeAlertByName(
          'LICENSE_NOTIFICATION_NOLICENSE'
        );
        this.globalMessageService.removeAlertByName(
          'LICENSE_NOTIFICATION_NOLICENSE_CONTROLLER'
        );
        this.globalMessageService.pushAlertIfNotPresent({
          name: 'LICENSE_NOTIFICATION',
          sticky: true,
          type: 'info',
          message:
            expiringToday === 'true'
              ? 'Your license is going to expire today.'
              : expiringTomorrow === 'true'
              ? 'Your license is going to expire tomorrow.'
              : `Your license is about to expire in ${licenseDaysApprox} days.`
        });
      } else if (licenseDays <= 0) {
        this.globalMessageService.removeAlertByName(
          'LICENSE_NOTIFICATION_PROCESSING'
        );
        this.globalMessageService.removeAlertByName(
          'LICENSE_NOTIFICATION_NOLICENSE'
        );
        this.globalMessageService.removeAlertByName(
          'LICENSE_NOTIFICATION_NOLICENSE_CONTROLLER'
        );

        this.globalMessageService.pushAlertIfNotPresent({
          name: 'LICENSE_NOTIFICATION',
          sticky: true,
          type: 'danger',
          message: 'Your license has expired.'
        });
      }
    } else if (_licenseDays === 'Processing') {
      this.globalMessageService.removeAlertByName(
        'LICENSE_NOTIFICATION_NOLICENSE'
      );
      this.globalMessageService.removeAlertByName(
        'LICENSE_NOTIFICATION_NOLICENSE_CONTROLLER'
      );
      this.globalMessageService.removeAlertByName('LICENSE_NOTIFICATION');
      if (controllerCount == 0) {
        if (!this.isControllerUploaded) {
          this.globalMessageService.pushAlertIfNotPresent({
            name: 'LICENSE_NOTIFICATION_PROCESSING_Controller_Count',
            sticky: true,
            type: 'danger',
            message:
              'Your license process is waiting for controller deployment.please deploy the Controller for processing of license.'
          });
        }
        this.isControllerUploaded = true;
      } else {
        this.isControllerUploaded = true;
        this.globalMessageService.pushAlertIfNotPresent({
          name: 'LICENSE_NOTIFICATION_PROCESSING',
          sticky: true,
          type: 'info',
          message:
            'Your license is being processed and you will have access to limited features of your Invisily installation.'
        });
      }
    } else if (_licenseDays === 'Invalid') {
      this.showViolationLicenseNotification(violated);
    } else {
      this.globalMessageService.removeAlertByName(
        'LICENSE_NOTIFICATION_PROCESSING'
      );
      this.globalMessageService.pushAlertIfNotPresent({
        name: 'LICENSE_NOTIFICATION_NOLICENSE',
        sticky: true,
        type: 'info',
        message:
          'Your Invisily installation does not have a valid license and requires a license from you to enable all features.'
      });
      this.globalMessageService.pushAlertIfNotPresent({
        name: 'LICENSE_NOTIFICATION_NOLICENSE_CONTROLLER',
        sticky: true,
        type: 'info',
        message: 'You need to upload a license before deploying Controller.'
      });
    }
  }

  private showViolationLicenseNotification(violated) {
    if (!this.isLicenseNotificationShown) {
      this.globalMessageService.pushAlertIfNotPresent({
          name: 'LICENSE_NOTIFICATION_INVALID',
          sticky: true,
          type: 'danger',
          message:
              violated === '15'
                  ? 'License violation detected. Max Number of Services limit exceeded.'
                  : violated === '16'
                  ? 'License violation detected. Max Number of Clients limit exceeded.'
                  : 'Invalid license. Something wrong has been detected with your license.'
      });
      this.isLicenseNotificationShown = true; // Set the flag to true after showing
   }
  }

  private showExpiryLicenseNotification() {
    this.globalMessageService.pushAlertIfNotPresent({
      name: 'EXPIRY_LICENSE_NOTIFICATION',
      sticky: false,
      type: 'danger',
      message: 'Your license has expired.'
    });
  }

  private showProcessingRestrictedLicenseNotification() {
    this.globalMessageService.pushAlertIfNotPresent({
      name: 'LICENSE_NOTIFICATION_PROCESSING_RESTRICTED',
      sticky: false,
      type: 'info',
      message:
        'You cannot access this feature until the license has finished processing.'
    });
  }

  intercept(
    request: HttpRequest<any>,
    next: HttpHandler
  ): Observable<HttpEvent<any>> {
    const currentUser = JSON.parse(localStorage.getItem('currentUser'));
    const is_first_login = JSON.parse(localStorage.getItem('is_first_login'));
    const is_question_changed = JSON.parse(
      localStorage.getItem('is_question_changed')
    );

    const user = JSON.parse(localStorage.getItem('userData'));
    if (currentUser && currentUser.token) {
      request = request.clone({
        setHeaders: {
          Authorization: `Bearer ${currentUser.token}`
        }
      });
    } else if (!is_question_changed && user?.token) {
      request = request.clone({
        setHeaders: {
          Authorization: `Bearer ${user?.token}`
        }
      });
    } else if (is_first_login && user?.token) {
      request = request.clone({
        setHeaders: {
          Authorization: `Bearer ${user?.token}`
        }
      });
    } else if (is_question_changed && user?.token) {
      request = request.clone({
        setHeaders: {
          Authorization: `Bearer ${user?.token}`
        }
      });
    }
    return next.handle(request).pipe(
      tap({
        next: (event: HttpEvent<any>) => {
          if (event instanceof HttpResponse) {
            const res = event as HttpResponse<any>;
            this.showInformativeLicenseNotification(
              res.headers.get('X-License-Days'),
              res.headers.get('X-License-Expiring-Today'),
              res.headers.get('X-License-Expiring-Tomorrow'),
              res.headers.get('X-License-Violation-Reason'),
              res.headers.get('X-Controller-Count')
            );
            this.hideNavItems(res.headers.get('X-License-Days'));
          }
        },
        error: (err: any) => {
          if (
            err?.error?.message == 'jwt expired' ||
            err?.statusText == 'Unknown Error'
          ) {
            const authenticationService = this.injector.get(
              AuthenticationService
            );
            authenticationService.logout(true);
          } else if (err.status === 403) {
            let errorMsg = '';
            if (typeof err.error === 'string') {
              errorMsg = err.error;
            } else if (
              err.error &&
              typeof err.error === 'object' &&
              err.error.error
            ) {
              errorMsg = err.error.error;
            }
            if (
              errorMsg === LicenseErrorEnum.Invalid ||
              errorMsg === 'Invalid' ||
              errorMsg === 'No License' ||
              errorMsg === 'ProcessingRestricted' ||
              errorMsg.indexOf('Violation-Reason::') > -1
            ) {
              if (errorMsg === LicenseErrorEnum.Invalid) {
                this.showExpiryLicenseNotification();
              } else if (errorMsg.indexOf('Violation-Reason::') > -1) {
                this.showViolationLicenseNotification(errorMsg.split('::')[1]);
              } else if (errorMsg !== 'No License') {
                this.showProcessingRestrictedLicenseNotification();
              }
              this.hideNavItems('-1');

              const router = this.injector.get(Router);
              router.navigate(['/license']);
            } else {
              this.globalMessageService.pushAlert({
                closed: false,
                type: 'danger',
                message: 'You are not authorized to perform this operation.'
              });
            }
          } else if (err.status === 401) {
            const authenticationService = this.injector.get(
              AuthenticationService
            );
            authenticationService.logout(true);
          } else {
            const errors = Array.isArray(err.error.message)
              ? err.error.message
              : [{ error: err.error.message }];
            const errorMessges = errors.map(e => e[Object.keys(e)[0]]);

            errorMessges.forEach(error => {
              if (error) {
                this.globalMessageService.pushAlert({
                  closed: false,
                  type: 'danger',
                  message: error
                });
              }
            });
          }
        }
      })
    );
  }
}
